#!/bin/bash
#
# $Date: 2008/04/21 23:23:00$

# This file should be sourced (i.e., '. <filename>' or 'source <filename>')

# JML_DEV_BASE  : directory under which MJ and JML2 will reside.
: ${JML_DEV_BASE:=$HOME}
# JML_PKG_PREFIX: path prefix to JML required third party packages.
: ${JML_PKG_PREFIX:=/usr/local}
# Major Java version to be used in building paths, etc. for JML.
: ${JML_JAVA_MAJ_VERS:=4}
# JDT directory name to be used in building JML:
: ${JML_JDK_DIR:=jdk1.$JML_JAVA_MAJ_VERS}

# Java CLASSPATH setting (comment this out if you don't need it)
export CLASSPATH=".:$JML_DEV_BASE/MJ:$JML_DEV_BASE/JML2:$JML_PKG_PREFIX/antlr/antlr.jar:$JML_PKG_PREFIX/junit/junit.jar:$JML_PKG_PREFIX/gnu:$JML_DEV_BASE/JML2/specs:$JML_PKG_PREFIX/ant/lib/ant.jar"

if [ "$JML_JAVA_MAJ_VERS" == "5" ]; then
    # The following are needed by the MJ build:
    export JAVA5=t
    export JAVATOOLS15=$JAVA_HOME/lib/tools.jar
    export CLASSPATH="$CLASSPATH:$JAVATOOLS15"
else
    export CLASSPATH="$CLASSPATH:$JML_PKG_PREFIX/$JML_JDK_DIR/lib/tools.jar"
fi

# For building mjdoc and jmldoc with multiple versions at once.
# You can comment these out, either one or both, if you like.
# export JAVATOOLS141="$JML_PKG_PREFIX/jdk1.4.1/lib/tools.jar"
# export JAVATOOLS142="$JML_PKG_PREFIX/jdk1.4/lib/tools.jar"

# For building the ant portions of JML and MJ
# export ANT_HOME=/usr/local/ant

# The shell's name of the directory containing the MJC project files
export PLSL_CLASSROOT="$JML_DEV_BASE/MJ"
export MJ_CLASSROOT="$JML_DEV_BASE/MJ"

# OS native (javac compatible) format for
# the name of the directory containing the MJC project files
export MJ_JCLASSROOT="$JML_DEV_BASE/MJ"

# The shell's name of the directory containing the JML project files
export JML_CLASSROOT="$JML_DEV_BASE/JML2"

# OS native (javac compatible) format for
# the name of the directory containing the JML project files
export JML_JCLASSROOT="$JML_DEV_BASE/JML2"

# OS native relative path name to the MJC project files
export MJ_RELATIVE_ROOT='../MJ'

# Java file separator character on this machine
export JFILESEP='/'

# Java path separator character on this machine
export JPATHSEP=':'

# Name of the java compiler
: ${JAVAC='javac'}
export JAVAC

# Name of the java virtual machine
: ${JAVA='java'}
export JAVA

: ${TEMP="$HOME/temp"}
export TEMP

# Temporary directory in OS native format
export JTEMP="$TEMP"
if [ -z "${JTEMP}" ]; then
    # Set JTEMP to a reasonable default that is user specific (to avoid
    # clashes that might occur when multiple users are working on MJ or JML2
    # from the same machine.
    temp_dir_name=$USER
    : ${temp_dir_name:=$UID} # if $USER is not defined, use $UID which always should be.
    export JTEMP=/usr/tmp/$temp_dir_name
    [ ! -e ${JTEMP} ] && mkdir ${JTEMP}	# Create the directory if it does not exist
    unset temp_dir_name
fi

# Use forward slashes on all machines for URLs below.

# URL for the directory containing the javadocs for the JDK on this machine
export JDKDOCDIR="file:$JML_PKG_PREFIX/$JML_JDK_DIR/docs/api"

# URL for the directory containing the javadocs for antlr on this machine
export ANTLRDOCDIR="file:$JML_PKG_PREFIX/antlr/javadocs"

# URL for the directory containing the javadocs for MJC on this machine
export JDLINKS="-link file:$JML_DEV_BASE/MJ/javadocs"

## Copyright (C) 2000-2008 Iowa State University
##
## This file is part of JML, the Java Modeling Language
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
