// @(#)$Id: CompileTask.java,v 1.2 2002/12/15 19:52:22 leavens Exp $

// Copyright (C) 2002 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package org.jmlspecs.ant.tasks;

import java.util.ArrayList;
import java.util.List;
import org.jmlspecs.jmlrac.Main;
import java.io.File;
import org.apache.tools.ant.BuildException;

/** An Ant task to run the JML runtime assertion checking compiler.
 * @version $Revision: 1.2 $
 * @author  Marko van Dooren
 */
public class CompileTask extends CommonOptionsTask {
  
    /*@
      @   post getVerbose() == false;
      @*/
    public CompileTask() {
        super();
        _source = false;
    }

    /**
     * Check all the files in the filesets.
     */
    public void execute() throws BuildException {
        boolean success = true;
        try{
            String[] arguments = getArguments();
            success = org.jmlspecs.jmlrac.Main.compile(arguments);
        }
        catch(Exception exc) {
            throw new BuildException(exc);
        }
        if(! success) {
            throw new BuildException("JML syntax error");
        }

    }

    /**
     * Return a list of the tool-specific options for running a JML tool.
     */
    public /*@ pure non_null @*/ List getToolSpecificOptions() {
        final ArrayList args = new ArrayList();
        if(getSource()) {
            args.add("-P");
        }
        return args;
    }

    /**
     * Set the source generation property of this CompileTask.
     *
     * @param source
     *        True if source should be generated, false otherwise.
     */
    /*@ public normal_behavior
      @   assignable \not_specified;
      @   post getSource() == source;
      @*/
    public void setSource(boolean source) {
        _source = source;
    }

    /**
     * Check whether or not this task will generate source instead
     * of class files.
     */
    public /*@ pure @*/ boolean getSource() {
        return _source;
    }

    private boolean _source;

}
