header {
/*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Jml.g,v 1.143 2008/05/14 18:27:52 wdietl Exp $
 */


    // common parser/lexer header
    package org.jmlspecs.checker;

    import org.multijava.util.compiler.*;
    import org.multijava.mjc.*;

    // import java.io.Reader;
}

{
    // Parser class preamble
    import java.util.ArrayList;

    import org.multijava.javadoc.JavadocComment;
    import org.multijava.mjc.*;
    import org.multijava.util.Utils;

    import antlr.CharScanner;
}

// ----------------------------------------------------------------------
// THE PARSER STARTS HERE
// ----------------------------------------------------------------------

class JmlParser extends MjcParser;
options {
    k = 2;                           // two token lookahead
    importVocab = JmlID;
    exportVocab = Jml;
    codeGenMakeSwitchThreshold = 2;  // Some optimizations
    codeGenBitsetTestThreshold = 3;
    defaultErrorHandler = false;     // Don't generate parser error handlers
}

{
    public JmlParser( org.multijava.mjc.Main compiler, TokenStream lexer,
        ParsingController parsingController,
        boolean allowGeneric, boolean allowMultiJava,
        boolean allowRelaxedMultiJava,
        boolean allowUniverseKeywords ) // WMD
    {
        this( compiler, lexer, parsingController, allowGeneric,
              allowMultiJava, allowRelaxedMultiJava, allowUniverseKeywords,
              false );
    }

    public JmlParser( org.multijava.mjc.Main compiler, TokenStream lexer,
        ParsingController parsingController, boolean allowGeneric,
        boolean allowMultiJava, boolean allowRelaxedMultiJava,
        boolean allowUniverseKeywords, // WMD
        boolean isRefinedCUnit )
    {
        this( lexer );
        this.utility =
        new JmlParserUtility( compiler, parsingController, allowGeneric,
            allowMultiJava, allowRelaxedMultiJava, allowUniverseKeywords, true);
        this.isRefinedCUnit = isRefinedCUnit;
    }

    /**
     * Returns an exception from our exception hierarchy by wrapping
     * the ANTLR excepton.  The exception is really positioned, by the
     * positioning information is embedded in the message. */
    public PositionedError beautifyParserError( ANTLRException e ) {
	return utility.beautifyParserError( e );
    }

    /**
     * Returns true if the argument mod is one of the privacy modifiers
     * (i.e., public, protected or privacy) or null (= 0). */
    public static boolean isPrivacyOrNull(long mod) {
        return (mod == 0 ||
	    mod == Constants.ACC_PUBLIC ||
	    mod == Constants.ACC_PROTECTED ||
	    mod == Constants.ACC_PRIVATE);
    }

    /**
     * Returns true if all elements of the argument are simple spec body
     * clauses; otherwise, print an appropriate error message and
     * return false */
    public boolean verifySimpleSpecBody(JmlSpecBodyClause[] clauses) {
        boolean result = true;
        for (int i = 0; i < clauses.length; i++)
	if (!clauses[i].isSimpleSpecBody()) {
	    utility.reportTrouble( new PositionedError(
                    clauses[i].getTokenReference(),
                    JmlMessages.BAD_SIMPLE_SPEC_BODY ));
	    result = false;
	}
        return result;
    }

    /**
     * Returns true if all elements of the argument are normal spec body
     * clauses; otherwise, print an appropriate error message and
     * return false */
    public boolean verifyNormalSpecBody(JmlSpecBodyClause[] clauses) {
        boolean result = true;
        for (int i = 0; i < clauses.length; i++)
	if (!clauses[i].isNormalSpecBody()) {
	    utility.reportTrouble( new PositionedError(
                    clauses[i].getTokenReference(),
                    JmlMessages.BAD_NORMAL_SPEC_BODY ));
	    result = false;
	}
        return result;
    }

    /**
     * Returns true if all elements of the argument are exceptional spec
     * body clauses; otherwise, print an appropriate error message and
     * return false */
    public boolean verifyExceptionalSpecBody(JmlSpecBodyClause[] clauses) {
        boolean result = true;
        for (int i = 0; i < clauses.length; i++)
	if (!clauses[i].isExceptionalSpecBody()) {
	    utility.reportTrouble( new PositionedError(
                    clauses[i].getTokenReference(),
                    JmlMessages.BAD_EXCEPTIONAL_SPEC_BODY ));
	    result = false;
	}
        return result;
    }

    /**
     * Returns true if all elements of the argument are simple spec
     * statement body clauses; otherwise, print an appropriate error
     * message and return false */
    public boolean verifySimpleSpecStatementBody(JmlSpecBodyClause[] clauses) {
        boolean result = true;
        for (int i = 0; i < clauses.length; i++)
	if (!clauses[i].isSimpleSpecStatementBody()) {
	    utility.reportTrouble( new PositionedError(
                    clauses[i].getTokenReference(),
                    JmlMessages.BAD_SIMPLE_SPEC_STATEMENT_BODY ));
	    result = false;
	}
        return result;
    }

    /**
     * Returns true if all elements of the argument are abrupt spec
     * body clauses; otherwise, print an appropriate error message and
     * return false */
    public boolean verifyAbruptSpecBody(JmlSpecBodyClause[] clauses) {
        boolean result = true;
        for (int i = 0; i < clauses.length; i++)
	if (!clauses[i].isAbruptSpecBody()) {
	    utility.reportTrouble( new PositionedError(
                    clauses[i].getTokenReference(),
                    JmlMessages.BAD_ABRUPT_SPEC_BODY ));
	    result = false;
	}
        return result;
    }

    /**
	 * Generates the appropriate data structure for buffering class
	 * members during parsing.  Implemented as a method for
	 * sub-grammars can use a different data structure (of the same
	 * name).  This is a hack that takes advantage of ANTLR's textual
	 * grammar inheritance.  */
    private CParseClassContext getParseClassContext() {
	// All these "CParseClassContext" refer to the class in the
	// local package (org.jmlspecs.checker).  The casts are
	// needed because the return type of getInstance() is
	// "org.multijava.mjc.CParseClassContext".
	return (CParseClassContext) CParseClassContext.getInstance();
    }

    // --------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // --------------------------------------------------------------------

	/**
	 * Instances of JmlParser delegate to utility.  */
    private ParserUtility utility;

    private boolean isInModelProgram = false;

    private boolean isRefinedCUnit = false;

    private JExpressionFactory exprFactory = new JmlExpressionFactory();

    private int unmatchedTypeLT = 0 ;
    //---------------------------------------------------------------------
    // NESTED CLASSES
    //---------------------------------------------------------------------

    // !FIXME! The following should be private when Sun fixes its javadoc bug - DRCok
    /*private*/ static class TypeBooleanPair {
	TypeBooleanPair( CClassType type, boolean bool ) {
	    this.type = type;
	    this.bool = bool;
	}
	CClassType type;
	boolean bool;
    }

    /**
	 * This nested class represents a list of implemented interfaces
	 * for a class declaration (or extends interfaces for an interface
	 * declaration) and whether they are implemented (or extended)
	 * weakly.  */
    // !FIXME! The following should be private when Sun fixes its javadoc bug - DRCok
    /*private*/ static class TypeWeaklyList {
	/**
		 * <pre><jml>
		 * model_program {
		 *    weaklyFlags.add( new Boolean( _weakly ) );
		 * }
         * also
		 * assignable weaklyFlags;
		 * </jml></pre>
		 */
	// Note: I'm using _weakly instead of weakly, because
	// weakly is a jml reserved word and causes parsing errors
	void add( CClassType type, boolean _weakly ) {
	    int pos = types.size();
	    types.add( type );
	    if ( _weakly ) {
		if ( indicesSize == indices.length ) {
		    growIndices();
		}
		indices[indicesSize++] = pos;
	    }
	}

	CClassType[] types() {
	    CClassType[] result = new CClassType[ types.size() ];
	    types.toArray( result );
	    return result;
	}

    /**
     * <pre><jml>
     * model_program {
     *    Boolean[] result = new Boolean[ weaklyFlags.size() ];
     *    weaklyFlags.toArray( result );
     *    boolean[] res = new boolean[ weaklyFlags.size() ];
     *    normal_behavior
     *      assignable res;
     *      ensures (\forall int i; 0<=i && i < res.length;
     *                    result[i].booleanValue() == res[i]);
     *    return res;
     * }
     * </jml></pre>
     */
	boolean[] weaklyFlags() {
	    boolean[] result = new boolean[ types.size() ];
	    for ( int i=0; i < indicesSize; i++ ) {
		result[indices[i]] = true;
	    }
	    return result;
	}

	//@  assignable weaklyFlags;
	private void growIndices() {
	    int[] newIndices = new int[ indices.length * 2 ];
	    System.arraycopy( indices, 0, newIndices, 0, indices.length );
	    indices = newIndices;
	}

	private ArrayList types = new ArrayList();

	/*@ protected model ArrayList weaklyFlags;
	  @ initially weaklyFlags.equals( new ArrayList() ); @*/

        /*@ 
		  @ private represents weaklyFlags \such_that
		  @    (\forall int i; 0<=i && i<weaklyFlags.size();
		  @              ((Boolean) weaklyFlags.get(i)).booleanValue() <==>
		  @                     (\exists int j; 0<=j && j<indicesSize;
		  @								 indices[j] == i ));
		  @ private represents_redundantly weaklyFlags \such_that
		  @    (\forall int i; 0<=i && i<indicesSize;
		  @        ((Boolean) weaklyFlags.get(indices[i])).booleanValue() );
		  @ private invariant
		  @    indices != null &&
		  @    indices.length > 0 &&
		  @	   0 <= indicesSize &&
		  @    indicesSize <= indices.length &&
		  @    indicesSize <= types.size();
		  @
		  @*/

	private int[] indices = new int[10];
    //@                in weaklyFlags;

	private int indicesSize = 0;
    //@                in weaklyFlags;

    }

}

// Compilation Unit: In Java, this is a single file.  This is the start
//   rule for this parser
jCompilationUnit []
returns [JCompilationUnitType self = null]
{
    JPackageName pack = JPackageName.UNNAMED;
    JmlRefinePrefix refinePrefix = null;
    JmlRefinePrefix rp2 = null;
    TokenReference sourceRef = utility.buildTokenReference();
    CParseCompilationUnitContext context =
        CParseCompilationUnitContext.getInstance();

    // automatic import of package 'org.jmlspecs.lang'

    context.addPackageImport(JmlCompilationUnit.JMLSPECS_LANG);
}
    :
	// A compilation unit starts with an optional package definition
	( pack = jPackageDefinition[] )?

	{ context.setPackage(pack); }

	// Option refine clause
	( refinePrefix = jmlRefinePrefix[]
	    ( rp2 = jmlRefinePrefix[] ) ?
	    { if (rp2 != null)
		throw
		new TokenStreamException(
		    "A compilation unit may have at most one refine clause." );
	    }

	)?

	// Next we have a series of zero or more import statements
	(
	    (
		options { greedy = true; } :
		jImportDefinition[context]
	    )*
	)

	// Wrapping things up with any number of top-level
	// definitions (Java classes or interfaces or MultiJava top-level
	// declarations)
	( mjTopLevelDefinition[context] )*

	EOF
	{
	    self = new JmlCompilationUnit(
		sourceRef,
		pack,
		context.compilationUnitExport(),
		context.getPackageImports(),
		context.getSingleImports(),
		context.getTypeDeclarations(),
		context.getMJTopLevelMethodDeclarations(),
		refinePrefix
	    );
	    context.release();
	}
    ;

jmlRefinePrefix []
returns [JmlRefinePrefix self = null]
{
    ArrayList identList = null;
}
    :
	( "refine" | "refines" )
	m:STRING_LITERAL n:SEMI
	    {
		utility.flushJavadocTokensWithWarning( n );
		self = new JmlRefinePrefix(
		    utility.buildTokenReference( m ),
		    m.getText()
		);
	    }
    ;

// Import statement: import followed by a package, class, or generic
// function name
jImportDefinition [CParseCompilationUnitContext cunit]
{
    StringBuffer	buffer = null;
    boolean	star = false;
    boolean isModel = false;
    String	name = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	( "model" { isModel = true; } )? "import" i:IDENT
	(
	    // CONFLICT: ANTLR thinks this is ambiguous, apparently because
	    //           it doesn't recognize that IDENT and STAR are
	    //           disjoint? !FIXME! if possible
	    options { warnWhenFollowAmbig = false; } :
	    DOT j:IDENT
	    { (buffer ==
		    null ? (buffer = new StringBuffer(i.getText())) : buffer)
		.append('/').append(j.getText());
	    }
	)*
	{ name = buffer == null ? i.getText() : buffer.toString(); }
	( DOT STAR { star = true; } )?
	e1:SEMI
	{
	    if (star) {
		cunit.addPackageImport(
		    new JmlPackageImport(sourceRef,
			name, utility.getStatementComment(), isModel));
	    } else {
		JClassOrGFImportType importee =
		new JmlClassOrGFImport(sourceRef,
		    name, utility.getStatementComment(), isModel);
		cunit.addSingleImport( importee );
	    }
	    utility.flushJavadocTokensWithWarning( e1 );
	}
    ;

// all modifiers, broken out as a separate rule so subgrammars can
// override this but inherit mjModifier
jModifier []
returns [long self = 0]
    :
	self = mjModifier[]
    |
	self = jmlModifier[]
    ;

// additional modifiers for JML
jmlModifier []
returns [long self = 0]
    :
	"model" { self = Constants.ACC_MODEL; }
    // |  WMD moved to Mjc.g
    // "pure" { self = Constants.ACC_PURE; }
    
    |
	"extract" { self = Constants.ACC_EXTRACT; }
    |
	"query" { self = Constants.ACC_QUERY; }
    |
	"secret" { self = Constants.ACC_SECRET; }
    |
	"instance" { self = Constants.ACC_INSTANCE; }
    |
	"spec_public"	{ self = Constants.ACC_SPEC_PUBLIC; }
    |
	"spec_protected"  { self = Constants.ACC_SPEC_PROTECTED; }
    |
	"ghost" { self = Constants.ACC_GHOST; }
    |
	"monitored" { self = Constants.ACC_MONITORED; }
    |
	"uninitialized"	{ self = Constants.ACC_UNINITIALIZED; }
    |
	"non_null" { self = Constants.ACC_NON_NULL; }
    |
	"nullable" { self = Constants.ACC_NULLABLE; }
    |
	"nullable_by_default" { self = Constants.ACC_NULLABLE_BY_DEFAULT; }
    |
	"non_null_by_default" { self = Constants.ACC_NON_NULL_BY_DEFAULT; }
    |
	"helper"	   { self = Constants.ACC_HELPER; }
    |
	"code_java_math"   { self = Constants.ACC_CODE_JAVA_MATH; }
    |
	"code_safe_math"   { self = Constants.ACC_CODE_SAFE_MATH; }
    |
	"code_bigint_math" { self = Constants.ACC_CODE_BIGINT_MATH; }
    |
	"spec_java_math"   { self = Constants.ACC_SPEC_JAVA_MATH; }
    |
	"spec_safe_math"   { self = Constants.ACC_SPEC_SAFE_MATH; }
    |
	"spec_bigint_math" { self = Constants.ACC_SPEC_BIGINT_MATH; }
    ;

// all BuiltInType, broken out as a separate rule so subgrammars can
// override this but inherit mjBuiltInType
jBuiltInType []
returns [CType self = null]
    :
	self = mjBuiltInType[]
    |
	self = jmlBuiltInType[]
    ;

jmlBuiltInType []
returns [CType self = null]
    :
        "\\bigint"	{ self = JmlStdType.Bigint; }
    |
        "\\real"	{ self = JmlStdType.Real; }
    ;


// A type specification is a type name with possible brackets afterwards
//   (which would make it an array type).
jTypeSpec []
returns [CType self = null]
{
    int bounds = 0;
}
    :
        // WMD inherit the TypeSpec from Mjc.g
        self = mjTypeSpec[]
    |
	"\\TYPE"
	{ self = JmlStdType.TYPE; }
	(
	    LBRACK RBRACK { bounds += 1; }
	)*
	{
	    if (bounds > 0) {
            // WMD TODO should we add a universe annotation here?
            // for now \\TYPE arrays are always peer.
            self = new CArrayType( self, bounds, null );
	    }
	}
    ;


// WMD
jUniversePeerSpec []
returns [CUniverse universe = null]
    :
        universe = mjUniversePeerSpec[]
    |
        universe = jmlUniversePeerSpec[]
    ;

// WMD
jmlUniversePeerSpec []
returns [CUniverse universe = null]
    :
        "\\peer"
        {
            // only do something if the universe checks are enabled
            if( utility.getCompiler().universeChecks() ) {
                // explicit peer reference
                universe = CUniversePeer.getUniverse(); 
            }
        }
    ;


// WMD
jUniverseRepSpec []
returns [CUniverse universe = null]
    :
        universe = mjUniverseRepSpec[]
    |
        universe = jmlUniverseRepSpec[]
    ;

// WMD
jmlUniverseRepSpec []
returns [CUniverse universe = null]
    :
        "\\rep"
        {
            // only do something if the universe checks are enabled
            if( utility.getCompiler().universeChecks() ) {
                // simple rep reference
                universe = CUniverseRep.getUniverse();
            }
        }
    ;


// WMD
jUniverseReadonlySpec []
returns [CUniverse universe = null]
    :
        universe = mjUniverseReadonlySpec[]
    |
        universe = jmlUniverseReadonlySpec[]
    ;

// WMD
jmlUniverseReadonlySpec []
returns [CUniverse universe = null]
    :
        "\\readonly"
        {
            // only do something if the universe checks are enabled
            if( utility.getCompiler().universeChecks() ) {
                universe = CUniverseReadonly.getUniverse();
            }
        }
    |
        "\\any"
        {
            // only do something if the universe checks are enabled
            if( utility.getCompiler().universeChecks() ) {
                universe = CUniverseReadonly.getUniverse();
            }
        }
    ;




// Definition of a Java class
jClassDefinition [long mods, Token startToken]
returns [JClassDeclarationType self = null]
{
    CTypeVariable[] typevariables = CTypeVariable.EMPTY;
    TypeBooleanPair	superClass = null;
    TypeWeaklyList 		interfaces = null;
    CParseClassContext	context = getParseClassContext();
    JavadocComment javadoc = null;
    JavaStyleComment[]	comments = utility.getStatementComment();
    TokenReference sourceRef = utility.buildTokenReference( startToken );
}
    :
	t:"class" ident:IDENT
    ( typevariables = jTypeVariableDeclarationList[] )?
	{
	    try { javadoc = utility.getJavadocComment( startToken );
	    } catch (Exception e) {
		utility.reportTrouble(
		    new CWarning(
			sourceRef,
			JmlMessages.JAVADOC_FAILURE, e.toString() ));
		javadoc = null;
	    }
	}
	// it _might_ have a superclass...
	superClass = jSuperClassClause[]
	// it might implement some interfaces...
	interfaces = jImplementsClause[]
	// now parse the body of the class
	jClassBlock[context]
	{
	    self = JmlClassDeclaration.makeInstance(
		sourceRef, mods, ident.getText(), typevariables,
		superClass.type, superClass.bool,
		interfaces.types(), interfaces.weaklyFlags(),
		context.getMethods(), context.getInnerClasses(),
		context.getFieldsAndInits(), context.invariants(),
		context.constraints(),
		context.representsDecls(),
		context.axioms(), context.varAssertions(),
		javadoc, comments, isRefinedCUnit );
	    context.release();
	}
    ;

jSuperClassClause []
returns [TypeBooleanPair self = null]
{
    CClassType type = null;
    boolean _weakly = false;
}
    :
	(
	    "extends" type = jTypeName[null] ( "weakly" {_weakly = true;} )?
	)?
	{ self = new TypeBooleanPair( type, _weakly ); }
    ;

// Definition of a Java Interface
jInterfaceDefinition [long mods, Token startToken]
returns [JInterfaceDeclarationType self = null]
{
    CTypeVariable[] typevariables = CTypeVariable.EMPTY;
    TypeWeaklyList		interfaces =  null;
    CParseClassContext context = getParseClassContext();
    JavadocComment javadoc = null;
    JavaStyleComment[]	comments = utility.getStatementComment();
    TokenReference sourceRef = utility.buildTokenReference( startToken );
}
    :
	t:"interface" ident:IDENT
    ( typevariables = jTypeVariableDeclarationList[] )?
	{
	    try { javadoc = utility.getJavadocComment( startToken );
	    } catch (Exception e) {
		utility.reportTrouble(
		    new CWarning(
			sourceRef,
			JmlMessages.JAVADOC_FAILURE, e.toString() ));
		javadoc = null;
	    }
	}
	// it might extend some other interfaces
	interfaces = jInterfaceExtends[]
	// now parse the body of the interface (looks like a class...)
	jClassBlock[context]
	{
	    self = JmlInterfaceDeclaration.makeInstance(
		sourceRef, mods, ident.getText(), typevariables, interfaces.types(),
		interfaces.weaklyFlags(), context.getMethods(),
		context.getInnerClasses(), context.getFieldsAndInits(),
		context.invariants(),
		context.constraints(),
		context.representsDecls(),
		context.axioms(), context.varAssertions(),
		javadoc, comments, isRefinedCUnit );
	    context.release();
	}
    ;

// Declaration of a MultiJava External Method
mjTopLevelMethodDeclaration [long mods, Token startToken]
returns [MJTopLevelMethodDeclaration self = null]
{
    CType type;					// the method return type
    CType openClass;			// the class that this method extends
    JFormalParameter[]	params;	// an array of the formals
    CClassType[] throwsList = CClassType.EMPTY;	// array of poss. exceptions
    JStatement[] body = null;	// array of the statements in the body
    int bounds = 0;				// does the method return an array
    JavadocComment javadoc = null;
    JavaStyleComment[]	comments = utility.getStatementComment();
    TokenReference sourceRef = utility.buildTokenReference( startToken );
    ParsingController.TokenWrapper declEnd =
    new ParsingController.TokenWrapper();
    JmlMethodSpecification methodSpec = null;
    JmlMethodSpecification internalMethodSpec = null;
}
    :
        // possible leading method specification
        (
            methodSpec = jmlMethodSpecification[mods]
            // the modifiers (mods) passed into
            // mjTopLevelMethodDeclaration go with the leading
            // specification if one exists, so we need to check for
            // mods again
            { startToken = LT(1); }
            mods = jModifiers[]
			("method"
                {
                    utility.flushJavadocTokensWithWarning( declEnd );
                }
            )?
        | "method"
                {
                    utility.flushJavadocTokensWithWarning( declEnd );
                }
        )?

	// get the return type of the method
	type = jTypeSpec[]
	{
	    javadoc = utility.getJavadocComment( startToken );
	}
	// get the class that this method enhances, the method identifier,
	// and the formals
	openClass = mjExternalClassTypeSpec[]
	// mjExternalClassTypeSpec consumes the DOT preceding the identifier
	ident:IDENT
	// parse the formal parameter declarations.
	LPAREN
	params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER]
	RPAREN
	(
	    LBRACK RBRACK { bounds += 1; }
	)*
	{
	    if (bounds > 0) {
            if( utility.getCompiler().universeChecks() ) {
                utility.reportTrouble(new CWarning(sourceRef, 
                        CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                        null));
            } else {
                utility.reportTrouble(new CWarning(sourceRef,
			            MjcMessages.OLD_STYLE_ARRAY_BOUNDS,
                        null));
            }
            type = new CArrayType(type, bounds, null);
	    }
	}
	// get the list of exceptions that this method is declared to throw
	(throwsList = jThrowsClause[])?

        // possible internal method specification
        (
            ( internalMethodSpec = jmlMethodSpecification[mods] )?
        )

        // only one method specification is allowed
        /* !FIXME!top-level-method-specs Need to give a better error
           message here */
        { methodSpec == null || internalMethodSpec == null }?

	// get the method body
	( body = jCompoundStatement[declEnd] | SEMI )
	{
            /* !FIXME!top-level-method-specs Do the right thing with
               methodSpec */

            if (methodSpec != null || internalMethodSpec != null) {
                System.out.println( "Ignoring top level method specification" );
            }

	    self = new MJTopLevelMethodDeclaration(
		sourceRef,
		mods,
        CTypeVariable.EMPTY,        
		type,
		openClass,
		ident.getText(),
		params,
		throwsList,
		body == null ? null : new JBlock(sourceRef, body, null),
		javadoc,
		comments );
	    utility.flushJavadocTokensWithWarning( declEnd );
	}
    ;

// An interface can extend several other interfaces...
jInterfaceExtends []
returns [TypeWeaklyList self = new TypeWeaklyList()]
    :
	( "extends" jNameWeaklyList[self] )?
    ;

// A class can implement several interfaces...
jImplementsClause []
returns [TypeWeaklyList self = new TypeWeaklyList()]
    :
	( "implements" jNameWeaklyList[self] )?
    ;

jNameWeaklyList [TypeWeaklyList self]
{
    CClassType type;
    boolean _weakly = false;
}
    :
	type = jTypeName[null] ( "weakly" { _weakly = true; } )?
	{
	    self.add( type, _weakly );
	    _weakly = false;		// reset for next use
	}
	(
	    COMMA
	    type = jTypeName[null]  ( "weakly" { _weakly = true; } )?
	    {
		self.add( type, _weakly );
		_weakly = false;		// reset for next use
	    }
	)*
    ;

// Now the various things that can be defined inside a class or interface:
// methods, constructors, or variable declaration
// Note that not all of these are really valid in an interface (constructors,
// for example), and if this grammar were used for a compiler there would
// need to be some semantic checks to make sure we're doing the right thing...
jMember [CParseClassContext context]
{
    CTypeVariable[] typevariables = CTypeVariable.EMPTY;
    JStatement[] body = null;
    JTypeDeclarationType decl;
    CType type;
    JFormalParameter[] params;
    int bounds = 0;
    CClassType[] throwsList = CClassType.EMPTY;
    JVariableDefinition[] vars;
    Token startToken = LT(1);
    TokenReference sourceRef = utility.buildTokenReference( startToken );
    ParsingController.TokenWrapper declEnd =
    new ParsingController.TokenWrapper();
    JavadocComment javadoc = null;
    long mods=0;
    JmlPredicate pred = null;
    JmlMethodSpecification methodSpec = null;
    JmlDataGroupAccumulator dataGroups = new JmlDataGroupAccumulator();
}
    :
	mods = jModifiers[]
	(
	    // if the member starts with a method-specification then it must
	    // be a constructor, method, or initializer
	    methodSpec = jmlMethodSpecification[mods]
	    //mods = jModifiers[]
	    {
		try {
		    javadoc = utility.getJavadocComment( LT(1) );
		} catch (Exception e) {
		    utility.reportTrouble(
			new CWarning(
			    sourceRef,
			    JmlMessages.JAVADOC_FAILURE, e.toString() ));
		    javadoc = null;
		}
		//if (javadoc != null) System.out.println("JAVADOC " + javadoc.text());
	    }
	    (
		// specification-only static initializer
		siSpecEnd:"static_initializer"
		{
		    context.addBlockInitializer(
			new JmlClassBlock(sourceRef, true, javadoc,
			    methodSpec));
		    utility.flushJavadocTokensWithWarning( siSpecEnd );
		}
	    |
		// specification-only instance initializer
		iSpecEnd:"initializer"
		{
		    context.addBlockInitializer(
			new JmlClassBlock(sourceRef, false, javadoc,
			    methodSpec));
		    utility.flushJavadocTokensWithWarning( iSpecEnd );
		}
	    |
		// static initializer with specification
		"static" body = jCompoundStatement[declEnd]
		{
		    context.addBlockInitializer(
			new JmlClassBlock(
			    sourceRef, true, body, javadoc, methodSpec
			));
		    utility.flushJavadocTokensWithWarning( siSpecEnd );
		}
	    |
		// instance initializer with specification
		body = jCompoundStatement[declEnd]
		{
		    context.addBlockInitializer(
			new JmlClassBlock(
			    sourceRef, false, body, javadoc, methodSpec
			));
		    utility.flushJavadocTokensWithWarning( siSpecEnd );
		}
	    |
		// method or constructor declaration with specification
		mods = jModifiers[]
        (("method"|"constructor")
                    { utility.flushJavadocTokensWithWarning( declEnd ); }
        )?
		(
		    // constructor declaration
		    scIdent:IDENT
		    // parse the formal parameter declarations.
		    LPAREN
		      params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER]
		    RPAREN

		    // get the list of exceptions that this method is
		    // declared to throw
		    ( throwsList = jThrowsClause[] )?
		    (
			body = jCompoundStatement[declEnd]
			{
			    utility.flushJavadocTokensWithWarning( declEnd );
			}
		    |
			smEnd4:SEMI
			{
			    body = null;
			    utility.flushJavadocTokensWithWarning( smEnd4 );
			}
		    )
		    {
			context.addMethodDeclaration(
			    JmlConstructorDeclaration.makeInstance(
				sourceRef, mods, scIdent.getText(), params,
				throwsList,
				new JConstructorBlockWrapper(sourceRef, body),
				javadoc, utility.getStatementComment(),
				methodSpec
			    ));
		    }

		|
		    // method declaration
            ( typevariables = jTypeVariableDeclarationList[] )?
		    type = jTypeSpec[]
		    smIdent:IDENT
		    // parse the formal parameter declarations.
		    LPAREN
              params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER]
            RPAREN
		    (
			LBRACK RBRACK { bounds += 1; }
		    )*
		    {
			    if (bounds > 0) {
                    if( utility.getCompiler().universeChecks() ) {
                        utility.reportTrouble(new CWarning(sourceRef, 
                            CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                            null));
                    } else {
                        utility.reportTrouble(new CWarning(sourceRef,
                            MjcMessages.OLD_STYLE_ARRAY_BOUNDS,
                            null));
                    }
			        type = new CArrayType(type, bounds, null);
			    }
		    }
		    (throwsList = jThrowsClause[])?
		    (
			body = jCompoundStatement[declEnd]
			{
			    utility.flushJavadocTokensWithWarning( declEnd );
			}
		    |
			smEnd2:SEMI
			{
			    body = null;
			    utility.flushJavadocTokensWithWarning( smEnd2 );
			}
		    )
		    {
			context.addMethodDeclaration(
			    JmlMethodDeclaration.makeInstance(
				sourceRef, mods, typevariables, type, smIdent.getText(),
				params, throwsList,
				body == null ? null :
				new JBlock(sourceRef, body, null),
				javadoc, utility.getStatementComment(),
				methodSpec
			    ));
		    }
		)
	    )
	|
	    // members without method specifications before their declarations
     (
        (("constructor"|"method"|"field")
                { utility.flushJavadocTokensWithWarning( declEnd ); }
        )?
      (
	    // constructor
	    ctor:IDENT
	    {
		try { javadoc = utility.getJavadocComment( startToken );
		} catch (Exception e) {
		    utility.reportTrouble(
			new CWarning(
			    sourceRef,
			    JmlMessages.JAVADOC_FAILURE, e.toString() ));
		    javadoc = null;
		}
	    }

	    LPAREN
	    params = jParameterDeclarationList[
		JLocalVariable.DES_PARAMETER]
	    RPAREN

	    ( throwsList = jThrowsClause[] )?

	    ( methodSpec = jmlMethodSpecification[0] )?

	    (
		body = jCompoundStatement[declEnd]
		{
		    utility.flushJavadocTokensWithWarning( declEnd );
		}
	    |
		smEnd3:SEMI
		{
		    body = null;
		    utility.flushJavadocTokensWithWarning( smEnd3 );
		}
	    )
	    {
		context.addMethodDeclaration(
		    JmlConstructorDeclaration.makeInstance(
			sourceRef, mods, ctor.getText(), params,
			throwsList, new JConstructorBlockWrapper(sourceRef, body),
			javadoc, utility.getStatementComment(), methodSpec
		    ));
	    }
	  |
	    // method or variable declaration(s)
        ( typevariables = jTypeVariableDeclarationList[] )?
	    type = jTypeSpec[]
	    {
		try { javadoc = utility.getJavadocComment( startToken );
		} catch (Exception e) {
		    utility.reportTrouble(
			new CWarning(
			    sourceRef,
			    JmlMessages.JAVADOC_FAILURE, e.toString() ));
		    javadoc = null;
		}
	    }
	    (
		ident:IDENT
		// parse the formal parameter declarations
		LPAREN params = jParameterDeclarationList[
		    JLocalVariable.DES_PARAMETER] RPAREN
		(
		    LBRACK RBRACK { bounds += 1; }
		)*
		{
		    if (bounds > 0) {
                if( utility.getCompiler().universeChecks() ) {
                    utility.reportTrouble(new CWarning(sourceRef, 
                        CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                        null));
                } else {
                    utility.reportTrouble(new CWarning(sourceRef,
                        MjcMessages.OLD_STYLE_ARRAY_BOUNDS,
                        null));
                }
                type = new CArrayType(type, bounds, null);
		    }
		}
		(throwsList = jThrowsClause[])?

		( methodSpec = jmlMethodSpecification[0] )?

		(
		    body = jCompoundStatement[declEnd]
		    {
			utility.flushJavadocTokensWithWarning( declEnd );
		    }
		|
		    f1:SEMI
		    {
			body = null;
			utility.flushJavadocTokensWithWarning( f1 );
		    }
		)
		{
		    context.addMethodDeclaration(
			JmlMethodDeclaration.makeInstance(
			    sourceRef, mods, typevariables, type, ident.getText(), params,
			    throwsList,
			    body == null ? null :
			    new JBlock(sourceRef, body, null),
			    javadoc, utility.getStatementComment(),
			    methodSpec
			));
		}
	    |
		vars = jVariableDefinitions[mods, type]
        f2:SEMI
        ( jmlDataGroupClause [dataGroups] )*
		{
		    utility.flushJavadocTokensWithWarning( f2 );
		    JavaStyleComment[] comment =
		    utility.getStatementComment();
		    for (int i = 0; i < vars.length; i++) {
			    context.addFieldDeclaration(
			        JmlFieldDeclaration.makeInstance(
				    vars[i].getTokenReference(), vars[i],
				    javadoc, comment, new JmlVarAssertion[0], dataGroups
			        ));
		    }
            if (dataGroups != null) {
                if (vars.length > 1) {
                    utility.reportTrouble(
                                new CWarning( sourceRef,
                        JmlMessages.ASSERTIONS_FOR_MORE_THAN_ONE_VAR,
                        null));
                }
                ArrayList mapsIntoList = dataGroups.mapsIntoClauses();
                for ( int i=0; i<mapsIntoList.size(); i++ ) {
                    JmlMapsIntoClause nextClause
                        = (JmlMapsIntoClause) mapsIntoList.get(i);
                    utility.reportTrouble(
                                new CWarning( nextClause.getTokenReference(), 
                        JmlMessages.INVALID_FIELD_NAME_IN_MAPS_CLAUSE, 
                        nextClause.fieldIdent()));
                }
            }
		}
	    )
      )
     )
	|
	    decl = jClassDefinition[mods,startToken] // inner class
	    { context.addInnerDeclaration(decl); }
	|
	    decl = jInterfaceDefinition[mods,startToken] // inner interface
	    { context.addInnerDeclaration(decl); }
	|
	    jmlDeclaration[context,mods,startToken]
	|
	    // "static { ... }" class initializer and "{ ... }" instance
	    // initializer
	    body = jCompoundStatement[declEnd]
	    // ??? Should this have a semicolon option??? !FIXME!
	    {
		if (Utils.hasOtherFlags(mods, Constants.ACC_STATIC)) {
		    utility.reportTrouble( new PositionedError( sourceRef,
			    JmlMessages.BAD_INITIALIZER_MODIFIERS ));
		}
		context.addBlockInitializer(
		    new JmlClassBlock(
			sourceRef,
			Utils.hasFlag(mods, Constants.ACC_STATIC),
			body, utility.getJavadocComment( startToken ), null
		    ));
		utility.flushJavadocTokensWithWarning( declEnd );
	    }
	|
	    aStart:"axiom" pred = jmlPredicate[] aEnd:SEMI
	    {
		if (mods!=0) {
		    utility.reportTrouble( new PositionedError( sourceRef,
			    JmlMessages.AXIOM_MODIFIERS ));
		}
		context.addAxiom(
		    new JmlAxiom( sourceRef, pred ));
		utility.flushJavadocTokensWithWarning( aEnd );
	    }
	)
    ;

jParameterDeclaration [int desc]
returns [JFormalParameter self = null]
{
    long	mods = 0;
    boolean	finalWasFirst = false;
    int		bounds = 0;
    CType	type;
    CType	ddType = null;		// type of (optional) dynamic dispatch annot.
    TokenReference	sourceRef = utility.buildTokenReference();
}
    :
	mods = jModifiers[]
	type = jTypeSpec[]
	// optional dynamic dispatch annotation
	( {utility.allowMultiJava}?
        ( AT ddType = jTypeSpec[]
            // two @ signs for a value dispatch!
        | AT AT ddType = jValueSpecializer[type, sourceRef]
        )
	)?
	ident:IDENT
	( LBRACK RBRACK { bounds += 1; } )*
	{
	    if (bounds > 0) {
            if( utility.getCompiler().universeChecks() ) {
                utility.reportTrouble(new CWarning(sourceRef, 
                    CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                    null));
            } else {
                utility.reportTrouble(new CWarning(sourceRef,
    			    MjcMessages.OLD_STYLE_ARRAY_BOUNDS));
            }
    		type = new CArrayType(type, bounds, null);
    	}
	    CSpecializedType specializedType;
	    if (ddType == null) {
		    specializedType = new CSpecializedType( type );
	    } else {
		    specializedType = new CSpecializedType( type, ddType );
	    }
	    self = new JmlFormalParameter( sourceRef, mods, desc, specializedType, ident.getText());
	}
    ;

jmlDataGroupClause [JmlDataGroupAccumulator self]
{
    boolean redundant = false;
    JmlStoreRefExpression[] groupList = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    String fieldId = null;
    JmlStoreRefExpression memberRef = null;
}
    :
	("in" | "in_redundantly" { redundant = true; } )
    groupList = jmlGroupList[] SEMI
	{
	    self.addInGroup( new JmlInGroupClause( sourceRef, redundant,
                                               groupList ) );
	}
    |
	("maps" | "maps_redundantly" { redundant = true; } )
    memberRef = jmlMapsReference []
	    {
            fieldId = memberRef.names()[0].ident();
	    }
    "\\into" groupList = jmlGroupList[] SEMI
	{
	    self.addMapsInto( new JmlMapsIntoClause( sourceRef, redundant,
                                                 fieldId,
                                                 memberRef,
                                                 groupList ) );
	}
    ;

jmlMapsReference []
returns [ JmlStoreRefExpression self = null ]
{
    ArrayList namesV = null;
    JmlName name = null;
    TokenReference sourceRef = utility.buildTokenReference();
}
    :
    i1:IDENT
	    {
            name = new JmlName( sourceRef, i1.getText() );
	        namesV = new ArrayList();
	        namesV.add( name );
        }
    (
      (  j3:LBRACK
         name = jmlSpecArrayRefExpr[ utility.buildTokenReference(j3) ]
         RBRACK
	            { namesV.add( name ); }
       )+
        ( DOT jmlMapsMemberRefExpr [namesV] )?
    |
        DOT jmlMapsMemberRefExpr [namesV]
     )
    {
	    JmlName[] names = new JmlName[ namesV.size() ];
	    namesV.toArray( names );
	    
	    self = new JmlStoreRefExpression( sourceRef, names );
    }
    ;

jmlMapsMemberRefExpr [ArrayList namesV]
{
    JmlName name = null;
}
    :
        j1:IDENT
        { 
            name = new JmlName(    
                utility.buildTokenReference(j1), j1.getText() ); 
	        namesV.add( name );
        }
	|
	    j2:STAR
		{
            name = new JmlName(    
                utility.buildTokenReference(j2), JmlName.SORT_FIELDS );
	        namesV.add( name );
		}
    ;

jmlGroupName []
returns [JmlStoreRefExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference();
    JExpression name = null;
    ArrayList namesA = new ArrayList(2);
}
    :
    (
      (
		s1:"super" DOT 
		{
            namesA.add( new JmlName(utility.buildTokenReference(s1),
			                        JmlName.SORT_SUPER) );
        }
      |
		t1:"this" DOT 
		{
            namesA.add( new JmlName(utility.buildTokenReference(t1),
			                        JmlName.SORT_THIS) );
        }
       )
     )?
		id:IDENT
	    {
            namesA.add( new JmlName(utility.buildTokenReference(id),
                                    id.getText()) );
	    }
    {
	    JmlName[] names = new JmlName[ namesA.size() ];
	    namesA.toArray( names );
	    
	    self = new JmlStoreRefExpression( sourceRef, names );
    }
    ;

jmlGroupList []
returns [JmlStoreRefExpression[] self = null]
{
    JmlStoreRefExpression name = null;
    ArrayList names = new ArrayList(8);
}
    :
    name = jmlGroupName []
	    {
            names.add( name );
	    }
	(
	    COMMA
        name = jmlGroupName []
	    {
            names.add( name );
	    }
	)*
	    {
            self = new JmlStoreRefExpression[ names.size() ];
            names.toArray(self);
            
	    }
    ;

jmlDeclaration [CParseClassContext context,long mods, Token startToken]
{
    TokenReference sourceRef = utility.buildTokenReference( startToken );
    JmlInvariant inv = null;
    JmlPredicate pred = null;
    boolean redundant = false;
    JmlMethodNameList mnList = null;
    JmlStoreRefExpression storeRef = null;
    JmlStoreRef[] storeRefList = null;
    JmlSpecExpression specExpr = null;
    JmlSpecExpression[] specs = null;
	JNameExpression fieldName = null;
}
    :
	(
	    ( "invariant" | "invariant_redundantly" { redundant = true; } )
	    pred = jmlPredicate[]
	    {
		context.addInvariant(
		    new JmlInvariant( sourceRef, mods, redundant, pred ));
	    }
	|
	    ( "constraint" | "constraint_redundantly" { redundant = true; } )
	    pred = jmlPredicate[]
	    ( "for"
		(
		    "\\everything"
		|
		    mnList = jmlMethodNameList[]
		)
	    )?
	    {
		context.addConstraint(
            new JmlConstraint( sourceRef, mods, redundant, pred,
                               mnList ));
	    }
	|
	    ( "represents" | "represents_redundantly" { redundant = true; } )
	    storeRef = jmlStoreRefExpression[]
	    (
		(ASSIGN | L_ARROW) specExpr = jmlSpecExpression[]
		{
		    context.addRepresentsDecl(
			new JmlRepresentsDecl( sourceRef, mods, redundant,
			    storeRef, specExpr ));

		}
	    |
		"\\such_that" pred = jmlPredicate[]
		{
		    context.addRepresentsDecl(
			new JmlRepresentsDecl( sourceRef, mods, redundant,
			    storeRef, pred ));

		}
	    )
    |
	    "initially" pred = jmlPredicate[]
	    {
		    context.addVarAssertion(
	            new JmlInitiallyVarAssertion( sourceRef, mods, pred ) );
	    }
	|
	    "readable"
        rId:IDENT  "if"
	    {
            fieldName = new JNameExpression(utility.buildTokenReference(rId),
                                            rId.getText());
	    }
        pred = jmlPredicate[]
	    {
		    context.addVarAssertion(
	            new JmlReadableIfVarAssertion( sourceRef, mods,
                                               fieldName, pred ) );
	    }
	|
	    "writable"
        wId:IDENT  "if"
	    {
            fieldName = new JNameExpression(utility.buildTokenReference(wId),
                                            wId.getText());
	    }
        pred = jmlPredicate[]
	    {
		    context.addVarAssertion(
	            new JmlWritableIfVarAssertion( sourceRef, mods,
                                               fieldName, pred ) );
	    }
	|
	    "monitors_for"
        mId:IDENT  (ASSIGN | L_ARROW)
	    {
            fieldName = new JNameExpression(utility.buildTokenReference(mId),
                                            mId.getText());
	    }
	    specs = jmlSpecExpressionList[]
	    {
		    context.addVarAssertion(
	            new JmlMonitorsForVarAssertion( sourceRef, mods,
                                                fieldName, specs ) );
	    }
	)
	e:SEMI { utility.flushJavadocTokensWithWarning( e ); }
    ;

jmlMethodNameList []
returns [JmlMethodNameList nameList=null]
{
    TokenReference sourceRef = utility.buildTokenReference();
    ArrayList nameV = null;
    JmlMethodName name = null;
    JmlMethodName[] names = null;
}
    :
	name = jmlMethodName[]
	{
	    nameV = new ArrayList();
	    nameV.add( name );
	}
	( COMMA name = jmlMethodName[] { nameV.add( name ); } )*
	{
	    names = new JmlMethodName[ nameV.size() ];
	    nameV.toArray( names );
	    nameList = new JmlMethodNameList(sourceRef, names);
	}
    ;

jmlMethodName []
returns [JmlMethodName self = null]
{
    StringBuffer methodRef = new StringBuffer();
    TokenReference sourceRef = utility.buildTokenReference();
    boolean isConstructorRef = false;
    CType[] paramTypes = null;
    CType type = null;
    ArrayList namesV = null;
    JmlName name = null;
    boolean hasDisambigList = false;
    CUniverse elem_univ = null;
}
    :
	// first a method-ref
	(
	    // method-ref-start
	    (
		i1:"super"
		{ name = new JmlName(
			utility.buildTokenReference(i1), JmlName.SORT_SUPER); }
	    |
		i2:"this"
		{ name = new JmlName(
			utility.buildTokenReference(i2), JmlName.SORT_THIS); }
	    |
		i3:IDENT
		{ name = new JmlName(
			utility.buildTokenReference(i3), i3.getText()); }
	    )
	    {
		namesV = new ArrayList();
		namesV.add( name );
	    }

	    (
		DOT
		(
		    j2:"this"
		    { name = new JmlName(
			    utility.buildTokenReference(j2),
			    JmlName.SORT_THIS); }
		|
		    j3:IDENT
		    { name = new JmlName(
			    utility.buildTokenReference(j3), j3.getText());
            }
	    |
            j4:STAR
          	{ name = new JmlName(
                utility.buildTokenReference(j4), JmlName.SORT_WILDCARD );
            }
		)
		{ namesV.add( name ); }
	    )*
	|
	    "new"
            // WMD add element universe annotation
            (elem_univ = jUniverseType[])?
        type = jClassTypeSpec[elem_univ, null]
	    { isConstructorRef = true; }
	)

	// optional param-disambig-list, which may optionally be empty
	(
	    LPAREN { hasDisambigList = true; }
	    ( paramTypes = jmlParamDisambigList[] )?
	    RPAREN
	)?
	{
	    if (paramTypes == null && hasDisambigList) {
			paramTypes = new CType[0];
	    }
	    if (isConstructorRef) {
			self = new JmlConstructorName( sourceRef, type, paramTypes );
	    } else {
			JmlName[] names = new JmlName[ namesV.size() ];
			namesV.toArray( names );
		
			self = new JmlMethodName( sourceRef, names, paramTypes );
	    }
	}
    ;

jmlParamDisambigList []
returns [CType[] types = null]
{
    ArrayList typesV = null;
    CType type = null;
}
    :
	type = jmlParamDisambig[]
	{
	    typesV = new ArrayList();
	    typesV.add( type );
	}
	( COMMA type = jmlParamDisambig[] { typesV.add( type ); } )*
	{
	    types = new CType[ typesV.size() ];
	    typesV.toArray( types );
	    
	}
    ;

jmlParamDisambig []
returns [CType type = null]
{
    int bounds = 0;
}
    :
	type = jTypeSpec[]
	(
	    id:IDENT
	    ( LBRACK RBRACK { bounds += 1; } )*
	)?
	{
	    if (bounds > 0) {
            if( utility.getCompiler().universeChecks() ) {
                utility.reportTrouble(
                    new CWarning(utility.buildTokenReference(id), 
                        CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                        null));
            } else {
                utility.reportTrouble(
                    new CWarning(utility.buildTokenReference(id),
                                 MjcMessages.OLD_STYLE_ARRAY_BOUNDS));
            }
            type = new CArrayType(type, bounds, null);
	    }
	}
    ;

jmlStoreRef []
returns [JmlStoreRef self = null]
{
    TokenReference sourceRef = utility.buildTokenReference();
    JmlSpecExpression specExpression = null;
    CType type = null;
    JmlStoreRefExpression storeRefExpression = null;
}
    :
	(
	    self = jmlStoreRefExpression[]
	|
	    i:INFORMAL_DESC
	    {
		self = new JmlInformalStoreRef( sourceRef, i.getText() );
	    }
	|
        self = jmlStoreRefKeyword []
    )
    ;

jmlStoreRefKeyword []
returns [JmlStoreRefKeyword self = null]
{
    int keywordType = 0;
    TokenReference sourceRef = utility.buildTokenReference();
}
    :
        (
            "\\nothing" { keywordType = JmlStoreRefKeyword.NOTHING; }
        |
            "\\everything" { keywordType = JmlStoreRefKeyword.EVERYTHING; }
        |
            "\\not_specified"
            { keywordType = JmlStoreRefKeyword.NOT_SPECIFIED; }
        )
        { self = new JmlStoreRefKeyword( sourceRef, keywordType ); }
    ;


jmlStoreRefList []
returns [JmlStoreRef[] self = null]
{
    ArrayList refsV = null;
    JmlStoreRef ref = null;
}
    :
	ref = jmlStoreRef[]
	{
	    refsV = new ArrayList();
	    refsV.add( ref );
	}
	(
	    COMMA ref = jmlStoreRef[]
	    { refsV.add( ref ); }
	)*
	{
	    self = new JmlStoreRef[ refsV.size() ];
	    refsV.toArray( self );
	    
	}
    ;

jmlStoreRefExpression []
returns [JmlStoreRefExpression self = null]
{
    ArrayList namesV = null;
    JmlName name = null;
    TokenReference sourceRef = utility.buildTokenReference();
}
    :
	// store-ref-name
	(
	    i1:IDENT
	    { name = new JmlName( sourceRef, i1.getText() ); }
	|
	    i2:"super"
	    { name =
		new JmlName( sourceRef, JmlName.SORT_SUPER ); }
	|
	    i3:"this"
	    { name =
		new JmlName( sourceRef, JmlName.SORT_THIS ); }
	)
	{
	    namesV = new ArrayList();
	    namesV.add( name );
	}
	(
        name = jmlStoreRefNameSuffix[]
	    { namesV.add( name ); }
	)*
	{
	    JmlName[] names = new JmlName[ namesV.size() ];
	    namesV.toArray( names );
	    
	    self = new JmlStoreRefExpression( sourceRef, names );
	}
    ;

jmlStoreRefNameSuffix []
returns [ JmlName self = null ]
    :
        DOT j1:IDENT
        {
            self = new JmlName(
                utility.buildTokenReference(j1), j1.getText() );
        }
    |
        DOT j2:"this"
        {
            self = new JmlName(
                utility.buildTokenReference(j2), JmlName.SORT_THIS );
        }
    |
        j3:LBRACK
        self = jmlSpecArrayRefExpr[ utility.buildTokenReference(j3) ]
        RBRACK
	|
	    DOT j4:STAR
		{
            self = new JmlName(
                utility.buildTokenReference(j4), JmlName.SORT_FIELDS );
		}
    ;

jmlSpecArrayRefExpr [ TokenReference sourceRef ]
returns [ JmlName self = null ]
{
    JExpression start = null;
    JExpression end = null;
}
    :
	start = jExpression[] ( DOT_DOT end = jExpression[] )?
	{
	    if (end == null) {
		self = new JmlName(sourceRef, new JmlSpecExpression( start ));
	    } else {
		self = new JmlName(
		    sourceRef,
		    new JmlSpecExpression( start ),
		    new JmlSpecExpression( end )
		);
		end = null;
	    }
	}
    |
	STAR
	{
	    self = new JmlName(sourceRef, JmlName.SORT_ALL);
	}
    ;

jmlMethodSpecification [long mods]
returns [ JmlMethodSpecification self = null ]
    :
	self = jmlSpecification[mods]
    |
        self = jmlExtendingSpecification[]
	{
	    if (mods != 0 ) {
		utility.reportTrouble( new PositionedError(
			self.getTokenReference(),
			JmlMessages.EXTENDING_SPEC_MODIFIERS ));
	    }
	}
    ;

jmlSpecification [long mods] 
returns [ JmlSpecification self = null ]
{
    JmlSpecCase specCases[] = null;
    JmlRedundantSpec redundant = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        specCases = jmlSpecCaseSeq[mods]
        ( redundant = jmlRedundantSpec[] )?
        {
            self = new JmlSpecification(sourceRef, specCases, 
                redundant);
        }

    |
        redundant = jmlRedundantSpec[]
        {
	        if (mods != 0 ) {
		        utility.reportTrouble( new PositionedError(
		    	    utility.buildTokenReference(),
		    	    JmlMessages.REDUNDANT_SPEC_MODIFIERS ));
	        }
            self = new JmlSpecification(sourceRef, redundant);
        }
    ;

jmlSpecCaseSeq [long mods]
returns [JmlSpecCase[] self = null ]
{
    ArrayList cases = new ArrayList();
    JmlSpecCase c1, c2;
}
    :   c1 = jmlSpecCase[mods] { cases.add(c1); }
        ( "also" c2 = jmlSpecCase[0] { cases.add(c2); } )*
        {
	    self = new JmlSpecCase[cases.size()];
	    cases.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;

jmlSpecCase [long mods]
returns [JmlSpecCase self = null ]
{
    long moreMods = 0;
}
    :
        self = jmlGenericSpecCase[false]
	{
        // lightweight specification

	    if (mods != 0) {
		utility.reportTrouble( new PositionedError(
			self.getTokenReference(),
			JmlMessages.GENERIC_SPEC_CASE_MODIFIERS ));
	    }
	}
    |
        {
        TokenReference sourceRef = utility.buildTokenReference();
        if (!isPrivacyOrNull(mods)) {
		    utility.reportTrouble( new PositionedError(
			    sourceRef,
		    	JmlMessages.BAD_PRIVACY_MODIFIER ));
	    }
        }

	// The first spec-case in a spec-case-seq will receive mods
	// from parameter, after that this rule may match.  We OR the
	// bit-masks together, effectively ignoring the one that is 0.
	moreMods = jModifiers[]
        {
            TokenReference sourceRef = utility.buildTokenReference();
            if (!isPrivacyOrNull(moreMods)) {
		        utility.reportTrouble( new PositionedError(
			        sourceRef,
			        JmlMessages.BAD_PRIVACY_MODIFIER ));
	        }

            if ((mods & moreMods) != 0) {
		        utility.reportTrouble( new PositionedError(
		        	sourceRef,
		        	JmlMessages.MULTIPLE_PRIVACY_MODIFIER ));
            }

            mods |= moreMods;
        }
    ( "code" 
            { mods |= Constants.ACC_CODE; }
    ) ?
	(
	    self = jmlHeavyweightSpec[mods]
	|
	    self = jmlModelProgram[mods]
	)
    ;

jmlExtendingSpecification []
returns [ JmlExtendingSpecification self = null ]
{
    JmlSpecification spec = null;
}
    :
	// pass 0 since modifiers aren't allowed before the "also"
        "also" spec = jmlSpecification[0]
	{
	    // In order to preserve the fact that this spec
	    // began with 'also', we need to convert this
	    // to a JmlExtendingSpecification - per the grammar
	    self = new JmlExtendingSpecification(spec);
	}
    ;

jmlRedundantSpec []
returns [ JmlRedundantSpec self = null ]
{
    JmlSpecCase implications[] = null;
    JmlExample examples[] = null;
    TokenReference sourceRef = utility.buildTokenReference( );
}
    :
        ( implications = jmlImplications[] ( examples = jmlExamples[] )? )
        {
            self = new JmlRedundantSpec( sourceRef, implications, examples );
        }
    |
        examples = jmlExamples[]
        {
            self = new JmlRedundantSpec( sourceRef, null, examples );
        }
    ;

jmlImplications []
returns [ JmlSpecCase[] self = null ]
    :
        "implies_that" self = jmlSpecCaseSeq [0]
    ;

jmlExamples []
returns [ JmlExample[] self = null ]
{
    ArrayList examples = new ArrayList();
    JmlExample e = null;
}
    :
        "for_example" e = jmlExample[] { examples.add(e); }
        ( "also" e = jmlExample[] { examples.add(e); } )*
        {
	    self = new JmlExample[examples.size()];
	    examples.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;

jmlGenericSpecCase [boolean isInSpecStatement]
returns [ JmlGenericSpecCase self = null ]
{
    JmlSpecVarDecl v[] = null;
    JmlRequiresClause r[] = null;
    JmlGenericSpecBody b = null;
    TokenReference s = utility.buildTokenReference( LT(1) );
}
    :
	( v = jmlSpecVarDecls[] )?
        (
            (
                r = jmlSpecHeader[]
                ( b = jmlGenericSpecBody[isInSpecStatement] )?
            )
	|
            b = jmlGenericSpecBody[isInSpecStatement]
        )
        {
            self = new JmlGenericSpecCase(s, v, r, b);
        }
    ;

jmlSpecHeader []
returns [JmlRequiresClause[] self = null ]
{
    ArrayList reqs = new ArrayList();
    JmlRequiresClause r;
}
    :   ( r = jmlRequiresClause[] { reqs.add(r); } )+
        {
	    self = new JmlRequiresClause[reqs.size()];
	    reqs.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;

jmlGenericSpecBody [boolean isInSpecStatement]
returns [ JmlGenericSpecBody self = null ]
{
    JmlSpecBodyClause simple[] = null;
    JmlGenericSpecCase generic[] = null;
}
    :
        simple = jmlSpecBody[]
        {
            if (isInSpecStatement) {
                verifySimpleSpecStatementBody(simple);
            } else {
                verifySimpleSpecBody(simple);
            }
            self = new JmlGenericSpecBody(simple);
        }
    |
        LCURLY_VBAR
        generic = jmlGenericSpecCaseSeq[isInSpecStatement]
        VBAR_RCURLY
        {
            self = new JmlGenericSpecBody(generic);
        }
    ;

jmlGenericSpecCaseSeq [boolean isInSpecStatement]
returns [JmlGenericSpecCase[] self = null ]
{
    ArrayList cases = new ArrayList();
    JmlGenericSpecCase c1, c2;
}
    :
        c1 = jmlGenericSpecCase[isInSpecStatement] { cases.add(c1); }
        (
            "also" c2 = jmlGenericSpecCase[isInSpecStatement]
	    { cases.add(c2); }
        )*
        {
	    self = new JmlGenericSpecCase[cases.size()];
	    cases.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;

jmlSpecBody []
returns [ JmlSpecBodyClause[] self = null ]
{
    ArrayList clauses = new ArrayList();
    JmlSpecBodyClause c;
    int curOrder = JmlSpecBodyClause.PORDER_UNKNOWN;
}
    :
        (
            c = jmlSpecBodyClause[]
            {
                clauses.add(c);
                if (c.preferredOrder() < curOrder) {
                    utility.reportTrouble(
                        new CWarning(c.getTokenReference(),
			    JmlMessages.BAD_SPEC_BODY_ORDER,
			    null));
                }
                curOrder = c.preferredOrder();
            }
        )+
        {
	    self = new JmlSpecBodyClause[clauses.size()];
	    clauses.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;

jmlSpecBodyClause []
returns [ JmlSpecBodyClause self = null ]
    :
	self = jmlEnsuresClause[]
    |
	self = jmlSignalsOnlyClause[]
    |
	self = jmlSignalsClause[]
    |
	self = jmlAssignableClause[]
    | 
        self = jmlCapturesClause[]
    |
	self = jmlDivergesClause[]
    |
	self = jmlWhenClause[]
    |
	self = jmlWorkingSpaceClause[]
    |
	self = jmlDurationClause[]
    |
	self = jmlContinuesClause[]
    |
	self = jmlBreaksClause[]
    |
	self = jmlReturnsClause[]
    |
        self = jmlAccessibleClause[]
    |
        self = jmlCallableClause[]
    |
        self = jmlMeasuredClause[]
    ;

jmlHeavyweightSpec [long mods]
returns [ JmlHeavyweightSpec self = null ]
{
    JmlGenericSpecCase generic = null;
    JmlGeneralSpecCase specCase = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        ( "behavior" | "behaviour" )
        generic = jmlGenericSpecCase[false]
        {
            self = new JmlBehaviorSpec(sourceRef, mods, generic);
        }
    |
        ( "exceptional_behavior" | "exceptional_behaviour" )
        specCase = jmlExceptionalSpecCase[]
        {
            self = new JmlExceptionalBehaviorSpec(sourceRef, mods, specCase);
        }
    |
        ( "normal_behavior" | "normal_behaviour" )
        specCase = jmlNormalSpecCase[]
        {
            self = new JmlNormalBehaviorSpec(sourceRef, mods, specCase);
        }
    ;

jmlExceptionalSpecCase []
returns [ JmlExceptionalSpecCase self = null ]
{
    JmlSpecVarDecl v[] = null;
    JmlRequiresClause h[] = null;
    JmlExceptionalSpecBody b = null;
    TokenReference s = utility.buildTokenReference( LT(1) );
}
    :
	( v = jmlSpecVarDecls[] )?
        (
            (
                h = jmlSpecHeader[]
                ( b = jmlExceptionalSpecBody[] )?
            )
	|
            b = jmlExceptionalSpecBody[]
        )
        {
            self = new JmlExceptionalSpecCase(s, v, h, b);
        }
    ;

jmlExceptionalSpecBody []
returns [ JmlExceptionalSpecBody self = null ]
{
    JmlSpecBodyClause specClauses[] = null;
    JmlGeneralSpecCase specCases[] = null;
}
    :
        specClauses = jmlSpecBody[]
        {
            verifyExceptionalSpecBody(specClauses);
            self = new JmlExceptionalSpecBody(specClauses);
        }
    |
        LCURLY_VBAR specCases = jmlExceptionalSpecCaseSeq[] VBAR_RCURLY
        {
            self = new JmlExceptionalSpecBody(specCases);
        }
    ;

jmlExceptionalSpecCaseSeq []
returns [JmlGeneralSpecCase[] self = null ]
{
    ArrayList cases = new ArrayList();
    JmlExceptionalSpecCase c1, c2;
}
    :
        c1 = jmlExceptionalSpecCase[] { cases.add(c1); }
        (
            "also" c2 = jmlExceptionalSpecCase[] { cases.add(c2); }
        )*
        {
	    self = new JmlGeneralSpecCase[cases.size()];
	    cases.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;

jmlNormalSpecCase []
returns [ JmlNormalSpecCase self = null ]
{
    JmlSpecVarDecl vars[] = null;
    JmlRequiresClause hdr[] = null;
    JmlNormalSpecBody body = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	( vars = jmlSpecVarDecls[] )?
        (
            (
                hdr = jmlSpecHeader[]
                ( body = jmlNormalSpecBody[] )?
            )
	|
            body = jmlNormalSpecBody[]
        )
        {
            self = new JmlNormalSpecCase(sourceRef, vars, hdr, body);
        }
    ;

jmlNormalSpecBody []
returns [ JmlNormalSpecBody self = null ]
{
    JmlSpecBodyClause specClauses[] = null;
    JmlGeneralSpecCase specCases[] = null;
}
    :
        specClauses = jmlSpecBody[]
        {
            verifyNormalSpecBody(specClauses);
            self = new JmlNormalSpecBody(specClauses);
        }
    |
        LCURLY_VBAR specCases = jmlNormalSpecCaseSeq[] VBAR_RCURLY
        {
            self = new JmlNormalSpecBody(specCases);
        }
    ;

jmlNormalSpecCaseSeq []
returns [JmlGeneralSpecCase[] self = null ]
{
    ArrayList cases = new ArrayList();
    JmlNormalSpecCase c1, c2;
}
    :
        c1 = jmlNormalSpecCase[] { cases.add(c1); }
        (
            "also" c2 = jmlNormalSpecCase[] { cases.add(c2); }
        )*
        {
	    self = new JmlGeneralSpecCase[cases.size()];
	    cases.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;

jmlCodeContract [long mods]
returns [ JmlCodeContract self = null ]
{
    JmlSpecBodyClause c = null;
    ArrayList accessibleClauses = new ArrayList();
    ArrayList callableClauses = new ArrayList();
    ArrayList measuredClauses = new ArrayList();
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        "code_contract"
        (
            ( c = jmlAccessibleClause[] { accessibleClauses.add(c); }
            | c = jmlCallableClause[] { callableClauses.add(c); }
            | c = jmlMeasuredClause[] { measuredClauses.add(c); }
            )+
            {
                JmlAccessibleClause[] acs 
                    = new JmlAccessibleClause[accessibleClauses.size()];
                accessibleClauses.toArray(acs);

                JmlCallableClause[] ccs 
                    = new JmlCallableClause[callableClauses.size()];
                callableClauses.toArray(ccs);

                JmlMeasuredClause[] mcs
                    = new JmlMeasuredClause[measuredClauses.size()];
                measuredClauses.toArray(mcs);
                
                self = new JmlCodeContract(sourceRef, mods, acs, ccs, mcs);
            }
        )
    ;

jmlAccessibleClause []
returns [ JmlAccessibleClause self = null ]
{
    boolean redundantly = false;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JmlStoreRef sref[];
}
    :
	(
	    "accessible"
	|
            "accessible_redundantly"
            {
                redundantly = true;
            }
    )
	sref = jmlStoreRefList[]
            {
                self = new JmlAccessibleClause( sourceRef, redundantly, sref );
            }
	SEMI
    ;

jmlCallableClause []
returns [ JmlCallableClause self = null ]
{
    boolean redundantly = false;
    JmlMethodNameList names = null;
    JmlStoreRefKeyword storeRefKeyword = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "callable"
	|
            "callable_redundantly"
            {
                redundantly = true;
            }
        )
	(
            names = jmlMethodNameList []
            {
                self = new JmlCallableClause( sourceRef, redundantly, names );
            }
	|
            storeRefKeyword = jmlStoreRefKeyword[]
            {
                names = new JmlMethodNameList( sourceRef, storeRefKeyword );
                self = new JmlCallableClause( sourceRef, redundantly, names );
            }
        )
	SEMI
    ;

jmlCapturesClause []
returns [ JmlCapturesClause self = null ]
{
    boolean redundantly = false;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JmlStoreRef sref[];
}
    :
	(
	    "captures"
	|
		"captures_redundantly"
	    { redundantly = true; }
	)
	sref = jmlStoreRefList[]
            {
                //@ assert sref != null && sref.length > 0;
                self = new JmlCapturesClause( sourceRef, redundantly, sref );
            }
	SEMI
    ;

jmlMeasuredClause []
returns [ JmlMeasuredClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate pred = null;
    JmlSpecExpression exp = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "measured_by"
	|
        "measured_by_redundantly"
            {
                redundantly = true;
            }
    )
	    (
            "\\not_specified"
        |
            ( exp = jmlSpecExpression[] ( "if" pred = jmlPredicate[] )? )
        )

	    SEMI
	{
	    self = new JmlMeasuredClause(sourceRef, redundantly, exp, pred);
	}
    ;

jmlModelProgram [long mods]
returns [ JmlModelProgram self = null ]
{
    JStatement[] statements;
    ParsingController.TokenWrapper declEnd =
    new ParsingController.TokenWrapper();
    TokenReference sourceRef = utility.buildTokenReference();
}
    :
	"model_program" { isInModelProgram = true; }
	statements = jCompoundStatement[declEnd]
	{
	    utility.flushJavadocTokensWithWarning( declEnd );
	    self = new JmlModelProgram( sourceRef, mods, statements );
	    isInModelProgram = false;
	}
    ;

jmlContinuesClause []
returns [ JmlContinuesClause self = null ]
{
    boolean redundantly = false;
    String label = null;
    JmlPredicate predicate = null;
    boolean isNS = false;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "continues"
	|
            "continues_redundantly" { redundantly = true; }
	)
        ( R_ARROW LPAREN id:IDENT { label = id.getText(); } RPAREN )?
	( predicate = jmlPredicate[] | "\\not_specified" { isNS = true; } )?
	SEMI
	{
	    self = new JmlContinuesClause(
                sourceRef, redundantly, label, predicate, isNS );
	}
    ;

jmlBreaksClause []
returns [ JmlBreaksClause self = null ]
{
    boolean redundantly = false;
    String label = null;
    JmlPredicate predicate = null;
    boolean isNS = false;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "breaks"
	|
            "breaks_redundantly" { redundantly = true; }
	)
        ( R_ARROW LPAREN id:IDENT { label = id.getText(); } RPAREN )?
	( predicate = jmlPredicate[] | "\\not_specified" { isNS = true; } )?
	SEMI
	{
	    self = new JmlBreaksClause(
                sourceRef, redundantly, label, predicate, isNS );
	}
    ;

jmlReturnsClause []
returns [ JmlReturnsClause self = null ]
{
    boolean redundantly = false;
    boolean isNS = false;
    JmlPredicate predicate = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "returns"
	|
            "returns_redundantly" { redundantly = true; }
	)
	( predicate = jmlPredicate[] | "\\not_specified" { isNS = true; } )?
	SEMI
	{
	    self = new JmlReturnsClause(
                sourceRef, redundantly, predicate, isNS );
	}
    ;

jmlExample []
returns [ JmlExample self = null ]
{
    boolean lightWeighted = true;
    long p = 0;
    JmlSpecVarDecl[] vars = null;
    JmlRequiresClause[] hdr = null;
    JmlSpecBodyClause[] body = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        p = jModifiers[]
        {
            if (!isPrivacyOrNull(p)) {
		utility.reportTrouble( new PositionedError(
			sourceRef,
			JmlMessages.BAD_PRIVACY_MODIFIER ));
	    }
        }
        (
            (
                ( "example" { lightWeighted = false; } )?
		( vars = jmlSpecVarDecls[] )?
                ( hdr = jmlSpecHeader[] )? body = jmlSpecBody[]
            )
            {
                verifySimpleSpecBody(body);
		if (lightWeighted) {
		    if (p != 0) {
				utility.reportTrouble(new PositionedError(
					sourceRef,
					JmlMessages.LIGHT_WEIGHTED_EXAMPLE_SPEC));
		    }
		    self = new JmlExample(sourceRef, vars, hdr, body);
		} else {
		    self = new JmlExample(sourceRef, p, vars, hdr, body);
		}
            }
	|
            (
                "exceptional_example" ( vars = jmlSpecVarDecls[] )?
                (
                    hdr = jmlSpecHeader[] ( body = jmlSpecBody[] )?
                    {
                        // check if body is exceptional-example-body.
                        if (body != null) {
                            verifyExceptionalSpecBody(body);
                        }
                        self = new JmlExceptionalExample(sourceRef,
                                                         p,vars,hdr,body);
                    }
                |
                    body = jmlSpecBody[]
                    {
                        // check if body is exceptional-example-body.
                        verifyExceptionalSpecBody(body);
                        self = new JmlExceptionalExample(sourceRef,
                                                         p,vars,hdr,body);
                    }
                )
            )
	|
            (
                "normal_example" ( vars = jmlSpecVarDecls[] )?
                (
                    hdr = jmlSpecHeader[] ( body = jmlSpecBody[] )?
                    {
                        // check if body is normal-example-body.
                        if (body != null) {
                            verifyNormalSpecBody(body);
                        }
                        self = new JmlNormalExample(sourceRef,p,vars,hdr,body);
                    }
                |
                    body = jmlSpecBody[]
                    {
                        // check if body is normal-example-body.
                        verifyNormalSpecBody(body);
                        self = new JmlNormalExample(sourceRef,p,vars,hdr,body);
                    }
                )
            )
        )
    ;

jmlSpecVarDecls []
returns [ JmlSpecVarDecl[] self = null ]
{
    ArrayList decls = new ArrayList();
    JmlSpecVarDecl decl;
}
    :
	(
	    ( decl = jmlForAllVarDecl[]    { decls.add( decl ); } )+
	    ( decl = jmlLocalSpecVarDecl[] { decls.add( decl ); } )*
	|
	    ( decl = jmlLocalSpecVarDecl[] { decls.add( decl ); } )+
	)
	{
	    self = new JmlSpecVarDecl[ decls.size() ];
	    decls.toArray( self );
	    
	}
    ;

jmlForAllVarDecl []
returns [ JmlForAllVarDecl self = null ]
{
    JmlVariableDefinition[] quantifiedVarDecls;
}
    :
	s:"forall" 
	quantifiedVarDecls = jmlQuantifiedVarDecls[] SEMI
	{
	    self = new JmlForAllVarDecl(
		utility.buildTokenReference( s ), quantifiedVarDecls );
	}
    ;

jmlLocalSpecVarDecl []
returns [ JmlLetVarDecl self = null ]
{
    long mods = 0;
    CType type;
    JVariableDefinition[] specVariableDeclarators;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	"old"
	((jmlNullityModifier[]) => mods = jmlNullityModifier[])?
	type = jTypeSpec[]
	specVariableDeclarators = jmlSpecVariableDeclarators[mods, type] SEMI
	{
	    self = new JmlLetVarDecl( sourceRef, specVariableDeclarators );
	}
    ;

jmlRequiresClause []
returns [ JmlRequiresClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate predicate = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "requires"
	|
	    "pre"
	|
	    (
		"requires_redundantly"
	    |
		"pre_redundantly"
	    )
	    { redundantly = true; }
	)
	(   predicate = jmlPredicate[]
      | "\\not_specified"
      | "\\same" 
        { 
            predicate = new JmlPredicateKeyword( sourceRef, 
                                                 Constants.SAME ); 
        }
    )
	SEMI
	{
	    self = new JmlRequiresClause( sourceRef, redundantly, predicate );
	}
    ;

jmlWhenClause []
returns [ JmlWhenClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate predicate = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "when"
	|
            "when_redundantly"
            {
                redundantly = true;
            }
        )
	( predicate = jmlPredicate[] | "\\not_specified" )
	SEMI
	{
	    self = new JmlWhenClause( sourceRef, redundantly, predicate );
	}
    ;

jmlWorkingSpaceClause []
returns [ JmlWorkingSpaceClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate pred = null;
    JmlSpecExpression exp = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "working_space"
	|
            "working_space_redundantly"
            {
                redundantly = true;
            }
        )
	(
            "\\not_specified"
        |
            ( exp = jmlSpecExpression[] ( "if" pred = jmlPredicate[] )? )
        )
	SEMI
	{
	    self = new JmlWorkingSpaceClause(sourceRef, redundantly, exp, pred);
	}
    ;

jmlDurationClause []
returns [ JmlDurationClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate pred = null;
    JmlSpecExpression exp = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "duration"
	|
            "duration_redundantly"
            {
                redundantly = true;
            }
        )
	(
            "\\not_specified"
        |
            ( exp = jmlSpecExpression[] ( "if" pred = jmlPredicate[] )? )
        )
	SEMI
	{
	    self = new JmlDurationClause(sourceRef, redundantly, exp, pred);
	}
    ;

jmlAssignableClause []
returns [ JmlAssignableClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate predicate = null;
    JmlStoreRef sref[];
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "assignable"
	|
	    "modifiable"
	|
	    "modifies"
	|
	    (
		"assignable_redundantly"
	    |
		"modifiable_redundantly"
	    |
		"modifies_redundantly"
	    )
	    { redundantly = true; }
	)
	sref = jmlStoreRefList[]
	SEMI
	{
            //@ assert sref != null && sref.length > 0;
	    self = new JmlAssignableClause( sourceRef, redundantly, sref );
	}
    ;

jmlEnsuresClause []
returns [ JmlEnsuresClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate predicate = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "ensures"
	|
	    "post"
	|
	    (
		"ensures_redundantly"
	    |
		"post_redundantly"
	    )
	    { redundantly = true; }
	)
	( predicate = jmlPredicate[] | "\\not_specified" )
	SEMI
	{
	    self = new JmlEnsuresClause( sourceRef, redundantly, predicate );
	}
    ;

jmlSignalsOnlyClause []
returns [ JmlSignalsOnlyClause self = null ]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    boolean isRedundantly = false;
    CClassType[] exceptions = null;
    boolean nothing = false;
}
    :
	(
	    "signals_only"
	|
	    "signals_only_redundantly" { isRedundantly = true; }
	)
    (
        exceptions = jNameList[]
    |
        "\\nothing" { nothing = true; }
    )
	SEMI
	{
	    self = new JmlSignalsOnlyClause(sourceRef, isRedundantly,
                                        exceptions, nothing);
	}
    ;

jmlSignalsClause []
returns [ JmlSignalsClause self = null ]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    boolean redundantly = false;
    boolean notSpecified = false;
    JmlPredicate pred = null;
    CType type = null;
    String ident = null;
}
    :
	(
	    "signals"
	|
            "signals_redundantly"
            {
                redundantly = true;
            }
        |
	    "exsures"
	|
            "exsures_redundantly"
            {
                redundantly = true;
            }
        )
        LPAREN type = jClassTypeSpec[null, null] // WMD TODO
	( id: IDENT { ident = id.getText(); } ) ? RPAREN
        (
            pred = jmlPredicate[]
        |
            "\\not_specified"  { notSpecified = true; }
        ) ?
	SEMI
	{
	    self = new JmlSignalsClause(sourceRef, redundantly,
		type, ident, pred, notSpecified);
	}
    ;

jmlDivergesClause []
returns [ JmlDivergesClause self = null ]
{
    boolean redundantly = false;
    JmlPredicate predicate = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	(
	    "diverges"
	|
            "diverges_redundantly"
            {
                redundantly = true;
            }
        )
	( predicate = jmlPredicate[] | "\\not_specified" )
	SEMI
	{
	    self = new JmlDivergesClause( sourceRef, redundantly, predicate );
	}
    ;

jmlPredicate []
returns [JmlPredicate self=null]
{
    JmlSpecExpression expr = null;
}
    :
	expr = jmlSpecExpression[]
	{
	    self = new JmlPredicate( expr );
	}
    ;

jmlSpecExpressionList []
returns [JmlSpecExpression[] self = null]
{
    ArrayList specArrayList = null;
    JmlSpecExpression spec = null;
}
    :
	spec = jmlSpecExpression[]
	{
	    specArrayList = new ArrayList();
	    specArrayList.add( spec );
	}
	( COMMA spec = jmlSpecExpression[] { specArrayList.add( spec ); } )*
	{
	    self = new JmlSpecExpression[ specArrayList.size() ];
	    specArrayList.toArray( self );
	    
	}
    ;

jmlSpecExpression []
returns [JmlSpecExpression self = null]
{
    JExpression expr = null;
}
    :
	expr = jExpression[]
	{
	    self = new JmlSpecExpression( expr );
	}
    ;

jmlPrimary []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JmlSpecExpression specExpression;
    JExpression expression;
    JmlStoreRef[] storeRefList;
    JmlMethodNameList names = null;
    JmlSpecExpression[] specExpressionList;
    CType type = null;
    JmlStoreRefExpression storeRefExpression = null;
}
	:
		"\\result"  { self = new JmlResultExpression( sourceRef ); }
	|
		"\\old" LPAREN 
                    specExpression = jmlSpecExpression[] 
                    ( COMMA label:IDENT)?
                RPAREN
		{ self = new JmlOldExpression( sourceRef, 
                                       specExpression,
                                       (label!=null? label.getText(): null) );
        }
	|
		"\\pre" LPAREN specExpression = jmlSpecExpression[] RPAREN
		{ self = new JmlPreExpression( sourceRef, specExpression ); }
	|
		"\\not_modified" LPAREN storeRefList = jmlStoreRefList[] RPAREN
		{ self = new JmlNotModifiedExpression( sourceRef, storeRefList ); }
	|
		"\\only_accessed" LPAREN storeRefList = jmlStoreRefList[] RPAREN
		{ self = new JmlOnlyAccessedExpression( sourceRef, storeRefList ); }
	|
		"\\not_assigned" LPAREN storeRefList = jmlStoreRefList[] RPAREN
		{ self = new JmlNotAssignedExpression( sourceRef, storeRefList ); }
	|
            
		"\\only_called" LPAREN names = jmlMethodNameList [] RPAREN
		{ self = new JmlOnlyCalledExpression( sourceRef, names ); }
	|
		"\\only_captured" LPAREN storeRefList = jmlStoreRefList[] RPAREN
		{ self = new JmlOnlyCapturedExpression( sourceRef, storeRefList ); }
	|
		"\\only_assigned" LPAREN storeRefList = jmlStoreRefList[] RPAREN
		{ self = new JmlOnlyAssignedExpression( sourceRef, storeRefList ); }
	|
		"\\fresh" LPAREN specExpressionList = jmlSpecExpressionList[] RPAREN
		{ self = new JmlFreshExpression( sourceRef, specExpressionList ); }
	|
		"\\working_space" LPAREN expression = jExpression[] RPAREN
		{ self = new JmlWorkingSpaceExpression( sourceRef, expression ); }
	|
		"\\space" LPAREN specExpression = jmlSpecExpression[] RPAREN
		{ self = new JmlSpaceExpression( sourceRef, specExpression ); }
	|
		"\\duration" LPAREN expression = jExpression[] RPAREN
		{ self = new JmlDurationExpression( sourceRef, expression ); }
	|
		"\\reach" LPAREN
		specExpression = jmlSpecExpression[]
		( COMMA type = jClassTypeSpec[null, null] // WMD TODO
			( COMMA storeRefExpression = jmlStoreRefExpression[] )? )?
		RPAREN
		{ self = new JmlReachExpression( sourceRef, specExpression, type,
				storeRefExpression );
          /*
           * The following isn't used, but is kept as an example
           * of how to deprecate something, if you want to make
           * something deprecated.  It can be deleted when you have
           * something you really want to deprecate.
           *
           * utility.reportTrouble(
           *     new CWarning( sourceRef,
           *                   JmlMessages.DEPRECATED_REACH ));
           */
        }
	|
		infDesc:INFORMAL_DESC
		{ self = new JmlInformalExpression( sourceRef, infDesc.getText() ); }
	|
		"\\nonnullelements" LPAREN specExpression = jmlSpecExpression[] RPAREN
		{ self = new JmlNonNullElementsExpression( sourceRef,
				specExpression ); }
	|
		"\\typeof" LPAREN specExpression = jmlSpecExpression[] RPAREN
		{ self = new JmlTypeOfExpression( sourceRef, specExpression ); }
	|
		"\\elemtype" LPAREN specExpression = jmlSpecExpression[] RPAREN
		{ self = new JmlElemTypeExpression( sourceRef, specExpression ); }
	|
		"\\type" LPAREN type = jTypeSpec[] RPAREN
		{ self = new JmlTypeExpression( sourceRef, type ); }
	|
		"\\lockset"  { self = new JmlLockSetExpression( sourceRef ); }
	|
		"\\max" LPAREN specExpression=jmlSpecExpression[] RPAREN
		    {  self = new JmlMaxExpression(sourceRef,specExpression); }
	|
		"\\is_initialized" LPAREN type = jClassTypeSpec[null, null] RPAREN
// WMD TODO
		{ self = new JmlIsInitializedExpression( sourceRef, type ); }
	|
		"\\invariant_for" LPAREN specExpression = jmlSpecExpression[] RPAREN
		{ self = new JmlInvariantForExpression( sourceRef, specExpression ); }
	|
		self = jmlWarnExpression []
	|
		self = jmlMathModeExpression []
	;

jmlSetComprehension [CType type]
returns [JExpression self = null]
{
    long mods = 0;
    CType memberType;
    int bounds = 0;
    JmlPredicate pred;
    JExpression supersetPred;
}
    :
	start:LCURLY

	((jmlNullityModifier[]) => mods = jmlNullityModifier[])?
	memberType = jTypeSpec[]
	id:IDENT ( LBRACK RBRACK { bounds += 1; } )*
	{
	    if (bounds > 0) {
            if( utility.getCompiler().universeChecks() ) {
                utility.reportTrouble(
                    new CWarning(utility.buildTokenReference(id), 
                    CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                    null));
            } else {
                utility.reportTrouble(
		            new CWarning(utility.buildTokenReference(id),
         			MjcMessages.OLD_STYLE_ARRAY_BOUNDS));
            }
            memberType = new CArrayType(memberType, bounds, null);
	    }
	}
	BOR // '|', denotes "such that"
	supersetPred = jPostfixExpression[]
	// semantic check required to verify that supersetPred invokes
	// a "has(Object id2)" method and id2.equals(id.getText)
	LAND pred = jmlPredicate[]
	RCURLY
	{
	    self = new JmlSetComprehension(
		utility.buildTokenReference(start), mods, type,
		memberType, id.getText(), supersetPred, pred );
	}
    ;

// Note that the parentheses had to be factored out
// in jPrimaryExpr.  Also for ANTLR, we make the second expression
// optional, even though we'd like to say it's the first one, and thus
// we use a spec_expression for both the range and the body.
jmlSpecQuantifiedExprRest [TokenReference sourceRef]
returns [JmlSpecQuantifiedExpression self = null]
{
    int oper = -1;
    JmlVariableDefinition[] quantifiedVarDecls;
    JmlSpecExpression predicate = null;
    JmlSpecExpression specExpression = null;
}
    :
	(	"\\forall"	{ oper = Constants.OPE_FORALL; }
	|	"\\exists"	{ oper = Constants.OPE_EXISTS; }
	|	"\\max"		{ oper = Constants.OPE_MAX; }
	|	"\\min"		{ oper = Constants.OPE_MIN; }
	|	"\\num_of"	{ oper = Constants.OPE_NUM_OF; }
	|	"\\product"	{ oper = Constants.OPE_PRODUCT; }
	|	"\\sum"		{ oper = Constants.OPE_SUM; }
	)
	quantifiedVarDecls = jmlQuantifiedVarDecls[] SEMI
	(
	    predicate = jmlSpecExpression[]
	    ( SEMI specExpression = jmlSpecExpression[] )?
	    {
		if (specExpression == null) {
		    // really the predicate is optional
		    specExpression = predicate;
		    predicate = null;
		}
	    }
	|
	    SEMI specExpression = jmlSpecExpression[]
	)
	{
	    self = new JmlSpecQuantifiedExpression( sourceRef, oper,
		quantifiedVarDecls,
		predicate == null ? null : new JmlPredicate( predicate ),
		specExpression );
	}
    ;

jmlQuantifiedVarDecls []
returns [JmlVariableDefinition[] self = null]
{
    CType baseType;
    CType type;
    long mods = 0;
    int bounds = 0;
    ArrayList decls = null;
}
    :

	( (jmlNullityModifier[]) => mods = jmlNullityModifier[]
	  baseType = jTypeSpec[]
	|
	  baseType = jTypeSpec[]
	)
	id1: IDENT
	(
	    LBRACK RBRACK { bounds += 1; }
	)*
	{
	    if (bounds > 0) {
            if( utility.getCompiler().universeChecks() ) {
                utility.reportTrouble(
                    new CWarning(utility.buildTokenReference(id1), 
                    CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                    null));
            } else {
                utility.reportTrouble(
                    new CWarning(utility.buildTokenReference(id1),
                    MjcMessages.OLD_STYLE_ARRAY_BOUNDS, null));
            }
            type = new CArrayType(baseType, bounds, null);
	    } else {
            type = baseType;
	    }
	    bounds = 0;
	    decls = new ArrayList();
	    decls.add(
		new JmlVariableDefinition(
		    utility.buildTokenReference( id1 ), mods, type,
		    id1.getText(), null ));
	}
	(
	    COMMA
	    id2: IDENT
	    (
		LBRACK RBRACK { bounds += 1; }
	    )*
	    {
		if (bounds > 0) {
            if( utility.getCompiler().universeChecks() ) {
                utility.reportTrouble(
                    new CWarning(utility.buildTokenReference(id2), 
                    CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                    null));
            } else {
                utility.reportTrouble(
                    new CWarning(utility.buildTokenReference(id2),
                    MjcMessages.OLD_STYLE_ARRAY_BOUNDS, null));
            }
		    type = new CArrayType(baseType, bounds, null);
		} else {
		    type = baseType;
		}
		bounds = 0;
		decls.add(
		    new JmlVariableDefinition(
			utility.buildTokenReference( id2 ), mods, type,
			id2.getText(), null ));
	    }
	)*
	{
	    self = new JmlVariableDefinition[ decls.size() ];
	    decls.toArray( self );
	    
	}
    ;

// jmlNullityModifier is a special grammar rule that allows us to parse both
// keywords and IDENTs that match "non_null" and "nullable".  This is
// necessary in some circumstances where a nullity keyword might be followed
// by an identifier.  In such a case, an occurrence of "non_null" or
// "nullable" is *always* taken to be a nullity modifier. If this is not the
// intent of the specifier, then he/she need only provide an explicit nullity
// modifier in addition to the identifier named non_null or nullable.
jmlNullityModifier []
returns [long mods = 0]
    :
	// Is the next token an identifier named "non_null"?
	{ "non_null".equals(LT(1).getText()) }?
	IDENT 
	{   // Interpret the identifier as a nullity modifier.
	    mods = Constants.ACC_NON_NULL; 
	}
    |
	// Is the next token an identifier named "nullable"?
	{ "nullable".equals(LT(1).getText()) }?
	IDENT 
	{   // Interpret the identifier as a nullity modifier.
	    mods = Constants.ACC_NULLABLE; 
	}
    |
	"non_null" { mods = Constants.ACC_NON_NULL; }
    |
	"nullable" { mods = Constants.ACC_NULLABLE; }
    ;

jmlNonNullNullityModifier []
returns [long mods = 0]
    :
	// Is the next token an identifier named "non_null"?
	{ "non_null".equals(LT(1).getText()) }?
	IDENT 
	{   // Interpret the identifier as a nullity modifier.
	    mods = Constants.ACC_NON_NULL; 
	}
    |
	"non_null" { mods = Constants.ACC_NON_NULL; }
    ;

jmlSpecVariableDeclarators [long mods, CType type]
returns [JVariableDefinition[] self = null]
{
    ArrayList vars = new ArrayList();
    JVariableDefinition	decl;
}
    :
	decl = jmlSpecVariableDeclarator[mods, type]
	{ vars.add(decl); }
	(
	    COMMA decl = jmlSpecVariableDeclarator[mods, type]
	    { vars.add(decl); }
	)*
	{
	    self = new JmlVariableDefinition[vars.size()];
	    self = (JVariableDefinition[]) vars.toArray(self);
	    
	}
    ;

jmlSpecVariableDeclarator [long mods, CType type]
returns [JVariableDefinition self = null]
{
    int	bounds = 0;
    JExpression	expr = null;
    TokenReference sourceRef = utility.buildTokenReference();
}
    :
	ident:IDENT
	( LBRACK RBRACK { bounds += 1; } )*
	( ASSIGN expr = jmlSpecInitializer[] )?
	{
	    if (bounds > 0) {
            if( utility.getCompiler().universeChecks() ) {
                utility.reportTrouble(new CWarning(sourceRef, 
                    CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, null));
            } else {
                utility.reportTrouble(new CWarning(sourceRef,
                    MjcMessages.OLD_STYLE_ARRAY_BOUNDS, null));
            }
            type = new CArrayType(type, bounds, null);
	    }
	    self = new JmlVariableDefinition(sourceRef, mods, type, ident.getText(), expr);
	}
    ;


jmlSpecArrayInitializer[]
returns [JArrayInitializer self = null]
{
    JExpression		expr = null;
    ArrayList		vect = new ArrayList();
    TokenReference	sourceRef = utility.buildTokenReference();
}
    :
	LCURLY
	(
	    expr = jmlSpecInitializer[]
	    { vect.add(expr); }
	    (
		// CONFLICT: does a COMMA after an initializer start a new
		//           initializer or start the option ',' at end?
		//           ANTLR generates proper code by matching
		//			 the comma as soon as possible.
		options { warnWhenFollowAmbig = false; } :
		COMMA expr = jmlSpecInitializer[]
		{ vect.add(expr); }
	    )*
	)?
	(
	    COMMA
	    {
		utility.reportTrouble(
		    new CWarning(sourceRef,
			MjcMessages.STRAY_COMMA, null));
	    }
	)?
	RCURLY
	{
	    self = new JArrayInitializer(sourceRef,
		(JExpression[]) vect.toArray(new JExpression[0]));
	    
	}
    ;

jmlSpecInitializer []
returns [JExpression self = null]
    :
	self = jmlSpecExpression[]
    |
	self = jmlSpecArrayInitializer[]
    ;

jStatement []
returns [JStatement self = null]
{
}
    :
	self = mjStatement[]	// inherited statements
    {
         if (self instanceof JExpressionStatement) {
             JExpressionStatement exprStmt = (JExpressionStatement)self;
             JExpression expr = exprStmt.expr();
             if (expr instanceof JAssignmentExpression) {
                 self = new JmlAssignmentStatement(exprStmt);
             }
         }
    }
    |
	self = jmlHenceByStatement[]
    |
	self = jmlAssertStatement[]
    |
	self = jmlAssumeStatement[]
    |
	self = jmlDebugStatement[]
    |
	self = jmlSetStatement[]
    |
	self = jmlRefiningStatement[]
    |
	self = jmlUnreachableStatement[]
    |
	self = jmlModelProgStatement[]
	{
	    if (!isInModelProgram) {
		utility.reportTrouble(
		    new PositionedError( self.getTokenReference(),
			JmlMessages.ILLEGAL_MODEL_PROG_STATEMENT ));
	    }
	}
    ;

/** A declaration is the creation of a reference or primitive-type variable
 *  Create a separate Type/Var tree for each var in the var list.
 */
jDeclaration []
returns [JVariableDeclarationStatement self = null]
{
    long         mods;
    CType        type;
    JVariableDefinition[] vars;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        mods = jModifiers[] type = jTypeSpec[]
        vars = jVariableDefinitions[mods, type]
        {
            if ( Utils.hasOtherFlags(mods,
                    Constants.ACC_FINAL
                    | Constants.ACC_GHOST
                    | Constants.ACC_NON_NULL
                    | Constants.ACC_NULLABLE) ) {
		        utility.reportTrouble(
			        new PositionedError(sourceRef,
			            JmlMessages.INVALID_LOCAL_MODIFIER,
                        utility.getModifierNames(mods)));
            }
            if ( Utils.hasFlag(mods, Constants.ACC_GHOST) ) {
                // this declaration is in an annotation
		        for (int i = 0; i < vars.length; i++) {
			        vars[i] = new JmlVariableDefinition(
				                vars[i].getTokenReference(), mods, type, 
                                vars[i].ident(), vars[i].expr() );
		        }
		    }
            self = new JVariableDeclarationStatement(
                sourceRef, vars, utility.getStatementComment() );
        }
    ;

jLoopStatement []
returns [JStatement self = null]
{
    JmlLoopInvariant[] invs = null;
    JmlVariantFunction[] vars = null;
    JStatement stmt = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	(
	    invs = jmlLoopInvariantList[]
	    (vars = jmlVariantFunctionList[]) ?
	|
	    vars = jmlVariantFunctionList[]
	)
	(
	    stmt = jLabeledStatement []
	    {
		if (!(((JLabeledStatement)stmt).stmt()
			instanceof JLoopStatement)) {
		    utility.reportTrouble(
			new PositionedError(
			    stmt.getTokenReference(),
			    JmlMessages.LOOP_STMT_EXPECTED));
			}
	    }
	|
	    stmt = jSimpleLoopStatement[]
	)
	{
	    self = new JmlLoopStatement( sourceRef, invs, vars, stmt, comments );
	}
    |
		self = jSimpleLoopStatement[]
    ;

//==============================================================================
// WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING 
// Any changes made to mjOrJmlUnaryExpressionNotPlusMinus (below) must be copied
// textually into Mjc.g (except for the part in between the "START/END of
// inserted JML-unique production(s)").
// WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING 

mjOrJmlUnaryExpressionNotPlusMinus []
returns [JExpression self = null]
{
    long             mods = 0;
    JExpression      expr;
    CType            dest=null;
    TokenReference   sourceRef = utility.buildTokenReference( LT(1) );
    CUniverse        array_univ = null;
    CUniverse        elem_univ = null;
    boolean          isExplicitNonNull = false;
}
 :
        op_b:BNOT self = jUnaryExpression[]
        { self = exprFactory.createUnaryExpression(op_b, self.getTokenReference(), Constants.OPE_BNOT, self); }
    |
        op_l:LNOT self = jUnaryExpression[]
        { self = exprFactory.createUnaryExpression(op_l, self.getTokenReference(), Constants.OPE_LNOT, self); }
    |
	// If typecast is built in type, must be numeric operand
        // array with one modifier
	( LPAREN (jUniverseSpec[])? jBuiltInTypeSpec[null,null] RPAREN )=>
	LPAREN (array_univ = jUniverseSpec[])? dest = jBuiltInTypeSpec[null, array_univ] RPAREN
	expr = jUnaryExpression[] 
	{ self = new JCastExpression(sourceRef, expr, dest); }
    |
	// If typecast is built in type, must be numeric operand
        // array with two modifiers
	( LPAREN (jUniverseSpec[] jUniverseSpec[])? jBuiltInTypeSpec[null,null] RPAREN )=>
	LPAREN (array_univ = jUniverseSpec[] elem_univ = jUniverseSpec[])? dest = jBuiltInTypeSpec[elem_univ, array_univ] RPAREN
	expr = jUnaryExpression[] 
	{ self = new JCastExpression(sourceRef, expr, dest); }
    | // START of inserted JML-unique production(s)
	( LPAREN 
	  jmlNonNullNullityModifier[]
	  (jClassTypeSpec[null,null])? 
	  RPAREN 
	  jUnaryExpressionNotPlusMinus[]
	) =>
	    LPAREN 
	    mods = jmlNonNullNullityModifier[]
	    (dest = jClassTypeSpec[elem_univ,array_univ])? 
	    RPAREN
	    expr = jUnaryExpressionNotPlusMinus[]
        { boolean isNonNull = (mods == Constants.ACC_NON_NULL);
	  if (isNonNull) {
	     self = new JmlCastExpression(sourceRef, expr, dest, isNonNull); 
	  } else {
	     self = new JCastExpression(sourceRef, expr, dest); 
	  }
	}
    // END of inserted JML-unique production(s)
    |
	// Have to backtrack to see if operator follows.  If no operator
	// follows, it's a typecast.  No semantic checking needed to parse.
	// if it _looks_ like a cast, it _is_ a cast; else it's a "(expr)"

	// WMD
	// old:
	// ( LPAREN jClassTypeSpec[] RPAREN jUnaryExpressionNotPlusMinus[] )=>
	// LPAREN dest = jClassTypeSpec[] RPAREN
	// expr = jUnaryExpressionNotPlusMinus[]

	( LPAREN (jUniverseSpec[])?
	  jClassTypeSpec[null,null] RPAREN
	  jUnaryExpressionNotPlusMinus[] 
	) =>
	LPAREN 
	  (array_univ = mjUniverseImplicitPeerSpec[] elem_univ = jUniverseSpec[])? 
	  dest = jClassTypeSpec[elem_univ,array_univ] 
        RPAREN
	  expr = jUnaryExpressionNotPlusMinus[]
	{ self = new JCastExpression(sourceRef, expr, dest); }
        // WMD added an implicit peer for the array universe
        // prevents a spurious warning
    |
    // allow cast of arrays with two universe specifiers
	( LPAREN (jUniverseSpec[] jUniverseSpec[])? jClassTypeSpec[null,null] RPAREN
	  jUnaryExpressionNotPlusMinus[] 
	) =>
	LPAREN 
	    (array_univ = jUniverseSpec[] elem_univ = jUniverseSpec[])? 
	    dest = jClassTypeSpec[elem_univ,array_univ] 
	RPAREN
	    expr = jUnaryExpressionNotPlusMinus[]
	{ self = new JCastExpression(sourceRef, expr, dest); }
    |
        self = jPostfixExpression[]
    ;


jSimpleLoopStatement []
returns [JLoopStatement self = null]
{
}
    :
	self = jForStatement[]
    |
	self = jWhileStatement[]
    |
	self = jDoStatement[]
    ;

jmlLoopInvariantList []
returns [JmlLoopInvariant[] self = null]
{
    JmlLoopInvariant inv = null;
    ArrayList invList = new ArrayList();
}
    :
	( inv = jmlLoopInvariant[] { invList.add(inv); } )+
	{
	    if (invList.size() > 0) {
		self = new JmlLoopInvariant[invList.size()];
		invList.toArray( self );
	    }
	    
    	}
    ;

jmlVariantFunctionList []
returns [JmlVariantFunction[] self = null]
{
    JmlVariantFunction var = null;
    ArrayList varList = new ArrayList();
}
    :
	( var = jmlVariantFunction[] { varList.add(var); } )+
	{
	    if (varList.size() > 0) {
		self = new JmlVariantFunction[varList.size()];
		varList.toArray( self );
	    }
	    
    	}
    ;

jmlLoopInvariant []
returns [JmlLoopInvariant self = null]
{
    boolean isRedundantly = false;
    JmlPredicate predicate = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	( "maintaining" | "loop_invariant" |
	    ( "maintaining_redundantly" | "loop_invariant_redundantly" )
	    { isRedundantly = true; } )
	predicate = jmlPredicate[] SEMI
	{
	    self = new JmlLoopInvariant( sourceRef, isRedundantly, predicate );
	}
    ;

jmlVariantFunction []
returns [JmlVariantFunction self = null]
{
    boolean isRedundantly = false;
    JmlSpecExpression specExpression = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	( "decreasing" | "decreases" |
	    ( "decreasing_redundantly" | "decreases_redundantly" )
	    { isRedundantly = true; } )
	specExpression = jmlSpecExpression[] SEMI
	{
	    self = new JmlVariantFunction( sourceRef, isRedundantly,
		specExpression );
	}
    ;

jmlAssertStatement []
returns [JmlAssertStatement self = null]
{
    JExpression expression = null;
    JExpression throwMessage = null;
    JavaStyleComment[] comments = utility.getStatementComment();
    JmlPredicate predicate = null;
}
    :
	s1:AFFIRM predicate = jmlPredicate[]
	( COLON throwMessage = jExpression[] )? SEMI
	{
	    self = new JmlAssertStatement( utility.buildTokenReference(s1),
		false, predicate, throwMessage, comments );
	}
    |
	s2:"assert_redundantly" predicate = jmlPredicate[]
	( COLON throwMessage = jExpression[] )? SEMI
	{
	    self = new JmlAssertStatement( utility.buildTokenReference(s2),
		true, predicate, throwMessage, comments );
	}
    ;

jmlHenceByStatement []
returns [JmlHenceByStatement self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    boolean isRedundantly = false;
    JmlPredicate predicate;
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	( "hence_by" | "hence_by_redundantly" { isRedundantly = true; } )
	predicate = jmlPredicate[] SEMI
	{
	    self = new JmlHenceByStatement( sourceRef, isRedundantly,
		predicate, comments );
	}
    ;

jmlAssumeStatement []
returns [JmlAssumeStatement self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    boolean isRedundantly = false;
    JmlPredicate predicate;
    JExpression throwMessage = null;
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	( "assume" | "assume_redundantly" { isRedundantly = true; } )
	predicate = jmlPredicate[] ( COLON throwMessage = jExpression[] )? SEMI
	{
	    self = new JmlAssumeStatement(
		sourceRef, isRedundantly, predicate, throwMessage, comments );
	}
    ;

jmlDebugStatement []
returns [JmlDebugStatement self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JExpression expression;
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	"debug" expression = jExpression[] SEMI
	{
	    self = new JmlDebugStatement( sourceRef, expression, comments );
	}
    ;

jmlSetStatement []
returns [JmlSetStatement self = null]
{
    JExpression assignmentExpression;
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	s:"set" assignmentExpression = jAssignmentExpression[] SEMI
	{
	    self = new JmlSetStatement(
		utility.buildTokenReference( s ), assignmentExpression,
		comments );
	}
    ;

jmlRefiningStatement []
returns [JmlRefiningStatement self = null]
{
    JmlGeneralSpecCase gencase = null;
    JmlSpecStatement specstmt = null;
    JStatement body;
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	r:"refining" 
        ( gencase = jmlGenericSpecCase[true] | specstmt = jmlSpecStatement[] ) 
        body = jStatement[]
	{
            if (gencase != null) {
                specstmt = new JmlSpecStatement(
                   utility.buildTokenReference( r ), gencase, comments);
            }
            //@ assert specstmt != null;
	    self = new JmlRefiningStatement(
		utility.buildTokenReference( r ), specstmt, gencase != null, 
                body,
                comments );
	}
    ;

jmlUnreachableStatement []
returns [JmlUnreachableStatement self = null]
{
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	s:"unreachable" SEMI
	{
	    self = new JmlUnreachableStatement(
		utility.buildTokenReference( s ), comments );
	}
    ;

jmlModelProgStatement []
returns [JmlModelProgStatement self = null]
{
}
    :
	self = jmlNondeterministicChoice[]
    |
	self = jmlNondeterministicIf[]
    |
	self = jmlInvariantStatement[]
    |
	self = jmlSpecStatement[]
    ;

jmlNondeterministicChoice []
returns [JmlNondetChoiceStatement self = null]
{
    ArrayList stmts = new ArrayList();
    JStatement[] compoundStatement = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	"choose" compoundStatement = jCompoundStatement[null]
	{ stmts.add( new JBlock( sourceRef, compoundStatement,
		    null ) ); }
	(
	    "or" compoundStatement = jCompoundStatement[null]
	    { stmts.add( new JBlock( sourceRef, compoundStatement,
			null ) ); }
	)*
	{
	    JBlock[] alternativeStatements
	    = new JBlock[stmts.size()];
	    stmts.toArray( alternativeStatements );
	    self = new JmlNondetChoiceStatement( sourceRef,
		alternativeStatements,
		utility.getStatementComment() );
	}
    ;

jmlNondeterministicIf []
returns [JmlNondetIfStatement self = null]
{
    JmlGuardedStatement[] guardedStatements = null;
    JStatement[] elseStatements = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	"choose_if" guardedStatements = jmlGuardedStatements[]
	(
            // CONFLICT: the old "dangling-else" problem...
            // ANTLR generates proper code matching as soon as possible,
            // binding dangling else's to the innermost if statement
            // Hush warning.
            options { warnWhenFollowAmbig = false; } :
            "else" elseStatements = jCompoundStatement[null]
        )?
	{
	    self = new JmlNondetIfStatement( sourceRef,
		guardedStatements,
		elseStatements,
		utility.getStatementComment() );
	}
    ;

jmlGuardedStatements []
returns [JmlGuardedStatement[] self = null]
{
    ArrayList stmts = new ArrayList();
    JmlGuardedStatement statement = null;
}
    :
	statement = jmlGuardedStatement[] { stmts.add( statement ); }
	( "or" statement = jmlGuardedStatement[] { stmts.add( statement ); } )*
	{
	    self = new JmlGuardedStatement[stmts.size()];
	    stmts.toArray(self);
	}
    ;

jmlGuardedStatement []
returns [JmlGuardedStatement self = null]
{
    JmlAssumeStatement assumeStatement;
    JStatement stmt;
    ArrayList stmts = new ArrayList();
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	LCURLY assumeStatement = jmlAssumeStatement[]
	stmt = jStatement[] { stmts.add( stmt ); }
	( stmt = jStatement[] { stmts.add( stmt ); } )*
	RCURLY
	{
	    JStatement[] statements = new JStatement[stmts.size()];
	    stmts.toArray(statements);
	    self = new JmlGuardedStatement( sourceRef, assumeStatement,
		statements );
	}
    ;

jmlSpecStatement []
returns [JmlSpecStatement self = null]
{
    JmlGeneralSpecCase specCase = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JavaStyleComment[] comments = utility.getStatementComment();
}
    :
	(
	    ( "behavior" | "behaviour" )
        specCase = jmlGenericSpecCase[true]
	|
	    ( "exceptional_behavior" | "exceptional_behaviour" )
        specCase = jmlExceptionalSpecCase[]
	|
	    ( "normal_behavior" | "normal_behaviour" )
        specCase = jmlNormalSpecCase[]
	|
	    ( "abrupt_behavior" | "abrupt_behaviour" )
        specCase = jmlAbruptSpecCase[]
	)
	{
	    self = new JmlSpecStatement( sourceRef, specCase, comments );
	}
    ;

jmlAbruptSpecCase []
returns [ JmlAbruptSpecCase self = null ]
{
    JmlSpecVarDecl v[] = null;
    JmlRequiresClause r[] = null;
    JmlAbruptSpecBody b = null;
    TokenReference s = utility.buildTokenReference( LT(1) );
}
    :
	( v = jmlSpecVarDecls[] )?
        (
            (
                r = jmlSpecHeader[]
                ( b = jmlAbruptSpecBody[] )?
            )
	|
            b = jmlAbruptSpecBody[]
        )
        {
            self = new JmlAbruptSpecCase(s, v, r, b);
        }
    ;

jmlAbruptSpecBody []
returns [ JmlAbruptSpecBody self = null ]
{
    JmlSpecBodyClause specClauses[] = null;
    JmlGeneralSpecCase specCases[] = null;
}
    :
        specClauses = jmlSpecBody[]
        {
            verifyAbruptSpecBody(specClauses);
            self = new JmlAbruptSpecBody(specClauses);
        }
    |
        LCURLY_VBAR specCases = jmlAbruptSpecCaseSeq[] VBAR_RCURLY
        {
            self = new JmlAbruptSpecBody(specCases);
        }
    ;

jmlAbruptSpecCaseSeq []
returns [JmlGeneralSpecCase[] self = null ]
{
    ArrayList cases = new ArrayList();
    JmlAbruptSpecCase c1, c2;
}
    :
        c1 = jmlAbruptSpecCase[] { cases.add(c1); }
        (
            "also" c2 = jmlAbruptSpecCase[] { cases.add(c2); }
        )*
        {
	    self = new JmlGeneralSpecCase[cases.size()];
	    cases.toArray(self);
	    
            //@ assert self.length > 0;
        }
    ;



jmlInvariantStatement []
returns [JmlInvariantStatement self = null]
{
    boolean isRedundantly = false;
    JmlPredicate predicate;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
	( "invariant" | "invariant_redundantly" { isRedundantly = true; } )
	predicate = jmlPredicate[] SEMI
	{
	    self = new JmlInvariantStatement( sourceRef, isRedundantly,
		predicate, utility.getStatementComment() );
	}
    ;

// conditional test (level 12)
jConditionalExpression []
returns [JExpression self = null]
{
    JExpression		middle;
    JExpression		right;
}
    :
	self = jmlEquivalenceExpression[]
	(
	    QUESTION middle = jConditionalExpression[] COLON right = jConditionalExpression[]
	    { self = new JConditionalExpression(self.getTokenReference(), self, middle, right); }
	)?
    ;


// equality/inequality (==/!=) (level 6)
jEqualityExpression []
returns [JExpression self = null]
{
    int			operator = -1;
    JExpression		right;
}
    :
	self = jRelationalExpression[]
	(
	    (
		NOT_EQUAL { operator = Constants.OPE_NE; }
	    |
		EQUAL { operator = Constants.OPE_EQ; }
	    )
	    right = jRelationalExpression[]
	    { self = new JmlEqualityExpression(self.getTokenReference(), operator, self, right); }
	)*
    ;


// boolean relational expressions (level 5)
jRelationalExpression []
returns [JExpression self = null]
{
    int			operator = -1;
    JExpression		right;
    CType			type;
}
    :
	self = jShiftExpression[]
	(	(	(	LT { operator = Constants.OPE_LT; }
		|	GT { operator = Constants.OPE_GT; }
		|	LE { operator = Constants.OPE_LE; }
		|	GE { operator = Constants.OPE_GE; }
		)
		right = jShiftExpression[]
		{ self = new JmlRelationalExpression(self.getTokenReference(), operator, self, right); }
	    )*
	|	"instanceof" type = jTypeSpec[]
	    { self = new JInstanceofExpression(self.getTokenReference(), self, type); }
	|	SUBTYPE_OF right = jShiftExpression[]
	    { self = new JmlRelationalExpression(self.getTokenReference(), Constants.OPE_SUBTYPE, self, right); }
	)
    ;


// equivalence test (level 11.75)
jmlEquivalenceExpression []
returns [JExpression self = null]
{
    JExpression right;
    int oper = -1;
}
    :
	self = jmlImpliesExpression[]
	(
	    ( EQUIV 		{ oper = Constants.OPE_EQUIV; }
	    | NOT_EQUIV 	{ oper = Constants.OPE_NOT_EQUIV; }
	    )
	    right = jmlEquivalenceExpression[]
	    {
		self = new JmlRelationalExpression( self.getTokenReference(),
		    oper, self, right );
	    }
	)?
    ;

// implication test (level 11.5)
jmlImpliesExpression []
returns [JExpression self = null]
{
    JExpression right;
}
    :
	self = jLogicalOrExpression[]
	(
	    IMPLIES right = jmlForwardImpliesExpression[]
	    {
		self = new JmlRelationalExpression( self.getTokenReference(),
		    Constants.OPE_IMPLIES, self, right );
	    }
	|
	    BACKWARD_IMPLIES right = jmlBackwardImpliesExpression[]
	    {
		self = new JmlRelationalExpression( self.getTokenReference(),
		    Constants.OPE_BACKWARD_IMPLIES, self, right );
	    }
	)?
    ;

jmlForwardImpliesExpression []
returns [JExpression self = null]
{
    JExpression right;
}
    :
	self = jLogicalOrExpression[]
	(
	    IMPLIES right = jmlForwardImpliesExpression[]
	    {
		self = new JmlRelationalExpression( self.getTokenReference(),
		    Constants.OPE_IMPLIES, self, right );
	    }
	)?
    ;

jmlBackwardImpliesExpression []
returns [JExpression self = null]
{
    JExpression right;
}
    :
	self = jLogicalOrExpression[]
	(
	    BACKWARD_IMPLIES right = jmlBackwardImpliesExpression[]
	    {
		self = new JmlRelationalExpression( self.getTokenReference(),
		    Constants.OPE_BACKWARD_IMPLIES, self, right );
	    }
	)?
    ;

jmlWarnExpression []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JExpression expr = null;
}
    :   op1:"\\warn"      LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op1.getText(), expr, true); }
    |   op2:"\\warn_op"   LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op2.getText(), expr, true, true); }
    |   op3:"\\nowarn"    LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op3.getText(), expr, false); }
    |   op4:"\\nowarn_op" LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op4.getText(), expr, false, true); }
    ;

jmlMathModeExpression []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JExpression expr = null;
}
    :   op1:"\\java_math"      LPAREN expr = jExpression[] RPAREN
        { self = new MJMathModeExpression(sourceRef, op1.getText(), expr, Constants.AMID_JAVA_MATH); }
    |   op2:"\\safe_math"   LPAREN expr = jExpression[] RPAREN
        { self = new MJMathModeExpression(sourceRef, op2.getText(), expr, Constants.AMID_SAFE_MATH); }
    |   op3:"\\bigint_math"    LPAREN expr = jExpression[] RPAREN
        { self = new MJMathModeExpression(sourceRef, op3.getText(), expr, Constants.AMID_BIGINT_MATH); }
    ;

jPrimaryExpression []
returns [JExpression self = null]
{
}
    :	self = mjPrimaryExpression[]
    |   self = jmlPrimary[]
    ;

jParenthesizedExpression []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    boolean isPosLabel = false;
    JmlSpecExpression specExpression;
}
    :
        LPAREN
	self = jParenthesizedExpressionRest[sourceRef]
    ;

jParenthesizedExpressionRest [TokenReference sourceRef]
returns [JExpression self = null]
{
    boolean isPosLabel = false;
    JmlSpecExpression specExpression;
}
    :
	(
	    { LA(1) != LITERAL_BS_max  || LA(2) == LPAREN }?
	    self = jAssignmentExpression[]
	    { self = new JParenthesedExpression( sourceRef, self ); }
	|
	    ( "\\lblneg" | "\\lblpos" { isPosLabel = true; } )
	    lblIdent:IDENT specExpression = jmlSpecExpression[]
	    {
		self = new JmlLabelExpression( sourceRef, isPosLabel,
		    lblIdent.getText(), specExpression );
	    }
	|
	    { LA(1) != LITERAL_BS_max  || LA(2) != LPAREN }?
	    self = jmlSpecQuantifiedExprRest[sourceRef]
	)
	RPAREN
    ;

/** object instantiation.
 *  Trees are built as illustrated by the following input/tree pairs:
 *
 *  new T()
 *
 *  new
 *   |
 *   T --  ELIST
 *           |
 *          arg1 -- arg2 -- .. -- argn
 *
 *  new int[]
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *
 *  new int[] {1,2}
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR -- ARRAY_INIT
 *                                  |
 *                                EXPR -- EXPR
 *                                  |      |
 *                                  1      2
 *
 *  new int[3]
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *                |
 *              EXPR
 *                |
 *                3
 *
 *  new int[1][2]
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *               |
 *         ARRAY_DECLARATOR -- EXPR
 *               |              |
 *             EXPR             1
 *               |
 *               2
 *
 */
jNewExpression [JExpression expr]
returns [JExpression self = null]
{
    CType			type;
    JExpression[]		args;
    JArrayInitializer	init = null;
    JClassDeclarationType	decl = null;
    CParseClassContext	context = null;
    CUniverse             array_univ = null;
    CUniverse             elem_univ = null;
    long                  mods = 0;
}
    :
	"new"

        // WMD: different cases for arrays and normal types
        ( ( jUniverseType[] jUniverseSpec[] )=>
            array_univ = jUniverseType[] elem_univ = jUniverseSpec[]
        |
            elem_univ = jUniverseType[]
        |
            { /* nothing */ }
        )

	(mods = jmlNullityModifier[])?
	type = jType[elem_univ]
	(
	    LPAREN args = jArgList[] RPAREN
	    (
		{ context = getParseClassContext(); }

        { // WMD: TODO anonymous classes can't have a universe right now
          if( array_univ != null || elem_univ != null ) {
            // report error if there was a universe specified
            utility.reportTrouble(
              new PositionedError(
                utility.buildTokenReference(), 
                CUniverseMessages.ANON_CLASS_WITH_UNIVERSE_FORBIDDEN, null));
          }
          if( mods !=0 ) {
            // report error if there was a nullity specified
            utility.reportTrouble(
              new PositionedError(
                utility.buildTokenReference(), 
                JmlMessages.UNEXPECTED_NULLITY_ANNOTATION, null));
          }
        }

		jClassBlock[context]
		{
		    decl = new JClassDeclarationWrapper(
			utility.buildTokenReference(), 0,
                        //(CClassType)type,
            ((CClassType)type).qualifiedName(), // !!!
			null, CClassType.EMPTY, context.getMethods(),
			context.getInnerClasses(), context.getFieldsAndInits(),
			null,	// no javadoc comments on anonymous classes
			utility.getStatementComment() );
		    context.release();
		}
		{
		    self = new JNewAnonymousClassExpression(
			utility.buildTokenReference(),
			(CClassType)type, expr, args, decl
		    );
		} // !!!
	    |
		{
            // WMD some error checking
            if( array_univ != null && elem_univ != null ) {
              // report error if there was an array universe specified
              utility.reportTrouble(
                new PositionedError(
                  utility.buildTokenReference(), 
                  CUniverseMessages.NONARRAY_WITH_TWO_UNIVERSES_FORBIDDEN, null));
            }
            if( mods !=0 ) {
              // report error if there was a nullity specified
              utility.reportTrouble(
                new PositionedError(
                  utility.buildTokenReference(), 
                  JmlMessages.UNEXPECTED_NULLITY_ANNOTATION, null));
            }
  
		    self = new JNewObjectExpression(
			utility.buildTokenReference(),
			(CClassType)type, expr, args );
		} // !!!
	    )
	|
	    // java 1.1
	    // Note: This will allow bad constructs like
	    //    new int[4][][3] {exp,exp}.
	    //    There needs to be a semantic check here...
	    // to make sure:
	    //   a) [ expr ] and [ ] are not mixed
	    //   b) [ expr ] and an init are not used together
	    args = jNewArrayDeclarator[] ( init = jArrayInitializer[])?
	    {

          // WMD distinction between a class type and a builtin
          if( type instanceof CClassType ) {

            self = new JNewArrayExpression(
                utility.buildTokenReference(), type,
                (mods == 0 ?
                  new   JArrayDimsAndInits( utility.buildTokenReference(), array_univ, args, init) :
                  new JmlArrayDimsAndInits( utility.buildTokenReference(), array_univ, args, init, mods)));
          } else {
            // WMD: the element is a builtin type

            if( args.length==1 ) {
              if( array_univ != null ) {
                // report error if there were two universes specified
                utility.reportTrouble(
                  new PositionedError( utility.buildTokenReference(), 
                         CUniverseMessages.BUILTIN_WITH_UNIVERSE_FORBIDDEN,
                         null));
              }

              // use the elem_univ for the whole array
              self = new JNewArrayExpression(
                            utility.buildTokenReference(), type,
                            (mods == 0 ?
                                new   JArrayDimsAndInits( utility.buildTokenReference(), elem_univ, args, init) :
                                new JmlArrayDimsAndInits( utility.buildTokenReference(), elem_univ, args, init, mods)));
            } else {
              // use the elem_univ and array_univ
              self = new JNewArrayExpression(
                            utility.buildTokenReference(), type,
                            (mods == 0 ?
                                new   JArrayDimsAndInits( utility.buildTokenReference(), array_univ, elem_univ, args, init) :
                                new JmlArrayDimsAndInits( utility.buildTokenReference(), array_univ, elem_univ, args, init, mods)));
            } 
          }
        }
	|
	    self = jmlSetComprehension[type]
	)
    ;

//----------------------------------------------------------------------------
// The JML scanner
//----------------------------------------------------------------------------
{
    import org.multijava.util.Message;

    // import java.io.Reader;
    import java.util.LinkedList;
    import java.util.ArrayList;

    /**
	 * This is the top level JML scanner.  It processes the normal
	 * (Multi)Java lexicon and switches to annotation lexers for
	 * processing the additional keywords that can appear inside
	 * annotations. */
}
class JmlLexer extends MjcLexer;

options {
    importVocab=JmlID;
    testLiterals=false;    // don't automatically test for literals
    k=4;
    //	charVocabulary='\u0003'..'\uFFFE'; !FIXME!this will blow-up the
    //	lexer because the ~(...) expression includes so many options.  Need
    //	an update to ANTLR for this to really work.
}

{
    public JmlLexer(ParsingController parsingController,
	TokenStreamSelector lexingController, boolean allowJavaAssert,
        boolean allowResend, 
        boolean allowUniverseKeywords, // WMD
        TroubleReporter reporter )
    {
	this( parsingController.sharedInputState() );
	this.parsingController = parsingController;
	this.lexingController = lexingController;
        this.allowJavaAssert = allowJavaAssert;
	this.allowResend = allowResend;
    this.allowUniverseKeywords = allowUniverseKeywords; // WMD
    lexingController.setAllowUniverseKeywords( allowUniverseKeywords ); // WMD
        this.reporter = reporter;
    }

    /**
     * If tok represents a conditionally reserved word that is not
     * reserved during this lexical analysis, then returns null, otherwise
     * returns tok.  For reserved words whose non-reserved use will be
     * deprecated (e.g., 'assert'), throws an . */
    private Token checkReservedTokens(Token tok) {
	if (tok == null) { return null; }
	Token result = tok;
	switch (tok.getType()) {
	    case MjcTokenTypes.LITERAL_assert:
	    if (!allowJavaAssert) {
		reporter.reportTrouble(
		    new CWarning(
			TokenReference.build( parsingController.file(),
			    getLine(), getColumn()),
			MjcMessages.ASSERT_IS_KEYWORD));
		result = null;
	    }
	    break;

	    case MjcTokenTypes.LITERAL_resend:
	    if (!allowResend) {
		result = null;
	    }
	    break;

        case JmlTokenTypes.LITERAL_secret:
        case JmlTokenTypes.LITERAL_query:
            if (!Main.hasOptionXobspure()) result = null;
            break;

        // WMD
        case MjcTokenTypes.LITERAL_peer:
        case MjcTokenTypes.LITERAL_readonly:
        case MjcTokenTypes.LITERAL_rep:
            if( !allowUniverseKeywords ) {
                result = null;
            }
            break;

	case JmlTokenTypes.LITERAL_non_null:
	case JmlTokenTypes.LITERAL_nullable:
	    if( !aboutToProcessABoundVariable ) {
		result = null;
	    }
	    break;

        // WMD allow the pure token for JML
        case MjcTokenTypes.LITERAL_pure:
            if( !inAnnotation && !allowUniverseKeywords ) {
                result = null;
            }
            break;
	}
	return result;
    }

    /**
	 * Called by the scanner when a tab is detected in the input
	 * stream.  Advances the column number according to tabSize.
	 *
	 * @see	#setTabSize(int)
	 * @see #tabSize() */
    public void tab() {
	setColumn( getColumn() + tabSize );
    }

    /**
	 * Sets the amount by which the column number is incremented when
	 * a tab character is detected in the input stream.
	 *
	 * @see #tabSize() */
    public void setTabSize( int ts ) { tabSize = ts; }

    /**
	 * Indicates the amount by which the column number is incremented
	 * when a tab character is detected in the input stream.
	 *
	 * @see #setTabSize(int) */
    public int tabSize() { return tabSize; }

    /**
	 * The amount by which the column number is incremented when a tab
	 * character is detected in the input stream.  */
    private int tabSize = 8;

    /**
	 * Used to switch lexers for nested languages (i.e., javadoc).
	 * Aliases should be established in all lexers that share the same
	 * input stream. */
    private ParsingController parsingController;

    /**
	 * Used to switch lexers when moving in and out of
	 * annotations. Aliases should be established in all lexers that
	 * share the same input stream. */
    private TokenStreamSelector lexingController;

    /**
	 * Used to convert token text from String to char[]. */
    private char[] charCache = null;

    /**
	 * Flag indicates whether we are lexing within an annotation.  A
	 * different definition of IDENT (allowing leading '\') is used in
	 * this case and a different keyword table is consulted for
	 * resolving IDENTs.  This field is false by default in the top-level
     * lexer and true by default in the multi-line and single-line annotation
     * lexers that inherit from this.
     */
    private boolean inAnnotation = false;

    /** Called to specify that the whole file is an annotation, that is, it is
	    a jml file rather than a spec or java file.
	*/
    public void inAnnotation() { inAnnotation = true; }

    /**
     * 
     */
    public boolean aboutToProcessABoundVariable = false;

    /**
	 * Flag indicates whether annotations should be allowed.  Used to
	 * prohibit nested annotations.
	 *
	 * <pre><jml>
	 * private invariant allowAnnotations <==> (* this is not an annotation lexer *);
	 * </jml></pre>
	 * */
    private final boolean allowAnnotations = true;

    /*
     * Indicate which conditionally reserved words should be
     * reserved during this analysis.
     */
    private boolean allowJavaAssert;
    private boolean allowResend;

    // WMD
    private boolean allowUniverseKeywords;

    /**
     * Used to report warnings during lexing.
     */
    private TroubleReporter reporter;
}

// (several) operators are inherited
// some overridden tokens to switch between keyword lookup sets
LPAREN			:	'('		{ lexingController.lexedLeftParen();  };
RPAREN			:	')'		{ lexingController.lexedRightParen(); };
SEMI			:	';'		{ lexingController.lexedSemicolon();  };
LCURLY          :   '{'     { lexingController.lexedLCurly();  };
RCURLY          :   '}'     { Token t = makeToken(_ttype); $setToken(t); lexingController.lexedRCurly(t);  };

// additional operators
L_ARROW         	: {inAnnotation}? "<-"  	;
R_ARROW         	: {inAnnotation}? "->"  	;
EQUIV           	: {inAnnotation}? "<==>"  	;
NOT_EQUIV       	: {inAnnotation}? "<=!=>"  	;
IMPLIES	        	: {inAnnotation}? "==>" 	;
BACKWARD_IMPLIES	: {inAnnotation}? "<==" 	;
SUBTYPE_OF      	: {inAnnotation}? "<:" 	    ;

// for delimitating spec case sequences
LCURLY_VBAR         : {inAnnotation}? "{|"      ;
VBAR_RCURLY         : {inAnnotation}? "|}"      ;


// Whitespace (WS) is inherited

// NON_NL_WS is inherited

// SL_COMMENT is overridden to recognize annotations and leave
// newlines on the input stream

// Single-line comments and annotations.  Single-line comments are
// consumed up to but not including the NEWLINE so that subgrammars
// can ignore special whitespace at the beginning of lines.
// !FIXME! Should make sure not to consume the end-of-file character when it
// occurs on the same line as the "//"
SL_COMMENT
    :	"//"
	(
	    // single line comment consisting of just '+'
	    { LA(2)=='\n' || LA(2)=='\r' }?
	    '+'
	    {
		$setType(Token.SKIP);
	    }
	|
	    ( NEWLINE )=>
	    // empty single line comment
	    {
		$setType(Token.SKIP);
	    }
	|
	    // single line comment without a leading '+'
	    ~('+'|'@'|'\n'|'\r')
	    (~('\n'|'\r'))*
	    {
		$setType(Token.SKIP);
	    }
	|
	    // single line comment with a leading '+'
	    '+'
	    ~('@'|'\n'|'\r')
	    (~('\n'|'\r'))*
	    {
		$setType(Token.SKIP);
	    }
	|
	    // the start of a single line JML annotation
	    ('+')? '@' ('@')*
	    {
		if (!allowAnnotations) {
		    // If we are in an annotation, we allow a single-line
		    // annotation - just ignore the initial token and
		    // continue on in annotation mode.
		    $setType(Token.SKIP);
		} else {

		    try {
			if (!allowAnnotations) {
			    //  						throw new TokenStreamException(
			    //  							new Message( JmlMessages.NESTED_ANNOTATION ).
			    //  							getMessage()
			    //  						);
			    // If we are in an annotation, we allow a single-line
			    // annotation - just ignore the initial token and
			    // continue on in annotation mode.
			    $setType(Token.SKIP);
			} else {
			    lexingController.push("jmlSL");
			}
		    } catch( IllegalArgumentException e ) {
			throw
			new TokenStreamException(
			    "No single-line JML lexer available." );
		    }
		    lexingController.retry();

		}
	    }
	)
    ;

// ML_COMMENT is overridden to recognize annotations
// ML_EMPTY_COMMENT is inherited

// multiple-line comments and annotations
ML_COMMENT
    :
	"/*" ML_COMMENT_REST
        {$setType(Token.SKIP);}
    ;

// refactored out of ML_COMMENT to handle comments within multiline annotations
protected
ML_COMMENT_REST
    :
	OPTIONAL_PLUS
	(
	    (
		NEWLINE
	    |
		~('@'|'*'|'\n'|'\r')
	    )
	    (	// see comments on NEWLINE
		options {
		    generateAmbigWarnings=false;
		}
	    :
		{ LA(2)!='/' }? '*'
	    |	NEWLINE
	    |	~('*'|'\n'|'\r')
	    )*
	    "*/"
	    {$setType(Token.SKIP);}
	|
	    // the start of a multi-line JML annotation
	    '@' ('@')*
	    {
		try {
		    if (!allowAnnotations) {
			throw new TokenStreamException(
			    new Message( JmlMessages.NESTED_ANNOTATION ).
			    getMessage()
			);
		    }
		    lexingController.push("jmlML");
		} catch( IllegalArgumentException e ) {
		    throw
		    new TokenStreamException( "No JML lexer available." );
		}
		lexingController.retry();
	    }
	)
    ;

protected
OPTIONAL_PLUS
    :
	(
	    // ANTLR does the right thing by greedily consuming the
	    // '+' here
	    options {
		generateAmbigWarnings=false;
	    }
	:
	    '+'
	|
	    // empty
	)
    ;

// protected NEWLINE is inherited

JAVADOC_OPEN
// make sure not to match an empty multi-line comment of the form /**/
    :	{ LA(4)!='/' }? "/**" {
	    try {
		parsingController.push("javadoc");
	    } catch( ParsingController.KeyException ke ) {
		throw
		new TokenStreamException( "No javadoc lexer available." );
	    }
	    $setToken( ParsingController.IGNORE );
	}
    ;


// CHARACTER_LITERAL is inherited

// STRING_LITERAL is inherited

// escape sequence (protected ESC) is inherited

// hexadecimal digit (protected HEX_DIGIT) is inherited

// a dummy rule to force vocabulary to be all characters (protected
// VOCAB) is inherited

// an identifier, overridden to recognize leading '\' if in annotation
IDENT
    :	( {inAnnotation}? '\\' )?
	('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$')*
	{
	    String stext = $getText;
	    int tokenLen = stext.length();
	    if( charCache == null || charCache.length < tokenLen ) {
		charCache = stext.toCharArray();
	    } else {
		stext.getChars( 0, tokenLen, charCache, 0 );
	    }
	    if (stext.equals("nowarn") && inAnnotation) {
		//System.out.println("NOWARN ON LINE " + getLine() );
		// For now, we just ignore the nowarn annotations;
		// eventually we want to remember the warning type and
		// filter logical (not parsing or typechecking) warnings
		// that are found. (!FIXME! someday - DRCok)
		$setType(Token.SKIP);
		ArrayList list = new ArrayList();
		mNOWARN_LABEL_LIST(false,list);
		//System.out.print("Nowarn at " + getFilename() + " " + getLine());
		//for (int i=0; i<list.size(); ++i) System.out.print(" " + (String)(list.elementAt(i)));
		//System.out.println("");
	    } else {

                Token tok;
                if( inAnnotation ) {
                    tok = lexingController.lookupJMLToken( charCache, 0,
                        tokenLen );
                } else {
                    tok = JmlTopIDKeywords.lookup( charCache, 0, tokenLen );
                }

		tok = checkReservedTokens( tok );

                if( tok == null ) {
                    if (charCache[0] == '\\') {
                        // We don't allow identifiers to start with '\'.  If we
                        // failed to find a keyword, then the leading '\' here
                        // is an error.
                        // To get this error to go away after adding a
                        // new \whatever expression form to JML, you
                        // need to edit the file JmlExprID.t
                        throw new TokenStreamException( "unexpected char: \\" );
                    }
                    tok = CToken.lookupToken( MjcTokenTypes.IDENT,
                        charCache, 0, tokenLen );
                }

                tok.setLine( getLine() );
                tok.setColumn( getColumn() );
                $setToken( tok );

	    }
	}
    ;

// match informal descriptions (* ... *), allowing newlines
INFORMAL_DESC
    :
        { inAnnotation }?
	// match and ignore leading (*****,
	// but don't match the final * before )
	("(*"! ( options { greedy = true; } : {LA(2) != ')'}? '*'!)*)

	(
	    // the predicate disambiguates between matching '*'
	    // here or in the exit branch of the entire
	    // non-terminal
	    //  			options {
	    //  				generateAmbigWarnings=false;
	    //  			}
	    //  		:
	    { LA(2)!=')' }? '*'
	|
	    ~('*')
	)*
	"*)"!
    ;


protected
NOWARN_LABEL_LIST[ArrayList list]
    :
	(WS)*
	( t:NOWARN_LABEL 			{ list.add(t.getText()); } (WS)*
	    ( COMMA (WS)*  tt:NOWARN_LABEL	{ list.add(tt.getText()); } (WS)* )*
	)?

	SEMI
    ;

protected
NOWARN_LABEL
    :
	( 'a'..'z' | 'A'..'Z' | '_' | '$' )+
    ;


protected
SKIP_TO_SEMI
    :
	( ~(';') )* SEMI
    ;

// INTEGER_LITERAL is overridden to disambiguate DOT_DOT
INTEGER_LITERAL
{boolean isDecimal=false;}
    :
	'.' {_ttype = DOT;}
	(
	    ('0'..'9')+ (EXPONENT)? (FLOAT_SUFFIX)?
	    { _ttype = REAL_LITERAL; }
	|
	    {inAnnotation}? '.'   {_ttype = DOT_DOT;}
	)?
    |
	(
	    '0' {isDecimal = true;} // special case for just '0'
	    (
		('x'|'X')
		(											// hex
		    // the 'e'|'E' and float suffix stuff look
		    // like hex digits, hence the (...)+ doesn't
		    // know when to stop: ambig.  ANTLR resolves
		    // it correctly by matching immediately.  It
		    // is therefore ok to hush warning.
		    options {
			warnWhenFollowAmbig=false;
		    }
		:	HEX_DIGIT
		)+
	    |	('0'..'7')+									// octal
	    )?
	|
	    ('1'..'9') ('0'..'9')*  {isDecimal=true;}		// non-zero decimal
	)
	(
	    ('l'|'L')
	|
	    // only check to see if it's a float if looks like decimal
	    // so far and if there are not two consecutive DOTs (these
	    // should be left on the character stream to be matched
	    // with DOT_DOT)
	    {isDecimal && !(LA(1)=='.' && LA(2)=='.')}?
	    (	'.' ('0'..'9')* (EXPONENT)? (FLOAT_SUFFIX)?
	    |	EXPONENT (FLOAT_SUFFIX)?
	    |	FLOAT_SUFFIX
	    )
	    { _ttype = REAL_LITERAL; }
	)?
    ;


// protected EXPONENT is inherited

// protected FLOAT_SUFFIX is inherited

