header { 
    /*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JmlSL.g,v 1.15 2008/03/16 16:03:50 wdietl Exp $
 */

    package org.jmlspecs.checker; 

    import org.multijava.mjc.*;
}

//----------------------------------------------------------------------------
// The JML single-line scanner
//----------------------------------------------------------------------------
{
    import org.multijava.util.compiler.*;

    import org.multijava.util.Message;

    // import java.io.Reader;
    import java.util.LinkedList;
    import java.util.ArrayList;
}
class JmlSLLexer extends JmlLexer;

options {
    importVocab=Jml;
    testLiterals=false;    // don't automatically test for literals
    k=4;
    //	charVocabulary='\u0003'..'\uFFFE'; !FIXME!this will blow-up the
    //	lexer because the ~(...) expression includes so many options.  Need
    //	an update to ANTLR for this to really work.
}

{
    public JmlSLLexer(ParsingController parsingController,
	TokenStreamSelector lexingController, boolean allowJavaAssert, 
        boolean allowResend, 
        boolean allowUniverseKeywords, // WMD
        TroubleReporter reporter ) 
    {
	this( parsingController.sharedInputState() );
	this.parsingController = parsingController;
	this.lexingController = lexingController;
        this.allowJavaAssert = allowJavaAssert;
	this.allowResend = allowResend;
    this.allowUniverseKeywords = allowUniverseKeywords; // WMD
    lexingController.setAllowUniverseKeywords( allowUniverseKeywords ); // WMD
        this.reporter = reporter;
    }

    /**
     * If tok represents a conditionally reserved word that is not 
     * reserved during this lexical analysis, then returns null, otherwise
     * returns tok.  For reserved words whose non-reserved use will be
     * deprecated (e.g., 'assert'), throws an . */
    private Token checkReservedTokens(Token tok) {
	if (tok == null) { return null; }
	Token result = tok;
	switch (tok.getType()) {
	    case MjcTokenTypes.LITERAL_assert:
	    if (!allowJavaAssert) {
		reporter.reportTrouble(
		    new CWarning(
			TokenReference.build( parsingController.file(),
			    getLine(), getColumn()), 
			MjcMessages.ASSERT_IS_KEYWORD));
		result = null;
	    }
	    break;
	    
	    case MjcTokenTypes.LITERAL_resend:
	    if (!allowResend) {
		result = null;
	    }
	    break;


        case JmlTokenTypes.LITERAL_secret:
        case JmlTokenTypes.LITERAL_query:
            if (!Main.hasOptionXobspure()) result = null;
            break;


        // WMD
        case MjcTokenTypes.LITERAL_peer:
        case MjcTokenTypes.LITERAL_readonly:
        case MjcTokenTypes.LITERAL_rep:
            if( !allowUniverseKeywords ) {
                result = null;
            }
            break;


        // WMD allow the pure token for JML
        case MjcTokenTypes.LITERAL_pure:
            if( !inAnnotation && !allowUniverseKeywords ) {
                result = null;
            }
            break;
	}
	return result;
    }

    /**
	 * Called by the scanner when a tab is detected in the input
	 * stream.  Advances the column number according to tabSize.
	 *
	 * @see	#setTabSize(int)
	 * @see #tabSize() */
    public void tab() {
	setColumn( getColumn() + tabSize );
    }
    
    /**
	 * Sets the amount by which the column number is incremented when
	 * a tab character is detected in the input stream. 
	 *
	 * @see #tabSize() */
    public void setTabSize( int ts ) { tabSize = ts; }

    /**
	 * Indicates the amount by which the column number is incremented
	 * when a tab character is detected in the input stream.
	 *
	 * @see #setTabSize(int) */
    public int tabSize() { return tabSize; }

    /**
	 * The amount by which the column number is incremented when a tab
	 * character is detected in the input stream.  */
    private int tabSize = 8;

    /**
	 * Used to switch lexers for nested languages (i.e., javadoc).
	 * Aliases should be established in all lexers that share the same
	 * input stream. */
    private ParsingController parsingController;

    /**
	 * Used to switch lexers when moving in and out of
	 * annotations. Aliases should be established in all lexers that
	 * share the same input stream. */
    private TokenStreamSelector lexingController;

    /**
	 * Used to convert token text from String to char[]. */
    private char[] charCache = null;

    /**
	 * Flag indicates whether we are lexing within an annotation.  A
	 * different definition of IDENT (allowing leading '\') is used in
	 * this case and a different keyword table is consulted for
	 * resolving IDENTs.  */
    private boolean inAnnotation = true;

    /**
	 * Flag indicates whether we are lexing within a JML expression.  Used
     * to switch between keyword tables for resolving IDENTs.
     */
    private boolean inJMLExpression = false;

    /**
	 * Flag indicates whether we are lexing within a JML quantifier
     * expression.  Used to detect whether or not a ';' indicates the
     * end of a JML expression.
     */
    private boolean inQuantifier = false;

    /**
     * Count of the number of left parens without matching right
     * parens found within a JML expression.  Used to detect whether
     * or not a ';' indicates the end of a JML expression.
     */
    private int parenCount = 0;

    /**
	 * Flag indicates whether annotations should be allowed.  Used to
	 * prohibit nested annotations.
	 *
	 * <pre><jml>
	 * private invariant allowAnnotations <==> (* this is not an annotation lexer *);
	 * </jml></pre>
	 * */
    private final boolean allowAnnotations = false;

    /*
     * Indicate which conditionally reserved words should be
     * reserved during this analysis.
     */
    private boolean allowJavaAssert;
    private boolean allowResend;

    // WMD
    private boolean allowUniverseKeywords;

    /**
     * Used to report warnings during lexing.
     */
    private TroubleReporter reporter;
}

// (several) operators are inherited

/* some overridden tokens so that we can distinguish JML specification
 * operators +, -, *, / and %, from Java operators with the same name.
 * Note that although the token flag is set, it is not current used.
 */
PLUS : '+' { CToken t = new CToken(PLUS , $getText, JmlIDKeywords.INSIDE_ANNOTATION);
             $setToken(t); };
MINUS: '-' { CToken t = new CToken(MINUS , $getText, JmlIDKeywords.INSIDE_ANNOTATION);
             $setToken(t); };
STAR : '*' { CToken t = new CToken(STAR , $getText, JmlIDKeywords.INSIDE_ANNOTATION);
             $setToken(t); };
SLASH: '/' { CToken t = new CToken(SLASH , $getText, JmlIDKeywords.INSIDE_ANNOTATION);
             $setToken(t); };
PERCENT: '%' { CToken t = new CToken(PERCENT , $getText, JmlIDKeywords.INSIDE_ANNOTATION);
             $setToken(t); };

// Whitespace (WS) is overridden to just include non-newline white
// space.  This allows newline to signal end-of-annotation.
// Whitespace -- ignored
WS	:	NON_NL_WS
	{ _ttype = Token.SKIP; }
    ;

// NON_NL_WS is inherited

// SL_COMMENT is inherited

// ML_COMMENT_REST is overridden to disallow NEWLINEs within multi-line comments
// ML_COMMENT is inherited
protected
ML_COMMENT_REST
    :	
	OPTIONAL_PLUS
	(
	    ~('@'|'*'|'\n'|'\r') 
	    (	// see comments on NEWLINE
		{ LA(2)!='/' }? '*'
	    |	~('*'|'\n'|'\r')
	    )*
	    "*/"
	    {$setType(Token.SKIP);}
	|
	    // the start of a multi-line JML annotation
	    '@' ('@')*
	    {
		throw new TokenStreamException( 
		    new Message( JmlMessages.NESTED_ANNOTATION ).getMessage()
		);
	    }
	)
    ;

// protected NEWLINE is inherited

// Overridden so that javadoc-like comments within annotations are handled
// as ML_COMMENTs
JAVADOC_OPEN
    :	
        (
	    options {
		generateAmbigWarnings=false;
	    }
	:
            
            "/**/"                  // handle empty comment
        |
            "/**" ML_COMMENT_REST
        )
        {$setType(Token.SKIP);}
    ;

// Overridden to be protected so that ANTLR does not think that
// ML_EMPTY_COMMENT is ambiguous with JAVADOC_OPEN
protected 
ML_EMPTY_COMMENT
    :
        "/**/"
    ;

// CHARACTER_LITERAL is inherited

// STRING_LITERAL is inherited

// escape sequence (protected ESC) is inherited

// hexadecimal digit (protected HEX_DIGIT) is inherited

// a dummy rule to force vocabulary to be all characters (protected
// VOCAB) is inherited

// IDENT is inherited

// INTEGER_LITERAL is imported

// JML_CLOSE is defined to end single-line annotations on NEWLINE
// the end of an annotation
JML_CLOSE
    :
	NEWLINE
	{
	    lexingController.pop(); 
	    lexingController.retry(); 
	}
    ;

// match informal descriptions (* ... *), disallowing newlines
INFORMAL_DESC
    :
	// match and ignore leading (*****, 
	// but don't match the final * before )
	("(*"! ( options { greedy = true; } : {LA(2) != ')'}? '*'!)*)
	
	(
	    // the predicate disambiguates between matching '*'
	    // here or in the exit branch of the entire
	    // non-terminal
	    //  			options {
	    //  				generateAmbigWarnings=false;
	    //  			}
	    //  		:
	    { LA(2)!=')' }? '*'
	|	
	    ~('*'|'\n'|'\r')
	)*
	"*)"!
    ;
