package org.jmlspecs.checker.testcase.admissibility;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;

import junit.framework.TestCase;

import org.jmlspecs.checker.Main;

/**
 * This class is an abstract JUnit test case for admissibility checks.
 * It runs the JML compiler on the target file with all different admissibility options
 * and compares the received output with the expected one, as well as the
 * return value of the compiler. The file with the expected output is named
 * as follows: [filename including suffix]-expected-[admissibility option], e.g.
 * the output file for Invariant.java with ownership admissibility option is
 * Invariant.java-expected-ownership.
 * A concrete test case must implement {@link #getFileName()}, which returns
 * the file name of the test file, as well as a public static method suite()
 * which returns a {@link junit.framework.Test} object describing the test suite.
 * 
 * @author rzakhejm
 */
public abstract class AdmissibilityTestCase extends TestCase {

	public String output;
	
	public String expected_output;
	
	abstract public String getFileName();
	
	public boolean runAdmissibilityTest(String filename, String admissibilityLevel) {
		File file = new File(filename + "-expected-" + admissibilityLevel);
		char[] buff = new char[(int) file.length()];
		try {
			new InputStreamReader(new FileInputStream(file)).read(buff);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		expected_output = String.valueOf(buff).replaceAll(System.getProperty("line.separator"), "\n");
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintStream stream = new PrintStream(baos);
		PrintStream err = System.err;
		System.setErr(stream);
		String[] strArr = {"-Q", "-E", "parse,check", "-admissibility", admissibilityLevel, filename};
		boolean exit = Main.compile(strArr);
		stream.flush();
		output = baos.toString().replaceAll(System.getProperty("line.separator"), "\n");
		System.setErr(err);
		return exit;
	}
	
	public void testNone() {
		// not assertTrue first, in order to see the errors.
		boolean ok = runAdmissibilityTest(getFileName(), "none");
		assertEquals(expected_output, output);
		assertTrue(ok);
	}
	
	public void testClassical() {
		assertFalse(runAdmissibilityTest(getFileName(), "classical"));
		assertEquals(expected_output, output);
	}
	
	public void testOwnership() {
		assertFalse(runAdmissibilityTest(getFileName(), "ownership"));
		assertEquals(expected_output, output);
	}

}
