/**
 * This class tests the admissibility checker with universe casts in 
 * invariants. It is only interesting for ownership
 * admissibility tests, since classical admissibility does not
 * use any universe information.<br>
 * This class is a part of the admissibility test cases.
 * 
 * @author rzakhejm
 */
abstract class Cast {
	
	int basic;
	final int final_basic;
	
	/*@ spec_public readonly @*/ Cast readonly_cast;
	private /*@ spec_public rep @*/ Cast rep_cast;
	/*@ spec_public peer @*/ Cast peer_cast;

	final /*@ spec_public readonly @*/ Cast final_readonly_cast;
	final private /*@ spec_public rep @*/ Cast final_rep_cast;
	final /*@ spec_public peer @*/ Cast final_peer_cast;

	/*@ spec_public peer peer @*/ Cast[] peer_peer_castarray;
	/*@ spec_public peer readonly @*/ Cast[] peer_readonly_castarray;
	private /*@ spec_public rep peer @*/ Cast[] rep_peer_castarray;
	private /*@ spec_public rep readonly @*/ Cast[] rep_readonly_castarray;
	/*@ spec_public readonly peer @*/ Cast[] readonly_peer_castarray;
	/*@ spec_public readonly readonly @*/ Cast[] readonly_readonly_castarray;

	/*@ spec_public peer peer @*/ Cast[][] peer_peer_cast_twodim_array;
	/*@ spec_public peer readonly @*/ Cast[][] peer_readonly_cast_twodim_array;
	private /*@ spec_public rep peer @*/ Cast[][] rep_peer_cast_twodim_array;
	private /*@ spec_public rep readonly @*/ Cast[][] rep_readonly_cast_twodim_array;
	/*@ spec_public readonly peer @*/ Cast[][] readonly_peer_cast_twodim_array;
	/*@ spec_public readonly readonly @*/ Cast[][] readonly_readonly_cast_twodim_array;

	/*@ spec_public peer peer @*/ Cast[][][] peer_peer_cast_threedim_array;
	/*@ spec_public peer readonly @*/ Cast[][][] peer_readonly_cast_threedim_array;
	private /*@ spec_public rep peer @*/ Cast[][][] rep_peer_cast_threedim_array;
	private /*@ spec_public rep readonly @*/ Cast[][][] rep_readonly_cast_threedim_array;
	/*@ spec_public readonly peer @*/ Cast[][][] readonly_peer_cast_threedim_array;
	/*@ spec_public readonly readonly @*/ Cast[][][] readonly_readonly_cast_threedim_array;

	// admissible
    /*@ invariant readonly_readonly_castarray == null;
     	invariant ((rep Cast) readonly_cast) == null;
      @*/
        
	// ownership, but not classical admissible
	/*@ invariant (readonly_cast instanceof rep Cast) && ((rep Cast) readonly_cast).rep_cast != null;
	    invariant ((rep peer Cast[]) rep_cast.readonly_readonly_cast_twodim_array[1])[3].rep_readonly_castarray[2] == null;
	    invariant ((rep Cast) rep_peer_cast_twodim_array[3][4].rep_readonly_castarray[3]).peer_cast.peer_cast == null;
	    invariant ((rep Cast) rep_peer_cast_twodim_array[3][4].peer_readonly_cast_twodim_array[3][1]).peer_cast.peer_cast == null;	
	    invariant ((rep peer Cast[]) rep_cast.readonly_readonly_cast_threedim_array[1][4])[3].rep_readonly_castarray[2] == null;
	    invariant ((rep peer Cast[][]) rep_cast.readonly_readonly_cast_threedim_array[1])[3][3].rep_readonly_castarray[2] == null;
	    invariant ((rep Cast) rep_peer_cast_threedim_array[3][4][4].rep_readonly_castarray[3]).peer_cast.peer_cast == null;
	    invariant ((rep Cast) rep_peer_cast_threedim_array[3][4][4].peer_readonly_cast_threedim_array[3][4][1]).peer_cast.peer_cast == null;	
	    invariant ((rep peer Cast[]) readonly_readonly_cast_threedim_array[1][4])[3].rep_readonly_castarray[2] == null;
	    invariant ((rep peer Cast[][]) readonly_readonly_cast_threedim_array[1])[3][3].rep_readonly_castarray[2] == null;
	
	  @*/
	
	// neither nor
	/*@ invariant ((rep Cast) rep_peer_cast_twodim_array[3][4].readonly_readonly_cast_twodim_array[3][1]).peer_cast.peer_cast == null;
	 	invariant ((rep Cast) readonly_readonly_cast_twodim_array[3][1]).peer_cast.peer_cast == null;
	 	invariant ((rep Cast) readonly_readonly_castarray[1]).peer_cast.peer_cast == null;
	    invariant ((rep readonly Cast[]) rep_cast.readonly_readonly_cast_twodim_array[1])[3].rep_readonly_castarray[2] == null;
	    invariant ((peer peer Cast[][]) readonly_readonly_cast_threedim_array[1])[3][3].rep_readonly_castarray[2] == null;
	    invariant ((peer peer Cast[]) rep_cast.readonly_readonly_cast_threedim_array[1][3])[3].rep_readonly_castarray[2] == null;


      @*/

	Cast() {
		final_basic = 1;
		final_peer_cast = null;
		final_rep_cast = null;
		final_readonly_cast = null;
	}
}