// Copyright (C) 2002 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

/**
 * Admissibility check testing classes
 * A line without any comment is OK. C means not classically admissible, O means not
 * ownership admissible. N means not yet supported expression.
 * @author rzakhejm
 * 
 */
abstract public class Invariant {

	/*@ public model int mod, childmod, childmod2; @*/
	/*@ public model Outer peer_mod_out; @*/
	/*@ public represents mod <- rep_out.a; @*/ // C
	/*@ public represents_redundantly mod \such_that rep_out.a == mod; @*/ // C
	/*@ public represents peer_mod_out \such_that peer_mod_out == (peer_inv != null ? peer_inv.peer_out : peer_out); @*/ // CO
	
	/*@ spec_public @*/ Invariant peer_inv;
	private /*@ spec_public rep @*/ Invariant rep_inv;
	/*@ spec_public @*/ Outer peer_out;
	private /*@ spec_public rep @*/ Outer rep_out;
	//@ maps rep_out.* \into b;
	/*@ spec_public rep @*/ Outer father_rep_out; // O
	
	private /*@ spec_public rep @*/ boolean[] b;
	//@ maps b[4..6] \into rep_out;
	/*@ spec_public @*/ static int static_a;
	/*@ spec_public @*/ static final int static_final_a = 1;

    //@ invariant rep_out.a == 1 && rep_out.final_out == null; // C
    //@ invariant this.static_a >= 0; // N
	//@ invariant mod < 1 && rep_inv.peer_mod_out.a < 1; // C
    //@ invariant b != null && b.length == 6 && !b[2]; // C
	//@ invariant getStaticA() == static_a; // N (2x)
	//@ static invariant static_a == static_a; // N (1x, since static invariant)
	//@ invariant rep_out.peer_d == rep_out.rep_d; // C
	//@ invariant rep_out.peer_d.a < 0; // C

    //@ assignable static_a, b, rep_out;
    public Invariant() {
    	static_a = 0;
        b = new /*@ rep @*/ boolean[6];
        rep_out = new /*@ rep @*/ Outer();
        rep_out.a = 1;
    }
    
    //@ ensures \result == static_a;
    public /*@ pure @*/ int getStaticA() {
    	return static_a;
    }
    
    public /*@ pure rep @*/ Outer getRepOut() { // O
    	return rep_out;
    }
    
	public static void main(String[] args)
	{
		new Invariantchild().leak(); 
		System.out.println("Alles OK !");
	}
}

class Outer {
	/*@ public model Data peer_d, peer_dd; @*/
	/*@ public model rep Data rep_d; @*/
	/*@ public model readonly Data readonly_d; @*/
	/*@ invariant (((Outer) (this))).a >= -5; @*/
	/*@ invariant peer_d.a < 0; @*/ // CO
	/*@ invariant rep_d.a < 0; @*/ // C
	/*@ public represents peer_d <- peer_dd; @*/
	/*@ public represents peer_dd <- peer_data; @*/
	/*@ public represents rep_d <- rep_data; @*/
	/*@ public represents readonly_d <- readonly_data; @*/
	/*@ spec_public @*/ Data peer_data = new Data();
	private /*@ spec_public rep @*/ Data rep_data = new /*@ rep @*/ Data();
	/*@ spec_public readonly @*/ Data readonly_data = new Data();
	/*@ spec_public @*/ int a = 0;
	private /*@ spec_public rep @*/ Outer rep_out = null;
	/*@ spec_public @*/ final /*@ rep @*/ Outer final_out = null; // O
}

class Invariantchild extends Invariant {

	private /*@ spec_public rep @*/ Outer rep_out_child;
	/*@ spec_public peer @*/ Outer peer_out_child;
	
	//@ public represents childmod <- rep_out_child.a; // C
	/*@ public represents childmod2 \such_that (rep_out_child.a == 0 ? childmod2 + 2 : childmod2 - 2) // C
	  @                                         == rep_out_child.final_out.a; @*/
	//@ invariant father_rep_out == rep_out_child; // CO
	//@ invariant peer_out_child.final_out.final_out.final_out.final_out.final_out.final_out.final_out == null;
	
	private /*@ spec_public rep readonly @*/ Outer[] rep_readonly_outerarray;
	/*@ invariant rep_readonly_outerarray[5] == rep_readonly_outerarray[6]; @*/ // C

	private /*@ spec_public rep readonly @*/ Outer[][][] rep_readonly_outerarray_tripple;
	/*@ invariant rep_readonly_outerarray_tripple[5][3] == rep_readonly_outerarray_tripple[6][2]; // C
	    invariant rep_readonly_outerarray_tripple[5][3][1] == rep_readonly_outerarray_tripple[6][2][3]; // C
	    invariant rep_readonly_outerarray_tripple[5] == rep_readonly_outerarray_tripple[6]; // C
	    invariant (rep_readonly_outerarray_tripple[2])[3] == (rep_readonly_outerarray); // C
	    invariant (this).rep_invchild.rep_readonly_outerarray[4].final_out == null; // C
	    invariant rep_invchild.rep_readonly_outerarray[4].a == 1; // CO
	    invariant peer_invchild.rep_readonly_outerarray[4].final_out == null; // CO 
	    invariant (rep_readonly_outerarray[3].final_out != null ? rep_readonly_outerarray :
	                           rep_readonly_outerarray_tripple[6][2])[2] == null; // C
	    invariant super.b[8] == true; // CO
	    invariant super.b != null; // CO
        invariant ((Invariant) (this)).b != null; // CO
	    invariant ((super.static_final_a) != 2);
	    invariant ((rep_peer_invchildarray[1])[7]).rep_readonly_outerarray[2].final_out == null; // C
	    invariant new rep readonly Invariantchild[][]
	              {rep_readonly_invchildarray, rep_peer_invchildarray[4]}[1][1] != null; // C
        invariant new rep readonly Invariantchild[]
                  {rep_readonly_invchildarray[0]} != null; // C
        invariant new peer readonly Invariantchild[]
                  {rep_readonly_invchildarray[0]} != null; // C
        invariant new peer peer Invariantchild[]
                  {peer_invchild, peer_invchild, peer_invchild, peer_invchild, this}[2] != null; // correct !. FIXME: CO reports
        invariant ((Invariantchild) this).rep_invchild != null;
	    invariant readonly_readonly_invchildarray == null;
        invariant (rep_out_child.a > 0 ? this : this).rep_invchild != null; // correct !. FIXME: CO reports
        invariant (rep_out_child.a > 0 ? this : this).father_rep_out != null; // CO
        invariant rep_out_child.rep_out.rep_out.rep_out.rep_out == null; // C
        invariant new peer int[]{1,4,2,rep_out_child.a,2,3,4}[3] == 1; // C
      @*/
	
	private /*@ spec_public rep @*/ Invariantchild rep_invchild;
	/*@ spec_public peer @*/ Invariantchild peer_invchild;
	/*@ spec_public readonly @*/ Invariantchild readonly_invchild;
	/*@ spec_public peer peer @*/ Invariantchild[] peer_peer_invchildarray;
	/*@ spec_public peer readonly @*/ Invariantchild[] peer_readonly_invchildarray;
	private /*@ spec_public rep peer @*/ Invariantchild[][] rep_peer_invchildarray;
	private /*@ spec_public rep readonly @*/ Invariantchild[] rep_readonly_invchildarray;
	/*@ spec_public readonly peer @*/ Invariantchild[] readonly_peer_invchildarray;
	/*@ spec_public readonly readonly @*/ Invariantchild[][] readonly_readonly_invchildarray;
	
	public void bla(/*@ rep @*/ Invariant a, Invariant a2) { // CO
		
	}

	// More invariants
	/*@
	  
	  
	  
	 	 
	 
	 @*/
	
	
	public void leak() {
		rep_out_child = father_rep_out;
		rep_out_child = getRepOut();
	}
	
	private /*@ pure rep readonly @*/ Outer[] getRepReadOnlyOuterarray() {
		boolean ok = (rep_readonly_outerarray.length < 2 ? rep_readonly_outerarray :
            rep_readonly_outerarray_tripple[6][2])[2] == null;
		boolean ok2 = new boolean[]{true, false, true}[3];
		/*@ readonly @*/ Invariantchild child = readonly_peer_invchildarray[3];
		return rep_readonly_outerarray;
	}
}

class Data {
	int a,b,c,d,e,f,g;
	/*@ peer @*/ Data2 peer_x = new Data2();
	private /*@ rep @*/ Data2 rep_y = null;
	/*@ readonly @*/ Data2 readonly_z = new Data2();
	/*@ static model int mod_a, mod_b; @*/
	/*@ static represents mod_a <- Invariant.static_a; @*/
	/*@ static represents mod_b \such_that mod_a == Invariant.static_a; @*/
}	
	
	
class Data2 {
	int a,b,c,d,e,f,g;
	final int final_a = 1, final_b = 0;
	final Data data = null;
}
