// @(#)$Id: StructureElement.java,v 1.3 2003/10/12 05:24:48 cruby Exp $

// Copyright (C) 2002 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package org.jmlspecs.checker.testcase.syntax;

public class StructureElement {

    /*@ spec_public @*/ Object _object;
    /*@ spec_public @*/ StructuredSet _set;

    /**
     * Initialize a new StructureElement for the given object,
     * connected to the given StructureSet.
     *
     * @param object
     *        The object at the n side of the 1-n binding.
     */
    /*@
      @ public behavior
      @
      @ pre object != null;
      @ modifies _object, _set;
      @ post getObject() == object;
      @ post getStructuredSet() == set;
      @ post set != null ==> set.contains(this);
      @*/
    public StructureElement(Object object, StructuredSet set) {
        _object = object;
        connectTo(set);
    }

    public /*@ pure @*/ Object getObject() { return _object; }
    public /*@ pure @*/ StructuredSet getStructuredSet() { return _set; }

    /*@
      @ behavior
      @     assignable _set;
      @     ensures _set == set;
      @*/
    void connectTo(StructuredSet set) { _set = set; }
}


class StructuredSet {
    public /*@ pure @*/ boolean contains(StructureElement se) { return true; }
}
