// @(#)$Id: nonesc_annotations.java,v 1.11 2005/12/23 17:02:09 chalin Exp $

// Copyright (C) 1998, 1999 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.


package org.jmlspecs.checker.testcase.syntax;
//+@ model import org.jmlspecs.models.*;

import org.jmlspecs.models.JMLType;
import org.jmlspecs.models.JMLValueSet;

public /*@spec_bigint_math@*/ class nonesc_annotations {
 // The defensive version.

  //+@ public model JMLValueSet entries;

  //+@ public invariant entries != null;

  /*+@ public invariant (\forall JMLType e; entries.has(e);
    @                                       e instanceof QueueEntry);
    @*/
   // Note that each e of type QueueEntry must obey the public
   // invariants for QueueEntry.

  /*+@ public invariant 
    @  (\forall QueueEntry e1; entries.has(e1);
    @   (\forall QueueEntry e2; entries.has(e2) && e2 != e1;
    @    !e2.iD.equals(e1.iD) && e2.timeStamp != e1.timeStamp ) );
    @+*/

  /*+@ public normal_behavior
    @   assignable entries;
    @   ensures entries.equals(new JMLValueSet());
    @   ensures_redundantly entries != null && entries.isEmpty(); @+*/
    public nonesc_annotations() { }

  /*+@  public normal_behavior
    @   requires argID != null && !argID.equals(new String()) 
    @     && argPriorityLevel >= 0;
    @   assignable entries;
    @   ensures
    @     (\exists int t;
    @      (\forall QueueEntry e; \old(entries.has(e)); t >= e.timeStamp + 1);
    @        entries.equals(\old(entries.insert(
    @               new QueueEntry(argID, argPriorityLevel, t)))) );
    @   ensures_redundantly entries.int_size() == \old(entries.int_size() + 1);
    @ also
    @  public exceptional_behavior
    @   requires argID == null || argID.equals(new String())
    @     || argPriorityLevel < 0;
    @   assignable \nothing;
    @   signals_only PriorityQueueException;
    @   signals (PriorityQueueException);
    @ for_example
    @  public normal_example
    @   requires entries.has(new QueueEntry("One", 3, 1))
    @     && entries.has(new QueueEntry("Two", 3, 2))
    @     && entries.int_size() == 2 && argID != null
    @     && argID.equals("Three") && argPriorityLevel == 2;
    @   ensures entries.has(new QueueEntry("One", 3, 1))
    @     && entries.has(new QueueEntry("Two", 3, 2))
    @     && entries.has(new QueueEntry("Three", 2, 3))
    @     && entries.int_size() == 3;
    @*/
   public void addEntry(String argID, int argPriorityLevel) 
       throws PriorityQueueException { }
 
   /*+@  public normal_behavior
     @   requires !entries.isEmpty();
     @   ensures
     @    (\exists QueueEntry e; entries.has(e);
     @        (\forall QueueEntry o; entries.has(o) && o != e;
     @         (   (e.priorityLevel > o.priorityLevel)
     @          || (   e.priorityLevel == o.priorityLevel
     @              && e.timeStamp < o.timeStamp)))
     @     && \result.equals(e.iD) );
     @ also
     @  public exceptional_behavior
     @   requires entries.isEmpty();
     @   signals (PriorityQueueException);
     @ for_example
     @  public normal_example
     @   requires entries.has(new QueueEntry("One", 3, 1))
     @    && entries.has(new QueueEntry("Two", 3, 2))
     @    && entries.has(new QueueEntry("Three", 2, 3))
     @    && entries.int_size() == 3;
     @   ensures \result.equals("One");
     */
    public /*+@ pure @+*/ String next() throws PriorityQueueException {
        return "";
    }
 
  /*+@
    @  public normal_behavior
    @   requires argID != null && !argID.equals(new String())
    @        && isUsedID(argID);
    @   assignable entries;
    @   ensures 
    @    (\exists QueueEntry e; \old(entries.has(e));
    @         e.iD.equals(argID) && entries.equals(\old(entries.remove(e))) );
    @   ensures_redundantly entries.int_size() == \old(entries.int_size() - 1);
    @ also
    @  public normal_behavior
    @   requires argID == null || argID.equals(new String())
    @       || !isUsedID(argID);
    @   assignable \nothing;
    @ for_example
    @  public normal_example
    @   requires entries.has(new QueueEntry("One", 3, 1))
    @    && entries.has(new QueueEntry("Two", 3, 2))
    @    && entries.has(new QueueEntry("Three", 2, 3))
    @    && entries.int_size() == 3 && argID.equals("Two");
    @   ensures entries.isProperSubset(\old(entries))
    @    && !entries.has(new QueueEntry("Two", 3, 2))
    @    && entries.int_size() == 2
    @    && entries.has(new QueueEntry("One", 3, 1))
    @    && entries.has(new QueueEntry("Three", 2, 3));
    @*/
    public void remove(String argID) { }
 
  /*+@  public normal_behavior
    @   requires true;
    @   ensures \result == (argID != null && !argID.equals(new String())
    @     && (\exists QueueEntry e; entries.has(e); e.iD.equals(argID)));
    @ for_example
    @  public normal_example
    @   requires entries.has(new QueueEntry("One", 3, 1))
    @    && entries.has(new QueueEntry("Two", 3, 2))
    @    && entries.has(new QueueEntry("Three", 2, 3))
    @    && entries.int_size() == 3 && argID.equals("Two");
    @   ensures \result == true;
    @*/
    public /*+@ pure @*/ boolean isUsedID(String argID) { return false; }

    protected JMLValueSet entries_;

    //@   protected represents entries <- entries_;

}  // end of class nonesc_annotations


/*+@ pure spec_public */ class QueueEntry implements JMLType {

  //+@ public model String         iD;
  //+@ public model int            priorityLevel;
  //+@ public model int            timeStamp;

  /*+@ public invariant
    @   iD != null && !iD.equals(new String()) 
    @   && priorityLevel >= 0 && timeStamp >= 0;
    @+*/

  /*+@  public normal_behavior
    @    requires argID != null & !argID.equals(new String())
    @     && argPriorityLevel >= 0 && argTimeStamp >= 0;
    @    assignable iD, priorityLevel, timeStamp;
    @    ensures \fresh(iD) && iD.equals(argID)
    @      && priorityLevel == argPriorityLevel && timeStamp == argTimeStamp;
    @+*/
    public QueueEntry(String argID, int argPriorityLevel, int argTimeStamp) { }

  /*+@ also
    @  public normal_behavior
    @    requires o instanceof QueueEntry;
    @    ensures \result == (   (((QueueEntry)o).iD.equals(iD))
    @                 && (((QueueEntry)o).priorityLevel == priorityLevel)
    @                 && (((QueueEntry)o).timeStamp == timeStamp) );
    @ also
    @  public normal_behavior
    @    requires !(o instanceof QueueEntry);
    @    ensures \result == false; */
    public boolean equals(/*@ nullable @*/ Object o) { return false; }

  /*+@ also
    @  public normal_behavior
    @   ensures \result instanceof QueueEntry && \fresh(\result)
    @         && ((QueueEntry)\result).equals(this);
    @   ensures_redundantly \result != this;
    @*/
    public Object clone() { return this; }

    protected String         iD_;
    protected int            priorityLevel_;
    protected int            timeStamp_;

    //@   protected represents iD <- iD_;
    //@   protected represents priorityLevel <- priorityLevel_;
    //@   protected represents timeStamp <- timeStamp_;

    //+ a random comment
    //+
    /*+ another random comment */
    /*+*/
    /*+
*/
}

class PriorityQueueException extends Exception {
}
