// @(#)$Id: BCPClient.java,v 1.1 2002/06/25 03:38:49 leavens Exp $

// Copyright (C) 1998, 1999 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.


package org.jmlspecs.checker.testcase.syntax.runchecker.BoundedColorPoint;

import org.jmlspecs.jmlrac.runtime.*;

public class BCPClient {
    /**
     * Method to test behavioral subtyping of assertion checking
     * @param p Point
     */
    //@ requires p != null;
    public static void expectsPoint(Point p)
    {
	// Does not violate the precondition of BoundedColorPoint
	p.setPoint(640,480);
	// 640 >= 0 && 480 >=0

	// Violates the precondition of BoundedColorPoint but not of Point
	// So by behavioral subtyping, it should not throw exception
	p.setPoint(1000,1000);
	// 1000>= 0 && 1000 >=0
    }

    /**
     * Method to test behavioral subtyping of assertion checking
     * @param cp ColorPoint
     */
    //@ requires cp != null;
    public static void expectsColorPoint(ColorPoint cp)
    {
	// Does not violate the precondition of BoundedColorPoint
	cp.setColorPoint(240,120,10); 
	// 240 >= 0 && 120 >= 0 && 10 >=0 

	// Violates the precondition of BoundedColorPoint but not of ColorPoint
	// So by behavioral subtyping, it should not throw exception
	cp.setColorPoint(1000,1000,1000);
	// 1000 >= 0 && 1000 >= 0 && 1000 >=0 

    }

    
    
    /**
     * main entrypoint - starts the application
     * @param args java.lang.String[]
     */
    public static void main(java.lang.String[] args) {
	System.out.println("***************************************************************");
	System.out.println("This is a test of assertion checking in class BoundedColorPoint");
	
	// You need to test the following
	// 1. Satisfy preconditions for all three methods
	// 2. Do not satisfy preconditions for any of the three
	// 3. Test behavioral subtyping by:
	//      a. having a method that expects Point object but receives 
	//         BoundedColorPoint, and
	//      b. a method that expects  ColorPoint interface but receives
	//         BoundedColorPoint

	BoundedColorPoint bcp = new BoundedColorPoint();
	Point pnt= new Point();
	
	try {
	    System.out.println("------TEST1-----------------------------");
	    bcp.setPoint(120,240); 

	    bcp.setResolution(1024,768);

	    bcp.setColorPoint(870,700,223322); 
	    
	    pnt.setPoint(700,1000);
	    System.out.println("---------------------------------------");

	}
	catch (JMLAssertionError ae) {
	    System.err.println("1. Should not throw this exception: " + ae);
	}
	try {
	    System.out.println("--------TEST2---------------------------");
	    bcp.setPoint(2000,-6); 
	    System.out.println("---------------------------------------");
	}
	catch (JMLAssertionError ae) {
	    System.err.println("2. Should throw this exception: " + ae);
	}
	try {
	    System.out.println("--------TEST3--------------------------");
	    bcp.setResolution(-20,100);
	    System.out.println("---------------------------------------");
	}
	catch (JMLAssertionError ae) {
	    System.err.println("3. Should throw this exception: " + ae);
	}
	try {
	    System.out.println("--------TEST4--------------------------");
	    bcp.setColorPoint(3000,-8,-6);
	    // !(7000 <= 768)
	    System.out.println("---------------------------------------");
	}
	catch (JMLAssertionError ae) {
	    System.err.println("4. Should throw this exception: " + ae);
	}
	try {
	    System.out.println("-------TEST5---------------------------");
	    expectsPoint(bcp);
	    System.out.println("---------------------------------------");
	}
	catch (JMLAssertionError ae) {
	    System.err.println("5. Should not throw this exception: " + ae);
	}
	try {
	    System.out.println("-------TEST6----------------------------");
	    expectsColorPoint(bcp);
	    System.out.println("---------------------------------------");
	}
	catch (JMLAssertionError ae) {
	    System.err.println("6. Should not throw this exception: " + ae);
	}
	try {
	    System.out.println("-------TEST7 Invariants----------------");
            bcp.setResolution(50,0);  // Will actually set the resolution to break the invariant (maxx / maxy < 2)
            bcp.setPoint(500,300);// Divide by zero exception should be caught;
	    System.out.println("---------------------------------------");
	}
	catch (JMLAssertionError ae) {
	    System.err.println("7. Should throw this exception for invariant " + ae);
	}

    }
}




