// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: ArithmeticModes.java,v 1.4 2003/08/29 20:38:47 f_rioux Exp $

// --------------------------------------------------
// SUMMARY
// - Classes
// - Methods (in Classes)
// - Interfaces
// - All operators in safe_mode
// - Operators
// - Misc.
//
// NOTE
// - A bug in JML doesn't allow to use the minimal value (i.e. -2147483648).
//   The expression (- 2147483647 - 1) is used as a workaround. (FRioux)
// --------------------------------------------------

// --------------------------------------------------
// Classes
// --------------------------------------------------
/*
 * 01: --    05: J-    09: S-    13: B-    17: -- (should not be influenced by 16)
 * 02: -J    06: JJ    10: SJ    14: BJ    18: PBB    /---
 * 03: -S    07: JS    11: SS    15: BS    19: BPB ---| where "P" is "pure"
 * 04: -B    08: JB    12: SB    16: BB    20: BBP    \---
 *
 * 21: Too much modifiers
 * 22: Contradictory modifiers (is it a bug?)
 */

public
class ArithmeticModes_01 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
    //@public model void testSpec2(int j){int i = -j;}	//in java_math, this should pass without any problem
}

public
/*@spec_java_math@*/
class ArithmeticModes_02 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
    //@public model void testSpec2(int j){int i = -j;}	//in java_math, this should pass without any problem
}

public
/*@spec_safe_math@*/
class ArithmeticModes_03 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	// character 55 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void testSpec2(int j){int i = -j;}	//in safe_math, this should pass without any problem
}

public
/*@spec_bigint_math@*/
class ArithmeticModes_04 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	// character 43 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@public model void testSpec2(int j){int i = -j;}	// character 48 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
}

public
/*@code_java_math@*/
class ArithmeticModes_05 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
    //@public model void testSpec2(int j){int i = -j;}	//in java_math, this should pass without any problem
}

public
/*@code_java_math@*/
/*@spec_java_math@*/
class ArithmeticModes_06 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
    //@public model void testSpec2(int j){int i = -j;}	//in java_math, this should pass without any problem
}

public
/*@code_java_math@*/
/*@spec_safe_math@*/
class ArithmeticModes_07 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	// character 55 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void testSpec2(int j){int i = -j;}	//in safe_math, this should pass without any problem
}

public
/*@code_java_math@*/
/*@spec_bigint_math@*/
class ArithmeticModes_08 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				//in java_math, this should pass without any problem
    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
    public void testCode2(int j){ int i = -j; }			//in java_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	// character 43 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@public model void testSpec2(int j){int i = -j;}	// character 48 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
}

public
/*@code_safe_math@*/
class ArithmeticModes_09 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				// character 40 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode(){ int i = -int_min; } 		// character 46 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode2(int j){ int i = -j; }			//in safe_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
    //@public model void testSpec2(int j){int i = -j;}	//in java_math, this should pass without any problem
}

public
/*@code_safe_math@*/
/*@spec_java_math@*/
class ArithmeticModes_10 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				// character 40 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode(){ int i = -int_min; } 		// character 46 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode2(int j){ int i = -j; }			//in safe_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
    //@public model void testSpec2(int j){int i = -j;}	//in java_math, this should pass without any problem
}

public
/*@code_safe_math@*/
/*@spec_safe_math@*/
class ArithmeticModes_11 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				// character 40 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode(){ int i = -int_min; } 		// character 46 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode2(int j){ int i = -j; }			//in safe_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	// character 55 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void testSpec2(int j){int i = -j;}	//in safe_math, this should pass without any problem
}

public
/*@code_safe_math@*/
/*@spec_bigint_math@*/
class ArithmeticModes_12 {
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;				// character 40 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode(){ int i = -int_min; } 		// character 46 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void testCode2(int j){ int i = -j; }			//in safe_math, this should pass without any problem
    //@public model void testSpec(){ int i = -int_min; }	// character 43 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@public model void testSpec2(int j){int i = -j;}	// character 48 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
}
//
//public
///*@code_bigint_math@*/
//class ArithmeticModes_13 {
//    public static final int int_min = - 2147483647 - 1;
//    public int neg_int_min = - int_min;
//    public void testCode(){ int i = -int_min; } 		//in bigint_math, this should fail
//    public void testCode2(int j){ int i = -j; }		//in bigint_math, this should fail
//    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
//    //@public model void testSpec2(int j){int i = -j;}	//
//}
//
//public
///*@code_bigint_math@*/
///*@spec_java_math@*/
//class ArithmeticModes_14 {
//    public static final int int_min = - 2147483647 - 1;
//    public int neg_int_min = - int_min;
//    public void testCode(){ int i = -int_min; } 		//in bigint_math, this should fail
//    public void testCode2(int j){ int i = -j; }		//in bigint_math, this should fail
//    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
//    //@public model void testSpec2(int j){int i = -j;}	//
//}
//
//public
///*@code_bigint_math@*/
///*@spec_safe_math@*/
//class ArithmeticModes_15 {
//    public static final int int_min = - 2147483647 - 1;
//    public int neg_int_min = - int_min;
//    public void testCode(){ int i = -int_min; } 		//in bigint_math, this should fail
//    public void testCode2(int j){ int i = -j; }		//in bigint_math, this should fail
//    //@public model void testSpec(){ int i = -int_min; }	//in safe_math, this should throw an error
//    //@public model void testSpec2(int j){int i = -j;}	//
//}
//
//public
///*@code_bigint_math@*/
///*@spec_bigint_math@*/
//class ArithmeticModes_16 {
//    public static final int int_min = - 2147483647 - 1;
//    public int neg_int_min = - int_min;
//    public void testCode(){ int i = -int_min; } 		//in bigint_math, this should fail
//    public void testCode2(int j){ int i = -j; }		//in bigint_math, this should fail
//    //@public model void testSpec(){ int i = -int_min; }	//in bigint_math, this should fail
//    //@public model void testSpec2(int j){int i = -j;}	//
//}
//
//public
//class ArithmeticModes_17 {
//    public static final int int_min = - 2147483647 - 1;
//    public int neg_int_min = - int_min;
//    public void testCode(){ int i = -int_min; } 		//in java_math, this should pass without any problem
//    public void testCode2(int j){ int i = -j; }		//in java_math, this should pass without any problem
//    //@public model void testSpec(){ int i = -int_min; }	//in java_math, this should pass without any problem
//    //@public model void testSpec2(int j){int i = -j;}	//
//}
//
//public
///*@pure*/
///*@code_bigint_math@*/
///*@spec_bigint_math@*/
//class ArithmeticModes_18 {}
//
//public
///*@spec_bigint_math@*/
///*@pure*/
///*@code_bigint_math@*/
//class ArithmeticModes_19 {}
//
//public
///*@code_bigint_math@*/
///*@spec_bigint_math@*/
///*@pure*/
//class ArithmeticModes_20 {}
//
//// ToDo: Factor out in another test file
//// public
//// /*@spec_bigint_math@*/
//// /*@code_bigint_math@*/
//// /*@spec_bigint_math@*/
//// class ArithmeticModes_21 {}

public
/*@code_java_math@*/
/*@code_bigint_math@*/
/*@spec_bigint_math@*/
class ArithmeticModes_22 {}	// character 6 error: A maximum of one arithmetic mode can be specified for the code or specification of a class or method [JML]

// --------------------------------------------------
// Methods (in Classes)
// --------------------------------------------------

public
/*@code_safe_math@*/
/*@spec_safe_math@*/
class ArithmeticModes_100{

    //by default, all java code or jml specification uses safe math

    /*
     * 01: --    05: J-    09: S-    13: B-    17: -- (should not be influenced by 16)
     * 02: -J    06: JJ    10: SJ    14: BJ    18: PBB    /---
     * 03: -S    07: JS    11: SS    15: BS    19: BPB ---| where "P" is "pure"
     * 04: -B    08: JB    12: SB    16: BB    20: BBP    \---
     *
     * 21: Too much modifiers
     * 22: Contradictory modifiers (is it a bug?)
     */

    public int i = 0;
    public static final int int_min = - 2147483647 - 1;
    public int neg_int_min = - int_min;		// character 40 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    public /*@pure@*/ static boolean test_int(int i){ return true; }

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 5 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    void method_01() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -5, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//in java_math, this should pass without any problem
      @*/
    public
    /*@spec_java_math@*/
    void method_02() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 6 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@spec_safe_math@*/
    void method_03() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -6, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 6 lines below) character 36 error: Cannot find method "test_int(\bigint)"
      @*/
    public
    /*@spec_bigint_math@*/
    void method_04() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -6, character 36 error: Cannot find method "test_int(\bigint)"

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 6 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_java_math@*/
    void method_05() {
        i = -int_min;				//in java_math, this should pass without any problem
    }							// line -6, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//in java_math, this should pass without any problem
      @*/
    public
    /*@code_java_math@*/
    /*@spec_java_math@*/
    void method_06() {
        i = -int_min;				//in java_math, this should pass without any problem
    }

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 7 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_java_math@*/
    /*@spec_safe_math@*/
    void method_07() {
        i = -int_min;				//in java_math, this should pass without any problem
    }							// line -7, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 7 lines below) character 36 error: Cannot find method "test_int(\bigint)"
      @*/
    public
    /*@code_java_math@*/
    /*@spec_bigint_math@*/
    void method_08() {
        i = -int_min;				//in java_math, this should pass without any problem
    }							// line -7, character 36 error: Cannot find method "test_int(\bigint)"

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 6 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_safe_math@*/
    void method_09() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -6, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//in java_math, this should pass without any problem
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_java_math@*/
    void method_10() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 7 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_safe_math@*/
    void method_11() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -7, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 7 lines below) character 36 error: Cannot find method "test_int(\bigint)"
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_bigint_math@*/
    void method_12() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -7, character 36 error: Cannot find method "test_int(\bigint)"
//
//    /*@
//      @ public normal_behavior
//      @ assignable i;
//      @ ensures test_int(-int_min);		//in safe_math, this should throw an error
//      @*/
//    public
//    /*@code_bigint_math@*/
//    void method_13() {
//        i = -int_min;					//in bigint_math, this should fail
//    }
//
//    /*@
//      @ public normal_behavior
//      @ assignable i;
//      @ ensures test_int(-int_min);		//in java_math, this should pass without any problem
//      @*/
//    public
//    /*@code_bigint_math@*/
//    /*@spec_java_math@*/
//    void method_14() {
//        i = -int_min;					//in bigint_math, this should fail
//    }
//
//    /*@
//      @ public normal_behavior
//      @ assignable i;
//      @ ensures test_int(-int_min);		//in safe_math, this should throw an error
//      @*/
//    public
//    /*@code_bigint_math@*/
//    /*@spec_safe_math@*/
//    void method_15() {
//        i = -int_min;					//in bigint_math, this should fail
//    }
//
//    /*@
//      @ public normal_behavior
//      @ assignable i;
//      @ ensures test_int(-int_min);		//in bigint_math, this should fail
//      @*/
//    public
//    /*@code_bigint_math@*/
//    /*@spec_bigint_math@*/
//    void method_16() {
//        i = -int_min;					//in bigint_math, this should fail
//    }
//
//    /*@
//      @ public normal_behavior
//      @ assignable i;
//      @ ensures test_int(-int_min);		//in safe_math, this should throw an error
//      @*/
//    public
//    void method_17() {
//        i = -int_min;					//in safe_math, this should throw an error
//    }
//
//    /*@
//      @ public normal_behavior
//      @ modifies \nothing;
//      @*/
//    public
//    /*@pure*/
//    /*@code_bigint_math@*/
//    /*@spec_bigint_math@*/
//    void method_18() {
//        return;
//    }
//
//    /*@
//      @ public normal_behavior
//      @ modifies \nothing;
//      @*/
//    public
//    /*@spec_bigint_math@*/
//    /*@pure*/
//    /*@code_bigint_math@*/
//    void method_19() {
//        return;
//    }
//
//    /*@
//      @ public normal_behavior
//      @ modifies \nothing;
//      @*/
//    public
//    /*@code_bigint_math@*/
//    /*@spec_bigint_math@*/
//    /*@pure*/
//    void method_20() {
//        return;
//    }
//
//// ToDo: Factor out in another test file
////    /*@
////      @ public normal_behavior
////      @ modifies \nothing;
////      @*/
////    public
////    /*@spec_bigint_math@*/
////    /*@code_bigint_math@*/
////    /*@spec_bigint_math@*/
////    void method_21() {
////        return;
////    }

    /*@
      @ public normal_behavior	// character 15 error: A maximum of one arithmetic mode can be specified for the code or specification of a class or method [JML]
      @ modifies \nothing;
      @*/
    public
    /*@code_java_math@*/
    /*@code_bigint_math@*/
    /*@spec_bigint_math@*/
    void method_22() {
        return;
    }

}

// --------------------------------------------------
// Interfaces
// --------------------------------------------------

// (Also see ArithmeticModes2.java)

public interface ArithmeticModes_interface_00{
    public static final int int_min = - 2147483647 - 1;
    public /*@pure@*/ boolean test(boolean b);
}


public /*@spec_java_math@*/ interface ArithmeticModes_interface_11 extends ArithmeticModes_interface_00{
    /*@ public normal_behavior
      @ ensures test(-int_min);		// character 32 error: Cannot find method "test(int)"
      @*/
    public /*@pure@*/  void testInsideInterface();
}
public /*@spec_safe_math@*/ interface ArithmeticModes_interface_12 extends ArithmeticModes_interface_00{
    /*@ public normal_behavior
      @ ensures test(-int_min);		// character 31 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public /*@pure@*/  void testInsideInterface();
}
public /*@spec_bigint_math@*/ interface ArithmeticModes_interface_13 extends ArithmeticModes_interface_00{
    /*@ public normal_behavior
      @ ensures test(-int_min);		// character 32 error: Cannot find method "test(\bigint)"
      @*/
    public /*@pure@*/  void testInsideInterface();
}
public interface ArithmeticModes_interface_14 extends ArithmeticModes_interface_00{
    /*@ public normal_behavior
      @ ensures test(\java_math(-int_min));		// character 43 error: Cannot find method "test(int)"
      @*/
    public /*@pure@*/  void testInsideInterface();
}
public interface ArithmeticModes_interface_15 extends ArithmeticModes_interface_00{
    /*@ public normal_behavior
      @ ensures test(\safe_math(-int_min));		// character 42 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public /*@pure@*/  void testInsideInterface();
}
public interface ArithmeticModes_interface_16 extends ArithmeticModes_interface_00{
    /*@ public normal_behavior
      @ ensures test(\bigint_math(-int_min));		// character 45 error: Cannot find method "test(\bigint)"
      @*/
    public /*@pure@*/ void testInsideInterface();
}


// --------------------------------------------------
// All operators in safe_mode
// --------------------------------------------------

public
/*@code_safe_math@*/
/*@spec_safe_math@*/
class ArithmeticModes_safe01{

    public static final int int_min = - 2147483647 - 1;
    public static final int int_max = 2147483647;
    public int i = 0;
    public /*@pure@*/ static boolean test_int(int i){ return true; }

    //UNARY EXPRESSIONS

    public void safe_unary_c01(){ int i = -int_min; } 		// character 52 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void safe_unary_c02(int j){ int i = -j; }			//in safe_math, this should pass without any problem
    //@public model void safe_unary_s01(){ int i = -int_min; }	// character 61 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void safe_unary_s02(int j){int i = -j;}		//in safe_math, this should pass without any problem

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(-int_min);		//(see 7 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_safe_math@*/
    void safe_unary_1() {
        i = -int_min;				// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -7, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    //BINARY ADDITION EXPRESSIONS

    public void safe_add_c01(){ int i = int_max + 1; } 		// character 50 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void safe_add_c02(int j){ int i = 1 + j; }			//in safe_math, this should pass without any problem
    //@public model void safe_add_s01(){ int i = 1 + int_max; }	// character 50 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void safe_add_s02(int j){int i = 1 + j;}	//in safe_math, this should pass without any problem

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(int_max + 1);	//(see 7 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_safe_math@*/
    void safe_add_1() {
        i = int_max + 1;			// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -7, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    //BINARY MULTIPLICATION EXPRESSIONS

    public void safe_mul_c01(){ int i = int_max * 2; } 		// character 50 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void safe_mul_c02(int j){ int i = 2 * j; }			//in safe_math, this should pass without any problem
    //@public model void safe_mul_s01(){ int i = 2 * int_max; }	// character 50 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void safe_mul_s02(int j){int i = 2 * j;}	//in safe_math, this should pass without any problem

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(int_max * 2);	//(see 7 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_safe_math@*/
    void safe_mul_1() {
        i = int_max * 2;			// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -7, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]


    //BINARY SUBTACTION EXPRESSIONS

    public void safe_sub_c01(){ int i = int_min - 1; } 		// character 50 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void safe_sub_c02(int j){ int i = j - 1; }			//in safe_math, this should pass without any problem
    //@public model void safe_sub_s01(){ int i = int_min - 1; }	// character 59 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void safe_sub_s02(int j){int i = j - 1;}	//in safe_math, this should pass without any problem

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(int_min - 1);	//(see 7 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_safe_math@*/
    void safe_sub_1() {
        i = int_min - 1;			// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -7, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    //BINARY DIVISION EXPRESSIONS

    public void safe_div_c01(){ int i = int_min /-1; } 		// character 50 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    public void safe_div_c02(int j){ int i = j /-1; }			//in safe_math, this should pass without any problem
    //@public model void safe_div_s01(){ int i = int_min /-1; }	// character 59 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@public model void safe_div_s02(int j){int i = j /-1;}	//in safe_math, this should pass without any problem

    /*@
      @ public normal_behavior
      @ assignable i;
      @ ensures test_int(int_min /-1);	//(see 7 lines below) character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
      @*/
    public
    /*@code_safe_math@*/
    /*@spec_safe_math@*/
    void safe_div_1() {
        i = int_min /-1;			// character 22 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    }							// line -7, character 35 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]

    //OTHER (?) EXPRESSIONS
    //ToDo..

}

//FIELDS (must be in different classes)
public /*@code_safe_math@*//*@spec_safe_math@*/ class ArithmeticModes_safe10{
    public static final int int_min = - 2147483647 - 1;
    public int negOverflow = - int_min;					// character 40 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
}
public /*@code_safe_math@*//*@spec_safe_math@*/ class ArithmeticModes_safe11{
    public static final int int_max = 2147483647;
    public int addOverflow = int_max + 1;					// character 39 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
}
public /*@code_safe_math@*//*@spec_safe_math@*/ class ArithmeticModes_safe12{
    public static final int int_max = 2147483647;
    public int multOverflow = int_max * 2;				// character 40 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
}
public /*@code_safe_math@*//*@spec_safe_math@*/ class ArithmeticModes_safe13{
    public static final int int_min = - 2147483647 - 1;
    public int minusOverflow = int_min -1;				// character 41 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
}
public /*@code_safe_math@*//*@spec_safe_math@*/ class ArithmeticModes_safe14{
    public static final int int_min = - 2147483647 - 1;
    public int divOverflow = int_min / -1;				// character 39 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
}


// --------------------------------------------------
// Operators
// --------------------------------------------------

public
class ArithmeticModes_op01{

    //by default, we are in java mode for code and specs

    public static final int int_min = - 2147483647 - 1;
    public static final int int_max = 2147483647;
    public int i = 0;
    public /*@pure@*/ static boolean test_int(int i){ return true; }

    //JAVA
    public void java_op_c01(){
    	  int a = -int_min;
        int b = /*@\java_math@*/(-int_min);	//in java_math, this should pass without any problem
        int c = -int_min;
    }
    public void java_op_c02(int j){
        int a = -j;
        int b = /*@\java_math@*/(-j);		//in java_math, this should pass without any problem
        int c = -j;
    }
    /*@
    public model void java_op_s01(){
        int a = -int_min;
        int b = \java_math(-int_min);		//in java_math, this should pass without any problem
        int c = -int_min;
    }
    public model void java_op_s02(int j){
        int a = -j;
        int b = \java_math(-j);			//in java_math, this should pass without any problem
        int c = -j;
    }
    @*/

    //SAFE
    public void safe_op_c01(){
        int a = -int_min;
        int b = /*@\safe_math@*/(-int_min);	// character 43 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
        int c = -int_min;
    }
    public void safe_op_c02(int j){
        int a = -j;
        int b = /*@\safe_math@*/(-j);		//in safe_math, this should pass without any problem
        int c = -j;
    }
    /*@
    public model void safe_op_s01(){
        int a = -int_min;
        int b = \safe_math(-int_min);		// character 37 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
        int c = -int_min;
    }
    public model void safe_op_s02(int j){
        int a = -j;
        int b = \safe_math(-j);
        int c = -j;
    }
    @*/

//    //BIGINT
//    public void bigint_op_c01(){
//        int a = -int_min;
//        int b = /*\@bigint_math@*/(-int_min);
//        int c = -int_min;
//    }
//    public void bigint_op_c02(int j){
//        int a = -j;
//        int b = /*@\bigint_math@*/(-j);
//        int c = -j;
//    }
    /*@
    public model void bigint_op_s01(){
        int a = -int_min;
        int b = \bigint_math(-int_min);		// character 14 error: Variable "b" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
        int c = -int_min;
    }
    public model void bigint_op_s02(int j){
        int a = -j;
        int b = \bigint_math(-j);			// character 14 error: Variable "b" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
        int c = -j;
    }
    @*/

}

// --------------------------------------------------
// Misc.
// --------------------------------------------------

//initially clauses with operators
public class ArithmeticModes_misc_initially01{
    public static final int int_min = - 2147483647 - 1;
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    //@ initially test(\java_math(-int_min));			// character 45 error: Cannot find method "test(int)"
    //@ initially test(\safe_math(-int_min));			// character 44 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@ initially test(\bigint_math(-int_min));			// character 47 error: Cannot find method "test(\bigint)"
}

//initially clauses without operators
public /*@spec_java_math@*/ class ArithmeticModes_misc_initially02{
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    //@ initially test(-(- 2147483647 - 1));			// character 44 error: Cannot find method "test(int)"
}
public /*@spec_safe_math@*/ class ArithmeticModes_misc_initially03{
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    //@ initially test(-(- 2147483647 - 1));			// character 25 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
}
public /*@spec_bigint_math@*/ class ArithmeticModes_misc_initially014{
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    //@ initially test(-(- 2147483647 - 1));			// character 44 error: Cannot find method "test(\bigint)"
}

//invariant clauses with operators
public class ArithmeticModes_misc_invariant01{
    public static final int int_min = - 2147483647 - 1;
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    public boolean b = true;
    //@ public instance invariant b == test(\java_math(-int_min));		// character 66 error: Cannot find method "test(int)"
    //@ public instance invariant b == test(\safe_math(-int_min));		// character 65 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
    //@ public instance invariant b == test(\bigint_math(-int_min));		// character 68 error: Cannot find method "test(\bigint)"
}

//invariant clauses without operators
public /*@spec_java_math@*/ class ArithmeticModes_misc_invariant02{
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    public boolean b = true;
    //@ public instance invariant b == test(-(- 2147483647 - 1));			// character 65 error: Cannot find method "test(int)"
}
public /*@spec_safe_math@*/ class ArithmeticModes_misc_invariant03{
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    public boolean b = true;
    //@public instance invariant b == test(-(- 2147483647 - 1));			// character 45 error: Computation of an integral constant expression has resulted in an arithmetic overflow (due to an operation or a narrowing cast).  This is not permitted when overflow checking is enabled [MultiJava Extension]
}
public /*@spec_bigint_math@*/ class ArithmeticModes_misc_invariant04{
    public /*@pure@*/ static boolean test(boolean b){ return true; }
    public boolean b = true;
    //@ public instance invariant b == test(-(- 2147483647 - 1));			// character 65 error: Cannot find method "test(\bigint)"
}
