import org.jmlspecs.models.JMLObjectSet;

public class MiscPrivacy {
    class Inner1 {}
    /*@ spec_public @*/ class Inner2 {}
    
    // variables
    public int xpub;
    protected int xpro;
    private int xpri;
    int xpkg;

    // methods
    public /*@ pure @*/ int xpub() { return 0; }
    protected /*@ pure @*/ int xpro() { return 0; }
    private /*@ pure @*/ int xpri() { return 0; }
    /*@ pure @*/ int xpkg() { return 0; }

    // Privacy check for old variables.
    // ERROR
    //@ old int x = xpro();
    //@ requires true;
    public void m1() {}

    // Purity check for old variables.
    // ERROR
    //@ old int x = ++xpri;
    //@ requires true;
    public void m2() {}

    // instanceof expression
    //@ requires o instanceof Inner1; // ERROR
    //@ requires o instanceof Inner1[]; // ERROR
    public void m3(Object o) {}

    // instanceof expression
    //@ requires o instanceof Inner2; // OK
    //@ requires o instanceof Inner2[]; // OK
    public void m4(Object o) {}

    // new object expression
    //@ requires o == new Inner1(); // ERROR
    public void m5(Object o) {}

    // new object expression
    //@ requires o == new Inner2(); // OK
    public void m6(Object o) {}

    // new array expression
    //@ requires o == new Inner1[2]; // ERROR
    public void m7(Object o) {}

    // new array expression
    //@ requires o == new Inner1[2]; // OK
    //@ requires o == new int[2]; // OK
    public void m8(Object o) {}

    // cast expression
    //@ requires (Inner1)o == null; // ERROR
    //@ requires (Inner1[])o == null; // ERROR
    public void m9(Object o) {}

    // cast expression
    //@ requires (Inner2)o == null; // OK
    public void m10(Object o) {}

    // JML type expression
    //@ requires \type(Inner1) != \type(Object); // ERROR
    //@ requires \type(Inner1[]) != \type(Object); // ERROR
    public void m11() {}

    // JML type expression
    //@ requires \type(Inner2) != \type(Object); // OK
    public void m12() {}

    // is_initialized expression
    //@ requires \is_initialized(Inner1); // ERROR
    public void m13() {}

    // is_initialized expression
    //@ requires \is_initialized(Inner2); // OK
    public void m14() {}

    // quantified expression
    //@ requires (\forall Inner1 o; ; true); // ERROR
    public void m15() {}

    // quantified expression
    //@ requires (\forall Inner2 o; ; true); // OK
    public void m16() {}

    /*@ spec_public @*/ java.util.Collection mySet;

    // set comprehension expression; ERROR
    //@ requires new JMLObjectSet {Inner1 i | mySet.contains(i) && true}
    //@   != null;
    public void m17() {}

    // set comprehension expression; OK
    //@ requires new JMLObjectSet {Inner2 i | mySet.contains(i) && true} 
    //@   != null;
    public void m18() {}

    // \not_modified expression
    //@ ensures \not_modified(xpub);
    //@ ensures \not_modified(xpro); // ERROR
    //@ ensures \not_modified(xpkg); // ERROR
    //@ ensures \not_modified(xpri); // ERROR
    public void m19() {}

    // \not_assigned expression
    //@ ensures \not_assigned(xpub);
    //@ ensures \not_assigned(xpro);
    //@ ensures \not_assigned(xpkg);
    //@ ensures \not_assigned(xpri); // ERROR
    void m20() {}

    // assignable clause
    //@ assignable xpub;
    //@ assignable xpro;
    //@ assignable xpkg;
    //@ assignable xpri; // ERROR
    void m21() {}

    // Privacy/purity check for initially clauses.
    // ERROR
    public int i;
    /*@ public initially i == ++xpro; @*/ 

    // Privacy/purity check for readable-if clauses.
    // ERROR
    Object o1; 
    /*@ readable o1 if i == ++xpri; @*/ 

    // Privacy/purity check for monitors_for clauses.
    // ERROR
    protected Object o2;
    /*@ protected monitors_for o2 = ++xpkg @*/ ;
}

// @(#)$Id: MiscPrivacy.java,v 1.9 2004/11/18 05:48:25 cruby Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
