/**
 * Checking use of refined specifications using nullity modifiers.  Errors are
 * expected due to inconsistencies between this spec and the refined spec.
 */

//@ refine "NullableRefinement2.java-refined";

public class NullableRefinement2 {

    // Next three declarations causes errors: must be non-null like in .java-refined.
    public static /*@ nullable @*/ Object nonNull0 = Object.class;
    private /*@ nullable @*/ Object nonNull1 = this;
    public  /*@ nullable @*/ Object nonNull2 = this;

    // Next three declarations cause errors: must be nullable like in .java-refined.
    public static Object nullable0 = null;
    private Object nullable1 = null;
    public  /*@ non_null @*/ Object nullable2 = null;

    // Nullity on return type does not match .java-refined
    public /*@ nullable @*/ Object methodReturnNonNull() { return ""; }
    public Object methodReturnNullable() { return null; }

    // Testing methods with parameters ...
    public void methodParamNullable(Object nullableParam) {
	nullable1 = nullableParam;
    }
    public void methodParamNonNull(/*@ nullable @*/ Object nonNullParam) {
	nonNull1 = nonNullParam; 
    }
    public void methodParamNonNull2(/*@ nullable @*/ Object nonNullParam) {
	nonNull1 = nonNullParam; 
    }
}

// @(#)$Id: NullableRefinement2.java,v 1.1 2005/12/14 21:00:11 chalin Exp $
//
// Copyright (C) 2005 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
