// @(#)$Id: Primitive_bigint_basic.java,v 1.26 2003/09/10 04:02:09 cruby Exp $
//
// Copyright (C) 2003 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

/*@code_java_math@*/ /*@spec_bigint_math@*/ class Primitive_bigint_basic {

    //Some fields that will be used in the test file
    //------------------------------------------------
    public boolean b = true;
    public byte  ib = 1;
    public char  ic = '1';
    public short is = 1;
    public int   i  = 1;
    public long  iL = 1;
    //@ model \bigint x;
    public float  f = 1.1F;
    public double d = 1.1;

    public static int si = 2;
    //@ static model \bigint sx;

    // BUG: Until we can use Short.MIN_VALUE, etc, we declare our own
    // constants.
    static final short short_min = -32768;
    static final short short_max = 32767;
    static final int  int_max = (int) 0x7fffffffL;
    static final int  int_min = (int) 0x80000000L;
    static final long  long_max = 0x7fffffffffffffffL;
    static final long  long_min = 0x8000000000000000L;

    //The testing funtions that will be used in this file
    //---------------------------------------------------

    public /*@pure@*/ static boolean test(boolean b)	{ return true; }
    public /*@pure@*/ static boolean test_byte(byte i)	{ return true; }
    public /*@pure@*/ static boolean test_char(char i)	{ return true; }
    public /*@pure@*/ static boolean test_short(short i) { return true; }
    public /*@pure@*/ static boolean test_int(int i)	{ return true; }
    public /*@pure@*/ static boolean test_long(long i)	{ return true; }

    // Test use of \bigint in main method clauses, and method body.
    /*@
      public normal_behavior
        requires i <= 0;
        ensures \result == (i > 0);
      public model pure static boolean test_bigint(\bigint i) {
        return (i > 0);
      }
     @*/

    /*@
      public normal_behavior
        requires i <= 0;
	modifies i, x;
        ensures \result == (i > 0);
     @*/
    public boolean test_int2(int x) {
        i++;
	x++;
	this.i++;
	// this.x++;
        return i == x;
    }

    /*@
      normal_behavior
        requires i <= 0;
	modifies i, this.x;
        ensures \result == (i > 0);
      public model boolean test_bigint2(\bigint x) {
        i++;
	x++;
	this.i++;
	this.x++; // BUG? when keywork 'model' is missing \\ character 15 error: Cannot find field "x" [JLS2 15.11]
        return i == this.x; //\\ character 27 error: Cannot find field "x" [JLS2 15.11]
      } //\\ line -10, character 13 error: Method "test_bigint2" must return a value [JLS 8.4.5]
     @*/

    // Basic expressions over operators in initially clause.
    //--------------------------------
    //@ model \bigint x2; initially x2 == 0 || x2 > 0;
    //@ model static \bigint sx2; static initially sx2 == 0 || sx2 > 0;


    // Cast to and from \bigint
    //-------------------------
    //@ model int i2; initially i2 == (\bigint)x || (\bigint)i2 == x;
    //@ model static int si2; static initially (int)sx < si2;


    // Widening conversion
    //--------------------
    /*@
      public normal_behavior
        ensures \result == i;
      model public pure static \bigint int2big(int i) { return i; }
      @*/

    // Narrowing conversion
    //---------------------
    /*@
      public normal_behavior
        ensures \result == i;
      model public pure static int big2int_err(\bigint i) {
        return i; // character 15 error: Type "\bigint" cannot be returned as a "int" [JLS2 14.16]
      } // line -4, character 13 error: Method "big2int_err" must return a value [JLS 8.4.5]

      public normal_behavior
        ensures \result == i;
      model public pure static int big2int(\bigint i) { return (int)i; }
      @*/

    // Tests with Literals
    //--------------------
    /*@
      public model void tl000(){ test(1); }		// character 41 error: Cannot find method "test(int)"
      public model void tl001(){ test_byte('1'); }	// character 48 error: Cannot find method "test_byte(char)"
      public model void tl002(){ test_byte(1); }		// character 46 error: Cannot find method "test_byte(int)"
      public model void tl003(){ test_int('1'); }
      public model void tl004(){ test_int(1); }
      public model void tl005(){ test_int(1L); }		// character 46 error: Cannot find method "test_int(long)"
      public model void tl006(){ test_long('1'); }
      public model void tl007(){ test_long(1L); }
      public model void tl008(){ test_bigint('1'); }
      public model void tl009(){ test_bigint(1); }
      public model void tl010(){ test_bigint(1L); }
      public model void tl011(){ test_bigint(null); }	// character 51 error: Cannot find method "test_bigint(null-literal)"
      public model void tl012(){ test_bigint("abc"); }	// character 52 error: Cannot find method "test_bigint(java.lang.String)"
      public model void tl013(){ test_bigint(true); }	// character 51 error: Cannot find method "test_bigint(boolean)"
      public model void tl014(){ test_bigint(1.1F); }	// character 51 error: Cannot find method "test_bigint(float)"
      public model void tl015(){ test_bigint(1.1); }	// character 50 error: Cannot find method "test_bigint(double)"
      public model void tl016(){ test_bigint(1.1D); }	// character 51 error: Cannot find method "test_bigint(double)"
      @*/

    // Tests with fields
    //------------------
    /*@
      public model void tf017(){ test_short(ib); }
      public model void tf018(){ test_short(ic); }	// character 49 error: Cannot find method "test_short(char)"
      public model void tf019(){ test_short(is); }
      public model void tf020(){ test_short(i); }	// character 48 error: Cannot find method "test_short(int)"
      public model void tf021(){ test_short(iL); }	// character 49 error: Cannot find method "test_short(long)"
      public model void tf022(){ test_short(x); }	// character 48 error: Cannot find method "test_short(\bigint)"
      public model void tf023(){ test_short(f); }	// character 48 error: Cannot find method "test_short(float)"
      public model void tf024(){ test_short(d); }	// character 48 error: Cannot find method "test_short(double)"

      public model void tf025(){ test_int(ib); }
      public model void tf026(){ test_int(ic); }
      public model void tf027(){ test_int(is); }
      public model void tf028(){ test_int(i); }
      public model void tf029(){ test_int(iL); }	// character 47 error: Cannot find method "test_int(long)"
      public model void tf030(){ test_int(x); }	// character 46 error: Cannot find method "test_int(\bigint)"
      public model void tf031(){ test_int(f); }	// character 46 error: Cannot find method "test_int(float)"
      public model void tf032(){ test_int(d); }	// character 46 error: Cannot find method "test_int(double)"

      public model void tf033(){ test_long(ib); }
      public model void tf034(){ test_long(ic); }
      public model void tf035(){ test_long(is); }
      public model void tf036(){ test_long(i); }
      public model void tf037(){ test_long(iL); }
      public model void tf038(){ test_long(x); }	// character 47 error: Cannot find method "test_long(\bigint)"
      public model void tf039(){ test_long(f); }	// character 47 error: Cannot find method "test_long(float)"
      public model void tf040(){ test_long(d); }	// character 47 error: Cannot find method "test_long(double)"

      public model void tf041(){ test_bigint(ib); }
      public model void tf042(){ test_bigint(ic); }
      public model void tf043(){ test_bigint(is); }
      public model void tf044(){ test_bigint(i); }
      public model void tf045(){ test_bigint(iL); }
      public model void tf046(){ test_bigint(x); }
      public model void tf047(){ test_bigint(f); }	// character 49 error: Cannot find method "test_bigint(float)"
      public model void tf048(){ test_bigint(d); }	// character 49 error: Cannot find method "test_bigint(double)"
      @*/

    // Tests with casts
    //-----------------
    /*@
      public model void tc049(){ test((\bigint)ib); }	// character 52 error: Cannot find method "test(\bigint)"
      public model void tc050(){ test((\bigint)ic); }	// character 52 error: Cannot find method "test(\bigint)"
      public model void tc051(){ test((\bigint)is); }	// character 52 error: Cannot find method "test(\bigint)"
      public model void tc052(){ test((\bigint)i); }	// character 51 error: Cannot find method "test(\bigint)"
      public model void tc053(){ test((\bigint)true); }	// character 39 error: Cannot cast from "boolean" to "\bigint" [JLS 15.15]
      public model void tc054(){ test((\bigint)'1'); }	// character 52 error: Cannot find method "test(\bigint)"
      public model void tc055(){ test((\bigint)1); }	// character 50 error: Cannot find method "test(\bigint)"
      public model void tc056(){ test((\bigint)1L); }	// character 51 error: Cannot find method "test(\bigint)"
      public model void tc057(){ test((\bigint)iL); }	// character 52 error: Cannot find method "test(\bigint)"
      public model void tc058(){ test((\bigint)x); }	// character 51 error: Cannot find method "test(\bigint)"
      public model void tc059(){ test((\bigint)f); }	// character 51 error: Cannot find method "test(\bigint)"
      public model void tc060(){ test((\bigint)d); }	// character 51 error: Cannot find method "test(\bigint)"

      public model void tc061(){ test((byte)x); }	// character 48 error: Cannot find method "test(byte)"
      public model void tc062(){ test((char)x); }	// character 48 error: Cannot find method "test(char)"
      public model void tc063(){ test((short)x); }	// character 49 error: Cannot find method "test(short)"
      public model void tc064(){ test((int)x); }	// character 47 error: Cannot find method "test(int)"
      public model void tc065(){ test((long)x); }	// character 48 error: Cannot find method "test(long)"
      public model void tc066(){ test((\bigint)x); }	// character 51 error: Cannot find method "test(\bigint)"
      public model void tc067(){ test((float)x); }	// character 49 error: Cannot find method "test(float)"
      public model void tc068(){ test((double)x); }	// character 50 error: Cannot find method "test(double)"
      @*/

    // Test over unary operators
    //--------------------------

    /*@
      // operands are literals (note that literals in arithmetic unary
      // expressions are not subject to implicit promotion)

      public model void tul069(){ test(+'c'); }		// character 45 error: Cannot find method "test(int)"
      public model void tul070(){ test(-'c'); }		// character 45 error: Cannot find method "test(int)"
      public model void tul071(){ test_int(+1); }
      public model void tul072(){ test_int(+1L); }	// character 48 error: Cannot find method "test_int(long)"
      public model void tul073(){ test_int(-1L); }	// character 48 error: Cannot find method "test_int(long)"
      public model void tul074(){ test_long(+1L); }
      public model void tul075(){ test_long(-1L); }
      public model void tul076(){ test_bigint(+1); }
      public model void tul077(){ test_bigint(-1); }
      public model void tul078(){ test_bigint(+1L); }
      public model void tul079(){ test_bigint(-1L); }

      public model void tul080(){ test(+(short)'c'); }	// character 52 error: Cannot find method "test(int)"
      public model void tul081(){ test(-(short)'c'); }	// character 52 error: Cannot find method "test(int)"
      public model void tul082(){ test(+(short)1); }	// character 50 error: Cannot find method "test(int)"
      public model void tul083(){ test(-(short)1); }	// character 50 error: Cannot find method "test(int)"
      public model void tul084(){ test(+(int)1); }	// character 48 error: Cannot find method "test(int)"
      public model void tul085(){ test(-(int)1); }	// character 48 error: Cannot find method "test(int)"
      public model void tul086(){ test(+(long)1); }	// character 49 error: Cannot find method "test(long)"
      public model void tul087(){ test(-(long)1); }	// character 49 error: Cannot find method "test(long)"
      public model void tul088(){ test(+(1L)); }	// character 46 error: Cannot find method "test(long)"
      public model void tul089(){ test(-(1L)); }	// character 46 error: Cannot find method "test(long)"
      public model void tul090(){ test(+(\bigint)1); }	// character 52 error: Cannot find method "test(\bigint)"
      public model void tul091(){ test(-(\bigint)'c');}	// character 54 error: Cannot find method "test(\bigint)"

      public model void tul391(){ test(\nowarn(-1)); }	// character 52 error: Cannot find method "test(int)"
      public model void tul392(){ test(\nowarn(-1L)); }	// character 53 error: Cannot find method "test(long)"

      // operands are almost literals
      public model void tul092(){ test_int(+(int)1); }
      public model void tul093(){ test_int(-(1)); }	// Apparently -(1) equiv. to -1.
      public model void tul094(){ test_bigint(-(1)); }
      public model void tul095(){ test_int(-(int)(+(-(int)(+(-(int)(1)))))); }

      public model void tul096(){ test(~1); }		// character 43 error: Cannot find method "test(int)"
      public model void tul097(){ test(~1L); }		// character 44 error: Cannot find method "test(long)"

      public model void t098(){ test(-(\bigint)1L);}            // character 51 error: Cannot find method "test(\bigint)"
      public model void t099(){ test(-((long)1));}              // character 49 error: Cannot find method "test(long)"
      public model void t100(){ test(-('c'));}                  // character 45 error: Cannot find method "test(int)"
      public model void t101(){ test(-((short)1));}             // character 50 error: Cannot find method "test(int)"
      public model void t102(){ test(-((short)1L));}            // character 51 error: Cannot find method "test(int)"

      public model void t103(){ test(+(\bigint)1L);}            // character 51 error: Cannot find method "test(\bigint)"
      public model void t104(){ test(+((long)1));}              // character 49 error: Cannot find method "test(long)"
      public model void t105(){ test(+('c'));}                  // character 45 error: Cannot find method "test(int)"
      public model void t106(){ test(+((short)1));}             // character 50 error: Cannot find method "test(int)"
      public model void t107(){ test(+((short)1L));}            // character 51 error: Cannot find method "test(int)"
      @*/

      public void t108(){ test(-('c'));}			// character 39 error: Cannot find method "test(int)"
      public void t109(){ test(-((short)1));}			// character 44 error: Cannot find method "test(int)"
      public void t110(){ test(-((short)1L));}			// character 45 error: Cannot find method "test(int)"
      public void t111(){ test(-((long)1));}			// character 43 error: Cannot find method "test(long)"

      public void t112(){ test(+('c'));}			// character 39 error: Cannot find method "test(int)"
      public void t113(){ test(+((short)1));}			// character 44 error: Cannot find method "test(int)"
      public void t114(){ test(+((short)1L));}			// character 45 error: Cannot find method "test(int)"
      public void t115(){ test(+((long)1));}			// character 43 error: Cannot find method "test(long)"

    //@ model public void t116(){ short i = +1; }
    //@ model public void t117(){ short i = -1; }
    //@ model public void t118(){ short i = +(1); }
    //@ model public void t119(){ short i = -(1); }
    //@ model public void t120(){ short i = +((int)1L); }
    //@ model public void t121(){ short i = -((short)1L); }

    //@ model public void t122(){ short s1 = - short_min; }		// character 43 error: Variable "s1" cannot be initialized by a value of type "int" [JLS 4.5.4]
    //@ model public void t123(){ int i = -int_min; }			// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t124(){ int i = \warn(-int_min); }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t125(){ int i = \warn_op(-int_min); }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t126(){ int i = \nowarn(-int_min); }
    //@ model public void t127(){ int i = \nowarn_op(-int_min); }
    //@ model public void t128(){ long x = - long_max; }
    //@ model public void t129(){ long x = - long_min; }		// character 41 error: Variable "x" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t130(){ long x = - (long) long_min; }		// character 41 error: Variable "x" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]

    public void tt131() { short s1 = - short_min; }			// character 35 error: Variable "s1" cannot be initialized by a value of type "int" [JLS 4.5.4]
    public void tt132() { long x1 = - long_max; }
    public void tt133() { long x2 = - long_min; }

    //@ model public void t134(){ short i = 1+1; }
    public void t135(){ short i = 1+1; }
    //@ model public void t136(){ short i = short_min + 1; }
    public void t137(){ short i = short_min + 1; }
    //@ model public void t138(){ short i = short_max + 1; }	// character 42 error: Variable "i" cannot be initialized by a value of type "int" [JLS 4.5.4]
    public void t139(){ short i = short_max + 1; }		// character 32 error: Variable "i" cannot be initialized by a value of type "int" [JLS 4.5.4]

    //@ model public void t140(){ int i = int_min + 1; }
    public void t141(){ int i = int_min + 1; }
    //@ model public void t142(){ int i = int_max + 1; }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t143(){ int i = -(int_min + 1); }
    //@ model public void t144(){ int i = \warn(int_max + 1); }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t145(){ int i = \warn_op(int_max + 1); }	// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t146(){ int i = \nowarn(int_max + 1); }
    //@ model public void t147(){ int i = \nowarn_op(int_max + 1); }

    //@ model public void t148(){ int i = \nowarn_op(-int_min + 1); }	// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]

    public void t149(){ int i = int_max + 1; }
    // public void t150(){ int i = _warn(int_max + 1); }		//

    /*@
    // operands are fields and variables
    public model void tuf151(){ test_int(+i); }
    public model void tuf152(){ test_int(\nowarn(-i)); }
    public model void tuf153(){ test(+i); }		// character 42 error: Cannot find method "test(int)"
    public model void tuf154(){ test(\nowarn_op(-(int)i)); }	// character 58 error: Cannot find method "test(int)"
    public model void tuf155(){ test(+(long)iL); }	// character 49 error: Cannot find method "test(long)"
    public model void tuf156(){ test(\nowarn(-iL)); }	// character 51 error: Cannot find method "test(long)"
    public model void tuf157(){ test_bigint(+x); }
    public model void tuf158(){ test_bigint(-x); }
    public model void tuf159(){ test_int(+(i)); }
    public model void tuf160(){ test_int(\nowarn(-(int)(i))); }
    public model void tuf161(){ test_bigint(+(i)); }
    public model void tuf162(){ test_bigint(-(i)); }
    public model void tuf163(){ test_bigint(+(x)); }
    public model void tuf164(){ test_bigint(-(x)); }
    public model void tuf165(){ test_int(\nowarn_op(-(+(\nowarn_op(-(+(\nowarn(-i)))))))); }
    public model void tuf166(){ test(-(+(-(+(-(x)))))); }	// character 55 error: Cannot find method "test(\bigint)"

    public model void tuf393(){ test(\nowarn(-i)); }		// character 50 error: Cannot find method "test(int)"

    public model void tux167(){ test_int((int)i); }
    public model void tux168(){ test_int((int)-i); }
    public model void tux169(){ test_int((int)x); }
    public model void tux170(){ test_int((int)-i); }
    @*/

    // Arithmetic (binary) tests
      //--------------------------
      /*@ // to each of these groups add, e.g. 1 + (int)x and (int)x + 1 --> int.

      //ADD
      public model void t171(){ test(x + x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t172(){ test(1 + x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t173(){ test(x + 1); }				// character 44 error: Cannot find method "test(\bigint)"
      public model void t174(){ test(\nowarn_op(i + (int)x)); }		// character 61 error: Cannot find method "test(int)"
      public model void t175(){ test((int) i + x); }			// character 51 error: Cannot find method "test(\bigint)"
      public model void t176(){ test(i + x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t177(){ test((\bigint) i + x); }		// character 55 error: Cannot find method "test(\bigint)"
      public model void t178(){ test((\bigint) i + (int) x); }		// character 61 error: Cannot find method "test(\bigint)"
      public model void t179(){ test(\nowarn((int) x + i)); }		// character 59 error: Cannot find method "test(int)"
      public model void t180(){ test(x + (int) i); }			// character 51 error: Cannot find method "test(\bigint)"
      public model void t181(){ test(x + i); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t182(){ test(x + (\bigint) i); }		// character 55 error: Cannot find method "test(\bigint)"
      public model void t183(){ test((int) x + (\bigint) i); }		// character 61 error: Cannot find method "test(\bigint)"

      //MINUS
      public model void t184(){ test(x - x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t185(){ test(1 - x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t186(){ test(x - 1); }                          // character 44 error: Cannot find method "test(\bigint)"
      public model void t187(){ test(\nowarn(i - (int)x)); }            // character 58 error: Cannot find method "test(int)"
      public model void t188(){ test((int) i - x); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t189(){ test(i - x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t190(){ test((\bigint) i - x); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t191(){ test((\bigint) i - (int) x); }          // character 61 error: Cannot find method "test(\bigint)"
      public model void t192(){ test(\nowarn((int) x - i)); }           // character 59 error: Cannot find method "test(int)"
      public model void t193(){ test(x - (int) i); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t194(){ test(x - i); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t195(){ test(x - (\bigint) i); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t196(){ test((int) x - (\bigint) i); }          // character 61 error: Cannot find method "test(\bigint)"

      //MULT
      public model void t197(){ test(x * x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t198(){ test(1 * x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t199(){ test(x * 1); }                          // character 44 error: Cannot find method "test(\bigint)"
      public model void t200(){ test(\nowarn(i * (int)x)); }            // character 58 error: Cannot find method "test(int)"
      public model void t201(){ test((int) i * x); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t202(){ test(i * x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t203(){ test((\bigint) i * x); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t204(){ test((\bigint) i * (int) x); }          // character 61 error: Cannot find method "test(\bigint)"
      public model void t205(){ test(\nowarn((int) x * i)); }           // character 59 error: Cannot find method "test(int)"
      public model void t206(){ test(x * (int) i); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t207(){ test(x * i); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t208(){ test(x * (\bigint) i); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t209(){ test((int) x * (\bigint) i); }          // character 61 error: Cannot find method "test(\bigint)"

      //MODULO
      public model void t210(){ test(x % x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t211(){ test(1 % x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t212(){ test(x % 1); }                          // character 44 error: Cannot find method "test(\bigint)"
      public model void t213(){ test(i % (int) x); }                    // character 51 error: Cannot find method "test(int)"
      public model void t214(){ test((int) i % x); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t215(){ test(i % x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t216(){ test((\bigint) i % x); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t217(){ test((\bigint) i % (int) x); }          // character 61 error: Cannot find method "test(\bigint)"
      public model void t218(){ test((int) x % i); }                    // character 51 error: Cannot find method "test(int)"
      public model void t219(){ test(x % (int) i); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t220(){ test(x % i); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t221(){ test(x % (\bigint) i); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t222(){ test((int) x % (\bigint) i); }          // character 61 error: Cannot find method "test(\bigint)"

      //DIVIDE
      public model void t223(){ test(x / x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t224(){ test(1 / x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t225(){ test(x / 1); }                          // character 44 error: Cannot find method "test(\bigint)"
      public model void t226(){ test(\nowarn(i / (int)x)); }            // character 58 error: Cannot find method "test(int)"
      public model void t227(){ test((int) i / x); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t228(){ test(i / x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t229(){ test((\bigint) i / x); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t230(){ test((\bigint) i / (int) x); }          // character 61 error: Cannot find method "test(\bigint)"
      public model void t231(){ test(\nowarn((int) x / i)); }           // character 59 error: Cannot find method "test(int)"
      public model void t232(){ test(x / (int) i); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t233(){ test(x / i); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t234(){ test(x / (\bigint) i); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t235(){ test((int) x / (\bigint) i); }          // character 61 error: Cannot find method "test(\bigint)"

      //SHIFT (>>, <<, >>>)

      public model void t236(){ test(x >> x); }				// character 46 error: Cannot find method "test(\bigint)"
      public model void t237(){ test(1 >> x); }                         // character 46 error: Cannot find method "test(int)"
      public model void t238(){ test(x >> 1); }                         // character 45 error: Cannot find method "test(\bigint)"
      public model void t239(){ test(i >> (int) x); }                   // character 52 error: Cannot find method "test(int)"
      public model void t240(){ test((int) i >> x); }                   // character 52 error: Cannot find method "test(int)"
      public model void t241(){ test(i >> x); }                         // character 46 error: Cannot find method "test(int)"
      public model void t242(){ test((\bigint) i >> x); }               // character 56 error: Cannot find method "test(\bigint)"
      public model void t243(){ test((\bigint) i >> (int) x); }         // character 62 error: Cannot find method "test(\bigint)"
      public model void t244(){ test((int) x >> i); }                   // character 52 error: Cannot find method "test(int)"
      public model void t245(){ test(x >> (int) i); }                   // character 52 error: Cannot find method "test(\bigint)"
      public model void t246(){ test(x >> i); }                         // character 46 error: Cannot find method "test(\bigint)"
      public model void t247(){ test(x >> (\bigint) i); }               // character 56 error: Cannot find method "test(\bigint)"
      public model void t248(){ test((int) x >> (\bigint) i); }         // character 62 error: Cannot find method "test(int)"

      public model void t249(){ test(x << x); }				// character 46 error: Cannot find method "test(\bigint)"
      public model void t250(){ test(1 << x); }                         // character 46 error: Cannot find method "test(int)"
      public model void t251(){ test(x << 1); }                         // character 45 error: Cannot find method "test(\bigint)"
      public model void t252(){ test(i << (int) x); }                   // character 52 error: Cannot find method "test(int)"
      public model void t253(){ test((int) i << x); }                   // character 52 error: Cannot find method "test(int)"
      public model void t254(){ test(i << x); }                         // character 46 error: Cannot find method "test(int)"
      public model void t255(){ test((\bigint) i << x); }               // character 56 error: Cannot find method "test(\bigint)"
      public model void t256(){ test((\bigint) i << (int) x); }         // character 62 error: Cannot find method "test(\bigint)"
      public model void t257(){ test((int) x << i); }                   // character 52 error: Cannot find method "test(int)"
      public model void t258(){ test(x << (int) i); }                   // character 52 error: Cannot find method "test(\bigint)"
      public model void t259(){ test(x << i); }                         // character 46 error: Cannot find method "test(\bigint)"
      public model void t260(){ test(x << (\bigint) i); }               // character 56 error: Cannot find method "test(\bigint)"
      public model void t261(){ test((int) x << (\bigint) i); }         // character 62 error: Cannot find method "test(int)"

      public model void t262(){ test(x >>> x); }			// character 47 error: Cannot find method "test(\bigint)"
      public model void t263(){ test(1 >>> x); }                        // character 47 error: Cannot find method "test(int)"
      public model void t264(){ test(x >>> 1); }                        // character 46 error: Cannot find method "test(\bigint)"
      public model void t265(){ test(i >>> (int) x); }                  // character 53 error: Cannot find method "test(int)"
      public model void t266(){ test((int) i >>> x); }                  // character 53 error: Cannot find method "test(int)"
      public model void t267(){ test(i >>> x); }                        // character 47 error: Cannot find method "test(int)"
      public model void t268(){ test((\bigint) i >>> x); }              // character 57 error: Cannot find method "test(\bigint)"
      public model void t269(){ test((\bigint) i >>> (int) x); }        // character 63 error: Cannot find method "test(\bigint)"
      public model void t270(){ test((int) x >>> i); }                  // character 53 error: Cannot find method "test(int)"
      public model void t271(){ test(x >>> (int) i); }                  // character 53 error: Cannot find method "test(\bigint)"
      public model void t272(){ test(x >>> i); }                        // character 47 error: Cannot find method "test(\bigint)"
      public model void t273(){ test(x >>> (\bigint) i); }              // character 57 error: Cannot find method "test(\bigint)"
      public model void t274(){ test((int) x >>> (\bigint) i); }        // character 63 error: Cannot find method "test(int)"

      //BITWISE (&, |, ^)

      public model void t275(){ test(x & x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t276(){ test(1 & x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t277(){ test(x & 1); }                          // character 44 error: Cannot find method "test(\bigint)"
      public model void t278(){ test(i & (int) x); }                    // character 51 error: Cannot find method "test(int)"
      public model void t279(){ test((int) i & x); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t280(){ test(i & x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t281(){ test((\bigint) i & x); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t282(){ test((\bigint) i & (int) x); }          // character 61 error: Cannot find method "test(\bigint)"
      public model void t283(){ test((int) x & i); }                    // character 51 error: Cannot find method "test(int)"
      public model void t284(){ test(x & (int) i); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t285(){ test(x & i); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t286(){ test(x & (\bigint) i); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t287(){ test((int) x & (\bigint) i); }          // character 61 error: Cannot find method "test(\bigint)"

      public model void t288(){ test(x | x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t289(){ test(1 | x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t290(){ test(x | 1); }                          // character 44 error: Cannot find method "test(\bigint)"
      public model void t291(){ test(i | (int) x); }                    // character 51 error: Cannot find method "test(int)"
      public model void t292(){ test((int) i | x); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t293(){ test(i | x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t294(){ test((\bigint) i | x); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t295(){ test((\bigint) i | (int) x); }          // character 61 error: Cannot find method "test(\bigint)"
      public model void t296(){ test((int) x | i); }                    // character 51 error: Cannot find method "test(int)"
      public model void t297(){ test(x | (int) i); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t298(){ test(x | i); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t299(){ test(x | (\bigint) i); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t300(){ test((int) x | (\bigint) i); }          // character 61 error: Cannot find method "test(\bigint)"

      public model void t301(){ test(x ^ x); }				// character 45 error: Cannot find method "test(\bigint)"
      public model void t302(){ test(1 ^ x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t303(){ test(x ^ 1); }                          // character 44 error: Cannot find method "test(\bigint)"
      public model void t304(){ test(i ^ (int) x); }                    // character 51 error: Cannot find method "test(int)"
      public model void t305(){ test((int) i ^ x); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t306(){ test(i ^ x); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t307(){ test((\bigint) i ^ x); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t308(){ test((\bigint) i ^ (int) x); }          // character 61 error: Cannot find method "test(\bigint)"
      public model void t309(){ test((int) x ^ i); }                    // character 51 error: Cannot find method "test(int)"
      public model void t310(){ test(x ^ (int) i); }                    // character 51 error: Cannot find method "test(\bigint)"
      public model void t311(){ test(x ^ i); }                          // character 45 error: Cannot find method "test(\bigint)"
      public model void t312(){ test(x ^ (\bigint) i); }                // character 55 error: Cannot find method "test(\bigint)"
      public model void t313(){ test((int) x ^ (\bigint) i); }          // character 61 error: Cannot find method "test(\bigint)"

      @*/

      // Conditional expression tests
      //-----------------------------
      /*@
      public model void t314(){ test(i <= x ? i : x + x); }			// character 58 error: Cannot find method "test(\bigint)"
      public model void t315(){ test(i <= x ? i : 1 % x); }                     // character 58 error: Cannot find method "test(\bigint)"
      public model void t316(){ test(i <= x ? i : x - 1); }                     // character 57 error: Cannot find method "test(\bigint)"
      public model void t317(){ test(i <= x ? i : \warn(i * (int) x)); }        // character 70 error: Cannot find method "test(\bigint)"
      public model void t318(){ test(i <= x ? i : (int) i >>> x); }             // character 66 error: Cannot find method "test(int)"
      public model void t319(){ test(i <= x ? i : i ^ x); }                     // character 58 error: Cannot find method "test(\bigint)"
      public model void t320(){ test(i <= x ? i : (\bigint) i / x); }           // character 68 error: Cannot find method "test(\bigint)"
      public model void t321(){ test(i <= x ? i : (\bigint) i | (int) x); }     // character 74 error: Cannot find method "test(\bigint)"
      public model void t322(){ test(i <= x ? i : (int) x & i); }               // character 64 error: Cannot find method "test(int)"
      public model void t323(){ test(i <= x ? i : x >> (int) i); }              // character 65 error: Cannot find method "test(\bigint)"
      public model void t324(){ test(i <= x ? i : x << i); }                    // character 59 error: Cannot find method "test(\bigint)"
      public model void t325(){ test(i <= x ? i : x + (\bigint) i); }           // character 68 error: Cannot find method "test(\bigint)"
      public model void t326(){ test(i <= x ? i : (int) x - (\bigint) i); }     // character 74 error: Cannot find method "test(\bigint)"
      public model void t327(){ test(i <= x ? i : i < 0 ? 1 : x); } 		// character 66 error: Cannot find method "test(\bigint)"
      public model void t328(){ test(i <= x ? i : (1 % x)); }			// character 59 error: Cannot find method "test(\bigint)"


      // General & misc arithmetic expressions
      //--------------------------------------

      public model void t329() {
        test((-x > 5 + i && (\bigint)i == 5 ? 5L : (\bigint) 0) * x - i);
      } // line -1, character 74 error: Cannot find method "test(\bigint)"
      @*/

    // Binary Addition with Integers and Constant Folding

    //@ model public void t351(){ int i = int_max + 1; }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t352(){ int i = \warn(int_max + 1); }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t353(){ int i = \nowarn(int_max + 1); }
    //@ model public void t354(){ int i = \warn_op(int_max + 1); }	// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t355(){ int i = \nowarn_op(int_max + 1); }

    // Binary Addition with Long and Constant Folding

    //@ model public void t356(){ long l = long_max + 1; }		// character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t357(){ long l = \warn(long_max + 1); }	// character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t358(){ long l = \nowarn(long_max + 1); }
    //@ model public void t359(){ long l = \warn_op(long_max + 1); }	// character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t360(){ long l = \nowarn_op(long_max + 1); }

    // Binary Subtraction with Integers and Constant Folding

    //@ model public void t361(){ int i = int_min - 1; }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t362(){ int i = \warn(int_min - 1); }		// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t363(){ int i = \nowarn(int_min - 1); }
    //@ model public void t364(){ int i = \warn_op(int_min - 1); }	// character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t365(){ int i = \nowarn_op(int_min - 1); }

    // Binary Subtraction with Long and Constant Folding

    //@ model public void t366(){ long l = long_min - 1; }		// character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t367(){ long l = \warn(long_min - 1); }	// character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t368(){ long l = \nowarn(long_min - 1); }
    //@ model public void t369(){ long l = \warn_op(long_min - 1); }	// character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t370(){ long l = \nowarn_op(long_min - 1); }

    // Binary Division with Integers and Constant Folding

    //@ model public void t371(){ int i = int_min / -1; }               // character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t372(){ int i = \warn(int_min / -1); }        // character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t373(){ int i = \nowarn(int_min / -1); }
    //@ model public void t374(){ int i = \warn_op(int_min / -1); }     // character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t375(){ int i = \nowarn_op(int_min / -1); }

    // Binary Division with Long and Constant Folding

    //@ model public void t376(){ long l = long_min / -1; }             // character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t377(){ long l = \warn(long_min / -1); }      // character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t378(){ long l = \nowarn(long_min / -1); }
    //@ model public void t379(){ long l = \warn_op(long_min / -1); }   // character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t380(){ long l = \nowarn_op(long_min / -1); }

    // Binary Multiplication with Integers and Constant Folding

    //@ model public void t381(){ int i = int_min * -1; }               // character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t382(){ int i = \warn(int_min * -1); }        // character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t383(){ int i = \nowarn(int_min * -1); }
    //@ model public void t384(){ int i = \warn_op(int_min * -1); }     // character 40 error: Variable "i" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t385(){ int i = \nowarn_op(int_min * -1); }

    // Binary Multiplication with Long and Constant Folding

    //@ model public void t386(){ long l = long_min * -1; }             // character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t387(){ long l = \warn(long_min * -1); }      // character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t388(){ long l = \nowarn(long_min * -1); }
    //@ model public void t389(){ long l = \warn_op(long_min * -1); }   // character 41 error: Variable "l" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
    //@ model public void t390(){ long l = \nowarn_op(long_min * -1); }

      // Another series, testing assignment conversions
      //-----------------------------------------------

    /*@
      public normal_behavior
      old int i0 = 3;
      old int i1 = -3;
      old int i2 = 3 + (\bigint)4; // character 17 error: Variable "i2" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
      old int i3 = 3 - (\bigint)4; // character 17 error: Variable "i3" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
      old int i4 = (\bigint)3 * 4; // character 17 error: Variable "i4" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
      old int i5 = 3 / (\bigint)4; // character 17 error: Variable "i5" cannot be initialized by a value of type "\bigint" [JLS 4.5.4]
      ensures true;
      @*/
    public /*@pure@*/ void test330() { }

    /*@
      public normal_behavior
      old \bigint b0 = 3;
      old \bigint b1 = -3;
      old \bigint b2 = 3 + 4;
      old \bigint b3 = 3 - 4;
      old \bigint b4 = 3 * 4;
      old \bigint b5 = 3 / 4;
      old \bigint b6 = -1 + 3 * 4 - 5 / 6;
      ensures true;
      @*/
    public /*@pure@*/ void test331() { }

    /*@
      public normal_behavior
      old \bigint b0 = true;	// character 21 error: Variable "b0" cannot be initialized by a value of type "boolean" [JLS 4.5.4]
      old \bigint b1 = (byte)1;
      old \bigint b2 = (short)2;
      old \bigint b3 = '3';
      old \bigint b4 = 1L + 3L;
      old \bigint b5 = 1.1F;	// character 21 error: Variable "b5" cannot be initialized by a value of type "float" [JLS 4.5.4]
      old \bigint b6 = 1.1;	// character 21 error: Variable "b6" cannot be initialized by a value of type "double" [JLS 4.5.4]
      ensures true;
      @*/
    public /*@pure@*/ void test332() { }

      ////////////////////////////////////////////////////////////////////////////////
      // REVISE FROM HERE
      // REVISE FROM HERE
      // REVISE FROM HERE
      // use casts to ensure that we get the same results with or without
      // explicit promotion.

    // Binary tests
    //-------------
    /*@
      public model void t333(){ test((\bigint) i + i); }	// character 55 error: Cannot find method "test(\bigint)"
      public model void t334(){ test(i + (\bigint) i); }	// character 55 error: Cannot find method "test(\bigint)"
      public model void t335(){ test_bigint((\bigint) i + i); }
      public model void t336(){ test_bigint(i + (\bigint) i); }
      public model void t337(){ test_bigint(i+x); }
      public model void t338(){ test_bigint(x+i); }
      public model void t339(){ test(i+x); }		// character 43 error: Cannot find method "test(\bigint)"
      public model void t340(){ test(x+i); }		// character 43 error: Cannot find method "test(\bigint)"
      @*/

    // Implicit Promotion
    //-------------------
    /*xxxxxx >>>>>>>>>>>>>>>>>>>>>> should be placed in another file.
      public model void t341(){ test_bigint(-i); }
      public model void t342(){ test_bigint(i + i); }
      public model void t343(){ test_int(-(short)i); }
      public model void t344(){ test_int(i + (int) i); }
      public model void t345(){ test_int((int)i - i); }
      public model void t346(){ test_bigint(-(i2)); }
      public model void t347(){ test_int(-(int)i2); }
      public model void t348(){ test_int(-i); }
      public model void t349(){ test_int(i + i); }
      public model void t350(){ test_int(-(i2)); }
      xxxxxxx*/

    //@ represents x <- i;
    //@ static represents sx <- si;
    //@ represents x2 <- i;
    //@ static represents sx2 <- si;
    //@ represents i2 <- i;
    //@ static represents si2 <- si;
}

// @(#)$Id $
//
// Copyright (C) 2003 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
