// @(#)$Id: PurityFromInterfacePropagation.java,v 1.3 2004/07/09 15:56:31 f_rioux Exp $

// Copyright (C) 2003 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

// This demonstrates errors in not propogating "pure" from an
// interface declaration to its methods.  Note that the errors don't
// occur if we change both interfaces to classes.

public /*@ spec_bigint_math @*/ interface PurityFromInterfacePropogation
    extends PurityFromInterfacePropogationHelperA
{
    //@ assignable \nothing;
    //@ ensures \result == this.hashCode();  // should be okay
    /*@ pure @*/ int hc0();

    //@ ensures \result == this.hc0();
    int hc1();

    // That the following is okay means it's only for overridden methods
    // in pure classes; perhaps only for those that come from Object?
    //@ ensures \result == this.hc();
    int hc2();

    //@ assignable \nothing;
    //@ ensures \result == hashCode();  // should be okay
    /*@ pure @*/ int hc3();

    //@ ensures \result == hc3();
    int hc4();

    // That the following is okay means it's only for overridden methods
    // in pure classes; perhaps only for those that come from Object?
    //@ ensures \result == hc();
    int hc5();

    // the following should be okay
    //@ ensures \result != null && \result.equals(this.toString());
    String tos();

    // the following should be okay
    //@ ensures \result != null && \result.equals(toString());
    String tos1();

    //@ ensures \result == zero() + one();
    int addzeroone();

    //@ ensures \result == this.zero() + this.one();
    int add01();
}

/*@ pure @*/ interface PurityFromInterfacePropogationHelperA extends Comparable
{
    //@ also
    //@ ensures \result >= 3;
    /*@ pure @*/ int hashCode();

    //@ ensures \result >= this.hashCode();
    int hc();

    /*@ pure @*/ String toString();

    //@ ensures \result != null && \result.equals(this.toString());
    /*@ pure @*/ String tos0();

    //@ ensures \result == 0;
    int zero();

    //@ ensures \result == 1;
    /*@ pure @*/ int one();
}

class PurityFromInterfacePropogationClient {

    //@ public model PurityFromInterfacePropogation pfip;

    //@ public invariant pfip != null;

    //@ requires pfip == o;
    //@ ensures pfip.toString() != null;  // should be okay
    //@ ensures \result >= pfip.hashCode();  // should be okay
    public int foo( Object o) {
        return o.hashCode();
    }

    protected PurityFromInterfacePropogation pfip_;
    //@ protected represents pfip <- pfip_;
}
