// @(#)$Id: PurityFromInterfacePropagationB.java,v 1.5 2007/02/08 14:05:50 leavens Exp $

// Copyright (C) 2003 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

// This demonstrates errors in not propogating "pure" from an
// interface declaration to its methods.  Note that the errors don't
// occur if we change the interfaces to classes.

public /*@ spec_bigint_math @*/ class PurityFromInterfacePropogationB
    implements PurityFromInterfacePropogationHelperB
{
    public /*@ pure @*/ int hc() { return 0; }

    //@ assignable \nothing;
    //@ ensures \result == this.hashCode();  // should be okay
    public /*@ pure @*/ int hc0() { return 0; }

    //@ ensures \result == this.hc0();
    public int hc1() { return 0; }

    // That the following is okay means it's only for overridden methods
    // in pure classes; perhaps only for those that come from Object?
    //@ ensures \result == this.hc();
    public int hc2() { return 0; }

    //@ assignable \nothing;
    //@ ensures \result == hashCode();  // should be okay
    public /*@ pure @*/ int hc3() { return 0; }

    //@ ensures \result == hc3();
    public int hc4() { return 0; }

    // That the following is okay means it's only for overridden methods
    // in pure classes; perhaps only for those that come from Object?
    //@ ensures \result == hc();
    public int hc5() { return 0; }

    // the following should be okay
    //@ ensures \result != null && \result.equals(this.toString());
    public String tos() { return ""; }

    public String tos0() { return ""; }

    // the following should be okay
    //@ ensures \result != null && \result.equals(toString());
    public String tos1() { return ""; }

    //@ ensures \result == zero() + one();
    public int addzeroone() { return 1; }

    //@ ensures \result == this.zero() + this.one();
    public int add01() { return 2; }

    public int zero() { return 0; }
    public int one() { return 1; }
    public int compareTo(/*@ non_null @*/ Object o) { return 1; }
}

/*@ pure @*/ interface PurityFromInterfacePropogationHelperB extends Comparable
{
    //@ also
    //@ ensures \result >= 3;
    /*@ pure @*/ int hashCode();

    //@ ensures \result >= this.hashCode();
    int hc();

    /*@ pure @*/ String toString();

    //@ ensures \result != null && \result.equals(this.toString());
    /*@ pure @*/ String tos0();

    //@ ensures \result == 0;
    int zero();

    //@ ensures \result == 1;
    /*@ pure @*/ int one();
}

class PurityFromInterfacePropogationClient {

    //@ public model PurityFromInterfacePropogationB pfip;

    //@ public invariant pfip != null;

    //@ requires pfip == o;
    //@ ensures pfip.toString() != null;  // should be okay
    //@ ensures \result >= pfip.hashCode();  // should be okay
    public int foo( Object o) {
        return o.hashCode();
    }

    protected PurityFromInterfacePropogationB pfip_;
    //@ protected represents pfip <- pfip_;
}
