/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestsWithNoOutput.java,v 1.3 2006/08/08 17:18:50 perryjames Exp $
 * Author: David R. Cok
 */
package org.jmlspecs.checker.testcase.typecheck.nonnulltypes;

import org.multijava.util.testing.TestCase;
import org.multijava.util.testing.*;
import junit.framework.*;
import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;

/** This is a JUnit test case that executes tests in the package in 
    which it resides.  There is just one test here.  It runs all the
    test files that produce no output; it does so in one 
    combined execution of the checker's compile().
<P>
    The test must be run from the directory in which it resides - because
    it opens files in the current directory.

    @author David R. Cok
*/
public class TestsWithNoOutput  extends TestCase {

    /** The constructor called by the JUnit framework. */
    public TestsWithNoOutput(String name) {
	    super(name);
    }

    /** The suite method called by the junit framework */
    public static junit.framework.Test suite() {
        return new junit.framework.TestSuite(TestsWithNoOutput.class);
    }

    /** This must match the flags used in the Makefile (JMLFLAGS). */
    String[] defaultFlags = { "-Q", "-n", "-j", "-k" };

    /** This test exercises all the tests that are supposed to execute
	without producing any output.  They are all run at once.
	The result of Main.compile() has to be true with no output.
    */
    public void testFilesWithNoOutput() throws java.io.IOException {
	//System.out.println("\nJML tests without output");
	
	// First get all the files that are to be tested.
	// Run the external process, which must produce a list of the
	// test filenames, one per line.  A reader reads them into
	// an ArrayList. 
	ArrayList a = new ArrayList(100);
	Iterator i = new FileIterator("listTestsWithoutOutput");
	while (i.hasNext()) {
	    String fn = (String)i.next();
	    a.add(fn);
	    //System.out.println("READ " + fn);
	}

	// Now create a command-line with the options and all the filenames.
	String[] args = new String[defaultFlags.length+a.size()];
	System.arraycopy(defaultFlags,0,args,0,defaultFlags.length);
	int j = defaultFlags.length;
	i = a.iterator();
	while (i.hasNext()) args[j++] = (String)(i.next());

	// Set the output streams to write into the ByteArray
	// and then execute the compile
	ByteArrayOutputStream ba = new ByteArrayOutputStream();
	boolean b = false;
	try {
	    Utils.setStreams(new PrintStream(ba));
	    b = org.jmlspecs.checker.Main.compile(args);
	} finally {
	    Utils.restoreStreams();
	}
	String s = ba.toString();

	// Check the the result value is true and that there is no
	// output.
	assertTrue("Running the jml checker on all no-output tests should produce no output - but this was produced: " + System.getProperty("line.separator") + s, s.length() == 0);
	assertTrue("Running the jml checker on all no-output tests should produce a success result (Main.compile() returns false when it should return true)",b);

    }
}


