/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JmlClassDoc.java,v 1.2 2004/03/12 03:13:59 davidcok Exp $
 * Author: David R. Cok
 */

package org.jmlspecs.jmldoc.jmldoc_141;
import org.jmlspecs.checker.*;
import org.multijava.mjdoc.mjdoc_141.*;
import org.multijava.mjc.*;
import com.sun.javadoc.*;
import java.util.ArrayList;

public class JmlClassDoc extends MjClassDoc {

    /** Constructs an object from a CClass object (to be used when source code is not available). */
    public JmlClassDoc(CClass cc) {
	super(cc); 
    }

    /** Returns an array of all the nested classes and nested interfaces of this class. */
    public ClassDoc[] innerClasses() { 
	if (isInterface()) return new MjClassDoc[0];
	CClass cl = cclass();
	JmlTypeDeclaration jt = JmlTypeLoader.getJmlSingleton().typeDeclarationOf(cl);
	
	if (jt == null) return super.innerClasses();
	
	ArrayList ct = jt.inners(); // FIXME - Inner or Nested?  are they really inner?
	ArrayList r = new ArrayList(ct.size());
	for (int i=0; i<ct.size(); ++i) {
	    CClass c = ((JTypeDeclarationType)ct.get(i)).getCClass();
	    if (memberFilter.use(c)) r.add(MjdocWrapper.lookup(c));
	}
	return (ClassDoc[])(r.toArray(new ClassDoc[0]));
    }
    
    public int compare(MjClassDoc d) {
	if (!(d instanceof JmlClassDoc)) return 0;
	JmlClassDoc jd = (JmlClassDoc)d;
	if (! name().equals(d.name()) ) return 0;
	if (!(cclass() instanceof JmlSourceClass) || !(jd.cclass() instanceof JmlSourceClass)) return 0;
	JmlSourceClass jsc = (JmlSourceClass)cclass();
	JmlSourceClass mjsc = (JmlSourceClass)jd.cclass();
	if (jsc.isMoreRefinedThan(mjsc)) {
//System.out.println("REMOVING " + mjsc.getAST().getTokenReference() + " IN FAVOR OF " + jsc.getAST().getTokenReference());
	    return -1;
	}
//System.out.println("SKIPPING " + jsc.getAST().getTokenReference() + " IN FAVOR OF " + mjsc.getAST().getTokenReference());
	return 1;
    }
}
