/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JmldocClassWriter.java,v 1.2 2005/05/03 04:25:27 kui_dai Exp $
 * Author: David R. Cok
 */

package org.jmlspecs.jmldoc.jmldoc_141;

import org.jmlspecs.jmldoc.*;
import org.multijava.mjc.JTypeDeclarationType;
import org.multijava.mjdoc.mjdoc_141.*;
import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.Arrays;
import org.jmlspecs.checker.*;
import org.multijava.mjc.JMemberDeclarationType;
import org.multijava.mjc.CMember;


/** A derivative of a class in the javadoc doclet API, so as to be able to
    instantiate a MjdocClassWriter instead of a ClassWriter and a 
    MjdocMethodSubWriter instead of a MethodSubWriter.
    @author David R. Cok
*/
public class JmldocClassWriter extends MjdocClassWriter {

    protected JmldocFieldSubWriter modelfieldSubWriter;
    protected JmldocFieldSubWriter ghostfieldSubWriter;
    protected JmldocMethodSubWriter modelMethodSubWriter;
    protected JmldocClassSubWriter modelNestedSubWriter;
    protected JmldocConstructorSubWriter modelConstructorSubWriter;

    public static void generate(ConfigurationStandard configuration,
			    ClassDoc classdoc, ClassDoc prev,
                             ClassDoc next, ClassTree classtree,
                             boolean nopackage) {
            ClassWriter clsgen;
            String pkgpath =
               DirectoryManager.getDirectoryPath(classdoc.containingPackage());
            String filename = classdoc.name() + ".html";
            try {
                clsgen = new JmldocClassWriter(configuration,
					pkgpath, filename, classdoc,
                                         prev, next, classtree, nopackage);
                clsgen.generateClassFile();
		// FIXME - what does this do ????
		PackageDoc cp = classdoc.containingPackage();
		if((configuration.packages == null ||
			Arrays.binarySearch(configuration.packages, cp) < 0) &&
		    ! containingPackagesSeen.contains(cp.name())){
		    
		    //Only copy doc files dir if the containing package is not documented
		    //AND if we have not documented a class from the same package already.
		    //Otherwise, we are making duplicate copies.
		    clsgen.copyDocFiles(configuration, getSourcePath(configuration, classdoc.containingPackage()),
				pkgpath + fileseparator + DOC_FILES_DIR_NAME, true);
		    containingPackagesSeen.add(cp.name());
		}

                clsgen.close();
            } catch (IOException exc) {
                configuration.standardmessage.
                    error("doclet.exception_encountered",
                           exc.toString(), filename);
                throw new DocletAbortException();
            }
    }
    
    public JmldocClassWriter(ConfigurationStandard configuration,
		String path, String filename, ClassDoc classdoc,
                ClassDoc prev, ClassDoc next, ClassTree classtree,
                boolean nopackage) throws IOException, DocletAbortException {

	super(configuration,path,filename,classdoc,prev,next,classtree,nopackage);

	((JmlMemberFilter)MjClassDoc.memberFilter).useModFields(0);
	methodSubWriter = new JmldocMethodSubWriter(this,classdoc,configuration);
	constrSubWriter = new JmldocConstructorSubWriter(this,classdoc);
	fieldSubWriter = new JmldocFieldSubWriter(this,classdoc,0); // no model fields
        nestedSubWriter = new JmldocClassSubWriter(this,classdoc);

	MjClassDoc.addDefaultConstructors = false;
	((JmlMemberFilter)MjClassDoc.memberFilter).useModFields(Constants.ACC_MODEL);
	modelMethodSubWriter = new JmldocMethodSubWriter(this,classdoc,Constants.ACC_MODEL,configuration);
	modelConstructorSubWriter = new JmldocConstructorSubWriter(this,classdoc,Constants.ACC_MODEL);
	modelfieldSubWriter = new JmldocFieldSubWriter(this,classdoc,Constants.ACC_MODEL); //  model fields
	modelNestedSubWriter = new JmldocClassSubWriter(this,classdoc,Constants.ACC_MODEL);

	((JmlMemberFilter)MjClassDoc.memberFilter).useModFields(Constants.ACC_GHOST);
	ghostfieldSubWriter = new JmldocFieldSubWriter(this,classdoc,Constants.ACC_GHOST); //  ghost fields

	MjClassDoc.addDefaultConstructors = true;
	((JmlMemberFilter)MjClassDoc.memberFilter).useModFields(0);
    }


    private JmlHTML jmlwriter = null;
    public JmlHTML jmlwriter() {
	if (jmlwriter == null) {
	    jmlwriter = new JmlHTML((JmldocOptions)Main.jmlOptions);
	}
	return jmlwriter;
    }
       
    protected void printAllMembers() {
	JTypeDeclarationType j = ((MjClassDoc)classdoc).jtype();
	if (j instanceof JmlTypeDeclaration) {
	    String s = jmlwriter().generateClassSpecification((JmlTypeDeclaration)j, this);
	    print(s);
	}
	//super.printAllMembers();

        println("<!-- ======== NESTED CLASS SUMMARY ======== -->"); println();
        modelNestedSubWriter.printMembersSummary();
        modelNestedSubWriter.printInheritedMembersSummary();
        nestedSubWriter.printMembersSummary();
        nestedSubWriter.printInheritedMembersSummary();
        println();
        println("<!-- ======== MODEL FIELD SUMMARY ======== -->"); println();
        modelfieldSubWriter.printMembersSummary();
        modelfieldSubWriter.printInheritedMembersSummary();
        println();
        println("<!-- ======== GHOST FIELD SUMMARY ======== -->"); println();
        ghostfieldSubWriter.printMembersSummary();
        ghostfieldSubWriter.printInheritedMembersSummary();
        println();
        println("<!-- =========== FIELD SUMMARY =========== -->"); println();
        fieldSubWriter.printMembersSummary();
        fieldSubWriter.printInheritedMembersSummary();
        println();
        println("<!-- ===== MODEL CONSTRUCTOR SUMMARY ===== -->"); println();
        modelConstructorSubWriter.printMembersSummary();
        println();
        println("<!-- ======== CONSTRUCTOR SUMMARY ======== -->"); println();
        constrSubWriter.printMembersSummary();
        println();
        println("<!-- ======= MODEL METHOD SUMMARY ======== -->"); println();
        modelMethodSubWriter.printMembersSummary();
        modelMethodSubWriter.printInheritedMembersSummary();
        println("<!-- ========== METHOD SUMMARY =========== -->"); println();
        methodSubWriter.printMembersSummary();
        methodSubWriter.printInheritedMembersSummary();

        p();

        println();
        println("<!-- ========= MODEL FIELD DETAIL ======== -->"); println();
        modelfieldSubWriter.printMembers();
        println();
        println("<!-- ========= GHOST FIELD DETAIL ======== -->"); println();
        ghostfieldSubWriter.printMembers();
        println();
        println("<!-- ============ FIELD DETAIL =========== -->"); println();
        fieldSubWriter.printMembers();
        println();
        println("<!-- ====== MODEL CONSTRUCTOR DETAIL ===== -->"); println();
        modelConstructorSubWriter.printMembers();
        println();
        println("<!-- ========= CONSTRUCTOR DETAIL ======== -->"); println();
        constrSubWriter.printMembers();
        println();
        println("<!-- ========= MODEL METHOD DETAIL ======= -->"); println();
        modelMethodSubWriter.printMembers();
        println();
        println("<!-- ============ METHOD DETAIL ========== -->"); println();
        methodSubWriter.printMembers();

    }
    
    public static class JmlMemberFilter extends org.multijava.mjdoc.mjdoc_141.MjClassDoc.MjMemberFilter {
	public boolean use(CMember f) { 
            if (!super.use(f)) return false;
            long mods = f.modifiers(); 
            boolean isModelOrGhost =
                ((mods & (org.jmlspecs.checker.Constants.ACC_MODEL+
                          org.jmlspecs.checker.Constants.ACC_GHOST)) != 0);
            if (useModFields == 0) return !isModelOrGhost;
            else return (mods & useModFields) != 0;

	}
	public boolean use(JMemberDeclarationType f) {
	    if (!super.use(f)) return false;
	    long mods = f.modifiers();
	    boolean isModelOrGhost =  
		((mods & (org.jmlspecs.checker.Constants.ACC_MODEL+
			  org.jmlspecs.checker.Constants.ACC_GHOST)) != 0); 
	    if (useModFields == 0) return !isModelOrGhost;
	    else return (mods & useModFields) != 0;
	}

	public void useModFields(long m) { useModFields = m; }

	private long useModFields = 0;
    }

    /**
     * This is overridden so that we can include the modifier weakly.
     */
    // Note - the List may be of subclasses, subinterfaces, or super interfaces
    // Only the latter case needs the weakly modifiers.
    protected void printSubClassLinkInfo(java.util.List list) {
        int i = 0;
        Object[] classDocList = list.toArray();
        //Sort the list to be printed.
        Arrays.sort(classDocList);
        print(' ');
        dd();
        for (; i < list.size() - 1; i++) {
            printClassLink((ClassDoc)(classDocList[i]));
	    //print(" <B>weakly</B>");
            print(", ");
        }
        printClassLink((ClassDoc)(classDocList[i]));
	//print(" <B>weakly</B>");
        ddEnd();
        dlEnd();
    }

    /**
     * Print the class description regarding interfaces implemented, classes
     * inherited; this is overridden just to introduce the weakly modifier
     */
    protected void printClassDescription() {
        boolean isInterface = classdoc.isInterface();
        dl();
        dt();
        print(classdoc.modifiers() + " ");
        if (!isInterface) {
            print("class ");
        }
        if (configuration().genSrc) {
            printSrcLink(classdoc, classdoc.name());
        } else {
            bold(classdoc.name());
        }
        if (!isInterface) {
            ClassDoc superclass = classdoc.superclass();
            if (superclass != null) {
                dt();
                print("extends ");
                printClassLink(superclass);
		//print(" <B>weakly</B>");
            }
        }
        ClassDoc[] implIntfacs = classdoc.interfaces();
        if (implIntfacs != null && implIntfacs.length > 0) {
            dt();
            print(isInterface? "extends " : "implements ");
            printClassLink(implIntfacs[0]);
	    //print(" <B>weakly</B>");
            for (int i = 1; i < implIntfacs.length; i++) {
                print(", ");
                printClassLink(implIntfacs[i]);
		//print(" <B>weakly</B>");
            }
        }
        dlEnd();
    }

}








