/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JmldocConstructorSubWriter.java,v 1.3 2005/05/03 04:25:27 kui_dai Exp $
 * Author: David R. Cok
 */
// FIXME - needs documentation in this file. 

package org.jmlspecs.jmldoc.jmldoc_141;

import org.multijava.mjdoc.mjdoc_141.*;
import com.sun.tools.doclets.standard.*;
import com.sun.javadoc.*;
import java.lang.reflect.Modifier;
import org.multijava.mjc.*;
import org.jmlspecs.checker.JmlConstructorDeclaration;
import org.jmlspecs.checker.JmlTypeDeclaration;
import org.jmlspecs.checker.Constants;

/** This is an extension of the doclet api in order to provide
    functionality for writing JML annotations.
    @author David R. Cok
*/
public class JmldocConstructorSubWriter extends MjdocConstructorSubWriter {

    protected JmlHTML jmlwriter;
    protected long modFields = 0; // Either 0 or ACC_MODEL 

    
    public JmldocConstructorSubWriter(SubWriterHolderWriter writer, ClassDoc classdoc, long mods) {
        super(writer, classdoc);
	jmlwriter = ((JmldocClassWriter)writer).jmlwriter();
	modFields = mods;
    }

    public JmldocConstructorSubWriter(SubWriterHolderWriter writer, ClassDoc classdoc) {
        super(writer, classdoc);
	jmlwriter = ((JmldocClassWriter)writer).jmlwriter();
	modFields = 0;
    }

    public JmldocConstructorSubWriter(SubWriterHolderWriter writer) {
        super(writer);
	jmlwriter = ((JmldocClassWriter)writer).jmlwriter();
	modFields = 0;
    }

    // Overridden in order to insert specifications
    protected void printFullComment(ProgramElementDoc emd) {
        if (configuration().nocomment) {
            return;
        }
        writer.dl();
        printDeprecated(emd);
        printCommentAndTags(emd);
	printSpecifications(emd);
        writer.dlEnd();    
    }
    
    protected void printSpecifications(ProgramElementDoc emd) {
	if (emd instanceof MjConstructorDoc) {
	    MjConstructorDoc mj = (MjConstructorDoc)emd;
	    JMethodDeclarationType jm = mj.jmethod();
	    if (jm instanceof JmlConstructorDeclaration) {
		CClass cc = ((MjClassDoc)(mj.containingClass())).cclass();
		String s = jmlwriter.generateMethodSpecification((JmlConstructorDeclaration)jm,cc,mj.signature(),writer); 
		if (s != null) print(s);
	    }
	}
    }

    public void printSummaryLabel(ClassDoc cd) {
        if (modFields == Constants.ACC_MODEL) writer.bold("Model Constructor Summary");
        else super.printSummaryLabel(cd);
    }   

    public void printSummaryAnchor(ClassDoc cd) {
        if (modFields == Constants.ACC_MODEL) writer.anchor("model_constructor_summary");
        else super.printSummaryAnchor(cd);
    }   
        
    protected void printHeader(ClassDoc cd) {
        if (modFields == Constants.ACC_MODEL) {
            writer.anchor("model_constructor_detail");
            writer.printTableHeadingBackground("Model Constructor Detail");
        }
        else super.printHeader(cd);
    }

    /** This is overridden in order to insert spec_public and 
	spec_protected as appropriate in the list of modifiers for
	fields.
    */
    protected void printSummaryType(ProgramElementDoc member) {
        if (foundNonPublicMember) { // If every constructor is public, no space is made for modifier/type
            writer.printTypeSummaryHeader();
	    if (member instanceof MjConstructorDoc) {
		long m = ((MjConstructorDoc)member).longModifiers();
		print(org.jmlspecs.jmldoc.Utils.modifierString(this,m));
	    }
	    if (member.isProtected()) {
		print("protected ");
	    } else if (member.isPrivate()) {
		print("private ");
	    } else if (member.isPublic()) {
		writer.space();
	    } else { // Package private
		writer.printText("doclet.Package_private");
	    }
            writer.printTypeSummaryFooter();
        }
    }

}
