/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JmldocFieldSubWriter.java,v 1.3 2005/05/03 04:25:27 kui_dai Exp $
 * Author: David R. Cok
 */
// FIXME - needs documentation in this file. 

package org.jmlspecs.jmldoc.jmldoc_141;

import org.multijava.mjdoc.mjdoc_141.*;
import com.sun.tools.doclets.*;
import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.standard.tags.*;
import com.sun.javadoc.*;
import java.lang.reflect.Modifier;
import org.multijava.mjc.*;
import org.jmlspecs.checker.JmlFieldDeclaration;
import org.jmlspecs.checker.Constants;
import org.jmlspecs.checker.JmlTypeDeclaration;
import org.jmlspecs.checker.JmlSourceClass;
import org.multijava.util.compiler.ModifierUtility;
import java.util.List;

/** 
    @author David R. Cok
*/
public class JmldocFieldSubWriter extends MjdocFieldSubWriter {

    protected JmlHTML jmlwriter;
    protected long modFields = 0; // Either 0 or ACC_MODEL or ACC_GHOST
    
    public JmldocFieldSubWriter(SubWriterHolderWriter writer, ClassDoc classdoc,
		long modFields) {
        super(writer, classdoc); // modFields==-1 and memberFilter are used in generating a VisibleMemberMap
	this.modFields = modFields;
	jmlwriter = ((JmldocClassWriter)writer).jmlwriter();
    }

    public JmldocFieldSubWriter(SubWriterHolderWriter writer) {
        super(writer);
	jmlwriter = ((JmldocClassWriter)writer).jmlwriter();
    }

    /** Overridden in order to insert any JML specification. */
    protected void printFullComment(ProgramElementDoc emd) {
        if(configuration().nocomment){
            return;
        }
        writer.dl();
        printDeprecated(emd);
        printCommentAndTags(emd);
	printSpecification(emd);
        writer.dlEnd();    
    }
    
    protected void printSpecification(ProgramElementDoc emd) {
    	if (emd instanceof MjFieldDoc) {
	    MjFieldDoc mj = (MjFieldDoc)emd;
	    JFieldDeclarationType jm = mj.jfield();
	    if (jm instanceof JmlFieldDeclaration) {
		String s = jmlwriter.generateFieldSpecification((JmlFieldDeclaration)jm,this); 
		if (s != null) {
		    print(s);
		}
	    }
	}
    }

    public String getLink(CClass container, CField fd, String label) {
	String id = fd.ident();
	CClass cc = fd.owner();
	if (cc instanceof JmlSourceClass) cc = ((JmlSourceClass)cc).getMostRefined();
	if (label == null) {
	    if (cc.equals(container))
		label = id;
	    else
		label = fd.qualifiedName().replace('/','.');
	}
	MjClassDoc cd = MjdocWrapper.lookup(cc);
	String s = writer.getClassLink(cd,id,label);
//System.out.println("LINL " + s + " " + (cd != null) + " " + cd.isIncluded() + " " + writer.isGeneratedDoc(cd) + " " + cd.name() + " " + cd.getClass());
	return s;
    }

    public void printSummaryLabel(ClassDoc cd) {
	if (modFields == Constants.ACC_MODEL) writer.bold("Model Field Summary");
	else if (modFields == Constants.ACC_GHOST) writer.bold("Ghost Field Summary");
        else super.printSummaryLabel(cd);
    }       
        
    public void printSummaryAnchor(ClassDoc cd) {
        if (modFields == Constants.ACC_MODEL) writer.anchor("model_field_summary");
        else if (modFields == Constants.ACC_GHOST) writer.anchor("ghost_field_summary");
	else super.printSummaryAnchor(cd);
    }                                         
    
    public void printInheritedSummaryAnchor(ClassDoc cd) {
        if (modFields == Constants.ACC_MODEL) writer.anchor("model_fields_inherited_from_class_" + cd.qualifiedName());
        else if (modFields == Constants.ACC_GHOST) writer.anchor("ghost_fields_inherited_from_class_" + cd.qualifiedName());
	else super.printInheritedSummaryAnchor(cd);
    }

    public void printInheritedSummaryLabel(ClassDoc cd) {
        if (modFields == 0) super.printInheritedSummaryLabel(cd);
	else {
	    String classlink = writer.getPreQualifiedClassLink(cd);
	    writer.bold();
	    writer.print((modFields==Constants.ACC_MODEL?"Model":"Ghost") + 
			    " fields inherited from " +
			    (cd.isClass()?"class ":"interface ") + 
			    classlink);
	    writer.boldEnd();
	}
    }

    protected void printHeader(ClassDoc cd) {
	if (modFields == Constants.ACC_MODEL) {
	    writer.anchor("model_field_detail");
	    writer.printTableHeadingBackground("Model Field Detail");
	} else if (modFields == Constants.ACC_GHOST) {
	    writer.anchor("ghost_field_detail");
	    writer.printTableHeadingBackground("Ghost Field Detail");
	}
        else super.printHeader(cd);
    }

    // This is an odd method to be overriding, but it is necessary in order to
    // get spec_public and spec_protected members of super classes to show up
    // in the javadoc document under fields inherited from the super class.
    protected boolean isInherited(ProgramElementDoc ped){
	if (super.isInherited(ped)) return true;
	if (!(ped instanceof MjFieldDoc)) return false;
	MjFieldDoc f = (MjFieldDoc)ped;
	if (org.multijava.util.Utils.hasFlag(f.longModifiers(),
			Constants.ACC_SPEC_PUBLIC)) return true;
	if (org.multijava.util.Utils.hasFlag(f.longModifiers(),
			Constants.ACC_SPEC_PROTECTED)) return true;
	return false;
    }

    /** This is overridden in order to insert spec_public and 
	spec_protected as appropriate in the list of modifiers for
	fields.
    */
    protected void printModifier(ProgramElementDoc member) {
	if (member instanceof MjFieldDoc) {
	    long m = ((MjFieldDoc)member).longModifiers();
	    print(org.jmlspecs.jmldoc.Utils.modifierString(this,m));
	}
	super.printModifier(member);
    }

}
