/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JmldocWrapper.java,v 1.8 2007/02/08 14:05:50 leavens Exp $
 * Author: David R. Cok
 */

package org.jmlspecs.jmldoc.jmldoc_141;

import org.multijava.mjc.*;
import org.multijava.mjdoc.JavadocOptions;
import org.multijava.mjdoc.mjdoc_141.*;
import org.multijava.util.compiler.Compiler;
import org.jmlspecs.checker.*;
import org.jmlspecs.jmldoc.JmldocOptions;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

public class JmldocWrapper extends MjdocWrapper {

    public JmldocWrapper(Compiler c) { 
        super(c);
        mapper = new JmlCClassMap();
    }

    /** This is overridden in order to register a Compilation Unit that is the
        combination of all refinements.
    */
    public void registerTypeDeclarations(JCompilationUnitType unit) {
        if (unit instanceof JmlCompilationUnit)
            registerTypeDeclarations(((JmlCompilationUnit)unit).combinedTypeDeclarations(),unit);
        else
            super.registerTypeDeclarations(unit);
    }
    
    /** This is overridden in order to select a Compilation Unit that is the
        combination of all refinements.
    */
    public void selectTypeDeclarations(JCompilationUnitType unit) {
        if (unit instanceof JmlCompilationUnit)
            selected(((JmlCompilationUnit)unit).combinedTypeDeclarations(),unit);
        else
            super.selectTypeDeclarations(unit);
    }

    static public class RefinementWrapper {
        public CClass cclass;
        public RefinementWrapper(CClass c) { cclass = c; }
        public boolean equals(/*@ nullable @*/ Object o) {
            if (!(o instanceof RefinementWrapper)) return false;
            RefinementWrapper r = (RefinementWrapper)o;
            return r.cclass.qualifiedName().equals(cclass.qualifiedName());
        }
        public int compareTo(/*@ non_null @*/ Object o) {
            RefinementWrapper r = (RefinementWrapper)o;
            return cclass.qualifiedName().compareTo(r.cclass.qualifiedName());
        }
        public int hashCode() {
            return cclass.qualifiedName().hashCode();
        }
    }
    
    Map map = new HashMap();

    public void select(JTypeDeclarationType jtype) {
        //System.out.println("JML-SEL " +jtype.getCClass() + " " + jtype.getTokenReference());
        RefinementWrapper rw = new RefinementWrapper(jtype.getCClass());
        MjClassDoc mjd = lookup(jtype.getCClass());
        RefinementWrapper mrw = (RefinementWrapper)map.get(rw);
        if (mrw == null) {
            map.put(rw,rw);
        } else if (mrw.cclass != rw.cclass) {
            //System.out.println("MATCH " + rw.cclass.qualifiedName() + " "
                //+ rw.cclass.sourceFile() + " " + mrw.cclass.sourceFile());
            // Pick the cclass value which is most refined
            if (rw.cclass instanceof JmlSourceClass && mrw.cclass instanceof JmlSourceClass) {
                JmlSourceClass jsc = (JmlSourceClass)rw.cclass;
                JmlSourceClass mjsc = (JmlSourceClass)mrw.cclass;
                if (jsc.isMoreRefinedThan(mjsc)) {
                    // jsc is more refined than mjsc
                    // remove mjsc from the selected set and add jsc
                    //System.out.println("REPLACING " + mrw.cclass.sourceFile() + " WITH " + rw.cclass.sourceFile());
                    root().unspecify(lookup(mrw.cclass));
                    map.remove(rw);
                    map.put(rw,rw);
                } else {
                    //System.out.println("OMITTING " + rw.cclass.sourceFile());
                    return; // Don't specify the less refined cclass
                }
            }
        }
        //System.out.println("SPECIFIED " + mjd + " " + mjd.jtype().getTokenReference());
        if (!root().isSpecified(mjd.containingPackage())) root().specify(mjd);
    }

    static public class JmlCClassMap extends CClassMap {
        public MjClassDoc newClassDoc(CClass cclass) {
            return new JmlClassDoc(cclass);
        }
    }
    
    /**
     *  Sets Jmldoc's Quiet option since Mjdoc cannot recognize it
     */
    protected void handleQuiet(MjcCommonOptions options) {
        ((JmldocOptions)options).setOption("Quiet", 
            new Boolean(!(((JmldocOptions)options).Quiet())));
    }
}
