/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JmldocClassSubWriter.java,v 1.3 2005/04/26 02:40:22 kui_dai Exp $
 * Author: David R. Cok
 */

package org.jmlspecs.jmldoc.jmldoc_142;
import com.sun.tools.doclets.standard.*;

import com.sun.javadoc.*;
import org.multijava.mjdoc.mjdoc_142.MjClassDoc;
import org.jmlspecs.checker.Constants;

/**
 *
 * @author Robert Field
 * @author Atul M Dambalkar
 */
public class JmldocClassSubWriter extends ClassSubWriter {

    protected long modFields = 0;

    public JmldocClassSubWriter(SubWriterHolderWriter writer, ClassDoc classdoc, long mods) {
        super(writer, classdoc);
	modFields = mods;
    }

    public JmldocClassSubWriter(SubWriterHolderWriter writer, ClassDoc classdoc) {
        super(writer, classdoc);
	modFields = 0;
    }

    public void printSummaryLabel(ClassDoc cd) {
        writer.bold((modFields!=0?"Model ":"")+ "Nested Class Summary"); 
    }

    public void printSummaryAnchor(ClassDoc cd) {
        writer.anchor((modFields!=0?"model_":"")+ "nested_class_summary"); 
    }
    
    public void printInheritedSummaryAnchor(ClassDoc cd) {
        writer.anchor((modFields!=0?"model_":"")+ "nested_classes_inherited_from_class_" + 
                       cd.qualifiedName());
    }   
    
    public void printInheritedSummaryLabel(ClassDoc cd) {
	if (modFields == 0) {
	    super.printInheritedSummaryLabel(cd);
	} else {
	    String clslink = writer.getPreQualifiedClassLink(cd);
	    writer.bold();
	    writer.print("Model nested classes inherited from class "+ clslink);
	    writer.boldEnd();
	}
    }
    
    // This is an odd method to be overriding, but it is necessary in order to
    // get spec_public and spec_protected members of super classes to show up
    // in the javadoc document under fields inherited from the super class.
    protected boolean isInherited(ProgramElementDoc ped){
	if (super.isInherited(ped)) return true;
	if (!(ped instanceof MjClassDoc)) return false;
	MjClassDoc f = (MjClassDoc)ped;
	if (org.multijava.util.Utils.hasFlag(f.longModifiers(), 
			Constants.ACC_SPEC_PUBLIC)) return true;
	if (org.multijava.util.Utils.hasFlag(f.longModifiers(), 
			Constants.ACC_SPEC_PROTECTED)) return true;
	return false;
    }

    /** This is overridden in order to insert spec_public and 
	spec_protected as appropriate in the list of modifiers for
	fields.
    */
    protected void printModifier(ProgramElementDoc member) {
	if (member instanceof MjClassDoc) {
	    long m = ((MjClassDoc)member).longModifiers(); 
	    print(org.jmlspecs.jmldoc.Utils.modifierString(this,m));
	}
	super.printModifier(member);
    }

}  
    
    
