/*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: SpecWriter.java,v 1.16 2006/06/19 01:29:59 cruby Exp $
 */

// Note: These operators have to use special HTML syntax:
// Java: <<= >>= >>>= &= && > < >= <= & << >> 
// JML:  <- -> <==> <=!=> ==> <== <:

package org.jmlspecs.jmldoc.jmldoc_142;

import org.jmlspecs.jmldoc.*;
import org.multijava.mjc.*;
import org.jmlspecs.checker.*;
import java.util.ArrayList;
import java.util.Iterator;
import org.multijava.util.compiler.JavaStyleComment;
import org.multijava.javadoc.JavadocComment;
import org.jmlspecs.checker.Constants;

/** This class is a Visitor class that generates appropriate portions of
 *  the javadoc documentation by walking the parse tree.
 */
public class SpecWriter extends JmlVisitorNI implements Constants {

    /* These methods are the beginning of experimentation about how to do some better pretty-printing
	of specifications in html pages.  I discovered a couple of things:  token references tend to mark the
	end of a token, and may even refer to the next line (e.g. for a simple name at the end of a line);
	infix operators don't record the position of the operator itself, but typically the position of the
	left operand.
    */
	private int lastLineNumber = -1;
	public void checkLine(JPhylum self) {
	/*
	    int ln = self.getTokenReference().line();
	    if (ln != lastLineNumber) {
		lastLineNumber = ln;
		System.out.println("FOUND NEW LINE " + ln + " " + self.getTokenReference().column() + " AT " + self.toString());
		sb.append("<BR>");
		sb.append(eol);
		sb.append("&nbsp;&nbsp;&nbsp;&nbsp;");
	    } else System.out.println("SAME LINE " + ln + " " + self.getTokenReference().column() + " FOR " + self.toString());
	*/
	}
	public void setLine(JPhylum self) {
	    //lastLineNumber = self.getTokenReference().line();
	    //System.out.println("SET LINE AT " + lastLineNumber);
	}
	
	/** The command-line options, set here by JmlHTML. */
        /*@ spec_public @*/ static JmldocOptions options = null;
	
	/** The string that terminates lines on this platform. */
	private static final String eol = System.getProperty("line.separator");

	/** This field is simply used to communicate the need to print 'also' from one method to another. */
	private boolean printAlso = false;
	
	/** This buffer accumulates text that later is written out as a file
		or turned into a String.
	 */
	//@ invariant sb != null; // initialized in constructors
	protected StringBuffer sb;

	/** Returns the accumulated text. */
	//@ also ensures \result != null;
	public String toString() {
		return sb.toString();
	}

	/** Constructs a SpecWriter with the given StringBuffer.
	*/
	public SpecWriter(StringBuffer s) {
		sb = s;
	}
	
	/** Constructs a SpecWriter with an empty text buffer. */
	protected SpecWriter() {
		this(new StringBuffer());
	}
	
	/** Constructs a SpecWriter with an empty text buffer and 
		then walks the AST with the SpecWriter, accumulating 
		text as it goes.
	*/
	public SpecWriter(JPhylum p) {
		this();
		p.accept(this);
	}
	
	/** Constructs a SpecWriter with the given StringBuffer and 
		then walks the AST with the SpecWriter, appending 
		text as it goes.
	*/
	public SpecWriter(StringBuffer s, JPhylum p) {
		this(s);
		p.accept(this);
	}
	
	/** Appends the given string to the SpecWriter's buffer. */
	public void append(String s) {
		sb.append(s);
	}
	
    // ---------------------------------------------------------------------
    // COMPILATION UNIT
    // ----------------------------------------------------------------------

    /**
     * visits a compilation unit
     */
    public void visitCompilationUnit( JCompilationUnitType self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    // ----------------------------------------------------------------------
    // TYPE DECLARATION
    // ----------------------------------------------------------------------

    /**
     * visits a class declaration
     */
    public void visitClassDeclaration( JClassDeclarationType self ) 
			{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an interface declaration
     */
    public void visitInterfaceDeclaration( JInterfaceDeclarationType self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a generic function anchor
     */
    public void visitGenericFunctionDecl( MJGenericFunctionDecl self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    // ----------------------------------------------------------------------
    // METHODS AND FIELDS
    // ----------------------------------------------------------------------

    /**
     * visits a field declaration
     */
    public void visitFieldDeclaration( JFieldDeclarationType self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a method declaration
     */
    public void visitMethodDeclaration( JMethodDeclarationType self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an initializer declaration
     */
    public void visitInitializerDeclaration( JInitializerDeclaration self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an external method declaration
     */
    public void visitTopLevelMethodDeclaration( MJTopLevelMethodDeclaration self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a constructor declaration
     */
    public void visitConstructorDeclaration( JConstructorDeclarationType self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    // ----------------------------------------------------------------------
    // STATEMENTS
    // ----------------------------------------------------------------------

    /**
     * visits a while statement
     */
    public void visitWhileStatement( JWhileStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a variable declaration statement
     */
    public void visitVariableDeclarationStatement(JVariableDeclarationStatement self) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a variable declaration statement
     */
    public void visitVariableDefinition( JVariableDefinition self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a variable declaration statement
     */
    public void visitJmlVariableDefinition( JmlVariableDefinition self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a try-catch statement
     */
    public void visitTryCatchStatement( JTryCatchStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a try-finally statement
     */
    public void visitTryFinallyStatement( JTryFinallyStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a throw statement
     */
    public void visitThrowStatement( JThrowStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a synchronized statement
     */
    public void visitSynchronizedStatement( JSynchronizedStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a switch statement
     */
    public void visitSwitchStatement( JSwitchStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a return statement
     */
    public void visitReturnStatement( JReturnStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a labeled statement
     */
    public void visitLabeledStatement( JLabeledStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a if statement
     */
    public void visitIfStatement( JIfStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a for statement
     */
    public void visitForStatement( JForStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a compound statement
     */
    public void visitCompoundStatement( JCompoundStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an expression statement
     */
    public void visitExpressionStatement( JExpressionStatement self ) 
		{ sb.append(" ? "); System.out.println("visitExpressionStatement: SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an expression list statement
     */
    public void visitExpressionListStatement( JExpressionListStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a empty statement
     */
    public void visitEmptyStatement( JEmptyStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a do statement
     */
    public void visitDoStatement( JDoStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a continue statement
     */
    public void visitContinueStatement( JContinueStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a break statement
     */
    public void visitBreakStatement( JBreakStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an expression statement
     */
    public void visitBlockStatement( JBlock self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a constructor block
     */
    public void visitConstructorBlock( JConstructorBlock self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }
    /**
     * visits a class block (initializer)
     */
    public void visitClassBlock( JClassBlock self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a type declaration statement
     */
    public void visitTypeDeclarationStatement( JTypeDeclarationStatement self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    // ----------------------------------------------------------------------
    // EXPRESSION
    // ----------------------------------------------------------------------

    /**
     * visits an unary expression
     */
    public void visitUnaryExpression( JUnaryExpression self ) {
		checkLine(self);
		int op = self.oper();
		if (op == Constants.OPE_MINUS) sb.append("-");
		else if (op == Constants.OPE_BNOT) sb.append("~");
		else if (op == Constants.OPE_LNOT) sb.append("!");
		else if (op == Constants.OPE_PLUS) sb.append("+");
		else { 
			System.out.println("ERROR: Unimplemented op in SpecWriter.visitUnaryExpression - " + op);
			sb.append(" OP:" + op + " " );
		}
		self.expr().accept(this);
	}

    /**
     * visits a type name expression
     */
    public void visitTypeNameExpression( JTypeNameExpression self ) 
		{ checkLine(self); sb.append(self.getType().toString()); }

    /**
     * visits a this expression
     */
    public void visitThisExpression( JThisExpression self ) { 
	checkLine(self);
	sb.append("this");  
    }

    /**
     * visits a super expression
     */
    public void visitSuperExpression( JSuperExpression self ) 
	{   checkLine(self);
	    if (self.prefix() != null) {
		self.prefix().accept(this);
		sb.append(".");
		}
	    sb.append("super"); 
	}

    /**
     * visits a shift expression
     */
    public void visitShiftExpression( JShiftExpression self ) {
		self.left().accept(this);
		checkLine(self);
		int op = self.oper();
		if (op == Constants.OPE_SR) sb.append(" &gt;&gt; " );
		else if (op == Constants.OPE_SL) sb.append(" &lt;&lt; " );
		else if (op == Constants.OPE_BSR) sb.append(" &gt;&gt;&gt; " );	
		else { 
			System.out.println("ERROR: Unimplemented op in SpecWriter.visitShiftExpression - " + op);
			sb.append(" OP:" + op + " " );
		}
		self.right().accept(this);
	}

    /**
     * visits a relational expression
     */
    public void visitRelationalExpression( JRelationalExpression self ) {
		self.left().accept(this);
		checkLine(self);
		int op = self.oper();
		sb.append(htmlop(op,self));
		self.right().accept(this);
	}

    /**
     * visits a prefix expression
     */
    public void visitPrefixExpression( JPrefixExpression self ) 
		// Should not have these in specification statements
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a postfix expression
     */
    public void visitPostfixExpression( JPostfixExpression self ) 
		// Should not have these in specification statements
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a parenthesed expression
     */
    public void visitParenthesedExpression( JParenthesedExpression self ) 
		{ 
		checkLine(self);
			sb.append("(");
			self.expr().accept(this);
			sb.append(")");
		}

    /**
     * visits an object allocator expression
     */
    public void visitNewObjectExpression( JNewObjectExpression self ) {
		// FIXME ??? Check what explicitThis is for
		checkLine(self);
		if (self.thisExpr() != null) {  // ??? This is how things are parsed, but I'm not sure about what this
			self.thisExpr().accept(this);  // ??? syntax means ( a.new Test() )
			sb.append(".");
		}
		sb.append("new " + self.getType());
		sb.append("(");
		JExpression[] ex = self.params();
		if (ex != null) for (int i=0; i<ex.length; ++i) {
			if (i != 0) sb.append(", ");
			ex[i].accept(this);
		}
		sb.append(")");
	}

    /**
     * visits an object allocator expression for an anonymous class
     */
    public void visitNewAnonymousClassExpression( JNewAnonymousClassExpression self ) // ???
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an array allocator expression
     */
    public void visitNewArrayExpression( JNewArrayExpression self ) {
		checkLine(self);
		sb.append(" <B>new</B> ");
		sb.append(self.getType().toString());
		sb.append(" ");
		JExpression[] dims = self.dims().dims();
		for (int i=0; i<dims.length; ++i) {
			if (dims[i] == null) {
				sb.append("[]");
			} else {
				sb.append("[");
				dims[i].accept(this);
				sb.append("]");
			}
		}
		
		JArrayInitializer init = self.dims().init();
		if (init != null) init.accept(this);
	}


    /**
     * visits a name expression
     */
    public void visitNameExpression( JNameExpression self ) 
		{ 
		checkLine(self);
			JExpression prefix = self.getPrefix();
			if (prefix != null) {
				prefix.accept(this);
				sb.append(".");
			}
			sb.append(self.getName());
		}

    /**
     * visits an add expression
     */
    public void visitAddExpression( JAddExpression self ) 
		{ 
			self.left().accept(this);
		checkLine(self);
			sb.append("+");
			self.right().accept(this);
		}

    /**
     * visits a boolean AND expression
     */
    public void visitConditionalAndExpression( JConditionalAndExpression self ) 
		{ 
			self.left().accept(this);
		checkLine(self);
			sb.append("&amp;&amp;");
			self.right().accept(this);
		}

    /**
     * visits a boolean OR expression
     */
    public void visitConditionalOrExpression( JConditionalOrExpression self ) 
		{ 
			self.left().accept(this);
		    checkLine(self);
			sb.append("||");
			self.right().accept(this);
		}

    /**
     * visits a divide expression
     */
    public void visitDivideExpression( JDivideExpression self ) 
		{ 
			self.left().accept(this);
		checkLine(self);
			sb.append("/");
			self.right().accept(this);
		}

    /**
     * visits a minus expression
     */
    public void visitMinusExpression( JMinusExpression self ) 
		{ 
			self.left().accept(this);
		checkLine(self);
			sb.append("-");
			self.right().accept(this);
		}

    /**
     * visits a modulo division expression
     */
    public void visitModuloExpression( JModuloExpression self ) 
		{ 
			self.left().accept(this);
		checkLine(self);
			sb.append("%");
			self.right().accept(this);
		}

    /**
     * visits a multiplication expression
     */
    public void visitMultExpression( JMultExpression self ) { 
	self.left().accept(this);
		checkLine(self);
	sb.append("*");
	self.right().accept(this);
    }

    /**
     * visits a method call expression
     */
    public void visitMethodCallExpression( JMethodCallExpression self ) {
		if (self.prefix() != null) {
			self.prefix().accept(this);
			sb.append(".");
		}
		checkLine(self);
		sb.append(self.ident());
		sb.append("(");
		JExpression[] a = self.args();
		for (int i = 0; i<a.length; ++i) {
			if (i != 0) sb.append(",");
			a[i].accept(this);
		}
		sb.append(")");
	}

    /**
     * visits a local variable expression
     */
    public void visitLocalVariableExpression( JLocalVariableExpression self ) {
		// This is a usage of a variable in an expression, not a declaration
		// so don't print the type.
		checkLine(self);
		sb.append(self.ident());
	}

    /**
     * visits an instanceof expression
     */
    public void visitInstanceofExpression( JInstanceofExpression self ) 
		{ 	self.expr().accept(this);
			sb.append(" instanceof " + self.dest());
		}

    /**
     * visits an equality expression
     */
    public void visitEqualityExpression( JEqualityExpression self ) { 	
	self.left().accept(this);
	checkLine(self); 
	sb.append(" ");
	int op = self.oper();
	if (op == Constants.OPE_EQ) sb.append("==");
	else if (op == Constants.OPE_NE) sb.append("!=");
	else {
		sb.append("OP:"+op+ " ");
		System.out.println("ERROR: Unimplemented op in SpecWriter.visitEqualityExpression - " + op);
	}
	sb.append(" ");
	self.right().accept(this);
    }

    /**
     * visits a conditional expression
     */
    public void visitConditionalExpression( JConditionalExpression self )  {
		checkLine(self);
		self.cond().accept(this);
		sb.append(" ? ");
		self.left().accept(this);
		sb.append(" : ");
		self.right().accept(this);
	}

    /**
     * visits a compound expression
     */
    public void visitCompoundAssignmentExpression(JCompoundAssignmentExpression self) 
		// No assignments in spec expressions
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }
		
    /**
     * visits a field expression
     */
    public void visitFieldExpression( JClassFieldExpression self ) {
		checkLine(self);
	self.prefix().accept(this);
	sb.append(".");
	sb.append(self.ident());
    }

    /**
     * visits a class expression
     */
    public void visitClassExpression( JClassExpression self ) {
		checkLine(self);
		sb.append(self.prefixType() + ".class");
	}

    /**
     * visits a cast expression
     */
    public void visitCastExpression( JCastExpression self ) {
		checkLine(self);
		sb.append("(");
		sb.append(self.getType() + ")");
		self.expr().accept(this);
	}

    /**
     * visits a cast expression
     */
    public void visitUnaryPromoteExpression( JUnaryPromote self ) {
		// This is used to cast an expression to a reference type
		checkLine(self);
		if (options.showPromotion()) {
			sb.append("(");
			sb.append(self.getType() + ")");
		}
		self.expr().accept(this);
	}


    /**
     * visits a bit operator expression
     */
    public void visitBitwiseExpression( JBitwiseExpression self ) {
		self.left().accept(this);
		checkLine(self);
		int op = self.oper();
		if (op == Constants.OPE_BAND) sb.append("&amp;");
		else if (op == Constants.OPE_BOR) sb.append("|");
		else if (op == Constants.OPE_BXOR) sb.append("^");
		else {
			System.out.println("ERROR: Invalid operator in SpecWriter.visitBitwiseExpression - " + op);
			sb.append(" OP:"+op+ " ");
		}
		self.right().accept(this);
	}

    /**
     * visits an assignment expression
     */
    public void visitAssignmentExpression( JAssignmentExpression self ) 
		// No assignments in spec expressions
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass() + " " + self.getTokenReference()); }

    /**
     * visits an array length expression
     */
    public void visitArrayLengthExpression(JArrayLengthExpression self ) {
		self.prefix().accept(this);
		sb.append(".length");
	}

    /**
     * visits an array access expression
     */
    public void visitArrayAccessExpression( JArrayAccessExpression self ) {
		self.prefix().accept(this);
		sb.append("[");
		self.accessor().accept(this);
		sb.append("]");
	}

    /**
     * visits a warn expression
     */
    public void visitWarnExpression( MJWarnExpression self ) {
		checkLine(self);
		sb.append(self.operatorName() + "(");
		self.expr().accept(this);
		sb.append(")");
	}

    /**
     * visits a math mode expression
     */
    public void visitMathModeExpression( MJMathModeExpression self ) {
		checkLine(self);
		sb.append(self.operatorName() + "(");
		self.expr().accept(this);
		sb.append(")");
	}

    // ----------------------------------------------------------------------
    // OTHERS
    // ----------------------------------------------------------------------

    /**
     * visits a switch label
     */
    public void visitSwitchLabel( JSwitchLabel self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a switch group
     */
    public void visitSwitchGroup( JSwitchGroup self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a catch clause
     */
    public void visitCatchClause( JCatchClause self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a boolean literal
     */
    public void visitBooleanLiteral( JBooleanLiteral self ) 
		{ checkLine(self); sb.append(self.booleanValue()); }

    /**
     * visits a character literal
     */
    public void visitCharLiteral( JCharLiteral self ) 
		{ checkLine(self); sb.append("'" + self.getValue().toString() + "'"); 
		}

    /**
     * prints an ordinal literal
     */
    public void visitOrdinalLiteral( JOrdinalLiteral self ) {
		checkLine(self);
	sb.append(self.getValue());
    }

    /**
     * prints a real literal
     */
    public void visitRealLiteral( JRealLiteral self ) 
		{ checkLine(self); sb.append(self.numberValue()); }

    /**
     * visits a string literal
     */
    public void visitStringLiteral( JStringLiteral self ) 
		{ checkLine(self); sb.append("\"" + self.stringValue() + "\""); }

    /**
     * visits a null literal
     */
    public void visitNullLiteral( JNullLiteral self ) 
		{ checkLine(self); sb.append("null"); }

    /**
     * visits a package name declaration
     */
    public void visitPackageName( JPackageName self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a package import declaration
     */
    public void visitPackageImport( JPackageImportType self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a class import declaration
     */
    public void visitClassOrGFImport( JClassOrGFImportType self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits a formal parameter
     */
    public void visitFormalParameters( JFormalParameter self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an explicit constructor invocation
     */
    public void visitExplicitConstructorInvocation(JExplicitConstructorInvocation self) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    /**
     * visits an array initializer expression
     */
    public void visitArrayInitializer( JArrayInitializer self ) {
		checkLine(self); 
		JExpression[] e = self.elems();
		sb.append("{ ");
		if (e.length > 0) e[0].accept(this);
		for (int i=1; i<e.length; ++i ) {
			sb.append(", ");
			e[i].accept(this);
		}
		sb.append("}");
	}

    /**
     * visits an array dimension and initialization expression
     */
    public void visitArrayDimsAndInit( JArrayDimsAndInits self ) 
		{ sb.append(" ? "); System.out.println("SpecWriter not implemented for " + self.getClass()); }

    // ----------------------------------------------------------------------
    // JML stuff
	// ----------------------------------------------------------------------

	//Note that in the original grammar, a spec beginning with also actually parses 
	// as a JmlSpecification - the also gets lost.  Jml.g was changed to fix this.

	public void visitJmlExtendingSpecification( JmlExtendingSpecification self) {
		//System.out.println("JmlExtendingSpecification " + self.getTokenReference().line());

		visitJmlSpecificationHelper(self.specCases(),true);

		// FIXME - what about subcclassing Contract ???

		if (self.redundantSpec() != null) self.redundantSpec().accept(this);
	}
	    
	public void visitJmlSpecification( JmlSpecification self) {
		//System.out.println("JmlSpecification " + self.getTokenReference().line());

		visitJmlSpecificationHelper(self.specCases(),false);

		if (self.redundantSpec() != null) self.redundantSpec().accept(this);
	}
	    
	public void visitJmlSpecificationHelper( JmlSpecCase[] scases, boolean initialAlso) {
		if (scases == null) return;
		printAlso = initialAlso;
		for (int i=0; i<scases.length; ++i) {
			JmlSpecCase scase = scases[i];
			scase.accept(this);
			printAlso = true; 
		}
		printAlso = false;
	}

    /** Prints a JML code contract. */
    public void visitJmlCodeContract( JmlCodeContract self )
    {
	sb.append("<DT>&nbsp;&nbsp;&nbsp;&nbsp;<B>code_contract</B><DD>");
	sb.append(eol);
	JmlSpecBodyClause[] subclassingContract = self.accessibleClauses();
	for (int i = 0; i < subclassingContract.length; i++) {
	    subclassingContract[i].accept(this);
	}
	subclassingContract = self.callableClauses();
	for (int i = 0; i < subclassingContract.length; i++) {
	    subclassingContract[i].accept(this);
	}
	subclassingContract = self.measuredClauses();
	for (int i = 0; i < subclassingContract.length; i++) {
	    subclassingContract[i].accept(this);
	}
    }

	public void visitJmlRedundantSpec( JmlRedundantSpec self ) {
		if (self.implications() != null) {
			//sb.append("</DD>" + eol);
			sb.append("<DT>&nbsp;&nbsp;&nbsp;&nbsp;<B>implies_that</B><BR>"); 
			visitJmlSpecificationHelper(self.implications(),false);
		}

		JmlExample[] examples = self.examples();
		if (examples == null) return;
		ArrayList a = new ArrayList(examples.length);
		for (int i = 0; i<examples.length; ++i) {
		    if (JmlHTML.checkModifiers(examples[i].privacy())) a.add(examples[i]);
		}
		if (a.size() != 0) {
			sb.append("<DT>&nbsp;&nbsp;&nbsp;&nbsp;<B>for_example</B><DD>" + eol); 
			Iterator j = a.iterator();
			if (j.hasNext()) {
			    JmlExample ex = (JmlExample)j.next();
			    ex.accept(this);
			}
			while (j.hasNext()) {
			    JmlExample ex = (JmlExample)j.next();
			    sb.append("<DT>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<B>also</B><DD>" + eol);
			    ex.accept(this);
			}
			sb.append(eol);
		}	
			
	}
	    
    public void visitJmlNode( JmlNode self )
			{ sb.append(" ? "); System.out.println("SpecWriter.visitJmlNode not implemented for " + self.getClass()); }

    public void visitJmlSpecExpression( JmlSpecExpression self ) { 
		self.expression().accept(this); 
	}

    public void visitJmlExpression( JmlExpression self ) { 
	    sb.append(self.getClass()); 
	}

	public void visitJmlGenericSpecCase( JmlGenericSpecCase self ) {
		//System.out.println("JmlGenericSpecCase " + self);
		visitJmlGeneralSpecCaseHelper(self,true);
	}

	public void visitJmlNormalSpecCase( JmlGenericSpecCase self ) {
		//System.out.println("JmlGenericSpecCase " + self);
		visitJmlGeneralSpecCaseHelper(self,false);
	}

	public void visitJmlExceptionalSpecCase( JmlGenericSpecCase self ) {
		//System.out.println("JmlGenericSpecCase " + self);
		visitJmlGeneralSpecCaseHelper(self,false);
	}

	public void visitJmlRequiresClause( JmlRequiresClause self ) {
		//System.out.println("JmlRequiresClause " + self);
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>requires_redundantly</B> ");
		else                      sb.append("<B>requires</B> ");
		visitJmlPredicateClause(self);
		sb.append(";<BR>" + eol);
	}

	public void visitJmlAssignableClause( JmlAssignableClause self ) {
		//System.out.println("JmlAssignableClause " + self);
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>assignable_redundantly</B> ");
		else                      sb.append("<B>assignable</B> ");
		JmlStoreRef[] srs = self.storeRefs();
		for (int i=0; i<srs.length; ++i) {
			if (i != 0) sb.append(", ");
			srs[i].accept(this);
		}
		sb.append(";<BR>" + eol);
	}

	public void visitJmlWorkingSpaceClause( JmlWorkingSpaceClause self ) {
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>working_space_redundantly</B> ");
		else                      sb.append("<B>working_space</B> ");
		if (self.isNotSpecified()) {
			sb.append(" \\not_specified");
		} else {
			self.specExp().accept(this);
			if (self.predOrNot() != null) {
				sb.append(" <B>if</B> ");
				self.predOrNot().accept(this);
			}
		}
		sb.append(";<BR>" + eol);
	}

	public void visitJmlDurationClause( JmlDurationClause self ) {
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>duration_redundantly</B> ");
		else                      sb.append("<B>duration</B> ");
		if (self.isNotSpecified()) {
			sb.append(" \\not_specified");
		} else {
			self.specExp().accept(this);
			if (self.predOrNot() != null) {
				sb.append(" <B>if</B> ");
				self.predOrNot().accept(this);
			}
		}
		sb.append(";<BR>" + eol);
	}

	public void visitJmlMeasuredClause( JmlMeasuredClause self ) {
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>measured_by_redundantly</B> ");
		else                      sb.append("<B>measured_by</B> ");
		if (self.isNotSpecified()) {
			sb.append(" \\not_specified");
		} else {
			self.specExp().accept(this);
			if (self.predOrNot() != null) {
				sb.append(" <B>if</B> ");
				self.predOrNot().accept(this);
			}
		}
		sb.append(";<BR>" + eol);
	}
	
	public void visitJmlWhenClause( JmlWhenClause self ) {
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>when_redundantly</B> ");
		else                      sb.append("<B>when</B> ");
		visitJmlPredicateClause(self);
		sb.append(";<BR>" + eol);
	}
		
	public void visitJmlEnsuresClause( JmlEnsuresClause self ) {
		//System.out.println("JmlEnsuresClause " + self);
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>ensures_redundantly</B> ");
		else                      sb.append("<B>ensures</B> ");
		visitJmlPredicateClause(self);
		sb.append(";<BR>" + eol);
	}

	public void visitJmlSignalsOnlyClause( JmlSignalsOnlyClause self ) {
		//System.out.println("JmlSignalsOnlyClause " + self);
		setLine(self);
                if (!self.isRedundantly()) {
                    sb.append("<B>signals_only </B>");
                } else {
                    sb.append("<B>signals_only_redundantly </B>");
                }
                if (self.isNothing()) {
                    sb.append("\\nothing");
                } else {
                    CClassType[] exceptions = self.exceptions();
                    for (int i=0; i<exceptions.length; i++) {
                        if (i != 0) sb.append(", ");
                        sb.append(exceptions[i]);
                    }
                }
		sb.append(";<BR>" + eol);
	}

	public void visitJmlSignalsClause( JmlSignalsClause self ) {
		//System.out.println("JmlSignalsClause " + self);
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>signals_redundantly </B>(");
		else                      sb.append("<B>signals </B>(");
		if (self.ident() == null) sb.append(self.type() + ") ");
		else                      sb.append(self.type() + " " + self.ident() + ") ");
		if (self.hasPredicate() || self.isNotSpecified()) visitJmlPredicateClause(self);
		else sb.append("true"); // ??? Should probably build a spec expression that is true here.
		sb.append(";<BR>" + eol);
	}

	public void visitJmlDivergesClause( JmlDivergesClause self ) {
		//System.out.println("JmlDivergesClause " + self);
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>diverges_redundantly</B> ");
		else                      sb.append("<B>diverges</B> ");
		visitJmlPredicateClause(self);
		sb.append(";<BR>" + eol);
	}
	
	public void visitJmlAccessibleClause( JmlAccessibleClause self ) {
		//System.out.println("JmlAccessibleClause " + self);
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>accessible_redundantly</B> ");
		else                      sb.append("<B>accessible</B> ");
		JmlStoreRef[] srs = self.storeRefs();
		for (int i=0; i<srs.length; ++i) {
			if (i != 0) sb.append(", ");
			srs[i].accept(this);
		}
		sb.append(";<BR>" + eol);
	}

	public void visitJmlCallableClause( JmlCallableClause self ) {
		setLine(self);
		if (self.isRedundantly()) 
		    sb.append("<B>callable_redundantly</B> ");
		else
		    sb.append("<B>callable</B> ");
		self.methodNames().accept(this);
		sb.append(";<BR>"+eol);
	}

	public void visitJmlCapturesClause( JmlCapturesClause self ) {
		setLine(self);
		if (self.isRedundantly()) sb.append("<B>captures_redundantly</B> ");
		else                      sb.append("<B>captures</B> ");
		JmlStoreRef[] srs = self.storeRefs();
		for (int i=0; i<srs.length; ++i) {
			if (i != 0) sb.append(", ");
			srs[i].accept(this);
		}
		sb.append(";<BR>" + eol);
	}

	public void visitJmlGeneralSpecCase( JmlGeneralSpecCase self ) {
		//System.out.println("JmlGeneralSpecCase " + self);
		visitJmlGeneralSpecCaseHelper(self,false);
	}

	public void visitJmlGeneralSpecCaseHelper( JmlGeneralSpecCase self, boolean indent ) {
		//System.out.println("JmlGeneralSpecCaseHelper " + self + " " + indent);
		boolean indented = printAlso || indent;
		if (printAlso) { sb.append("<DT><B>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;also</B><DD>" + eol); printAlso = false; }
		else if (indent) sb.append("<DD>" + eol); 
		JmlSpecVarDecl[] decls = self.specVarDecls();
 		if (decls != null) for (int i=0; i<decls.length; ++i) {
			decls[i].accept(this);
		}
		JmlRequiresClause[] req = self.specHeader();
		if (req != null) for (int i=0; i<req.length; ++i) {
			req[i].accept(this);
		}
		JmlSpecBody body = self.specBody();
		if (body != null) body.accept(this);
		if (indented) sb.append("</DD>" + eol);
	}

	public void visitJmlPredicate( JmlPredicate self) {
		self.specExpression().accept(this);
	}
	
	public void visitJmlPredicateKeyword( JmlPredicateKeyword self) {
	        if ( self.isSameKeyword() ) {
	                sb.append("\\same ");
		} else {
			sb.append("\\not_specified ");
	        }
	}

	public void visitJmlPredicateClause( JmlPredicateClause self ) {
		checkLine(self);
		if (self.isNotSpecified()) {
			sb.append("\\not_specified ");
		} else {
			self.predOrNot().accept(this);
		}
	}
	
	public void visitJmlSpecBody( JmlSpecBody self ) {
		//System.out.println("JmlSpecBody " + self + " " + self.specClauses() + " " + self.specCases());
		
		// This is a simple-spec-body, which is a sequence of clauses
		JmlSpecBodyClause[] clauses = self.specClauses();
		if (clauses != null) for (int i=0; i<clauses.length; ++i) {
			clauses[i].accept(this);
		}
		
		JmlGeneralSpecCase[] cases = self.specCases();
		if (cases != null && cases.length != 0) {
			sb.append("<DL><BT><B>{|</B><DD>" + eol);
			visitJmlGeneralSpecCaseHelper(cases[0],false);
			for (int i=1; i<cases.length; ++i) {
				sb.append("<DT><B>also</B><DD>" + eol);
				visitJmlGeneralSpecCaseHelper(cases[i],false);
			}
			sb.append("<DT><B>|}</B></DL>" + eol);
		}
	}
	
	public void visitJmlSpecBodyClause( JmlSpecBodyClause self ) {
		// All subclasses of this should have visitors implemented, so if we get here it is an error.
		System.out.println("NOT IMPLEMENTED in SpecWriter.visitJmlSpecBodyClause " + self);
	}

	public void visitJmlGenericSpecBody( JmlGenericSpecBody self) {
		//System.out.println("JmlGenericSpecBody " + self + " " + self.simpleSpecBodies() + " " + self.specCases());
		visitJmlSpecBody(self);
	}

	public void visitJmlNormalSpecBody( JmlNormalSpecBody self) {
		visitJmlSpecBody(self);
	}

	public void visitJmlExceptionalSpecBody( JmlExceptionalSpecBody self) {
		visitJmlSpecBody(self);
	}

	public void visitJmlResultExpression( JmlResultExpression self) {
		sb.append("\\result ");
	}

	public String htmlop(int op, JRelationalExpression s) {
		if (op == OPE_EQUIV) {
			return ("&lt;==&gt;");
		} else if (op == OPE_NOT_EQUIV) {
			return ("&lt;=!=&gt;");
		} else if (op == OPE_IMPLIES) {
			return ("==&gt;");
		} else if (op == OPE_BACKWARD_IMPLIES) {
			return ("&lt;==");
		} else if (op == OPE_SUBTYPE) {
			return ("&lt;:");
		} else if (op == OPE_GT) {
			return "&gt;";
		} else if (op == OPE_GE) {
			return "&gt;=";
		} else if (op == OPE_LT) {
			return "&lt;";
		} else if (op == OPE_LE) {
			return "&lt;=";
		} else {
			return s.opString();
		}
	}




	public void visitJmlRelationalExpression( JmlRelationalExpression self) {
		checkLine(self);
		self.left().accept(this);
		sb.append(" ");
		int op = self.oper();
		sb.append(htmlop(op,self));
		sb.append(" ");
		self.right().accept(this);
	}

	public void visitJmlFreshExpression( JmlFreshExpression self) {
		sb.append("\\fresh(");
		JmlSpecExpression[] jsl = self.specExpressionList();
		for (int i=0; i<jsl.length; ++i) {
			if (i != 0) sb.append(", ");
			jsl[i].accept(this);
		}
		//System.out.println("JmlFreshExpression " + self);
		sb.append(")");
	}
	
	public void visitJmlIsInitializedExpression( JmlIsInitializedExpression self) {
		sb.append("\\is_initialized(");
		sb.append(self.referenceType() + ")"); 
	}
	
	public void visitJmlLabelExpression( JmlLabelExpression self) {
		if (self.isPosLabel()) {
			sb.append("(\\lblpos <I>");
		} else {
			sb.append("(\\lblneg <I>");
		}
		sb.append(self.ident());
		sb.append("</I> ");
		self.specExpression().accept(this);
		sb.append(")");
	}
	
	public void visitJmlLockSetExpression( JmlLockSetExpression self) {
		sb.append("\\lockset ");
	}
	
	public void visitJmlReachExpression( JmlReachExpression self) {
		checkLine(self);
		sb.append("\\reach(");
		self.specExpression().accept(this);
		if (self.referenceType() != null) {
			sb.append(", ");
			sb.append(self.referenceType().toString());
			if (self.storeRefExpression() != null) {
				sb.append(", ");
				self.storeRefExpression().accept(this);
			}
		}
		sb.append(")");
	}
	
	public void visitJmlSpecQuantifiedExpression( JmlSpecQuantifiedExpression self) {
		checkLine(self);
		sb.append("( ");
		if (self.isForAll()) sb.append("\\forall ");
		else if (self.isExists()) sb.append("\\exists ");
		else if (self.isMax()) sb.append("\\max ");
		else if (self.isMin()) sb.append("\\min ");
		else if (self.isNumOf()) sb.append("\\num_of ");
		else if (self.isProduct()) sb.append("\\product ");
		else if (self.isSum()) sb.append("\\sum ");
		else {
			System.out.println("ERROR: Unknown operator in SpecWriter.visitJmlSpecQuantifiedExpression " );
			sb.append("?? ");
		}
		JVariableDefinition[] vars = self.quantifiedVarDecls();
		if (vars == null || vars.length == 0) {
			System.out.println("ERROR: Empty quantifiedVarDecls in SpecWriter.visitJmlSpecQuantifiedExpression");
			return;
		}
		sb.append(vars[0].getType().toString());
		sb.append(" ");
		sb.append(vars[0].ident());
		for (int i=1; i<vars.length; ++i) {
			sb.append(", ");
			sb.append(vars[i].ident());
		}
		sb.append("; ");
		if (self.hasPredicate()) {
			self.predicate().accept(this);
		}
		sb.append("; ");
		self.specExpression().accept(this);
		sb.append(")");
	}
	
	public void visitJmlTypeExpression( JmlTypeExpression self) {
		checkLine(self);
		sb.append("\\type(");
		sb.append(self.type() + ")");
	}

	
	public void visitJmlOldExpression( JmlOldExpression self) {
		checkLine(self);
		sb.append("\\old(");
		self.specExpression().accept(this);
		if( self.label() != null ) {
		    sb.append(", ");
		    sb.append(self.label());
		}
		sb.append(")");
	}
	
	public void visitJmlPreExpression( JmlPreExpression self) {
		checkLine(self);
		sb.append("\\pre(");
		self.specExpression().accept(this);
		sb.append(")");
	}

	public void visitJmlMaxExpression( JmlMaxExpression self) {
		checkLine(self);
		sb.append("\\max(");
		self.expression().accept(this);
		sb.append(")");
	}

	public void visitJmlDurationExpression( JmlDurationExpression self) {
		checkLine(self);
		sb.append("\\duration(");
		self.expression().accept(this);
		sb.append(")");
	}

	public void visitJmlSpaceExpression( JmlSpaceExpression self) {
		checkLine(self);
		sb.append("\\space(");
		self.specExpression().accept(this);
		sb.append(")");
	}

	public void visitJmlWorkingSpaceExpression( JmlWorkingSpaceExpression self) {
		checkLine(self);
		sb.append("\\working_space(");
		self.expression().accept(this);
		sb.append(")");
	}


	public void visitJmlNotModifiedExpression( JmlNotModifiedExpression self) {
		//System.out.println("JmlNotModifiedExpression " + self);
		checkLine(self);
		sb.append("\\not_modified(");
		JmlStoreRef[] jsl = self.storeRefList();
		for (int i=0; i<jsl.length; ++i) {
			if (i != 0) sb.append(", ");
			jsl[i].accept(this);
		}
		sb.append(")");
	}

	public void visitJmlNotAssignedExpression( JmlNotAssignedExpression self) {
		//System.out.println("JmlNotAssignedExpression " + self);
		checkLine(self);
		sb.append("\\not_assigned(");
		JmlStoreRef[] jsl = self.storeRefList();
		for (int i=0; i<jsl.length; ++i) {
			if (i != 0) sb.append(", ");
			jsl[i].accept(this);
		}
		sb.append(")");
	}

	public void visitJmlOnlyAssignedExpression( JmlOnlyAssignedExpression self) {
		//System.out.println("JmlOnlyAssignedExpression " + self);
		checkLine(self);
		sb.append("\\only_assigned(");
		JmlStoreRef[] jsl = self.storeRefList();
		for (int i=0; i<jsl.length; ++i) {
			if (i != 0) sb.append(", ");
			jsl[i].accept(this);
		}
		sb.append(")");
	}

	public void visitJmlNonNullElementsExpression( JmlNonNullElementsExpression self) {
		checkLine(self);
		sb.append("\\nonnullelements(");
		self.specExpression().accept(this);
		sb.append(")");
	}

	public void visitJmlTypeOfExpression( JmlTypeOfExpression self) {
		checkLine(self);
		sb.append("\\typeof(");
		self.specExpression().accept(this); 
		sb.append(")");
	}

	public void visitJmlElemTypeExpression( JmlElemTypeExpression self) {
		checkLine(self);
		sb.append("\\elemtype(");
		self.specExpression().accept(this); 
		sb.append(")");
	}

	public void visitJmlInvariantForExpression( JmlInvariantForExpression self) {
		checkLine(self);
		sb.append("\\invariant_for(");
		self.specExpression().accept(this);
		sb.append(")");
	}

	public void visitJmlStoreRef( JmlStoreRef self ) {
		// All subclasses should be implemented
		System.out.println("NOT IMPLEMENTED in SpecWriter.visitJmlStoreRef: " + self);
	}

	public void visitJmlStoreRefExpression( JmlStoreRefExpression self ) {
		JmlName[] names = self.names();
		names[0].accept(this);
		for (int i=1; i<names.length; ++i) {	
			visitJmlNameHelper(names[i],true); // OK if there are no derived classes of JmlName
		}
	}

	public void visitJmlInformalStoreRef( JmlInformalStoreRef self ) {  
		sb.append("(* ");
		sb.append(convertToHtml(self.text()));
		sb.append(" *)");
	}

	public void visitJmlName( JmlName self) {
		visitJmlNameHelper(self,false);
	}
	
	public void visitJmlNameHelper( JmlName self, boolean addDot) {
		checkLine(self);
		if (addDot && (self.isIdent() || self.isThis() || self.isSuper())) sb.append(".");
		if (self.isIdent()) sb.append(self.ident());
		else if (self.isThis()) sb.append("this");
		else if (self.isSuper()) sb.append("super");
		else if (self.isRange()) {
			sb.append("[");
			self.start().accept(this);
			sb.append(" .. ");
			self.end().accept(this);
			sb.append("]");
		} else if (self.isPos()) {
			sb.append("[");
			self.start().accept(this);
			sb.append("]");
		} else if (self.isAll()) {
			sb.append("[*]");
		} else if (self.isFields()) {
			sb.append(".*");
		} else {
			System.out.println("ERROR: JmlName option NOT IMPLEMENTED in SpecWriter.visitJmlName");
		}
	}

	public void visitJmlStoreRefKeyword( JmlStoreRefKeyword self ) {
		if (self.isNothing()) sb.append("\\nothing");
		else if (self.isEverything()) sb.append("\\everything");
		else if (self.isNotSpecified()) sb.append("\\not_specified");
		else { 	
			System.out.println("ERROR: Unknown option in SpecWriter.visitJmlStoreRefKeyword");
		}
	}
	
	static void writePrivacy(StringBuffer sb, long p) {
		if ((p & Constants.ACC_PUBLIC) != 0) sb.append("public ");
		if ((p & Constants.ACC_PROTECTED) != 0) sb.append("protected ");
		if ((p & Constants.ACC_PRIVATE) != 0) sb.append("private ");
		if ((p & Constants.ACC_SPEC_PUBLIC) != 0) sb.append("spec_public ");
		if ((p & Constants.ACC_SPEC_PROTECTED) != 0) sb.append("spec_protected ");
	}
	
	static void writeModifiers(StringBuffer sb, long p) {
		if ((p & Constants.ACC_ABSTRACT) != 0) sb.append("abstract ");
		if ((p & Constants.ACC_PURE) != 0) sb.append("pure ");
		if ((p & Constants.ACC_STATIC) != 0) sb.append("static ");
		if ((p & Constants.ACC_MODEL) != 0) sb.append("model ");
		if ((p & Constants.ACC_GHOST) != 0) sb.append("ghost "); 
		if ((p & Constants.ACC_FINAL) != 0) sb.append("final ");
		if ((p & Constants.ACC_SYNCHRONIZED) != 0) sb.append("synchronized ");
		if ((p & Constants.ACC_NON_NULL) != 0) sb.append("non_null ");
		if ((p & Constants.ACC_NULLABLE) != 0) sb.append("nullable ");
		if ((p & Constants.ACC_INSTANCE) != 0) sb.append("instance ");
		if ((p & Constants.ACC_VOLATILE) != 0) sb.append("volatile ");
		if ((p & Constants.ACC_TRANSIENT) != 0) sb.append("transient ");
		if ((p & Constants.ACC_NATIVE) != 0) sb.append("native ");
		if ((p & Constants.ACC_STRICT) != 0) sb.append("strictfp ");
		if ((p & Constants.ACC_MONITORED) != 0) sb.append("monitored ");
		if ((p & Constants.ACC_UNINITIALIZED) != 0) sb.append("uninitialized ");
		writePrivacy(sb,p);
	}


	public void visitJmlBehaviorSpecHelper( JmlHeavyweightSpec self, String s) {
		//System.out.println("JmlBehaviorSpecHelper " + self);
		if (!JmlHTML.checkModifiers(self.privacy())) { return; } 

		if (printAlso) { 
			sb.append("<DT><B>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;also</B><DD>" + eol); 
			printAlso = false; 
		} else sb.append("<DD>" + eol); 

		sb.append("<B> ");
		writePrivacy(sb,self.privacy());
		if (self.isCodeSpec()) { sb.append("code "); }
		sb.append("<I>" + s + "</I></B><BR>" + eol);
		visitJmlGeneralSpecCaseHelper(self.specCase(),false);
		sb.append("</DD>" + eol);
	}
	
	public void visitJmlBehaviorSpec( JmlBehaviorSpec self) {
		//System.out.println("JmlBehaviorSpec " + self);
		visitJmlBehaviorSpecHelper(self,"behavior");
	}

	public void visitJmlNormalBehaviorSpec( JmlNormalBehaviorSpec self) {
		//System.out.println("JmlNormalBehaviorSpec " + self);
		visitJmlBehaviorSpecHelper(self,"normal_behavior");
	}


	public void visitJmlExceptionalBehaviorSpec( JmlExceptionalBehaviorSpec self) {
		//System.out.println("JmlExceptionalBehaviorSpec " + self);
		visitJmlBehaviorSpecHelper(self,"exceptional_behavior");
	}

	public void visitJmlModelProgram( JmlModelProgram self) {
	    if (!JmlHTML.checkModifiers(self.modifiers())) { return; }

	    if (printAlso) { 
		sb.append("<DT><B>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;also</B><DD>" + eol); 
		printAlso = false; 
	    } else sb.append("<DD>" + eol); 
              
	    sb.append("<B> ");
	    writePrivacy(sb,self.modifiers());
	    if (self.isCodeSpec()) { sb.append("code "); }

	    // !FIXME! Need to implement and visit model program statements 
	    // instead of the ... below.
	    sb.append(" model_program</B> { ... } ");
	    sb.append(eol);
	}

    public void visitJmlAssignmentStatement( JmlAssignmentStatement self ) {
        self.assignmentStatement().accept(this);
    }

    public void visitJmlNondetChoiceStatement( JmlNondetChoiceStatement self ) {
	/*
	JBlock[] alternativeStatements = self.alternativeStatements();
	for( int i = 0; i < alternativeStatements.length; i++ ) {
	    alternativeStatements[i].accept(this);
	}
	*/
    }
    public void visitJmlNondetIfStatement( JmlNondetIfStatement self ) {
	/*
	JmlGuardedStatement[]  guardedStatements = self.guardedStatements();
	for( int i = 0; i < guardedStatements.length; i++ ) {
	    guardedStatements[i].accept(this);
	}
	*/
    }
    public void visitJmlGuardedStatement( JmlGuardedStatement self ) {
	/*
	JStatement[] statements = self.statements();
	for( int i = 0; i < statements.length; i++ ) {
	    statements[i].accept(this);
	}
	*/
    }
 
	public void visitJmlSpecVarDecl( JmlSpecVarDecl self) {
		// All subclasses should be implemented
		System.out.println("NOT IMPLEMENTED in SpecWriter.visitJmlSpecVarDecl " + self);
	}
	
	public void visitJmlForAllVarDecl( JmlForAllVarDecl self) {
		checkLine(self);
		JVariableDefinition[] vars = self.quantifiedVarDecls();
		if (vars == null || vars.length == 0) return;
		for (int i=0; i<vars.length; ++i) {
			JVariableDefinition v = vars[i];
			sb.append("<B>forall</B> ");
			sb.append(v.getType());
			sb.append(" ");
			sb.append(v.ident());
			sb.append(";<BR>" + eol);
		}
	}
	
	public void visitJmlLetVarDecl( JmlLetVarDecl self) {
		checkLine(self);
		JVariableDefinition[] vars = self.specVariableDeclarators();
		if (vars == null || vars.length == 0) return;
		for (int i=0; i<vars.length; ++i) {
			JVariableDefinition v = vars[i];
			sb.append("<B>old</B> ");
			sb.append(v.getType());
			sb.append(" ");
			sb.append(v.ident());
			if (v.hasInitializer()) {
				sb.append(" = ");
				v.getValue().accept(this);
			}
			sb.append(";<BR>" + eol);
		}
	}
	
	public void visitFieldDeclaration( JFieldDeclaration self) {
		setLine(self);
		JVariableDefinition v = self.variable();
		sb.append("<B>");
		writeModifiers(sb,v.modifiers());
		sb.append("</B> ");
		sb.append(v.getType().toString());
		sb.append(" ");
		sb.append(v.ident());
		sb.append(BREOL);
	}
		
	public void visitJmlFieldDeclaration( JmlFieldDeclaration self) {
		setLine(self);
		JVariableDefinition v = self.variable();
		sb.append("<B>");
		writeModifiers(sb,v.modifiers());
		sb.append("</B> ");
		sb.append(v.getType().toString());
		sb.append(" ");
		sb.append(v.ident());
		/* !FIXME!
		if (self.hasAssertions()) {
                    JmlVarAssertion[] vars = self.varAssertions();
                    for (int i = 0; i < vars.length; i++) {
                        vars[i].accept(this);
                    }
                }
		*/
		sb.append(BREOL);
	}

	    // This is actually a clause now, just like invariant
	public void visitJmlInitiallyVarAssertion( JmlInitiallyVarAssertion self ) {
		setLine(self);
		sb.append("<B>");
		SpecWriter.writeModifiers(sb,self.modifiers());
		String s = "initially</B> ";
		if (self.isRedundantly()) s = "initially_redundantly</B> ";
		sb.append(s);
		self.predicate().accept(this);
		sb.append(";<BR>" + eol);
	}
	
	    // This is actually a clause now
	public void visitJmlReadableIfVarAssertion( JmlReadableIfVarAssertion self ) {
		setLine(self);
		sb.append("<B>");
		SpecWriter.writeModifiers(sb,self.modifiers());
		String s = "readable</B> ";
		if (self.isRedundantly()) s = "readable_redundantly</B> ";
		sb.append(s);
		sb.append(" ");
		sb.append(self.fieldIdent());
		sb.append(" <B>if</B> ");
		self.predicate().accept(this);
		sb.append(";<BR>" + eol);
	}
	
	public void visitJmlWritableIfVarAssertion( JmlWritableIfVarAssertion self ) {
		setLine(self);
		sb.append("<B>");
		SpecWriter.writeModifiers(sb,self.modifiers());
		String s = "writable</B> ";
		if (self.isRedundantly()) s = "writable_redundantly</B> ";
		sb.append(s);
		sb.append(" ");
		sb.append(self.fieldIdent());
		sb.append(" <B>if</B> ");
		self.predicate().accept(this);
		sb.append(";<BR>" + eol);
	}
	
	public void visitJmlMonitorsForVarAssertion( JmlMonitorsForVarAssertion self ) {
		setLine(self);
		sb.append("<B>");
		SpecWriter.writeModifiers(sb,self.modifiers());
		sb.append("monitors_for</B> ");
		sb.append(self.fieldIdent());
		sb.append(" <B>=</B> ");
		JmlSpecExpression[] exprs = self.specExpressionList();
		exprs[0].accept(this);
		for (int i=1; i<exprs.length; ++i) {
			sb.append(", ");
			exprs[i].accept(this);
		}
		sb.append(";<BR>" + eol);
	}
	
	public void visitJmlInformalExpression( JmlInformalExpression self) {
		checkLine(self);
		sb.append("(*");
		sb.append(convertToHtml(self.text()));
		sb.append("*)");
	}
	
	public void visitJmlAxiom( JmlAxiom self ) {
		setLine(self);
		sb.append("<B>axiom</B> ");
		self.predicate().accept(this);
		sb.append(";<BR>" + eol);
	}

	/** This method gets called only if a derived class does not implement accept. */
	public void visitJmlDeclaration( JmlDeclaration self) {
		System.err.println("SpecWriter does not implement a derived class of JmlDeclaration: " + self);
	}
	
	public void visitJmlInvariant( JmlInvariant self) {
		setLine(self);
		sb.append("<B>");
		SpecWriter.writeModifiers(sb,self.modifiers());
		String s = "invariant</B> ";
		if (self.isRedundantly()) s = "invariant_redundantly</B> ";
		sb.append(s);
		self.predicate().accept(this);
		sb.append(";<BR>" + eol);
	}
	
	public void visitJmlConstraint( JmlConstraint self) {
		setLine(self);
		sb.append("<B>");
		SpecWriter.writeModifiers(sb,self.modifiers());
		String s = "constraint</B> ";
		if (self.isRedundantly()) s = "constraint_redundantly</B> ";
		sb.append(s);
		self.predicate().accept(this); 
		if (! self.isForEverything()) {
			sb.append(" <B>for</B>");
			self.methodNames().accept(this);
		}
		sb.append(";<BR>" + eol);
	}
	
	public void visitJmlRepresentsDecl( JmlRepresentsDecl self) {
		setLine(self);
		sb.append("<B>");
		SpecWriter.writeModifiers(sb,self.modifiers());
		String s = "represents</B> ";
		if (self.isRedundantly()) s = "represents_redundantly</B> ";
		sb.append(s + " ");
		self.storeRef().accept(this);
		if (self.usesAbstractionFunction()) {
			sb.append(" &lt;- ");
			self.specExpression().accept(this);
		} else {
			sb.append(" <B>\\such_that</B> ");
			self.predicate().accept(this);
		}
		sb.append(";<BR>" + eol);
	}

	public void visitJmlMethodNameList( JmlMethodNameList self ) {
		if (self.isStoreRefKeyword()) {
		    self.storeRefKeyword().accept(this);
		} else {
		    JmlMethodName[] names = self.methodNameList();
		    for (int i=0; i<names.length; ++i) {
			if (i!=0) sb.append(", ");
			names[i].accept(this);
		    }
		}
	}

	public void visitJmlMethodName( JmlMethodName self ) {
		checkLine(self);
		sb.append(self.toString());
	}
	
	public void visitJmlConstructorName( JmlConstructorName self ) {
		checkLine(self);
		sb.append(" <B>new</B> ");
		sb.append(self.toString());
	}

	public void visitJmlSetComprehension( JmlSetComprehension self) {
		checkLine(self);
		sb.append(" <B>new</B> ");
		sb.append(self.getType().toString());
		sb.append(" { ");
		sb.append(self.memberType().toString());
		sb.append(" ");
		sb.append(self.ident());
		sb.append(" | ");
		self.supersetPred().accept(this);
		sb.append(" &amp;&amp; ");
		self.predicate().accept(this);
		sb.append(" }");
	}
	
	
	// Examples

	public void visitJmlExample( JmlExample self) {
		visitJmlExampleHelper(self,"example");
	}
	
	public void visitJmlNormalExample( JmlNormalExample self) {
		visitJmlExampleHelper(self,"normal_example");
	}
	
	public void visitJmlExceptionalExample( JmlExceptionalExample self) {
		visitJmlExampleHelper(self,"exceptional_example");
	}
	
	public void visitJmlExampleHelper( JmlExample self, String title) {
		setLine(self);
		sb.append("<B>");
		writeModifiers(sb,self.privacy());
		sb.append("<I>");
		sb.append(title);
		sb.append("</I></B><BR>" + eol);
		
		JmlSpecVarDecl[] vars = self.specVarDecls();
		if (vars != null && vars.length != 0) {
			for (int i=0; i<vars.length; ++i) {
				vars[i].accept(this);
			}
		}
				
		JmlRequiresClause[] reqs = self.specHeader();
		if (reqs != null && reqs.length != 0) {
			for (int i=0; i<reqs.length; ++i) {
				reqs[i].accept(this);
			}
		}
		
		JmlSpecBodyClause[] specs = self.specClauses();
		if (specs != null && specs.length != 0) {
			for (int i=0; i<specs.length; ++i) {
				specs[i].accept(this);
			}
		}
	}

	static public String jmlModifiers(long mods) {
	    StringBuffer sb = new StringBuffer();
	    if ((mods & ACC_PURE) != 0) sb.append(" pure");
	    if ((mods & ACC_SPEC_PUBLIC) != 0) sb.append(" spec_public");
	    if ((mods & ACC_SPEC_PROTECTED) != 0) sb.append(" spec_protected");
	    if ((mods & ACC_INSTANCE) != 0) sb.append(" instance");
	    if ((mods & ACC_MONITORED) != 0) sb.append(" monitored");
	    if ((mods & ACC_UNINITIALIZED) != 0) sb.append(" uninitialized");
	    if ((mods & ACC_NON_NULL) != 0) sb.append(" non_null");
	    if ((mods & ACC_NULLABLE) != 0) sb.append(" nullable");
	    if ((mods & ACC_HELPER) != 0) sb.append(" helper");
	    String s = sb.toString();
	    return s;
	}

/*	
	void visitJmlAbruptSpecBody( JmlAbruptSpecBody self ) {}
	void visitJmlAbruptSpecCase( JmlAbruptSpecCase self ) {}
	void visitJmlAssertStatement( JmlAssertStatement self ) {}
	void visitJmlAssumeStatement( JmlAssumeStatement self ) {}
	void visitJmlBreaksClause( JmlBreaksClause self ) {}
	void visitJmlClassBlock( JmlClassBlock self) {}
	void visitJmlClassDeclaration( JmlClassDeclaration self) {}
	void visitJmlClassOrGFImport( JmlClassOrGFImport self) {}
	void visitJmlCompilationUnit( JmlCompilationUnit self) {}
	void visitJmlConstructorDeclaration( JmlConstructorDeclaration self) {}
	void visitJmlConstructorName( JmlConstructorName self) {}
	void visitJmlContinuesClause( JmlContinuesClause self) {}
    void visitJmlExceptionalSpecBody( JmlExceptionalSpecBody self ) {}
    void visitJmlExceptionalSpecCase( JmlExceptionalSpecCase self ) {}

    void visitJmlNormalSpecBody( JmlNormalSpecBody self ) {}
    void visitJmlNormalSpecCase( JmlNormalSpecCase self ) {}
*/

	//--------------------------------------------------------------------------------------------------------------
	/** This method removes a "<BR>"+eol combination from the end of the given 
		StringBuffer, if it is present.
	*/
	// ??? Move this all to SpecWriter so can remove references to JmlHTML in SpecWriter
	/*@ requires classspec != null;
		ensures (* if \old(classpec) ends in "<BR>"+eol then classspec is
					truncated to remove it, otherwise classspec is unchanged *);
	*/
	public static void removeBR(StringBuffer classspec) {
		int n = classspec.length();
		if (n >= BREOLlength) {
			if (classspec.substring(n-BREOLlength).equals(BREOL))
				classspec.setLength(n-BREOLlength);
		}
	}
	
	public static final String BREOL = "<BR>" + eol;
	public static final int BREOLlength = BREOL.length();

	/** Copies the input string to the output, replacing instances of
	    special html characters by html representations (less than
	    symbols, greater than symbols, ampersands).
	*/
	public String convertToHtml(String s) {
		s = replace(s,'&',"&amp;"); // Do this one first or the &'s
					// in &lt; and &gt; get converted
		s = replace(s,'<',"&lt;");
		s = replace(s,'>',"&gt;");
		// FIXME - Do we need to do this for "
        	return s;
	}

	private String replace(String s, char c, String rep) {
		StringBuffer sb = new StringBuffer();
		int i = 0;
		int j;
		while ((j=s.indexOf(c,i)) != -1) {
		    sb.append(s.substring(i,j));
		    sb.append(rep);
		    i = j+1;
		}
		sb.append(s.substring(i));
		return sb.toString();
	}
}
