/*
 * @(#)$Id: JChoice.java,v 1.1 2008/03/04 23:48:20 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlexec.constraints;

import java.util.*;
import org.jmlspecs.models.JMLCollection;
import org.jmlspecs.jmlexec.runtime.InsufficientInformationException;
import org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem;
import org.jmlspecs.jmlexec.jack.evaluator.UDConstraint;
import org.jmlspecs.jmlexec.jack.evaluator.SChoice;
import org.jmlspecs.jmlexec.jack.evaluator.SExploration;
import org.jmlspecs.jmlexec.jack.evaluator.SDepthFirstExploration;
import org.jmlspecs.jmlexec.jack.evaluator.SSearch;
import org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer;
import org.jmlspecs.jmlexec.runtime.MyInteger;
import org.jmlspecs.jmlexec.runtime.MyNumber;
import org.jmlspecs.jmlexec.runtime.MyLong;
import org.jmlspecs.jmlexec.runtime.FDomain;
import org.jmlspecs.jmlexec.runtime.MyBoolean;
import org.jmlspecs.jmlexec.runtime.VarObject;
import org.jmlspecs.jmlexec.runtime.GenMethod;

public class JChoice implements SChoice {

  // change the following to allow enumeration of FD variables with larger
  // or smaller domains
  public static int DOMAIN_SIZE = 50;
  private static int numChoices = 0;
  private int number;
  private boolean failed;
  private boolean trace = false;
  private SChoice continuation;
  private UDConstraint active1;
  private UDConstraint active2;
  private Object variable;
  private long curVal = 0;
  
  public JChoice() {
    numChoices++;
    number = numChoices;
    failed = false;
  }
  
  public JChoice(boolean trace) {
    this();
    this.trace = trace;
  }
  
  public JChoice(boolean trace, SChoice continuation) {
    this(trace);
    this.continuation = continuation;
  }
  
  private boolean ground(Object obj, String classname, ConstraintSystem cs) {
    try {
      boolean ground = obj != null && 
        cs.getVarValue(obj).getClass().equals(Class.forName(classname));
      return ground;
    } catch (ClassNotFoundException e) {
      System.out.println("Class not found.");
      return false;
    } catch (org.jmlspecs.jmlexec.runtime.NoValueException e) {
      return false;
    }
  }
  
  /** requires: obj is ground, not null and a subclass of JMLCollection */
  private boolean isEmpty(Object obj, ConstraintSystem cs) {
    return ((org.jmlspecs.models.JMLCollection)
      cs.getVarValue(obj)).int_size() == 0;
  }
  
  private boolean is(UDConstraint cons, String type) {
    return cons != null && cons.getIdentifier().equals(type);
  }
  
  public static boolean runGoal(ConstraintSystem cs, boolean trace, 
                                boolean useBackTracking) {
    boolean success = cs.callGoal(trace);
    if (success && useBackTracking) {
      JChoice c = new JChoice(trace);
      SExploration exploration = new SDepthFirstExploration(cs, c);
      success = SSearch.first(exploration);
    }
    return success;
  }
  
  public static boolean runGoal(ConstraintSystem cs, boolean trace) {
    return runGoal(cs, trace, true);
  }
  
  private boolean equalVars(Object arg1, Object arg2, ConstraintSystem cs) {
    return cs.getVariableTable().getEqualVars(arg1).contains(arg2);
  }
  
  private boolean findConstraintWGroundArg(UDConstraint current, String c1,
                                           int arg, int offSet,
                                           ConstraintSystem cs) {
    if (is(current, c1)) {
      String argtype = MPATH + (String) 
                         current.getArgAt(current.getArity() - offSet);
      boolean ground = ground(current.getArgument(arg), argtype, cs);
      if (ground) ground = !isEmpty(current.getArgument(arg), cs);
      if (ground) active1 = current;
      return ground;
    }
    return false;
  }

  
  private boolean findConstraintWTrailer(UDConstraint current, String c1, 
                                         int arg, ConstraintSystem cs) {
    UDConstraint current2;
    ObjectContainer mem = cs.getActiveConstraints();
    if (is(current, c1)) {
      active1 = current;
      String typ = (String) current.getArgAt(current.getArity() - 2);
      boolean ground = ground(current.getArgument(arg), MPATH + typ, cs);
      if (ground) {
        ground = !isEmpty(current.getArgument(arg), cs);
      }  
      Iterator i2 = mem.iterator();
      while (i2.hasNext()) {
        current2 = (UDConstraint) i2.next();  
        if (ground || (is(current2, "trailer") && 
                      equalVars(current.getArgument(arg), 
                                current2.getArgument(0), cs))) {
          if (ground) {
            variable = new VarObject();
            cs.addVariable(variable, MPATH + typ);
            active2 = new TRAILERConstraint(active1.getArgument(arg),
                                            typ, variable);
            cs.addGoalConstraint(active2);
          } else active2 = current2;
          return true;
        }
      }
    }
    return false;
  }

/*
  private UDConstraint findArrayLength(ConstraintSystem cs) {
    ObjectContainer mem = cs.getActiveConstraints();
    Iterator it = mem.iterator();
    UDConstraint current;
    while (it.hasNext()) {
      current = (UDConstraint) it.next();
      if (is(current, "arraylength") && 
          ground(current.getArgument(2), RPATH + "MyInteger", cs)) {
        return current;
      }
    }
    return null;
  }
*/

  private UDConstraint findSmallestDomain(ConstraintSystem cs) {
    ObjectContainer mem = cs.getActiveConstraints();
    Iterator it = mem.iterator();
    UDConstraint current = null;
    UDConstraint smallest = null;
    double size = 0;
    while (it.hasNext()) {
      current = (UDConstraint) it.next();
      if (is(current, "fdVar")) {
        FDomain fd = (FDomain) current.getArgument(2);
        if (smallest == null || fd.size() < size) {
          smallest = current;
          size = fd.size();
        }
      }
    }
    if (smallest != null) {
      FDomain sd = (FDomain) smallest.getArgument(2);
      if (sd.size() > DOMAIN_SIZE && !sd.inRange(sd.oldValue())) {
        throw new InsufficientInformationException("Error: enumerated a " +
                   "finite domain variable with domain size " + sd.size() +
                   " > max size of " + DOMAIN_SIZE);
      }
    }
    return smallest;
  }

  private UDConstraint findOr(ConstraintSystem cs) {
    ObjectContainer mem = cs.getActiveConstraints();
    Iterator it = mem.iterator();
    UDConstraint current = null;
    while (it.hasNext()) {
      current = (UDConstraint) it.next();
      if (is(current, "or")) {
        return current;
      }
    }
    return null;
  }

/*
  private UDConstraint findNewException(ConstraintSystem cs) {
    ObjectContainer mem = cs.getUserDefinedConstraintMemory();
    if (mem.size() == 1) {
       UDConstraint current = (UDConstraint) mem.first();
       if (is(current, "newException")) {
         return current;
       }
    }
    return null;
  }
*/

  private SChoice handleChoice(ConstraintSystem cs) {
    failed = !(cs.callGoal(trace));
    if (trace) {
      System.out.println("Goals succeeded? "
      + !failed);
      System.out.println(cs.getUserDefinedConstraintMemory());
    }
    if (failed) {
      return null;
    } else if (cs.getUserDefinedConstraintMemory().size() == 0) {
      return continuation;
    } else {
      return new JChoice(trace, continuation);
    }
  }

  public SChoice choose(ConstraintSystem cs, boolean firstChoice) {
    try {
    if (trace) System.out.println(cs.getCopyEnvironment());
    if (cs.getUserDefinedConstraintMemory().size() == 0) {
      failed = false;
      if (firstChoice) {
        return continuation;
      } else {
        return null;
      }
    }
    ObjectContainer mem = cs.getActiveConstraints();
    UDConstraint current;
    if (firstChoice) {
/*
      current = findNewException(cs);
      if (current != null) {
        active1 = current;
        cs.removeGoalConstraint(current);
        EQUALEQUALConstraint neweq =
            new EQUALEQUALConstraint(current.getArgument(0), 
            org.jmlspecs.jmlexec.runtime.ObjUtil.newExceptionNoMesg( 
              (Class) current.getArgument(1)));
        cs.addGoalConstraint(neweq);
        if (trace) System.out.println("replacing newException with " +
            neweq);
        return handleChoice(cs);
      }
*/
      current = findOr(cs);
      if (current != null) {
        active1 = current;
          ((GenMethod) current.getArgument(0)).run(cs);
          cs.removeGoalConstraint(current);
        if (trace) {
          System.out.println("Choice 1");
          System.out.println("JChoice number " + number);
          System.out.println("replacing or with " +
          cs.getUserDefinedConstraintMemory());
        }
        failed = !(cs.callGoal(trace));
        return handleChoice(cs);
      }
/*
      current = findArrayLength(cs);
      if (current != null) {
        active1 = current;
        cs.removeGoalConstraint(current);
        EQUALEQUALConstraint neweq =
            new EQUALEQUALConstraint(current.getArgument(0), 
              QuantUtil.fromSize(
                (String) cs.getVarValue(active1.getArgument(1)), 
                (MyInteger) cs.getVarValue( active1.getArgument(2)), cs));
        cs.addGoalConstraint(neweq);
        if (trace) System.out.println("replacing arraylength with " + neweq);
        return handleChoice(cs);
      }
*/
      Iterator i = mem.iterator(); 
      while (i.hasNext()) {
        current = (UDConstraint) i.next();
        if (findConstraintWTrailer(current, "count", 0, cs) &&
            current.getArgAt(current.getArity() - 2).equals(
              "JMLObjectSequence")) {
          // count(JC1, O1, St1, I1) with trailer(JC1, St1, JC2) replaced by
          //    first(JC1, St1, O1) && gt(I1, 0) && sub(I1, 1, I2) &&
          //       count(JC2, O1, St1, I2) 
          String typ = (String) current.getArgAt(current.getArity() - 2);
          cs.removeGoalConstraint(active1);
          cs.addGoalConstraint(new FIRSTConstraint(active2.getArgument(0),
                                                   typ,
                                                   active1.getArgument(1)));
          Object IN = new VarObject();
          cs.addVariable(IN, "org.jmlspecs.jmlexec.runtime.MyInteger");
          cs.addGoalConstraint(new LTConstraint(new MyInteger(0), 
                                                active1.getArgument(3), 
                                                MyBoolean.TRUE()));
          ADDNUMSConstraint wfe2 = new ADDNUMSConstraint(active1.getArgument(3),
                                                 new MyInteger(-1), IN);
          cs.addGoalConstraint(wfe2);
          COUNTConstraint wfe = new COUNTConstraint(active2.getArgument(2),
                                                    active1.getArgument(1),
                                                    typ, IN); 
          cs.addGoalConstraint(wfe);
          if (trace) {
            System.out.println("Choice 1: " + active1 + " replaced by " + wfe);
            System.out.println("JChoice number " + number);
            System.out.println(wfe2);
          }
          return handleChoice(cs);
        } else if (findConstraintWTrailer(current, "concat", 3, cs)) {
          // concat(JC1, JC2, St1, JC3) with trailer(JC3, St1, JC4) replaced by
          //    isEmpty(JC1, St1) && equals(JC2, JC3, St1) 
          String typ = (String) current.getArgAt(current.getArity() - 2);
          ISEMPTYConstraint wfe = new ISEMPTYConstraint(active1.getArgument(0),
                                                        typ);
          EQUALSConstraint wfe2 = new EQUALSConstraint(active1.getArgument(1),
                                                       active1.getArgument(3),
                                                       typ);
          cs.removeGoalConstraint(active1);
          cs.addGoalConstraint(wfe);
          cs.addGoalConstraint(wfe2);
          if (trace) {
            System.out.println("JChoice number " + number);
            System.out.println("Choice 1: " + active1 + " replaced by " + wfe);
            System.out.println("and " + wfe2);
          }
          return handleChoice(cs);
        } else if (findConstraintWTrailer(current, "itemAt", 0, cs)) {
          // itemAt(JC1, I1, St1, O1) with trailer(JC1, St1, JC2) replaced by
          //    first(JC1, St1, O1) && I1 = 0 
          String typ = (String) current.getArgAt(current.getArity() - 2);
          FIRSTConstraint wfe1 = new FIRSTConstraint(active1.getArgument(0),
                                                     typ,
                                                     active1.getArgument(3));
          EQUALEQUALConstraint wfe2 = 
            new EQUALEQUALConstraint(active1.getArgument(1), new MyInteger(0));
          cs.removeGoalConstraint(active1);
          cs.addGoalConstraint(wfe1);
          cs.addGoalConstraint(wfe2);
          if (trace) {
            System.out.println("JChoice number " + number);
            System.out.println("Choice 1: " + active1 + " replaced by " + wfe1);
          }
          return handleChoice(cs);
        } else if (findConstraintWTrailer(current, "indexOf", 0, cs) &&
            current.getArgAt(current.getArity() - 2).equals(
              "JMLObjectSequence")) {
          // indexOf(JC1, O1, St1, I1) with trailer(JC1, St1, JC2) replaced by
          //    first(JC1, St1, O1) && I1 = 0 
          String typ = (String) current.getArgAt(current.getArity() - 2);
          cs.removeGoalConstraint(active1);
          UDConstraint wfe = new FIRSTConstraint(active1.getArgument(0), 
                                                 typ,
                                                 active1.getArgument(1));
          UDConstraint wfe2 = new EQUALEQUALConstraint(active1.getArgument(3),
                                                       new MyInteger(0));
          cs.addGoalConstraint(wfe);
          cs.addGoalConstraint(wfe2);
          if (trace) {
            System.out.println("JChoice number " + number);
            System.out.println("Choice 1: " + active1 + " replaced by " + wfe);
            System.out.println("and " + wfe2);
          }
          return handleChoice(cs);
        } else if (findConstraintWTrailer(current, "insertAfterIndex", 0, cs)) {
          // insertAfterIndex(JC1, I1, O1, St1, JC2) with 
          //        trailer(JC1, St1, JC3) replaced by
          //    trailer(JC2, St1, JC4) && first(JC2, St1, O1) && 
          //        equals(JC1, JC4, St1) && I1 = -1
          String typ = (String) current.getArgAt(current.getArity() - 2);
          cs.removeGoalConstraint(active1);
          Object R1 = new VarObject();
          cs.addVariable(R1, MPATH + typ);
          UDConstraint wfe = new TRAILERConstraint(active1.getArgument(4),
                                                   typ, R1);
          UDConstraint wfe1 = new FIRSTConstraint(active1.getArgument(4),
                                                  typ,
                                                  active1.getArgument(2));
          UDConstraint wfe2 = new EQUALSConstraint(active1.getArgument(0), R1,
                                                   typ);
          UDConstraint wfe3 = new EQUALEQUALConstraint(active1.getArgument(1),
                                                       new MyInteger(-1));
          cs.addGoalConstraint(wfe);
          cs.addGoalConstraint(wfe1);
          cs.addGoalConstraint(wfe2);
          cs.addGoalConstraint(wfe3);
          if (trace) {
            System.out.println("JChoice number " + number);
            System.out.println("Choice 1: " + active1 + " replaced by " + 
                               wfe + ",");
            System.out.println(wfe + ", " + wfe1 + ", " + wfe2 + ", " + wfe3);
          }
          return handleChoice(cs);
        } else if (findConstraintWGroundArg(current, "insert", 3, 2, cs)) {
          // insert(JC1, O1, St1, JC2) with JC2 ground replaced by
          //    equals(JC1, JC2) && has(JC1, O1, St1) && has(JC2, O1, St1)
          String typ = (String) current.getArgAt(current.getArity() - 2);
          cs.removeGoalConstraint(active1);
          UDConstraint wfe = new EQUALSConstraint(active1.getArgument(0),
                                                  active1.getArgument(3),
                                                  typ);
          UDConstraint wfe1 = new HASConstraint(active1.getArgument(0),
                                                active1.getArgument(1),
                                                typ);
          UDConstraint wfe2 = new HASConstraint(active1.getArgument(3),
                                                active1.getArgument(1),
                                                typ);
          cs.addGoalConstraint(wfe);
          cs.addGoalConstraint(wfe1);
          cs.addGoalConstraint(wfe2);
          if (trace) {
            System.out.println("JChoice number " + number);
            System.out.println("Choice 1: " + active1 + " replaced by " + 
                               wfe + ",");
            System.out.println(wfe1 + ", and " + wfe2);
          }
          return handleChoice(cs);
        } else if (findConstraintWGroundArg(current, "has", 0, 1, cs)) {
          // has(JC1, O1, St1) with JC1 ground replaced by
          //    choose(JC1, St1, O1) 
          String typ = (String) current.getArgAt(current.getArity() - 1);
          cs.removeGoalConstraint(active1);
          JMLCollection S = (JMLCollection) 
                              cs.getVarValue(active1.getArgument(0));
          variable = org.jmlspecs.jmlexec.runtime.JMLTool.choose(S, typ);
          UDConstraint wfe = new EQUALEQUALConstraint(active1.getArgument(1),
                                                      variable);
          cs.addGoalConstraint(wfe);
          if (trace) {
            System.out.println("JChoice number " + number);
            System.out.println("Choice 1: " + active1 + " replaced by " + wfe);
          }
          return handleChoice(cs);
        } else if (findConstraintWGroundArg(current, "isFSubset", 1, 1, cs)) {
          // isFSubset(JC1, JC2, I2, JC4, St1) with JC2 ground replaced by
          //    choose(JC2, St1, O1) && has(JC1, O1, St1) && add(I2, 1, I1) &&
          //       remove(JC2, O1, St1, JC3) && 
          //       isFSubset(JC1, JC3, I1, JC4, St1) 
          String typ = (String) current.getArgAt(current.getArity() - 1);
          cs.removeGoalConstraint(active1);
          JMLCollection S = (JMLCollection) 
                              cs.getVarValue(active1.getArgument(1));
          Object S1 = new VarObject();
          Object I1 = new VarObject();
          cs.addVariable(S1, MPATH + typ);
          cs.addVariable(I1, "org.jmlspecs.jmlexec.runtime.MyInteger");
          variable = org.jmlspecs.jmlexec.runtime.JMLTool.choose(S, typ);
          UDConstraint wfe = new HASConstraint(active1.getArgument(0),
                                               variable, typ);
          UDConstraint wfe1 = new ADDNUMSConstraint(active1.getArgument(2), 
                                                    new MyInteger(1), I1);
          UDConstraint wfe2 = new REMOVEConstraint(active1.getArgument(1),
                                                   variable, typ, S1);
          UDConstraint wfe3 = new ISFSUBSETConstraint(active1.getArgument(0),
                                                      S1, I1, 
                                                      active1.getArgument(3),
                                                      typ);
          cs.addGoalConstraint(wfe);
          cs.addGoalConstraint(wfe1);
          cs.addGoalConstraint(wfe2);
          cs.addGoalConstraint(wfe3);
          if (trace) {
            System.out.println("JChoice number " + number);
            System.out.println("Choice 1: " + active1 + " replaced by "
                               + wfe + ", " + wfe1);
          }
          return handleChoice(cs);
        }
      }
      current = findSmallestDomain(cs);
      if (current != null) {
        active1 = current;
        curVal = ((FDomain) current.getArgument(2)).getFirst();
        cs.removeGoalConstraint(current);
        EQUALEQUALConstraint eq = 
          new EQUALEQUALConstraint(active1.getArgument(0), 
               MyNumber.cast(new MyLong(curVal), 
              (String) active1.getArgument(1)));
        cs.addGoalConstraint(eq);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 1: " + active1 + " replaced by " +
             eq);
        }
        return handleChoice(cs);
      }
      if (trace) System.out.println(cs.getCopyEnvironment());
      throw new InsufficientInformationException("Insufficient information.");
    } else {
// SECOND CHOICE
      if (is(active1, "fdVar")) {
        cs.removeGoalConstraint(active1);
        if (trace) {
            System.out.println("JChoice number " + number);
            System.out.print("Choice 2: " + active1 + " replaced by ");
        }
        FDomain fd = (FDomain) active1.getArgument(2);
        FDomain fd2 = fd.copy();
        fd2.remove(curVal);
        FDVARConstraint fdv = new FDVARConstraint(active1.getArgument(0),
          (String) active1.getArgument(1), fd2);
        cs.addGoalConstraint(fdv);
        if (trace) {
          System.out.println(fdv + ".");
        }
        return handleChoice(cs);
/*
      } else if (is(active1, "arraylength")) {
        failed = true;
        throw new InsufficientInformationException("Insufficient information.");
*/
/*
      } else if (is(active1, "newException")) {
        failed = true;
        throw new InsufficientInformationException("Insufficient information.");
*/
      } else if (is(active1, "or")) {
        cs.removeGoalConstraint(active1);
        ((GenMethod) active1.getArgument(1)).run(cs);
        if (trace) {
          System.out.println("Choice 2");
          System.out.println("JChoice number " + number);
          System.out.println("replacing or with " +
          cs.getUserDefinedConstraintMemory());
        }
        return handleChoice(cs);
      } else if (is(active1, "count") && is(active2, "trailer")) {
          // count(JC1, O2, St1, I1) with trailer(JC1, St1, JC2) replaced by
          //    first(JC1, St1, O2) && gt(I1, 0) && sub(I1, 1, I2) &&
          //       count(JC2, O2, St1, I2) 
          //  fails, so replaced by:
          //    first(JC1, St1, O1) && O1 != O2 && count(JC2, O2, St1, I1)
        String typ = (String) active1.getArgAt(active1.getArity() - 2);
        if (variable != null) {
          cs.addVariable(variable, MPATH + typ);
          cs.addGoalConstraint(active2);
        }
        Object O2 = active1.getArgument(1);
        Object O1 = new VarObject();
        cs.addVariable(O1, "java.lang.Object");
        cs.addGoalConstraint(new FIRSTConstraint(active1.getArgument(0),
                                                 typ, O1));
        cs.addGoalConstraint(new INEQUALITYConstraint(O1, O2,
           org.jmlspecs.jmlexec.runtime.MyBoolean.TRUE()));
        COUNTConstraint wfe = new COUNTConstraint(active2.getArgument(2), O2,
                                                  typ, active1.getArgument(3)); 
        cs.addGoalConstraint(wfe);
        if (trace) {
          System.out.println("Choice 2: " + active1 + " replaced by " + wfe);
          System.out.println("JChoice number " + number);
        }
        cs.removeGoalConstraint(active1);
        return handleChoice(cs);
      } else if (is(active1, "concat") && is(active2, "trailer")) {
          // concat(JC1, JC2, St1, JC3) with trailer(JC3, St1, JC4) replaced by
          //    isEmpty(JC1, St1) && equals(JC2, JC3, St1) 
          //  fails, so replaced by:
          //    trailer(JC1, St1, JC5) && concat(JC5, JC2, St1, JC4) &&
          //      first(JC1, St1, O2) && first(JC3, St1, O2)
        String typ = (String) active1.getArgAt(active1.getArity() - 2);
        if (variable != null) {
          cs.addVariable(variable, MPATH + typ);
          cs.addGoalConstraint(active2);
        }
        Object H1 = new VarObject();
        Object R1 = new VarObject();
        cs.removeGoalConstraint(active1);
        cs.addVariable(H1, MPATH + typ);
        cs.addVariable(R1, MPATH + typ);
        cs.addGoalConstraint(active2);
        TRAILERConstraint wfe = new TRAILERConstraint(active1.getArgument(0),
                                                      typ, R1);
        CONCATConstraint wfe2 = new CONCATConstraint(R1, 
                                                     active1.getArgument(1),
                                                     typ,
                                                     active2.getArgument(2));
        FIRSTConstraint wfe3 = new FIRSTConstraint(active1.getArgument(0), 
                                                   typ, H1);
        FIRSTConstraint wfe4 = new FIRSTConstraint(active1.getArgument(3),
                                                   typ, H1);
        cs.addGoalConstraint(wfe);
        cs.addGoalConstraint(wfe2);
        cs.addGoalConstraint(wfe3);
        cs.addGoalConstraint(wfe4);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 2: " + active1 + " replaced by");
          System.out.println(wfe + ", " + wfe2 + ", " + wfe3 + ", " + wfe4);
        }
        return handleChoice(cs);
      } else if (is(active1, "itemAt") && is(active2, "trailer")) {
          // itemAt(JC1, I2, St1, O1) with trailer(JC1, St1, JC2) replaced by
          //    first(JC1, St1, O1) && I2 = 0 
          //  fails, so replaced by:
          //    sub(I2, 1, I1) && itemAt(JC2, I1, St1, O1)
        String typ = (String) active1.getArgAt(active1.getArity() - 2);
        if (variable != null) {
          cs.addVariable(variable, MPATH + typ);
          cs.addGoalConstraint(active2);
        }
        Object I1 = new VarObject();
        cs.addVariable(I1, "org.jmlspecs.jmlexec.runtime.MyInteger");
        ADDNUMSConstraint wfe = new ADDNUMSConstraint(active1.getArgument(1),
                                              new MyInteger(-1), I1);
        ITEMATConstraint wfe1 = new ITEMATConstraint(active2.getArgument(2),
                                                     I1, typ,
                                                     active1.getArgument(3));
        cs.removeGoalConstraint(active1);
        cs.addGoalConstraint(wfe);
        cs.addGoalConstraint(wfe1);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 2: " + active1 + " replaced by");
          System.out.println(wfe + " and " + wfe1);
        }
        return handleChoice(cs);
      } else if (is(active1, "indexOf") && is(active2, "trailer")) {
          // indexOf(JC1, O2, St1, I1) with trailer(JC1, St1, JC2) replaced by
          //    first(JC1, St1, O2) && I1 = 0 
          //  fails, so replaced by:
          //    sub(I1, 1, I2) && indexOf(JC2, O2, St1, I2) && 
          //      first(JC1, St1, O1) && O1 != O2
        String typ = (String) active1.getArgAt(active1.getArity() - 2);
        if (variable != null) {
          cs.addVariable(variable, MPATH + typ);
          cs.addGoalConstraint(active2);
        }
        Object I2 = new VarObject();
        Object O1 = new VarObject();
        cs.addVariable(I2, "org.jmlspecs.jmlexec.runtime.MyInteger");
        cs.addVariable(O1, "java.lang.Object");
        cs.removeGoalConstraint(active1);
        UDConstraint wfe = new ADDNUMSConstraint(active1.getArgument(3),
                                             new MyInteger(-1), I2);
        UDConstraint wfe1 = new INDEXOFConstraint(active2.getArgument(2),
                                                  active1.getArgument(1),
                                                  typ, I2);
        UDConstraint wfe2 = new FIRSTConstraint(active1.getArgument(0),
                                                typ, O1);
        UDConstraint wfe3 = new INEQUALITYConstraint(O1,
                               active1.getArgument(1),
                               org.jmlspecs.jmlexec.runtime.MyBoolean.TRUE());
        cs.addGoalConstraint(wfe);
        cs.addGoalConstraint(wfe1);
        cs.addGoalConstraint(wfe2);
        cs.addGoalConstraint(wfe3);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 2: " + active1 + " replaced by");
          System.out.println(wfe + ", " + wfe1 + ", " + wfe2 + ", " + wfe3);
        }
        return handleChoice(cs);
      } else if (is(active1, "insertAfterIndex") && is(active2, "trailer")) {
          // insertAfterIndex(JC1, I2, O2, St1, JC2) with 
          //        trailer(JC1, St1, JC3) replaced by
          //    trailer(JC2, St1, JC4) && first(JC2, St1, O2) && 
          //        equals(JC1, JC4, St1) && I2 = -1
          //  fails, so replaced by:
          //    sub(I2, 1, I1) && trailer(JC2, St1, JC5) &&
          //      insertAfterIndex(JC3, I1, O2, St1, JC5) &&
          //      first(JC2, St1, O1) && gt(I2, -1)
        String typ = (String) active1.getArgAt(active1.getArity() - 2);
        cs.removeGoalConstraint(active1);
        if (variable != null) {
          cs.addVariable(variable, MPATH + typ);
          cs.addGoalConstraint(active2);
        }
        Object R1 = new VarObject();
        Object I1 = new VarObject();
        Object O1 = new VarObject();
        cs.addVariable(R1, MPATH + typ);
        cs.addVariable(I1, "org.jmlspecs.jmlexec.runtime.MyInteger");
        cs.addVariable(O1, "java.lang.Object");
        UDConstraint wfe = new ADDNUMSConstraint(active1.getArgument(1),
                                             new MyInteger(-1), I1);
        UDConstraint wfe1 = new TRAILERConstraint(active1.getArgument(4), typ,
                                                  R1);
        UDConstraint wfe2 = new INSERTAFTERINDEXConstraint(
           active2.getArgument(2), I1, active1.getArgument(2), typ, R1);
        UDConstraint wfe3 = new FIRSTConstraint(active1.getArgument(0),
                                                typ, O1);
        UDConstraint wfe4 = new FIRSTConstraint(active1.getArgument(4), typ,
                                                O1);
        UDConstraint wfe5 = new LTConstraint(new MyInteger(-1),
                                             active1.getArgument(1),
                                             MyBoolean.TRUE());
        cs.addGoalConstraint(wfe);
        cs.addGoalConstraint(wfe1);
        cs.addGoalConstraint(wfe2);
        cs.addGoalConstraint(wfe3);
        cs.addGoalConstraint(wfe4);
        cs.addGoalConstraint(wfe5);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 2: " + active1 + " replaced by");
          System.out.println(wfe + ", " + wfe1 + ", " + wfe2 + ", "
                             + wfe3 + ", " + wfe4);
        }
        return handleChoice(cs);
      } else if (is(active1, "insert")) {
          // insert(JC1, O1, St1, JC2) with JC2 ground replaced by
          //    equals(JC1, JC2) && has(JC1, O1, St1) && has(JC2, O1, St1)
          //  fails, so replaced by:
          //    remove(JC2, O1, St1, JC1)
        String typ = (String) active1.getArgAt(active1.getArity() - 2);
        cs.removeGoalConstraint(active1);
        Object I1 = new VarObject();
        UDConstraint wfe = new REMOVEConstraint(active1.getArgument(3),
                                                active1.getArgument(1),
                                                typ, active1.getArgument(0));

        cs.addGoalConstraint(wfe);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 1: " + active1 + " replaced by " + wfe);
        }
        return handleChoice(cs);
      } else if (is(active1, "has")) {
          // has(JC1, O1, St1) with JC1 ground replaced by
          //    choose(JC1, St1, O2) && O1 = O2
          //  fails, so replaced by:
          //    remove(JC1, O2, St1, JC2) && has(JC2, O1, St1)
        String typ = (String) active1.getArgAt(active1.getArity() - 1);
        cs.removeGoalConstraint(active1);
        JMLCollection S1 = (JMLCollection) 
            cs.getVarValue(active1.getArgument(0));
        JMLCollection S = (JMLCollection)
          org.jmlspecs.jmlexec.runtime.JMLTool.remove(S1, variable, typ);
        UDConstraint wfe = new HASConstraint(S, active1.getArgument(1), typ);
        cs.addGoalConstraint(wfe);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 2: " + active1 + " replaced by " + wfe);
        }
        return handleChoice(cs);
      } else if (is(active1, "isFSubset")) {
          // isFSubset(JC1, JC2, I2, JC4, St1) with JC2 ground replaced by
          //    choose(JC2, St1, O1) && has(JC1, O1, St1) && add(I2, 1, I1) &&
          //       remove(JC2, O1, St1, JC3) && 
          //       isFSubset(JC1, JC3, I1, JC4, St1) 
          //  fails, so replaced by:
          //    notHas(JC1, O1, St1) && remove(JC2, O1, St1, JC3) &&
          //      isFSubset(JC1, JC3, I2, JC4, St1)
        String typ = (String) active1.getArgAt(active1.getArity() - 1);
        cs.removeGoalConstraint(active1);
        Object S1 = new VarObject();
        cs.addVariable(S1, MPATH + typ);
        UDConstraint wfe = new NOTHASConstraint(active1.getArgument(0),
                                                variable, typ);
        UDConstraint wfe1 = new REMOVEConstraint(active1.getArgument(1),
                                                 variable, typ, S1);
        UDConstraint wfe2 = new ISFSUBSETConstraint(active1.getArgument(0), S1,
                                                    active1.getArgument(2),
                                                    active1.getArgument(3),
                                                    typ);
        cs.addGoalConstraint(wfe);
        cs.addGoalConstraint(wfe1);
        cs.addGoalConstraint(wfe2);
        if (trace) {
          System.out.println("JChoice number " + number);
          System.out.println("Choice 2: " + active1 + " replaced by " +
                             wfe + ", " + wfe1 + ", " + wfe2);
        }
        return handleChoice(cs);
      } else {
        //throw new RuntimeException("Fatal error.");
        return null;
      }
      //throw new InsufficientInformationException("Insufficient information");
    }
    //  throw new RuntimeException("Fatal error.");
    } catch (InsufficientInformationException e) {
      throw e;
    } catch (Throwable e) {
      if (e instanceof RuntimeException) {
        throw (RuntimeException) e;
      } else {
        throw new IllegalArgumentException(e.getClass().toString());
      }
    }
  }
  
  public boolean failed() {return failed;}
  
  public void reset() {
    failed = false;
  }

  private static String MPATH = "org.jmlspecs.models.";
  private static String RPATH = "org.jmlspecs.jmlexec.runtime.";
}
