package org.jmlspecs.jmlexec.jack.compiler;

import java.io.FileWriter;
import java.io.IOException;
import org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer;

public class ConstraintExtender {

    //////////////////////////////////////////////////
    // fields

    private String constraintName;
    private String constraintClassName;
    private String path;
    private ObjectContainer argClassNames; // contains Strings of class names
    private boolean first;

    //////////////////////////////////////////////////
    // constructor

    public ConstraintExtender(String p, String n, ObjectContainer acn) {

	path = p;
	if (!path.endsWith("/"))
	    path += "/";

	constraintName = n;
	constraintClassName = n.toUpperCase()+"Constraint";
	argClassNames = acn;

    }

    public int getArity() {

	return argClassNames.size();

    }

    public String getPathConstraintClassName() {

	return path+constraintClassName;

    }

    public String getConstraintClassName() {

	return constraintClassName;

    }

    public boolean extend() throws IOException {

	String of = path+constraintClassName+".java";
	FileWriter cfile = new FileWriter(of, false);
	
	cfile.write("package org.jmlspecs.jmlexec.constraints;\n");
	cfile.write("import org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer;\n");
	cfile.write("public class "+constraintClassName+" extends org.jmlspecs.jmlexec.jack.evaluator.UDConstraint {\n\n");
	cfile.write("\tObjectContainer args;\n");
	cfile.write("\tObjectContainer classes;\n\n");

	// ---------------------------------------------------------------------------------------
	// generate constructor

	cfile.write("\tpublic "+constraintClassName+"(");

	first = true;
	int i;
	for (i = 0; i < argClassNames.size(); i++) {

	    if (first)
		first = false;
	    else
		cfile.write(", ");
	    
	    cfile.write("Object arg"+i);
	    
	}
	
	cfile.write(") {\n\n\t\tsuper(\""+constraintName+"\", "+argClassNames.size()+");\n\n");
	
	if (argClassNames.size() > 0) {

	    cfile.write("\t\tclasses = new ObjectContainer();\n");
	    for (i = 0; i < argClassNames.size(); i++)
		cfile.write("\t\tclasses.add(\""+(String)argClassNames.get(i)+"\");\n");

	    cfile.write("\n\t\ttry {\n\n");
	    cfile.write("\t\t\tdeclareConstraint(classes);\n\n");
	    cfile.write("\t\t} catch(java.lang.ClassNotFoundException e) {\n\n");
	    cfile.write("\t\t\tSystem.err.println(\"\"+e);\n\n");
	    cfile.write("\t\t}\n\n");
	    
	    cfile.write("\t\targs = new ObjectContainer();\n");
	    for (i = 0; i < argClassNames.size(); i++)
		cfile.write("\t\targs.add(arg"+i+");\n");
	    cfile.write("\t\tsetArguments(args);\n\n");
	    
	}

	cfile.write("\n\t}\n\n");

	// -------------------------------------------------------------------------------------------
	// generate access to arguments
	
	for (i = 0; i < argClassNames.size(); i++) {

	    cfile.write("\tpublic Object getArg"+(i+1)+"() {\n\n");
	    cfile.write("\t\treturn args.get("+i+");\n\n\t}\n\n");
   
	}

	// -------------------------------------------------------------------------------------------
	// generate clone method

	/*
	  public Object clone() {
	  
	  FDENUConstraint tmp = new FDENUConstraint(args.get(0), args.get(1)); 
	  cloneInwards(tmp);
	  
	  return tmp;
	  
	  }
	 */

	cfile.write("\tpublic Object clone() {\n\n");
	cfile.write("\t\t"+constraintClassName+" tmp = new "+constraintClassName+"(");

	first = true;
	for(i = 0; i < argClassNames.size(); i++) {

	    if (first)
		first = false;
	    else 
		cfile.write(", ");

	    cfile.write("args.get("+i+")");

	}

	cfile.write(");\n\t\tcloneInwards(tmp);\n\n\t\treturn tmp;\n\n\t}\n\n");
	
	cfile.write("}\n");
	cfile.close();
	
	return true;

    }

}
