package org.jmlspecs.jmlexec.jack.compiler;

import java.io.Writer;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;
import org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer;
import org.jmlspecs.jmlexec.jack.evaluator.Bool;

public class JCHRGenerator implements JCHRParserTokenTypes {

    /////////////////////////////////////
    // constants 
  
    public static final String OUT_FILE_EXT         = ".java";

    /////////////////////////////////////
    // fields
  
    private JCHRAST jchrAST;

    private ObjectContainer classList; // list of strings
    private ObjectContainer ruleList;  // list of strings
    private ObjectContainer goalList;  // list of strings
    private Hashtable methodTable; // table with method strings and method extenders
    private Hashtable constraintTable;  // table with constraint strings and arities

    private ObjectContainer variableListRules;  // list of strings
    private Hashtable instanceHashRules; // table (instance name, class)

    private String path, outFileName;
    private boolean quiet;

    private Writer outWriter;

    public JCHRAST current;
    
    /////////////////////////////////////
    // constructors
  
    public JCHRGenerator(JCHRAST cast, Writer outw, boolean q) {
    
	jchrAST = cast;

	classList = new ObjectContainer();
	ruleList = new ObjectContainer();
	goalList = new ObjectContainer();
	methodTable = new Hashtable();
	constraintTable = new Hashtable();

	variableListRules = new ObjectContainer();
	instanceHashRules = new Hashtable();

	path = null;
	outFileName = null;
	outWriter = outw;

	quiet = q;

    }
  
    public JCHRGenerator(boolean q, Writer outw) {
    
	this(null, outw, q);
    
    }

    /////////////////////////////////////
    // public methods

    public int getLine() {

	if (current != null)
	    return current.getLine();
	else return -1;

    }       	

    public String getOutFileName() {

	return outFileName;

    }

    public void writeFile(JCHRAST cast, String p) // writeFile
	throws ClassNotFoundException, IOException, WrongConstraintTypeException, UnknownIdentifierException {
	
	current = cast;
	JCHRAST child, tmp1, tmp2, tmp3, tmp4;
	FileWriter outFile;
	String fileName;
	
	path = p;
	if (!path.endsWith("/"))
	    path = path+"/";
	
	if (cast.getType() == HANDLERKW) { // HANDLERKW?
	    
	    // -------------------------------------------------------------
	    // open FileWriter ---------------------------------------------
	    
	    child = (JCHRAST)cast.getFirstChild();
	    current = child;
	    fileName = child.getText()+"Handler";
	    outFileName = path+fileName+OUT_FILE_EXT;
	    outFile = new FileWriter(outFileName, false);
	    
	    // -------------------------------------------------------------
	    // write header to outFile

	    outFile.write("package org.jmlspecs.jmlexec.constraints;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.Bool;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.Equality;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.JMethod;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.SimplificationRule;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.PropagationRule;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.SimpagationRule;\n");
	    outFile.write("import org.jmlspecs.jmlexec.jack.evaluator.SimpagationHead;\n");
	    outFile.write("\n");
	    outFile.write("public class "+fileName+" extends org.jmlspecs.jmlexec.jack.evaluator.Handler {\n\n");
	    outFile.write("\tpublic ConstraintSystem cs;\n\n");
	    outFile.write("\tpublic "+fileName+"() {\n\n");
	    outFile.write("\t\tcs = new ConstraintSystem();\n\n");
	    outFile.write("\t}\n\n");

	    // -------------------------------------------------------------
	    // generate class import list

	    // documentation of used classes
	    child = (JCHRAST)child.getNextSibling();
	    current = child;
	    if ((child != null) && (child.getType() == CLASSDECLKW)) {
		
		outFile.write("\t// ---------------------------------------------\n");
		outFile.write("\t// used classes:\n\t//\n");
		
	    }
	    
	    // test, whether used classes exist or not (i.e. can be loaded)
	    while  ((child != null) && (child.getType() == CLASSDECLKW)) {

		String className = child.getFirstChild().getText();
		
		// if this doesn't work ClassNotFoundException will be thrown 
		Class.forName(className);

		if (!classList.contains(className)) {

		    // add class to class list
		    // following constructs like <variable class id> will use it
		    classList.add(className);

		    // write a comment to outfile in order to document the used
		    // classes for the user
		    outFile.write("\t// "+className+"\n");

		}

		child = (JCHRAST)child.getNextSibling();
		current = child;
		          
	    } 

	    // -------------------------------------------------------------
	    // generate constraint declarations
	    
	    if ((child != null) && (child.getType() == CONSTRDECLKW)) {

		outFile.write("\n\t// ---------------------------------------------\n");
		outFile.write("\t// created constraints:\n\t//\n");

	    }

	    while ((child != null) && (child.getType() == CONSTRDECLKW)) {

		tmp1 = (JCHRAST)child.getFirstChild(); // constraint name
		current = tmp1;
		tmp2 = (JCHRAST)tmp1.getNextSibling(); // argument list

		if (!constraintTable.containsKey(tmp1.getText())) {

		    // the ConstraintExtender object provides the method extend, which creates a 
		    // JAVA class, which is derived from the class UDConstraint.
		    // This new class will have a constructor with the same arity as the currently 
		    // declared constraint, in order to used it in a nicer way (without adding each
		    // argument to the argument list manually).
		    ConstraintExtender ce = new ConstraintExtender(path, 
								   tmp1.getText(), 
								   flattenArgList(tmp2, variableListRules, instanceHashRules));
		    // see above
		    if (ce.extend() && !quiet) {
			
			outWriter.write(ce.getPathConstraintClassName()+".java was created!\n");
			    outWriter.flush();
			    
		    } else if (!quiet) {
			
			    outWriter.write("!!! "+ce.getPathConstraintClassName()+".java was not created!\n");
			    outWriter.flush();			
			    
		    }

		    // add new declared constraint to the constraint list
		    // the method tocToString will use it.
		    constraintTable.put(tmp1.getText(), new Integer(ce.getArity()));
		    
		    // document the creation of the constraint clas in the outfile
		    outFile.write("\t// "+tmp1.getText()+" ("+ce.getConstraintClassName()+")\n");

		}
		
		child = (JCHRAST)child.getNextSibling();
		current = child;
		    
	    }

	    // generate rule definition section

	    if ((child != null) && (child.getType() == RULESKW)) {
		
		outFile.write("\n\tpublic void defineRules(ConstraintSystem cs) {\n\n");
		    
		genRulesDeclaration(outFile, child, variableListRules, instanceHashRules);
		child = (JCHRAST)child.getNextSibling();
		current = child;

		outFile.write("\n\t}\n");

	    }

	    // generate goal definition sections

	    while ((child != null) && (child.getType() == GOALKW)) {
		
		outFile.write("\n\tpublic void define"+child.getFirstChild().getText()+"(ConstraintSystem cs) {\n\n");

		goalList.add(child.getFirstChild().getText());
		genGoalsDeclaration(outFile, child, new ObjectContainer(), new Hashtable());
		child = (JCHRAST)child.getNextSibling();
		current = child;

		outFile.write("\n\t}\n\n");

	    }

	    // -------------------------------------------------------------
	    // write method extensions
	    Enumeration e = methodTable.elements();
	    while (e.hasMoreElements()) {

		JMethodExtender me = (JMethodExtender)e.nextElement();
		if (me.extend() && !quiet) {

		    outWriter.write(me.getPathMethodClassName()+".java was created!\n");
		    outWriter.flush();

		} else if (!quiet) {

		    outWriter.write("!!! "+me.getPathMethodClassName()+".java was not created!\n");
		    outWriter.flush();		    

		}


	    }    

	    // -------------------------------------------------------------
	    // write footer to outfile

	    footer(outFile, fileName);
	    
	    // -------------------------------------------------------------
	    // close FileWriter
	    
	    outFile.close();
	    if (!quiet) {

		outWriter.write(outFileName+" was created!\n");
		outWriter.flush();

	    }
	    
	} 
	
    } 
    
    public void writeFile(JCHRAST cast) 
	throws ClassNotFoundException, IOException, WrongConstraintTypeException, UnknownIdentifierException {
	
	writeFile(cast, ".");
	
    }
    
    public void writeFile(String path)
	throws ClassNotFoundException, IOException, WrongConstraintTypeException, UnknownIdentifierException {
	
	writeFile(jchrAST, path);
	
    }
    
    public void writeFile()
	throws ClassNotFoundException, IOException, WrongConstraintTypeException, UnknownIdentifierException {
	
	writeFile(jchrAST, ".");
	
    }
    
    /////////////////////////////////////
    // private methods

    private void footer(FileWriter outFile, String fileName) throws IOException {

	// -------------------------------------------------------------
	// write call goal method

	outFile.write("\n\tpublic void callGoal(boolean trace) {\n\n");
	outFile.write("\t\tSystem.out.println(\"\\ncalling goal:    \"+cs.getUserDefinedConstraintMemory());\n");
	outFile.write("\t\tSystem.out.println(\"variable table: \"+cs.getVariableTable()+\"\\n\");\n\n");
	outFile.write("\t\tif (cs.callGoal(trace)) {\n\n");
	outFile.write("\t\t\tSystem.out.println(\"\\nyes!\\n\");\n");
	outFile.write("\t\t\tSystem.out.println(\"variable table:      \"+cs.getVariableTable());\n");
	outFile.write("\t\t\tSystem.out.println(\"user defined memory: \"+cs.getUserDefinedConstraintMemory()+\"\\n\");\n\n");
	outFile.write("\t\t} else {\n\n");
	outFile.write("\t\t\tSystem.out.println(\"\\nno!\\n\");\n");
	outFile.write("\t\t\tSystem.out.println(\"variable table:             \"+cs.getVariableTable());\n");
	outFile.write("\t\t\tSystem.out.println(\"user defined memory:        \"+cs.getUserDefinedConstraintMemory());\n");
	outFile.write("\t\t\tSystem.out.println(\"failure causing constraint: \"+cs.getFailer()+\"\\n\");");
	outFile.write("\t\t\t\n\n");
	outFile.write("\t\t}\n\n");
	outFile.write("\t}\n\n");
	
	// -------------------------------------------------------------
	// write main method to outfile
	    
	outFile.write("\tpublic static void main(String args[]) {\n\n");
	
	outFile.write("\t\tboolean trace = false;\n\n");
	outFile.write("\t\tint goal = -1;\n\n");
	outFile.write("\t\tif (args.length > 0) {\n\n");
	outFile.write("\t\tfor (int currentarg = args.length - 1; currentarg >= 0; currentarg--) {\n\n");
	outFile.write("\t\t\tif (args[currentarg].equals(\"-h\")) {\n\n");
	outFile.write("\t\t\t\tSystem.out.println(\"< java "+fileName+" -t > prints evaluation trace (default is notrace)\\n\");\n");
	outFile.write("\t\t\t\treturn;\n\n");			
	outFile.write("\t\t\t} else if (args[currentarg].equals(\"-t\"))\n");
	outFile.write("\t\t\t\ttrace = true; \n\n\t\t\n\n");
	outFile.write("\t\t\t else if (args[currentarg].equals(\"-g\"))\n");
	outFile.write("\t\t\t\tgoal = Integer.parseInt(args[currentarg + 1]);\n\n\t}\t\n\n");
	outFile.write("\t\t}\n\n");
	
	outFile.write("\t\t"+fileName+" tmp = new "+fileName+"();\n");
	outFile.write("\t\ttmp.defineRules(tmp.cs);\n");
	
	for(int i = 0; i < goalList.size(); i++) {
	    
	    outFile.write("\t\tif (goal == -1 || goal == " + i + ") {");
	    outFile.write("\t\ttmp.define"+goalList.get(i)+"(tmp.cs);\n");
	    outFile.write("\t\tSystem.out.println(\"Calling goal \" + " + i + ");\n");
	    outFile.write("\t\ttmp.callGoal(trace);\n\n }");
	    
	}
	
	outFile.write("\t}\n\n");
	outFile.write("\n}");
	
    }
    
    private void genRulesDeclaration(FileWriter fw, JCHRAST cast, ObjectContainer vList, Hashtable iHash) 
	throws WrongConstraintTypeException, UnknownIdentifierException, IOException {

	JCHRAST tmp1, tmp2, tmp3, tmp4, sibling = (JCHRAST)cast.getFirstChild();
	current = sibling;
	String ruleID;

	// -------------------------------------------------------------
	// get variable and instance declarations for the rules block

	while (genVarInstDeclaration(fw, sibling, variableListRules, instanceHashRules)) {

	    sibling = (JCHRAST)sibling.getNextSibling();
	    current = sibling;

	}
		
	// -------------------------------------------------------------
	// get rule declarations

	while (sibling != null) {

	    fw.write("\n\t\t// ---------------------------------------------------------------\n");

	    tmp1 = (JCHRAST)sibling.getFirstChild();
	    current = tmp1;
	    tmp2 = (JCHRAST)tmp1.getNextSibling();
	    tmp3 = (JCHRAST)tmp2.getNextSibling();

	    // determine, whether this is a rule with guard or not
	    if (tmp1.getType() == GUARDKW) { // tmp1 is a guard, tmp2 is a head, tmp3 is a body
		
		tmp4 = (JCHRAST)tmp3.getNextSibling(); // tmp4 is a rule identifier
		if (tmp4 == null)
		    ruleID = "rule"+(ruleList.size()+1);
		else
		    ruleID = tmp4.getText();

		insertRule(fw, sibling.getType(), tmp2, tmp1, tmp3, ruleID, vList, iHash);
		ruleList.add(ruleID);
		
	    } else { // tmp1 is a head, tmp2 is a body, tmp3 is a rule identifier
		
		if (tmp3 == null)
		    ruleID = "rule"+(ruleList.size()+1);
		else
		    ruleID = tmp3.getText();

		insertRule(fw, sibling.getType(), tmp1, tmp2, ruleID, vList, iHash);
		ruleList.add(ruleID);
		
	    }
	    
	    sibling = (JCHRAST)sibling.getNextSibling();
	    current = sibling;
	    
	}
	
    } 
    
    private void genGoalsDeclaration(FileWriter fw, JCHRAST cast, ObjectContainer vList, Hashtable iHash) 
	throws UnknownIdentifierException, WrongConstraintTypeException, IOException {

	JCHRAST tmp1, tmp2, tmp3, tmp4, sibling = (JCHRAST)(cast.getFirstChild()).getNextSibling(); 
	current = sibling;
	// sibling = vardef or silbing = constraintlist or sibling = null

	if (sibling != null) {

	    // -------------------------------------------------------------
	    // get variable and instance declarations for the rules block

	    while (genVarInstDeclaration(fw, sibling, vList, iHash)) {

		sibling = (JCHRAST)sibling.getNextSibling();
		current = sibling;

	    }
	      
	
	    // -------------------------------------------------------------
	    // get goal declarations
	
	    insertGoal(fw, flattenConstraintList(sibling, vList, iHash));

	}
	    
    } 

    private boolean  genVarInstDeclaration(FileWriter fw, JCHRAST cast, ObjectContainer vList, Hashtable iHash) 
	throws IOException, WrongConstraintTypeException, UnknownIdentifierException {
	
	JCHRAST tmp1, tmp2, tmp3;

	if (cast != null) {
	
	    if (cast.getType() == VARDECLKW) { // transform variable declaration ----------------------
		
		tmp1 = (JCHRAST)cast.getFirstChild(); // variable class
		current = tmp1;
		tmp2 = (JCHRAST)tmp1.getNextSibling(); // variable id
		
		insertVarDeclaration(fw, tmp1.getText(), flattenIDList(tmp2), vList);
		
		return true;

	    } else if (cast.getType() == INSTDECLKW) { // transform instance declaration ---------------
		
		tmp1 = (JCHRAST)cast.getFirstChild(); // instance class
		current = tmp1;
		tmp2 = (JCHRAST)tmp1.getNextSibling(); // instance name
		tmp3 = (JCHRAST)tmp2.getNextSibling(); // argument list
		
		iHash.put(tmp2.getText(), tmp1.getText());
		insertInstanceDeclaration(fw, tmp1.getText(), tmp2.getText(), flattenArgList(tmp3, vList, iHash));
	    
		return true;
		
	    }

	}
	 
	return false;
    
    }

    private void insertVarDeclaration(FileWriter fw, String cl, ObjectContainer idlist, ObjectContainer vList) 
	throws IOException, UnknownIdentifierException {
	
	if (!classList.contains(cl))
	    throw new UnknownIdentifierException(cl);

	for (int i=0; i<idlist.size(); i++) {

	    if (!vList.contains(idlist.get(i))) {

		vList.add(idlist.get(i));
		fw.write("\t\tObject "+idlist.get(i)+" = new Object(); cs.addVariable("+idlist.get(i)+", \""+cl+"\", \""+idlist.get(i)+"\");\n");

	    }

	}

    }

    private void insertInstanceDeclaration(FileWriter fw, String cl, String instName, ObjectContainer args) 
      throws IOException {

	fw.write("\t\t"+cl+" "+instName+" = new "+cl+"("+args+");\n");

    }

    private void insertRule(FileWriter fw, int type, JCHRAST h, JCHRAST g, JCHRAST b, 
			    String ruleName, ObjectContainer vList, Hashtable iHash) 
	throws IOException, WrongConstraintTypeException, UnknownIdentifierException {

	ObjectContainer normalHead = null, minusHead = null, body, guard = null, head = null;
	JCHRAST tmp1, tmp2;
	String conStr;
	int i;

	fw.write("\n");
	
	// -----------------------------------------------------------------------
	// determine rule type
	String ruleType;
	if (type == PROPOP)
	    ruleType = "PropagationRule";
	else if (h.getType() == MINUSOP)
	    ruleType = "SimpagationRule";
	else
	    ruleType = "SimplificationRule";;

	// -----------------------------------------------------------------------
	// define containers for head (take care for simpagation rules: two heads are needed)
        fw.write("\t\tObjectContainer ");
	if (ruleType.equals("SimpagationRule")) {
	    
	    fw.write(ruleName+"NormalHead = new ObjectContainer();\n");
	    fw.write("\t\tObjectContainer "+ruleName+"MinusHead");

	} else
	    fw.write(ruleName+"Head");
	fw.write(" = new ObjectContainer();\n");

	// -----------------------------------------------------------------------
	// define container for guard (if exisiting)
	if (g != null)
	    fw.write("\t\tObjectContainer "+ruleName+"Guard = new ObjectContainer();\n");

	// -----------------------------------------------------------------------
	// define container for body
	fw.write("\t\tObjectContainer "+ruleName+"Body = new ObjectContainer();\n\n");
	
	// -------------------------------------------------------------------------
	// put head definition into ObjectContainer
	if (ruleType.equals("SimpagationRule")) {

	    tmp1 = (JCHRAST)h.getFirstChild();
	    current = tmp1;
	    normalHead = flattenConstraintList(tmp1, vList, iHash);
	    minusHead = flattenConstraintList((JCHRAST)tmp1.getNextSibling(), vList, iHash);

	} else {

	    head = flattenConstraintList(h, vList, iHash);

	}

	// -------------------------------------------------------------------------
	// put guard definition into ObjectContainer if there is a guard existing
	if (g != null) {
	    guard = flattenConstraintList((JCHRAST)g.getFirstChild(), vList, iHash);
         }

	// -------------------------------------------------------------------------
	// put body definition into ObjectContainer	
	body = flattenConstraintList(b, vList, iHash);
	
	// -------------------------------------------------------------------------
	// add head constraints into ObjectContainer
	if (ruleType.equals("SimpagationRule")) {

	    for (i = 0; i < normalHead.size(); i++) {
		
		conStr = (String)normalHead.get(i);
		if (isDeclaredUDConstraint(conStr))
		    fw.write("\t\t"+ruleName+"NormalHead.add("+conStr+");\n");
		else
		    throw new WrongConstraintTypeException(conStr, ruleName,
							   "maybe it should be moved to the guard");

	    }

	    for (i = 0; i < minusHead.size(); i++) {

		conStr = (String)minusHead.get(i);
		if (isDeclaredUDConstraint(conStr))
		    fw.write("\t\t"+ruleName+"MinusHead.add("+conStr+");\n");
		else
		    throw new WrongConstraintTypeException(conStr, ruleName,
							   "maybe it should be moved to the guard");
		
	    }
	    
	} else
	    for (i = 0; i < head.size(); i++) {

		conStr = (String)head.get(i);		
		if (isDeclaredUDConstraint(conStr))
		    fw.write("\t\t"+ruleName+"Head.add("+conStr+");\n");
		else
		    throw new WrongConstraintTypeException(conStr, ruleName,
							   "maybe it should be moved to the guard");

	    }

	// -------------------------------------------------------------------------
	// add guard constraints into ObjectContainer if there is a guard existing
	if (g != null) 
	    for (i = 0; i < guard.size(); i++) {

		conStr = (String)guard.get(i);		
		if (!isDeclaredUDConstraint(conStr))
		    fw.write("\t\t"+ruleName+"Guard.add("+conStr+");\n");
		else
		    throw new WrongConstraintTypeException(conStr, ruleName,
							   "maybe it should be moved out of the guard");

	    }

	// -------------------------------------------------------------------------
	// add body constraints into ObjectContainer
	for (i = 0; i < body.size(); i++)
	    fw.write("\t\t"+ruleName+"Body.add("+body.get(i)+");\n");

	// --------------------------------------------------------------------------
	// write rule definition
	fw.write("\n");
	if (guard == null) {

	    if (ruleType.equals("SimpagationRule")) {

	        fw.write("\t\t"+ruleType+" "+ruleName+" = new "+ruleType+"(\""+ruleName+"\", new SimpagationHead("+
			 ruleName+"NormalHead, "+ruleName+"MinusHead), "+ruleName+"Body);\n\n");

	    } else {

	        fw.write("\t\t"+ruleType+" "+ruleName+" = new "+ruleType+"(\""+ruleName+"\", "+ruleName+"Head, "+ruleName+"Body);\n\n");

	    }
		
	} else {

	    if (ruleType.equals("SimpagationRule")) {

	        fw.write("\t\t"+ruleType+" "+ruleName+" = new "+ruleType+"(\""+ruleName+"\", new SimpagationHead("+
			 ruleName+"NormalHead, "+ruleName+"MinusHead), "+ruleName+"Guard, "+ruleName+"Body);\n\n");

	    } else {

	        fw.write("\t\t"+ruleType+" "+ruleName+" = new "+ruleType+"(\""+ruleName+"\", "+ruleName+"Head, "+ruleName+"Guard, "+
			 ruleName+"Body);\n\n");

	    }

	}

	// --------------------------------------------------------------------------
	// add rule to ConstraintSystem
	fw.write("\t\tcs.addRule("+ruleName+");\n");
    
    }

    private void insertRule(FileWriter fw, int type, JCHRAST head, JCHRAST body, 
			    String ruleName, ObjectContainer vList, Hashtable iHash) 
	throws IOException, WrongConstraintTypeException, UnknownIdentifierException {

	insertRule(fw, type, head, null, body, ruleName, vList, iHash);

    }

    private void insertGoal(FileWriter fw, ObjectContainer constrList) throws IOException{

	fw.write("\n\t\tcs.init();\n");
	for (int i = 0; i < constrList.size(); i++)
	    fw.write("\t\tcs.addGoalConstraint("+constrList.get(i)+");\n");	   

    }
    
    /////////////////////////////////////////////////////////////////////////////////////

    /**
       this method returns an ObjectContainer with Strings of arguments
    */

    private ObjectContainer flattenConstraintList(JCHRAST cast, ObjectContainer variableList, Hashtable instanceHash) 
	throws UnknownIdentifierException {

      if (cast != null) {

	ObjectContainer left, right;
	
	if (cast.getType() == ANDOP) {
	
	    left = flattenConstraintList((JCHRAST)cast.getFirstChild(), variableList, instanceHash);
	    right = flattenConstraintList((JCHRAST)((JCHRAST)cast.getFirstChild()).getNextSibling(), variableList, instanceHash);
	    
	    for (int i=0; i<right.size(); i++)
		left.add(right.get(i));
	    
	    return left;

	} else {

	    left = new ObjectContainer();
	    left.add(constraintToString(cast, variableList, instanceHash)); // not the end
	    
	    return left;

	}
	    
      } else
	return new ObjectContainer();

    }

    private ObjectContainer flattenArgList(JCHRAST cast, ObjectContainer variableList, Hashtable instanceHash) 
	throws UnknownIdentifierException {
	
      if (cast != null) {

	ObjectContainer left, right;
	
	if (cast.getType() == COMMA) {
	    
	    left = flattenArgList((JCHRAST)cast.getFirstChild(), variableList, instanceHash);
	    right = flattenArgList((JCHRAST)((JCHRAST)cast.getFirstChild()).getNextSibling(), variableList, instanceHash);
	    
	    for (int i=0; i<right.size(); i++)
		left.add(right.get(i));
	    
	    return left;
	    
	} else { // convert a termORconstraint production into a string and add it to an ObjectContainer
	    
	    left = new ObjectContainer();
	    String s = tocToString(cast, variableList, instanceHash); 
	    left.add(s); 
	    
	    return left;
	    
	}

      } else
	return new ObjectContainer();
	
    }
  
    private ObjectContainer flattenIDList(JCHRAST cast) {
	
      if (cast != null) {

	ObjectContainer left, right;
	
	if (cast.getType() == COMMA) {
	    
	    left = flattenIDList((JCHRAST)cast.getFirstChild());
	    right = flattenIDList((JCHRAST)((JCHRAST)cast.getFirstChild()).getNextSibling());
	    
	    for (int i=0; i<right.size(); i++)
		left.add(right.get(i));
	    
	    return left;
	    
	} else { 
	    
	    left = new ObjectContainer(); 
	    left.add(cast.getText()); 
	    
	    return left;
	    
	}

      } else
	return new ObjectContainer();
	
    }

    private String constraintToString(JCHRAST cast, ObjectContainer variableList, Hashtable instanceHash) 
	throws UnknownIdentifierException {

	String result = "";
	JCHRAST tmp1, tmp2;

	if (cast.getType() == EQUALSOP) {

	    tmp1 = (JCHRAST)cast.getFirstChild();
	    current = tmp1;
	    tmp2 = (JCHRAST)tmp1.getNextSibling();

	    result = "new Equality("+tocToString(tmp1, variableList, instanceHash)+", "+
	    tocToString(tmp2, variableList, instanceHash)+")";

	} else
	    return tocToString(cast, variableList, instanceHash); 

	return result;

    }

    private String tocToString(JCHRAST cast, ObjectContainer variableList, Hashtable instanceHash) 
	throws UnknownIdentifierException {

	String rootName = cast.getText();
	String result = "", castType = null, castString = "";
	JCHRAST tmp1, tmp2;
	ObjectContainer oc;
	
	if (cast.getType() == TYPECAST) {
	    
	    castType = ((JCHRAST)cast.getFirstChild()).getText();
	    castString ="("+castType+")";
	    cast = (JCHRAST)cast.getNextSibling();
	    rootName = cast.getText();
	    
	}

	if (cast.getType() == CONSTRKW) { // cast is an object
	    
	    tmp1 = (JCHRAST)cast.getFirstChild(); // class of new Object
	    current = tmp1;
	    tmp2 = (JCHRAST)tmp1.getNextSibling(); // possible arguments	    

	    result = castString+"new "+tmp1.getText()+"("+flattenArgList(tmp2, variableList, instanceHash)+")";
	    
	} else { // cast is a constant, a constraint or a method application
	    
	    tmp1 = (JCHRAST)cast.getFirstChild();
	    current = tmp1;
	    
	    if (constraintTable.containsKey(rootName)) {
		
		ObjectContainer argList = flattenArgList(tmp1, variableList, instanceHash);
		
		if (argList.size() == ((Integer)constraintTable.get(rootName)).intValue())
		    result = "new "+rootName.toUpperCase()+"Constraint("+argList+")";
		else {

		    throw new UnknownIdentifierException(rootName+"/"+argList.size());

		}
		
	    } else if (isMethodApplication(rootName, instanceHash)) {
		
	        JMethodExtender jme;
		ObjectContainer argList = flattenArgList(tmp1, variableList, instanceHash);
	        ObjectContainer names = dotSplit2(rootName);
		String instanceClassName = (String)names.get(0);
		String methodName = (String)names.get(1);
		
		if (classList.contains(instanceClassName)) {
		    
		    jme = new JMethodExtender(path, instanceClassName, methodName, argList.size(), true);
		    result = "new "+jme.getMethodClassName()+"("+argList+")";
		    
		} else if (instanceHash.containsKey(instanceClassName)) { 
		    
		    jme = new JMethodExtender(path, (String)instanceHash.get(instanceClassName), 
					      methodName, argList.size(), false);
		    result = "new "+jme.getMethodClassName()+"(";

		    if (argList.size() > 0)
		      result += instanceClassName+", ";

		    result += argList+")";

		} else 
		    throw new UnknownIdentifierException(instanceClassName);

		if (!methodTable.containsKey(jme.getMethodClassName()))
		    methodTable.put(jme.getMethodClassName(), jme);

		return result;

	    } else if (cast.getType() == NULLKW) {

		result = castString+"new Bool(\"false\")";

	    } else if ((cast.getType() == TRUEKW) || 
		       (cast.getType() == FALSEKW)) {

		result = castString+"new Bool(\""+rootName+"\")";

	    } else if (cast.getType() == STRING_LITERAL) {

		result = castString+"new String("+rootName+")";

	    } else if (cast.getType() == CHAR_LITERAL) {

		result = castString+"new Character("+rootName+")";

	    } else if (cast.getType() == NUM_INT) {

		// result = castString+"new Integer("+rootName+")";
		result = castString+"new org.jmlspecs.jmlexec.runtime.MyInteger("+rootName+")";
		// result = castString+"new MyInteger("+rootName+")";

	    } else if (cast.getType() == NUM_FLOAT) {

		result = castString+"new Float("+rootName+")";

	    } else if (cast.getType() == CSREFERENCE) {

		result = castString+"cs";
		
	    } else {

		if (classList.contains(rootName) || variableList.contains(rootName))
		    result = rootName;
		else if (instanceHash.containsKey(rootName))
		    result = castString+rootName;
		else {
		    
		    current = cast;
		    throw new UnknownIdentifierException(rootName);

		}

	    }

	}
   
	return result;

    }

    /**
       This method tests, whether a String is a method call or not!
      */

    private boolean isMethodApplication(String id, Hashtable instanceHash) {

      String first;
      ObjectContainer oc = dotSplit2(id);

      if (oc.size() < 2)
	return false;
      else {

	first = (String)oc.get(0);

	if (classList.contains(first) || instanceHash.containsKey(first))
	    return true;
	return false;

      }
	
    }

    /**
       This method splits a given String, which contains dots, into partial Strings 
       and writes them into an ObjectContainer.
    */

    private ObjectContainer dotSplit(String s) {

      ObjectContainer oc = new ObjectContainer();
      int from = 0, to = 0;

      while(to < s.length()) {

	to = s.indexOf('.', from);

	if (to < 0)
	  to = s.length();

	oc.add(s.substring(from, to));
	from = to+1;

      }

      return oc;

    }

    private ObjectContainer dotSplit2(String s) {
      ObjectContainer oc = new ObjectContainer();
      int to = s.lastIndexOf(".");
      if (to == -1) {
        oc.add(s);
      } else {
        oc.add(s.substring(0, to));
        oc.add(s.substring(to + 1, s.length()));
      }
      return oc;
    }

    private boolean isDeclaredUDConstraint(String s) {
	
	if (getIDfromString(s).endsWith("Constraint"))
	    return true;
	return false;

    }

    private String getIDfromString(String s) {

	if (s.startsWith("new")) {

	    int parentIndex = s.indexOf('(');
	    return s.substring(4, parentIndex);

	} else
	    return s;

    }

}

