package org.jmlspecs.jmlexec.jack.compiler;

import java.io.IOException;
import java.io.FileReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.io.FileNotFoundException;
import java.lang.Runtime;
import java.lang.Process;
import java.lang.InterruptedException;

public class jchrc {

    public static final int NO_ERROR_ID = 0;
    public static final int IO_EXCEPTION_ID = 1;
    public static final int R_EXCEPTION_ID = 2;
    public static final int CNF_EXCEPTION_ID = 3;
    public static final int WCT_EXCEPTION_ID = 4;
    public static final int UI_EXCEPTION_ID = 5;
    public static final int IE_EXCEPTION_ID = 6;
    public static final int TS_EXCEPTION_ID = 7;

    public String error;

    private JCHRLexer     jchrl;
    private JCHRParser    jchrp;
    private JCHRGenerator jchrg;

    private Reader read;
    private Writer out;
    private Writer err;
    private String outdir;
    boolean quiet;

    public jchrc(Reader r, Writer o, Writer e, String od, boolean q) {

	error = null;
	jchrl = null;
	jchrp = null; 
	jchrg = null;

	read = r;
	out = o;
	err = e;

	outdir = od;
	quiet = q;

    }

    public static void printHeader() {

	System.out.println("jchrc (JCHR to JCHRCore Compiler) Version 0.0.1");

    }

    public static void printUsage() {

	jchrc.printHeader();

	System.out.println("usage: java jchrc [options] file.jchr");
	System.out.println("\t-o outputDir \t specify output directory where all output generated.");
	System.out.println("\t-quiet \t\t generate no output.");

    }

    public boolean compile() throws IOException {

	try {

	    jchrl = new JCHRLexer(read);
	    jchrp = new JCHRParser(jchrl); 
	    jchrp.setASTNodeClass("org.jmlspecs.jmlexec.jack.compiler.JCHRAST");
	    jchrp.program();
		    
	} catch (antlr.TokenStreamException e) {
		   
	    error = ""+e;
	    err.write(""+TS_EXCEPTION_ID);
	    return false;
	
	} catch(antlr.RecognitionException e) {
	
	    error = ""+e;
	    err.write(""+R_EXCEPTION_ID);
	    return false;
	    
	}

	if (!jchrp.errorFlag) { //

	    try {
			
		jchrg = new JCHRGenerator((JCHRAST)jchrp.getAST(), out, quiet); 
		jchrg.writeFile(outdir);

	    } catch(ClassNotFoundException e) {
		    
		error = "line "+jchrg.getLine()+": "+e;
		err.write(""+CNF_EXCEPTION_ID);
		return false;
			
	    } catch(IOException e) {
			
		error = "line "+jchrg.getLine()+": "+e;
		err.write(""+IO_EXCEPTION_ID);
		return false;
		
	    } catch(WrongConstraintTypeException e) {

		error = "line "+jchrg.getLine()+": "+e;
		err.write(""+WCT_EXCEPTION_ID);
		return false;

	    } catch(UnknownIdentifierException e) {

		error = "line "+jchrg.getLine()+": "+e;
		err.write(""+UI_EXCEPTION_ID);
		return false;

	    }

	    // compile generated ".java" file
	    Runtime rt = Runtime.getRuntime();
		    
	    try {

//		Process pc = rt.exec("javac "+jchrg.getOutFileName());
		boolean success = org.multijava.mjc.Main.compile(
                   new String [] {jchrg.getOutFileName()});
//		if (pc.waitFor() == 0) {
		if (success) {

		    if (!quiet)
			out.write(jchrg.getOutFileName()+" compiled! Class file generated!\n");

		} else {

		    if (!quiet)
			out.write("compilation of "+jchrg.getOutFileName()+" failed!\n");

		}

		out.flush();

	    } catch (java.io.IOException e) {
		
		error = ""+e;
		err.write(""+IO_EXCEPTION_ID);
		return false;
	
	    } 
/*
              catch (java.lang.InterruptedException e) {

		error = ""+e;
		err.write(""+IE_EXCEPTION_ID);
		return false;

	    }
*/

	    error = "";
	    err.write(NO_ERROR_ID);
	    return true;

	} else {

	    error = "compilation interrupted in line "+jchrg.current.getLine();
	    return false;

	}

    } 

    public static void main(String args[]) {

	String outdir = ".";
	String infile = null;
	boolean quiet = false;
	int argscounter = 0;
	jchrc compiler = null;

	if (args.length == 0) {

	    jchrc.printUsage();

	} else {
	    
	    while (argscounter < args.length) {

		if (args[argscounter].equals("-o")) {

		    outdir = args[++argscounter];
		    ++argscounter;

		} else if (args[argscounter].equals("-quiet")) {

		    quiet = true;
		    ++argscounter;

		} else {

		    infile = args[argscounter];
		    if (!infile.endsWith(".jchr")) {

			System.out.println("javac: invalid argument: "+infile);
			jchrc.printUsage();
			System.exit(0);

		    }
			
		    ++argscounter;

		}

	    }

	    if (infile == null)
		jchrc.printUsage();
	    else {

		if (!quiet)
		    jchrc.printHeader();

		PrintWriter out = new PrintWriter(System.out);
		PrintWriter err = new PrintWriter(System.err);

		try {

		    compiler = new jchrc(new FileReader(infile), out, err, outdir, quiet);	     

		} catch (FileNotFoundException e) {

		    System.out.println(""+e);

		}

		try {

		    if (!compiler.compile()) {

			System.out.println("Compilation failed!");
			System.out.println(compiler.error);

		    }

		} catch(IOException e) {
		  
		    System.out.println(""+e);
		    
		}
	  
	    }
	    
	}
	
    }
    
}
