package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class Equality <br>
   $Id: Equality.java,v 1.1 2008/03/05 00:08:34 wahlst Exp $
   @version 1 (30/05/1999), $Revision: 1.1 $
   @author Matthias SchmauB (last modified by $Author: wahlst $)
*/

public class Equality extends JExpression {

    //////////////////////////////
    // fields

    private boolean occurs;

    //////////////////////////////
    // constructor

    public Equality(Object left, Object right) {

	identifier = "=";

	argList = new ObjectContainer();
	argList.add(left);
	argList.add(right);

	arity = 2;
	occurs = false;

    }

    //////////////////////////////
    // methods

    public void setArguments(ObjectContainer oc) {
	
	// does nothing, because the arguments are set by the constructor

    }

    public boolean checkTypes() 
	throws WrongArgumentTypeException, UnknownConstraintException {
	
	// does nothing, because equality is "not typed"

	return true;
	
    }

    public boolean execute(boolean guard,TrailStack trailStack) {

	clearExecuteFailReason();

	Object left  = getArgument(0);
	Object right = getArgument(1);

	// If either of the values is a method invocation, the method
	// is invoked right now. Else, it would not be possible to
	// have any methods that can return variables as well as
	// values (variables are of type Object, while values are
	// normally not (i.e. Integer)

	if (left instanceof JMethod)
	{
	    try
	    {
		left=((JMethod)left).executeMethod();
	    }
	    catch (NoSuchMethodException e) {}
	    catch (VariableUnboundException e) {}
	}

	if (right instanceof JMethod)
	{
	    try
	    {
		right=((JMethod)right).executeMethod();
	    }
	    catch (NoSuchMethodException e) {}
	    catch (VariableUnboundException e) {}
	}

	boolean leftlocal = vTable.isLocalVar(left);
	boolean rightlocal = vTable.isLocalVar(right);

	//System.out.println("left="+left+", right="+right+", leftLocal="+leftlocal+", rightLocal="+rightlocal+", leftIsVar="+vTable.isVariable(left)+", leftIsBound="+vTable.isBound(left));

	if (vTable.isVariable(left)) {
	    
	    if (vTable.isBound(left)) {

		if (vTable.isVariable(right)) {

		    if (vTable.isBound(right)) {

			return equalTwoBoundVariables(left, right, guard, leftlocal, rightlocal, trailStack);

		    } else { // right is not bound

			return equalUnboundVarBoundVar(right, left, guard, leftlocal, rightlocal, trailStack);

		    }

		} else { // right is not a variable

		    return equalBoundVarNonVariable(left, right, guard, leftlocal, rightlocal, trailStack);

		}

	    } else { // left is not bound

		if (vTable.isVariable(right)) {

		    if (vTable.isBound(right)) {

			return equalUnboundVarBoundVar(left, right, guard, leftlocal, rightlocal, trailStack);

		    } else { // right is not bound

			return equalTwoUnboundVariables(left, right, guard, leftlocal, rightlocal, trailStack);

		    }

		} else { // right is not a variable

		    return equalUnboundVarNonVariable(left, right, guard, leftlocal, rightlocal, trailStack);

		}

	    }

	} else { // left is not a variable

	    if (vTable.isVariable(right)) {
	
		if (vTable.isBound(right)) {

		    return equalBoundVarNonVariable(right, left, guard, leftlocal, rightlocal, trailStack);

		} else { // right is not bound
	  
		    return equalUnboundVarNonVariable(right, left, guard, leftlocal, rightlocal, trailStack);

		}

	    } else { // right is not a variable

		return equalTwoNonVariables(left, right, guard);  // c'est bon!

	    }

	}

    }
    
    //---------------------------------------------------------------------------------

    /**
       equals two bound Variables
    */

    private boolean equalTwoBoundVariables(Object left, Object right, 
					   boolean guard, 
					   boolean ll, boolean rl,
					   TrailStack trailStack) {


	// if the result of an evalValue call is a variable
	// this variable is unbound
	Object valueRight = evalValue(vTable.getBinding(right), null, guard);
	Object valueLeft = evalValue(vTable.getBinding(left), valueRight, guard); 

	boolean leftIsVariable=vTable.isVariable(valueLeft);
	boolean rightIsVariable=vTable.isVariable(valueRight);
	
	if (leftIsVariable && rightIsVariable) {
	
	    if (occurs) {	// X = X
	  
		return true;

	    } else {		// X = Y

		return equalTwoUnboundVariables(valueLeft, valueRight, guard, ll, rl, trailStack);

	    }

	} else if (leftIsVariable || rightIsVariable) {

	    if (leftIsVariable)	// X = val
		return equalUnboundVarBoundVar(valueLeft, right, guard, ll, rl, trailStack);
	    else		// val = X
		return equalUnboundVarBoundVar(valueRight, left, guard, rl, ll, trailStack);

	} else {		// val1 = val2
	    
//	    return valueLeft.equals(valueRight);

/* modified by T. Wahls 8/3/2004, 9/14/2007
*/
         if (valueLeft instanceof org.jmlspecs.jmlexec.runtime.MyPrimitive &&
             valueRight instanceof org.jmlspecs.jmlexec.runtime.MyPrimitive) {
           return valueLeft.equals(valueRight);
         } else if (valueLeft instanceof org.jmlspecs.jmlexec.runtime.MyArray &&
              valueRight instanceof org.jmlspecs.jmlexec.runtime.MyArray) {
            return ((org.jmlspecs.jmlexec.runtime.MyArray) valueLeft).equals(
            (org.jmlspecs.jmlexec.runtime.MyArray) valueRight, guard);
          } else {
            return valueLeft == valueRight;
          }

	}    
	
    }

    /**
       This method binds two unbound variables to eachother.
    */

    private boolean equalTwoUnboundVariables(Object leftVar, Object rightVar, 
					     boolean guard, 
					     boolean ll, boolean rl,
					     TrailStack trailStack) {

	// Left variable must be a local
	if (guard && !ll) 
	{
	    setExecuteFailReason("can't bind a non-local variable in a guard");
	    return false;	// ??? TODO (is this correct? document!)
	}

	// If the variables are the same, we are finished
	if (leftVar.equals(rightVar))
	    return true;
	    
/* removed by T. Wahls 7/15/2004
	// Type mismatch -> fail
	if (!vTable.getVariableClass(leftVar).equals(vTable.getVariableClass(rightVar)))
	{
	    setExecuteFailReason("two unbound variables, type mismatch ("+vTable.getVariableClass(leftVar)+" vs "+vTable.getVariableClass(rightVar)+")");
	    return false;
	}
*/

	// Not in guard -> bind variable
	if (guard)
	{
	    setExecuteFailReason("can't bind variable in guard");
	    return false;
	}

	vTable.bind(leftVar, rightVar, trailStack);
	return true;
    }

    /**

     */

    private boolean equalTwoNonVariables(Object left, Object right, boolean guard) {
	
	Object leftValue = evalValue(left, null, guard);
	Object rightValue = evalValue(right, null, guard);

/* modified by T. Wahls 7/15/2004, 9/14/2007
   NOTE: this change causes the following examples to fail: fd, schur
	return leftValue.equals(rightValue);
*/
        if (leftValue instanceof org.jmlspecs.jmlexec.runtime.MyPrimitive &&
            rightValue instanceof org.jmlspecs.jmlexec.runtime.MyPrimitive) {
          return leftValue.equals(rightValue);
        } else if (leftValue instanceof org.jmlspecs.jmlexec.runtime.MyArray &&
             rightValue instanceof org.jmlspecs.jmlexec.runtime.MyArray) {
          return ((org.jmlspecs.jmlexec.runtime.MyArray) leftValue).equals(
            (org.jmlspecs.jmlexec.runtime.MyArray) rightValue, guard);
        } else {
          return leftValue == rightValue;
/*
            if (leftValue == rightValue) {
              return true;
            } else {
              System.out.println("NOT EQUAL: " + leftValue + ", " + rightValue);
              return false;
            }
*/
        }

    }
    
    //---------------------------------------------------------------------------------

    /**
       This methd equals an unbound variable to the value of a bound variable
    */

    private boolean equalBoundVarNonVariable(Object boundVar, Object value, 
					     boolean guard, 
					     boolean ll, boolean rl,
					     TrailStack trailStack) {

	Object varValue = evalValue(vTable.getBinding(boundVar), null, guard);

	if (vTable.isVariable(varValue))
	    return equalUnboundVarNonVariable(varValue, value, guard, ll, rl, trailStack);
	else
	    return equalTwoNonVariables(varValue, value, guard);

    }

    private boolean equalUnboundVarBoundVar(Object unboundVar, Object boundVar,
					    boolean guard, 
					    boolean ll, boolean rl,
					    TrailStack trailStack) {
    
	Object value = evalValue(vTable.getBinding(boundVar), unboundVar, guard);
	
	if (occurs) {

	    return true;

	} else {

	    if (vTable.isVariable(value))
		return equalTwoUnboundVariables(unboundVar, value, guard, ll, rl, trailStack);
	    else
		return equalUnboundVarNonVariable(unboundVar, value, guard, ll, rl, trailStack);

	}

    }

    private boolean equalUnboundVarNonVariable(Object unboundVar, Object value,
					       boolean guard, 
					       boolean ll, boolean rl,
					       TrailStack trailStack) {
    
	value = evalValue(value, null, guard);

	// constraints with variable arguments must not be assigned to
	// variables
	if (value instanceof Constraint)
	{
	    setExecuteFailReason("can't bind a constraint to a variable");
	    return false;
	}
	
	if (guard && !ll)
	{
	    setExecuteFailReason("can't bind a non-local variable in a guard");
	    return false;
	}

	if (value == null)
	{
	    setExecuteFailReason("can't bind a variable to a null value");
	    return false;
	}

/* removed by T. Wahls 7/15/2004
	// only do the binding, if the classes fit togehter
	// (JCHRCore is typed!)  (XXX TODO: funny, further above
	// it was stated that equality is not typed)
	if (!vTable.getVariableClass(unboundVar).equals(value.getClass()))
	{
	    setExecuteFailReason("unbound variable, type mismatch ("+vTable.getVariableClass(unboundVar)+" vs "+value.getClass()+")");
	    return false;
	}
*/

	// if the equality constraint is part of a guard execution
	// the binding will cancelled after the execution
	//if (guard)
	// vTable.addToBlock(unboundVar);
	    
	vTable.bind(unboundVar, value, trailStack);		
	return true;
    }

    //---------------------------------------------------------------------------------
    
    /* 
       expects non variables (integers, methods) or variables for the
       argument value.  The argument testVar may be null. If not, test
       var is a variable object, which is to be tested whether it is
       contained in a variable chain started by the bound variable in
       value.  
    */

    private Object evalValue(Object value, Object testVar, boolean guard) {
	
	if (value instanceof JMethod) {
	    
	  JMethod method = (JMethod)value;

	  try {

	      // XXX TODO PROBLEM: no changes to the system state
	      // (except through ConstraintSystem) must be done here,
	      // since we cannot pass TrailStack...
	      return method.executeMethod();

	  } catch (VariableUnboundException e) {

	      if (!guard)
		  System.out.println(""+e);

	  } catch (NoSuchMethodException e) {
	      
	      System.out.println(e+":\nCould not find method "+method.instanceName()+"."+method.getIdentifier());

	  }
	   

	} else if (vTable.isVariable(value)) {

	    occurs = false;
	    
	    if ((testVar != null) && (value.equals(testVar))) {
		
		occurs = true;
		return value;
		
	    }
	    
	    if (vTable.isBound(value)) {

		Object nextBinding = vTable.getBinding(value);
		return evalValue(nextBinding, testVar, guard);
		    
	    } 	 
	    
	} 

	return value;
	
    }
    
    public String toString() {

	// This code here never makes sense (returns "0=0", vor example)

// 	Object left  = getArgument(0);
// 	Object right = getArgument(1);

// 	if (vTable.isVariable(left)) {

// 	    if (vTable.isBound(left))
// 		left = vTable.getBinding(left);
// 	    else 
// 		left = vTable.getVariableName(left);

// 	}	

// 	if (vTable.isVariable(right)) {

// 	    if (vTable.isBound(right))
// 		right = vTable.getBinding(right);
// 	    else 
// 		right= vTable.getVariableName(right);

// 	}	

// 	return (""+left+" = "+right);

	return toVariableString();
    }


	/**
	 * if an assignment contains a variable, give its name 
	 * instead of eventual bindings
	 *
	 * @return a String in form of 'left = right'
	 */
    public String toVariableString() {

		Object left  = getArgument(0);
		Object right = getArgument(1);

		if(vTable.isVariable(left)) {
			left = vTable.getVariableName(left);
		}//end of if

		if(vTable.isVariable(right)) {
			right= vTable.getVariableName(right);
		}//end of if

		return new String(Util.JMLEString(left) + " = " + Util.JMLEString(right));

    }//end of toVariableString

    
    public Object clone() {
	
	Object left  = getArgument(0);
	Object right = getArgument(1);

	if (vTable.isVariable(left)) {
	    
	    if (vTable.isBound(left)){
		
		left = vTable.getBinding(left);
	
	    } else {

		left = new Object(); 
		// XXX trailStack -> we have a problem here perhaps?!
		vTable.declare(left, vTable.getVariableClass(getArgument(0)).getName());
		
	    }
	    
	} /* removed by T. Wahls 7/15/2004
           else
	    left = ObjectContainer.generalClone(left);
          */

	if (vTable.isVariable(right)) {
	    
	    if (vTable.isBound(right)) {
		
		right = vTable.getBinding(right);		
		
	    } else {
		
		right = new Object(); 
		// XXX trailStack -> we have a problem here perhaps?!	       
		vTable.declare(right, vTable.getVariableClass(getArgument(0)).getName());
		
	    }
	}
	    
          /* removed by T. Wahls 7/15/2004
          else
	    right = ObjectContainer.generalClone(right);
          */
	
	Equality eq = new Equality(left, right);
	eq.setVTable(vTable);
	
	return eq;
	
    }

    private Object unchain(Object var) {

	boolean goon = true;
	while(goon) {
	    
	    if ((vTable.isVariable(var)) && (vTable.isBound(var)))
		var = vTable.getBinding(var);
	    else
		goon = false;
	    
	}
	
	return var;
	
    }

}
