package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class JMethod
   @version 1 (29/05/1999)
   @author Matthias SchmauB
*/

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
   This class provides a wrapper for JAVA methods. All java types and classes 
   can be used as arguments. If an argument is just an JAVA standard object 
   the variable table is searched for a binding, which is given to the method
   instead of the pure object (of course only in case of existance). 
*/

public class JMethod extends BuiltInConstraint {
    
    //////////////////////////////////////
    // fields
    
    // reflextion method object
    private Method invokeableMethod = null;

    // This fields contains an instance of the class, which contains
    // the method.
    private Object instance;

    private Class methodClass;

    // all methods of the encapsulated class with the same identifier
    // as the constraint
    private ObjectContainer classMethods=new ObjectContainer();	

    private int arity;

    //////////////////////////////////////
    // constructor

    public JMethod(String id, Object o) {

	instance = o;
	methodClass = instance.getClass();
	identifier = id;
	argList = new ObjectContainer();
	arity=0;

	initMethods();
    }

    public JMethod(String id, String cl) {
	
	try {
	    
	    methodClass = Class.forName(cl);
	
	} catch(ClassNotFoundException e) {
	    
	    System.out.println(""+e);
	    
	}
	
	instance = null;
	identifier = id;
	argList = new ObjectContainer();
	arity=0;
	
	initMethods();
    }

    // -> clone
    private JMethod(String id, Object o,
		   ObjectContainer classMethods,Method invokeableMethod) {

	instance = o;
	methodClass = instance.getClass();
	identifier = id;
	argList = new ObjectContainer();
	arity=0;

	this.classMethods=classMethods;
	this.invokeableMethod=invokeableMethod;
    }

    // -> clone
    private JMethod(String id, String cl,
		   ObjectContainer classMethods,Method invokeableMethod) {
	
	try {
	    
	    methodClass = Class.forName(cl);
	
	} catch(ClassNotFoundException e) {
	    
	    System.out.println(""+e);
	    
	}
	
	instance = null;
	identifier = id;
	argList = new ObjectContainer();
	arity=0;
	
	this.classMethods=classMethods;
	this.invokeableMethod=invokeableMethod;
    }

    ///////////////////////////////////////
    // privates

    private void initMethods() throws SecurityException {

	classMethods = new ObjectContainer();
	Method[] methods = methodClass.getMethods();

	final int size=methods.length;
	final String id=getIdentifier();
	for (int i=0; i<size; i++)
	    if (methods[i].getName().equals(id))
		classMethods.add(methods[i]);		

	if (classMethods.size() == 1) {
	    invokeableMethod = (Method)classMethods.get(0);
        }
    }

    private boolean matchingParameterTypes(Class methodClasses[], Class invokeClasses[]) {

	if (methodClasses.length == invokeClasses.length) {

	    final int size=methodClasses.length;
	    for (int i=0; i<size; i++) {

/*
		if (!((methodClasses[i].equals(invokeClasses[i])) || 
		      (methodClasses[i].getSuperclass() == null)))  // .getName().equals("java.lang.Object")
*/
                if (!(methodClasses[i].isAssignableFrom(invokeClasses[i])))
		    return false;

	    }

	    return true;

	} else
	    return false;
	
    }

    private Method getMatchingMethod(String name, Class classes[], Method methods[]) throws NoSuchMethodException {

	Method m = null;

	final int size=methods.length;
	for (int i=0; i<size; i++) {

	    if ((methods[i].getName().equals(name)) && 
		(matchingParameterTypes(methods[i].getParameterTypes(), classes)))
		return methods[i];		

	}

	if (m == null) {
	    throw new NoSuchMethodException("Method "+m+" was not found!");
        }

	return m;

    }

    //////////////////////////////////////
    // methods

    public int getArity() {

	return arity;

    }

    public String instanceName() {
	
	if (instance == null) 
	    return methodClass.getName();
	else
	    return instance.toString();
	
    }

    public boolean checkTypes() throws WrongArgumentTypeException, UnknownConstraintException {

	// testing is done at execution time, when the reflection api can't find a methdo with wrong types
	return true;

    }

    public void setArguments(ObjectContainer oc) {

	argList = oc;
	arity=oc.size();
	
    }

    public boolean execute(boolean guard, TrailStack trailStack) {
	
	clearExecuteFailReason();

	Object result;

	try {
	    
	    // XXX TODO PROBLEM: assure that this does not change the
	    // system state (except through calls of ConstraintSystem
	    // methods)
	    result = executeMethod();

	    if (result instanceof Boolean)
	    {
		boolean rc=((Boolean)result).booleanValue();
		if (!rc) 
		    setExecuteFailReason("boolean method returned false");
		return rc;
	    }
	    else
	    {

                System.out.println( "ERROR: method " + instanceName() +
                  "." + getIdentifier() + " has non-boolean return type!");

		setExecuteFailReason("method has a non-boolean return type");
		return false;
	    }

	      
	} catch(NoSuchMethodException e) {

	    if (!guard)
		System.out.println(e+":\nCould not find method "+instanceName()+"."+getIdentifier());
	    setExecuteFailReason("could not find method "+instanceName()+"."+getIdentifier());
	    return false;
	    
	} catch (VariableUnboundException e) {

	    if (!guard)
		System.out.println(""+e);
	    setExecuteFailReason(""+e);
	    return false;
	    
	} 

    }

    /**
       This method calls the wrapped method. It is used in the overwritten Method execute and in cases,
       when other return values than true/false are expected.
    */

     public Object executeMethod() throws NoSuchMethodException, VariableUnboundException {

	// class method contains method to invoke method ;-)
	Class c, argsClasses[];
	Object argsObjects[], returnObject = new Object();

	// create class array and object array of method's arguments
	argsObjects = new Object[arity];

	// get class of the method's object
	if (instance == null)
	    c = methodClass;
	else 
	    c = instance.getClass();
	
	// create class array and object array of method's arguments
	if (arity > 0)
	    argsClasses = new Class[arity];
	else
	    argsClasses = null;
	
//  System.out.println("Calling " + getIdentifier());
	for (int i=0; i<arity; i++) {
	    
	    argsObjects[i] = getArgument(i);
	    
//  System.out.println(argsObjects[i]);
	    if (vTable.isVariable(argsObjects[i])) {
		
// System.out.println(argsObjects[i] + " is a variable.");
//  modified by T. Wahls 8/5/2004
/*
		Object tmp = vTable.unchain(argsObjects[i]);
System.out.println("tmp is " + tmp);
		} else if (vTable.isBound(tmp))
		    argsObjects[i] = vTable.getBinding(tmp);
		else 
		    throw (new VariableUnboundException(this));
*/
                Object tmp = argsObjects[i];
                Object back = tmp;
 //System.out.println("starting loop with tmp " + tmp);
                while (tmp != null && vTable.isVariable(tmp) &&
                       vTable.isBound(tmp)) {
 //System.out.println("tmp is " + tmp);
                       tmp = vTable.getBinding(tmp);
                }
//System.out.println("after loop tmp is " + tmp);
                if (tmp == null) tmp = org.jmlspecs.jmlexec.runtime.Null.NullObj; // shouldn't be necessary
                if (vTable.isVariable(tmp) && !vTable.isBound(tmp)) {
// added by TAW 6/22/2006
/*
                  if (getIdentifier().equals("getVar")) {
	            argsObjects[i] = tmp;
                  } else {
*/
// end addition
		    throw (new VariableUnboundException(this));
//                  }
                } else {
	            argsObjects[i] = tmp;
                }
		
	    }
	    
	    argsClasses[i] = argsObjects[i].getClass();
	    
	}

// START - need to add this back in?

        Method invokeMethod = null;
	if (invokeableMethod == null) {

	    Method ms[] = c.getMethods();
	    // invokeableMethod = getMatchingMethod(getIdentifier(), argsClasses, ms); 
	    invokeMethod = getMatchingMethod(getIdentifier(), argsClasses, ms); 

	} else {
          invokeMethod = invokeableMethod;
        }

	try {
	    
	    // invoke method
	    if (instance == null) {
 /*
System.out.println("method: " + invokeMethod);
//System.out.println(classMethods);
System.out.print("args: ");
for (int i = 0; i < argsObjects.length; i++) {
//  System.out.println(argsObjects[i]);
  System.out.println(argsObjects[i].getClass());
}
System.out.println();
  */
		// returnObject = invokeableMethod.invoke(methodClass, argsObjects);
		returnObject = invokeMethod.invoke(methodClass, argsObjects);
	    } else

		// returnObject = invokeableMethod.invoke(instance, argsObjects);
		returnObject = invokeMethod.invoke(instance, argsObjects);
	    
	} catch (IllegalAccessException e) {
	    
	    System.out.println("IllegalAccessException while calling method "+identifier);
	    
	} catch (InvocationTargetException e) {

            if (e.getTargetException() instanceof RuntimeException) {
              throw (RuntimeException) e.getTargetException();
            } else {
	    
              throw new org.jmlspecs.jmlexec.runtime.ExceptionException(
                e.getTargetException());
/*
	    System.out.println("InvocationTargetException "+e.getTargetException()+" while calling method "+identifier+" (instance="+instance+", methodClass="+methodClass+", arity="+arity+")");
	    e.getTargetException().printStackTrace();
*/
            }
	    
	}

	return returnObject;
	
    }
    
    public Object clone() {
	
	ObjectContainer args = new ObjectContainer();
	Object o;
	
	final int size=arity;
	for (int i = 0; i < size; i++) {
	    
	    o = getArgument(i);
	    
	    if ((vTable.isVariable(o)) && (vTable.isBound(o))) {
		
		o = vTable.getBinding(o);
		
		boolean goon = true;
		while(goon) {
		    
		    if ((vTable.isVariable(o)) && (vTable.isBound(o)))
			o = vTable.getBinding(o);
		    else
			goon = false;
		    
		}

/*   modified by T. Wahls 7/15/2004
		if (vTable.isVariable(o))
		    args.add(o);
		else 
		    args.add(ObjectContainer.generalClone(o));
*/
                args.add(o);
		
	    } else if (vTable.isVariable(o)) {
			      
		Object v = new Object(); 
		vTable.declare(v, vTable.getVariableClass(o).getName());		
		args.add(v);
		
	    } else
/*   modified by T. Wahls 7/15/2004
		args.add(ObjectContainer.generalClone(o));
*/
                args.add(o);
	    
	}
	
	JMethod tmp;
 	if (instance == null)
 	    tmp = new JMethod(identifier, methodClass.getName(),classMethods,invokeableMethod);
 	else 
 	    tmp = new JMethod(identifier, instance,classMethods,invokeableMethod);
	
	tmp.setVTable(vTable);
	tmp.setArguments(args);
	
	return tmp;
	
    }    

  public String toString() {

    String s;
    boolean first = true;


    if (instance == null)
      s = ""+methodClass.getName()+"."+identifier+"(";
    else
      s = ""+instance.getClass().getName()+"."+identifier+"(";

    if (arity > 0) {

      for (int i = 0; i < arity; i++) {

	if (first)
	  first = false;
	else 
	  s += ", ";

	Object o = getArgument(i);
	if (vTable.isVariable(o)) {

	    if (vTable.isBound(o)) {
                Object binding = vTable.getBinding(o);
                s += Util.JMLEString(binding);
	    } else 
		s+= vTable.getVariableName(o);

	} else
	  s += ""+Util.JMLEString(getArgument(i));

      }

    }

    return s+")";

  }

}
