package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class ObjectContainer <br>
   $Id: ObjectContainer.java,v 1.1 2008/03/05 00:08:34 wahlst Exp $
   @version 1 (26/05/1999), $Revision: 1.1 $
   @author Matthias Schmau (last modified by $Author: wahlst $)
*/

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Enumeration;
 import java.util.Hashtable;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;


/**
   The class ObjectContainer provides a data structure to store any
   objects dynamically.  In it's functionality, it is similar to the
   class java.util.Vector, but it has some useful additions. Moreover
   the class ObjectContainer is a wrapper for future data strucures of
   Java, which are now easily to integrate into JCHR. 
   @see java.util.Vector 
*/

public class ObjectContainer {

    //////////////////////////////////////
    // fields
    
    private ArrayList objectList;
    
    //////////////////////////////////////
    // constructors

    public ObjectContainer() {
	
		objectList = new ArrayList();
	
    }

    public ObjectContainer(int c) {
      
		objectList = new ArrayList(c);

    }

    // Empties the enumeration
    public ObjectContainer(Enumeration v)
    {
        objectList=new ArrayList();
        while (v.hasMoreElements())
        {
            objectList.add(v.nextElement());
        }
    }

    public ObjectContainer(Iterator v)
    {
        objectList=new ArrayList();
        while (v.hasNext())
        {
            objectList.add(v.next());
        }
    }
    
    //////////////////////////////////////
    // methods


    public void push(Object o)
    {
	add(o);
    }

    public Object pop()
    {
	int s=size();
	Object o=get(s-1);
	remove(s-1);
	return o;
    }

    /** 
        empties the collection
     */

    public void clear()
    {
        objectList.clear(); // .clear() is new in JDK 1.2 
    }

    /**
       returns the first element of this collection
    */

    public Object first() {

		return get(0);

    }

    /** 
	returns the rest of this collection as an ObjectContainer object
    */

    public ObjectContainer rest() {

		ObjectContainer tmp = new ObjectContainer();

		final int size=objectList.size();
		for (int i = 1; i < size; i++)
		    tmp.add(get(i));

		return tmp;

    }

    /** 
	adds a new object at the end of this collection
    */

    public void add(Object o) {

// modified by T. Wahls 6/7/2004
//              objectList.addElement(o);
      if (o == null) {
        objectList.add(org.jmlspecs.jmlexec.runtime.Null.NullObj);
      } else {
        objectList.add(o);
      }

    }

    /**
       replaces the object at position <b>i</b> by the object <b>o</b>
     */

    public void setAt(Object o, int i) {
	
		objectList.set(i, o);

    }

    /**
       inserts the object <b>o</b> before position <b>i</b> 
     */

    public void insertAt(Object o, int i) {

		objectList.add(i, o);
	
    }

    /**
       returns the object at the position <b>i</b>
     */
    
    public Object get(int i) {

		return objectList.get(i);

    }

    /** 
	returns the index of <b>o</b>
    */

    public int indexOf(Object o)
    {
	return objectList.indexOf(o);
    }

    /**
       removes a certain object of the collection
    */

    public void remove(Object o) {

		objectList.remove(o);

    }

    /**
       removes a certain object of the collection
    */

    public void remove(int i) {

		objectList.remove(i);

    }

    /**
       removes all elements of the collection
    */

    public void removeAll() {

		objectList.clear();
	
    }

    /**
       tests, whether the collection contains the object <b>o</b>, or not
     */

    public boolean contains(Object o) {

/*
		return objectList.contains(o);
*/
       Iterator it = objectList.iterator();
       while (it.hasNext()) {
          if (it.next().equals(o)) return true;
       }
       return false;
   
    }

    /**
       returns an object of the class Enumeration of this collection
    */

/*
    public Enumeration elements() {
      
		return objectList.elements();

    }
*/
    public Iterator iterator() {
      return objectList.iterator();
    }

    /**
       returns the number of elements of this collection
    */

    public int size() {

		return objectList.size();

    }

    /**
       returns an intersection of the current and the given ObjectContainer.
    */

    public ObjectContainer intersection(ObjectContainer oc) {
	
		ObjectContainer result = new ObjectContainer();
		ObjectContainer runthrough, compare;
	
		if (size() < oc.size()){
	    
			runthrough = this;
			compare = oc;
	    
		} else {
	    
			runthrough = oc;
			compare = this;
	    
		}      


		// XXX this can be improved from O(n^2) to O(nlogn)
		// with Hashtable (only for large ObjectContainers)
		final int size=runthrough.size();
		for (int i = 0; i < size; i++) {
		    
		    Object o=runthrough.get(i);
		    if (compare.contains(o))
			result.add(o);
			
		}
	
		return result;
	
    }

    /**
       returns a union of the current and the given ObjectContainer
    */
    
    public ObjectContainer union(ObjectContainer oc) {
	
		ObjectContainer runthrough, compare;
		ObjectContainer result = new ObjectContainer();
	
		if (size() < oc.size()){
	    
			runthrough = this;
			compare = oc;
	    
		} else {
	    
			runthrough = oc;
			compare = this;
	    
		}      
	
		result = (ObjectContainer)compare.clone();
	
		// XXX this can be improved from O(n^2) to O(nlogn)
		// with Hashtable (only for large ObjectContainers)
		final int size=runthrough.size();		
		for (int i = 0; i < size; i++) {
	    
		    Object o=runthrough.get(i);
		    if (!compare.contains(o))
			result.add(o);
	    
		}
	
		return result;
	
    }

    /**
       concatenates an ObjectContainer to the current
    */
    
    public void concatenate(ObjectContainer oc) {

	final int size=oc.size();
	for (int i=0; i<size; i++)
	    add(oc.get(i));

    }	      
		
    /**
       returns the list of elements without separating commas;
       if the ObjectContainer contains the Objects ("a1", "a2", "a4"), this method returns "a1a2a4".
    */

    public String toRawString() {
	
	String s = "";
	
	final int size=objectList.size();
	for (int i=0; i < size; i++)
	    s += objectList.get(i).toString();
	
	return s;
	
    }
    
    /**
       overrides the standard toString method
    */

    public String toString() {
	
	String s = "";
	
	final int size=objectList.size();
	if (size > 0)
	    s = objectList.get(0).toString();
	
	for (int i=1; i<size; i++)
	    s += ", "+objectList.get(i);
	
	return s;
	
    }
    
    /**
       standard cloning of the ObjectContainer without the containing elements
     */

    public Object clone() {
	
	ObjectContainer oc = new ObjectContainer();
	
	final int size=objectList.size();
	for (int i = 0; i < size; i++) 
	    oc.add(objectList.get(i));
	
	return oc;
	
    }

    /** 
	also clones the contents of the ObjectContainer;
	requires the elements to be constraints
    */
    
    public ObjectContainer constraintDeepCopy() {
	
	ObjectContainer oc = new ObjectContainer();

	final int size=objectList.size();
	for (int i = 0; i < size; i++) 
	    oc.add(((Constraint)objectList.get(i)).clone());

	return oc;

    }
    
    /**
       clones arbitrary objects and wrappers for simple Java types
     */

    public static Object generalClone(Object o) {
	
	if (o instanceof Boolean)
	    return new Boolean(((Boolean)o).booleanValue());
	else if (o instanceof Byte)
	    return new Byte(((Byte)o).byteValue());
	else if (o instanceof Character)
	    return new Character(((Character)o).charValue());
	else if (o instanceof Double)
	    return new Double(((Double)o).doubleValue());
	else if (o instanceof Float)
	    return new Float(((Float)o).floatValue());
	else if (o instanceof Integer)
	    return new Integer(((Integer)o).intValue());
	else if (o instanceof Long)
	    return new Long(((Long)o).longValue());
	else if (o instanceof Short)
	    return new Short(((Short)o).shortValue());
	else if (o instanceof String)
	    return new String((String)o);
	else if (o instanceof JMethod)
	    return ((JMethod)o).clone();
	else {
	    
	    Object retObject = new Object();
	    
	    try {
		
		retObject = callClone(o);
		
	    } catch(NoSuchMethodException e) {
		

// 		System.out.println("Could not find the method clone in in class "+o.getClass().getName());
// 		System.out.println("Tried to clone object: "+o);

		throw new RuntimeException("Could not find the method clone in in class "+o.getClass().getName()+". Tried to clone object: "+o);

	    }	
	    
			return retObject;
			
	}
	
    }
    
    /**
       calls the method <b>clone</b> of an arbitrary object
     */

    public static Object callClone(Object o) throws NoSuchMethodException {
	
	Method m;
	Class c, argsClasses[];
	Object argsObjects[], returnObject = new Object();
	
	c = o.getClass();
	
	argsClasses = new Class[0];
	argsObjects = new Object[0];
	
	m = c.getMethod("clone", argsClasses);
	
	try {
	    
	    returnObject = m.invoke(o, argsObjects);
	    
	} catch (IllegalAccessException e) {
	    
	    System.out.println("IllegalAccessException while calling method "+o.getClass().getName()+".clone()");
	    
	} catch (InvocationTargetException e) {
	    
	    System.out.println("InvocationTargetException while calling method "+o.getClass().getName()+".clone()");
	    
	}
	
	return returnObject;
	
    }
    

    /**
     * get the difference between this and given ObjectContainer
     *
     * @return this without oc
     */

    public ObjectContainer without(ObjectContainer oc) {
	
	ObjectContainer result = new ObjectContainer();
	
	int i, len = size();
	for(i=0; i<len; i++) {
	    Object o=get(i);
	    if(!oc.contains(o))
		result.add(o);
	}//end of for
	
	return result;
    }//end of without
    
}//end of class ObjectContainer
