package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class PathNode
   @version 1 (02/06/1999)
   @author Matthias SchmauB
*/

import java.util.Hashtable;
import java.util.Enumeration;

/**
   This class provides a data structure to store and find user defined
   constraints in an efficient way.  For example the user defined
   constraint leq(7, X) can be indentfied by the following paths:

   [0,leq]
   [0,leq, 1, 7]
   [0,leq, 2, *]

   The last path means, that there is a user defined constraint
   existing, with the identifier "leq" (at position 0) and with a
   variable as the second argument.

*/

public class PathNode {
    
    //////////////////////////////////////////
    // fields
    
    protected Hashtable leafSons;
    protected Hashtable nodeSons;

    //////////////////////////////////////////
    // constructors 

    public PathNode() {

	leafSons = new Hashtable();
	nodeSons = new Hashtable();
 
    }

    //////////////////////////////////////////
    // methods

    public void setLeafSons(Hashtable ls) {

	leafSons = ls;

    }

    public void setNodeSons(Hashtable ns) {

	nodeSons = ns;

    }

    /**
       inserts a user defined constraint into the path index. The
       method uses a path p to store c in the correct position.
    */
    
    public void addByPath(Path p, UDConstraint c) {

	PathNode pn;
	Path localPath = new Path(p.pos, p.id, null);
	ObjectContainer oc;

	if (p.next == null) { // insert into leafSons

	    oc = (ObjectContainer)leafSons.get(localPath);
	    if (oc != null) {

		if (!oc.contains(c))
		    oc.add(c);

	    } else {

		oc = new ObjectContainer();
		oc.add(c);
		leafSons.put(localPath, oc);

	    }

	} else { // insert into nodeSons
	    
	    pn = (PathNode)nodeSons.get(localPath);
	    if (pn != null) {
      
		pn.addByPath(p.next, c);

	    } else {

		pn = new PathNode();
		pn.addByPath(p.next, c);
		nodeSons.put(localPath, pn);

	    }
	    
	}
	
    }

    public void removeByPath(Path p, UDConstraint c) {

	Path localPath = new Path(p.pos, p.id, null);

	if (p.next == null) { // remove from leafSons

	    ObjectContainer oc = (ObjectContainer)leafSons.get(localPath);
	    if (oc != null) {

		oc.remove(c);
		if (oc.size() == 0) {
		    leafSons.remove(localPath);
                }
	    }

	} else {  // remove from nodeSons
	    
	    PathNode pn = (PathNode)nodeSons.get(localPath);
	    if (pn != null) {
		
		pn.removeByPath(p.next, c);
		if (pn.size() == 0)
		    nodeSons.remove(localPath);
		
	    }
	    
	}
	
    }

    /**
       returns an ObjectContainer which contains an union of 
       the leaf ObjectContainers of the index

       may return null,
    */

/* START - start here if working on using == rather than equals for
   comparing variables used in constraints (for finding partner constraints) */
    
    public ObjectContainer getByPath(Path p) {
	
	Path localPath = new Path(p.pos, p.id, null);
	
	if (p.next == null) {
	    
	    return (ObjectContainer)leafSons.get(localPath);
	    
	} else {
	    
	    return ((PathNode)nodeSons.get(localPath)).getByPath(p.next);
	    
	}
	
    }
    
    public ObjectContainer getLeafUnion() {
	
	Object o;
	PathNode pn;
	ObjectContainer oc = new ObjectContainer();
	Enumeration enuSons;

	//----------------------------------
	
	enuSons = leafSons.elements();
	
	while (enuSons.hasMoreElements())
	    oc = oc.union((ObjectContainer)enuSons.nextElement());

	enuSons = nodeSons.elements();

	while(enuSons.hasMoreElements())
	    oc = oc.union(((PathNode)enuSons.nextElement()).getLeafUnion());
	
	//----------------------------------

	return oc;
	
    }

    public int size() {
	
	return (leafSons.size()+nodeSons.size());

    }

    public String toString() {
	
	return  ""+getLeafUnion();
	
    }

    public Object clone() {

	PathNode pn = new PathNode();

	Enumeration lsenu = leafSons.keys();
	Hashtable ls = new Hashtable();

	while (lsenu.hasMoreElements()) {

	    Path p =(Path)lsenu.nextElement();
 	    ls.put(p, ((ObjectContainer)leafSons.get(p)).clone());
	    
	    // or better this here?       XXX TODO
	    // 	    ls.put(p, ((ObjectContainer)leafSons.get(p)).constraintDeepCopy());

	}

	pn.setLeafSons(ls);
	
	Enumeration nsenu = nodeSons.keys();
	Hashtable ns = new Hashtable();

	while (nsenu.hasMoreElements()) {

	    Path p =(Path)nsenu.nextElement();
	    ns.put(p, ((PathNode)nodeSons.get(p)).clone());

	}

	pn.setNodeSons(ns);

	return pn;

    }

}

