package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class Rule <br>
   $Id: Rule.java,v 1.1 2008/03/05 00:08:34 wahlst Exp $
   @version 1 (06/06/1999), $Revision: 1.1 $
   @author Matthias SchmauB (last modified by $Author: wahlst $)
*/

public abstract class Rule {

    //////////////////////////////////////////
    // fields
    
    /**
	   components of a rule
    */
    
    protected String identifier;
    
    protected ObjectContainer head;
    protected ObjectContainer guard;
    protected ObjectContainer body;
    
    protected ObjectContainer firingHeadInstances; // no need to keep track in TrailStack, since reset frequently
    protected ObjectContainer currentHeadInstance;

    protected ObjectContainer globalVars;
    
    protected UDConstraint candidateConstraint;
    protected VariableTable vTable;

    /** added for debugging */
    private ObjectContainer currentHeadConstraintsInstance;
    private ObjectContainer currentBodyConstraintsInstance;

    
    //////////////////////////////////////
    // constructors
    
    public Rule(String id, ObjectContainer h, ObjectContainer g, 
		ObjectContainer b) 
    {
	
	identifier =id;
	
	head = h;
	guard = g;
	body = b;

	candidateConstraint = null;
	
	firingHeadInstances = new ObjectContainer();
	currentHeadInstance = new ObjectContainer();
	currentHeadConstraintsInstance = new ObjectContainer();
	
	for (int i=0; i<head.size(); i++) {
	    currentHeadInstance.add(new Integer(i)); // those Integers are Dummy values
	    currentHeadConstraintsInstance.add(new Integer(i));
	}//end of for

	globalVars = new ObjectContainer();
	
    }
    
    public Rule(String id, ObjectContainer h, ObjectContainer b) {
	
	this(id, h, null, b);
    
    }

    ////////////////////////////////////////////
    // methods
    
    public void extractGlobalVars(UDConstraint c) {

	final int size=c.getArity();
	for (int i=0; i<size; i++) {
	    
	    Object arg = c.getArgAt(i);
	    
	    if (vTable.isVariable(arg) && !(globalVars.contains(arg)))
		globalVars.add(arg);
	    
	}
	
    }
    
    public String getIdentifier() {
	
	return identifier;
	
    }
    
    public void setVTable(VariableTable vt) {
	
	int i;
	vTable = vt;
	
	final int hSize=head.size();
	for (i = 0; i < hSize; i++) {

	    UDConstraint c = (UDConstraint)head.get(i);
	    c.setVTable(vt);	    
	    extractGlobalVars(c);
	    
	}
	
	if (guard != null)
	{
	    final int gSize=guard.size();
	    for (i = 0; i < gSize; i++) {
		((Constraint)guard.get(i)).setVTable(vt);
            }
	}
	
	final int bSize=body.size();
	for (i = 0; i < bSize; i++)
	    ((Constraint)body.get(i)).setVTable(vt);
	
    }
    
    public void reInitRule() {
	
	firingHeadInstances = new ObjectContainer();
	currentHeadInstance = new ObjectContainer();
	currentHeadConstraintsInstance = new ObjectContainer();
	
	final int size=head.size();
	for (int i=0; i<size; i++) {
	    currentHeadInstance.add(new Integer(i)); // those Integer objects are dummies, in order
	    currentHeadConstraintsInstance.add(new Integer(i));
	    // to use setAt later on
	}//end of for
	
    }
    
    /**
       This method is used for propagation rules. When a propagation
       rules has fired with a certain combination of constraints, this
       combination is stored with this method.  
       @see Rule#hasFiredWithInstance 
    */
    
    public void addFiringInstance(ObjectContainer oc) {
	
	firingHeadInstances.add(oc.toRawString());
	
    }
    
    /**
       With this methods a combination of constraints can be tested,
       whether it has already fired with the current rule.
       @see Rule#addFiringInstance 
    */

    public boolean hasFiredWithInstance(ObjectContainer oc) {
	
	return firingHeadInstances.contains(oc.toRawString());
	
    }

    // XXX TODO this doesn't make much sense
    public ObjectContainer getMatchingHead() {
	
	return head;
	
    }
    
    /**
       unbinds all variables
    */
    
    public void unbindAllVariables(TrailStack trailStack) {
	
	unbindObjectContainer(head,trailStack);	// XXX???
	if (guard != null)
	    unbindObjectContainer(guard,trailStack); // XXX???
	unbindObjectContainer(body,trailStack);	// XXX???
	
    }
    
    private void unbindObjectContainer(ObjectContainer al,
				       TrailStack trailStack) {
	
	final int size=al.size();
	for (int i = 0; i < size; i++) {
	    
	    ((Constraint)al.get(i)).unbindAllVariables(trailStack); // XXX???
	    
	}

    }
    
    public boolean executeGuard(TrailStack trailStack,boolean trace) {
	
	// no guard at all -> same as "true"
	if (guard == null)
	    return true;
	    
	final int size=guard.size();
	for (int i = 0; i < size; i++) 
	{       	
	    BuiltInConstraint builtin = (BuiltInConstraint)guard.get(i);

	    if (!builtin.execute(true, trailStack)) 
	    {
                if (trace) System.out.println("\t trying rule: " + this);
		    
		//vTable.unbindAllBlockVars();
		if (trace)
		{
		    System.out.println("\tguard failed\n"+
				       "\t  failing constraint: "+builtin+"\n"+
				       "\t  constraint type: "+builtin.getClass().getName()+"\n"+
				       "\t  failure reason: "+builtin.getExecuteFailReason());
		}
		return false;
		
	    }
	}
		
	// if (trace)
	if (trace)
	    System.out.println("\tguard succeeded");

	//vTable.unbindAllBlockVars();
        return true;  
    }
    
    /**
       Tests whether the rule fires with the currently active
       constraint and some partner constraints. If it fires, performs
       all actions necessary.
    */
    public boolean apply(UserDefinedConstraintMemory udcm, 
			 BuiltInConstraintMemory bcm, 
			 UDConstraint constraint, boolean simpRule,
			 TrailStack trailStack, boolean trace) {
	
	currentBodyConstraintsInstance = new ObjectContainer(); // Debugging

	// the variable globalVars contains all global variables of
	// the current rule

	// does not affect TrailStack

	vTable.setGlobalVars(globalVars);

	// test whether the current memories cause the current rule to
	// fire; execution of the guard is included
	
	boolean result=false;

	if (ruleFiresWith(udcm, constraint, trailStack, trace)) { // XXX TrailStack?
	    
	    // remove (if neccessary) head constraints
	    // by the way removeables are "unmarked" (by removing them ;-)
	    // removes all constraints that are marked as "removable"
	    // TrailStack is handled in there
	    if (simpRule) 
		udcm.removeConstraints(trailStack);
	    
	    // insert body constraints into memory, but execute
	    // builtins before, in order to work with them in the next
	    // turns
	    final int size=body.size();
            java.util.Map ht = new java.util.HashMap();
	    for (int i = 0; i < size; i++) {
		
		Constraint c = (Constraint)body.get(i); 

		if (c instanceof BuiltInConstraint) {
		    
		    if (!((BuiltInConstraint)c).execute(false, trailStack)) {
			
			bcm.add(new Bool("false"), c, trailStack);
			
		    } 
		    
		}
		
		Constraint cc = (Constraint)c.clone(ht);
		currentBodyConstraintsInstance.add(cc); // Debugging
		
		addGoalConstraint(udcm, bcm, cc, trailStack);
		
	    }   
	    
	    // document the firing constraint combination, so that it
	    // won't fire again
	    // no need for TrailStack
	    addFiringInstance(currentHeadInstance);
	    
	    result=true;
	    
	} 
	else
	{

	    // the rule did not fire
	    // reset all constraints which have been marked to remove
	    udcm.unsetRemoveables();

	    result = false;
	}
	
	// unbind all variables of the rule, in order to use it with
	// another constraint
	
	// we do not need to update the trailing stack here
	unbindAllVariables(null/*trailStack*/);
	
	return result;
	
    }
    
    /**
       Tests whether c is an instance of !!one!! head constraint of
       (this=)r. So r is a firing-candidate.  
    */

    private boolean ruleFiresWith(UserDefinedConstraintMemory memory, 
				  UDConstraint c,TrailStack trailStack,
				  boolean trace) {
	
	ObjectContainer head = getMatchingHead();
	UDConstraint headConstraint;
	
	// run through all head constraints
	final int size=head.size();
	for (int i = 0; i < size; i++) {
	    
	    // test whether c is an instance of current head constraint
	    headConstraint = (UDConstraint)head.get(i);
	    
	    // XXX TrailStack?

	    if (headConstraint.match(c,trailStack)) {
		
		//System.out.println("\trule is candidate!");
		//System.out.println("\tg: "+headConstraint+"; i: "+c);
		
		memory.setMatching(c);
		candidateConstraint = headConstraint;
		
		// if the head constraint should be removed from the
		// UDConstraintMemory (e.g. in simplification and
		// simpagation rules, mark the test constraint in the
		// UDConstraintMemory
		if (headConstraint.isToRemove())  
		    memory.setRemoveable(c);
		
		// note that this constraint is possibly part of a
		// firing head instance
		// XXX TODO: this is not nice! see the definition of
		// hashCode() -> are you sure that this is the right usage?
		currentHeadInstance.setAt(new Integer(c.hashCode()), i);
		currentHeadConstraintsInstance.setAt(c, i);
		
		// yes, the rule is a firing-candidate, find partner
		// constraints only return if there are partner
		// constraints, otherwise test another head constraint
		if (findPartnerConstraints(memory,trailStack,trace)) {// XXX TrailStack?
		    
		    if (trace)
			System.out.println("\tfound partner constraints");
		    return true;
		    
		} // else
  
/*
		if (trace)
		    System.out.println("\tcould not find partner constraints");
*/
		
	    } 
	    
	    // reset all constraints, which have been marked to remove
	    memory.unsetRemoveables();
	    
	    // unmark all constraints, which have been involved in the
	    // matching process
	    memory.unsetMatching();

	    // unbind all variables of the rule, in order to use it
	    // with another constraint
	    // XXX carefull with TrailStack
	    unbindAllVariables(null);// XXX ?!? is "null" correct?
	    
	}
	
	// none of the head constraints fits to the test constraint,
	// so the rule is not a firing-candidate
	
	return false;
	
    }

    public boolean findPartnerConstraints(UserDefinedConstraintMemory memory,
					  TrailStack trailStack,boolean trace) {

	return searchPCTree(0, memory,trailStack,trace);

    }

    public boolean searchPCTree(int headNumber, 
				UserDefinedConstraintMemory memory,
				TrailStack trailStack,boolean trace) { // XXX???
	
	UDConstraint headConstraint, instanceCandidate;
	ObjectContainer candidates;
	
	final int hSize=head.size();
	if (headNumber < hSize) {
		    
	    // we are in a node of the search tree

	    headConstraint = (UDConstraint)head.get(headNumber);
	    if (candidateConstraint == headConstraint) 
		return searchPCTree(++headNumber, memory,trailStack,trace);

	    // if headConstraint is not cloned for the search,
	    // the result will be more common, i.e. more candidates
	    // but be careful: if cloning of headConstraint will be
	    // introduced, all constraints, which are going to be 
	    // inserted into the UDCMemory have to be cloned, too.
	    // Otherwise, the result of the candidate search won't be correct
	    candidates = memory.getInstanceCandidates(headConstraint);
	    //  	    System.out.println("\t\tcandidates for pc: "+candidates);
	    
	    ObjectContainer changes;
	    changes = headConstraint.getChangeableVars();
	    
	    final int cSize=candidates==null?0:candidates.size();
	    for (int i = 0; i < cSize; i++) {
		
		// test, whether the current instance candidate is an
		// instance of the current head constraint
		instanceCandidate = (UDConstraint)candidates.get(i);
		  //		System.out.println("\t\ttrying pc "+instanceCandidate+"; "+vTable);
		
		if ((!memory.isMatching(instanceCandidate)) && 
		    (headConstraint.match(instanceCandidate,trailStack))) {		    
		    
		  //  System.out.println("\t\tg: "+headConstraint+"; i: "+instanceCandidate);
		    
		    // mark the instance, so that it is not tested twice
		    memory.setMatching(instanceCandidate);
		    
		    // System.out.println("---------------> betrachte Kopf "+headNumber+"; "+vTable);
		    currentHeadInstance.setAt(new Integer(instanceCandidate.hashCode()), headNumber);
		    currentHeadConstraintsInstance.setAt(instanceCandidate, headNumber);
		    
		    if (searchPCTree(++headNumber, memory, trailStack, trace)) {
			
			// if the head constraint should be removed from the UDConstraintMemory
			// (e.g. in simplification and simpagation rules, mark the test constraint
			// in the UDConstraintMemory
			if (headConstraint.isToRemove())
			    memory.setRemoveable(instanceCandidate);
			
			
			return true;
			
		    } else {
			
			memory.unsetMatching(instanceCandidate);
			vTable.unbindVars(changes,trailStack); // XXX???
			--headNumber;
			
		    }
		    
		}
		
	    }
	    
	    return false;
	    
	} else {
	    
	    // we are in a leaf of the search tree
	    
	    if (!hasFiredWithInstance(currentHeadInstance)) {
                boolean res = executeGuard(trailStack,trace);
                // System.out.println("executeGuard is " + res);
		return  res;
	    } else
		return false;
	    
	}
	
    }
    
    public ObjectContainer cloneBody() {
	
	ObjectContainer oc = new ObjectContainer();
	
	final int size=body.size();
	for (int i = 0; i < size; i++)
	    oc.add(((Constraint)body.get(i)).clone());
		
	return oc;
	
    }
    
    public boolean checkTypes() 
	throws UnknownConstraintException, WrongArgumentTypeException {
	
	if (guard != null)
	    return (checkOCTypes(head) && checkOCTypes(body) && 
		    checkOCTypes(guard));
	else
	    return (checkOCTypes(head) && checkOCTypes(body));
	  
    }

    private boolean checkOCTypes(ObjectContainer oc) 
	throws UnknownConstraintException, WrongArgumentTypeException {

	final int size=oc.size();
	for (int i=0; i < size; i++) {
	    
	    Object o=oc.get(i);
	    if (o instanceof Constraint) {
		
		if (!((Constraint)o).checkTypes())
		    return false;
		
	    } else
		throw new UnknownConstraintException(""+o);
	    
	}
	
	return true;
		
    }

    // When a rule fires, this method is used to insert (copies of)
    // the body constraints
    private void addGoalConstraint(UserDefinedConstraintMemory udcm, 
				   BuiltInConstraintMemory bcm, Constraint c,
				   TrailStack trailStack) {
	
	// XXX TrailStack

	c.updateGO(true, true);
		
	if (c instanceof UDConstraint) {
	    
	    udcm.add((UDConstraint)c, false, trailStack);

	} else if (c instanceof BuiltInConstraint) {
	    
	    bcm.add((BuiltInConstraint)c, trailStack);
	    
	}
 
    }
    
    public abstract String toString();
    


    // added for VisualCHR    
    
	/**
	 * get head constraints of fired rule
	 *
	 * @return ObjectContainer containing the hashCodes of current head constraints
	 */
	public ObjectContainer getCurrentHeadInstance() {
		return currentHeadInstance;
	}//end of getCurrentHeadInstance


	/**
	 * get head constraints of fired rule
	 *
	 * @return ObjectContainer containing the current head constraints (not hashCode!)
	 */
	public ObjectContainer getCurrentHeadConstraintsInstance() {
		return currentHeadConstraintsInstance;
	}//end of getCurrentHeadConstraintsInstance


	/**
	 * get body constraints of fired rule
	 *
	 * @return ObjectContainer containing the current body constraints (not hashCode!)
	 */
	public ObjectContainer getCurrentBodyConstraintsInstance() {
		return currentBodyConstraintsInstance;
	}//end of getCurrentBodyConstraintsInstance


}
