package org.jmlspecs.jmlexec.jack.evaluator;

/**
 * SCollectorChoice.java <br>
 * $Id: SBestSearchCollector.java,v 1.1 2008/03/05 00:08:35 wahlst Exp $
 *
 * @author Ekkehard Kramer (last modified by $Author: wahlst $)
 * @version $Date: 2008/03/05 00:08:35 $, $Revision: 1.1 $
 */

// A SChoice which collects the values of a bunch of variables at one
// successful node. Jused together with SSearch.best() to do
// branch-and-bound search.

// TODO: make the passing of varNames optional

abstract class SBestSearchCollector extends SChoiceOperator
{

    private ObjectContainer vars;
    private ObjectContainer varNames;
    private ObjectContainer bestSolution=null;
    private int consideredSolutions=0; // number of solutions encountered (including the last one)

    SBestSearchCollector(ObjectContainer vars,ObjectContainer varNames)
    {
	this.vars=vars;
	this.varNames=varNames;
    }

    // Accumulate the solutions
    public void run(ConstraintSystem system)
    {

	// Either this is the first try to run the constraint system,
	// in which case any solution is better than no solution; or a
	// constraint "the next solution must be better than the last
	// one" has been inserted before, so this one is guaranteed to
	// be better.

	bestSolution=system.getVarValues(vars,true);

	consideredSolutions++;
    }

    // Get the current best solution
    public ObjectContainer getSolution()
    {
	return bestSolution;
    }

    // This should insert one or more proper constraints into the system.
    // getSolution() should be used to query the current best solution,
    // and constraints must be inserted that guarantee that the next
    // solution is "better".
    abstract public void addBetterConstraint(ConstraintSystem system);

    // Returns all the solutions in a nice way
    public String toBeautifulString()
    {
	StringBuffer buf=new StringBuffer("Best solution:\n");

	int varCount=vars.size();

	for (int var=0; var<varCount; var++)
	{
	    buf.append(varNames.get(var)+" = "+bestSolution.get(var)+"\n");
	}	    

	buf.append(consideredSolutions+" solution(s) have been considered.");
		
	return buf.toString();
	
    }

    public String toString()
    {
	return "SCollectorChoice["+varNames+"]";
    }

}
