package org.jmlspecs.jmlexec.jack.evaluator;

/**
 * SCollectorChoice.java <br>
 * $Id: SCollectorChoice.java,v 1.1 2008/03/05 00:08:35 wahlst Exp $
 *
 * @author Ekkehard Kramer (last modified by $Author: wahlst $)
 * @version $Date: 2008/03/05 00:08:35 $, $Revision: 1.1 $
 */

// A simple SChoice which collects the values of a bunch of variables
// at each successful node; all the solutions can be retrieved
// together afterwards.

// TODO: make the passing of varNames optional

public class SCollectorChoice extends SChoiceOperator
{

    private ObjectContainer vars;
    private ObjectContainer varNames;
    private ObjectContainer solutions=new ObjectContainer();

    public SCollectorChoice(ObjectContainer vars,ObjectContainer varNames)
    {
	this.vars=vars;
	this.varNames=varNames;
    }

    // Accumulate the solutions
    public void run(ConstraintSystem system)
    {

	ObjectContainer values=system.getVarValues(vars,true);
	solutions.add(values);

    }

    // Get all solutions accumulated during the search
    public ObjectContainer getSolutions()
    {
	return solutions;
    }

    // Reset everything
    public void reset()
    {
	solutions.removeAll();
    }

    // Returns all the solutions in a nice way
    public String toBeautifulString()
    {
	return toBeautifulString(false);
    }

    public String toBeautifulString(boolean dense)
    {
	StringBuffer buf=new StringBuffer();

	int solCount=solutions.size();
	int varCount=vars.size();

	if (dense)
	{
	    // Output the names of the variables

	    buf.append("Variables:\n");

	    String comma="";
	    for (int var=0; var<varCount; var++)
	    {
		buf.append(comma+varNames.get(var));
		comma=", ";
	    }	    

	    buf.append("\n\nSolutions:\n");	    
	}

	for (int sol=0; sol<solCount; sol++)
	{
	    
	    if (dense)
		buf.append((sol+1)+":\t");
	    else
		buf.append("Solution "+(sol+1)+"\n");

	    ObjectContainer solution=(ObjectContainer)(solutions.get(sol));

	    String comma="";
	    for (int var=0; var<varCount; var++)
	    {
		if (dense)
		{
		    buf.append(comma+solution.get(var));
		    comma=", ";
		}
		else
		    buf.append("    "+varNames.get(var)+
			       " = "+solution.get(var)+"\n");
	    }	    

	    buf.append("\n");

	}

	return buf.toString();
	
    }

    public String toString()
    {
	return "SCollectorChoice["+varNames+"]";
    }

}
