package org.jmlspecs.jmlexec.jack.evaluator;

/**
 * SMarkedConstraintSystem.java <br>
 * $Id: SMarkedConstraintSystem.java,v 1.1 2008/03/05 00:08:35 wahlst Exp $
 *
 * @author Ekkehard Kramer (last modified by $Author: wahlst $)
 * @version $Date: 2008/03/05 00:08:35 $, $Revision: 1.1 $
 */

// Encapsulates an instance of ConstraintSystem with some current state

// TODO: remove the current implementation of get/setEnvironment()
// from ConstraintSystem, and do them in different subclasses of
// SMarkedConstraintSystem (then, newSystem() will make sense)

abstract public class SMarkedConstraintSystem 
{

    protected ConstraintSystem system;

    public SMarkedConstraintSystem(ConstraintSystem system)
    {
	this.system=system;
    }

    // Backtrack the system to the state it was in when the
    // SMarkedConstraintSystem was created
    abstract public ConstraintSystem getSystem();

    // From a ConstraintSystem cs, create a new SMarkedConstraintSystem,
    // which encapsulates the current state of cs
    abstract public SMarkedConstraintSystem newSystem(ConstraintSystem cs);

    // use trailing
    public static SMarkedConstraintSystem 
	createTrailing(ConstraintSystem system)
    {
	return new SMCSTrailing(system);
    }

    // use copying
    public static SMarkedConstraintSystem 
	createCopying(ConstraintSystem system)
    {
	return new SMCSCopying(system);
    }
}

class SMCSCopying extends SMarkedConstraintSystem
{
    private CopyEnvironment environment;

    public SMCSCopying(ConstraintSystem system)
    {
	super(system);
	environment=system.getCopyEnvironment();
    }

    public ConstraintSystem getSystem()
    {
	environment.set(system);
	return system;
    }

    public SMarkedConstraintSystem newSystem(ConstraintSystem cs)
    {
	return new SMCSCopying(cs);
    }
}

class SMCSTrailing extends SMarkedConstraintSystem
{
    private TrailEnvironment environment;

    public SMCSTrailing(ConstraintSystem system)
    {
	super(system);
	environment=system.getTrailEnvironment();
    }

    public ConstraintSystem getSystem()
    {
	environment.set(system);
	return system;
    }

    public SMarkedConstraintSystem newSystem(ConstraintSystem cs)
    {
	return new SMCSTrailing(cs);
    }
}

