package org.jmlspecs.jmlexec.jack.evaluator;

/**
 * SNode.java <br>
 * $Id: SNode.java,v 1.1 2008/03/05 00:08:35 wahlst Exp $
 *
 * @author Ekkehard Kramer (last modified by $Author: wahlst $)
 * @version $Date: 2008/03/05 00:08:35 $, $Revision: 1.1 $
 */

// One node of the search tree. It is distinguished from SChoice by
// managing the system state.

// TODO: generalize this for any number of childs; also,
// createLeftChild() and crateRightChild() are mostly the same code

// TODO: provide random access to the children

public class SNode
{
    private int children=2;	// binary child
    private int currentChild=0;

    private SMarkedConstraintSystem markedSystem;
    private SChoice choice;

    private static SNode successNode,failureNode;

    public SNode(SMarkedConstraintSystem markedSystem,SChoice choice)
    {
	this.markedSystem=markedSystem;
	this.choice=choice;
    }

    public int getCurrentChild()
    {
	return currentChild;
    }

    public boolean hasMoreChildren()
    {
	return currentChild<children;
    }

    public SNode createNextChild()
    {
	if (currentChild>=children)
	    throw new RuntimeException("No more children.");

	// Backtrack the constraint system up to the state which was
	// current before entering this node

	ConstraintSystem mcs=markedSystem.getSystem();

	// Do the next choice (this modifies the constraint system)

	// This here needs to be changed to support more than 2 children

	SChoice c=choice.choose(mcs,currentChild==0);
	
	SNode child;

	if (c!=null)
	{

	    // Ok, we have a real new child now

	    child=new SNode(markedSystem.newSystem(mcs),c);

	}
	else
	{

	    // We are finished (done all, or failed)

	    if (choice.failed())
		child=getFailureNode();
	    else
		child=getSuccessNode();

	}

	currentChild++;

	return child;
    }


    // Return the system to the state before this node
    public void reset()
    {
	markedSystem.getSystem();
    }

    // Two special kinds of nodes

    public boolean isSuccess() { return false; }
    public boolean isFailure() { return false; }

    // Get a "success" node
    public static SNode getSuccessNode()
    {
	if (successNode==null)
	    successNode=new SuccessNode();

	return successNode;
    }
    
    // Get a "failure" node
    public static SNode getFailureNode()
    {
	if (failureNode==null)
	    failureNode=new FailureNode();

	return failureNode;
    }
    

}

class SuccessNode extends SNode 
{

    SuccessNode() 
    {
	super(null,null);
	return;
    }

    public boolean isSuccess() 
    { 
	return true; 
    }
}

class FailureNode extends SNode 
{
    FailureNode() 
    {
	super(null,null);
	return;
    }

    public boolean isFailure() 
    { 
	return true; 
    }
}
