package org.jmlspecs.jmlexec.jack.evaluator;

/**
 * SSearch.java <br>
 * $Id: SSearch.java,v 1.1 2008/03/05 00:08:35 wahlst Exp $
 *
 * @author Ekkehard Kramer (last modified by $Author: wahlst $)
 * @version $Date: 2008/03/05 00:08:35 $, $Revision: 1.1 $
 */

public class SSearch
{

    // Given an instance of SExploration, perform first-solution
    // search (i.e., stop after the first solution found). Returns
    // true if a solution was found.
    public static boolean first(SExploration exp)
    {

	SNode node=exp.performOneStep();

	while (node!=null)
	{
	    if (node.isSuccess())
		return true;

	    node=exp.performOneStep();
	}

	return false;
    }

    // Given an instance of SExploration, perform all-solution search
    // (i.e., stop when nothing more can be done). Returns true if at
    // least one solution was found.
    public static boolean all(SExploration exp)
    {
	
	boolean success=false;

	SNode node=exp.performOneStep();

	while (node!=null)
	{
	    if (!success && node.isSuccess())
		success=true;

	    node=exp.performOneStep();
	}

	return success;
    }

    // Given an instance of SExploration, perform best-solution
    // (branch-n-bound) search (i.e., injects a constraint
    // solutionIsBetter(PreviousBestSolution,NextSolution) after a
    // solution is found).  Returns true if a solution was found.
    public static boolean best(SExploration exp,SBestSearchCollector coll,
			       ConstraintSystem system)
    {
	boolean success=false;

	SMarkedConstraintSystem marked=system.createMarkedSystem();

	// Do a normal first solution search
	while (first(exp))
	{
	    success=true;

	    // Reset to the original system (removes "make it
	    // better" constraints)
	    system=marked.getSystem();

	    // Tell the system that a better solution is needed
	    coll.addBetterConstraint(system);

	    // Reset the exploration
	    exp.reset(system);
	}

	return true;
    }

}
