package org.jmlspecs.jmlexec.jack.evaluator;

/**
 * STraceChoice.java <br>
 * $Id: STraceChoice.java,v 1.1 2008/03/05 00:08:35 wahlst Exp $
 *
 * @author Ekkehard Kramer (last modified by $Author: wahlst $)
 * @version $Date: 2008/03/05 00:08:35 $, $Revision: 1.1 $
 */

// This modifies a given instance of SChoice and adds debugging output
// at each node of the search tree

// Used like this:

//   SChoice choice=new STraceChoice(System.err,new SSomeOtherChoice(...));

import java.io.OutputStream;
import java.io.PrintWriter;

public class STraceChoice implements SChoice
{
    private SChoice choice;	// the real choice
    private boolean isFailure;	// the failure state of the real choice

    private PrintWriter stream;	// the output stream
    private int level;		// how deep we are in the tree
    private String prefix;	// printed before each line (indentation)

    public STraceChoice(OutputStream stream,SChoice choice)
    {
	this.stream=new PrintWriter(stream, true);
	this.choice=choice;
	level=0;
	prefix="";	
    }

    private STraceChoice(PrintWriter stream,SChoice choice,int level)
    {
	this.stream=stream;
	this.choice=choice;
	this.level=level;

	prefix="";
	for (int t=0; t<level; t++)
	    prefix+="  ";
    }

    public SChoice choose(ConstraintSystem system,boolean firstChoice)
    {
	stream.println(prefix+"choose("+(firstChoice?"first":"second")+
		       "), level "+level+", choice="+choice);

	isFailure=false;

	System.out.println(prefix+"(before) variable table:      "+ 
			   system.getVariableTable());
	System.out.println(prefix+"(before) user defined memory: "+
			   system.getUserDefinedConstraintMemory());

	SChoice result=choice.choose(system,firstChoice);

	System.out.println(prefix+"(after) variable table:      "+ 
			   system.getVariableTable());
	System.out.println(prefix+"(after) user defined memory: "+
			   system.getUserDefinedConstraintMemory());

	if (result!=null)
	{
	    stream.println(prefix+"  -> more choices at level "+level);
	    result=new STraceChoice(stream,result,level+1);
	}
	else
	{
	    isFailure=choice.failed();

	    if (isFailure)
		stream.println(prefix+"  -> failed at level "+level);
	    else
		stream.println(prefix+"  -> succeeded at level "+level);
	}

	return result;
    }

    public boolean failed()
    {
	return isFailure;
    }

    public void reset()
    {
	stream.println("---RESET---");
    }

}
