package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class UserDefinedConstraintMemory
   @version 1 (06/06/1999)
   @author Matthias SchmauB
*/

import java.util.Enumeration;

public class UserDefinedConstraintMemory {

    /////////////////////////////
    // fields
    
    public static final String VAR = "VAR";
    private static final int maxDepth = 3;
    
    private VariableTable vTable;

    // datastructure to store flat user defined constraints
    private PathNode index;

    // datastructure to store references to user defined constraints,
    // which are to be removed from the index
    // is not kept in TrailStack (??? XXX)
    private ObjectContainer removables;

    // datastructure to store references to active user defined constraints
    // (constraints, to which no rule has been applied yet)
    // does not need to be kept trail of in TrailStack (see 
    // ConstraintSystem.callGoal)
    private ObjectContainer actives;

    // datastructure to store references to all matching constraints of a rule
    // probably not needed in TrailStack, because it is internally used
    // in Rules.apply() (XXX)
    private ObjectContainer matches;

    /////////////////////////////
    // constructors
    
    protected UserDefinedConstraintMemory(VariableTable vt) {
	
	vTable = vt;
	index = new PathNode();
	removables = new ObjectContainer();
	actives = new ObjectContainer();
	matches = new ObjectContainer();
	
    }

    /////////////////////////////
    // methods

    ////////////////////////////////////////////
    // operations for the reference memories

    // ----------------------------------------
    
    public void setAllConstraintsActive() {
	ObjectContainer tmp = index.getLeafUnion();

	actives = new ObjectContainer();
	
	final int size=tmp.size();
	for(int i = 0; i < size; i++)
	    setActive((UDConstraint)tmp.get(i));
	
    }
    
    public void setExceptionConstraintsActive() {
	ObjectContainer tmp = index.getLeafUnion();

//	actives = new ObjectContainer();
	
	final int size=tmp.size();
	for(int i = 0; i < size; i++) {
            UDConstraint udc = (UDConstraint) tmp.get(i);
            if (udc.getIdentifier().equals("newException") && 
                ! actives.contains(udc)) {
	      setActive((UDConstraint)tmp.get(i));
            }
        }
	
    }
    
    
    public ObjectContainer getPartnerConstraints() {
	
	return (ObjectContainer)matches.clone();

    }

    public void setMatching(UDConstraint c) {
      
	matches.add(c);
	
    }
    
    public void unsetMatching(UDConstraint c) {

	matches.remove(c);

    }

    public boolean isMatching(UDConstraint c) {
	
	return matches.contains(c);

    }

    public void unsetMatching() {

	matches.removeAll();

    }

    // ----------------------------------------
    
    public ObjectContainer getActives() {

	return actives;

    }

    public boolean isActive(UDConstraint c) {

	return actives.contains(c);

    }
 
    // added by T. Wahls 7/21/2005
    private boolean ground(Object var) {
//      System.out.print(var);
      var = vTable.getValue(var);
//      System.out.println(" " + var + " : " +  vTable.isVariable(var));
      return !vTable.isVariable(var);
    }

    private boolean allGround(UDConstraint c) {
       int size = c.getArity();
       if (!c.lastGround()) {// result need not be ground
         size--;
       }
       for (int i = 0; i < size; i++) {
         if (!ground(c.getArgAt(i))) return false;
       }
       return true;
    }
    
    public void setActive(UDConstraint c) {

      if (!c.inAssert() || allGround(c)) {
	actives.add(c);
      }

    }

    public void setInitialActive(UDConstraint c) {

      if (!c.inAssert() || allGround(c)) {
	actives.insertAt(c, 0);
      }

    }

    public void setPassive(UDConstraint c) {

	actives.remove(c);

    }

    public UDConstraint getNextActive() {

        int size = actives.size();
        for (int i = 0; i < size; i++) {
          UDConstraint udc = (UDConstraint) actives.get(i);
          if (udc.getIdentifier().equals("equalequal")) {
            return udc;
          }
        }
	return (UDConstraint)actives.get(0);
//	return (UDConstraint)actives.get(0);

    }
    
    public boolean hasMoreActiveConstraints() {

	return (actives.size() > 0);
	
    }
    
    // ------------------------------------------

    public void setRemoveable(UDConstraint c) {

	removables.add(c);

    }

    public void unsetRemoveable(UDConstraint c) {

	removables.remove(c);

    }

    public void unsetRemoveables() {

	removables.removeAll();

    }

    public void removeConstraints(TrailStack trailStack) {


	final int size=removables.size();
	for (int i = 0; i < size; i++) {
	    
	    // remove user-defined constraints from memory

	    // the TrailStack is handled in the next call. if
	    // removables.size() is big, it would be better to handle
	    // all this in one single TrailStack.add() call (XXX TODO:
	    // check if this is true, and implement it)

	    remove((UDConstraint)removables.get(i), trailStack);
	    
	}

	removables.removeAll();

    }

    ////////////////////////////////////////////
    // operations for the path index

    public boolean isEmpty() {

	return (index.size() == 0);
	
    }

    public void add(final UDConstraint c,boolean initial,
		    TrailStack trailStack) {
	
	// this could become ugly :-)

	if (trailStack!=null)
	{

	    trailStack.add(//"UDCM.add() -> undo by removing "+c,
			   new Runnable() { public void run() {
			       
			       remove(c, null /* no recursion please! */ );

			   }});


	}


	// add c to path index
	ObjectContainer paths = getPaths(c, maxDepth);
	final int size=paths==null?0:paths.size();
	for (int i = 0; i < size; i++) 
	{
	    if (!(paths.get(i) instanceof Path))
		throw new RuntimeException("no path: "+paths.get(i));
	    index.addByPath((Path)paths.get(i), c);
	}
	
	// add c to actives
	if (initial)
	    setInitialActive(c);
	else
	    setActive(c);
	
    }

    public void remove(final UDConstraint c, TrailStack trailStack) {
	
	// update variable observer lists
	
	vTable.updateVariableObservers(c);

	// this could become ugly :-)

	if (trailStack!=null)
	{

	    // XXX do we have to care about the active state?  

	    // XXX if we do, we must re-insert it at the correct
	    // position in the active list!!!!!!!

	    final boolean wasActive = isActive(c); 

	    trailStack.add(//"UDCM.remove() -> undo by adding "+c+" again, active="+wasActive,
			   new Runnable() { public void run() {
			       
			       add(c, false, null /* no recursion please! */ );
			       c.updateGO(false,true); // observers

			       // c is now active

			       if (!wasActive)
				   setPassive(c); // beware: no recursion!

			   }});


	}


	// remove c from path index
	ObjectContainer paths = getPaths(c, maxDepth);
	final int size=paths==null?0:paths.size();
// System.out.println("index pre: " + index);
	for (int i = 0; i < size; i++) 
	    index.removeByPath((Path)paths.get(i), c);
// System.out.println("index post: " + index);

	// remove c from actives
	setPassive(c);

    }
    

    // may return null
    public ObjectContainer getInstanceCandidates(UDConstraint c) {

	ObjectContainer result = null;
	
	// eval pattern path
	Object paths = getPatternPaths(c, maxDepth);
	
	// if the constraint's argumentlist contained only variables
	// the list of candidates is searched with the base path;
	// so the leafs of a subtree are united to the list of candidates

	if (paths==null)
	{
	    result = index.getByPath(new Path(0, c.getIdentifier(), null));
	}
	else
	{
	    if (paths instanceof ObjectContainer)
	    {
		// more than one path was found; return their intersection

		ObjectContainer thePaths=(ObjectContainer)paths;
		result = index.getByPath((Path)thePaths.get(0));
      
		// the list of candidates is received by the
		// intersection of the results of all paths;
		final int size=thePaths.size();
		for (int j = 1; j < size && result!=null && result.size()>0; j++) 
		{
		    ObjectContainer other=index.getByPath((Path)thePaths.get(j));
		    if (other!=null)
			result = result.intersection(other);
		    else 
			result=null;
		}
	    }
	    else
	    {
		// exactly one path was found

		result = index.getByPath((Path)paths);
	    }
	}
	
	return result;
      
    }
    
    /**
       returns paths in an ObjectContainer; if only one path is found,
       itself is returned (without ObjectContainer), if none is found, null
       is returned
     */

    private Object getPatternPaths(UDConstraint c, int depth) {
	
	return evalBranches(c, 0, depth, false);
    }

    /**
       returns paths in an ObjectContainer; if only one path is found,
       itself is returned (without ObjectContainer), if none is found, null
       is returned.

       This is done so complicated, because this function is called
       *really* often, and in this way, we can save lots of
       unneccessary creations of unused ObjectContainer instances.
     */

    private Object evalBranches(Object o, int localDepth, 
				int globalDepth,boolean includeVars) {
	
	Object rc=null;

	if (o instanceof Constraint) { // branch
	   
	    Constraint c = (Constraint)o;

	    // This is the path for the constraint itself
	    rc=new Path(localDepth, c.getIdentifier(), null);

	    if (localDepth != globalDepth) {
		
		final int iSize=c.getArity();
		for (int i = 0; i < iSize; i++) {
		    
		    // get the paths for the children

		    Object tmp = evalBranches(c.getArgument(i),
					      localDepth+1+i,
					      globalDepth,
					      includeVars);
		    
		    if (tmp!=null)
		    {
			if (tmp instanceof ObjectContainer)
			{
			    // the child returned more than one path

			    if (!(rc instanceof ObjectContainer))
			    {
				// we reuse their objectcontainer

				ObjectContainer oc=(ObjectContainer)tmp;
				final int jSize=oc.size();
			    
				for (int j = 0; j < jSize; j++)
				    oc.setAt(new Path(localDepth, 
						      c.getIdentifier(),
						      (Path)oc.get(j)),
					     j);

				oc.insertAt(rc,0);
				rc=oc;
			    }
			    else
			    {
				// ad the new paths

				ObjectContainer oc=(ObjectContainer)tmp;
				final int jSize=oc.size();
				ObjectContainer old=(ObjectContainer)rc;
			    
				for (int j = 0; j < jSize; j++)
				    old.add(new Path(localDepth, 
						    c.getIdentifier(),
						    (Path)oc.get(j)));
			    }
			}
			else
			{
			    // the child returned exactly one path
			    
			    if (!(rc instanceof ObjectContainer))
			    {
				ObjectContainer oc=new ObjectContainer();
				oc.add(rc);
				rc=oc;
			    }

			    ((ObjectContainer)rc).add(new Path(localDepth,
							       c.getIdentifier(),
							       (Path)tmp));
			}
		    }
		}
		
	    }
	    
	} else if (includeVars && vTable.isVariable(o)) {
	    
	    rc=new Path(localDepth, VAR, null);
	    
	} else if (includeVars || !vTable.isVariable(o)) {
	    rc=new Path(localDepth, Util.JMLEString(o), null);
	    
	}
	
	return rc;

    }


    /**
       
     */
    
    public ObjectContainer getPaths(UDConstraint c, int depth) {
	
	int localDepth = 0;
	Object paths = evalBranches(c, localDepth, depth, true);

	if (paths==null)
	    return null;

	if (paths instanceof ObjectContainer)
	    return (ObjectContainer)paths;

	ObjectContainer oc=new ObjectContainer();
	oc.add(paths);	
	
	return oc;
	
    }
    
    /**
       
     */
    
//     private ObjectContainer evalBranches(Object o, int localDepth, int globalDepth) {
	
// 	ObjectContainer paths = new ObjectContainer();
	
// 	if (o instanceof Constraint) { // branch
	    
// 	    Constraint c = (Constraint)o;

// 	    if (localDepth == globalDepth) {
		
// 		paths.add(new Path(localDepth, c.getIdentifier(), null));
		
// 	    } else {
		
// 		paths.add(new Path(localDepth, c.getIdentifier(), null));

// 		final int iSize=c.getArity();
// 		for (int i = 0; i < iSize; i++) {
		    
// 		    ObjectContainer tmp = evalBranches(c.getArgument(i), localDepth+1+i, globalDepth);
		    
// 		    final int jSize=tmp.size();
// 		    for (int j = 0; j < jSize; j++)
// 			paths.add(new Path(localDepth, c.getIdentifier(), (Path)tmp.get(j)));
		    
// 		}
		
// 	    }
	    
// 	} else if (vTable.isVariable(o)) { // return single path
	    
// 	    paths.add(new Path(localDepth, VAR, null));
	    
// 	} else { // return single path
	    
// 	    paths.add(new Path(localDepth, ""+o, null));
	    
// 	}

// 	return paths;
	
//     }
    
    public String toString() {
	
	return ""+index;
	
    }

    public void analyze() {

	System.out.println("index: "+index);
	System.out.println("removables: "+removables);
	System.out.println("actives: "+actives);
	System.out.println("matches: "+matches);

    }

    public ObjectContainer elements() {

	return index.getLeafUnion();

    }

    public UDCMState getUDCMState() {

	return new UDCMState(vTable.getVTableState(), 
			     (PathNode)index.clone(), 
			     (ObjectContainer)removables.clone(), 
			     (ObjectContainer)actives.clone(), 
			     (ObjectContainer)matches.clone());
	
    }

    public void setUDCMState(UDCMState udcms) {

	vTable.setVTableState(udcms.vts);
	index = (PathNode)(udcms.index.clone());
	removables = (ObjectContainer)(udcms.removables.clone());
	actives = (ObjectContainer)(udcms.actives.clone());
	matches = (ObjectContainer)(udcms.matches.clone());

	// Now create the observers for all the constraints
	ObjectContainer udcs=index.getLeafUnion();
	int size=udcs.size();
	for (int i=0; i<size; i++)
	    ((Constraint)(udcs.get(i))).updateGO(false,true);

    }
}
