package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class VariableTable <br>
   $Id: VariableTable.java,v 1.2 2008/03/07 21:48:46 wahlst Exp $
   @version 1 (26/05/1999), $Revision: 1.2 $
   @author Matthias Schmau (last modified by $Author: wahlst $)
*/

// import java.util.Hashtable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
// import java.util.Enumeration;

/*
  
  Frage: Warum sind Variablen nur vom Typ java.lang.Object und nicht
  von dem Typ, den sie repr sentieren?

Question: Why are variables of the type java.lang.Object and not of the type 
they represent? 


  Antwort: F r eine Variable mu ein Objekt erstellt werden! Es ist
  jedoch nicht immer m glich, ein Object nur mit der Kenntnis seiner
  Klasse zu erstellen, da f r die Instanziierung evtl Argumente
  ben tigt werden.  Man mu also ein Platzhalterobjekt f r Variablen
  verwenden. In diesem Fall ist die Entscheidung zugunsten der Technik
  der Variablentabelle gefallen. In dieser Tabelle wird einem Objekt
  (das eine Variable repr s.) eine Klasse und ein Bezeichner
  zugewiesen.
  
Answer: For a variable an object has to be created! It isn't however always 
possible to create an object only with the knowledge of its class, since for 
the instantiation potentially arguments are needed. Thus you have to use a 
proxy object for the variables. In this case the decision was made to use 
the technique of a variable table. In this table a class and a name gets 
assigned to an object (which represents a variable)
  

  ABER: Es w re m glich, statt Object eine Klasse Variable zu nehmen:

    class Variable
    {
        String name;
	Class theClass;
	// weitere statische Informationen, falls n tig
    }

  Dann k nnte man sich n mlich einige der Hashtables in VariableTable
  ersparen (und auch teure Lookups darauf). Au erdem w rde man dann nicht
  permanent auf die Variablentabelle zugreifen m ssen, nur um den Namen
  der Variable herauszufinden, und k nnte die Variable "einfach so"
  ausgeben. Wohlgemerkt, der Wert der Variable (bei Bindung) kann nicht
  direkt bei der Variablen gespeichert werden, da beim Backtracken etc. ja 
  verschiedene Bindungen vorr tig gehalten werden m ssen.

BUT: It would be possible, instead of an object to use a class variable: 

class Variable { 
	String name;
	Class theClass;
	// further static information, if necessary 
} 

Then one could save oneself some of the hash tables in the variable table 
(and expensive lookups to them). Additionally you wouldn't have to constantly 
access the variable table, just to find out the name of the variable table, and 
thus could "just" print the name of the variable. Note though, you can't store 
the value of the variable directly at the variable (upon binding), since during 
the backtracking different bindings have to be stored.
  
*/

/**
   This class is based on java.util.Hashtable. It provides a table to
   store variables, which are used in constraint handling rules and in
   the goal constraints. This table is a global variable table. All
   constraints using this table, have to be registered. That means,
   this class assign a reference on itself to the constraint. @see
   java.util.Hashtable.  

*/

public class VariableTable {
    
    //////////////////////////////////
    // fields
    
    /**
       This fields contains the bindings the variables.
    */

    // (variable, binding <Object>)
    private HashMap variableTable;// trailed

    // (variable, class <Class>)
    private HashMap variableClasses;// trailed

    // (variable, name <String>)
    private HashMap variableNames;// trailed

    // (name <String>, variable)
    private HashMap variableObjects;// trailed

    // (variable, ObjectContainer <VarObersver>)
    private HashMap variableObserver;// trailed
    
    // (constraint, ObjectContainer <variable>)
    private HashMap usedVariables;// trailed (really?)

    private ObjectContainer unboundVars; // trailed

    // if unboundVars is to be used, set this here to true
    public static boolean supportUnboundVars=false;

    private ObjectContainer blockVars; // trailed

    // not a good name XXX TODO
    private ObjectContainer tmpGlobalVars; // not trailed

    private ObjectContainer goalVars; // trail this? probably not XXX check this out

    private static int counter;	// not trailed

    private boolean changed;

    //////////////////////////////////
    // constructors

    public VariableTable() {

	variableTable = new HashMap();
	variableClasses = new HashMap();
	variableNames = new HashMap();
	variableObjects = new HashMap();
	variableObserver = new HashMap();
	usedVariables = new HashMap();
	blockVars = new ObjectContainer();
	tmpGlobalVars = new ObjectContainer();
	goalVars = new ObjectContainer();	
	unboundVars = new ObjectContainer();
	counter = 1;
// 	change = null;
	changed = false;
	
    }

    //////////////////////////////////
    // methods

    // the given variable is going to be unbound, so the variables observers 
    // are to be notified
    // XXX TODO: the name is not good
    public void updateVariableObservers(Object var) {

	// XXX TrailStack?

	ObjectContainer oc = getVariableObservers(var);
	if (oc!=null)
	{
	    final int size=oc.size();
	    for (int i=0; i<size; i++) {
		
		VarObserver vo = (VarObserver)oc.get(i);
		vo.observer.setAt(var, vo.position);
		
	    }
	}

    }

    // the given constraint udc is going to be removed from the user-defined
    // constraint memory, when this method is called. So the observertables 
    // need to be updated, that means, that udc needs to be removed from the 
    // observerlists of its variables
    // XXX TODO: the name is not good
    public void updateVariableObservers(UDConstraint udc) {

	// XXX probably no need to use TrailStack

	// get all used variables
	ObjectContainer variables = getUsedVariables(udc);

	if (variables!=null)
	{
	    // remove udc from each observerlist
	    final int size=variables.size();
	    for (int i=0; i<size; i++) 
		removeFromObserverList(variables.get(i), udc);
	} 

    }

    // --------------------------------------------------------------------

    public void addUsedVariables(Constraint c, ObjectContainer vars) {

	usedVariables.put(c, vars);

    }

    // returns null if no variables are found
    public ObjectContainer getUsedVariables(Constraint c) {

	ObjectContainer result = (ObjectContainer)usedVariables.get(c);

	return result;
    }
  
    public boolean hasVariables(Constraint c) {

	return usedVariables.containsKey(c);

    }

    public void removeVariableUser(Constraint c) {

	usedVariables.remove(c);

    }

    // --------------------------------------------------------------------------

    public void addAsVariableObserver(Object var, int p, Constraint c) {

	// XXX trailstack?

	ObjectContainer constraints;

	if ((constraints = (ObjectContainer)variableObserver.get(var)) == null)
	    constraints = new ObjectContainer();

	VarObserver vo = new VarObserver(p, c);
	if (!constraints.contains(vo)) {

	    constraints.add(vo);
	    variableObserver.put(var, constraints);	

	}

    }

    public boolean hasObservers(Object var) {

	return variableObserver.containsKey(var);

    }

    public ObjectContainer getVariableObservers(Object var) {

	if (hasObservers(var))
	    return (ObjectContainer)variableObserver.get(var);
	else
	    return null;
    }

    public void removeFromObserverList(Object var, Constraint c) {

	// XXX TrailStack! (?)

	ObjectContainer oc = (ObjectContainer)variableObserver.get(var);

	if (oc != null) {

	    final int size=oc.size();
	    for (int i=0; i<size; i++) {

		VarObserver vo = (VarObserver)oc.get(i);
		if (vo.observer.equals(c)) {

		    oc.remove(vo);
		    usedVariables.remove(vo.observer);
		    break;

		}

	    }

	}

    }

    public void removeAllObservers() {

	// XXX TrailStack?

	Iterator enu = variableObserver.keySet().iterator();
	Object key;

	while (enu.hasNext()) {

	    key = enu.next();
//	    variableObserver.remove(key);
            enu.remove();

	}
	
	enu = usedVariables.keySet().iterator();
	while (enu.hasNext()) {

	    key = enu.next();
//	    usedVariables.remove(key);
            enu.remove();

	}

    }

    // -------------------------------------------------------------------------

    public ObjectContainer getBoundVarsWith(Object var) {

	ObjectContainer boundVars = new ObjectContainer();

	if (variableTable.containsValue(var)) {

	    Iterator enu = variableTable.keySet().iterator();
	    Object key;

	    while (enu.hasNext()) {

		key = enu.next();
		if (variableTable.get(key).equals(var))
		    boundVars.add(key);

	    }

	}
	    
	return boundVars;

    }

    /*
    public void updateTable(Object var, Object object) {

	// XXX TrailStack!

	ObjectContainer updateKeys = getBoundVarsWith(var);
	for (int i=0; i<updateKeys.size(); i++) {
	    
	    variableTable.put(updateKeys.get(i), object);
// 	    System.out.println("variableTable.put "+variableToString(updateKeys.get(i))+" "+variableToString(object));
	    updateTable(updateKeys.get(i), object);

	}

    }
    */

    // --------------------------------------------------------------------------------------------

    // TODO: Typ des Objects und Typ der Variablen m ssen bereinstimmen
    public void bind(final Object var, Object object, TrailStack trailStack) {

	// just a quick test: we assume that only unbound variables
	// are bound here

	if (variableTable.get(var)!=null) {
	    throw new RuntimeException("Paranoia: var is already in the variableTable. old value="+variableTable.get(var)+", new value="+object);
        }
	if (trailStack!=null)
	{

	    trailStack.add(/*"VT.bind() -> undo by unbinding "+getVariableName(var),*/
			   new Runnable() { public void run() {

			       // XXX is this really enough?

			       unbind(var,null/* no recursion, please! */);
			       
			   }});

	}

	// XXX more TrailStack stuff?

	variableTable.put(var, object);
// System.out.println("Binding " + getVariableName(var) + " = " + var + " to " + object + " goal? " + isGoalVar(var));

	if (supportUnboundVars)
	    unboundVars.remove(var);

 	// unchains equalities in the variable table
	// the problem here is that bound variables are bound to other values
 	//updateTable(var, object);

	if (isGoalVar(var)) {
	    
	    changed = true;
	    
	}
    }
    
    // has a variable been bound since the last call to isUpdated()?
    public boolean isUpdated() {
	
	if (changed) {
	    
	    changed = false;
	    return true;
	    
	} else
	    return false;
		
    }

    public void addGoalVar(Object var) {

	goalVars.add(var);

    }
    
    public boolean isGoalVar(Object var) {

	return goalVars.contains(var);

    }

    public boolean isBound(Object var) {
	
        if (Util.isJMLE(var)) return false;
	Object o = variableTable.get(var);

	return (o != null);

    }

    public Object getBinding(Object var) {

        if (Util.isJMLE(var)) return null;
	return variableTable.get(var);

    }

    public ObjectContainer getUnboundVars() {

	if (!supportUnboundVars)
	    throw new RuntimeException("To use this method, set VariableTable.supportUnboundVars=true");

	return unboundVars;

    }
    
    public ObjectContainer toObjectContainer() {
	    return new ObjectContainer(variableTable.keySet().iterator());
    }

    public ObjectContainer getGoalVars() {return goalVars;}

    public void unbindAll(TrailStack trailStack) {
	
	// XXX TrailStack! (?)

        Iterator iter = variableTable.keySet().iterator();
        while (iter.hasNext()) {
//        for (Object var : variableTable.keySet()) {
            Object var = iter.next();
	    unbind(var, trailStack, false); // XXX trail these all together!
        }
        variableTable.clear();
	
    }

    // it is assumed that all of the objects inside the container are
    // actually bound variables
    public void unbindVars(ObjectContainer oc,TrailStack trailStack) {
	
	final int size=oc.size();
	for (int i=0; i<size; i++)
	{
	    Object o=oc.get(i);
	    unbind(o,trailStack);	// XXX trail these all together!

	}
	
    }
    
    public void unbindVarsWithCheck(ObjectContainer oc,TrailStack trailStack) {
	
	final int size=oc.size();
	for (int i=0; i<size; i++)
	{
	    Object o=oc.get(i);

	    if (/*isVariable(o) && */isBound(o))		
		unbind(o,trailStack);	// XXX trail these all together!

	}
	
    }

    public void unbind(final Object var,TrailStack trailStack) {
      unbind(var, trailStack, true);
    }
    
    public void unbind(final Object var,TrailStack trailStack, boolean remove) {
	
	if (trailStack!=null)
	{

	    final Object oldValue = variableTable.get(var);

	    if (oldValue==null) 
		throw new RuntimeException("Paranoia: oldValue==null");

	    trailStack.add(/*"VT.unbind() -> undo by binding "+getVariableName(var)+ " to "+oldValue,*/
			   new Runnable() { public void run() {

			       // XXX is this really enough?

			       bind(var,oldValue,
				    null /* no recursion, please! */);
			       
			   }});

	}

	VarObserver vo;

	// XXX more TrailStack stuff?

	// notify all observers
	// XXX TODO: the same thing is done in updateVariableObservers (??)
	ObjectContainer oc = getVariableObservers(var);
	if (oc!=null)
	{
	    final int size=oc.size();
	    for (int i=0; i<size; i++) {
		
		vo = (VarObserver)oc.get(i);
		vo.observer.setAt(var, vo.position);
		
	    }
	}

	// remove value from table
	if (remove) {
          variableTable.remove(var);
        }

	if (supportUnboundVars && !unboundVars.contains(var))
	    unboundVars.add(var);
	
    }
    
    public void addToBlock(Object var) {
	
	// XXX TrailStack!

	blockVars.add(var);

    }
    
    public void unbindAllBlockVars(TrailStack trailStack) {

	// XXX TrailStack! (?)
	
	unbindVars(blockVars,trailStack);

    }

    // not a good name XXX TODO
    public void setGlobalVars(ObjectContainer vars) {

	tmpGlobalVars = vars;

    }

    // not a good name XXX TODO
    public void resetGlobalVars() {

	tmpGlobalVars.removeAll(); // = new ObjectContainer();

    }

    public boolean isLocalVar(Object var) {
	return !tmpGlobalVars.contains(var);

    }

    public Object unchain(Object var) {

	while (isBound(var))
	{
	    Object binding=getBinding(var);
	    // if (isVariable(binding))
	    if ( binding != null && variableClasses.get(binding) != null)
		var=binding;
	    else 
		break;
	}
	
	return var;
    }

    public Object getValue(Object var) {

	while (var != null && isBound(var) && 
               variableClasses.get(var) != null) {
	    var =getBinding(var);
	}
	
	return var;
    }

    /**
       In this package, variables are represented as objects (@see
       java.lang.Object).  This Method tests, whether the given object
       is a pure java.lang.Object or a derived class. In the first
       case, it is a variable.  

       ... or something like that.
    */
    
    public boolean isVariable(Object o) {
	
/* modified by T. Wahls 7/15/2004
	return variableClasses.get(o) != null;
*/
        if (o == null) {
          return false;
        } else if (Util.isNonVar(o)) {
          return false;
        } else {
          return variableClasses.get(o) != null;
        }
                                                                                
    }

    // returns true if any of the objects inside the container is a
    // variable
    public boolean isAnyVariable(ObjectContainer oc)
    {
	final int size=oc.size();

/* modified by T. Wahls 7/15/2004
	for(int i =0; i < size; i++)
	    if (variableClasses.get(oc.get(i))!=null)
		return true;
	
	return false;
*/
	for(int i =0; i < size; i++)
	    if (isVariable(oc.get(i)))
		return true;
	
	return false;
    }
    
    // -------------------------------------------------------
    
    public void declare(Object var, String classString) {
	
	declare(var, classString, "_VAR"+(counter++));
	
    }
    
    public void declare(Object var, String classString, String varName) {

	// XXX TrailStack?
	
	Class c = null;    
	
	try {
		    
	    c = Class.forName(classString); 

	} catch(ClassNotFoundException e) {

	    System.out.println("could not find class "+classString);

	}
// removed by TAW 2/1/2007 
//	if (var.getClass().equals(Object.class)) { // is var an object?
	    variableClasses.put(var, c);
	    variableNames.put(var, varName);
	    variableObjects.put(varName, var);

	    if (supportUnboundVars)
		unboundVars.add(var);

/*
	}
	else
	{
	    throw new RuntimeException("Only objects of type java.lang.Object can be variables; tried to declare something of type "+var.getClass()+" as variable.");
	}
*/
	
    }

    public Object getVariableObject(String varName) {

	return variableObjects.get(varName);

    }

    public Class getVariableClass(Object var) {

	// TODO: VariableNotDeclaredException
	return (Class)variableClasses.get(var);
		
    }

    public String getVariableName(Object var) {

	// TODO: VariableNotDeclaredException
	return (String)variableNames.get(var); //+"@"+var;

    }
    
    // -------------------------------------------------------

    public ObjectContainer getEQConstraints() {

	ObjectContainer oc = new ObjectContainer();
	Iterator enu = variableTable.keySet().iterator();
	Object var;

	while (enu.hasNext()) {

	    var = enu.next();
	    Equality eq=new Equality(var, variableTable.get(var));
	    eq.setVTable(this);
	    oc.add(eq);

	}

	return oc;

    }

    // -------------------------------------------------------
    
    public int size() {

	return variableTable.size();
	
    }

    public String toString() {

	// these things are sorted to make the regression test work
	
	Iterator e = variableTable.keySet().iterator(); 
	ObjectContainer output=new ObjectContainer();
	
	while (e.hasNext()) {
	    
	    String table="";

	    Object key = e.next();
	    table += (String)variableNames.get(key)+"<-";
	    
	    Object binding = variableTable.get(key);
	    if (isVariable(binding))
		table += (String)variableNames.get(binding);
	    else
		table += ""+Util.JMLEString(binding);

	    output.add(table);
	    
	}  
	
	Util.sortContainer(output);

	boolean first=true;

	String rc="{";
	for (int t=0; t<output.size(); t++)
	{
	    if (first)
		first=false;
	    else
		rc+=", ";
	    rc+=output.get(t);
	}

	rc+="}";
	
	return rc;
	
    }
    
    public String classesToString() {
	
	return ""+variableClasses;

    }

    public String namesToString() {
	
	return ""+variableNames;

    }

    public String blockVarsToString() {
	
	return ""+blockVars;
		
    }


    public VTableState getVTableState() {

	return new VTableState((HashMap)variableTable.clone(), 
			       (HashMap)variableClasses.clone(), 
			       (HashMap)variableNames.clone(), 
			       (ObjectContainer)blockVars.clone(),
			       supportUnboundVars?(ObjectContainer)unboundVars.clone():null);
	
    }

    public void setVTableState(VTableState vts) {
	
	int i;
	Object var;
//	Enumeration keys;
	ObjectContainer vo, tmpvo;

	variableTable = (HashMap)(vts.variableTable.clone());
	variableClasses = (HashMap)(vts.variableClasses.clone());
	variableNames = (HashMap)(vts.variableNames.clone());
	blockVars = (ObjectContainer)(vts.blockVars.clone());

	if (supportUnboundVars)
	    unboundVars = (ObjectContainer)(vts.unboundVars.clone());

	// update all observers and used variables (will be added by
	// UserDefinedConstraintMemory.setUDCMState())

	removeAllObservers();
	usedVariables.clear();

// 	for (i=0; i<unboundVars.size(); i++) {

// 	    var = unboundVars.get(i);
// 	    updateVariableObservers(var);

// 	}

    }

	/**
	 * get variable table as string array
	 *
	 * @return a string array containing the bindings
	 */
    public String[] toStringArray() {
	
		int i = 0, len = variableTable.size();
		String arr[] = new String[len];

		Iterator e = variableTable.keySet().iterator();
		while(e.hasNext() && (i < len)) {
			arr[i] = "";
			Object key = e.next();
			arr[i] += (String)variableNames.get(key) + "=";
	  
			Object binding = variableTable.get(key);
                        if(isVariable(binding))
				arr[i] += (String)variableNames.get(binding);
			else
				arr[i] += ""+Util.JMLEString(binding);
			i++;
		}//end of while
		return arr;
    }//end of toStringArray

    // Returns a string which describes the variable. Either just the
    // variable name, or the name together with the value. If the
    // object is no variable, itself is returned.

     String variableToString(Object var)
    {
	if (isVariable(var))
	{
	    if (isBound(var))
	    {
		Object val=getBinding(var);
		if (isVariable(val))
		    val=getVariableName(val); // no recursion here
		return getVariableName(var)+"(="+Util.JMLEString(val)+")";
		// return getVariableName(var)+"(="+var+")";
	    }
	    else
		return getVariableName(var);
        } else
	    return Util.JMLEString(var);

    }

    // Returns true if one of the following holds:
    //    1. A is the same as B
    //    2. A is a variable, and A = B 
    //    3. B is a variable, and B = A
    // Does NOT check chains of variables.
    // XXX TODO either make this method check chains of variables, or remove it
    public boolean variablesAreEqualOrBound(Object var1,Object var2)
    {
	if (var1==null && var2==null) return true;
	if (var1==null || var2==null) return false;

	// Both are non-null, check for equality

	if (var1.equals(var2)) return true;

	// Check the bindings 

	if (isVariable(var1) &&
	    var2.equals(getBinding(var1)))
	    return true;

	if (isVariable(var2) &&
	    var1.equals(getBinding(var2)))
	    return true;
	    
	return false;
    }

    // Returns a collection which contains all variables that are
    // equal to "var" (no matter how long the chain is). Includes var.
    public ObjectContainer getEqualVars(Object var)
    {
	ObjectContainer result=new ObjectContainer(); // the result...
	HashMap hash=new HashMap();	// contains all variables that
					// have already been checked
	ObjectContainer candidates=new ObjectContainer(); // contains
							  // current
							  // candidates

	candidates.add(var);
	hash.put(var,var);

	// Process all candidates until no more are left

	while (candidates.size()>0)
	{

	    // Get the next candidate (X) and remove it from the candidates

	    Object cand=candidates.get(candidates.size()-1);
	    candidates.remove(candidates.size()-1);
	    
	    // We know that it is equal to the original variable, so
	    // it goes into the result list, and into the hash (so we
	    // don't check it again).
	    
	    result.add(cand);

	    // Is this candidate bound to another variable (X=Y)? If yes,
	    // then that variable is a candidate too.
                                                                                
            if (isBound(cand))
            {
                Object binding=getBinding(cand);
                if (isVariable(binding) && !hash.containsKey(binding))
                {
                    candidates.add(binding);
                    hash.put(binding,binding);
                }
            }

	    
//	    Object varcursor = var;
	    /* modified 6/28/05 by Ben Krause */
/*
	    while (isBound(varcursor))
	    {
		Object binding=getBinding(varcursor);
		if (isVariable(binding) && !hash.containsKey(binding))
		{
		    candidates.add(binding);
		    hash.put(binding,binding);
		    varcursor = binding;
		} else {
			varcursor = new Object();
		}
	    }
*/

	    // Is there some variable Y with Y=X? This here is very
	    // expensive... maybe we should have a hashtable of
	    // "inverse bindings".

	    // An experiment showed that a hashtable of reverse
	    // bindings is actually slower than this solution (e.g.,
	    // 5-queens).

	    for (Iterator e=variableTable.keySet().iterator(); e.hasNext();)
	    {
		Object someVar=e.next();
		if (isBound(someVar) && !hash.containsKey(someVar))
		{
// modified by T. Wahls 7/2/2005
		    Object binding=someVar;

                    while (binding != null && isBound(binding) && !hash.containsKey(binding)) {
                      binding = getBinding(binding);
                    }
		    if (binding != null && hash.containsKey(binding) && !someVar.equals(binding))
/*
		    Object binding=getBinding(var);
		    if (var.equals(binding))
*/
		    {
			candidates.add(someVar);
			hash.put(someVar,someVar);
		    }
		}
	    }	    
	}

	return result;
    }

// added by TAW 2/1/2007

  public Iterator getVariables() { 
    return variableClasses.keySet().iterator();
  }

  public void removeVariables() {
    Iterator enu = getVariables();
    while (enu.hasNext()) {
      Object var = enu.next();
      if (var instanceof org.jmlspecs.jmlexec.runtime.VarObject) {
        variableNames.remove(var);
        variableObserver.remove(var);
        variableTable.remove(var);
        enu.remove();
      }
    }
  }

}
