package org.jmlspecs.jmlexec.jack.evaluator;

/**
   Class nl (nested list)
   @version 1 (01/09/1999)
   @author Matthias SchmauB
*/

import java.util.Enumeration;

public class nl {

    ///////////////////////////////////////
    // fields

    // a nested list consists of an object called first, 
    // and a rest list

    // if list has size > 1 then first!=null and rest!=null

    // if list has size 1, then first is the object, and rest is a "nl()"

    // if list has size 0, then first==rest==null

    public Object first;
    public nl rest;

    ///////////////////////////////////////
    // constructors

    // constructs a list with an object and a rest list
    public nl(Object o, nl l) {

	first = o;
	rest = l;

    }

    public static nl makeNL(Object o, nl l) {
	return new nl(o, l);
    }

 /* public nl(Object o, Object l) {
	
	if (l instanceof nl) {
		first = o;
		rest = (nl) l;
	} else throw new RuntimeException("Rest must be an instance of nl.");
    }*/

    // constructs a list with onlu one object and an empty 
    // rest list
    public nl(Object o) {

	this(o, new nl());

    }

    // constructs an empty list
    public nl() {

	this(null, null);

    }

    ///////////////////////////////////////
    // class methods

    public static Object head(nl  n)
    {
	return n.first;
    }

    public static nl rest(nl n) { 

	return n.rest; 

    }

   public static nl setHead(nl n, Object o) {
	n.first = o;
	return n;
   }

   public static nl setRest(nl n, nl r) {
	n.rest = r;
	return n;
   }

    // Returns a new list with only the first <number> objects from this list.
    // If size(n)<number, then n itself is returned.
    //
    // Semantics:     list == firstN(list,n)  o  afterFirstN(list,n)    AND
    //                ( length(firstN(list,n)) == n  OR  length(list)<n )
    public static nl firstN(nl n,int number)
    {
	if (number==0) return new nl();

	ObjectContainer oc = new ObjectContainer();

	nl cur=n;

	while (!cur.isEmpty() && number>0)
	{
	    oc.add(cur.first);
	    cur=cur.rest;
	    number--;
	}

	if (number>0) 
	    return n;

 	return nl.tonl(oc);	
	
    }

    // Returns a new list with all but the first <number> objects from
    // this list.  If size(n)<number, then the empty list is returned.
    // If number==0, then n itself is returned. Else, a list is
    // returned which consists of the exact same objects as <n>, but
    // starting in between the start and the end (i.e., nothing is
    // copied).
    //
    // Semantics:     list == firstN(list,n)  o  afterFirstN(list,n)    AND
    //                ( length(firstN(list,n)) == n  OR  length(list)<n )
    public static nl afterFirstN(nl n,int number)
    {
	// skip the first <number> objects

	while (number>0 && n!=null)
	{
	    n=n.rest;
	    number--;
	}

	if (n==null) return new nl();

	return n;
    }

    public static boolean ground(nl l) {

	return true;

    }

    // transforms an ObjectContainer to a nested list
    public static nl tonl(ObjectContainer oc) {

	final int size=oc.size();

	nl rc=new nl();
	for (int t=size-1; t>=0; t--)
	    rc=new  nl(oc.get(t),rc);

	return rc;

// 	if (oc.size() > 0) {

// 	    ObjectContainer tmp = (ObjectContainer)oc.clone();
// 	    Object o = tmp.get(0); tmp.remove(o);

// 	    if (tmp.size() > 0)
// 		return new nl(o, nl.tonl(tmp));
// 	    else
// 		return new nl(o);

// 	} else
// 	    return new nl();

    }

    // transforms an Enumeration to a nested list. The enumeration is
    // used up in the process.
    public static nl tonl(Enumeration oc) {

	nl rc=new nl();
	while (oc.hasMoreElements())
	    rc=new  nl(oc.nextElement(),rc);

	return rc;

// 	if (oc.hasMoreElements()) {
		    
// 	    Object o = oc.nextElement();

// 	    if (oc.hasMoreElements())
// 		return new nl(o, nl.tonl(oc));
// 	    else
// 		return new nl(o);

// 	} else
// 	    return new nl();

    }

    // creates an ascending list of integers from min to max
    // if max<min, then the empty list is returned
    public static nl createIntegers(int min,int max)
    {
	nl rc=new nl();

	while (max>=min)
	{
	    rc=new nl(new Integer(max),rc);
	    max--;
	}

	return rc;

// 	if (max<min) return new nl();

// 	if (max==min) return new nl(new Integer(min));

// 	return new nl(new Integer(min),createIntegers(min+1,max));
    }

    // creates an descending list of integers from max to min
    // if max<min, then the empty list is returned
    public static nl createReverseIntegers(int max,int min)
    {
	nl rc=new nl();

	while (min<=max)
	{
	    rc=new nl(new Integer(min),rc);
	    min++;
	}

	return rc;

// 	if (max<min) return new nl();

// 	if (max==min) return new nl(new Integer(min));

// 	return new nl(new Integer(max),createReverseIntegers(max-1,min));
    }

    public static boolean notEmpty(nl n) { 

	return !n.isEmpty(); 

    }

    // tests, whether the given nested list is empty
    public static boolean isEmpty(nl l) {

	return l.isEmpty();//((l.first == null) && (l.rest == null));

    }

    // returns an intersection of the two given lists
    public static nl intersection(nl l1, nl l2) {

	ObjectContainer oc1 = l1.toObjectContainer();
	ObjectContainer oc2 = l2.toObjectContainer();
	ObjectContainer oc3 = oc1.intersection(oc2);

	return nl.tonl(oc3);
      
    }

    // returns the number of objects stored in the given list.
    public static int size(nl l) {

	// ARGH!!!!
	// return l.toObjectContainer().size();

	int size=0;
	nl current=l;

	while (current!=null)
	{
	    if (current.first!=null) // ==null on last item
		size++;

	    current=current.rest;
	}

	return size;
    }

    ///////////////////////////////////////
    // instance methods

    // tests, whether the current list is empty
    public boolean isEmpty() {

	return ((first == null) && (rest == null));

    }

    // transforms the current list into an ObjectContainer
    public ObjectContainer toObjectContainer() {	

	ObjectContainer oc = new ObjectContainer();

	nl cur=this;

	while (cur.rest!=null && cur.first!=null)
	{
	    oc.add(cur.first);
	    cur=cur.rest;
	}
	
// 	if (!isEmpty()) {

// 	    oc.add(first);	    
// 	    if (!rest.isEmpty())
// 		oc.concatenate(rest.toObjectContainer());

// 	}

	return oc;

    }

    // transforms the current list into a String
    public String toString() {

	return "["+toObjectContainer()+"]";

    }

    // creates a copy of the current list
    public Object clone() {

	if (rest==null)
	    return this;

 	return nl.tonl(toObjectContainer());	

//  	ObjectContainer oc = (ObjectContainer)(toObjectContainer().clone());
//  	return nl.tonl(oc);	

    }

    // removes the given object from the given list;
    // the result is a new nested list without the given object.
    public static nl remove(nl l, Object i) {

	ObjectContainer oc = l.toObjectContainer();

	oc.remove(i);

	return nl.tonl(oc);
    }

    public static nl cons(Object i, nl l) {
      return new nl(i, l);
    }


}
