/*
 * @(#)$Id: BagArrayList.java,v 1.1 2008/03/05 00:11:12 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlexec.runtime;

import java.util.*;

interface BagElem {
  public boolean sameKey(Object key);
  public Object getKey();
  public int getCount();
  public boolean countIsFirm();
  public void setFirm(boolean firm);
  public void setCount(int count);
}

class ObjectBagElem implements BagElem {
  private Object key;
  private int count;
  private boolean isFirm;

  public ObjectBagElem(Object key, int count, boolean isFirm) {
    this.key = key;
    this.count = count;
    this.isFirm = isFirm;
  }

  public boolean sameKey(Object key) {
    return this.key == key;
  }

  public Object getKey() {
    return key;
  }

  public int getCount() {
    return count;
  }

  public boolean countIsFirm() {
    return isFirm;
  }

  public void setFirm(boolean firm) {
    isFirm = firm;
  }

  public void setCount(int count) {
    this.count = count;
  }

  public String toString() {
    return "(" + key + ", " + count + ")";
  }
}

class EqualsBagElem extends ObjectBagElem {

  public EqualsBagElem(Object key, int count, boolean isFirm) {
    super(key, count, isFirm);
  }

  public boolean sameKey(Object key) {
    return getKey().equals(key);
  }
}

public class BagArrayList extends ArrayList {
  String type;
	
  public BagArrayList(String type) {
    super();
    this.type = type;
  }

  BagElem getElem(Object key) {
    Iterator it = iterator();
    while (it.hasNext()) {
      BagElem elem = (BagElem) it.next();
      if (elem.sameKey(key)) {
        return elem;
      }
    }
    return null;
  }
	
  public boolean contains(Object elem) {
    return getElem(elem) != null;
  }

  public boolean canAdd(Object key, int count, boolean isFirm) {
    BagElem elem = getElem(key);
    if (elem == null) {
      return true;
    } else {
      if (elem.countIsFirm()) {
        if (isFirm) {
          return count == elem.getCount();
        } else {
          return count <= elem.getCount();
        }
      } else {
        if (isFirm) {
          return count >= elem.getCount();
        }
      }
      return true;
    }
  }
      

  /*@ requires canAdd(key, count, isFirm); */
  public boolean add(Object key, int count, boolean isFirm) {
    BagElem elem = getElem(key);
    BagElem newElem;
    if (type.equals("JMLObjectBag")) {
      newElem = new ObjectBagElem(key, count, isFirm);
    } else {
      newElem = new EqualsBagElem(key, count, isFirm);
    }
    if (elem == null) {
      add(newElem); 
    } else if (!elem.countIsFirm()) {
      remove(elem);
      if (!isFirm) {
        int newCount = elem.getCount();
        if (count > newCount) newCount = count;
        newElem.setCount(newCount);
      }
      add(newElem);
    }
    return true;
  }

  public int bagSize() {
    int lstSize = super.size();
    int sz = 0;
    for (int i = 0; i < lstSize; i++) {
      BagElem elem = (BagElem) get(i);
      sz += elem.getCount();
    }
    return sz;
  }
	
}
