/*
 * @(#)$Id: JMLTool.java,v 1.2 2008/03/28 19:41:21 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlexec.runtime;

import org.jmlspecs.models.JMLCollection;
import org.jmlspecs.models.JMLIterator;
import org.jmlspecs.models.JMLObjectSet;
import org.jmlspecs.models.JMLObjectSequence;
import java.lang.reflect.*;
import java.util.ArrayList;
import java.util.Iterator;

public class JMLTool {

  public static boolean has(JMLCollection l1, java.lang.Object o1,
                            String typ) {
    if (l1 == null) {
      return false;
    } else if ((o1 instanceof MyPrimitive) || (o1 instanceof MyArray)) {
      JMLIterator it = l1.iterator();
      while (it.hasNext()) {
        if (o1.equals(it.next())) {
          return true;
        }
      }
      return false;
    } else {
      return l1.has(o1);
    }
  }

  public static boolean notHas(JMLCollection l1, java.lang.Object o1,
                               String typ) {
    if (l1 == null) {
      return false;
    } else {
      return !has(l1, o1, typ);
    }
  }

  public static MyInteger int_size(JMLCollection l1, String typ) {
    if (l1 == null) {
      return new MyInteger();
    } else {
      return new MyInteger(l1.int_size());
    }
  }

  public static boolean equals(Object l1, Object l2, String typ) {
    if (l1 == null) {
      return false;
    } else {
      return l1.equals(l2);
    }
  } 

  public static ArrayList combine(ArrayList a1, ArrayList a2, String typ) {
    ArrayList a3 = (ArrayList) a1.clone();
    Iterator it = a2.iterator();
    while (it.hasNext()) {
       Object ob = it.next();
       if (! a3.contains(ob)) a3.add(ob);
    }
    return a3;
  }

  public static boolean containsAllElems(ArrayList a1, JMLCollection l1,
                                         String typ) {
    Iterator it = a1.iterator();
    while (it.hasNext()) {
      Object ob = it.next();
      if (! l1.has(ob)) return false;
    }
    return true;
  }

  public static JMLCollection convertFrom(java.util.Collection col, String typ)
    throws Throwable {
    JMLCollection jc = emptyCollection(typ);
    Iterator it = col.iterator();
    while (it.hasNext()) {
      if (isMap(typ)) {
        Object next = it.next();
        jc = (JMLCollection) extend(jc, ObjUtil.getField(next, null,  "key"),
                                    ObjUtil.getField(next, null,  "value"),
                                    typ);
      } else if (isBag(typ) && 
                 col instanceof org.jmlspecs.jmlexec.runtime.BagArrayList) {
        BagElem next = (BagElem) it.next();
        for (int i = 0; i < next.getCount(); i++) {
          jc = (JMLCollection) insert(jc, next.getKey(), typ);
        }
      } else {
        jc = (JMLCollection) insert(jc, it.next(), typ);
      }
    }
    freshObjs.add(jc);
    return jc;
  }

  public static ArrayList newArrayList(String typ) {
    if (typ.equals("JMLObjectSet")) return new ObjectArrayList();
    else if (isSetMapOrRel(typ)) return new ArrayList();
    else if (isBag(typ)) {
       return new BagArrayList(typ);
    }
    else throw new IllegalArgumentException("Invalid set type: " + typ);
  }

  public static ArrayList add(ArrayList a1, Object ob, String typ) {
    if (isBag(typ)) {
      return addBag((BagArrayList) a1, ob, MyInteger.ONE, 
                    MyInteger.ZERO);
    } else if (a1.contains(ob)) {
      return a1;
    } else {
      ArrayList a2 = (ArrayList) a1.clone();
      a2.add(ob);
      return a2;
    }
  }

  public static MyInteger size(ArrayList a1) {
    if (a1 instanceof BagArrayList) {
      return new MyInteger(((BagArrayList) a1).bagSize());
    } else {
      return new MyInteger(a1.size());
    }
  }
    
  public static boolean isSet(String typ) {
    return typ.endsWith("Set");
  } 
    
  public static boolean isRel(String typ) {
    return typ.endsWith("Relation");
  }
    
  public static boolean isMap(String typ) {
    return typ.endsWith("Map");
  }
    
  public static boolean isSetMapOrRel(String typ) {
    return isSet(typ) || isMapOrRel(typ);
  }
    
  public static boolean isSetBagMapOrRel(String typ) {
    return isSet(typ) || isMapOrRel(typ) || isBag(typ);
  }
    
  public static boolean isMapOrRel(String typ) {
    return isMap(typ) || isRel(typ);
  }
    
  public static boolean isSeq(String typ) {
    return typ.endsWith("Sequence");
  } 
    
  public static boolean isBag(String typ) {
    return typ.endsWith("Bag");
  }
    
  public static boolean isSeqOrBag(String typ) {
    return isBag(typ) || isSeq(typ);
  }

  public static String keyType(String typ) {
    if (typ.substring(3, 4).equals("O")) {
      return "Object";
    } else if (typ.substring(3, 4).equals("E")) {
      return "Equals";
    } else {
      return "Value";
    }
  }

  public static String valueType (String typ) {
    int valInd = keyType(typ).length() + 5;
    if (typ.substring(valInd, valInd + 1).equals("O")) {
      return "Object";
    } else if (typ.substring(valInd, valInd + 1).equals("E")) {
      return "Equals";
    } else {
      return "Value";
    }
  }
    
  public static boolean notValueSeq(String typ) {
    return isSeq(typ) && ! typ.equals("JMLValueSequence");
  } 
    
  public static boolean isObjSeq(String typ) {
    return typ.equals("JMLObjectSequence");
  } 

  public static boolean notEquals(JMLCollection l1, JMLCollection l2, 
                                  String typ) {
    if (l1 == null) {
      return false;
    } else {
      return ! l1.equals(l2);
    }
  }

  public static JMLCollection emptyCollection(String typ) {
    try {
      Class colClass = Class.forName(MPATH + typ);
/*
      Field fld = colClass.getField("EMPTY");
      JMLCollection emp = (JMLCollection) fld.get(null);
*/
      Constructor con = colClass.getConstructor(new Class[0]);
      JMLCollection emp = (JMLCollection) con.newInstance(new Object[0]);
      freshObjs.add(emp);
      return emp;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Model type not found: " + typ);
    } catch (NoSuchMethodException e) {
      throw new IllegalArgumentException(
        "Model type has no default constructor: " + typ);
    } catch (InvocationTargetException e) {
      throw new IllegalArgumentException(
        "Model type has no default constructor: " + typ);
    } catch (IllegalAccessException e) {
      throw new IllegalArgumentException(
        "Model type has no public default constructor: " + typ);
    } catch (InstantiationException e) {
      throw new IllegalArgumentException(
        "Model type has no default constructor: " + typ);
    }
/*
    } catch (NoSuchFieldException e) {
      throw new IllegalArgumentException("Model type has no EMPTY: " + typ);
    } catch (IllegalAccessException e) {
      throw new IllegalArgumentException("Model type has no public EMPTY: " 
                                         + typ);
    }
*/
  }

  private static Object call(String name, JMLCollection l1, Class [] params,
                             Object [] args, String typ) throws Throwable {
    Class colClass = l1.getClass();
    try {
      Method emp = colClass.getMethod(name, params);
      return emp.invoke(l1, args);
    } catch (NoSuchMethodException e) {
      throw new IllegalArgumentException("Model type has no " + name + 
                                         ": " + colClass);
    } catch (IllegalAccessException e) {
      throw new IllegalArgumentException("Model type has no public " + name +
                                         ": " + colClass);
    }  catch (InvocationTargetException e) {
      throw e.getTargetException();
      // return Null.NullObj;
    } catch (Throwable t) {
      return Null.NullObj;
    }
  }
	
  public static boolean notEmpty(JMLCollection l, String typ) {
    if (l == null) {
      return false;
    } else {
      return l.int_size() != 0;
    }
  }

  public static boolean isEmpty(JMLCollection l1, String typ) throws Throwable {
    if (l1 == null) {
      return false;
    } else {
      return ((Boolean) call("isEmpty", l1, new Class[0],
               new Object[0], typ)).booleanValue();
    }
  }

  public static Object powerSet(JMLCollection l1, String typ) 
    throws Throwable {
    Object ps = call("powerSet", l1, new Class[0], new Object[0], typ);
    freshObjs.add(ps);
    return ps;
  }

  private static boolean equalElems(JMLCollection l1, JMLCollection l2) {
    if (l1.int_size() != l2.int_size()) {
      return false;
    } else {
      Iterator it = l1.iterator();
      while (it.hasNext()) {
        Object cur = it.next();
        Iterator it2 = l2.iterator();
        boolean occurs = false;
        while (it2.hasNext() && !occurs) {
          if (cur.equals(it2.next())) occurs = true;
        }
        if (!occurs) return false;
      }
      return true;
    }
  }

  public static Object reversePowerSet(JMLCollection l1, String typ) 
    throws Throwable {
    if (l1 == null) {
      return Null.NullObj;
    } else {
      Iterator it = l1.iterator();
      int thesize = (int)(Math.log(l1.int_size())/Math.log(2));
      JMLCollection cur;
      while (it.hasNext()) {
        try {
          cur = (JMLCollection) it.next();
        } catch (ClassCastException e) {
          //throw new IllegalArgumentException("Error: " + l1 + 
          //                                   " is not a power set!");
          return Null.NullObj;
        }
        if (cur.int_size() == thesize) {
          JMLCollection pow = (JMLCollection) powerSet(cur, typ);
          if (equalElems(pow, l1)) {
            return cur;
          }
        }
      }
   //   throw new IllegalArgumentException("Error: " + l1 + " is not a power set!");
          return Null.NullObj;
    }
  }

  public static Object insert(JMLCollection l1, Object ob, String typ) 
    throws Throwable {
    String argType = null;
    try {
      if (isMapOrRel(typ)) {
        argType = MPATH + pairType(typ);
      } else if (l1 instanceof org.jmlspecs.models.JMLValueSet ||
                 l1 instanceof org.jmlspecs.models.JMLValueBag) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { Class.forName(argType) };
      Object [] args = { ob };
      Object res = call("insert", l1, params, args, typ);
      freshObjs.add(res);
      return res;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + argType);
    } 
  }

  public static boolean containsAll(JMLCollection l1, java.util.Collection c,
                                    String typ) 
    throws Throwable {
    try {
      Class [] params = { Class.forName("java.util.Collection") };
      Object [] args = { c };
      return ((Boolean) call("containsAll", l1, params, 
                             args, typ)).booleanValue();
    } catch (ClassNotFoundException e) {
      throw new 
        IllegalArgumentException("Type not found: java.util.Collection");
    } 
  }

  public static Object toSet(JMLCollection l1, String typ) 
    throws Throwable {
    Object ts = call("toSet", l1, new Class[0], new Object[0], typ);
    freshObjs.add(ts);
    return ts;
  }

  public static Object toFunction(JMLCollection l1, String typ) 
    throws Throwable {
    Object tf = call("toFunction", l1, new Class[0], new Object[0], typ);
    freshObjs.add(tf);
    return tf;
  }

  public static java.lang.Object choose(JMLCollection l1, String typ) 
    throws Throwable {
    if (isMapOrRel(typ)) {
      // needed because the difference of two Relations or Maps is a
      // JMLValueSet, and choose is used in conjunction with difference
      // in one of the rules for remove
      if (!(l1 instanceof org.jmlspecs.models.JMLValueSet)) {
        l1 = (JMLCollection) toSet(l1, typ);
      }
      typ = "JMLValueSet";
    }
    return call("choose", l1, new Class[0], new Object[0], typ);
  }

  public static boolean isSubset(JMLCollection l1, JMLCollection l2,
                                 String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      if (isMapOrRel(typ)) {
        l1 = (JMLCollection) toSet(l1, typ);
        l2 = (JMLCollection) toSet(l2, typ);
        typ = "JMLValueSet";
      } 
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isSubset", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    }
  }

  public static boolean isNotSubset(JMLCollection l1, JMLCollection l2,
                                    String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      return ! isSubset(l1, l2, typ);
    }
  }

  public static boolean isProperSubset(JMLCollection l1, JMLCollection l2,
                                       String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      if (isMapOrRel(typ)) {
        l1 = (JMLCollection) toSet(l1, typ);
        l2 = (JMLCollection) toSet(l2, typ);
        typ = "JMLValueSet";
      } 
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isProperSubset", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    }
  }

  public static Object remove(JMLCollection l1, Object ob, String typ) 
    throws Throwable {
    String argType = null;
    try {
      if (isMapOrRel(typ)) {
        argType = MPATH + pairType(typ);
      } else if (l1 instanceof org.jmlspecs.models.JMLValueSet ||
                 l1 instanceof org.jmlspecs.models.JMLValueBag) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { Class.forName(argType) };
      Object [] args = { ob };
      Object rm = call("remove", l1, params, args, typ);
      freshObjs.add(rm);
      return rm;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + argType);
    } 
  }

  public static Object union(JMLCollection l1, JMLCollection l2,
                                    String typ) 
    throws Throwable {
    try {
      Class [] params = { Class.forName(MPATH + typ) };
      Object [] args = { l2 };
      Object un = call("union", l1, params, args, typ);
      freshObjs.add(un);
      return un;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + typ);
    } 
  }

  public static Object intersection(JMLCollection l1, JMLCollection l2,
                                           String typ) 
    throws Throwable {
    try {
      Class [] params = { Class.forName(MPATH + typ) };
      Object [] args = { l2 };
      Object in = call("intersection", l1, params, args, typ);
      freshObjs.add(in);
      return in;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + typ);
    } 
  }

  public static Object difference(JMLCollection l1, JMLCollection l2,
                                  String typ) 
    throws Throwable {
    try {
      Class [] params = { Class.forName(MPATH + typ) };
      Object [] args = { l2 };
      Object dif = call("difference", l1, params, args, typ);
      freshObjs.add(dif);
      return dif;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + typ);
    } 
  }

  public static JMLObjectSet boundSet(MyIntegerType L1,
                                      MyIntegerType L2) {
    JMLObjectSet res = new JMLObjectSet();
    for (long l =  L1.longValue(); l <= L2.longValue(); l++) {
       res = res.insert(L1);
       L1 = L1.succ();
    }
    return res;
  }

  public static boolean first(JMLCollection l1, Object o1, String typ) 
    throws Throwable {
    if (l1 == null) {
      return false;
    } else {
      Object o2 = first(l1, typ);
      if (typ.equals("JMLValueSequence")) {
        return o1.equals(o2);
      } else {
        return o1 == o2;
      }
    }
  }

  public static boolean last(JMLCollection l1, Object o1, String typ) 
    throws Throwable {
    if (l1 == null) {
      return false;
    } else {
      Object o2 = last(l1, typ);
      if (typ.equals("JMLValueSequence")) {
        return o1.equals(o2);
      } else {
        return o1 == o2;
      }
    }
  }

  public static java.lang.Object first(JMLCollection l1, String typ) 
    throws Throwable {
    return call("first", l1, new Class[0], new Object[0], typ);
  }

  public static boolean trailer(JMLCollection l1, JMLCollection l2,
                                String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      JMLCollection l3 = (JMLCollection) trailer(l1, typ);
      return l2.equals(l3);
    }
  }

  public static Object trailer(JMLCollection l1, String typ) 
    throws Throwable {
    return call("trailer", l1, new Class[0], new Object[0], typ);
  }

  public static Object insertFront(JMLCollection l1, java.lang.Object o1,
                                   String typ) 
    throws Throwable {
    String argType = null;
    try {
      if (l1 instanceof org.jmlspecs.models.JMLValueSequence) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { Class.forName(argType) };
      Object [] args = { o1 };
      Object inf = call("insertFront", l1, params, args, typ);
      freshObjs.add(inf);
      return inf;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + argType);
    } 
  }

  public static Object consSeq(java.lang.Object o1, JMLCollection l1,
                               String typ) 
    throws Throwable {
    return insertFront(l1, o1, typ);
  }

  public static MyInteger count(JMLCollection l1, Object o1, String typ) 
    throws Throwable {
    if (l1 == null) {
      return MyInteger.ZERO;
    } else {
      String argType = null;
      try {
        if (l1 instanceof org.jmlspecs.models.JMLValueSequence ||
            l1 instanceof org.jmlspecs.models.JMLValueBag) {
          argType = MPATH + "JMLType";
        } else {
          argType = "java.lang.Object";
        }
        Class [] params = { Class.forName(argType) };
        Object [] args = { o1 };
        return new MyInteger(((Integer) call("count", l1, params, args,
                                             typ)).intValue());
      } catch (ClassNotFoundException e) {
        throw new IllegalArgumentException("Type not found: " + argType);
      } 
    } 
  }

  public static Object reverse(JMLCollection l1, String typ) 
    throws Throwable {
    Object rev = call("reverse", l1, new Class[0], new Object[0], typ);
    freshObjs.add(rev);
    return rev;
  }

  public static Object last(JMLCollection l1, String typ) 
    throws Throwable {
    return call("last", l1, new Class[0], new Object[0], typ);
  }

  public static Object header(JMLCollection l1, String typ) 
    throws Throwable {
    Object head = call("header", l1, new Class[0], new Object[0], typ);
    freshObjs.add(head);
    return head;
  }

  public static Object insertBack(JMLCollection l1, Object ob,
                                         String typ) 
    throws Throwable {
    String argType = null;
    try {
      if (l1 instanceof org.jmlspecs.models.JMLValueSequence) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { Class.forName(argType) };
      Object [] args = { ob };
      Object bk = call("insertBack", l1, params, args, typ);
      freshObjs.add(bk);
      return bk;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + argType);
    } 
  }

  public static Object itemAt(JMLCollection l1, MyInteger i1, String typ) 
    throws Throwable {
    Class [] params = { int.class };
    Object [] args = { new Integer(i1.intValue()) };
    return call("itemAt", l1, params, args, typ);
  }

  public static MyInteger indexOf(JMLCollection l1, Object o1, String typ) 
    throws Throwable {
    if (l1 == null) {
      return MyInteger.ZERO;
    } else {
      String argType = null;
      try {
        if (l1 instanceof org.jmlspecs.models.JMLValueSequence) {
          argType = MPATH + "JMLType";
        } else {
          argType = "java.lang.Object";
        }
        Class [] params = { Class.forName(argType) };
        Object [] args = { o1 };
        return new MyInteger(((Integer) call("indexOf", l1, params, args,
                                             typ)).intValue());
      } catch (ClassNotFoundException e) {
        throw new IllegalArgumentException("Type not found: " + argType);
      } 
    }
  }

  public static Object insertAfterIndex(JMLCollection l1, MyInteger i1,
                                        Object o1, String typ) 
    throws Throwable {
    String argType = null;
    try {
      if (l1 instanceof org.jmlspecs.models.JMLValueSequence) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { int.class, Class.forName(argType) };
      Object [] args = { new Integer(i1.intValue()), o1 };
      Object ia = call("insertAfterIndex", l1, params, args, typ);
      freshObjs.add(ia);
      return ia;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + argType);
    } 
  }

  public static Object removeItemAt(JMLCollection l1, MyInteger i1, String typ) 
    throws Throwable {
    Class [] params = { int.class };
    Object [] args = { new Integer(i1.intValue()) };
    Object rm = call("removeItemAt", l1, params, args, typ);
    freshObjs.add(rm);
    return rm;
  }

  public static Object concat(JMLCollection l1, JMLCollection l2, String typ) 
    throws Throwable {
    Class [] params = { l1.getClass() };
    Object [] args = { l2 };
    Object cat = call("concat", l1, params, args, typ);
    freshObjs.add(cat);
    return cat;
  }

  public static Object removePrefix(JMLCollection l1, MyInteger i1, String typ) 
    throws Throwable {
    Class [] params = { int.class };
    Object [] args = { new Integer(i1.intValue()) };
    Object suf = call("removePrefix", l1, params, args, typ);
    freshObjs.add(suf);
    return suf;
  }

  public static Object prefix(JMLCollection l1, MyInteger i1,
                                     String typ) 
    throws Throwable {
    Class [] params = { int.class };
    Object [] args = { new Integer(i1.intValue()) };
    Object pref = call("prefix", l1, params, args, typ);
    freshObjs.add(pref);
    return pref;
  }

  public static Object suffix(JMLCollection l1, MyInteger i1,
                                     String typ) 
    throws Throwable {
    Class [] params = { int.class };
    Object [] args = { new Integer(i1.intValue()) };
    Object suf = call("suffix", l1, params, args, typ);
    freshObjs.add(suf);
    return suf;
  }

  public static boolean isPrefix(JMLCollection l1, JMLCollection l2,
                                 String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isPrefix", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    }
  }

  public static boolean isProperPrefix(JMLCollection l1, JMLCollection l2,
                                       String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isProperPrefix", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    } 
  }

  public static boolean isSubsequence(JMLCollection l1, JMLCollection l2,
                                      String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isSubsequence", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    } 
  }

  public static boolean isSupersequence(JMLCollection l1, JMLCollection l2,
                                      String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isSupersequence", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    }
  }
  
  public static boolean isProperSubsequence(JMLCollection l1, JMLCollection l2,
                                            String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isProperSubsequence", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    } 
  }

  public static boolean isProperSupersequence(JMLCollection l1, 
                                              JMLCollection l2,
                                              String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isProperSupersequence", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    }
  }

  public static boolean isInsertionInto(JMLCollection l1, JMLCollection l2,
                                        java.lang.Object ob, String typ) 
    throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      String argType = null;
      try {
        if (l1 instanceof org.jmlspecs.models.JMLValueSet) {
          argType = MPATH + "JMLType";
        } else {
          argType = "java.lang.Object";
        }
        Class [] params = { Class.forName(MPATH + typ), 
                            Class.forName(argType) };
        Object [] args = { l2, ob };
        return ((Boolean) call("isInsertionInto", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + MPATH + typ);
      } 
    }
  }

  public static Object replaceItemAt(JMLCollection l1, MyInteger i1,
                                     java.lang.Object ob, String typ) 
    throws Throwable {
    String argType = null;
    try {
      if (l1 instanceof org.jmlspecs.models.JMLValueSequence) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { int.class, Class.forName(argType) };
      Object [] args = { new Integer(i1.intValue()), ob };
      Object rep = call("replaceItemAt", l1, params, args, typ);
      freshObjs.add(rep);
      return rep;
    } catch (ClassNotFoundException e) {
      throw new 
        IllegalArgumentException("Type not found: " + argType);
    } 
  }

  public static String pairType(String typ) {
    String keyType = keyType(typ);
    String valType = valueType(typ);
    return "JML" + keyType + valType + "Pair";
  }

  public static Object toPair(Object key, Object val, String typ) {
    String elemType = pairType(typ);
    String keyType = keyType(typ);
    String valType = valueType(typ);
    try {
      Class elemClass = Class.forName(MPATH + elemType);
      if (keyType.equals("Value")) {
        keyType = MPATH + "JMLType";
      } else {
        keyType = "java.lang.Object";
      }
      if (valType.equals("Value")) {
        valType = MPATH + "JMLType";
      } else {
        valType = "java.lang.Object";
      }
      Class [] params = { Class.forName(keyType), Class.forName(valType) };
      Constructor con = elemClass.getConstructor(params);
      Object [] args = { key, val };
      Object pair = con.newInstance(args);
      freshObjs.add(pair);
      return pair;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + elemType);
    } catch (NoSuchMethodException e) {
      throw new IllegalArgumentException("Type has no 2 arg constructor: " +
                                         elemType);
    } catch (InstantiationException e) {
      throw new IllegalArgumentException("Could not create instance of " +
                                         elemType);
    } catch (IllegalAccessException e) {
      throw new IllegalArgumentException(
        "Type has no public 2 arg constructor: " + elemType);
    }  catch (InvocationTargetException e) {
      throw new IllegalArgumentException("Bad invocation target for: " +
                                         elemType);
    }
  }

  public static Object extend(JMLCollection l1, Object key, Object val, 
                              String typ) 
    throws Throwable {
    String keyType = keyType(typ);
    String valType = valueType(typ);
    try {
      if (keyType.equals("Value")) {
        keyType = MPATH + "JMLType";
      } else {
        keyType = "java.lang.Object";
      }
      if (valType.equals("Value")) {
        valType = MPATH + "JMLType";
      } else {
        valType = "java.lang.Object";
      }
      Class [] params = { Class.forName(keyType), Class.forName(valType) };
      Object [] args = { key, val };
      Object ex = call("extend", l1, params, args, typ);
      freshObjs.add(ex);
      return ex;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + keyType +
                                         " or " + valType);
    } 
  }

  public static Object inverse(JMLCollection l1, String typ) 
    throws Throwable {
    Object inv = call("inverse", l1, new Class[0], new Object[0], typ);
    freshObjs.add(inv);
    return inv;
  }

  public static boolean stillFunction(Object newPair, ArrayList lst, 
                                      String typ) {
    if (!isMap(typ)) return true;
    else {
      String keyType = JMLTool.keyType(typ);
      String valType = JMLTool.valueType(typ);
      Object key = ObjUtil.getField(newPair, null,  "key");
      Object val = ObjUtil.getField(newPair, null,  "value");
      Iterator it = lst.iterator();
      while (it.hasNext()) {
        Object elem = it.next();
        Object key2 = ObjUtil.getField(elem, null,  "key");
        boolean foundKey;
        if (keyType.equals("Object")) {
          foundKey = key == key2;
        } else {
          foundKey = key.equals(key2);
        }
        if (foundKey) {
          Object val2 = ObjUtil.getField(elem, null,  "value");
          boolean sameVal;
          if (valType.equals("Object")) {
            sameVal = val == val2;
          } else {
            sameVal = val.equals(val2);
          }
          if (!sameVal) return false;
        }
      }
      return true;
    }
  }

  public static boolean isaFunction(JMLCollection l1, String typ) 
    throws Throwable {
    if (l1 == null) {
      return false;
    } else {
      return ((Boolean) call("isaFunction", l1, new Class [0], 
                             new Object [0], typ)).booleanValue();
    }
  }

  public static boolean isDefinedAt(JMLCollection l1, Object key, String typ) 
    throws Throwable {
    if (l1 == null) {
      return false;
    } else {
      String keyType = keyType(typ);
      try {
        if (keyType.equals("Value")) {
          keyType = MPATH + "JMLType";
        } else {
          keyType = "java.lang.Object";
        }
        Class [] params = { Class.forName(keyType) };
        Object [] args = { key };
        return ((Boolean) call("isDefinedAt", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new IllegalArgumentException("Type not found: " + keyType);
      } 
    }
  }

  public static Object restrictDomainTo(JMLCollection l1, JMLCollection l2,
                                        String typ) 
    throws Throwable {
      Class [] params = { l2.getClass() };
      Object [] args = { l2 };
      Object res = call("restrictDomainTo", l1, params, args, typ);
      freshObjs.add(res);
      return res;
  }

  public static Object restrictRangeTo(JMLCollection l1, JMLCollection l2,
                                       String typ) 
    throws Throwable {
      Class [] params = { l2.getClass() };
      Object [] args = { l2 };
      Object res = call("restrictRangeTo", l1, params, args, typ);
      freshObjs.add(res);
      return res;
  }

  public static Object elementImage(JMLCollection l1, Object key, String typ) 
    throws Throwable {
    String keyType = keyType(typ);
    try {
      if (keyType.equals("Value")) {
        keyType = MPATH + "JMLType";
      } else {
        keyType = "java.lang.Object";
      }
      Class [] params = { Class.forName(keyType) };
      Object [] args = { key };
      Object elem = call("elementImage", l1, params, args, typ);
      freshObjs.add(elem);
      return elem;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + keyType);
    } 
  }

  public static JMLCollection makePairs(Object key, JMLCollection values,
                                        String typ) throws Throwable {
    JMLCollection r1 = emptyCollection(typ);
    Iterator it = values.iterator();
    while (it.hasNext()) {
      Object pair = toPair(key, it.next(), typ);
      r1 = (JMLCollection) insert(r1, pair, typ);
    }
    freshObjs.add(r1);
    return r1;
  }

  public static String resultSetType(String relType) {
    return "JML" + valueType(relType) + "Set";
  }

  public static Object apply(JMLCollection l1, Object key, String typ) 
    throws Throwable {
    String keyType = keyType(typ);
    try {
      if (keyType.equals("Value")) {
        keyType = MPATH + "JMLType";
      } else {
        keyType = "java.lang.Object";
      }
      Class [] params = { Class.forName(keyType) };
      Object [] args = { key };
      return call("apply", l1, params, args, typ);
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + keyType);
    } 
  }

  public static boolean canAddBag(BagArrayList ba, Object key, MyInteger count,
                              MyInteger firm) {
    return ba.canAdd(key, count.intValue(), firm.intValue() == 1);
  }

  /*@ requires canAddBag(ba, key, count, firm); */
  public static BagArrayList addBag(BagArrayList ba, Object key, 
                                    MyInteger count, MyInteger firm) {
    BagArrayList ba2 = (BagArrayList) ba.clone();
    ba2.add(key, count.intValue(), firm.intValue() == 1);
    return ba2;
  }

  public static boolean isSubbag(JMLCollection l1, JMLCollection l2,
                                 String typ) throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isSubbag", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    }
  }

  public static boolean isProperSubbag(JMLCollection l1, JMLCollection l2,
                                       String typ) throws Throwable {
    if (l1 == null || l2 == null) {
      return false;
    } else {
      try {
        Class [] params = { Class.forName(MPATH + typ) };
        Object [] args = { l2 };
        return ((Boolean) call("isProperSubbag", l1, params, 
                               args, typ)).booleanValue();
      } catch (ClassNotFoundException e) {
        throw new 
          IllegalArgumentException("Type not found: " + typ);
      } 
    }
  }

  public static Object remove2(JMLCollection l1, Object ob, MyInteger i,
                               String typ) throws Throwable {
    String argType = null;
    try {
      if (l1 instanceof org.jmlspecs.models.JMLValueBag) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { Class.forName(argType), int.class };
      Object [] args = { ob, new Integer(i.intValue()) };
      Object rm = call("remove", l1, params, args, typ);
      freshObjs.add(rm);
      return rm;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + argType);
    } 
  }

  public static Object removeAll(JMLCollection l1, Object ob,
                                 String typ) throws Throwable {
    String argType = null;
    try {
      if (l1 instanceof org.jmlspecs.models.JMLValueBag) {
        argType = MPATH + "JMLType";
      } else {
        argType = "java.lang.Object";
      }
      Class [] params = { Class.forName(argType) };
      Object [] args = { ob };
      Object rem = call("removeAll", l1, params, args, typ);
      freshObjs.add(rem);
      return rem;
    } catch (ClassNotFoundException e) {
      throw new IllegalArgumentException("Type not found: " + argType);
    } 
  }

	
  private static String MPATH = "org.jmlspecs.models.";

  public static ObjectArrayList freshObjs = null;

}
