/*
 * @(#)$Id: Map.java,v 1.2 2008/03/07 21:48:46 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlexec.runtime;

/*
class AlreadyOccursException extends RuntimeException {
  public AlreadyOccursException() {}
}
*/

public class Map {
  private java.util.List elems;

  public class Entry {
    private Object key;
    private Object value;

    public Entry(Object k, Object v) {
      key = k;
      value = v;
    }

    public Object getKey() {return key;}
    public Object getValue() {return value;}
    public String toString() {
      return "(" + key + " -> " + value + ")";
    }
  }

  public Map() {elems = new java.util.ArrayList();}

  public void clear() {elems = new java.util.ArrayList();}

  public void put(Object key, Object value) {
     if (containsKey(key)) throw new AlreadyOccursException();
     else elems.add(new Entry(key, value));
  }

  public int size() {return elems.size();}

  public boolean containsKey(Object key) {
    boolean ret = false;
    java.util.Iterator iter = iterator();
    while (iter.hasNext() && !ret) {
       if (((Entry) iter.next()).getKey() == key) ret = true;
    }
    return ret;
  }

  public boolean containsValue(Object val) {
    boolean ret = false;
    java.util.Iterator iter = iterator();
    while (iter.hasNext() && !ret) {
       if (((Entry) iter.next()).getValue() == val) ret = true;
    }
    return ret;
  }

  public Object get(Object key) {
    java.util.Iterator iter = iterator();
    Entry e;
    while (iter.hasNext()) {
      e = (Entry) iter.next();
      if (e.getKey() == key) return e.getValue();
    }
    return null;
  }

  public Entry get(int i) {
    return (Entry) elems.get(i);
  }

  public Object remove(Object var) {
    java.util.Iterator iter = iterator();
    while (iter.hasNext()) {
       if (((Entry) iter.next()).getKey() == var) {
         iter.remove();
         return var;
       }
    }
    return null;
  }

  public Object remove(int pos) {
    Object ob = elems.get(pos);
    elems.remove(pos);
    return ob;
  }

  public Object getKey(Object val) {
    java.util.Iterator iter = iterator();
    Entry e;
    while (iter.hasNext()) {
      e = (Entry) iter.next();
      if (e.getValue() == val) return e.getKey();
    }
    return null;
  }

  public java.util.Iterator iterator() {
    return new java.util.Iterator() {
      private int pos = 0;
      public boolean hasNext() { return pos < elems.size(); }
      public Object next() throws java.util.NoSuchElementException {
        if (pos >= elems.size()) {
          throw new java.util.NoSuchElementException();
        } else {
          int oldp = pos;
          pos++;
          return elems.get(oldp);
        }
      }
      public void remove() { 
        Map.this.remove(pos - 1);
      }
    };
  }

  public String toString() {
    String res = "{";
    boolean first = true;
    java.util.Iterator iter = iterator();
    while (iter.hasNext()) {
      if (!first) res+=", "; else first = false;
      res += iter.next().toString();
    }
    res+="}";
    return res;
  }

  public static void main(String argv[]) {
    Map m = new Map();
    Integer i = new Integer(2);
    m.put(i, new Integer(3));
    m.put(new Integer(3), new Integer(4));
    System.out.println(m.get(new Integer(2)));
    System.out.println(m.get(i));
    java.util.Iterator iter = m.iterator();
    while(iter.hasNext()) {
       System.out.println(iter.next());
    }
    System.out.println(m);
  }
}
