/*
 * @(#)$Id: MyArray.java,v 1.2 2008/03/28 19:41:21 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlexec.runtime;

import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem;

public class MyArray {
    private Object[] theArray;
    private Object oldArray = null; // original value converted from
    // created from use of length constraint
    private boolean newFromLength = false; 
    private ConstraintSystem csref; // the current constraint system
	
	// Initializes a new empty array of null Objects
	public MyArray(int size, ConstraintSystem cs) {
          theArray = new Object[size];
          csref = cs;
          JMLTool.freshObjs.add(this);
	}

	public MyArray(int size, boolean fromLength, ConstraintSystem cs) {
          this(size, cs);
          newFromLength = fromLength;
          csref = cs;
	}
	
	public MyArray(org.jmlspecs.models.JMLCollection c, 
                       ConstraintSystem cs) {
		java.util.Iterator it = c.iterator();
		java.util.ArrayList arr = new java.util.ArrayList();
		while (it.hasNext()) {
			arr.add(it.next());
		}
		theArray = arr.toArray();
          csref = cs;
          JMLTool.freshObjs.add(this);
	}

    public MyArray(Object ob, ConstraintSystem cs) {
      Class cls = ob.getClass();
      if (cls.isArray()) {
        oldArray = ob;
        Class comp = cls.getComponentType();
        if (comp.equals(char.class)) {
          char [] ar = (char []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyChar(ar[i]);
          }
        } else if (comp.equals(byte.class)) {
          byte [] ar = (byte []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyByte(ar[i]);
          }
        } else if (comp.equals(short.class)) {
          short [] ar = (short []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyShort(ar[i]);
          }
        } else if (comp.equals(int.class)) {
          int [] ar = (int []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyInteger(ar[i]);
          }
        } else if (comp.equals(long.class)) {
          long [] ar = (long []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyLong(ar[i]);
          }
        } else if (comp.equals(float.class)) {
          float [] ar = (float []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyFloat(ar[i]);
          }
        } else if (comp.equals(double.class)) {
          double [] ar = (double []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyDouble(ar[i]);
          }
        } else if (comp.equals(boolean.class)) {
          boolean [] ar = (boolean []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            theArray[i] = new MyBoolean(ar[i]);
          }
        } else {
          Object [] ar = (Object []) ob;
          theArray = new Object [ar.length];
          for (int i = 0; i < ar.length; i++) {
            if (ar[i] == null) {
              theArray[i] = Null.NullObj;
            } else if (ar[i].getClass().isArray()) {
              theArray[i] = new MyArray(ar[i], cs);
            } else {
              theArray[i] = ar[i];
            }
          }
        }
      } else {
        throw new RuntimeException("Fatal error: " + ob.getClass() + 
                                   " is not an array type!");
      }
      csref = cs;
      JMLTool.freshObjs.add(this);
   }

	public void set(int i, Object o) {
		theArray[i] = o;
	}
	
	public Object get(int i) {
          if (i >= 0 && i < theArray.length) {
            return theArray[i];
          } else {
            return Null.NullObj;
          }
	}
	
	public int length() {
		return theArray.length;
	}
	
	public static MyInteger slength(Object ma1) {
          if (ma1 == Null.NullObj) {
             return MyInteger.ZERO;
          } else {
	     return new MyInteger(((MyArray) ma1).length());
          }
	}
	
	public static Object arrayindex(Object ma1, MyInteger i) {
          if (ma1 == Null.NullObj) {
             return Null.NullObj;
          } else {
		Object ob = ((MyArray) ma1).get(i.intValue());
                if (ob == null) {
                  return Null.NullObj;
                } else {
                  return ob;
                }
          }
	}

    private Object getVal(int ind, ConstraintSystem cs) {
      Object obj = get(ind);
      if (obj instanceof Variable) {
        Object val = cs.getVarValue(((Variable) obj).var);
        set(ind, val);
        return val;
      } else {
        return obj;
      }
    }

    public static Object convertTo(MyArray ma, Class cls, ConstraintSystem cs) {
      if (ma == null) return null;
      else if (!cls.isArray()) {
        throw new RuntimeException("Fatal error: " + cls + 
                                   " is not an array type!");
      } else {
        int len = ma.length();
        Class compType = cls.getComponentType();
        if (compType.equals(char.class)) {
          char [] ar = (char [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyChar) ma.getVal(i, cs)).charValue();
          }
          return ar;
        } else if (compType.equals(byte.class)) {
          byte [] ar = (byte [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyByte) ma.getVal(i, cs)).byteValue();
          }
          return ar;
        } else if (compType.equals(short.class)) {
          short [] ar = (short [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyShort) ma.getVal(i, cs)).shortValue();
          }
          return ar;
        } else if (compType.equals(int.class)) {
          int [] ar = (int [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyInteger) ma.getVal(i, cs)).intValue();
          }
          return ar;
        } else if (compType.equals(long.class)) {
          long [] ar = (long [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyLong) ma.getVal(i, cs)).longValue();
          }
          return ar;
        } else if (compType.equals(float.class)) {
          float [] ar = (float [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyFloat) ma.getVal(i, cs)).floatValue();
          }
          return ar;
        } else if (compType.equals(double.class)) {
          double [] ar = (double [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyDouble) ma.getVal(i, cs)).doubleValue();
          }
          return ar;
        } else if (compType.equals(boolean.class)) {
          boolean [] ar = (boolean [] ) 
                         java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = ((MyBoolean) ma.getVal(i, cs)).booleanValue();
          }
          return ar;
        } else if (compType.isArray()) {
          Object [] ar = (Object [])
                           java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            ar[i] = convertTo((MyArray) ma.getVal(i, cs), compType, cs);
          }
          return ar;
        } else { // general object
          Object [] ar = (Object [])
                           java.lang.reflect.Array.newInstance(compType, len);
          for (int i = 0; i < len; i++) {
            Object ob = ma.getVal(i, cs);
            ar[i] = ob == Null.NullObj? null: ob;
          }
          return ar;
        }
      }
    }
	
    public void convertBack(Class oldCls, ConstraintSystem cs) {
      try {
      if (oldArray == null) {
        oldArray = convertTo(this, oldCls, cs);
      } else if (oldArray.getClass().isArray()) {
        Class comp = oldArray.getClass().getComponentType();
        if (comp.equals(char.class)) {
          char [] ar = (char []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            ar[i] = ((MyChar) getVal(i, cs)).charValue();
          }
        } else if (comp.equals(byte.class)) {
          byte [] ar = (byte []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            ar[i] = ((MyByte) getVal(i, cs)).byteValue();
          }
        } else if (comp.equals(short.class)) {
          short [] ar = (short []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            ar[i] = ((MyShort) getVal(i, cs)).shortValue();
          }
        } else if (comp.equals(int.class)) {
          int [] ar = (int []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            ar[i] = ((MyInteger) getVal(i, cs)).intValue();
          }
        } else if (comp.equals(float.class)) {
          float [] ar = (float []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            ar[i] = ((MyFloat) getVal(i, cs)).floatValue();
          }
        } else if (comp.equals(double.class)) {
          double [] ar = (double []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            ar[i] = ((MyDouble) getVal(i, cs)).doubleValue();
          }
        } else if (comp.equals(boolean.class)) {
          boolean [] ar = (boolean []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            ar[i] = ((MyBoolean) getVal(i, cs)).booleanValue();
          }
        } else {
          Object [] ar = (Object []) oldArray;
          for (int i = 0; i < ar.length; i++) {
            Object elem = getVal(i, cs);
            if (elem == null || elem == Null.NullObj) {
              ar[i] = null;
            } else if (elem instanceof MyArray) {
              ((MyArray) getVal(i, cs)).convertBack(
                  oldCls.getComponentType(), cs);
            } else {
              ar[i] = getVal(i, cs);
            }
          }
        }

      } else {
        throw new RuntimeException("Fatal error: " + oldArray.getClass() + 
                                   " is not an array type!");
      }
      } catch (ClassCastException e) {
        throw new NoValueException("Post-state value in array not bound!");
      }
    }

    public void setValues(ConstraintSystem cs) {
      for (int i = 0; i < theArray.length; i++) {
        theArray[i] = getVal(i, cs);
        Object val = theArray[i];
        if (val instanceof MyArray) {
          ((MyArray) val).setValues(cs);
        }
      }
    }

	public Object oldArray() {
		return oldArray;
	}
	
	public ArrayList toArrayList() {
		ArrayList arr = new ArrayList();
		for (int i = 0; i < theArray.length; i++) {
			arr.add(i, theArray[i]);
		}
                JMLTool.freshObjs.add(arr);
		return arr;
	}
	
	public String toString() {
		String str = "[";
		int i;
		for (i = 0; i < theArray.length-1; i++) {
			str += theArray[i] + ", ";
		}
		str += theArray[i] + "]";
		return str;
	}

   public static org.jmlspecs.models.JMLObjectSequence toSequence(MyArray ma) {
     org.jmlspecs.models.JMLObjectSequence ret =
         org.jmlspecs.models.JMLObjectSequence.convertFrom(ma.theArray);
     JMLTool.freshObjs.add(ret);
     return ret;
   }

   public boolean allGround(ConstraintSystem cs) {
     try {
       for (int i = 0; i < theArray.length; i++) {
         Object val = cs.getVarValue(theArray[i]);
         if (val instanceof Variable) {
           val = cs.getVarValue(((Variable) val).var);
         }
         if (val instanceof MyArray) {
           if (!((MyArray) val).allGround(cs)) {
             return false;
           }
         }
       }
       return true;
     } catch (NoValueException e) {
       return false;
     }
   }

   private boolean equalArrays(MyArray ma, boolean guard) {
     if (ma == null) {
       return false;
     } else {
       if (newFromLength || ma.newFromLength) {
         if (length() == ma.length()) {
           for (int i = 0; i < length(); i++) {
             Object ob1 = get(i);
             Object ob2 = ma.get(i);
             if (ob1 instanceof Variable) {
               ob1 = ((Variable) ob1).var;
             }
             if (ob2 instanceof Variable) {
               ob2 = ((Variable) ob2).var;
             }
             if (ob1 instanceof VarObject && csref.hasVarValue(ob1)) {
               ob1 = csref.getVarValue(ob1);
             }
             if (ob2 instanceof VarObject && csref.hasVarValue(ob2)) {
               ob2 = csref.getVarValue(ob2);
             }
             if (ob1 instanceof VarObject || ob2 instanceof VarObject) {
               if (guard) return false;
             } else if (ob1 instanceof MyPrimitive && !ob1.equals(ob2)) {
               return false;
             } else if (ob2 instanceof MyPrimitive && !ob2.equals(ob1)) {
               return false;
             } else if (ob1 instanceof MyArray && (!(ob2 instanceof MyArray) || 
                !((MyArray) ob1).equalArrays((MyArray) ob2, guard))) {
               return false;
             } else if (ob2 instanceof MyArray && (!(ob1 instanceof MyArray) || 
                !((MyArray) ob2).equalArrays((MyArray) ob1, guard))) {
               return false;
             } else if (ob1 == null && ob2 != null && ob2 != Null.NullObj) {
               return false;
             } else if (ob1 == Null.NullObj && ob2 != null && 
                        ob2 != Null.NullObj) {
               return false;
             } else if (ob1 != ob2) {
               return false;
             }
           }
           return true;
         }
       } else {
         return ma == this;
       } 
     } 
     return false;
   }

   public void unify(MyArray ma) {
     if (newFromLength) {
       oldArray = ma.oldArray;
     } else if (ma.newFromLength) {
       ma.oldArray = oldArray;
     } 
     for (int i = 0; i < length(); i++) {
       Object ob1 = get(i);
       Object ob2 = ma.get(i);
       if (ob1 instanceof Variable) {
         ob1 = ((Variable) ob1).var;
       }
       if (ob2 instanceof Variable) {
         ob2 = ((Variable) ob2).var;
       }
       if (ob1 instanceof VarObject && csref.hasVarValue(ob1)) {
         ob1 = csref.getVarValue(ob1);
       }
       if (ob2 instanceof VarObject && csref.hasVarValue(ob2)) {
         ob2 = csref.getVarValue(ob2);
       }
       if (ob1 instanceof VarObject) {
         csref.addGoalConstraint(
           new org.jmlspecs.jmlexec.jack.evaluator.Equality(ob1, ob2));
       } else if (ob2 instanceof VarObject) {
         csref.addGoalConstraint(
           new org.jmlspecs.jmlexec.jack.evaluator.Equality(ob2, ob1));
       } else if (ob1 instanceof MyArray) {
         ((MyArray) ob1).unify((MyArray) ob2);
       }
     }
   }

   public boolean equals(MyArray ob, boolean guard) {
     if (equalArrays(ob, guard)) {
       if (!guard) {
         unify((MyArray) ob);
       }
       return true;
     } else {
       return false;
     }
   }

   private static void addFD(VarObject var, String type,
       org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem cs) {
      boolean isFD = false;
      long low = 0, hi = 0;
      if (type.equals("int")) {
        isFD = true;
        low = Integer.MIN_VALUE;
        hi = Integer.MAX_VALUE;
      } else if (type.equals("char")) {
        low = Character.MIN_VALUE;
        hi = Character.MAX_VALUE;
        isFD = true;
      } else if (type.equals("byte")) {
        low = Byte.MIN_VALUE;
        hi = Byte.MAX_VALUE;
        isFD = true;
      } else if (type.equals("short")) {
        low = Short.MIN_VALUE;
        hi = Short.MAX_VALUE;
        isFD = true;
      } else if ( type.equals("long")) {
        low = Long.MIN_VALUE;
        hi = Long.MAX_VALUE;
        isFD = true;
      }
      if (isFD) {
        org.jmlspecs.jmlexec.jack.evaluator.UDConstraint fd =
          new org.jmlspecs.jmlexec.jack.evaluator.UDConstraint("fdVar", 3);
        org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer classes =
          new org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer(3);
        classes.add("org.jmlspecs.jmlexec.runtime.MyNumber");
        classes.add("java.lang.String");
        classes.add("org.jmlspecs.jmlexec.runtime.FDomain");
        try {
          fd.declareConstraint(classes);
        } catch(java.lang.ClassNotFoundException e) {
                System.err.println(""+e);
        }

        org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer oc =
          new org.jmlspecs.jmlexec.jack.evaluator.ObjectContainer(3);
        oc.add(var);
        oc.add(type);
        oc.add(new FDomain(low, hi, null));
        fd.setArguments(oc);
        cs.addGoalConstraint(fd);
      }
   }


   public static MyArray fromSize(String type, MyInteger s,
       org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem spec$csPost) {
     int size = s.intValue();
     MyArray newAr = new MyArray(size, true, spec$csPost);
     VarObject var;
     for (int i = 0; i < size; i++) {
       var = new VarObject();
       spec$csPost.addVariable(var, "java.lang.Object");
       newAr.set(i, new Variable(var, null));
       addFD(var, type, spec$csPost);
     }
     return newAr;
  }


    public static void main(String argv[]) {
      int [] ia = {5,4,3,2,1};
      int [][] ia2 = {ia, ia};
      boolean [] ba = {true, false, true};
      Object [] ra = {ia, ia2, ba};
      System.out.println(new MyArray(ia, null));
      System.out.println(new MyArray(ia2, null));
      System.out.println(new MyArray(ra, null));
/*
      Integer [] ia3 = new Integer [3];
      ra = ia3;
      ra[0] = new Object();
*/
    }
}
