/*
 * @(#)$Id: SpecCase.java,v 1.1 2008/03/05 00:11:18 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlexec.runtime;

public class SpecCase {
  private final Object var;
  private final GenMethod assign;
  private final GenMethod post;
  private final boolean isNormal;
  private final boolean isExceptional;

  public SpecCase(Object vr, GenMethod a, GenMethod p) {
    var = vr;
    assign = a;
    post = p;
    isNormal = false;
    isExceptional = false;
  }

  public SpecCase(Object vr, GenMethod a, GenMethod p, boolean isNorm,
                  boolean isExcept) {
    var = vr;
    assign = a;
    post = p;
    isNormal = isNorm;
    isExceptional = isExcept;
  }

  public boolean isNormal() {return isNormal;}
  public boolean isExceptional() {return isExceptional;}

  public String toString() {
    return "case(" + var + ", assign, post)";
  }

  public boolean preEval(
      org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem cs) {
    try {
       Object val = cs.getVarValue(var);
       return (val instanceof MyBoolean);
    } catch (NoValueException e) {
      return false;
    }
  }

  // requires: var is ground
  public boolean preSat(
      org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem cs) {
    MyBoolean val = (MyBoolean) cs.getVarValue(var);
    return val.booleanValue();
  }

  public void runAssign(
      org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem cs) 
      throws Exception {
    if (preSat(cs)) {
      ObjUtil.run(assign, cs);
    }
  }

  public void runPost(
      org.jmlspecs.jmlexec.jack.evaluator.ConstraintSystem cs) 
      throws Exception {
    if (preSat(cs)) {
      ObjUtil.run(post, cs);
    }
  }
}
