package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestArray.java,v 1.1 2008/04/18 15:44:26 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.multijava.util.testing.TestCase;

public class TestArray extends TestCase {
  private Array t;

  public TestArray(String msg) {
     super(msg);
  }

  protected void setUp() {
    t = new Array();
  }

  public void testDefaultConstructor() {
    assertNull(t.getTests());
    assertNull(t.getObjs());
    assertNull(t.getShorts());
  }

  public void testSetTests() {
    Test [] tt = new Test [2];
    Test t1 = new Test();
    tt[0] = t1;
    Test t2 = new Test();
    tt[1] = t2;
    t.setTests(tt);
    assertEquals(t.getTests(), tt);
    assertEquals(t.getTest(0), t1);
    assertEquals(t.getTest(1), t2);
  }

  public void testSetObjs() {
    Object [] tt = new Object [2];
    Object o1 = new Object();
    Object o2 = new Object();
    tt[0] = o1;
    tt[1] = o2;
    t.setObjs(tt);
    assertEquals(t.getObjs(), tt);
    assertEquals(t.getObj(0), o1);
    assertEquals(t.getObj(1), o2);
  }

  public void testSetShorts() {
    short [][] tt = new short [2][3];
    t.setShorts(tt);
    assertEquals(t.getShorts(), tt);
    assertEquals(t.getShort(0, 1), 0);
    assertEquals(t.getShort(1, 2), 0);
  }

  public void testSndConstructor() {
    t = new Array(1, 3, 2);
    assertEquals(t.getTests().length, 1);
    assertEquals(t.getObjs().length, 3);
    short [][] st = t.getShorts();
    assertEquals(st.length, 2);
    assertEquals(st[0].length, 2);
    assertEquals(st[1].length, 2);
    assertEquals(st[1][1], (short) 2);
  }

  public void test3rdConstructor() {
    t = new Array(2);
    Object [] ob = t.getObjs();
    assertEquals(ob.length, 2);
    assertNull(ob[0]);
    assertNull(ob[1]);
  }

  public void testBooleanArrayTrue() {
    boolean [] ba = {true, false};
    assertEquals(ba, t.testBoolean(ba));
  }

  public void testBooleanArrayFalse() {
    boolean [] ba = {false, false};
    try {
      assertEquals(ba, t.testBoolean(ba));
      fail("postcondition should not be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testNewArray1() {
    char [] na = t.testNewArray1(3);
    assertEquals(na.length, 3);
    assertEquals(na[0], 0);
  }

  public void testNewArray2() {
    Object [][] na = t.testNewArray2(3);
    assertEquals(na.length, 3);
    assertEquals(na[0].length, 3);
    assertNull(na[1][2]);
  }

  public void testNewArrayDims1() {
    int [] na = t.testNewArrayDims1();
    assertEquals(na.length, 3);
    assertEquals(1, na[0]);
    assertEquals(3, na[2]);
  }

  public void testNewArrayDims2() {
    int [][] na = t.testNewArrayDims2();
    assertEquals(na.length, 2);
    assertEquals(na[0].length, 3);
    assertEquals(na[1].length, 3);
    assertEquals(1, na[0][0]);
    assertEquals(3, na[0][2]);
    assertEquals(3, na[1][0]);
    assertEquals(1, na[1][2]);
  }

  public void testNewArrayDims3() {
    int [][][] na = t.testNewArrayDims3();
    assertEquals(na.length, 2);
    assertEquals(na[0].length, 3);
    assertEquals(na[0][0].length, 1);
    assertEquals(na[1][2].length, 1);
    assertEquals(1, na[0][0][0]);
    assertEquals(3, na[0][2][0]);
    assertEquals(3, na[1][0][0]);
    assertEquals(1, na[1][2][0]);
  }

  public void testNewArray3() {
    int [] i = new int [3];
    i[0] = 1;
    i[2] = 3;
    int [] i2 = t.testNewArray3(i);
    assertEquals(i2.length, 3);
    assertEquals(i2[0], 1);
    assertEquals(i2[2], 3);
  }

  public void testSort() {
    double [] d = {2.4, 1.3, 1.1};
// System.out.println(d[0] + " " + d[1] + " " + d[2]);
    t.sort(d);
// System.out.println(d[0] + " " + d[1] + " " + d[2]);
    assertEquals(d[0], 1.1, 0.01);
    assertEquals(d[1], 1.3, 0.01);
    assertEquals(d[2], 2.4, 0.01);
  }

  public void testSet() {
    double [] d = new double [3];
    t.set(d);
    assertEquals(d[0], 0, 0.01);
    assertEquals(d[1], 1, 0.01);
    assertEquals(d[2], 2, 0.01);
  }

  public void testInsufficient() {
     int [] ia = t.testInsufficient(2);
     assertEquals(ia[0], 1);
     assertEquals(ia[1], 2);
  }

  public void testInsufficient2() {
     int [] ia = t.testInsufficient(3);
     assertEquals(ia[0], 1);
     assertEquals(ia[1], 2);
     assertEquals(ia[2], 0);
  }

  public void testGetElemType() {
    t = new Array(2, 2, 2);
    assertEquals(t.getElemType(), (new short [1]).getClass());
  }

  public void testGetElemTypeNew() {
    assertEquals(t.getElemTypeNew(), Test.class);
  }

  public void testGetElemTypeNull() {
    assertNull(t.getElemTypeNull());
  }

  public void testNonNullArrayNull() {
    assertFalse(t.testNonNull());
  }

  public void testNonNullArrayNullAssert() {
    try {
      t.testNonNullAssert();
      fail("Expected postconditionexception");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testNonNullElemsNull() {
    t = new Array(2, 2, 2);
    assertFalse(t.testNonNull());
  }

  public void testNonNullElemsNullAssert() {
    try {
      t = new Array(2, 2, 2);
      t.testNonNullAssert();
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testNonNullElemsTrue() {
    Test [] tt = new Test [] {new Test(), new Test()};
    t.setTests(tt); 
    assertTrue(t.testNonNull());
  }

  public void testNonNullElemsAssertTrue() {
    Test [] tt = new Test [] {new Test(), new Test()};
    t.setTests(tt); 
    t.testNonNullAssert();
  }

  public void testExistentialArray() {
    Array a = t.testExistentialArray();
    assertEquals(a.getShort(0, 0), 1);
    assertEquals(a.getShort(1, 1), 4);
  }

  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestArray.class);
  }
    
  public static void main(String argv[]) {
     junit.textui.TestRunner.run (suite());
/*
    TestArray tt = new TestArray("test");
    tt.setUp();
//    tt.testSetShorts();
//    tt.testSort();
    tt.testExistentialArray();
//    tt.testSndConstructor();
//    tt.test3rdConstructor();
//    tt.testBooleanArrayTrue();
//   tt.testNewArrayDims2();
*/
  }
}
