package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestInherit.java,v 1.1 2008/04/18 15:44:27 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.multijava.util.testing.TestCase;

public class TestInherit extends TestCase {
  private Sub s;

  public TestInherit(String msg) {
     super(msg);
  }

  protected void setUp() {
    s = new Sub();
  }

  public void testSuperPre() {
    assertEquals(s.baz(11), 1);
  }

  public void testSubPre() {
    assertEquals(s.baz(1), 0);
  }

  public void testOverlap() {
    try {
    assertEquals(s.baz(7), 0);
    fail("should thow PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testOverlap2() {
    try {
    assertEquals(s.baz(-7), 0);
    fail("should thow PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testSetI() {
    s.setI(3);
    assertEquals(s.getI(), 3);
  }

  public void testSetIFail() {
    try {
    s.setI(-3);
    fail("should fail superclass invariant");
//    assertEquals(s.getI(), -3);
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testSubTypeSameTrue() {
    s.testSubType(Sub.class, Sub.class);
  }

  public void testSubTypeTrue() {
    s.testSubType(Sub.class, Super.class);
  }

  public void testSubTypeFalse() {
    try {
      s.testSubType(Super.class, Sub.class);
      fail("ensures clause should not be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testSubTypeAssertTrue() {
    assertTrue(s.testSubTypeRes(Sub.class, Super.class));
  }

  public void testSubTypeAssertFalse() {
    assertFalse(s.testSubTypeRes(Super.class, Sub.class));
  }

  public void testTypeAssertTrue() {
    assertTrue(s.testSubTypeRes());
  }

  public void testPrivateSuperInv() {
    try {
      Super ss = new Super();
      ss.setI(2);
      fail("invariant should not be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testSubWPrivateSuperInv() {
    s.setI(0);
  }

  public void testSuper() {
    assertEquals(s.superTest(), 4);
  }

  public void testSuperFail() {
    try {
      s.setI(6);
      assertEquals(s.superTest(), 4);
      fail("should throw PreconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testSuperField() {
    s.setJ(54);
    assertEquals(s.getJ(), 0);   
    assertEquals(s.getSuperJ(), 54);   
  }

  public void testSuperAssignable() {
    s.testSuperAssignable();
    assertEquals(s.getJ(), 0);
    assertEquals(s.getSuperJ(), 42);
    assertEquals(s.getI(), 57);
  }

  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestInherit.class);
  }
    
  public static void main(String argv[]) {
     junit.textui.TestRunner.run (suite());
/*
    TestInherit tt = new TestInherit("test");
    tt.setUp();
//    tt.testSubPre();
//    tt.testSuperAssignable();
//    tt.testSetI();
    tt.testSuperFail();
//    tt.testOverlap();
//    tt.testSuper();
*/

  }
}
