package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestObjectObjectMap.java,v 1.1 2008/04/18 15:44:28 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.jmlspecs.models.JMLObjectToObjectRelation;
import org.jmlspecs.models.JMLObjectToObjectMap;
import org.jmlspecs.models.JMLObjectSet;
import org.multijava.util.testing.TestCase;
                                                                               
public class TestObjectObjectMap extends TestCase {
  private ObjectObjectMap t;
  private Integer i1, i2;
  private JMLObjectToObjectMap jor;
                                                                               
  public TestObjectObjectMap(String test) {
    super(test);
  }
                                                                               
  protected void setUp() { 
    t = new ObjectObjectMap(); 
    i1 = new Integer(1); 
    i2 = new Integer(2);
    jor = new JMLObjectToObjectMap();
    jor = jor.extend(i1, i2);
    jor = jor.extend(i2, i1);
  }

  protected void tearDown() {}

  public void testConstructor() {
    assertEquals(t.isEmpty(), true);
  }

  public void testisEmpty() {
    t.extend(i1, i2); 
    assertEquals(t.isEmpty(), false);
  }

  public void testGetMap() {
    t.extend(i1, i2); 
    t.extend(i2, i1); 
    assertEquals(t.getMap(), jor);
  }

  public void testApply() {
    t.extend(i1, i2); 
    t.extend(i2, i1); 
    assertEquals(t.apply(i2), i1);
  }

  public void testExtend() {
    t.extend(i1, i2); 
    t.extend(i2, i1); 
    assertEquals(t.getMap(), jor);
  }

  public void testisAFunctionR() {
    t.extend(i1, i2); 
    t.extend(i2, i1); 
    assertEquals(t.isaFunctionR(), true);
  }

  public void testisAFunction() {
    t.extend(i1, i2); 
    t.extend(i2, i1); 
    t.isaFunction();
  }

  public void testRestrictDomain() {
    Integer i3 = new Integer(3);
    t.extend(i3, i1);
    t.extend(i1, i2); 
    t.extend(i2, i1); 
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    assertEquals(t.restrictedTo(js), jor);
  }

  public void testRangeRestrict() {
    Integer i3 = new Integer(3);
    t.extend(i1, i3);
    t.extend(i1, i2); 
    t.extend(i2, i1); 
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    assertEquals(t.restrictedTo(js), jor);
  }

  public void testExtendUnion() {
    Integer i3 = new Integer(3);
    t.extend(i1, i3);
    t.extend(i2, i1); 
    JMLObjectToObjectMap jor2 = new JMLObjectToObjectMap();
    jor2 = jor2.extend(i1, i2);
    assertEquals(t.extendUnion(jor2), jor);
  }

  public void testClashUnion() {
    Integer i3 = new Integer(3);
    Integer i4 = new Integer(4);
    t.extend(i1, i3);
    t.extend(i3, i1); 
    JMLObjectToObjectMap jor2 = new JMLObjectToObjectMap();
    jor2 = jor2.extend(i1, i4);
    jor2 = jor2.extend(i2, i1);
    jor2 = jor2.extend(i3, i1);
    assertEquals(t.clashReplaceUnion(jor, i4), jor2);
  }

  public void testDisjointUnion() {
    Integer i3 = new Integer(3);
    t.extend(i3, i1);
    JMLObjectToObjectMap jor2;
    jor2 = jor.extend(i3, i1);
    assertEquals(t.disjointUnion(jor), jor2);
  }

/*
  public void testDisjointFail() {
    try {
      t.extend(i2, i1);
      JMLObjectToObjectMap jor2;
      jor2 = t.disjointUnion(jor);
      fail("should throw ClassCastException (for JMLMapException)");
    } catch (org.jmlspecs.models.JMLMapException e) {
    } catch (ClassCastException e) {
    }
  }
*/

  public void testApply2() {
    t.testApply(i1, i2);
    assertEquals(t.getMap(), jor);
  }

  public void testApplyFail() {
    try {
      t.testApplyFail(i1, i2);
      fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

/*
  public void testRemove1() {
    t.add(i1, i2); 
    t.remove(p1); 
    assertEquals(t.getRel(), JMLObjectToObjectRelation.EMPTY);
  }

  public void testRemove2() {
    t.add(i1, i2); 
    t.insert(p2);
    jor = new JMLObjectToObjectRelation();
    jor = jor.insert(p2);
    t.remove(i1, i2); 
    assertEquals(t.getRel(), jor);
  }

  public void testRemove3() {
    t.insert(p2);
    assertEquals(t.testRemove(jor), p1);
  }

  public void testRemove4() {
    t.insert(p2);
    assertEquals(t.testRemove2(jor), i1);
  }

  public void testCompose() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2.add(i3, i2);
    assertEquals(t.compose(jor), jor2);
  }

  public void testDiff() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.add(i3, i1);
    assertEquals(t.difference(jor), jor2);
  }

  public void testIntersect() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.insert(p1);
    assertEquals(t.intersection(jor), jor2);
  }

  public void testInverse() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.insert(p2);
    jor2 = jor2.add(i1, i3);
    assertEquals(t.inverse(), jor2);
  }

  public void testInverse2() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.insert(p2);
    jor2 = jor2.add(i1, i3);
    assertEquals(t.inverse2(), jor2);
  }

  public void testInverseImage() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectSet js = new JMLObjectSet();
    JMLObjectSet js2 = new JMLObjectSet();
    js = js.insert(i1);
    js2 = js2.insert(i3);
    assertEquals(t.inverseImage(js), js2);
  }

  public void testRange() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.range(), js);
  }

  public void testDom() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.domain(), js);
  }

  public void testEquals2() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.testEquals(), t.getRel());
  }

  public void testisDefinedAtR() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.isDefinedAtR(i1), true);
  }

  public void testisDefinedAtRfalse() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.isDefinedAtR(new Integer(3)), false);
  }

  public void testisDefinedAt() {
    t.insert(p1);
    t.insert(p2);
    t.isDefinedAt(i1);
  }

  public void testToBag() {
    t.set(jor);
    org.jmlspecs.models.JMLValueBag b = new org.jmlspecs.models.JMLValueBag();
    b = b.insert(p1);
    b = b.insert(p2);
    assertEquals(t.toBag(), b);
  }

  public void testToSet() {
    t.set(jor);
    org.jmlspecs.models.JMLValueSet b = new org.jmlspecs.models.JMLValueSet();
    b = b.insert(p1);
    b = b.insert(p2);
    assertEquals(t.toSet(), b);
  }

  public void testToSequence() {
    t.set(jor);
    org.jmlspecs.models.JMLValueSequence b = 
      new org.jmlspecs.models.JMLValueSequence();
    b = b.insertFront(p2);
    b = b.insertFront(p1);
    assertEquals(t.toSequence(), b);
  }

  public void testToFunction() {
    t.set(jor);
    org.jmlspecs.models.JMLObjectToObjectMap b = 
      new org.jmlspecs.models.JMLObjectToObjectMap();
    b = b.extend(p1.key, p1.value);
    b = b.extend(p2.key, p2.value);
    assertEquals(t.toFunction(), b);
  }

  public void testHas1() {
    t.set(jor);
    assertEquals(t.has(p1), true);
  }

  public void testHas2() {
    t.set(jor);
    t.has(i1, i2);
  }

  public void testHas3() {
    t.size(p1, p2);
    assertEquals(t.getRel(), jor);
  }

  public void testHas23() {
    t.size(i1, i2);
    assertEquals(t.getRel(), jor);
  }

  public void testHas21() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.testHas(i1, i2), true);
  }

  public void testHas21Not() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.testHas(new Integer(1), i2), false);
  }

  public void testEquals() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.testEquals(), jor);
  }

  public void testImage() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.image(js), js);
  }

  public void testImagePairSet() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.imagePairSet(), jor.imagePairSet());
  }

  public void testRemoveFromDomain() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.removeFromDomain(i1), jor.removeFromDomain(i1));
  }

  public void testElementImage() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.elementImage(i2), js);
  }

  public void testInverseElementImage() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.inverseElementImage(i2), js);
  }

  public void testElementImage2() {
    JMLObjectSet js = new JMLObjectSet();
    JMLObjectSet js2;
    Integer i3 = new Integer(3);
    js = js.insert(i1);
    js2 = js.insert(i3);
    js = js.insert(i2);
    t.testElementImage(i1, i2, js, js2);
    jor = jor.add(i1, i1);
    jor = jor.add(i2, i3);
    assertEquals(t.getRel(), jor);
  }
*/

  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestObjectObjectMap.class);
  }

  public static void main(String argv[]) {
    junit.textui.TestRunner.run (suite());

/*
    TestObjectObjectMap t = new TestObjectObjectMap("testing");
    t.setUp();
    t.testDisjointFail();
*/
  }
}
