package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestObjectObjectRelation.java,v 1.1 2008/04/18 15:44:28 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.jmlspecs.models.JMLObjectToObjectRelation;
import org.jmlspecs.models.JMLObjectObjectPair;
import org.jmlspecs.models.JMLObjectSet;
import org.multijava.util.testing.TestCase;
                                                                               
public class TestObjectObjectRelation extends TestCase {
  private ObjectObjectRel t;
  private JMLObjectObjectPair p1, p2;
  private Integer i1, i2;
  private JMLObjectToObjectRelation jor;
                                                                               
  public TestObjectObjectRelation(String test) {
    super(test);
  }
                                                                               
  protected void setUp() { 
    t = new ObjectObjectRel(); 
    i1 = new Integer(1); 
    i2 = new Integer(2);
    p1 = new JMLObjectObjectPair(i1, i2);
    p2 = new JMLObjectObjectPair(i2, i1);
    jor = new JMLObjectToObjectRelation();
    jor = jor.insert(p1);
    jor = jor.insert(p2);
  }

  protected void tearDown() {}

  public void testConstructor() {
    assertEquals(t.isEmpty(), true);
  }

  public void testisEmpty() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.isEmpty(), false);
  }

  public void testGetRel() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.getRel(), jor);
  }

  public void testAdd() {
    t.add(i1, i2); 
    t.add(i2, i1); 
    assertEquals(t.getRel(), jor);
  }

  public void testAdd2() {
    jor = new JMLObjectToObjectRelation();
    jor = jor.insert(p1);
    assertEquals(t.testAdd(i1, i2), JMLObjectToObjectRelation.EMPTY);
    assertEquals(t.getRel(), jor);
  }

  public void testRemove1() {
    t.add(i1, i2); 
    t.remove(p1); 
    assertEquals(t.getRel(), JMLObjectToObjectRelation.EMPTY);
  }

  public void testRemove2() {
    t.add(i1, i2); 
    t.insert(p2);
    jor = new JMLObjectToObjectRelation();
    jor = jor.insert(p2);
    t.remove(i1, i2); 
    assertEquals(t.getRel(), jor);
  }

  public void testRemove3() {
    t.insert(p2);
    assertEquals(t.testRemove(jor), p1);
  }

  public void testRemove4() {
    t.insert(p2);
    assertEquals(t.testRemove2(jor), i1);
  }

  public void testCompose() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2.add(i3, i2);
    assertEquals(t.compose(jor), jor2);
  }

  public void testDiff() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.add(i3, i1);
    assertEquals(t.difference(jor), jor2);
  }

  public void testUnion() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.add(i3, i1);
    jor2 = jor2.union(jor);
    assertEquals(t.union(jor), jor2);
  }

  public void testIntersect() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.insert(p1);
    assertEquals(t.intersection(jor), jor2);
  }

  public void testInverse() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.insert(p2);
    jor2 = jor2.add(i1, i3);
    assertEquals(t.inverse(), jor2);
  }

  public void testInverse2() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectToObjectRelation jor2 = new JMLObjectToObjectRelation();
    jor2 = jor2.insert(p2);
    jor2 = jor2.add(i1, i3);
    assertEquals(t.inverse2(), jor2);
  }

  public void testInverseImage() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    JMLObjectSet js = new JMLObjectSet();
    JMLObjectSet js2 = new JMLObjectSet();
    js = js.insert(i1);
    js2 = js2.insert(i3);
    assertEquals(t.inverseImage(js), js2);
  }

  public void testRange() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.range(), js);
  }

  public void testDom() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.domain(), js);
  }

  public void testEquals2() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.testEquals(), t.getRel());
  }

  public void testisAFunctionR() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.isaFunctionR(), true);
  }

  public void testisAFunctionRfalse() {
    t.insert(p1);
    t.insert(p2);
    t.add(i1, new Integer(3));
    assertEquals(t.isaFunctionR(), false);
  }

  public void testisAFunction() {
    t.insert(p1);
    t.insert(p2);
    t.isaFunction();
  }

  public void testisDefinedAtR() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.isDefinedAtR(i1), true);
  }

  public void testisDefinedAtRfalse() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.isDefinedAtR(new Integer(3)), false);
  }

  public void testisDefinedAt() {
    t.insert(p1);
    t.insert(p2);
    t.isDefinedAt(i1);
  }

  public void testRestrictDomain() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    t.insert(p2);
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    assertEquals(t.restrictDomainTo(js), jor);
  }

  public void testRestrictRange() {
    Integer i3 = new Integer(3);
    t.add(i3, i1);
    t.insert(p1);
    t.insert(p2);
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    jor = jor.add(i3, i1);
    assertEquals(t.restrictRangeTo(js), jor);
  }

  public void testToBag() {
    t.set(jor);
    org.jmlspecs.models.JMLValueBag b = new org.jmlspecs.models.JMLValueBag();
    b = b.insert(p1);
    b = b.insert(p2);
    assertEquals(t.toBag(), b);
  }

  public void testToSet() {
    t.set(jor);
    org.jmlspecs.models.JMLValueSet b = new org.jmlspecs.models.JMLValueSet();
    b = b.insert(p1);
    b = b.insert(p2);
    assertEquals(t.toSet(), b);
  }

  public void testToSequence() {
    t.set(jor);
    org.jmlspecs.models.JMLValueSequence b = 
      new org.jmlspecs.models.JMLValueSequence();
    b = b.insertFront(p2);
    b = b.insertFront(p1);
    assertEquals(t.toSequence(), b);
  }

  public void testToFunction() {
    t.set(jor);
    org.jmlspecs.models.JMLObjectToObjectMap b = 
      new org.jmlspecs.models.JMLObjectToObjectMap();
    b = b.extend(p1.key, p1.value);
    b = b.extend(p2.key, p2.value);
    assertEquals(t.toFunction(), b);
  }

  public void testHas1() {
    t.set(jor);
    assertEquals(t.has(p1), true);
  }

  public void testHas2() {
    t.set(jor);
    t.has(i1, i2);
  }

  public void testHas3() {
    t.size(p1, p2);
    assertEquals(t.getRel(), jor);
  }

  public void testHas23() {
    t.size(i1, i2);
    assertEquals(t.getRel(), jor);
  }

  public void testHas21() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.testHas(i1, i2), true);
  }

  public void testHas21Not() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.testHas(new Integer(1), i2), false);
  }

  public void testEquals() {
    t.insert(p1); 
    t.insert(p2);
    assertEquals(t.testEquals(), jor);
  }

  public void testImage() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    js = js.insert(i2);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.image(js), js);
  }

  public void testImagePairSet() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.imagePairSet(), jor.imagePairSet());
  }

  public void testRemoveFromDomain() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.removeFromDomain(i1), jor.removeFromDomain(i1));
  }

  public void testElementImage() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.elementImage(i2), js);
  }

  public void testInverseElementImage() {
    JMLObjectSet js = new JMLObjectSet();
    js = js.insert(i1);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.inverseElementImage(i2), js);
  }

  public void testApply() {
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.apply(i2), i1);
  }

  public void testExtend() {
    Integer i3 = new Integer(3);
    t.insert(p1);
    t.insert(p2);
    assertEquals(t.extend(i2, i3), jor.toFunction().extend(i2, i3));
  }

  public void testElementImage2() {
    JMLObjectSet js = new JMLObjectSet();
    JMLObjectSet js2;
    Integer i3 = new Integer(3);
    js = js.insert(i1);
    js2 = js.insert(i3);
    js = js.insert(i2);
    t.testElementImage(i1, i2, js, js2);
    jor = jor.add(i1, i1);
    jor = jor.add(i2, i3);
    assertEquals(t.getRel(), jor);
  }

  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestObjectObjectRelation.class);
  }


  public static void main(String argv[]) {
    junit.textui.TestRunner.run (suite());
/*
    TestObjectObjectRelation t = new TestObjectObjectRelation("testing");
    t.setUp();
    t.testConstructor();   
*/
/*
    Integer i = new Integer(4);
    t.insert(new Integer(2));
    t.insert(new Integer(3));
    System.out.println(t.getSet());
    System.out.println(t.testHas(i));
    t.insert(i);
    System.out.println(t.getSet());
    System.out.println(t.testHas(i));
*/
//    System.out.println(t.isEmpty());
/*
    Integer i1 = new Integer(1);
    t.insert(i1);
    t.insert(new Integer(2));
    TestSet t2 = new TestSet();
    t2.insert(i1);
    System.out.println(t2.isProperSubset(t));
*/
  }
}
