package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestObjectSeq.java,v 1.1 2008/04/18 15:44:28 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.jmlspecs.models.JMLObjectSequence;
import org.multijava.util.testing.TestCase;

public class TestObjectSeq extends TestCase {
  private ObjectSeq t;
  private JMLObjectSequence jos;
  private Integer i, j, k;

  public TestObjectSeq(String test) {
    super(test);
  }

  protected void setUp() { 
    t = new ObjectSeq();
    jos = new JMLObjectSequence();
    i = new Integer(1);
    j = new Integer(2);
    k = new Integer(3);
    jos = jos.insertBack(i).insertBack(j).insertBack(k);
  }

  protected void tearDown() {}

  public void testConstructor() {
    assertEquals(t.getSeq(), JMLObjectSequence.EMPTY);
  }

  public void testSet() {
    t.set(jos);
    assertEquals(t.getSeq(), jos);
 }

  public void testMakeEmpty() {
    t.set(jos);
    t.makeEmpty();
    assertEquals(t.getSeq(), JMLObjectSequence.EMPTY);
  }

  public void testFirst() {
    t.set(jos);
    assertEquals(t.first(), i);
  }

  public void testTrailer() {
    t.set(jos);
    t.trailer();
    assertEquals(t.getSeq(), jos.trailer());
  }

  public void testInsert1() {
    t.insert1(i);
    assertEquals(t.getSeq(), new JMLObjectSequence(i));
  }

  public void testInsert2() {
    t.insert2(i);
    assertEquals(t.getSeq(), new JMLObjectSequence(i).insertBack(i));
  }

  public void testInsert3() {
    t.insert3(i);
    assertEquals(t.getSeq(), new JMLObjectSequence(i).insertBack(i));
  }

  public void testHasAssert() {
    t.set(jos);
    assertEquals(t.testHas(i), true);
  }

  public void testHasAssertFalse() {
    t.set(jos);
    assertEquals(t.testHas(new Integer(4)), false);
  }

  public void testHas() {
    t.set(jos);
    t.testHas2(i);
  }

  public void testHasFail() {
    try {
      t.set(jos);
      t.testHas2(new Integer(4));
      fail("should have postcondition exception");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testIsEmpty() {
    t.set(jos);
    assertEquals(t.isEmpty(), false);
  }

  public void testIsEmptyTrue() {
    assertEquals(t.isEmpty(), true);
  }

  public void testSize() {
    t.size(i, j);
    assertEquals(t.getSeq(), new JMLObjectSequence(i).insertBack(j));
  }

  public void containsAll() {
    t.set(jos);
    java.util.ArrayList a = new java.util.ArrayList();
    a.add(i);
    a.add(j);
    assertEquals(t.testContains(a), true);
  }

  public void containsAll2() {
    t.set(jos);
    java.util.ArrayList a = new java.util.ArrayList();
    a.add(i);
    a.add(j);
    t.testContains2(a);
  }

  public void testToBag() {
    t.set(jos);
    org.jmlspecs.models.JMLObjectBag b = new org.jmlspecs.models.JMLObjectBag();
    b = b.insert(i);
    b = b.insert(j);
    b = b.insert(k);
    assertEquals(t.toBag(), b);
  }

  public void testToSet() {
    t.set(jos);
    org.jmlspecs.models.JMLObjectSet b = new org.jmlspecs.models.JMLObjectSet();
    b = b.insert(i);
    b = b.insert(j);
    b = b.insert(k);
    assertEquals(t.toSet(), b);
  }

  public void testToArray() {
    t.set(jos);
    Object [] b = t.toArray();
    assertEquals(b[2], k);
    assertEquals(b[1], j);
    assertEquals(b[0], i);
  }

  public void testClone() {
    t.set(jos);
    assertEquals(t.getSeq(), t.clone());
  }

  public void testString() {
    t.set(jos);
    assertEquals(t.toString(), "(<1, 2, 3>)");
  }

  public void testForall() {
    t.size(i, j);
    t.testForall(k);
  }

  public void testHash() {
    t.set(jos);
    assertEquals(t.hashCode(), jos.hashCode());
  }

  public void testCount() {
    t.set(jos);
    assertEquals(t.count(i), 1);
  }

  public void testTestCount() {
    t.testCount(i);
    assertEquals(t.getSeq(), new JMLObjectSequence(i).insertBack(i));
  }

  public void testReverse() {
    t.set(jos);
    t.reverse();
    assertEquals(t.getSeq(), jos.reverse());
  }

  public void testLast() {
    t.set(jos);
    assertEquals(t.last(), k);
  }

  public void testHeader() {
    t.set(jos);
    assertEquals(t.header(), jos.header());
  }

  public void testTestHeader() {
    t.set(jos);
    t.testHeader(i);
    assertEquals(t.getSeq(), jos.insertBack(i));
  }

  public void testTestReverse() {
    t.testReverse(i);
    assertEquals(t.getSeq(), new JMLObjectSequence(i).insertBack(i));
  }
    
  public void testItemAt() {
    t.set(jos);
    assertEquals(t.itemAt(1), j);
  }

  public void testItemAt2() {
    t.itemAt(i);
    assertEquals(t.getSeq(), new JMLObjectSequence(i).insertBack(i));
  }
   
  public void testIndexOf() {
    t.set(jos);
    assertEquals(t.indexOf(j), 1);
  }
  
  public void testIndexOf2() {
    t.testIndexOf(i, j);
    assertEquals(t.getSeq(), new JMLObjectSequence(j).insertBack(i));
  }
   
  public void testEquals() {
    t.set(jos);
    assertEquals(t.testEquals(), jos);
  }
   
  public void testInsertAfter() {
    t.set(jos);
    assertEquals(t.insertAfterIndex(k), jos.insertAfterIndex(1, k));
  }
   
  public void testInsertBefore() {
    t.set(jos);
    assertEquals(t.insertBeforeIndex(k), jos.insertBeforeIndex(1, k));
  }
   
  public void testInsertFront() {
    t.set(jos);
    assertEquals(t.insertFront(k), jos.insertFront(k));
  }
   
  public void testConcat() {
    t.set(jos);
    assertEquals(t.concat(jos), jos.concat(jos));
  }
   
  public void testTestConcat() {
    assertEquals(t.testConcat(jos), new JMLObjectSequence(i));
    assertEquals(t.getSeq(), new JMLObjectSequence(j).insertBack(k));
  }
   
  public void testRemove() {
    t.set(jos);
    t.removeItemAt(1);
    assertEquals(t.getSeq(), jos.removeItemAt(1));
  }
   
  public void testPrefix() {
    t.set(jos);
    assertEquals(t.prefix(2), jos.prefix(2));
  }
   
  public void testIsPrefix() {
    t.set(jos);
    t.removeItemAt(2);
    assertEquals(t.isPrefix(jos), true);
  }
   
  public void testIsPPrefix() {
    t.set(jos);
    t.removeItemAt(2);
    assertEquals(t.isProperPrefix(jos), true);
  }
   
  public void testIsPPrefixFalse() {
    t.set(jos);
    assertEquals(t.isProperPrefix(jos), false);
  }
   
  public void testIsPPrefixNonAssert() {
    t.testProperPrefix(jos);
    assertEquals(t.getSeq(), new JMLObjectSequence(i).insertBack(j));
  }
   
  public void testTestPrefix() {
    t.set(jos);
    t.removeItemAt(2);
    t.testPrefix(jos);
  }
   
  public void testIsSub() {
    t.set(jos);
    t.removeItemAt(2);
    assertEquals(t.isSubsequence(jos), true);
  }
   
  public void testIsSubNonAssert() {
    t.set(jos);
    t.removeItemAt(2);
    t.testSubsequence(jos);
  }
   
  public void testIsPSub() {
    t.set(jos);
    t.removeItemAt(2);
    assertEquals(t.isProperSubsequence(jos), true);
  }
   
  public void testIsPSubFalse() {
    t.set(jos);
    assertEquals(t.isProperSubsequence(jos), false);
  }
   
  public void testIsSup() {
    t.set(jos);
    jos.removeItemAt(2);
    assertEquals(t.isSupersequence(jos), true);
  }
   
  public void testIsPSup() {
    t.set(jos);
    assertEquals(t.isProperSupersequence(new JMLObjectSequence(i)), true);
  }
   
  public void testIsPSupFalse() {
    t.set(jos);
    assertEquals(t.isProperSupersequence(jos), false);
  }
   
  public void testRemovePre() {
    t.set(jos);
    t.removePrefix(2);
    assertEquals(t.getSeq(), new JMLObjectSequence(k));
  }
   
  public void testIsInsertion() {
    t.set(jos);
    assertEquals(t.isInsertionInto(new JMLObjectSequence(i).insertBack(k),
                                   j), true);
  }
   
  public void testIsDeletion() {
    t.set(new JMLObjectSequence(i).insertBack(k));
    assertEquals(t.isDeletionFrom(jos, j), true);
  }
   
  public void testReplace() {
    t.set(jos);
    t.replaceItemAt(1, i);
    assertEquals(t.getSeq(), new 
       JMLObjectSequence(i).insertBack(i).insertBack(k));
  }
   
  public void testGet() {
    t.set(jos);
    assertEquals(t.get(1), j);
  }

  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestObjectSeq.class);
  }

  public static void main(String argv[]) {
    junit.textui.TestRunner.run (suite());
/*
     TestObjectSeq ts = new TestObjectSeq("test");
     ts.setUp();
     ts.testInsertBefore();
//     ts.testTestReverse();
//     ts.testIsSubNonAssert();
*/
//    ObjectSeq t = new ObjectSeq();
//    t.testReverse(new Integer(1));
/*
    ObjectSeq t = new ObjectSeq();
    JMLObjectSequence jos = new JMLObjectSequence();
    Integer i = new Integer(1);
    Integer j = new Integer(2);
    Integer k = new Integer(3);
    jos = jos.insertBack(i).insertBack(j).insertBack(k);
    t.set(jos);
    t.removePrefix(2);
    System.out.println(t.getSeq());
*/

  }
}
