package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestTest.java,v 1.2 2008/04/25 16:51:45 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.multijava.util.testing.TestCase;

public class TestTest extends TestCase {
  private Test t;

  public TestTest(String msg) {
     super(msg);
  }

  protected void setUp() {
    t = new Test();
  }

  public void testDefConstructor() {
    assertEquals(t.getCT(), 0);
    assertEquals(t.getChar(), 'z');
  }

  public void testOneArgConstructor() {
    t = new Test(4);
    assertEquals(t.getCT(), 4);
  }

  public void testSet() {
    t.set(5);
    assertEquals(t.getCT(), 5);
  }

  public void testSetFail() {
    try {
    t.set(-12);
    assertEquals(t.getCT(), -12);
    fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testStaticInvStatic() {
    try {
    t.testStaticInvStatic();
    fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testStaticInvNonStatic() {
    try {
    t.testStaticInvNonStatic();
    fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testSetOr1() {
    t.setOr(5);
    assertEquals(t.getCT(), 5);
  }

  public void testSetOr2() {
    t.setOr(3);
    assertEquals(t.getCT(), 4);
  }

  public void testPrec() {
    t.set(4);
    t.testPrec();
    assertEquals(t.getCT(), 18);
  }

  public void testInc() {
    t.incBy(3);
    assertEquals(t.getCT(), 3);
    t.incBy(4);
    assertEquals(t.getCT(), 7);
  }

  public void testReSet() {
    t.set(5);
    t.reset(6);
    assertEquals(t.getCT(), 6);
  }

  public void testReSet2() {
    try {
      t.set(5);
      t.reset(5);
      fail("ensures clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testNoSet() {
      t.set(5);
      t.noset(6);
      assertEquals(t.getCT(), 5);
  }

  public void testNoSet2() {
    try {
      t.set(5);
      t.noset(5);
      fail("ensures clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testFoo0() {
    t.set(5);
    t.foo0(6);
    assertEquals(t.getCT(), 0);
  }

  public void testFoo02() {
    try {
      t.set(5);
      t.foo0(5);
      fail("ensures clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testNull() {
    t.testNull();
    assertEquals(t.getElems(), null);
  }

  public void testFoo1() {
    Integer i = new Integer(3);
    t.set(3);
    t.foo1(i, 4);
    assertEquals(t.getElems(), null);
    assertEquals(t.getCT(), 57);
  }

  public void testFoo12() {
    try {
      Integer i = new Integer(3);
      t.foo1(i, 4);
      fail("requires clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testFoo13() {
    try {
      t.foo1(null, 4);
      fail("requires clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testSetElems() {
    Object [] ob = new Object[3];
    t.setElems(ob);
    assertEquals(t.getElems()[0], ob[0]);
    assertEquals(t.getElems()[1], ob[1]);
    assertEquals(t.getElems()[2], ob[2]);
  }

  public void testSetElems2() {
    try {
      Object [] ob = new Object[3];
      t.setElems(ob);
      t.setElems(ob);
      fail("requires clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testfoo2() {
    try {
      Object [] ob = new Object[3];
      Object ob2 = new Object();
      t.foo2(ob, ob2);
      fail("requires clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testfoo22() {
    try {
      Object ob2 = new Object();
      t.foo2(ob2, ob2);
      fail("ensures clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testfoo3() {
    Object [] ob = new Object[3];
    t.foo3(ob);
    assertEquals(t.getElems(), null);
  }

  public void testfoo4() {
    t.foo4(3, 4);
    assertEquals(t.getCT(), 0);
  }

  public void testfoo42() {
    try {
      t.foo4(3, 3);
      fail("requires clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testAssert01() {
     assertTrue(t.testAssert(0));
  }

  public void testAssert11() {
     boolean res = t.testAssert1(0);
     assertEquals(res, true);     
  }

  public void testAssert12() {
     boolean res = t.testAssert1(1);
     assertEquals(res, false);     
  }

  public void testLT01() {
     boolean res = t.testLT(0);
     assertEquals(res, false);     
  }

  public void testLT02() {
     boolean res = t.testLT(1);
     assertEquals(res, true);     
  }

  public void testLT2() {
    assertTrue(t.testLT2(1));
  }

  public void testResult() {
    boolean res = t.testResult();
    assertEquals(res, false);
  }

  public void testResult1() {
    try {
      boolean res = t.testResult1();
      fail("expected InsufficientInformationException");
    } catch (org.jmlspecs.jmlexec.runtime.InsufficientInformationException e) {
    }
  }

  public void testAndAssert1() {
    boolean res = t.testAndAssert(3);
    assertEquals(res, true);
  }

  public void testAndAssert2() {
    boolean res = t.testAndAssert(5);
    assertEquals(res, false);
  }

  public void testOrAssert1() {
    boolean res = t.testOrAssert(3);
    assertEquals(res, true);
  }

  public void testOrAssert2() {
    boolean res = t.testOrAssert(3);
    assertEquals(res, true);
  }

  public void testOrAssert3() {
    boolean res = t.testOrAssert(4);
    assertEquals(res, false);
  }

  public void testOrAssert4() {
    assertFalse(t.testOrAssert());
  }

  public void testNot01() {
    boolean res = t.testNot();
    assertEquals(res, true);
  }

  public void testNot02() {
    t = new Test(3);
    boolean res = t.testNot();
    assertEquals(res, false);
  }

  public void testNot2() {
    assertTrue(t.testNot2());
  }

  public void testNot31() {
    boolean res = t.testNot3();
    assertEquals(res, true);
  }

  public void testNot32() {
    t = new Test(1);
    boolean res = t.testNot3();
    assertEquals(res, false);
  }

  public void testCast() {
    t = new Test(1);
    short res = t.testCast();
    assertEquals(res, (short) 1);
  }

  public void testCast2() {
    t = new Test(1);
    t.testCast2((short) 1);
    assertEquals(t.getD(), 1.0, 0.001);
  }

// START - requires backtracking through all possible int values before failing
  public void testCast31() {
    try {
      t.testCast3(3.1415);
      fail("should not satisfy ensures clause");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {}
  }

  public void testCast32() {
    t.testCast3(3.00);
    assertEquals(t.getCT(), 3);
  }

  public void testMixed() {
    float f = t.testMixed(1, (short) 2, (float) 3.5);
    assertEquals(f, (float) 6.5, 0.001);
  }

  public void testNotEquiv1() {
    boolean res = t.testNotEquiv(0);
    assertEquals(res, false);
  }

  public void testNotEquiv2() {
    boolean res = t.testNotEquiv(3);
    assertEquals(res, true);
  }

  public void testImply11() {
    t.testImply1();
    assertEquals(t.getCT(), 5);
  }

  public void testImply12() {
    t.set(-1);
    t.testImply1();
    assertEquals(t.getCT(), 4);
  }

  public void testImply21() {
    boolean res = t.testImply2();
    assertEquals(res, true);
  }

  public void testImply22() {
    t.set(5);
    boolean res = t.testImply2();
    assertEquals(res, true);
  }

  public void testImply31() {
    t.set(-1);
    t.testImply3();
    assertEquals(t.getCT(), 4);
  }

  public void testImply32() {
    t.testImply3();
    assertEquals(t.getCT(), 0);
  }

  public void testImply33() {
    t.set(-3);
    t.testImply3();
    assertEquals(t.getCT(), -3);
  }

  public void testImplyB() {
    t.testImplyB();
    assertEquals(t.getD(), 4.2, 0.001);
  }

  public void testExists1() {
    t.testExists1();
    assertEquals(t.getCT(), 0);
  }

  public void testExists2() {
    t.testExists2();
    assertEquals(t.getCT(), 1);
  }

  public void testExists3() {
    t.testExists3();
    assertEquals(t.getCT(), 1);
  }

  public void testExists5() {
    t.testExists5();
    assertEquals(t.getCT(), 3);
  }

  public void testExists6() {
    t.testExists6();
    assertEquals(t.getCT(), 2);
  }

  public void testCall1() {
    Integer i = new Integer(3);
    assertEquals(t.testCall(i), 3);
  }

  public void testCall2() {
    Integer i = new Integer(3);
    assertEquals(t.testCall2("bar"), 'a');
  }

  public void testAdd2() {
    assertEquals(Test.add2(2), 4);
  }

  public void testAddJ() {
    assertEquals(Test.addJ(2), 5);
  }

  public void testAddOther() {
    assertEquals(Test.addOtherJ(2, new Test()), 5);
  }

  public void testSetJ() {
    assertEquals(Test.setJ(), 5);
  }

  public void testInverse() {
    assertEquals(Test.testInverse(5), -5);
  }

  public void testInverse2() {
    assertEquals(Test.testInverse(-5), 5);
  }

  public void testUPlus() {
    short s = (short) 5;
    assertEquals(Test.testUPlus(s), 5);
  }

  public void testTestUPlus() {
    short s = (short) 5;
    assertEquals(Test.testTestUPlus(s), -5);
  }

  public void testBitNot() {
    short s = (short) 5;
    assertEquals(Test.testBitNot(s), ~5);
  }

  public void testTestCasesA() {
    assertEquals(t.testCases(2), 1);
  }

  public void testTestCasesB() {
    assertEquals(t.testCases(-1), 0);
  }

  public void testSumAll() {
    assertEquals(t.sumAll(4), 10);
  }

  public void testSumAll2() {
    assertEquals(t.sumAll2(4), 10);
  }

  public void testCast4() {
    assertEquals(t.testCast4(), 0.0, 0.01);
  }

  public void testCondTrue() {
    assertEquals(t.testCond(4), 1);
  }

  public void testCondFalse() {
    assertEquals(t.testCond(-4), -1);
  }

  public void testBool0() {
    assertEquals(t.testReturnBool(-4), 0);
  }

  public void testBool1() {
    assertEquals(t.testReturnBool(14), 1);
  }

  public void testTestAssertTrue() {
    assertEquals(t.testTestAssert(3), 3);
  }

  public void testTestAssertFalse() {
    try {
      assertEquals(t.testTestAssert(13), 13);
      fail("requires clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testTestModel() {
    assertEquals(t.testTestModel(3), 4);
 }

  public void testNotModified() {
    t.testNotModified();
    assertEquals(t.j, 3);
    assertEquals(t.getCT(), 0);
  }

  public void testNotModifiedFail() {
    try {
      t.testNotModifiedFail();
      assertEquals(t.j, 3);
      assertEquals(t.getCT(), 0);
      fail("ensures clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testNotModifiedTrue() {
    assertTrue(t.testNotModifiedAssertTrue());
  }

  public void testNotModifiedFalse() {
    assertFalse(t.testNotModifiedAssertFalse());
  }

  public void testTypeOf() {
    assertEquals(t.testTypeOf(), t.getClass());
  }

  public void testTypeOfNot() {
    assertFalse(t.testTypeOf().equals(this.getClass()));
  }

  public void testInstanceOfSame() {
    t.testInstanceOf(new Super());
  }

  public void testInstanceOfSub() {
    t.testInstanceOf(new Sub());
  }

  public void testInstanceOfFail() {
    try {
      t.testInstanceOf(new Integer(3));
      fail("ensures clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testInstanceOfAssertTrue() {
    assertTrue(t.testInstanceOfAssert(new Sub()));
  }

  public void testInstanceOfAssertFalse() {
    assertFalse(t.testInstanceOfAssert(new Object()));
  }

  public void testSetConc() {
    Conc c = new Conc();
    t.setConc(c);
    assertEquals(t.conc, c);
  }

  public void testNoSetConc() {
    t.noSetConc(null);
    assertNull(t.conc);
  }

  public void testNoSetConcFail() {
    try {
      Conc c = new Conc();
      t.noSetConc(c);
      fail("ensures clause shouldn't be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testInvariantFor() {
    Conc s = new Conc();
    t.testInvariantFor(s);
  }
                                                                              
  public void testInvariantForFail() {
    Conc s = new Conc();
    try {
      s.specFld_i = new org.jmlspecs.jmlexec.runtime.MyInteger(-7);
      t.testInvariantFor(s);
      fail("should thow PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }
                                                                              
                                                                              
  public void testInvariantForTrue() {
    Conc s = new Conc();
    assertTrue(t.testInvariantForAssert(s));
  }
                                                                              
  public void testInvariantForFalse() {
    Conc s = new Conc();
    s.specFld_i = new org.jmlspecs.jmlexec.runtime.MyInteger(-7);
    assertFalse(t.testInvariantForAssert(s));
  }
                                                                              
  public void testInvariantForStaticTrue() {
    Conc s = new Conc();
    assertTrue(t.testInvariantForAssertStatic(s));
  }
                                                                              
  public void testInvariantForStaticFalse() {
    Conc s = new Conc();
    s.specFld_stat = new org.jmlspecs.jmlexec.runtime.MyInteger(100);
    assertFalse(t.testInvariantForAssertStatic(s));
  }

  public void testExceptionalNormal() {
     assertTrue(t.testExceptional(2));
  }

  public void testExceptionalExceptional() {
    try {
       t.testExceptional(-1);
       fail("should throw NullPointerException");
    } catch (NullPointerException e) {
    }
  }

/*
  public void testExceptionalExceptional2() {
    try {
       t.testExceptional(0);
       fail("should throw IllegalArgumentException");
    } catch (IllegalArgumentException e) {
    }
  }
*/

  public void testExceptionalExceptional3() {
    try {
       t.testExceptional(1);
       fail("should throw IllegalArgumentException");
    } catch (IllegalArgumentException e) {
       assertEquals(t.getCT(), 3);
    }
  }

  public void testThrows() throws Exception {
    try {
       t.testThrows(0);
       fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testThrows4() {
    try {
       t.testThrows(-1);
       fail("should throw Exception");
    } catch (Exception e) {
       assertEquals(e.getMessage(), "bar");
    }
  }

  public void testThrows2() throws Exception {
    try {
       t.testThrows(1);
       fail("should throw TestE");
    } catch (TestE e) {
    }
  }

  public void testThrows3() throws Exception {
    try {
      t.testThrows(2);
      fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }
  
  public void testSum() {
    assertEquals(t.testSum(0, 5, 0), 10);
  }
  
  public void testSumHole() {
    assertEquals(t.testSumHole(0, 5, 3, 0), 7);
  }
  
  public void testProduct() {
    assertEquals(t.testProduct(1, 4, 0), 6);
  }
  
  public void testProductHole() {
    assertEquals(t.testProductHole(1, 5, 2, 0), 12);
  }
  
  public void testSumEmpty() {
    assertEquals(t.testSum(1, 1, 0), 0);
  }
  
  public void testProductEmpty() {
    assertEquals(t.testProduct(2, 2, 0), 1);
  }
  
  public void testMax() {
    assertEquals(t.testMax(0, 5, 0), 4);
  }
  
  public void testMaxEmpty() {
    assertEquals(t.testMax(0, 0, 0), java.lang.Integer.MIN_VALUE);
  }
  
  public void testMin() {
    assertEquals(t.testMin(0, 5, 0), 0);
  }
  
  public void testMinEmpty() {
    assertEquals(t.testMin(0, 0, 0), java.lang.Integer.MAX_VALUE);
  }
  
  public void testNumOf() {
   assertEquals(t.testNumOf(1, 4), 3);
  }
  
  public void testNumOfEmpty() {
   assertEquals(t.testNumOf(0, 0), 0);
  }

  public void testExistsRTrue() {
    assertTrue(t.testExistsR(2));
  }

  public void testExistsRFalse() {
    assertFalse(t.testExistsR(7));
  }

  public void testForallRTrue() {
    assertTrue(t.testForallR(1));
  }

  public void testForallRFalse() {
    assertFalse(t.testForallR(2));
  }

  public void testForallEmpty() {
     t.testForallEmpty();
  }

  public void testShiftArray2() {
     int [] arr = new int [5];
     for (int i = 0; i < arr.length; i++) {
       arr[i] = i;
     }
     t.testShiftArray(arr, 2);
     assertEquals(arr[0], 0);
     assertEquals(arr[1], 1);
     assertEquals(arr[2], 3);
     assertEquals(arr[3], 4);
     assertEquals(arr[4], 4);
  }

  public void testShiftArray4() {
     int [] arr = new int [5];
     for (int i = 0; i < arr.length; i++) {
       arr[i] = i;
     }
     t.testShiftArray(arr, 4);
     assertEquals(arr[0], 0);
     assertEquals(arr[1], 1);
     assertEquals(arr[2], 2);
     assertEquals(arr[3], 3);
     assertEquals(arr[4], 4);
  }

  public void testShiftArray0() {
     int [] arr = new int [5];
     for (int i = 0; i < arr.length; i++) {
       arr[i] = i;
     }
     t.testShiftArray(arr, 0);
     assertEquals(arr[0], 1);
     assertEquals(arr[1], 2);
     assertEquals(arr[2], 3);
     assertEquals(arr[3], 4);
     assertEquals(arr[4], 4);
  }

  public void testShiftArray1() {
     int [] arr = new int [5];
     for (int i = 0; i < arr.length; i++) {
       arr[i] = i;
     }
     t.testShiftArray(arr, 1);
     assertEquals(arr[0], 0);
     assertEquals(arr[1], 2);
     assertEquals(arr[2], 3);
     assertEquals(arr[3], 4);
     assertEquals(arr[4], 4);
  }

  public void testNotModifiedArrayNoMod() {
    int [] a = {0,1,2,3};
    t.testNotModifiedArrayNoMod(a);
    for (int i = 0; i < 4; i++) {
      assertEquals(i, a[i]);
    }
  }

  public void testNotModifiedArray() {
    int [] a = {0,1,2,3};
    t.testNotModifiedArray(a);
    for (int i = 0; i < 4; i++) {
      assertEquals(i, a[i]);
    }
  }

  public void testNotModifiedArrayTrue() {
    int [] a = {0,1};
    assertTrue(t.testNotModifiedArrayAssert(a, 0));
  }

  public void testNotModifiedArrayFalse() {
    int [] a = {0,1};
    assertFalse(t.testNotModifiedArrayAssert(a, 5));
  }

  public void testNotModifiedArrayFail() {
    try {
      int [] a = {0,1,2,3};
      t.testNotModifiedArrayFail(a);
      fail("should thow PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testNotModifiedArrayNestedTrue() {
    int [] a = {0,1};
    assertTrue(t.testNotModifiedNested(a));
  }

  public void testNotModifiedArrayNestedFalse() {
    int [] a = {0,2};
    assertFalse(t.testNotModifiedNested(a));
  }

  public void testFields() {
    t.testFields();
    assertEquals(t.j, 0);
    assertEquals(t.getCT(), 1);
  }

  public void testFieldsNoMod() {
    t.testFieldsNoMod();
    assertEquals(t.j, 3);
    assertEquals(t.getCT(), 0);
    assertFalse(t.b);
  }

  public void testFieldsNoModAssert() {
    assertTrue(t.testFieldsNoModAssert());
  }

  public void testArrayFields() {
    Sub [] ta = new Sub[2];
    ta[0] = new Sub();
    ta[1] = new Sub();
    t.testArrayFields(ta);
    assertEquals(ta[0].j, 0);
    assertEquals(ta[1].j, 0);
    assertEquals(ta[0].i, 1);
    assertEquals(ta[1].i, 2);
  }

  public void testLet() {
    assertEquals(19, t.testLet(18));
  }

  public void testLetFail() {
    try {
      assertEquals(20, t.testLet(19));
      fail();
    } catch (org.jmlspecs.jmlexec.runtime.PreconditionException e) {
    }
  }

  public void testIsSmallCast() {
    assertFalse(t.testIsSmallCast());
  }

  public void testModelConstraint() {
    assertEquals(3, t.testModelConstraint(4));
  }

  public void testModelConstraint2() {
    t.testModelConstraint2(4);
    assertEquals(3, t.getCT());
  }

  public void testModelConstraint3() {
    t.testModelConstraint3(4);
    assertEquals(2, t.getCT());
  }

  public void testSubType() {
    assertTrue(t.testSubType());
  }

  public void testStaticRef0() {
    assertEquals(StaticFields.getI(), 4);
    assertEquals(StaticFields.getThing(), null);
  }

  public void testStaticRef1And2() {
    t.testStaticRef1();
    assertEquals(StaticFields.getI(), 3);
    t.testStaticRef2();
    assertEquals(StaticFields.getI(), 3);
  }

  public void testStaticRef3() {
    StaticFields.set();
    t.testStaticRef3();
    assertEquals(StaticFields.getThing().getCT(), 3);
    assertEquals(StaticFields.getThing().getJ(), 4);
    assertEquals(StaticFields.getThing().getD(), 6.7, 0.001);
  }

  public void testStaticRef4() {
    StaticFields.set();
    t.testStaticRef4();
    assertEquals(StaticFields.getThing().getCT(), 0);
    assertEquals(StaticFields.getThing().getJ(), 3);
    assertEquals(StaticFields.getThing().getD(), 0.0, 0.001);
  }

  public void testStaticRef5() {
    StaticFields.set();
    StaticFields.setThings();
    assertNull(StaticFields.getThings()[0]);
    assertNull(StaticFields.getThings()[1]);
    t.testStaticRef5(new Test(), new Test(3));
    assertEquals(StaticFields.getThings()[0].getCT(), 0);
    assertEquals(StaticFields.getThings()[1].getCT(), 3);
  }

  public void testStaticRef6() {
    StaticFields.set();
    StaticFields.setThings();
    t.testStaticRef5(new Test(), new Test(3));
    t.testStaticRef6();
    assertEquals(StaticFields.getThings()[0].getCT(), 0);
    assertEquals(StaticFields.getThings()[1].getCT(), 3);
  }

  public void testStaticRef7() {
    StaticFields.set();
    StaticFields.setThings();
    t.testStaticRef5(new Test(), new Test(3));
    t.testStaticRef7();
    assertEquals(StaticFields.getThings()[0].getCT(), 1);
    assertEquals(StaticFields.getThings()[1].getCT(), 2);
  }

  public void testStaticRef8() {
    StaticFields.set();
    StaticFields.setThings();
    t.testStaticRef5(new Test(), new Test(3));
    t.testStaticRef8();
    assertEquals(StaticFields.getThings()[0].getCT(), 0);
    assertEquals(StaticFields.getThings()[1].getCT(), 3);
  }

  public void testStaticRef9() {
    StaticFields.set();
    StaticFields.setThings();
    t.testStaticRef5(new Test(), new Test(3));
    t.testStaticRef9();
    assertEquals(StaticFields.getThings()[0].getCT(), 1);
    assertEquals(StaticFields.getThings()[1].getCT(), 2);
  }

  public void testStaticRef10() {
    StaticFields.set();
    StaticFields.setThings();
    t.testStaticRef5(new Test(), new Test(3));
    t.testStaticRef10();
    assertEquals(StaticFields.getThings()[0].getCT(), 0);
    assertEquals(StaticFields.getThings()[1].getCT(), 3);
  }

  public void testStaticRef11() {
    t.testStaticRef11();
    assertEquals(StaticFields.getNums()[0][0], 1);
    assertEquals(StaticFields.getNums()[1][1], 4);
  }

  public void testStaticRef12() {
    t.testStaticRef12();
    assertEquals(StaticFields.getNums()[0][0], 1);
    assertEquals(StaticFields.getNums()[1][1], 4);
  }

  public void testStaticRef15() {
    t.testStaticRef15();
    assertEquals(org.jmlspecs.jmlexec.testcase.testStatic.StaticFields2.getI(), 0);
  }

  public void testStaticRef14() {
    t.testStaticRef14();
    assertEquals(org.jmlspecs.jmlexec.testcase.testStatic.StaticFields2.getI(), 3);
  }

  public void testShiftRight() {
    assertEquals(t.testShiftRight(4), 1);
  }

  public void testShiftRightPad() {
    assertEquals(t.testShiftRight(255), 63);
  }

  public void testShiftLeft() {
    assertEquals(t.testShiftLeft('a'), 388);
  }

  public void testExists8() {
    t.testExists8();
  }

  public void testFD1() {
    assertEquals(t.testFD1(), -4);
  }

  public void testFD2() {
    assertEquals(t.testFD2(), -5);
  }

  public void testFD3() {
    assertEquals(t.testFD3(), -4);
  }

  public void testFD4() {
    assertEquals(t.testFD4(), -5);
  }

  public void testDetermined() {
    assertEquals(t.testDetermined(), 3);
  }

  public void testLTFD() {
    assertEquals(t.testLTFD(), 2);
  }

  public void testLEFD() {
    assertEquals(t.testLEFD(), 1);
  }

  public void testAddFD() {
    assertEquals(t.testAddFD(), 3);
  }

  public void testAddFD2() {
    assertEquals(t.testAddFD2(), 3);
  }

  public void testAddFD3() {
    assertEquals(t.testAddFD3(), 3);
  }

  public void testAddFD4() {
    assertEquals(t.testAddFD4(), 3);
  }

  public void testAddMult1() {
    assertEquals(t.testAddMult1(), 22);
  }

  public void testAddMult2() {
    assertEquals(t.testAddMult2(), 22);
  }

  public void testAddMult3() {
    assertEquals(t.testAddMult3(), 42);
  }

  public void testAddMult4() {
    assertEquals(t.testAddMult4(), 42);
  }

  public void testAddMult4a() {
    assertEquals(t.testAddMult4a(), 2);
  }

  public void testAddMult5() {
    assertEquals(t.testAddMult5(), 12);
  }

  public void testAddMult5a() {
    assertEquals(t.testAddMult5a(), 6);
  }

  public void testAddMult6() {
    assertEquals(t.testAddMult6(), 60);
  }

  public void testAddMult7() {
    assertEquals(t.testAddMult7(), -12);
  }

  public void testAddMult8() {
    try {
      assertEquals(t.testAddMult8(), 42);
      fail("should throw postconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testAddMult9() {
    assertEquals(t.testAddMult9(), -6);
  }

  public void testAddFDArray() {
    int [] ar = t.testFDArray();
    assertEquals(ar[0], 1);
    assertEquals(ar[1], 3);
  }

  public void testPrefix() {
    assertEquals(t.testPrefix(), "fear the");
  }

  public void testSuffix() {
    assertEquals(t.testSuffix(), "foo");
  }

  public void testPrefixSuffix() {
    assertEquals(t.testPrefixSuffix(), "the"); }

  public void testFresh() {
    Point p = t.testFresh();
    assertEquals(3, p.getX());
    assertEquals(4, p.getY());
  }

  public void testToString() {
    Point p = new Point(2, 3);
    assertEquals(p.toString(), "(2, 3)");
  }

  public void testFreshToString() {
    assertEquals(t.testFreshToString(), "(3, 4)");
  }

  public void testShortValue() {
    assertEquals(2, t.testValue(new Short((short) 2)));
  }

  public void testFresh1() {
    Byte b = t.testFresh1();
    assertEquals(2, b.byteValue());
  }

  public void testFresh2() {
    Float f = t.testFresh2();
    assertEquals((float) 2.0, f.floatValue(), 0.001);
  } 

  public void testFresh3() {
    t.testFresh3();
    assertEquals(3, t.getWrap().intValue());
  } 

  public void testFresh4() {
    t.testFresh4();
    assertEquals(3, t.getWrap().intValue());
  } 

  public void testFresh5() {
    Integer [] ia = t.testFresh5(new Integer [2]);
    assertEquals(2, ia[0].intValue());
    assertEquals(3, ia[1].intValue());
  } 

  public void testIsFresh() {
    Point p = t.testFresh();
    try {
      t.testIsFresh(p);
      fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testIsFreshAssertTrue() {
    assertTrue(t.testIsFreshAssertTrue());
  }

  public void testIsFreshAssertFalse() {
    Point p = t.testFresh();
    assertFalse(t.testIsFreshAssertFalse(p));
  }

  public void testSignals() {
    try {
      t.testSignals();
      fail("should throw IllegalArgumentException");
    } catch (IllegalArgumentException e) {
      assertEquals(e.getMessage(), "bad arg: 1");
      assertEquals(t.getCT(), 1);
    }
  } 

  public void testAndAssert3() {
    assertEquals(1, t.testAndAssert2(null));
  }

  public void testConstraintCallNormal() {
    assertTrue(t.testConstraintCallException(new Integer(2)));
  }

  public void testConstraintCallException() {
    assertFalse(t.testConstraintCallException(null));
  }

  public void testMethodCallNormal() {
    assertTrue(t.testMethodCallException(new Point(3, 4)));
  }

  public void testMethodCallException() {
    assertFalse(t.testMethodCallException(null));
  }

  public void testArrayLength1() {
    try {
      int [] ar = t.arrayLength1();
      fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testArrayLength2() {
    try {
      int [] ar = t.arrayLength2();
      fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testCompareArrayLength() {
//    try {
      int [] ar = {1, 2};
      t.compareArrayLength(ar);
//      fail("should throw PostconditionException");
//    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
//    }
  }

  public void testCompareArrayLength2() {
      int [] ar = t.compareArrayLength();
      assertEquals(ar[0], 1);
      assertEquals(ar[1], 2);
  }

  public void testCompareArrayLength3() {
      int [] ar = {4, 5};
      int [] ar2 = t.compareArrayLength2(ar);
      assertEquals(ar2[0], 1);
      assertEquals(ar2[1], 2);
      assertEquals(ar, ar2);
  }

  public void testExistsR2() {
    assertEquals(1, t.testExistsR2(3));
  }

  public void testOldMethod() {
    int [] ar = new int [5];
    t.set(1);
    assertEquals(t.testOldMethod(ar), 2);
    assertEquals(ar[2], 4);
    assertEquals(t.getCT(), 3);
  }

  public void testNotModifiedArrayPos() {
    int [] arr = {3,4,5,6,7,8};
    t.testNotModifiedArrayPos(arr, 2);
    assertEquals(arr[2], 5);
    for (int i = 0; i < arr.length; i++) {
      if (i != 2) {
         assertEquals(arr[i], 37);
      }
    }
  }

  public void testNormalExceptionalNoError() {
    t.testNormalExceptional(4);
  }

  public void testNormalExceptionalNormExceptError() {
    try {
      t.testNormalExceptional(2);
      fail("should throw InconsistentSpecificationException");
    } catch (org.jmlspecs.jmlexec.runtime.InconsistentSpecificationException e) {
    }
  }

  public void testNormalExceptionalExceptExceptError() {
    try {
      t.testNormalExceptional(0);
      fail("should throw InconsistentSpecificationException");
    } catch (org.jmlspecs.jmlexec.runtime.InconsistentSpecificationException e) {
    }
  }

  public void testConstructorNoAssignable() {
    Test t = new Test(false);
    assertEquals(t.getCT(), 5);
    assertFalse(t.getB());
    assertNull(t.getElems());
    assertEquals(t.getJ(), 7);
    assertEquals(t.getD(), 3.14, 0.001);
  }

  public void testConstructorNoAssignableFail() {
    try {
      Test t = new Test(true);
      fail("should throw PostconditionException");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }
    
  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestTest.class);
  }

  public static void main(String argv[]) throws Exception {
     junit.textui.TestRunner.run (suite());
//    Test t = new Test();
//    t.testNotModified();
/*
    TestTest tt = new TestTest("test");
    tt.setUp();
    tt.testThrows4();
//    tt.testExceptionalExceptional3();
//    tt.testIsFreshAssertTrue();
//    tt.testStaticRef3();
//    tt.testNormalExceptionalExceptExceptError();
*/
//    tt.testForallEmpty();
//    tt.testExistsRTrue();
//    tt.testCompareArrayLength();
//    tt.testPrefix();
//    tt.testSetOr1();
//    tt.testFreshToString();
//    tt.testConstraintCallException();
//    tt.testAndAssert3();
//    tt.testNot2();

//    tt.testResult1();
//    tt.testExceptionalExceptional3();
//    tt.testThrows3();
//    tt.testSignals();

//    tt.testStaticRef3();
//    tt.testTestAssertTrue();
//    tt.testAddMult8();
//    tt.testModelConstraint2();
//    tt.testTestModel();
//    tt.testExists5();
//    tt.testStaticRef0();
//    tt.testAddFDArray();
//    tt.testTestCasesA();
//    tt.testSumAll();
//    tt.testArrayFields();
//    tt.testFields();
//    tt.testSetElems();
//   tt.testNoSetConc();

/*
  try {
    Test t = new Test();
  } catch (java.lang.VerifyError e) {
    System.out.println(e.getCause());
    e.printStackTrace();
  }
//    t.testStaticInvStatic();
//    System.out.println(t.getCT());
*/
  }
}
