package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestTest2.java,v 1.1 2008/04/18 15:44:29 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.multijava.util.testing.TestCase;

public class TestTest2 extends TestCase {
  private Test2 t;

  public TestTest2(String msg) {
     super(msg);
  }

  protected void setUp() {
    t = new Test2();
  }

  public void testCallThrows() {
    try {
      t.testCallThrows(0);
//      fail("should throw TestE (via ExceptionException)");
      fail("should throw InsufficientInformationException");
    } catch (org.jmlspecs.jmlexec.runtime.InsufficientInformationException e) {
    }
  }

  public void testIsSmall() {
    assertTrue(t.testSmall(4));
  }

  public void testIsSmallFalse() {
    assertFalse(t.testSmall(6));
  }

  public void testSmall2() {
    t.testSmall2(4);
  }

  public void testSmall2Fail() {
    try {
      t.testSmall2(6);
      fail("ensures clause should not be satisfied");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testFieldArrayPos() {
    Test [] ta = {new Test(), new Test(), new Test()};
    Test t2 = new Test();
    t = new Test2(ta);
    Test t3 = t.get(1);
    Test t4 = t.get(2);
    t.testFieldArrayPos(t2);
    assertEquals(t.get(0), t2);
    assertEquals(t.get(1), t3);
    assertEquals(t.get(2), t4);
  }

  public void testFieldArrayRange() {
    Test [] ta = {new Test(), new Test(), new Test()};
    Test t2 = new Test();
    t = new Test2(ta);
    Test t4 = t.get(2);
    t.testFieldArrayRange(t2);
    assertEquals(t.get(0), t2);
    assertEquals(t.get(1), t2);
    assertEquals(t.get(2), t4);
  }

  public void testFieldArrayAll() {
    Test [] ta = {new Test(), new Test(), new Test()};
    Test t2 = new Test();
    t = new Test2(ta);
    Test t4 = t.get(2);
    t.testFieldArrayAll(t2);
    assertEquals(t.get(0), t2);
    assertEquals(t.get(1), t2);
    assertEquals(t.get(2), t2);
  }

  public void testFieldArrayFieldPos() {
    Test [] ta = {new Test(), new Test(), new Test()};
    t = new Test2(ta);
    t.testFieldArrayFieldPos();
    assertEquals(t.get(0).getCT(), 0);
    assertEquals(t.get(1).getCT(), 5);
    assertEquals(t.get(2).getCT(), 0);
  }

  public void testFieldArrayFieldRange() {
    Test [] ta = {new Test(), new Test(), new Test()};
    t = new Test2(ta);
    t.testFieldArrayFieldRange();
    assertEquals(t.get(0).getCT(), 0);
    assertEquals(t.get(1).getCT(), 1);
    assertEquals(t.get(2).getCT(), 0);
  }

  public void testFieldArrayFieldAll() {
    Test [] ta = {new Test(), new Test(), new Test()};
    t = new Test2(ta);
    t.testFieldArrayFieldAll();
    assertEquals(t.get(0).getCT(), 0);
    assertEquals(t.get(1).getCT(), 1);
    assertEquals(t.get(2).getCT(), 2);
  }

  public void testSignalsOnlyNoSignals() {
    assertEquals(3, t.testSignalsOnly(4));
  }

  public void testSignalsOnly() {
    try {
      assertEquals(0, t.testSignalsOnly(0));
      fail("should throw IllegalArgumentException");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testNested() {
    Test2.Nested n = t.newNested(3, 4);
    assertEquals(n.getI(), 3);
    assertEquals(n.getJ(), 4);
  }

  public void testNestedFailInvariant() {
    try {
      Test2.Nested n = t.newNested(50, 4);
      fail("should throw PostconditionException (invariant not satisfied)");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }

  public void testStaticNested() {
    Test2.StaticNested n = t.newStaticNested(3);
    assertEquals(n.getX(), 3);
  }

  public void testStaticNestedExternal() {
    Test2.StaticNested n = new Test2.StaticNested(3);
    assertEquals(n.getX(), 3);
  }

  public void testNestedNested() {
    Test2.Nested.NestedNested n = t.newNestedFoo();
    assertEquals(n.getD(), 3.14, 0.01);
  }

  public void testFreshTwice() {
    Test2 t2 = t.testFreshTwice();
    assertEquals(t2.getCT(), 0);
    assertEquals(t2.getJ(), 3);
  }

  public void testFreshTwiceDelay() {
    Test2 t2 = t.testFreshTwiceDelay(3);
    assertEquals(t2.getCT(), 0);
    assertEquals(t2.getJ(), 3);
  }

  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestTest2.class);
  }
    
  public static void main(String argv[]) {
     junit.textui.TestRunner.run (suite());
/*
    TestTest2 tt = new TestTest2("test");
    tt.setUp();
    tt.testNestedNested();
//    tt.testFreshTwice();
//    tt.testFreshTwiceDelay();
//    tt.testCallThrows();
//    tt.testIsSmall();
//    tt.testExceptionalExceptional2();
//   tt.testNoSetConc();
*/

/*
  try {
    Test t = new Test();
  } catch (java.lang.VerifyError e) {
    System.out.println(e.getCause());
    e.printStackTrace();
  }
//    t.testStaticInvStatic();
//    System.out.println(t.getCT());
*/
  }
}
