package org.jmlspecs.jmlexec.testcase;

/*
 * @(#)$Id: TestTestSeq.java,v 1.1 2008/04/18 15:44:29 wahlst Exp $
 *
 * Copyright (C) 2006 Iowa State University
 *
 * This file is part of JML
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import org.jmlspecs.models.JMLObjectSequence;
import org.multijava.util.testing.TestCase;

public class TestTestSeq extends TestCase {
  private TestSeq t;

  public TestTestSeq(String test) {
    super(test);
  }

  protected void setUp() { t = new TestSeq();}
  protected void tearDown() {}

  public void testConstructor() {
    assertEquals(t.getElems(), JMLObjectSequence.EMPTY);
  }

  public void testCons() {
    Integer i = new Integer(3);
    t.cons(i);
    assertEquals(t.getElems(), new JMLObjectSequence(i));
  }

  public void testConcat() {
    Integer i = new Integer(3);
    t.cons(i);
    t.concat(t);
    assertEquals(t.getElems(), (new JMLObjectSequence(i)).concat(new JMLObjectSequence(i)));
  }

  public void testCount() {
    Integer i1 = new Integer(1), i2 =new Integer(2), i3 = new Integer(3);
    t.testCount(i1, i2, i3);
    JMLObjectSequence s = new JMLObjectSequence().insertBack(i1).insertBack(i3).insertBack(i2);
    assertEquals(t.getElems(), s);
  }

  public void testCount2() {
    Integer i1 = new Integer(1), i2 =new Integer(2), i3 = new Integer(3);
    t.testCount2(i1, i2, i3);
// <1, 3, 2, 2, 3>
    JMLObjectSequence s = new JMLObjectSequence().insertBack(i1).insertBack(i3).insertBack(i2).insertBack(i2).insertBack(i3);
    assertEquals(t.getElems(), s);
  }

  public void testAppend() {
    assertEquals(t.append(), new JMLObjectSequence());
  }

  public void testAppend2() {
    Integer i = new Integer(3);
    Integer j = new Integer(4);
    t.cons(i);
    t.cons(j);
    t.cons(i);
    t.cons(j);
    assertEquals(t.append(), new JMLObjectSequence(j).insertBack(i));
  }

  public void testHas() {
    Integer i = new Integer(3);
    Integer j = new Integer(4);
    t.cons(i);
    t.cons(j);
    assertEquals(t.has(j), true);
  }

  public void testHas01() {
    Integer i = new Integer(3);
    Integer j = new Integer(4);
    t.cons(i);
    t.cons(j);
    assertEquals(t.has(new Integer(3)), false);
  }

  public void testHas2() {
    try {
      Integer i = new Integer(3);
      Integer j = new Integer(4);
      t.cons(i);
      t.cons(j);
      assertEquals(t.has2(new Integer(3)), false);
      fail("should have insufficient info");
    } catch (org.jmlspecs.jmlexec.runtime.InsufficientInformationException e) {
    }
  }

  public void testExists() {
    Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
    t.cons(i);
    t.cons(j);
    t.cons(k);
    assertEquals(t.testExists(k, i), j);
  }

  public void testExists2() {
    try {
      t.testExists2();
      fail("should have postcondition exception");
    } catch (org.jmlspecs.jmlexec.runtime.PostconditionException e) {
    }
  }


  public void testHas31() {
    Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
    t.cons(i);
    t.cons(j);
    t.cons(k);
    assertEquals(t.has3(k), true);
  }

  public void testHas32() {
    Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
    t.cons(i);
    t.cons(j);
    t.cons(k);
    assertEquals(t.has3(new Integer(3)), false);
  }

  public void testHas4() {
    try {
      Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
      t.cons(i);
      t.cons(j);
      t.cons(k);
      assertEquals(t.has4(k), true);
      fail("should have insufficient info");
    } catch (org.jmlspecs.jmlexec.runtime.InsufficientInformationException e) {
    }
  }

  public void testForall() {
    Integer j = new Integer(2), k = new Integer(1);
    t.cons(j);
    t.cons(k);
    t.testForall();
    assertEquals(t.getElems(), new org.jmlspecs.models.JMLObjectSequence(k).insertBack(k).insertBack(j).insertBack(j));
  }

/*
  public void testForall2() {
    Integer j = new Integer(2), k = new Integer(1);
    t.cons(j);
    t.cons(k);
    t.testForall2();
    assertEquals(t.getElems().int_size(), 4);
  }
*/

  public void testForall3() {
    Integer j = new Integer(2), k = new Integer(1);
    t.cons(j);
    t.cons(k);
    t.testForall3(k);
    assertEquals(t.getElems().int_size(), 1);
  }

  public void testForall31() {
    Integer j = new Integer(2), k = new Integer(1);
    t.cons(k);
    t.testForall3(k);
    assertEquals(t.getElems().int_size(), 0);
  }

  public void testForall4() {
    t.testForall4();
    assertEquals(t.getElems().int_size(), 0);
  }

  public void testInsertionInto() {
    Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
    t.cons(j);
    t.cons(k);
    TestSeq t2 = new TestSeq();
    t2.cons(j);
    t2.cons(i);
    t2.cons(k);
    assertEquals(t2.testInsertion(t, i), true);
  }

  public void testDeletionFrom() {
    Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
    t.cons(j);
    t.cons(k);
    TestSeq t2 = new TestSeq();
    t2.cons(j);
    t2.cons(i);
    t2.cons(k);
    assertEquals(t.testDeletion(t2, i), true);
  }

  public static junit.framework.Test suite() {
     return new junit.framework.TestSuite(TestTestSeq.class);
  }

  public static void main(String argv[]) {
    junit.textui.TestRunner.run (suite());
/*
    TestTestSeq tt = new TestTestSeq("testing");
    tt.setUp();
    tt.testCons();
*/
/*
   TestSeq t = new TestSeq();
    Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
    t.cons(i);
    t.cons(j);
    t.cons(k);
    System.out.println(t.testExists(k, i));
*/
/*
    Integer i1 = new Integer(1), i2 =new Integer(2), i3 = new Integer(3);
    t.testCount(i1, i2, i3);
*/
/*
    JMLObjectSequence s = new JMLObjectSequence().insertBack(i1).insertBack(i3).insertBack(i2);
    Integer i = new Integer(3);
    Integer j = new Integer(4);
    t.cons(i);
    t.cons(j);
    System.out.println(t.has(j));
*/
/*
   Integer i = new Integer(3);
   t.testLength(i);
   System.out.println(t.getElems());
*/
/*
   Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
   System.out.println("calling testCount");
   t.testCount3(i, j);
   System.out.println(t.getElems());
*/
/*
   TestSeq t2 = new TestSeq();
   t2.cons(j);
   t2.cons(i);
   t2.cons(k);
   System.out.println(t2.testInsertion(t, i));
*/
  
/*
   TestTestSeq d = new TestTestSeq("testing");
   d.setUp();
   d.testCount();
*/
/*
   TestSeq t = new TestSeq();
   Integer i = new Integer(3), j = new Integer(2), k = new Integer(1);
   t.cons(i);
   t.cons(j);
   System.out.println(t.getElems());
   System.out.println(t.has(i));
   System.out.println(t.has(new Integer(3)));
*/
/*
   t.testCount2(k, j, i);
   System.out.println(t.getElems());
*/
/*
   Integer i = new Integer(3);
   t.cons(i);
   t.cons(i);
   // t.cons(new Integer(3));
   // System.out.println(t.getElems());
   JMLObjectSequence s = t.append();
    System.out.println(s);
*/
/*
     suite.addTest(new TestTestSeq("testConstructor"));
     suite.addTest(new TestTestSeq("testCons"));
*/
/*
     TestSuite suite= new TestSuite();
     suite.addTestSuite(TestTestSeq.class);
     TestResult result = new TestResult();
     suite.run(result);
*/

  }
}
