/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlrac.testcase.racopt;

import org.multijava.util.testing.*;
import junit.framework.*;

/** This is a JUnit test case that executes the tests in the package
 * in which it resides.  This test is actually a suite of tests
 * produced from a sequence of filenames (contained in
 * <code>listTests</code> and others).
 *
 * <P> The test must be run from the directory in which it resides -
 * because it opens files in the current directory.
 *
 * @author David R. Cok
 * @author Yoonsik Cheon
 */
public class TestSuite extends org.multijava.util.testing.TestSuite {

    static public Test suite() {
        // compile helper classes
        org.multijava.util.testing.TestSuite suite =
            new TestFilesCompileOnlyTestSuite(
                "TestSuite for listTestsHelper",
                "listTestsHelper",
                new String[]{"-Q", "-p"},
                org.jmlspecs.jmlrac.Main.class);

        // generic testing
        suite.addTest(
            new TestFilesCompileAndRunTestSuite(
                "TestSuite for listTests",
                "listTests",
                new String[]{"-Q", "-p", "-O"},
                org.jmlspecs.jmlrac.Main.class));

        // test option -t
        suite.addTest(
            new TestFilesCompileAndRunTestSuite(
                "TestSuite for listTestsOptionT",
                "listTestsOptionT",
                new String[]{"-Q", "-p", "-t"},
                org.jmlspecs.jmlrac.Main.class));

        // test option -Y
        suite.addTest(
            new TestFilesCompileAndRunTestSuite(
                "TestSuite for listTestsOptionY",
                "listTestsOptionY",
                new String[]{"-Q", "-p", "-Y"},
                org.jmlspecs.jmlrac.Main.class));

        // test option -F
        suite.addTest(
            new TestFilesCompileAndRunTestSuite(
                "TestSuite for listTestsOptionF",
                "listTestsOptionF",
                new String[]{"-Q", "-p", "-F"},
                org.jmlspecs.jmlrac.Main.class));

        // test option -U
        suite.addTest(
            new TestFilesCompileAndRunTestSuite(
                "TestSuite for listTestsOptionU",
                "listTestsOptionU",
                new String[]{"-Q", "-p", "-U"},
                org.jmlspecs.jmlrac.Main.class));


        // test files to be compiled without RAC
        suite.addTest(
            new TestFilesCompileAndRunTestSuite(
                "TestSuite for listTestsNoRac",
                "listTestsNoRac",
                new String[]{},
                org.multijava.mjc.Main.class));

        return suite;
    }
}
