// Test if down calls from the boot classes such as JML's
// java.util.Enumeration work correctly. For this, the model field
// moreElements defined in the Enumeration interface is given a
// represent clause in this class, and used in the postconidtions of
// test methods m1 and m2.

import java.util.Enumeration;
import org.jmlspecs.jmlrac.runtime.*;

public class BootClassDownCall implements Enumeration {

    // dummy methods to implement the Enumeration interface
    public boolean hasMoreElements() { return true; }
    public /*@ nullable @*/ Object nextElement() { return null; }

    public boolean hasMore = true;
    //@ public represents moreElements <- hasMore;
    // The model field moreElements is defined in Enumeration

    // no assertion violation
    //@ ensures moreElements;
    public void m1() {}

    // postcondition violation
    //@ ensures !moreElements;
    public void m2() {}

    public static void main(String args[]) {
        BootClassDownCall exp = new BootClassDownCall();
        int fcnt = 0;

        try { exp.m1(); }
        catch (JMLAssertionError e) { fcnt++; }

        try { exp.m2(); fcnt++; }
        catch (JMLPostconditionError e) {}
        catch (JMLAssertionError e) {}

        if (fcnt > 0) {
            System.out.println(fcnt + "F(BootClassDownCall.java)");
        }
    }
}
