import org.jmlspecs.jmlrac.runtime.*;

public class CompoundAssignmentExp_bigint {

    /* Excerpt from JLS, Section 15.26.2 Compound Assignment Operators A
     * compound assignment expression of the form E1 op= E2 is equivalent to
     * E1 = (T)((E1) op (E2)), where T is the type of E1, except that E1 is
     * evaluated only once. Note that the implied cast to type T may be either
     * an identity conversion ('5.1.1) or a narrowing primitive conversion
     * ('5.1.3).
     */

    /*@  public static pure model void test() { 
      @   \bigint i=0, j=0;
      @   int k = 0;
      @   short s = 0;
      @
      @   // k = i; // error: Illegal attempt to assign a value of type "\bigint" to a variable of type "int" [JLS 15.25]
      @   // k = k * i; // error: Illegal attempt to assign a value of type "\bigint" to a variable of type "int" [JLS 15.25]
      @   k = (int)i;
      @   k *= i; // ok, equivalent to k = (int)(k * i);
      @   i *= k;
      @   s += k; // ok
      @
      @   i *= j;
      @   i /= j;
      @   i %= j;
      @   i += j;
      @   i -= j;
      @   i <<= j;
      @   i >>= j;
      @   //i >>>= j;
      @   i &= j;
      @   i ^= j;
      @   i |= j;
      @ }
      @*/

    /*@
    // ------------------------------------------------------------------------------
    // Mul-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testMulAssExp(int i, int j) { 
	return i *= j;
    }
    
    public static pure model \bigint testMulAssExp(\bigint i, int j) { 
	return i *= j;
    }

    // ------------------------------------------------------------------------------
    // Div-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testDivAssExp(int i, int j) { 
	return i /= j;
    }
    
    public static pure model \bigint testDivAssExp(\bigint i, int j) { 
	return i /= j;
    }

    // ------------------------------------------------------------------------------
    // Plus-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testPlusAssExp(int i, int j) { 
	return i += j;
    }
    
    public static pure model \bigint testPlusAssExp(\bigint i, int j) { 
	return i += j;
    }

    // ------------------------------------------------------------------------------
    // Minus-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testMinusAssExp(int i, int j) { 
	return i -= j;
    }
    
    public static pure model \bigint testMinusAssExp(\bigint i, int j) { 
	return i -= j;
    }

    // ------------------------------------------------------------------------------
    // Rem-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testRemAssExp(int i, int j) { 
	return i %= j;
    }
    
    public static pure model \bigint testRemAssExp(\bigint i, int j) { 
	return i %= j;
    }

    // ------------------------------------------------------------------------------
    // ShiftLeft-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testShiftLeftAssExp(int i, int j) { 
	return i <<= j;
    }
    
    public static pure model \bigint testShiftLeftAssExp(\bigint i, int j) { 
	return i <<= j;
    }

    // ------------------------------------------------------------------------------
    // ShiftRight-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testShiftRightAssExp(int i, int j) { 
	return i >>= j;
    }
    
    public static pure model \bigint testShiftRightAssExp(\bigint i, int j) { 
	return i >>= j;
    }

    // ------------------------------------------------------------------------------
    // And-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testAndAssExp(int i, int j) { 
	return i &= j;
    }
    
    public static pure model \bigint testAndAssExp(\bigint i, int j) { 
	return i &= j;
    }

    // ------------------------------------------------------------------------------
    // Or-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testOrAssExp(int i, int j) { 
	return i |= j;
    }
    
    public static pure model \bigint testOrAssExp(\bigint i, int j) { 
	return i |= j;
    }

    // ------------------------------------------------------------------------------
    // Xor-Ass expressions
    // ------------------------------------------------------------------------------
    public static pure model int testXorAssExp(int i, int j) { 
	return i ^= j;
    }
    
    public static pure model \bigint testXorAssExp(\bigint i, int j) { 
	return i ^= j;
    }
    @*/


    //-------------------  test methods --------------------------

    //@ requires testMulAssExp(i, j) == testMulAssExp((\bigint)i, j);
    public void mMulAssExp(int i, int j) { }

    //@ requires testDivAssExp(i, j) == testDivAssExp((\bigint)i, j);
    public void mDivAssExp(int i, int j) { }

    //@ requires testPlusAssExp(i, j) == testPlusAssExp((\bigint)i, j);
    public void mPlusAssExp(int i, int j) { }

    //@ requires testMinusAssExp(i, j) == testMinusAssExp((\bigint)i, j);
    public void mMinusAssExp(int i, int j) { }

    //@ requires testRemAssExp(i, j) == testRemAssExp((\bigint)i, j);
    public void mRemAssExp(int i, int j) { }

    //@ requires testShiftLeftAssExp(i, j) == testShiftLeftAssExp((\bigint)i, j);
    public void mShiftLeftAssExp(int i, int j) { }

    //@ requires testShiftRightAssExp(i, j) == testShiftRightAssExp((\bigint)i, j);
    public void mShiftRightAssExp(int i, int j) { }

    //@ requires testAndAssExp(i, j) == testAndAssExp((\bigint)i, j);
    public void mAndAssExp(int i, int j) { }

    //@ requires testOrAssExp(i, j) == testOrAssExp((\bigint)i, j);
    public void mOrAssExp(int i, int j) { }

    //@ requires testXorAssExp(i, j) == testXorAssExp((\bigint)i, j);
    public void mXorAssExp(int i, int j) { }


    //------------------ main method -----------------------------

    public static void main(String args[]) {
	CompoundAssignmentExp_bigint test = new CompoundAssignmentExp_bigint();
	int fcnt = 0, i = 8, j = 2;

	try { test.mMulAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mDivAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mPlusAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mMinusAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mRemAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mShiftLeftAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mShiftRightAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mAndAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mOrAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	try { test.mXorAssExp(i, j); }
        catch (JMLAssertionError e) { fcnt++; }

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(CompoundAssignmentExp_bigint.java)");
        }
    }
}
