import org.jmlspecs.jmlrac.runtime.*;

// Old semantics: the order of subexpression evaluations shouldn't
// matter. This error is reported in Bug Report 1161560.
// New semantics: Java conditional operators remain conditional.

public class Contextual5 {
    public /*@ nullable @*/ Contextual5 junk;
    public int x;

    //@ requires (x == 0 || junk.x > 0) == (x == 0);
    public void m1() { }

    //@ requires (junk.x > 0 || x == 0) == (x == 0);
    public void m2() { }

    public static void main(String args[]) {
	Contextual5 test = new Contextual5();
	int fcnt = 0;

        // no assertion violation
	try { test.m1(); }
        catch (JMLAssertionError e) { fcnt++; }

        // old sem: no assertion violation
	if(JMLRacUtil.oldSem()) {
	    try { test.m2(); }
	    catch (JMLAssertionError e) { fcnt += 10; }
	} else {
	// new sem: invalid precondition.
	    try { test.m2(); fcnt += 2; }
	    catch (JMLEvaluationError e) {
		if(!JMLRacUtil.matchCause(JMLPreconditionError.class, e)) fcnt += 20;
	    }
	}

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(Contextual5.java)");
        }
    }
}

// @(#)$Id: Contextual5.java,v 1.6 2006/05/30 02:27:19 f_rioux Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
