// @(#)$Id: DesugaringInheritance1.java,v 1.1 2006/01/28 05:13:19 chalin Exp $
// (C) - see below.

/**
 * Purpose: ensure that a subclass method with no explicit contract
 * inherits contract of its superclass wholesale.
 *
 * @author Patrice Chalin (chalin@dsrg.org)
 */

import org.jmlspecs.jmlrac.runtime.*;

public class DesugaringInheritance1 extends org.jmlspecs.util.RacRunTestCase {

    public static void main(String[] args) {
	(new DesugaringInheritance1()).run(args);
    }

    // Test cases ============================================================

    private CDesugaringInheritance1 testee = new CDesugaringInheritance1();

    public void test_m0_0() {
	try { testee.m0(0); fail();
	} catch(JMLPreconditionError e) { /* ok */ }
    }

    public void test_m0_1() {
	testee.m0(1);
    }

    public void test_m0_5() {
	testee.m0(5);
    }

    public void test_m0_minus1() {
	try { 
	    testee.m0(-1); fail();
	} catch(IllegalArgumentException e) { /* ok */ }
    }

    public void test_m0_minus1_for_parent() {
	CDesugaringInheritance1Super testee = new CDesugaringInheritance1Super();
	try { 
	    testee.m0(-1); fail();
	} catch(IllegalArgumentException e) { /* ok */ }
    }

    public void test_m1_0() {
	try { testee.m1(0); fail();
	} catch(JMLPreconditionError e) { /* ok */ }
    }

    public void test_m1_1() {
	try { testee.m1(1); fail();
	} catch(JMLPostconditionError e) { /* ok */ }
    }

    public void test_m1_5() {
	try { testee.m1(5); fail();
	} catch(JMLPostconditionError e) { /* ok */ }
    }

    public void test_m1_minus1() {
	try { testee.m1(-1); fail();
	} catch(JMLPostconditionError e) { /* ok */ }
    }

    public void test_m1_minus1_for_parent() {
	CDesugaringInheritance1Super testee = new CDesugaringInheritance1Super();
	try { 
	    testee.m1(-1); fail();
	} catch(IllegalArgumentException e) { /* ok */ }
    }
}

class CDesugaringInheritance1 extends CDesugaringInheritance1Super {

    // no explicit contract, hence it is inherited from parent
    int m0(int i) { return super.m0(i); }

    /*@ also normal_behavior
      @  requires true;
      @  assignable \nothing;
      @  ensures \result <= i;
      @ also exceptional_behavior
      @  requires true;
      @  assignable \nothing;
      @  signals (Exception e) false;
      @*/
    int m1(int i) { return super.m1(i); }
}

class CDesugaringInheritance1Super {
    /*@ normal_behavior
      @  requires i > 0;
      @  assignable \nothing;
      @  ensures \result < 10;
      @ also exceptional_behavior
      @  requires i < 0;
      @  assignable \nothing;
      @  signals (IllegalArgumentException e) true;
      @*/
    int m0(int i) {
	if(i > 0) {
	    return i; 
	} else {
	    throw new IllegalArgumentException("i < 0");
	}
    }

    /*@ normal_behavior
      @  requires i > 0;
      @  assignable \nothing;
      @  ensures \result < 10;
      @ also exceptional_behavior
      @  requires i < 0;
      @  assignable \nothing;
      @  signals (IllegalArgumentException e) true;
      @*/
    int m1(int i) {
	if(i > 0) {
	    return i*2;
	} else {
	    throw new IllegalArgumentException("i < 0");
	}
    }
}

// Copyright (C) 2006 Iowa State University

// This file is part of JML.

// JML is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

