// Assertion check rules for finalizers: everything except for
// post-state instance invariants and constraints. That is, the
// following assertions are checked: precondition, postcondition,
// pre-static invariants (both static and instance), post-state
// static invariants and static constraints.

import org.jmlspecs.jmlrac.runtime.*;

public class Finalizer1 {

    //@ also requires false;
    public void finalize() throws Throwable {
        super.finalize();
    }

    public static void main(String args[]) {
	int fcnt = 0;

        // precondition violation
	try { new Finalizer1Helper1().finalize(); fcnt++; }
        catch (JMLPreconditionError e) {}
        catch (Throwable e) { fcnt++; }

        // postcondition violation
	try { new Finalizer1Helper2().finalize(); fcnt++; }
        catch (JMLPostconditionError e) {}
        catch (Throwable e) { fcnt++; }

        // pre-state instance invariant violation
	try { new Finalizer1Helper3().finalize(); fcnt++; }
        catch (JMLInvariantError e) {}
        catch (Throwable e) { fcnt++; }

        // pre-state static invariant violation
	try { new Finalizer1Helper4().finalize(); fcnt++; }
        catch (JMLInvariantError e) {}
        catch (Throwable e) { fcnt++; }

        // post-state static invariant violation
	try { new Finalizer1Helper5().finalize(); fcnt++; }
        catch (JMLInvariantError e) {}
        catch (Throwable e) { fcnt++; }

        // static constraint violation
	try { new Finalizer1Helper6().finalize(); fcnt++; }
        catch (JMLHistoryConstraintError e) {}
        catch (Throwable e) { fcnt++; }


        // no check for post-state instance invariant
	try { new Finalizer1Helper7().finalize(); }
        catch (Throwable e) { fcnt++; }

        // no check forinstance constraint
	try { new Finalizer1Helper8().finalize(); }
        catch (Throwable e) { fcnt++; }

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(Finalizer1.java)");
        }
    }
}

class Finalizer1Helper1 {
    //@ also requires false;
    public void finalize() throws Throwable {
        super.finalize();
    }    
}

class Finalizer1Helper2 {
    //@ also ensures false;
    public void finalize() throws Throwable {
        super.finalize();
    }    
}

class Finalizer1Helper3 {
    int x = 0;
    //@ invariant x > 0;

    public void finalize() throws Throwable {
        super.finalize();
    }    
}

class Finalizer1Helper4 {
    static int x = 0;
    //@ static invariant x > 0;

    public void finalize() throws Throwable {
        super.finalize();
    }    
}

class Finalizer1Helper5 {
    static int x = 0;
    //@ static invariant x >= 0;

    public void finalize() throws Throwable {
        super.finalize();
        x = -10;
    }    
}

class Finalizer1Helper6 {
    static int x = 0;
    //@ static constraint x > \old(x);

    public void finalize() throws Throwable {
        super.finalize();
        x = -10;
    }    
}

class Finalizer1Helper7 {
    int x = 0;
    //@ in objectState;
    //@ invariant x >= 0;

    public void finalize() throws Throwable {
        super.finalize();
        x = -10;
    }    
}

class Finalizer1Helper8 {
    int x = 0;
    //@ in objectState;
    //@ constraint x > \old(x);

    public void finalize() throws Throwable {
        super.finalize();
        x = -10;
    }    
}

// @(#)$Id: Finalizer1.java,v 1.2 2003/09/10 04:02:10 cruby Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
