// Assertion check rules for finalizers: everything except for
// post-state instance invariants and constraints. That is, the
// following assertions are checked: precondition, postcondition,
// pre-static invariants (both static and instance), post-state
// static invariants and static constraints.

import org.jmlspecs.jmlrac.runtime.*;

public class Finalizer2 {

    //@ also requires false;
    public void finalize() throws Throwable {
        super.finalize();
    }

    public static void main(String args[]) {
	// fcnt is increased if any unexpected event occurs, the
	// increases are by powers of 2 so that the kind of unexpected
	// event can be identified
	int fcnt = 0;

        // precondition violation
	try { new Finalizer2Helper1().finalize(); fcnt += 1; }
        catch (JMLPreconditionError e) {}
        catch (Throwable e) { fcnt += 2; }

        // postcondition violation
	try { new Finalizer2Helper2().finalize(); fcnt += 4; }
        catch (JMLPostconditionError e) {}
        catch (Throwable e) { fcnt += 8; }

        // pre-state instance invariant violation
	try { new Finalizer2Helper3().finalize(); fcnt += 16; }
        catch (JMLInvariantError e) {}
        catch (Throwable e) { fcnt += 32; }

        // no check for post-state instance invariant
	try { new Finalizer2Helper7().finalize(); }
        catch (Throwable e) { fcnt += 64; }

        // no check forinstance constraint
	try { new Finalizer2Helper8().finalize(); }
        catch (Throwable e) { fcnt += 128; }

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(Finalizer2.java)");
        }
    }
}

class Finalizer2Helper1 implements Finalizer2Helper1IF {
    public void finalize() throws Throwable {
        super.finalize();
    }    
}
interface Finalizer2Helper1IF {
    //@ also requires false;
    void finalize() throws Throwable;
}

class Finalizer2Helper2 implements Finalizer2Helper2IF {
    public void finalize() throws Throwable {
        super.finalize();
    }    
}
interface Finalizer2Helper2IF {
    //@ also ensures false;
    void finalize() throws Throwable;
}

class Finalizer2Helper3 implements Finalizer2Helper3IF {
    public void finalize() throws Throwable {
        super.finalize();
    }    
}
interface Finalizer2Helper3IF {
    //@ invariant false;
    void finalize() throws Throwable;
}

class Finalizer2Helper7 implements Finalizer2Helper7IF {
    public void finalize() throws Throwable {
        super.finalize();
        //@ set x = -10;
    }    
}
interface Finalizer2Helper7IF {
    //@ ghost int x = 0;
    //@ invariant x >= 0;
    void finalize() throws Throwable;
}

class Finalizer2Helper8 implements Finalizer2Helper8IF {
    public void finalize() throws Throwable {
        super.finalize();
    }    
}
interface Finalizer2Helper8IF {
    //@ constraint false;
    void finalize() throws Throwable;
}

// @(#)$Id: Finalizer2.java,v 1.2 2003/02/03 03:54:07 cclifton Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
