// loop invariants for while statements

import org.jmlspecs.jmlrac.runtime.*;

public class LoopInvariant1 {

    void m0() {
        int y = 10;
        while (y > 0) {
            y--;
        }
    }

    void m1(int x) {
        int y = x;
        //@ maintaining y <= x; // OK
        while (y > 0) {
            y--;
        }
    }

    void m2(int x) {
        int y = x;
        //@ maintaining y < x; // initially ERROR
        while (y > 0) {
            y--;
        }
    }

    // loop inside another loop
    void m3(boolean b) {
        int y1 = 0;
        int y2 = 0;

        boolean flag1 = true;
        //@ maintaining y1 == 0; // OK
        while (flag1) {

            boolean flag2 = true;
            //@ maintaining y2 == 0; // ERROR if !b
            while (flag2) {
                if (!b) 
                    y2--;
                flag2 = false;
            }
            flag1 = false;
        }
    }

    // loop with continue
    void m4(boolean b) {
        int y = 0;
        boolean flag = true;
        //@ maintaining y == 0; // ERROR if !b
        while (flag) {
            if (!b) {
                if (y < 0) {
                    flag = false;
                } else {
                    y--;
                    continue;
                }
            }
            flag = false;
        }
    }

    // loop with break
    void m5(boolean b) {
        int y = 0;
        boolean flag = true;
        //@ maintaining y == 0; // ERROR if !b
        while (flag) {
            if (!b) {
                y--;
                break;
            }
            flag = false;
        }
    }

    // loop inside a loop
    void m6(boolean b) {
        do {
            int y = 0;
            boolean flag = true;
            //@ maintaining y == 0; // ERROR if !b
            while (flag) {
                if (!b) {
                    y--;
                    break;
                }
                flag = false;
            }
        }
        while (false);
    }

    public static void main(String args[]) {
	LoopInvariant1 test = new LoopInvariant1();
	int fcnt = 0;

	try { test.m0(); } catch (JMLAssertionError e) { fcnt++; }

	try { test.m1(10); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m1(-10); } catch (JMLAssertionError e) { fcnt++; }
        
	try { test.m2(10); fcnt++; } 
        catch (JMLLoopInvariantError e) {}
        catch (JMLAssertionError e) { fcnt++; }

	try { test.m3(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m3(false); fcnt++; } 
        catch (JMLLoopInvariantError e) {}
        catch (JMLAssertionError e) { fcnt++; }

	try { test.m4(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m4(false); fcnt++; }
        catch (JMLLoopInvariantError e) {}
        catch (JMLAssertionError e) { fcnt++; }

	try { test.m5(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m5(false); fcnt++; }
        catch (JMLLoopInvariantError e) {}
        catch (JMLAssertionError e) { fcnt++; }

	try { test.m6(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m6(false); fcnt++; }
        catch (JMLLoopInvariantError e) {}
        catch (JMLAssertionError e) { fcnt++; }

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(LoopInvariant1.java)");
        }
    }
}

// @(#)$Id: LoopInvariant1.java,v 1.1 2003/01/22 04:20:56 cheon Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
