// loop invariants for labeled loop statements

import org.jmlspecs.jmlrac.runtime.*;

public class LoopInvariant4 {

    // loop with continue
    void m1(boolean b) {
        int y = 0;
        boolean flag = true;
        //@ maintaining y == 0; // ERROR if !b
        l1: while (flag) {
            if (!b) {
                if (y < 0) {
                    flag = false;
                } else {
                    y--;
                    continue l1;
                }
            }
            flag = false;
        }
    }

    // loop with break
    void m2(boolean b) {
        int y = 0;
        boolean flag = true;
        //@ maintaining y == 0; // ERROR if !b
        l1: while (flag) {
            if (!b) {
                y--;
                break l1;
            }
            flag = false;
        }
    }

    // loop with continue
    void m3(boolean b) {
        int y = 0;
        boolean flag = true;
        //@ maintaining y == 0; // ERROR if !b
        l1: do {
            if (!b) {
                if (y < 0) {
                    flag = false;
                } else {
                    y--;
                    continue l1;
                }
            }
            flag = false;
        }
        while (flag);
    }

    // loop with break
    void m4(boolean b) {
        int y = 0;
        boolean flag = true;
        //@ maintaining y == 0; // ERROR if !b
        l1: do {
            if (!b) {
                y--;
                break l1;
            }
            flag = false;
        }
        while (flag);
    }

    // loop with continue
    void m5(boolean b) {
        int y = 0;
        //@ maintaining y == 0; // ERROR if !b
        l1: for (int i = 0; i < 3; i++) {
            if (!b) {
                y--;
                continue l1;
            }
        }
    }

    // loop with break
    void m6(boolean b) {
        int y = 0;
        //@ maintaining y == 0; // ERROR if !b
        l1: for (int i = 0; i < 3; i++) {
            if (!b) {
                y--;
                break l1;
            }
        }
    }
    public static void main(String args[]) {
	LoopInvariant4 test = new LoopInvariant4();
	int fcnt = 0;

	try { test.m1(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m1(false); fcnt++; } catch (JMLAssertionError e) {}

	try { test.m2(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m2(false); fcnt++; } catch (JMLAssertionError e) {}

	try { test.m3(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m3(false); fcnt++; } catch (JMLAssertionError e) {}

	try { test.m4(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m4(false); fcnt++; } catch (JMLAssertionError e) {}

	try { test.m5(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m5(false); fcnt++; } catch (JMLAssertionError e) {}

	try { test.m6(true); } catch (JMLAssertionError e) { fcnt++; }
	try { test.m6(false); fcnt++; } catch (JMLAssertionError e) {}


	if (fcnt > 0) {
	    System.out.println(fcnt + "F(LoopInvariant4.java)");
        }
    }
}

// @(#)$Id: LoopInvariant4.java,v 1.1 2003/01/22 04:20:56 cheon Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
