// loop invariants and variants for loop statements

import org.jmlspecs.jmlrac.runtime.*;

public class LoopInvariantAndVariant1 {

    void m1(int x) {
        int y = x;
        int i = 5;
        //@ maintaining y == x; // ERROR if x < 0;
        //@ decreasing i; // OK
        while (i > 0) {
            if (x < 0) {
                y++;
            }
            i--;
        }
    }

    void m2(int x) {
        int y = x;
        int i = 5;
        //@ maintaining y == x; // OK
        //@ decreasing i; // ERROR if x < 0
        while (i > 0) {
            if (x >= 0) {
                i--;
            }
        }
    }

    void m3(boolean b) {
        int y = 10;
        int i = 5;
        //@ maintaining y > 0; // ERROR if !b;
        //@ decreasing i; // OK
        do {
            if (!b) {
                y = -10;
            }
            i--;
        }
        while (i > 0);
    }

    void m4(boolean b) {
        int y = 10;
        int i = 5;
        //@ maintaining y > 0; // OK
        //@ decreasing y; // ERROR if !b
        do {
            if (!b) {
                y++;
            }
            y--;
            i--;
        }
        while (i > 0);
    }

    void m5(boolean b) {
        int y = 10;
        //@ maintaining y > 1; // ERROR if !b
        //@ maintaining_redundantly y > 0;
        //@ decreasing y; // OK
        for (int i = 0; i < 3; i++) {
            if (!b) {
                y = 0;
            }
            y--;
        }
    }

    void m6(boolean b) {
        int y = 10;
        //@ maintaining y > 1; // OK
        //@ maintaining_redundantly y > 0;
        //@ decreasing y; // ERROR if !b
        for (int i = 0; i < 3; i++) {
            if (!b) {
                y++;
            }
            y--;
        }
    }

    public static void main(String args[]) {
	LoopInvariantAndVariant1 test = new LoopInvariantAndVariant1();
	int fcnt = 0;

 	try { test.m1(10); } catch (JMLAssertionError e) { fcnt++; }
 	try { test.m1(-10); fcnt++; } catch (JMLAssertionError e) {}

 	try { test.m2(10); } catch (JMLAssertionError e) { fcnt++; }
 	try { test.m2(-10); fcnt++; } catch (JMLAssertionError e) {}

 	try { test.m3(true); } catch (JMLAssertionError e) { fcnt++; }
 	try { test.m3(false); fcnt++; } catch (JMLAssertionError e) {}

 	try { test.m4(true); } catch (JMLAssertionError e) { fcnt++; }
 	try { test.m4(false); fcnt++; } catch (JMLAssertionError e) {}

 	try { test.m5(true); } catch (JMLAssertionError e) { fcnt++; }
 	try { test.m5(false); fcnt++; } catch (JMLAssertionError e) {}

 	try { test.m6(true); } catch (JMLAssertionError e) { fcnt++; }
 	try { test.m6(false); fcnt++; } catch (JMLAssertionError e) {}

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(LoopInvariantAndVariant1.java)");
        }
    }
}

// @(#)$Id: LoopInvariantAndVariant1.java,v 1.1 2003/01/22 04:20:56 cheon Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
