/** @(#)$Id: NonNullMethodExp.java,v 1.1 2006/02/11 03:06:50 chalin Exp $
 * 
 * Test suites NonNullMethod* test the proper checking of (explicitly
 * or implicitly declared) non-null parameters and return types ofr
 * methods.
 *
 * @author Patrice Chalin (chalin@dsrg.org)
 */

import org.jmlspecs.jmlrac.runtime.*;

public class NonNullMethodExp extends org.jmlspecs.util.RacRunTestCase {

    public static void main(String[] args) {
	(new NonNullMethodExp()).run(args);
    }

    public NonNullMethodExp() {
	testee = new CNonNullMethodExp();
    }

    private final CNonNullMethodExp testee;
    private final Object notn = new Object();

    public void test_m1_ok()   { try { testee.m1(notn);         } catch (JMLAssertionError e   ) { fail(); } }
    public void test_m1_err()  { try { testee.m1(null); fail(); } catch (JMLPreconditionError e) {         } }

    public void test_m2_ok()   { try { testee.m2(10, notn);         } catch (JMLAssertionError e   ) { fail(); } }
    public void test_m2_err1() { try { testee.m2(-1, notn); fail(); } catch (JMLPreconditionError e) {} }
    public void test_m2_err2() { try { testee.m2(10, null); fail(); } catch (JMLPreconditionError e) {} }
    public void test_m2_err3() { try { testee.m2(-1, null); fail(); } catch (JMLPreconditionError e) {} }

    public void test_m3_ok()   { try { testee.m3(notn, 10);         } catch (JMLAssertionError e    ) { fail(); } }
    public void test_m3_err1() { try { testee.m3(notn, -1); fail(); } catch (JMLPostconditionError e) {} }
    public void test_m3_err2() { try { testee.m3(null, 10); fail(); } catch (JMLPreconditionError e ) {} }
    public void test_m3_err3() { try { testee.m3(null, -1); fail(); } catch (JMLPreconditionError e ) {} }

    public void test_m4_ok()   { try { testee.m4(notn, notn);         } catch (JMLAssertionError e) { fail(); } }
    public void test_m4_err1() { try { testee.m4(null, notn); fail(); } catch (JMLPreconditionError e) {} }
    public void test_m4_err2() { try { testee.m4(notn, null); fail(); } catch (JMLPreconditionError e) {} }
    public void test_m4_err3() { try { testee.m4(null, null); fail(); } catch (JMLPreconditionError e) {} }

    public void test_r1_ok()   { try { testee.r1(true);          } catch (JMLAssertionError e) { fail(); } }
    public void test_r1_err1() { try { testee.r1(false); fail(); } catch (JMLPostconditionError e) {} }

    public void test_r2_ok()   { try { testee.r2(10, true);          } catch (JMLAssertionError e   ) { fail(); } }
    public void test_r2_err1() { try { testee.r2(-1, true);  fail(); } catch (JMLPreconditionError e) {} }
    public void test_r2_err2() { try { testee.r2(10, false); fail(); } catch (JMLPostconditionError e) {} }
    public void test_r2_err3() { try { testee.r2(-1, false); fail(); } catch (JMLPreconditionError e) {} }

    public void test_r3_ok()   { try { testee.r3(true,  10);         } catch (JMLAssertionError e    ) {fail();}}
    public void test_r3_err1() { try { testee.r3(true,  -1); fail(); } catch (JMLPostconditionError e) {}}
    public void test_r3_err2() { try { testee.r3(false, 10); fail(); } catch (JMLPostconditionError e) {}}
    public void test_r3_err3() { try { testee.r3(false, -1); fail(); } catch (JMLPostconditionError e) {}}

    public void test_f1_ok()   { try { testee.f1(true);          } catch (JMLAssertionError e) { fail(); } }
    public void test_f1_err1() { try { testee.f1(false); fail(); } catch (JMLInvariantError e) {} }
}

class CNonNullMethodExp {
    /*@ spec_public @*/ private int i = 1;

    // Parameters ---------------------------------------------------------------

    // annotation w/o pre/post
    public void m1(/*@ non_null @*/ Object x) { }

    // annotation with precondition
    //@ requires x > 0;
    public void m2(int x, /*@ non_null @*/ Object y) { }

    // annotation with postecondition
    //@ ensures i > 0;
    public void m3(/*@ non_null @*/ Object y, int x) { i = x; }

    // multiple non_nulls
    public void m4(/*@ non_null @*/ Object x, /*@ non_null @*/ Object y) { }

    // Return types -------------------------------------------------------------

    // annotation w/o pre/post
    public /*@ non_null @*/ Object r1(boolean b) { return b ? new Object() : null; }

    // annotation with precondition
    //@ requires x > 0;
    public /*@ non_null @*/ Object  r2(int x, boolean b) { return b ? new Object() : null; }

    // annotation with postecondition
    //@ ensures i > 0;
    public /*@ non_null @*/ Object  r3(boolean b, int x) { i = x; return b ? new Object() : null; }

    // Fields ------------------------------------------------------------------
    // Only minimal testing for fields is being done here ...

    private /*@ non_null @*/ String s = "";

    public void f1(boolean b) { s = b ? "" : null; }
}

// Copyright (C) 2006 Iowa State University

// This file is part of JML.

// JML is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
