// There is a case when an object that is not compiled with 
// RAC appears in a place where only a RAC-compiled 
// object is expected. This violates some assumptions 
// made by the RAC implementation, and thus causes an 
// internal error. An example is given below. Thank to Roy 
// Tan for reporting this error.

import org.jmlspecs.jmlrac.runtime.*;

public class RACed implements Twin {

    // This method nherits postcondition from Twin and tries to evaluate
    // it. But \result is an object not RAC-compiled, thus violating
    // some assumption of RAC implementation.
    public Twin twin() {
        return new UnRACed();
    }

    public static void main(String [] argv) {
	RACed test = new RACed();
	int fcnt = 0;

        // no assertion violation, as \result is not RAC-compiled.
	try { test.twin(); }
        catch (JMLAssertionError e) { fcnt++; }
        
	if (fcnt > 0) {
	    System.out.println(fcnt + "F(RACed.java)");
        }
    }

    // This class not compiled with RAC; inner classes not supported yet.
    public static class UnRACed implements Twin {
        public Twin twin() { return null; }
    }
}

interface Twin {
    //@ ghost instance int age;

    //@ ensures \result.age > 0;
    Twin twin();
}
