// The surrogate class of an interface should properly delegate
// the implicitly inherited methods, i.e., those inherited from 
// the class Object. This one tests the equals method.

import org.jmlspecs.jmlrac.runtime.*;

public class Surrogate1 implements Surrogate1IF {

    public boolean equiv(Object other) {
        return other.equals(this);
    }

    public boolean equals(/*@ nullable @*/ Object other) {
        return other instanceof Surrogate1;
    }

    public static void main(String args[]) {
	Surrogate1 test = new Surrogate1();
	int fcnt = 0;

        // no assertion violation
	try { test.equiv(test); }
        catch (JMLAssertionError e) { fcnt++; }
        

        // no assertion violation
	try { test.equiv(new Surrogate1()); }
        catch (JMLAssertionError e) { fcnt++; }

        // postcondition violation
	try { test.equiv(new Object()); }
        catch (JMLAssertionError e) { fcnt++; }

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(Surrogate1.java)");
	}
    }
}

interface Surrogate1IF {
    
    //@ ensures \result == equals(other);
    boolean equiv(Object other);
}

// @(#)$Id: Surrogate1.java,v 1.2 2005/12/23 17:02:07 chalin Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
