// The surrogate class of an interface should properly delegate
// the implicitly inherited methods, i.e., those inherited from 
// the class Object. This one tests hashCode, getClass, and
// toString methods.

import org.jmlspecs.jmlrac.runtime.*;

public class Surrogate3 implements Surrogate3IF {

    public int myHashCode() {
        return hashCode();
    }

    public Class myGetClass() {
        return getClass();
    }

    public String myToString() {
        return toString();
    }

    public static void main(String args[]) {
	Surrogate3 test = new Surrogate3();
	int fcnt = 0;

        // no assertion violation
	try { test.myHashCode(); }
        catch (JMLAssertionError e) { fcnt++; }
        

        // no assertion violation
	try { test.myGetClass(); }
        catch (JMLAssertionError e) { fcnt++; }

        // postcondition violation
	try { test.myToString(); }
        catch (JMLAssertionError e) { fcnt++; }

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(Surrogate3.java)");
	}
    }
}

interface Surrogate3IF {
    
    //@ ensures \result ==getClass();
    Class myGetClass();

    //@ ensures \result == hashCode();
    int myHashCode();

    //@ ensures \result != null && \result.equals(toString());
    String myToString();

    /*@ pure @*/ int hashCode();
    /*@ pure @*/ String toString();
}

// @(#)$Id: Surrogate3.java,v 1.1 2003/01/22 04:20:56 cheon Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
