// Thanks to Peter Chan and Gary Leavens for this testcase, which
// originally produced a compile-time error about type casting.

import org.jmlspecs.jmlrac.runtime.*;

public class UnaryPromotion {
    static final Integer ZERO = new Integer(0);
    
    /*@ behavior
      @ ensures \result instanceof Comparable
      @ ==> ((Comparable)\result).compareTo(ZERO) == 0; @*/
    /*@ pure @*/ Number m1(boolean b) {
        if (b) 
            return ZERO;
        else 
            return new MyNumber();
    }
    
    //@ ensures ((Comparable)\result).compareTo(ZERO) == 0;
    Number m2(boolean b) {
        if (b) 
            return ZERO;
        else 
            return new MyNumber();
    }

    public static void main(String args[]) {
	UnaryPromotion test = new UnaryPromotion();
	int fcnt = 0;

        test.m2(true);

        // no assertion violation
	try { test.m1(true); }
        catch (JMLAssertionError e) { fcnt++; }
        
        // no assertion violation as the antecedent is false
	try { test.m1(false); }
        catch (JMLAssertionError e) { fcnt++; }

        // no assertion violation
	try { test.m2(true); }
        catch (JMLAssertionError e) { fcnt++; }
        
        // postcondition violation due to type-cast error
	try { test.m2(false); fcnt++; }
	catch (JMLAssertionError e) { if(!JMLRacUtil.checkErr(JMLPostconditionError.class, e)) fcnt += 100; }
    catch (Throwable t) { fcnt++; }

	if (fcnt > 0) {
	    System.out.println(fcnt + "F(UnaryPromotion.java)");
        }
    }
}

class MyNumber extends Number {
    public double doubleValue() { return 3.0; }
    public float floatValue() { return 3.0f; }
    public int intValue() { return 3; }
    public long longValue() { return 3L; }
}

// @(#)$Id: UnaryPromotion.java,v 1.3 2006/07/03 22:14:36 f_rioux Exp $
//
// Copyright (C) 2002 Iowa State University
//
// This file is part of JML
//
// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
