package org.jmlspecs.jmlspec;

import java.io.*;
import org.multijava.mjc.*;

public class JspBinaryPrinter {

    protected CClass cclass;
    protected long privacy;
    protected long nprivacy;

    public boolean ok(long mods) {
	if (privacy == 3) return true;
	if (privacy == 2) return (mods & Constants.ACC_PRIVATE) == 0;
	return (nprivacy & mods)!= 0 && (Constants.ACC_PRIVATE & mods) == 0;
    }

    public JspBinaryPrinter(CClass cclass, long privacy) {
	this.cclass = cclass;
	this.privacy = privacy;
	this.nprivacy =
	    privacy == 0 ? Constants.ACC_PUBLIC :
	    privacy == 1 ? Constants.ACC_PUBLIC | Constants.ACC_PROTECTED :
	    privacy == 2 ? Constants.ACC_PUBLIC | Constants.ACC_PROTECTED :
			    Constants.ACC_PUBLIC | Constants.ACC_PROTECTED | Constants.ACC_PRIVATE
	 ;
    }

    public void print(String dir, String suffix) {
	try {

	File out = new File(dir, cclass.qualifiedName().replace('/',File.separatorChar) + suffix);
	if (out.getParentFile() != null) out.getParentFile().mkdirs();
	//System.out.println("WRITING " + out.getAbsolutePath());
	PrintWriter io = new PrintWriter(new FileWriter(out));
        io.println("// Generated by jmlspec from the .class file");
	io.println();

	String p = cclass.packageName().replace('/','.');
	if (p.length() > 0) {
	    p = p.substring(0,p.length()-1);
	    io.println("package " + p + ";");
	}
	io.println();

	print(cclass,io,"");

	io.close();
	} catch (Exception e) { 
		System.out.println("Failed - " + e);
		e.printStackTrace(System.out);
	} // FIXME - report error
    }

    public void print(CClass cclass, PrintWriter io, String otab) {
	String tab = otab + "    ";
	CModifier mod = new CModifier();
	long noprint = ~0;

	long mods = cclass.modifiers();
	mods &= ~Constants.ACC_SYNCHRONIZED;
	if ( (mods&Constants.ACC_PROTECTED)!= 0) 
				mods &= ~Constants.ACC_PUBLIC;

	if (cclass.isInterface()) {
	    noprint = ~Constants.ACC_ABSTRACT & ~Constants.ACC_STATIC;
	    io.print(otab + mod.asString(noprint&mods));
	    io.print("interface " + cclass.ident());
	    CClassType[] interfaces = cclass.getInterfaces();
	    if (interfaces != null && interfaces.length > 0) {
		io.print(" extends ");
		for (int i=0; i<interfaces.length; ++i) {
		    CClassType t = interfaces[i];
		    if (i != 0) io.print(", ");
		    io.print(t.qualifiedName().replace('/','.').replace('$','.'));
		}
	    }
	} else {
	    io.print(otab + mod.asString(noprint&mods));
	    io.print("class " + cclass.ident());
	    io.print(" extends " + cclass.getSuperClass().qualifiedName().replace('/','.').replace('$','.'));
	    CClassType[] interfaces = cclass.getInterfaces();
	    if (interfaces != null && interfaces.length > 0) {
		io.print(" implements ");
		for (int i=0; i<interfaces.length; ++i) {
		    CClassType t = interfaces[i];
		    if (i != 0) io.print(", ");
		    io.print(t.qualifiedName().replace('/','.').replace('$','.'));
		}
	    }
	}
	io.println(" {");

	io.println();
	io.println(tab + "// CLASS SPECIFICATIONS");
	for (int j=0; j<Main.classAnnotation.length; ++j) {
	    io.println(tab + Main.classAnnotation[j]);
	}

	// Fields
	io.println();
	io.println(tab + "// FIELDS");

	java.util.Collection fields = cclass.fields();
	java.util.Iterator iter = fields.iterator();
	while (iter.hasNext()) {
	    CField field = (CField)iter.next();
	    if (field.ident().indexOf('$')!=-1) continue;
	    long m = field.modifiers();
	    if (ok(m)) {
	        // AHS: JUTS field annotation
	        if (CTopLevel.getCompiler().universeBytecode()
                    && field.getType().isClassType()) {
	            CClassType cct = (CClassType) field.getType();
	            io.println(tab + mod.asString(m&noprint) +
	                       "/*+@ " + cct.getCUniverseJMLString() + " @+*/ " +
	                       field.getType() + " " + field.ident() + ";");
	        } else {
	            io.println(tab + mod.asString(m&noprint) + field.getType() + " " + field.ident() + ";");
	        }
	    }
	}

	// Methods
	io.println();
	io.println(tab + "// METHODS AND CONSTRUCTORS");

	CMethodSet methods = cclass.methods();
	CMethodSet.Iterator itera = methods.iterator();
	while (itera.hasNext()) {
	    CMethod method = itera.next();
	    long m = method.modifiers();
	    if (!ok(m)) continue;
	    if (method.isSynthetic()) continue;

	    io.println();
	    for (int j=0; j<Main.annotation.length; ++j) {
		io.println(tab + Main.annotation[j]);
	    }
	    if (method.ident().equals("<clinit>")) {
		io.println(tab + "//@ static_initializer");
		continue;
	    }
	    
	    // AHS: purity is included in modifiers (when run with --universes)
	    // Mask it out and display it in comments
	    if (CTopLevel.getCompiler().universeBytecode() && method.isPure()) {
	        io.print(tab + mod.asString(m & noprint & ~Constants.ACC_PURE));
	        io.print(" /*+@ pure @+*/ ");
	    } else {
	        io.print(tab + mod.asString(m&noprint));
	    }
	    
	    if (method.isConstructor()) {
	        io.print(cclass.ident() + "(");
	    } else {
	        // AHS: JUTS return type annotation
	        if (CTopLevel.getCompiler().universeBytecode() && method.returnType().isClassType()) {
	            io.print("/*+@ " + ((CClassType) method.returnType()).getCUniverseJMLString() 
	                     + " @+*/ " + method.returnType() + " " + method.ident() + "(");
	        } else {
	            io.print(method.returnType() + " " + method.ident() + "(");
	        }
	    }

	    CSpecializedType[] params = method.parameters();
	    for (int i=0; i<params.length; ++i) {
		if (i != 0) io.print(", ");
		
		// AHS: JUTS parameter annotations
		if (CTopLevel.getCompiler().universeBytecode() && params[i].staticType().isClassType()) {
		    io.print("/*+@ " +
		             ((CClassType) params[i].staticType()).getCUniverseJMLString()
		             + " @+*/ " + params[i]);
		} else {
		    io.print(params[i]);
		}
		io.print(" Param" + i);
	    }

	    io.print(")");
	
	    CClassType[] exs = method.throwables();
	    if (exs.length != 0) {
		io.println(); 
		io.print(tab + tab + "throws ");
	    }
	    for (int i=0; i<exs.length; ++i) {
		CClassType t = exs[i];
		if (i != 0) io.print(", ");
		io.print(t);
	    }
	    io.println(";");
	}

	// Nested Types
	CClassType[] inners = cclass.getInnerClasses();
	if (inners != null && inners.length > 0) {
	    io.println();
	    io.println(tab + "// NESTED TYPES");
	    for (int i=0; i<inners.length; ++i) {
		CClass t = inners[i].getCClass();
		long m = t.modifiers();
		// Nested protected classes are also labelled public
		if ( (m&Constants.ACC_PROTECTED)!= 0) 
					m &= ~Constants.ACC_PUBLIC;
		if (!ok(m)) continue;
		print(t, io, tab);

	    }

	}

	io.println(otab + "}");
    }



}
