/* $Id: JspPrettyPrinter.java,v 1.23 2006/11/29 17:51:04 chalin Exp $
 *
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of JML
 *
 * JML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * JML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jmlspecs.jmlspec;
import org.multijava.mjc.*;
import org.multijava.util.compiler.ModifierUtility;
import java.io.File;
import java.util.ArrayList;
import java.util.ArrayList;

/** This class overrides some of the pretty-printer methods to produce
    a specification skeleton for the class AST begin walked.
*/
public class JspPrettyPrinter extends MjcPrettyPrinter {

    /** Alias to the command-line options structure. */
    protected JspOptions options;

    protected String originalName;
    
    protected String ident;
    
    // These two fields are used to pass information from one method to another
    // in this class.  That is somewhat dangerous - and is an instance of using
    // global data - but there does not seem to be another mechanism within the
    // current API.
    boolean inInterface = false;
    boolean interfaceIsPublic = false;

    /**
     * construct a pretty printer object for java code
     * @param	file	the file into which the code is generated
     */
    public JspPrettyPrinter( File file, ModifierUtility modUtil, JspOptions options, String originalName ) {
	super(file, modUtil);
        TAB_SIZE = 4;
	this.options = options;
	this.originalName = originalName;
	
    }

    /** Returns true if the given member modifier value makes the class member 
	visible with the current options.specLevel() setting.
    */
    public boolean isVisible(long modifiers) {
	int level = options.specLevel();
	if (level == 0) return (modifiers & ACC_PUBLIC) != 0;
	if (level == 3) return true;
	if (level == 1) return (modifiers & (ACC_PUBLIC|ACC_PROTECTED)) != 0;
	// level == 2
	return (modifiers & ACC_PRIVATE) == 0;
    }

    public void visitCompilationUnit( JCompilationUnit self ) {
	JPackageName packageName = self.packageName();
	JPackageImportType[] importedPackages = self.importedPackages();
	JClassOrGFImportType[] importedUnits = self.importedUnits();
	JTypeDeclarationType[] typeDeclarations = self.typeDeclarations();
	ArrayList tlMethods = self.tlMethods();

	if (packageName.getName().length() > 0) {
	    packageName.accept(this);
	    newLine();
	}
	
	if (typeDeclarations.length > 0) {
	    print("//@ refine \"" + originalName + "\";");
	    newLine();
	}
	newLine();

	for (int i = 0; i < importedPackages.length ; i++) {
	    if (!importedPackages[i].getName().equals("java/lang")) {
		importedPackages[i].accept(this);
		newLine();
	    }
	}

	for (int i = 0; i < importedUnits.length ; i++) {
	    if (!(importedUnits[i].isClassImport())) continue; 
	    importedUnits[i].accept(this);
	    newLine();
	}

	for (int i = 0; i < importedUnits.length ; i++) {
	    if (!(importedUnits[i].isGFImport())) continue; 
	    importedUnits[i].accept(this);
	    newLine();
	}
	
	for (int i = 0; i < typeDeclarations.length ; i++) {
	    newLine();
	    typeDeclarations[i].accept(this);
	    newLine();
	}
	
	if (tlMethods.size() > 0) {
	    newLine(); print("// TOP LEVEL METHODS"); newLine();
	}
	acceptAll( tlMethods );

    }
    public void visitClassDeclaration( JClassDeclaration self ) {
	// !FIXME!comment visit Comments
	long modifiers = self.modifiers();
	if (!isVisible(modifiers) || (options.deprecation() && self.isDeprecated())) return;
	printDeprecatedIfAppropriate(self);
	this.ident = self.ident();
	super.visitClassDeclaration(self);
    }
    
    public void visitInterfaceDeclaration( JInterfaceDeclaration self ) {
	// !FIXME!comment visit Comments
	long modifiers = self.modifiers();
	if (!isVisible(modifiers) || (options.deprecation() && self.isDeprecated())) return;
	printDeprecatedIfAppropriate(self);
	interfaceIsPublic = ((modifiers & Constants.ACC_PUBLIC)!= 0);
	super.visitInterfaceDeclaration(self);
	newLine(); 
    }

    // We omit abstract from interface declarations.
    protected void printInterfaceModifiers(JInterfaceDeclaration self, long modifiers) {
	print(modUtil.asString(modifiers & ~(Constants.ACC_ABSTRACT)));
    }
    
    // We omit strictfp from class declarations.
    protected void printClassModifiers(JClassDeclaration self, long modifiers) {
	print(modUtil.asString(modifiers & ~(Constants.ACC_STRICT)));
    }
    
    protected void visitClassBody( ArrayList inners,
				 ArrayList methods,
				 JPhylum[] fieldsAndInits ) {
	inInterface = false;
	
	print("{");
	newLine(); 
	pos += TAB_SIZE;

	newLine(); print("// CLASS SPECIFICATIONS"); newLine();
	printClassAnnotationStart();

	newLine(); print("// METHODS AND CONSTRUCTORS"); newLine();
	doDefaultConstructor(methods);
	acceptAll( methods );
	

	if (inners.size() > 0) {
	    newLine(); print("// INNER CLASSES"); newLine();
	}
	acceptAll( inners );

	inInterface = false;

	if (fieldsAndInits.length > 0) {
	    newLine(); print("// FIELDS"); newLine();
	}
	for (int i = 0; i < fieldsAndInits.length ; i++) {
	    if (fieldsAndInits[i] instanceof JFieldDeclarationType) {
	    	if (!isVisible(((JFieldDeclarationType)fieldsAndInits[i]).modifiers())) continue;
		fieldsAndInits[i].accept(this);
	    }
	}

	pos -= TAB_SIZE;
	newLine();
	print("}");
	newLine();
    }

    /** Checks if there are any constructors; if not adds a default constructor. */
    protected void doDefaultConstructor(ArrayList methods) {
	int s = methods.size() - 1;
	for (; s >= 0; --s) {
	    if (methods.get(s) instanceof JConstructorDeclarationType) return;
	}

	print("// Default Constructor");
        printAnnotationStart();
	print("public " + ident + "();"); newLine();	
	newLine();
    }

    protected void visitInterfaceBody(ArrayList methods,
				 JPhylum[] fieldsAndInits, ArrayList inners ) {
	print(" {");
	pos += TAB_SIZE;

	newLine(); print("// INTERFACE SPECIFICATIONS");
        printClassAnnotationStart();

	inInterface = true;

	if (fieldsAndInits.length > 0) {
	    newLine(); print("// FIELDS"); newLine();
	}
	for (int i = 0; i < fieldsAndInits.length ; i++) {
	    if (fieldsAndInits[i] instanceof JFieldDeclarationType) {
	    	if (!isVisible(((JFieldDeclarationType)fieldsAndInits[i]).modifiers())) continue;
		fieldsAndInits[i].accept(this);
		newLine();
	    }
	}

	if (methods.size() > 0) {
	    newLine(); print("// METHODS"); newLine();
	}
	acceptAll( methods );

	inInterface = false;

	if (inners.size() > 0) {
	    newLine(); print("// INNER CLASSES"); newLine();
	}
	acceptAll( inners );

	pos -= TAB_SIZE;
	newLine();
	print("}");
    }

    /** Print an annotations start. */
    private void printAnnotationStart() {
	newLine();
	for (int i = 0; i<Main.annotation.length; ++i) {
	    print(Main.annotation[i]);
	    newLine();
        }
    }

    /** Print an annotations start. */
    private void printClassAnnotationStart() {
	newLine();
	for (int i = 0; i<Main.classAnnotation.length; ++i) {
	    print(Main.classAnnotation[i]);
	    newLine();
        }
    }
    
    final static long modifiersToOmit = ACC_NATIVE | ACC_STRICT;

    /** Adds an empty specification comment and does not print the body. */
    public void visitMethodDeclaration( JMethodDeclaration self ) {
	long modifiers = self.modifiers();
	if ((!inInterface && !isVisible(modifiers))  || (options.deprecation() && self.isDeprecated())) return;
	CType returnType = self.returnType();
	String ident = self.ident();
	JFormalParameter[] parameters = self.parameters();
	CClassType[] exceptions = self.getExceptions();
	JBlock body = self.body();

	printDeprecatedIfAppropriate(self);
        printAnnotationStart();
	long omitMod = modifiersToOmit | ACC_STRICT;
	if (inInterface && interfaceIsPublic) omitMod |= ACC_PUBLIC;
	if (inInterface) omitMod |= ACC_ABSTRACT;
	print(modUtil.asString(modifiers & ~omitMod));
	if(options.allNonNull() && returnType.isReference()) {
	    // modifiers |= ACC_NON_NULL;
	    print(non_null_decl_mod);
	    print(" ");
	}
	print(returnType);
	print(" ");
	print(ident);
	print("(");
	int count = 0;

	for (int i = 0; i < parameters.length; i++) {
	    if (count != 0) {
		print(", ");
	    }
	    JFormalParameter p = parameters[i];
	    if (!p.isGenerated()) {
		if(options.allNonNull() && p.getType().isReference()) {
		    if("equals".equals(ident) || (p.modifiers() & ACC_NULLABLE) != 0) {
			// p.setModifiers(p.modifiers());
			print("/*@nullable*/");
		    } else {
			p.setNonNull();
		    }
		}
		p.accept(this);
		count++;
	    }
	}
	print(")");

	for (int i = 0; i < exceptions.length; i++) {
	    if (i != 0) {
		print(", ");
	    } else {
		print(" throws ");
	    }
	    print(exceptions[i].toString());
	}
   
	print(";");
	newLine();
    }
    
    /** Adds an empty specification comment and does not print the body. */
    public void visitConstructorDeclaration( JConstructorDeclaration self ) {
	long modifiers = self.modifiers();
	if (!isVisible(modifiers)) return;
	String ident = self.ident();
	JFormalParameter[] parameters = self.parameters();
	CClassType[] exceptions = self.getExceptions();
	JBlock body = self.body();

	printDeprecatedIfAppropriate(self);
        printAnnotationStart();
	print(modUtil.asString(modifiers & ~ACC_STRICT));
	print(ident);
	print("(");
	int count = 0;
	for (int i = 0; i < parameters.length; i++) {
	    if (count != 0) {
		print(", ");
	    }
	    JFormalParameter p = parameters[i];
	    if (!p.isGenerated()) {
		if(options.allNonNull() 
		   && !"equals".equals(ident) 
		   && p.getType().isReference()) {
		    p.setNonNull();
		}
		p.accept(this);
		count++;
	    }
	}
	print(")");

	for (int i = 0; i < exceptions.length; i++) {
	    if (i != 0) {
		print(", ");
	    } else {
		print(" throws ");
	    }
	    print(exceptions[i].toString());
	}
	print(";");
	newLine();
    }

    /**
     * prints an external method declaration
     */
    public void 
	visitTopLevelMethodDeclaration( MJTopLevelMethodDeclaration self ) 
    {
	long modifiers = self.modifiers();
	if (!isVisible(modifiers) || (options.deprecation() && self.isDeprecated())) return;
	printDeprecatedIfAppropriate(self);
        printAnnotationStart();
	super.visitTopLevelMethodDeclaration(self);
    }
    
    public void visitTLMethodBody(JBlock body) {
	print(";");
    }

    /**
     * prints a field declaration without any initializer
     */
    public void visitFieldDeclaration( JFieldDeclaration self ) {
	JVariableDefinition variable = self.variable();
	long modifiers = variable.modifiers();
	CType type = variable.getType();
	String ident = variable.ident();
	JExpression expr = variable.getValue();

	if (ident.indexOf("$") != -1) {
	    return; // dont print generated elements
	}

	printDeprecatedIfAppropriate(self);
	newLine();
	if (!options.keepPublic() 
	    && inInterface && interfaceIsPublic) {
	    modifiers = modifiers & ~ACC_PUBLIC;
	}
	print(modUtil.asString(modifiers));
	if(options.allNonNull() && type.isReference()) {
	    print(non_null_decl_mod);
	    print(" ");
	}
	print(type);
	print(" ");
	print(ident);
	print(";");
	if (expr != null && !options.noInitially()) {
	    print(" /*@ initially " + ident + " == ");
	    expr.accept(this);
	    print(";");
	    print(" @*/");
	}
    }

    /**
     * prints a formal parameter, omitting "final" modifier
     */
    public void visitFormalParameters( JFormalParameter self ) {
	// Omits modifiers (final).  
	if(options.allNonNull() && self.isDeclaredNonNull()) {
	    print(non_null_decl_mod);
	    print(" ");
	}
	print(self.typeToString());
	print(self.paramToString());
    }

    public void printDeprecatedIfAppropriate(JMemberDeclaration self) {
    	if (self.isDeprecated()) { newLine(); print("/** @deprecated */"); }
    }

    private final String non_null_decl_mod = "/*@non_null*//*?*/";
}
