/*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestsWithOutputTestSuite.java,v 1.2 2003/03/08 17:06:28 davidcok Exp $
 */

package org.jmlspecs.jmlspec.testcase;

import org.multijava.util.testing.TestCase;
import org.multijava.util.testing.*;
import junit.framework.*;
import java.io.*;
import java.util.Iterator;
import java.util.ArrayList;

/** This class runs the test cases that have a different set of command-line
    options for each test.  Thus each one needs its own execution of compile().
    For each one, the options are determined (they are specified in the test file
    by a line containing JMLFLAGS), the compile is run and the output is
    compared to the expected output.  There is a TestCase for each compile and a
    separate TestCase for each file comparison.
<P>
    The set of external test files is obtained by running the command
    'listTestsWithNonDefaultOptions'.
<P>
    The test must be run from the directory in which it resides - because
    it runs an external process that is expected to be in the current
    directory and opens/creates files in the current directory.

    @author David R. Cok
*/
public class TestsWithOutputTestSuite  extends NonDefaultOptionsTestSuite {

    public static Test suite() {
        return (new TestsWithOutputTestSuite("TestsWithOutputTestSuite"));
    }

    /** Standard constructor used by the JUnit framework. 
	It fills the TestSuite with all the tests.
    */
    public TestsWithOutputTestSuite(String name) { 
	super(0); // Create the superclass empty
	
	try {
	
	    Iterator i = new FileIterator("listTestsWithOutput");
	    while (i.hasNext()) {
		String s = (String)i.next();
		addTest(new Helper(s)); // the TestCase for the compile
	    }
	
	} catch (java.io.IOException e) {
	    throw new RuntimeException(e.toString());
	}
     }
     
     protected TestsWithOutputTestSuite(int i) {
	super(0);
     }
    
}


