/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
package org.jmlspecs.launcher;

import org.multijava.launcher.AbstractLaunchTool;

/**
 *  A launcher for the JML tools
 *
 *@author     Kristina Boysen
 *@version    $Revision: 1.8 $
 */

public class JmlLauncher extends org.multijava.launcher.Launcher {
        
    // ----------------------------------------------------------------------
    // MAIN METHOD
    // ----------------------------------------------------------------------
        
    /**
     *  Main method for running the JmlLauncher
     */
    public static void main(String[] args) {
        JmlLauncher launcher = new JmlLauncher("JML Launcher", args);
    }
        
    // ----------------------------------------------------------------------
    // CONSTRUCTOR
    // ----------------------------------------------------------------------
        
    /**
     *  Constructor for the JmlLauncher
     */
    public JmlLauncher(String name, String[] args) {
        this.args = args;
                
        getImpLauncher(name, LOGO_NAME);
    }
        
    // ----------------------------------------------------------------------
    // ACCESS METHODS
    // ----------------------------------------------------------------------
    
    /**
     *  Returns the iterator that contains the tool description and the
     *  AbstractLaunchTool that can access the tool
     */
    protected ToolIterator getTools() {
        if (iterator == null) {
            iterator = createToolIterator();
        }
        
        return iterator;
    }
        
    // ----------------------------------------------------------------------
    // BUILDER METHODS
    // ----------------------------------------------------------------------
        
    /**
     *  Builds the iterator that contains the tool description and the
     *  AbstractLaunchTool that can access the tool
     */
    private ToolIterator createToolIterator() {
        ToolIterator iterator = new ToolIterator();
        
        ToolIteratorPair pair1 = new ToolIteratorPair(
            "JML Type Checker (jml)",
            new AbstractLaunchTool(this) {
                public void runTool() {
                    org.jmlspecs.checker.JmlGUI.init(args, false);
                }
            });
        iterator.add(pair1);
                
        ToolIteratorPair pair2 = new ToolIteratorPair(
            "JML Runtime Assertion Checker Compiler (jmlc)",
            new AbstractLaunchTool(this) {
                public void runTool() {
                    org.jmlspecs.jmlrac.RacGUI.init(args, false);
                }
            });
        iterator.add(pair2);
                                  
        ToolIteratorPair pair3 = new ToolIteratorPair(
            "JML Documentation (jmldoc)",
            new AbstractLaunchTool(this) {
                public void runTool() {
                    org.jmlspecs.jmldoc.JmldocGUI.init(args, false);
                }
            });
        iterator.add(pair3);
        
        ToolIteratorPair pair4 = new ToolIteratorPair(
            "JML Skeleton Spec Generator (jmlspec)",
            new AbstractLaunchTool(this) {
                public void runTool() {
                    org.jmlspecs.jmlspec.JspGUI.init(args, false);
                }
            });
        iterator.add(pair4);
        
        ToolIteratorPair pair5 = new ToolIteratorPair(
            "JML/JUnit Test Oracle Generator (jmlunit)",
            new AbstractLaunchTool(this) {
                public void runTool() {
                    org.jmlspecs.jmlunit.JntGUI.init(args, false);
                }
            });
        iterator.add(pair5);
                
        return iterator;
    }
        
    // ----------------------------------------------------------------------
    // PRIVATE CLASS VARIABLES
    // ----------------------------------------------------------------------
    
    /**
     *  The arguments to be used by these tools
     */
    private String[] args;
        
    /**
     *  The iterator that cycles through all of the tool names and launchers
     *  for the JML tools
     */
    private ToolIterator iterator = null;
        
    /**
     *  The path to the logo
     */
    private String LOGO_NAME = "org/jmlspecs/launcher/logo/JmlLogo.gif";
}
