/* 
 * Copyright (C) 2003 Virginia Tech
 *
 * This file is part of JML
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id $
 */

package org.jmlspecs.racwrap;

import org.jmlspecs.jmlrac.*;
import org.jmlspecs.checker.*;
import org.multijava.mjc.*;
import org.multijava.util.compiler.ModifierUtility;

import java.util.*;

class OrigPrettyPrinter extends RacPrettyPrinter {

    public OrigPrettyPrinter(java.io.Writer wr, JmlModifier modUtil ) {
        super(wr, modUtil);
    }

    //FIXME: this is a kludge... classScopeLevel indicates the scoping
    //level of a class, this is used so that we can determine whether
    //a constructor is at the top level or not.
    private int classScopeLevel = 0;

    /**
     *  Same as <code>MjcPrettyPrinter</code>'s 
     *  <code>visitCompilationUnit</code> method, except it adds the 
     *  imported runner package.
     */
    //FIXME: instead of copying the whole file separate out the printing
    //of imports into a method, and just override that.
    public void visitCompilationUnit( JCompilationUnit self ) {
        JPackageName packageName = self.packageName();
        JPackageImportType[] importedPackages = self.importedPackages();
        JClassOrGFImportType[] importedClasses = self.importedClasses();
        JClassOrGFImportType[] importedGFs = self.importedGFs();
        JTypeDeclarationType[] typeDeclarations = self.typeDeclarations();
        ArrayList tlMethods = self.tlMethods();

        if (packageName.getName().length() > 0) {
            packageName.accept(this);
            if (importedPackages.length + importedClasses.length > 0) {
                newLine();
            }
        }
        
        for (int i = 0; i < importedPackages.length ; i++) {
            if (!importedPackages[i].getName().equals("java/lang")) {
                importedPackages[i].accept(this);
                newLine();
            }
        }

        print("import org.jmlspecs.racwrap.runner.*;");
        newLine();

        for (int i = 0; i < importedClasses.length ; i++) {
            importedClasses[i].accept(this);
            newLine();
        }

        for (int i = 0; i < importedGFs.length ; i++) {
            importedGFs[i].accept(this);
            newLine();
        }
/*
        for (int i = 0; i < typeDeclarations.length ; i++) {
            newLine();
            typeDeclarations[i].accept(this);
            newLine();
        }
*/        
        acceptAll( tlMethods );
    }

    /**
     *  Prints the modified class declaration of the orig file.
     *  This method prints $chx_Orig_Foo, instead of Foo as the class name.
     *  It will also extend $chx_Orig_Bar, instead of just Bar, except
     *  when the parent is Object, in which case it will extend
     *  org.jmlspecs.racwrap.runner.Wrapped instead.
     */
    public void visitJmlClassDeclaration( JmlClassDeclaration self ) {
        //System.out.println("Visiting class declaration");
        classScopeLevel++;
        // !FIXME!comment visit Comments
        long modifiers = self.modifiers();
        String ident = self.ident();
        String superName = self.superName();
        CClassType[] interfaces = self.interfaces();
        JPhylum[] fieldsAndInits = self.fieldsAndInits();
        ArrayList methods = self.methods();
        ArrayList inners = self.inners();

        newLine();
        print(modUtil.asString(modifiers));
        print("class ");
        if(classScopeLevel == 1) {
            print("$chx_Orig_");
        }
        print(ident);

        if(classScopeLevel != 1) {
            if (superName != null && !superName.equals("Object")
                && !superName.equals("java.lang.Object")) {
                print(" extends " + superName.replace('/', '.'));
            }
        } else {

            //extend the orig superclass instead of the original superclass
            if( superName == null) {
                superName = "org.jmlspecs.racwrap.runner.Wrapped";
            } else {
                superName = superName.replace('/', '.');
                if( superName.equals("java.lang.Object") 
                        || superName.equals("Object")) {
                    superName = "org.jmlspecs.racwrap.runner.Wrapped";
                } else {
                    int lindex = superName.lastIndexOf(".");
                    if(lindex == -1) {
                        superName = "$chx_Orig_" + superName;
                    } else {
                        String className = superName.substring(lindex + 1);
                        String packageName = superName.substring(0, lindex);
                        superName = packageName + "$chx_Orig_" + className;
                    }
                }
            }
            print(" extends " + superName);
        }
        newLine();


        if (interfaces.length != 0) {
            print(" implements ");
            for (int i = 0; i < interfaces.length; i++) {
                print((i == 0 ? "" : ", ") + interfaces[i]);
            }
        }

        if(classScopeLevel == 1) {
            print((interfaces.length != 0) ? "," : " implements ");
            print(ident);
        }

        print(" ");
        visitClassBody(inners, methods, fieldsAndInits);
        classScopeLevel--;
    }

    public void visitJmlConstructorDeclaration(
        JmlConstructorDeclaration self ) {
        long modifiers = self.modifiers();
        String ident = self.ident();
        JFormalParameter[] parameters = self.parameters();
        CClassType[] exceptions = self.getExceptions();
        JBlock body = self.body();

        newLine();
        print(modUtil.asString(modifiers));
        if(classScopeLevel == 1) 
            print("$chx_Orig_");
        print(ident);
        print("(");
        int count = 0;
        for (int i = 0; i < parameters.length; i++) {
            if (count != 0) {
                print(", ");
            }
            if (!parameters[i].isGenerated()) {
                parameters[i].accept(this);
                count++;
            }
        }
        print(")");

        for (int i = 0; i < exceptions.length; i++) {
            if (i != 0) {
                print(", ");
            } else {
                print(" throws ");
            }
            print(exceptions[i].toString());
        }
        print(" ");
        body.accept(this);
        newLine();

    }

    public void visitJmlMethodDeclaration( JmlMethodDeclaration self ) {

        if(self.isModel()) {
            return;
        }
        long modifiers = self.modifiers();

        modifiers = self.stripPrivateModifier(modifiers);
        modifiers = modifiers & (~self.ACC_PROTECTED);
        modifiers = modifiers | self.ACC_PUBLIC;

        CType returnType = self.returnType();
        String ident = self.ident();
        JFormalParameter[] parameters = self.parameters();
        CClassType[] exceptions = self.getExceptions();
        JBlock body = self.body();

        newLine();
        newLine();
        print(modUtil.asString(modifiers));
        print(returnType);
        print(" ");
        print(ident);
        print("(");
        int count = 0;

        for (int i = 0; i < parameters.length; i++) {
            if (count != 0) {
                print(", ");
            }

            if (!parameters[i].isGenerated()) {
                parameters[i].accept(this);
                count++;
            }
        }
        print(")");

        for (int i = 0; i < exceptions.length; i++) {
            if (i != 0) {
                print(", ");
            } else {
                print(" throws ");
            }
            print(exceptions[i].toString());
        }
   
        print(" ");
        if (body != null) {
            body.accept(this);
        } else {
            print(";");
        }
        newLine();
    }

    public void visitJmlFieldDeclaration( JmlFieldDeclaration self ) {

        if(self.hasFlag(self.modifiers(), self.ACC_MODEL)) {
            return;
        }
        super.visitJmlFieldDeclaration(self);
        
        if( self.getField().isStatic() ) {
            return;
        }

        newLine();
        print("public " + self.getType().toString() + " ");
        print("_chx_get_" + self.ident() + "() {");
        pos += TAB_SIZE;
        newLine();
        print("return " + self.ident() + ";");
        newLine();
        pos -= TAB_SIZE;
        print("}");
        newLine();

        newLine();
        print("public void _chx_set_" + self.ident());
        print("( " + self.getType().toString() + " obj) {");
        pos += TAB_SIZE;
        newLine();
        print(self.ident() + " = obj;");
        newLine();
        pos -= TAB_SIZE;
        print("}");
        newLine();
    }

}
