/* 
 * Copyright (C) 2003 Virginia Tech
 *
 * This file is part of JML
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: WrapperPrettyPrinter.java,v 1.7 2005/04/26 02:40:33 kui_dai Exp $
 */

package org.jmlspecs.racwrap;

import org.jmlspecs.jmlrac.*;
import org.jmlspecs.checker.*;
import org.multijava.mjc.*;
import org.multijava.util.compiler.*;

import java.io.*;

/**
 * WrapperPrettyPrinter prints the wrapper.
 */
public class WrapperPrettyPrinter extends RacPrettyPrinter {

    /** We need information about the wrapped class name. */
    private String wrappedClassName = null;

    public WrapperPrettyPrinter(File file,  JmlModifier modUtil ) {
        super(file, modUtil);
    }

    public WrapperPrettyPrinter(String fileName, JmlModifier modUtil) {
        super(fileName, modUtil);
    }

    public WrapperPrettyPrinter(TabbedPrintWriter writer) {
        super(writer);
    }

    public WrapperPrettyPrinter(java.io.Writer writer, JmlModifier modUtil) {
        super(writer, modUtil);
    }

    public void visitThisExpression( JThisExpression self ) {
        JExpression prefix = self.prefix();
        if (prefix != null) {
            prefix.accept(this);
            print(".this");
        } else {
            newLine();
//            print( "----DEBUG-----");
//            print(self.toString());
//            newLine();
            print("((" + wrappedClassName + ") _wrapped_object)");
//            print("this");
        }
    }

    public void visitJmlFieldDeclaration(JmlFieldDeclaration self) {
        JVariableDefinition variable = self.variable();
        long modifiers = variable.modifiers();
        CType type = variable.getType();
        String ident = variable.ident();

//        if( (self.hasFlag(self.modifiers(), ACC_PUBLIC)
//             || self.hasFlag(self.modifiers(), ACC_PROTECTED)
//            ) && ! self.hasFlag(self.modifiers(), ACC_MODEL)
          if(! self.hasFlag(self.modifiers(), ACC_MODEL)
          ) {
            //print getter
            newLine();
            print("//generated getter method");
            newLine();
            print("public " + type + " _chx_get_" + ident + "() {");
            newLine();
            print("    return ((" + wrappedClassName);
            print(") _wrapped_object)._chx_get_" + ident + "();");
            newLine();
            print("}");
            //print setter
            newLine();
            print("//generated setter method");
            newLine();
            print("public void _chx_set_" + ident + "(" + type + " obj) {"); 
            newLine();
            print("    ((" + wrappedClassName + ") _wrapped_object)._chx_set_");
            print(ident + "(obj);");
            newLine();
            print("}");
        } //end of print getters and setters for public and protected fields

        // --- COPIED FROM RacPrettyPrinter
        // print assertion check code, if any, e.g., private variable
        // declarations for ghost variables and their initializers
        // (see TransClass.ghostFieldAccessors).
        if (self.hasAssertionCode()) {
            self.assertionCode().accept(this);
        }


    }

    public int classScopeLevel = 0;
    public JmlClassDeclaration current_class = null;
    /**
     * Prints the wrapper declaration
     */
    public void visitJmlClassDeclaration(JmlClassDeclaration self) {
        JmlClassDeclaration old_class = current_class;
        current_class = self; 
        classScopeLevel ++;
        if(classScopeLevel > 1) {
            //System.out.println("SKIPPING::::: " + self.ident());
            classScopeLevel --;
            current_class = old_class;
            return;
        }
        long modifiers = self.modifiers();
        String ident = self.ident();
        String orig_ident = ident;
        String superName = self.superName();
        CClassType[] interfaces = self.interfaces();


        //first, set the wrapped_object class name
        this.wrappedClassName = "$chx_Orig_" + ident;

        //the rest of the methos prints the declaration
        ident = "$chx_Wrap_" + ident;
        newLine();

        mayStartAnnotation(hasFlag(modifiers, ACC_MODEL));
        print(modUtil.asString(modifiers));
        print("class " + ident);


        //extend the wrapper superclass instead of the original superclass
        if( superName == null) {
            superName = "org.jmlspecs.racwrap.runner.Wrapper";
        } else {
            superName = superName.replace('/', '.');
            if( superName.equals("java.lang.Object") 
                    || superName.equals("Object")) {
                superName = "org.jmlspecs.racwrap.runner.Wrapper";
            } else {
                int lindex = superName.lastIndexOf(".");
                if(lindex == -1) {
                    superName = "$chx_Wrap_" + superName;
                } else {
                    String className = superName.substring(lindex + 1);
                    String packageName = superName.substring(0, lindex);
                    superName = packageName + "$chx_Wrap_" + className;
                    
                }
            }
        }
        print(" extends " + superName);
        if (self.isWeakSubtype()) {
            printJml("weakly");
        }

        if (interfaces.length != 0 || self.isRacable()) {
            print(" implements ");
            boolean interfaceWeaklyFlags[] = self.interfaceWeaklyFlags();
            for (int i = 0; i < interfaces.length; i++) {
                print((i == 0 ? "" : ", ") + interfaces[i]);
                if (interfaceWeaklyFlags[i])
                    printJml("weakly");
            }

            // implement the interface JMLCheckable
            if (self.isRacable()) {
                print((interfaces.length != 0) ? "," : "");
                print(TN_JMLCHECKABLE);
                print(", " + orig_ident);
            }
        }

        print(" ");
        visitClassBody( self );

        mayEndAnnotation(hasFlag(modifiers, ACC_MODEL));
        classScopeLevel --;
        current_class = old_class;
    }

    /**
     * Prints the body of the class.
     */
    protected void visitClassBody( JmlTypeDeclaration self )
    {
        print("{");
        pos += TAB_SIZE;

        newLine();
        //print("static org.jmlspecs.racwrap.runner.Node access = null;");
        newLine();
        //print("$chx_Orig_" + self.ident() + " wrapped_object;");
        //newLine();

        JPhylum[] fieldsAndInits = self.fieldsAndInits();
        for (int i = 0; i < fieldsAndInits.length ; i++) {
                fieldsAndInits[i].accept(this);
        }

//      acceptAll( self.axioms() );
//      acceptAll( self.invariants() );
//      acceptAll( self.constraints() );
//      acceptAll( self.dependsDecls() );
//      acceptAll( self.representsDecls() );
        acceptAll( self.inners() );
        acceptAll( self.methods() );

        pos -= TAB_SIZE;
        newLine();
        print("}");

    }

    public void visitJmlMethodDeclaration( JmlMethodDeclaration self ) {
        if(self.ident().startsWith("$chx_IGNORE_THIS_")) {
            return;
        }
        if(self.ident().startsWith("$chx_")) {
            System.out.println("CHX BEGIN ---------->" + self.ident());
        }
        super.visitJmlMethodDeclaration(self);
    }

    public void visitJmlConstructorDeclaration( 
        JmlConstructorDeclaration self ) {
        if(! current_class.hasFlag(current_class.modifiers(), 
            current_class.ACC_ABSTRACT)) {
            //System.out.println("PRINTING CONSTRUCTOR: ------>" + self.ident());
            //System.out.println("OF: ------>" + current_class.ident());
            super.visitJmlConstructorDeclaration(self);
        } else {
            //System.out.println("IGNORING CONSTRUCTOR: =======>" + self.ident());
        }
    }
}
