/**
 * Simple doubly linked list using the Universe type system.
 */
public class DList {

    /**
     * The list nodes are part of the representation of the doubly
     * linked list and it should not be possible to leak them to the
     * outside or to mix two different doubly linked lists.
     */
    private rep DListNode first;
    private rep DListNode last;

    public DList() {
	first = last = null;
    }


    /**
     * The elements we add into the list can be owned by any
     * object. We do not want to modify that object, we just safe a
     * reference to it. Therefore the parameter is readonly.
     */
    public void addObject( readonly Object param ) {
	// We create the new list node as an representation object.
	// Note how the constructor parameter annotations are adapted!
	rep DListNode nn = new rep DListNode( param, null, first );

	if( first != null ) {
	    // Again note how the parameter type is adapted!
	    // The method signature is "relative" to the caller.
	    first.setPrevious( nn );
	}

	first = nn;

	if( last == null ) {
	    last = first;
	}
    }


    /**
     * The result is a readonly reference.
     * If the caller has more specific ownership information he can
     * use a downcast to peer or rep.
     */
    public readonly Object getFirst() {
	return first.getElement();
    }


    public void removeFirst() {
	if( first == last ) {
	    first = last = null;
	} else {
	    first = first.getNext();
	}
    }
}
