/**
 * The list nodes for the doubly linked list.
 */
class DListNode {

    /**
     * We only need to safe a reference to the element, we do not need
     * to modify it. Therefore elem is readonly and can belong to any
     * object.
     */
    private readonly Object elem;


    /**
     * The previous and next list nodes are owned by the same object
     * as the current list node.
     * Therefore these nodes are peers.
     */
    private peer DListNode prev;
    private peer DListNode next;


    DListNode( readonly Object elem,
	       peer DListNode prev, peer DListNode next ) {

	this.elem = elem;
	this.prev = prev;
	this.next = next;
    }


    readonly Object getElement() {
	return elem;
    }

    peer DListNode getPrevious() {
	return prev;
    }

    peer DListNode getNext() {
	return next;
    }


    void setPrevious( peer DListNode prev) {
	this.prev = prev;
    }

    void setNext( peer DListNode next) {
	this.next = next;
    }

}
