/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: HandleCreator.java,v 1.7 2007/12/20 12:25:26 chalin Exp $
 */

package org.multijava.dis;

import org.multijava.util.classfile.AccessorContainer;
import org.multijava.util.classfile.AccessorTransformer;
import org.multijava.util.classfile.BadAccessorException;
import org.multijava.util.classfile.Instruction;
import org.multijava.util.classfile.InstructionAccessor;
import org.multijava.util.ArrayLocator;

/**
 * This class replaces all references to instructions by their associated handle.
 * At the same time, it adds line numbers to the handles and checks if they
 * are references by an instruction, handler or local variable info.
 */
public class HandleCreator implements AccessorTransformer {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * Creates a new HandleCreator object
   * @param	insns			the array of instructions
   * @param	handles			the array of instruction handles
   */
  //x@ modifies this.*; // FIXME - this line does not work
  public HandleCreator(Instruction[] insns, InstructionHandle[] handles) {
    this.locator = new ArrayLocator(insns);
    this.handles = handles;
  }

  /**
   * Transforms the specified accessor.
   * @param	accessor		the accessor to transform
   * @return	the transformed accessor
   */
  public InstructionAccessor transform(InstructionAccessor accessor,
				       AccessorContainer container)
    throws BadAccessorException
  {
    int		index = locator.getIndex(accessor);

    if (index == -1) {
      throw new BadAccessorException("not in array");
    }

    handles[index].attachTo(container);

    return handles[index];
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  //@ spec_public
  private final ArrayLocator		locator;
  //@ spec_public
  private final InstructionHandle[]	handles;
}
