/*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JavadocComment.java,v 1.2 2002/03/15 19:46:22 cclifton Exp $
 */

package org.multijava.javadoc;

/**
 * An Javadoc AST node representing the root of a Javadoc AST.
 */
public class JavadocComment {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a node in the parsing tree
     * @param	text		the string representation of this comment
     */
    public JavadocComment( String text ) {
	this.text = text;
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    public JavadocComment add( JavadocComment more ) {
	text = text + " " + more.text;
	return this;
    }

    /**
     * Indicates whether this javadoc comment contains a deprecated clause.
     */
    public boolean isDeprecated() {
	return text.indexOf("@deprecated") >= 0;
    }

    /**
     * Returns the text of this comment.
     */
    public String text() {
	return text;
    }

    public String toString() {
	return "/**" + text + "*/";
    }

    //---------------------------------------------------------------------
    // PRIVATE MEMBERS
    //---------------------------------------------------------------------

    /**
     * The contents of this javadoc comment as a string.  */
    private String text;
    
}
