/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.multijava.launcher;

import java.util.Iterator;

import javax.swing.JFrame;

/**
 *  The superclass for all of the different implementations of the launchers
 *  for the tools.
 *
 *@author     Kristina Boysen
 *@version    $Revision: 1.3 $
 */

public abstract class ImpLauncher extends JFrame {

    // ----------------------------------------------------------------------
    // CONSTRUCTOR
    // ----------------------------------------------------------------------
        
    /**
     *  Constructor for the ImpLauncher
     */
    public ImpLauncher(String name, Iterator tools, String logoName) {
        super(name);
        this.tools = tools;
        this.logoName = logoName;
                
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        createLauncherLayout();
    }
        
    // ----------------------------------------------------------------------
    // ABSTRACT METHODS
    // ----------------------------------------------------------------------

    /**
     *  Creates either a simple or full layout for the launcher, depending
     *  on which subclass is called
     */
    protected abstract void createLauncherLayout();

    // ----------------------------------------------------------------------
    // PROTECTED CLASS VARIABLES
    // ----------------------------------------------------------------------
        
    /**
     *  Iterator through all the tools.  The tools are represented as 
     *  Hashtables, with the keys being the tool descriptions and the values 
     *  are represented as AbstractLaunchTools
     */
    protected Iterator tools;
        
    /**
     *  Filename of the logo for this tool
     */
    protected String logoName;
}
