/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
package org.multijava.launcher;

/**
 *  A factory that provides access to the correct implementation of the
 *  launcher
 *
 *@author     Kristina Boysen
 *@version    $Revision: 1.3 $
 */

public class LauncherFactory {
        
    // ----------------------------------------------------------------------
    // LAUNCHER FACTORY INSTANCE
    // ----------------------------------------------------------------------
        
    /**
     *  Provides an instance of the LauncherFactory
     */
    public static LauncherFactory instance() {
        if (factory == null) {
            factory = new LauncherFactory();
        }
                
        return factory;
    }
        
    // ----------------------------------------------------------------------
    // LAUNCHER INSTANCE
    // ----------------------------------------------------------------------
        
    /**
     *  Determines which instance of ImpLaunchers to provide to the caller.
     */
    public ImpLauncher getImpLauncherInstance(String name, 
	                                          ResettableIterator tools, 
                                              String logoName) {
        this.name = name;
        this.tools = tools;
        this.logoName = logoName;
        return getCompactLauncher();
    }
        
    /**
     *  Returns a singleton instance of the CompactLauncher
     */
    public CompactLauncher getCompactLauncher() {
        if (compactLauncher == null) {
            compactLauncher = new CompactLauncher(name, tools, logoName);
        }
                
        return compactLauncher;
    }

    /**
     *  Returns a singleton instance of the FullLauncher
     */
    public FullLauncher getFullLauncher() {
        if (fullLauncher == null) {
            fullLauncher = new FullLauncher(name, tools, logoName);
        }
                
        return fullLauncher;
    }

    // ----------------------------------------------------------------------
    // LAUNCHER INSTANCE
    // ----------------------------------------------------------------------
    
    /**
     *  Singleton instance of the LauncherFactory
     */
    private static LauncherFactory factory = null;
        
    /**
     *  Singleton instance of the CompactLauncher
     */
    private CompactLauncher compactLauncher = null;
        
    /**
     *  Singleton instance of the FullLauncher
     */
    private FullLauncher fullLauncher = null;
        
    /**
     *  The name of the set of tools that this Launcher can run
     */
    protected String name;
        
    /**
     *  Iterator through all the tools.  The tools are represented as 
     *  a Map.Entry, with the keys being the tool descriptions and the values 
     *  are represented as AbstractLaunchTools
     */
    protected ResettableIterator tools;
        
    /**
     *  Filename of the logo for this tool
     */
    protected String logoName;
        
}
