/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
package org.multijava.launcher;

import org.multijava.launcher.AbstractLaunchTool;

/**
 *  A launcher for the Multijava tools
 *
 *@author     Kristina Boysen
 *@version    $Revision: 1.10 $
 */

public class MjLauncher extends Launcher {
    
    // ----------------------------------------------------------------------
    // MAIN METHOD
    // ----------------------------------------------------------------------
        
    /**
     *  Main method for running the MjLauncher
     */
    public static void main(String[] args) {
        MjLauncher launcher = new MjLauncher("Multijava Launcher", args);
    }

    // ----------------------------------------------------------------------
    // CONSTRUCTOR
    // ----------------------------------------------------------------------
        
    /**
     *  Constructor for the MjLauncher
     */
    public MjLauncher(String name, String[] args) {
        this.args = args;
                
        getImpLauncher(name, LOGO_NAME);
    }
        
        
    // ----------------------------------------------------------------------
    // ACCESS METHODS
    // ----------------------------------------------------------------------
        
    /**
     *  Returns the iterator that contains the tool description and the
     *  AbstractLaunchTool that can access the tool
     */
    protected ToolIterator getTools() {
        if (iterator == null) {
            iterator = createToolIterator();
        }
        
        return iterator;
    }
        
    // ----------------------------------------------------------------------
    // BUILDER METHODS
    // ----------------------------------------------------------------------
        
    /**
     *  Builds the iterator that contains the tool description and the
     *  AbstractLaunchTool that can access the tool
     */
    private ToolIterator createToolIterator() {
        ToolIterator iterator = new ToolIterator();
        
        ToolIteratorPair pair1 = new ToolIteratorPair(
            "Multijava Compiler (mjc)",
            new AbstractLaunchTool(this) {
                public void runTool() {
                    org.multijava.mjc.MjcGUI.init(args, false);
                }
            });
        iterator.add(pair1);

        ToolIteratorPair pair2 = new ToolIteratorPair(
            "Multijava Documentation (mjdoc)",
            new AbstractLaunchTool(this) {
                public void runTool() {
                    org.multijava.mjdoc.MjdocGUI.init(args, false);
                }
            });
        iterator.add(pair2);
        
        return iterator;
    }
        
    // ----------------------------------------------------------------------
    // PRIVATE CLASS VARIABLES
    // ----------------------------------------------------------------------
    
    /**
     *  The arguments to be used by these tools
     */
    private String[] args;
        
    /**
     *  The iterator that cycles through all of the tool names and launchers
     *  for the MJ tools
     */
    private ToolIterator iterator;
        
    /**
     *  The path to the logo
     */
    private String LOGO_NAME = "org/multijava/launcher/logo/LogoHuge.png";
}
