/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: AbstractFileFinder.java,v 1.1 2002/04/19 19:08:10 cclifton Exp $
 */

package org.multijava.mjc;

/**
 * This class provides a method for converting anchor class names to
 * the prefixes of the files declaring the anchored generic function.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $ */
public abstract class AbstractFileFinder implements FileFinder, Constants {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public AbstractFileFinder() {
    }
    
    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------

    /**
     * Returns the prefix of name of the source file that should
     * declare the given class.  Useful in converting from anchor
     * class names to the files declaring the anchored generic
     * function.  */
    //@ requires className != null;
    //@ ensures \result != null;
    protected String sourceFilePrefixFor(String className) {
	int pos = className.lastIndexOf(MJ_ANCHOR);
	if (pos < 0) {
	    return className;
	} else {
	    return className.substring(0,pos);
	}
    }

}
