/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CAugmentationMap.java,v 1.4 2004/01/16 23:21:08 cclifton Exp $
 */

package org.multijava.mjc;

/**
 * This class implements a function mapping from classes and contexts
 * to method sets, <code>CAugmentationMap: CClass x CContextType ->
 * CMethodSet</code>.  It is used to store the set of methods
 * augmenting a class in a given context.  The map is stored as a
 * member of a CClass instance and so directly maps contexts to
 * methods sets.
 *
 * @see CClass
 * @see CCompilationUnitContextType
 * @see CMethodSet */
// !FIXME!augment-inners: actually the argument should be a CContextType
// through-out this c.u., see language idea 2 in the TODO file
public class CAugmentationMap extends org.multijava.util.Utils {

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------

    // just uses generic constructor

    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------

    /**
     * Given a context, returns the corresponding set of external methods.
     *
     * @param	context		the context for which to find the method set
     * @return	the external methods
     */
    public CMethodSet getAddedMethods( CCompilationUnitContextType context ) {
	CMethodSet result = backingMap.getMethodSet( context );
	return (result == null) ? new CMethodSet() : result;
    }

    public void addMethod( CMethod m, CCompilationUnitContextType context ) {
	CMethodSet set = backingMap.getMethodSet( context );
	if( set == null ) {
	    set = new CMethodSet();
	    backingMap.addMethodSet( context, set );
	}
	set.addMethod( m );
    }
 
    //---------------------------------------------------------------------
    // PRIVATE FIELDS
    //---------------------------------------------------------------------

	// !FIXME! Change the following to private when Sun fixes its javadoc bug - DRCok
    /*private*/ static class ContextMap extends java.util.HashMap {
	CMethodSet getMethodSet( CCompilationUnitContextType context ) {
	    return (CMethodSet) get( context );
	}

	void addMethodSet( CCompilationUnitContextType context, CMethodSet m ) {
	    put( context, m );
	}
    }

    private ContextMap backingMap = new ContextMap();
}
