/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBadClass.java,v 1.14 2005/04/26 02:39:30 kui_dai Exp $
 */

package org.multijava.mjc;

import java.util.Hashtable;
import org.multijava.util.classfile.FieldInfo;

/**
 * This class represents an undefined class (something that comes from
 * a bad classfile) This class is not usable to anything, so it will
 * sooner or later produce a comprehensive error.  
 */
public class CBadClass extends CClass {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Constructs a class export from file
     */
    public CBadClass(String qualifiedName) {
	super(null, new MemberAccess(null, null, 0L),  
	      null, getIdentFrom(qualifiedName), 
	      qualifiedName, null, false);
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Indicates whether this host is a subclass of the given class,
     * where "subclass" is the reflexive, transitive closure of the
     * extends relation.
     *
     * @param	maybeSuper	the potential superclass
     */
    public boolean descendsFrom( CClass maybeSuper ) {
	return false;
    }

    /**
     * Returns the signature of the field with the given name declared
     * in this class, or null if this class does not declare a field
     * with the given name. */
    protected final CField getDeclaredField( String ident ) {
	return null;
    }

    public boolean isAnonymous() { return false; }

    public boolean isMember() { return false; }

    // ----------------------------------------------------------------------
    // LOOKUP
    // ----------------------------------------------------------------------

    /**
     * Searches for a class with the given simple name according the
     * procedure in JLS2 6.5.5.  Since this is a bad class this method
     * just returns null.
     * 
     * @param	name	the class name, without qualifiers
     * @param	context		the referencing context, used for accessibility
     *				checks
     * @return	the class if found, null otherwise */
    public CClass lookupClass( String name, CContextType context ) {
	return null;
    }

    /**
     * Searches for the most specific method applicable to the given
     * identifier and argument type tuple, in the given context,
     * looking in parent hierarchy and augmentation map as needed.
     * Returns null always since this has no methods.
     *
     * @param	name		method name
     * @param	params		method parameter types
     * @param	context		the context in which the method reference
     *				should be resolved, used to identifier the
     *				external methods augmenting this class
     */
    public CMethodSet lookupMethodOrSet(String name, CType[] params, 
					CContextType context) 
    {
	return null;
    }

    /**
     * lookupField
     * search for a field
     * look in parent hierarchy as needed
     * @param	name		method name
     * @param	context		the context of the field access
     */
    public CFieldAccessor lookupField(String name,
				      CExpressionContextType context) {
	return null;
    }

    /**
     * Indicates whether this is accessible from the given host.
     *
     * @param	from	the host that wants access
     * @return	true iff the given host is allowed access to this host's 
     *		members
     */
    public boolean isAccessibleFrom( CMemberHost from ) {
	return false;
    }

    /**
     * Returns a list of interface methods
     */
    public CMethodSet.MethodArgsPair[] getInterfaceMethods() {
	return new CMethodSet.MethodArgsPair[0];
    }

    /**
     * Returns a list of abstract methods
     */
    public CMethodSet.MethodArgsPair[] getAbstractMethods() {
	return new CMethodSet.MethodArgsPair[0];
    }

    /**
     * collectInterfaceMethods
     * search for a matching method with the provided type parameters
     * look in parent hierarchy as needed
     */
    public void collectInterfaceMethods(java.util.ArrayList v, 
					CClassType[] args) {
    }

    /**
     * collectAbstractMethods
     * search for a matching method with the provided type parameters
     * look in parent hierarchy as needed
     */
    public void collectAbstractMethods(java.util.ArrayList v) {
    }

    // ----------------------------------------------------------------------
    // GENERATE CLASSFILE INFO
    // ----------------------------------------------------------------------

    /**
     * Generate the code in a class file
     * @param	destination	the location where the class file should be 
     *                          generated
     */
    public void genClassFile(org.multijava.util.Destination destination) {
	throw new RuntimeException();
    }
}
