/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBinaryClassContext.java,v 1.9 2004/01/22 20:04:16 leavens Exp $
 */

package org.multijava.mjc;

/**
 * This represents the context of a binary (i.e., read from .class
 * file) class declaration.  It is used for resolving the top methods
 * and privileged access of multimethods read from class files.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.9 $ */

public class CBinaryClassContext extends CClassContext {

    /**
     * Creates a new <code>CBinaryClassContext</code> instance.
     * Clients should not call this but should instead call
     * <code>CCompilationUnitContextType.createBinaryClassContext</code>.
     *
     * <pre><jml>
     * requires (* \caller instanceof CContextType *);
     * </jml></pre> 
     *
     * @param cunit a generated <code>CCompilationUnit</code> value
     * @param self a <code>CBinaryClass</code> value
     *
     * @see CCompilationUnitContextType#createBinaryClassContext(CBinaryClass)
     */
    CBinaryClassContext( CCompilationUnitContextType cunit, CBinaryClass self ) {
	super( cunit, self );
    }

    // ----------------------------------------------------------------------
    // CLASS AND EXTERNAL GF HANDLING
    // ----------------------------------------------------------------------

    /**
     * Registers with the surrounding context that a reference to the
     * given type occurs in this context.  Overrides superclass method
     * to not construct set of visible nested types, since these are
     * only needed for code generation and since the hashcodes used
     * there cause an infinite regress.  */
    public void registerVisibleType( CType type ) {
	getCompilationUnit().registerVisibleType( type );
    }


}
