/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBinaryField.java,v 1.11 2006/07/16 20:35:14 wdietl Exp $
 */

package org.multijava.mjc;

import org.multijava.util.classfile.FieldInfo;
import org.multijava.util.compiler.UnpositionedError;

/**
 * This class represents loaded (compiled) class fields.
 */
public class CBinaryField extends CField {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Constructs a field export
     * @param	owner		the owner of this field
     * @param	fieldInfo	a field info from a class file
     */
    public CBinaryField(CClass owner, FieldInfo fieldInfo) {
        this(new MemberAccess(owner, fieldInfo.getModifiers()), fieldInfo);
    }

    public void checkTypes(CContextType context) throws UnpositionedError {
	setType(getType().checkType(context));
    }

    protected CBinaryField(MemberAccess access, FieldInfo fieldInfo) {
        // AHS: pass universe annotations to signature parser
        super(access,
              fieldInfo.getName().intern(),
              // TODO[OJM]: Remove CClass parameter as soon as Java generics are supported!!!
              buildType(access.owner(), fieldInfo),
              fieldInfo.isDeprecated());        

       	if (access().isFinal() && access().isStatic()) {
	    Object value = fieldInfo.getConstantValue();
	    if (value != null) {
		setValue(JLiteral.createLiteral(getType(), value));
	    }
	}
    }

    /**
     * Builds the type of this field from the given <code>fieldInfo</code> by
     * parsing the appropriate field signature extracted from the *.class file.
     *
     * @param fieldInfo  The information representing this field as parsed
     *                   from a *.class file.
     * @return           The type of this field.
     */
    // TODO[OJM]: Remove CClass parameter as soon as Java generics are supported!!!
    private static CType buildType(CClass host, FieldInfo fieldInfo) {
        return CType.parseGenericTypeSignature(
                getCorrectSignature(host, fieldInfo),
                fieldInfo.getUniverseAnnotation());
    }

    /**
     * Simple convenience method which returns the appropriate field signature
     * stored in the given <code>fieldInfo</code> depending on whether Java
     * generics are turned on or off in the current compilation session.
     *
     * @param fieldInfo  The information representing this field as parsed
     *                   from a *.class file.
     * @return           The signature to be used for this field.
     */
    // TODO[OJM]: Remove CClass parameter as soon as Java generics are supported!!!
    private static String getCorrectSignature(CClass host, FieldInfo fieldInfo) {
        // TODO[OJM]: Remove CClass parameter as soon as Java generics are supported!!!
        if (CTopLevel.getCompiler().Generic(host)) {
            return fieldInfo.getGenericSignature();
        } else {
            return fieldInfo.getSignature();
        }
    }
}
