/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBinaryGFCollection.java,v 1.12 2004/01/21 22:12:40 cclifton Exp $
 */

package org.multijava.mjc;

import java.util.Map;

import org.multijava.util.classfile.ClassInfo;
import org.multijava.util.classfile.MultimethodInfo;

import org.multijava.util.compiler.UnpositionedError;
import org.multijava.util.compiler.CompilationAbortedError;

/**
 * This class represents a single generic function collection as read
 * from a .class file. */
public class CBinaryGFCollection extends CBinaryClass 
    implements CGenericFunctionCollection 
{

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------

    /**
     * Constructs a new generic function collection from a
     * <code>ClassInfo</code> object.
     *
     * @param file the info from the file */
    public CBinaryGFCollection( Main compiler, ClassInfo file ) {
	super( compiler, file );
	MultimethodInfo[] extMethInfo = file.getExternalMethods();
	CBinaryMethod[] extMeth = new CBinaryMethod[extMethInfo.length];
	for (int i = 0; i < extMethInfo.length; i++) {
	    CBinaryMethod m = new CBinaryMethod( this, extMethInfo[i], 
						 declContext );
	    m.registerVisibleTypes();
	    extMeth[i] = m; 
	}
	this.collection = 
	    new CGenericFunctionCollection.Impl( extMeth );
	applyAugmentationFor( declContext );
    }

    //-------------------------------------------------------------
    // HELPER METHODS
    //-------------------------------------------------------------

    /**
     * Adds this instance to the appropriate cache to prevent an
     * infinite regress if the initialization of this results in
     * another request to re-read the class file.  This method should
     * be called at the start of the constructor before any recursive
     * initialization the other types referenced by the class file.
     * This method should also be overridden as necessary in
     * subclasses so that their instances are added to the appropriate
     * cache.
     */
    protected void registerInstanceWithCache() {
	CTopLevel.addGFCollection(this);
    }

    //-------------------------------------------------------------
    // ACCESSORS
    //-------------------------------------------------------------

    /**
     * Augments the classes given by the receivers of methods in this
     * collection for the given context.  For each method <em>m</em>
     * in this collection, a mapping from the given context to
     * <em>m</em> is added to the <code>CClass</code> representing
     * <em>m</em>'s receiver type.
     *
     * @param	context		the context in which the augmentation occurs
     */
    public void applyAugmentationFor( CContextType context ) {
	// It is possible that collection is null if the
	// registrationof visible types in the constructor of this
	// forms a circular dependency which causes this collection to
	// be applied before it is fully initialized.  Thus far this
	// has only been observed with RMJ code.  I suspect that the
	// MultiJava type system prevents bytecode from being
	// generated that has the particular problem, but that is pure
	// speculation.  See bug 704335 for more info.
	if (collection != null) {
	    collection.applyAugmentationFor( context );
	}
    }

    public String toString() {
	return "CBinaryGFCollection: " + collection;
    }

    //-------------------------------------------------------------
    // PRIVILEGED DATA
    //-------------------------------------------------------------

    /**
     * Delegee for the storage of augmenting methods and the
     * application thereof. */
    private CGenericFunctionCollection.Impl collection;
}
