/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBooleanType.java,v 1.9 2004/03/10 10:09:28 tongjie Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.FastStringBuffer;
import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.UnpositionedError;

/**
 * This class represents the Java boolean type.
 * @see	CType
 */
public class CBooleanType extends CType {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Constructor
     */
    protected CBooleanType() {
	super(TID_BOOLEAN);
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Check if a type is the boolean type
     * @return is it the boolean type ?
     */
    public /*@ pure @*/ boolean isBoolean() {
	return true;
    }

    /**
     * Transforms this type to a string
     */
    public /*@ pure @*/ String toString() {
	return "boolean";
    }

    /**
     * Transforms this type to a string
     */
    public /*@ pure @*/ String getSignature() {
	return "Z";
    }

    /**
     * Transforms this type to a string
     */
    public void appendSignature(FastStringBuffer buff) {
	buff.append('Z');
    }

    /**
     * Returns the size used in stack by value of this type
     */
    public /*@ pure @*/ int getSize() {
	return 1;
    }

    public /*@ pure @*/ boolean isCodedAsInt() {
	return true;
    }

    // ----------------------------------------------------------------------
    // BODY CHECKING
    // ----------------------------------------------------------------------

    /**
     * Checks that this type is valid, throwing an exception if it is
     * not.  For class types this check resolves type names into their
     * fully qualified versions and sets the class parameter so that
     * getCClass returns the appropriate singleton.
     *
     * @param		context		the context in which this type was
     *					declared, may be null
     * @exception	UnpositionedError	this unpositioned error
     *						should be positioned
     *						by the calling node of
     *						the AST, typically by
     *						calling the
     *						addPosition method of
     *						UnpositionedError 
     */
    public CType checkType(CContextType context) throws UnpositionedError {
	return this;
    }

    /**
     * Compute the value of a specializer expression used to dispatch on
     * a compile-time constant.
     *
     * @param	context		the context in which this class 
     *				declaration appears
     * @exception PositionedError if the check fails */
    public void resolveValueType(CExpressionContextType context)
	throws PositionedError {}

    /**
     * Is this type assignable to the given type by assignment type
     * conversion [JLS2 5.2].
     * @param	dest		the destination type
     * @return true iff assignment is valid */
    public boolean isAlwaysAssignableTo(CType dest) {
	return dest == CStdType.Boolean;
    }

    /**
     * Can this type be converted to the specified type by casting conversion (JLS 5.5) ?
     * @param	dest		the destination type
     * @return	true iff the casting conversion is valid
     */
    public boolean isCastableTo(CType dest) {
	return dest == CStdType.Boolean;
    }
}
