/*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.UnpositionedError;

/**
 * This class represents a value specializer for booleans.  That is,
 * this represents the value specializers true and false.
 * @see	CBooleanType
 */
public class CBooleanValueType extends CValueType {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Constructor
     * @param 	dispatchExpr	the AST of the specializer expression
     */
    public CBooleanValueType(JExpression dispatchExpr) {
	super(CStdType.Boolean, dispatchExpr);
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------



	/**
	 * Is the value of this specializer equal to the value of the
	 * given value specializer?
	 *
	 * @param	t	the type to compare against
	 * @return	true iff the values are equal
	 */
    protected boolean valueEquals(CValueType t) {
	if (!(t instanceof CBooleanValueType))
	    return false;
	CBooleanValueType ot = (CBooleanValueType) t;
	return dispatchValue.getBooleanLiteral().booleanValue()
	    == ot.dispatchValue.getBooleanLiteral().booleanValue();
    }

	/**
	 * Returns an integer representation of the specializer's value,
	 * for use in computing its hashCode.
	 */
    protected int intValue() {
	return dispatchValue.getBooleanLiteral().booleanValue() ? 1 : 0;
    }
    
	

    	/*
	 * Is this value specializer of the appropriate type?
	 * requires dispatchValue != null	
	 */
    protected boolean isValid() {
	return dispatchValue.isBooleanLiteral();
    }

    // ----------------------------------------------------------------------
    // CODE GENERATION
    // ----------------------------------------------------------------------

	/* Performs the instanceof / equality test for specializers of this
	 * type.  We assume that the actual argument has already been loaded.
	 *
	 * @param	code		the bytecode sequence to append to
	 * @param	nextInstruction	the instruction to branch to if the
	 *				dispatch test fails
	 */
    public void plantDispatchTest( CodeSequence code,
				   CodeLabel nextInstruction ) {
	    // load the specializer's value
	dispatchValue.getBooleanLiteral().genCode(code);
	
	    // do the equality test and go to the next
	    // alternative if the test fails
	code.plantLabelRef( opc_if_icmpne, nextInstruction );
    }   
}
