/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CCompilationUnitContext.java,v 1.25 2006/12/11 05:55:56 chalin Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.UnpositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * This class represents the context for a compilation unit during checking
 * passes (checkInterface, checkInitializers, typecheck).
 * @see CContextType
 */
public class CCompilationUnitContext extends CContext
    implements CCompilationUnitContextType 
{

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a compilation unit context.
     */
    public CCompilationUnitContext(Main compiler, CCompilationUnit cunit) {
	this.compiler = compiler;
	this.cunit = cunit;
    }

    // ----------------------------------------------------------------------
    // ACCESSORS (INFOS)
    // ----------------------------------------------------------------------

    /**
     * Fields cannot be declared in the context of a compilation unit
     * (yet), consequently we do not expect to invoke this method on a
     * compilation unit context.
     *
     * @exception	UnsupportedOperationException */
    public CVariableState fieldInfo(int pos) {
	throw new UnsupportedOperationException();
    }

    // ----------------------------------------------------------------------
    // ACCESSORS (LOOKUP)
    // ----------------------------------------------------------------------

    /**
     * Searches for a class with the given simple name according the procedure
     * in JLS2 6.5.5.
     * 
     * @param	name	the class name, without qualifiers
     * @return	the class if found, null otherwise
     * @exception	UnpositionedError	if search fails
     */
    public CClass lookupClass( String name ) throws UnpositionedError {
	return cunit.lookupClass( name, this );
    }

    public /*@nullable*/ CTypeVariable lookupTypeVariable(String ident) throws UnpositionedError {
	return null;
    }
     /**
     * Searches for the most specific method applicable to the given
     * identifier and argument type tuple, in the current context.
     *
     * @param	name		method name
     * @param	params		method parameter types
     * @param	context		the context of the class containing
     * 				the method call
     * @exception UnpositionedError if the result is ambiguous */
    public CMethod lookupMethod( String name, CType[] params,
				 CClassContextType context ) 
	throws UnpositionedError 
    {
	return null;
    }

     /**
     * Searches for the most specific method(s) applicable to the given
     * identifier and argument type tuple, in the current context.
     *
     * @param	name		method name
     * @param	params		method parameter types
     * @param	context		the context of the class containing
     * 				the method call
     * @exception UnpositionedError if the result is ambiguous */
    public CMethodSet lookupMethodOrSet( String name, CType[] params,
					 CClassContextType context ) 
	throws UnpositionedError 
    {
	return null;
    }

   /**
     * searches for a field with the given identifier
     * @return a field from an ident in current context
     * @exception	UnpositionedError	this error will be positioned soon
     */
    public CFieldAccessor lookupField(String ident,
				      CExpressionContextType context)
	throws UnpositionedError {
	return null;
    }

    /**
     * Finds the variable with the given identifier in this context.
     * @param	ident		the name of the local variable
     * @return	a variable from an ident in current context
     */
    public JLocalVariable lookupLocalVariable(String ident) {
	return null;
    }

    /**
     * Finds a local variable with the given name that appears outside
     * the current lexical contour.
     *
     * @param	ref		a token reference used to build a new
     *				<code>JOuterLocalVariableExpression</code>
     * @param	ident		the name of the outer variable
     * @return a variable from an ident in the surrounding context, or null
     *		if not found */
    public JExpression lookupOuterLocalVariable( TokenReference ref, 
						 String ident ) {
	return null;
    }

    /**
     * Searches for a field of the given name in the context surrounding the
     * current lexical contour.
     *
     * @param	ident		the name of the field
     * @param	context		the context of the field access
     * @return	a variable from a field in the surrounding context, or null
     *		if none is found
     * @exception UnpositionedError	this error will be positioned soon
     */
    public CFieldAccessor lookupOuterField(String ident,
					   CExpressionContextType context)
	throws UnpositionedError {
	return null;
    }

    /**
     * Searches for any imported external generic functions.
     *
     * @param	ident	the name of the generic function to search for
     */
    public void resolveMaybeExtMethodRef( String ident ) {
	cunit.resolveMaybeExtMethodRef( this, ident );
    }

    /**
     * Registers the declaration of an external generic function in
     * this context. */
    public void registerGFDecl(String ident, CGenericFunctionCollection coll) {
	cunit.registerGFDecl( ident, coll );
    }

    /**
     * Registers that a declaration of the given method occurs within
     * the compilation unit. */
    public void registerVisibleMethod( CMethod method ) {
	cunit.registerVisibleMethod( method );
    }

    /**
     * Registers that a reference to the given type occurs within
     * the compilation unit. */
    public void registerVisibleType( CType type ) {
	cunit.registerVisibleType( type );
    }

    // ----------------------------------------------------------------------
    // ACCESSORS (TREE HIERARCHY)
    // ----------------------------------------------------------------------

    /**
     * getParentContext
     * @return	the parent
     */
    public CContextType getParentContext() {
	fail();
	return null;
    }

    /**
     * getClass
     * @return	the near parent of type CClassContextType
     */
    public CClassContextType getClassContext() {
	return null;
    }

    /**
     * getMethod
     * @return	the near parent of type CMethodContextType
     */
    public CMethodContextType getMethodContext() {
	return null;
    }

    /**
     * @return	the compilation unit
     */
    public CCompilationUnitContextType getCompilationUnit() {
	return this;
    }

    /** 
     * Returns the nearest control flow context (where control flow
     * information is stored.)
     * @return the nearest parent of type CFlowControlContextType
     */
    public CFlowControlContextType getFlowControlContext() {
	return null;
    }  

    /**
     * Returns the signature of the nearest lexically enclosing
     * context that can host member declarations (i.e., a
     * <code>CClass</code> or a <code>CCompilationUnit</code>).  Used
     * for access checks, this is slightly different than
     * <code>getHostClass()</code>.  The later method returns an
     * anchor class signature for external methods, which is useful in
     * code generation and in typechecking external method bodies.  On
     * the other hand, this method returns a
     * <code>CCompilationUnit</code> for external methods, which is
     * useful in checking access between two differently named generic
     * functions declared in the same compilation unit.  !FIXME! It
     * may be possible to eliminate this method and just use
     * <code>getHostClass</code> now that private external methods are
     * duplciated in every anchor.
     *
     * @return	the nearest parent that can be coerced to a 
     * <code>CMemberHost</code>, i.e., <code>this.cunit</code> */
    public CMemberHost findNearestHost() {
	return (CMemberHost) cunit;
    }

    // ----------------------------------------------------------------------
    // ACCESSORS (NEW CONTEXT)
    // ----------------------------------------------------------------------

    public CBinaryClassContext createBinaryClassContext(CBinaryClass clazz) {
	return new CBinaryClassContext( this, clazz );
    }

    // ----------------------------------------------------------------------
    // ERROR HANDLING
    // ----------------------------------------------------------------------

    /**
     * Adds an trouble into the list and eats it. This method should be
     * called after a try catch block after catching exception or
     * directly without exception thrown.
     * @param trouble the trouble 
     */
    public void reportTrouble(Exception trouble) {
	compiler.reportTrouble(trouble);
    }

    /**
     * Gets the compiler
     */
    public Main getCompiler() {
	return compiler;
    }
  
    // ----------------------------------------------------------------------
    // CLASS HANDLING
    // ----------------------------------------------------------------------

    /**
     * Adds a class to generate
     */
    public void classToGenerate( CSourceClass clazz ) {
	compiler.classToGenerate( clazz );
    }

    /**
     * Indicates whether this is equal to a given object.
     *
     * @param	o		the object to compare against
     * @return	true iff <code>o</code> is another 
     *		<code>CCompilationUnitContextType</code> formed around an
     *		identical (i.e., <code>==</code>) <code>CCompilationUnit</code>
     */
    public boolean equals(/*@ nullable @*/  Object o ) {
	return (o instanceof CCompilationUnitContext)
	    && (cunit == ((CCompilationUnitContext) o).cunit);
    }

    /**
     * Returns the hash code for this, equivalent to 
     * <code>this.cunit.hashCode()</code>.
     *
     * @return	the hash code for this
     */
    public int hashCode() {
	return cunit.hashCode();
    }

    // -------------------------------------------------------------
    // UTILITY METHODS
    // -------------------------------------------------------------

    public MJMathMode arithmeticMode() {
    	if(compiler.safeMath()){
    		return MJMathMode.newMJMathMode(Constants.AMID_SAFE_MATH);
    	}
	//return Constants.AMID_JAVA_MATH;
	return super.arithmeticMode();
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private /*final*/ Main compiler;
    private /*final*/ CCompilationUnit cunit;
}
