/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CConstructorContext.java,v 1.5 2004/01/16 23:21:08 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * This class represents the context for a constructor during checking
 * passes (checkInterface, checkInitializers, typecheck).  It includes
 * data on whether the constructor calls a super constructor.
 * @see CContextType 
 */
public class CConstructorContext extends CMethodContext 
    implements CConstructorContextType {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * CConstructorContext
     * Clients should not call this but should instead call
     * <code>CClassContextType.createConstructorContext</code>.
     *
     * <pre><jml>
     * requires (* \caller instanceof CContextType *);
     * </jml></pre>
     *
     * @param	parent		the parent context
     * @param	self		the corresponding method interface
     *
     * @see CClassContextType#createConstructorContext(CMethod)
     */
    public CConstructorContext(CContextType parent, CMethod self) {
	super(parent, self);

	isSuperConstructorCalled = true;
    }

    /**
     * Verifies that all checked exceptions are defined in the throw
     * list.  Sets the throwables of the constructor if it is the
     * synthetic constructor of an anonymous class.
     *
     * @exception PositionedError if checks fail */
    public void verifyExceptions(TokenReference ref) throws PositionedError {
	// If this is the lexical context for an anonymous class's
	// constructor, then the constructor is necessarily synthetic.
	// So set the throwables clause of the synthetic constructor
	// to include just the exceptions found to be thrown in this
	// context.
	if (getClassContext().getHostClass().isAnonymous()) {
	    getCMethod().setThrowables(throwables());
	}

	super.verifyExceptions(ref);
    }



    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    /**
     * Indicates whether this context is enclosed in a constructor.  */
    public boolean isInConstructor() {
	return true;
    }

    /**
     *
     */
    public void setSuperConstructorCalled(boolean b) {
	isSuperConstructorCalled = b;
    }

    /**
     *
     */
    public boolean isSuperConstructorCalled() {
	return isSuperConstructorCalled;
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private	boolean	isSuperConstructorCalled = true;
}
