/*
 * Copyright (C) 2006, DSRG, Concordia University 
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package org.multijava.mjc;

import java.util.List;
import java.util.ArrayList;

/**
 * <p>This class provides routines used to implement the Nullity 
 * Accessor methods that are delcared in CContextType. </P>
 *
 **/
public class CContextNullity {

    // ----------------------------------------------------------------------
    // ACCESSORS (NULLITY)
    // ----------------------------------------------------------------------

    /**
     * Indicates whether expr is conditionally NonNull is this context.
     */
    public /*@pure*/ boolean isFANonNull(/*@non_null*/ JPhylum expr) {
	    return nonNulls.contains(expr);
    }
    public /*@pure*/ boolean isFANonNull(/*@non_null*/ CFieldAccessor expr) {
	    return nonNulls.contains(expr);
    }
    /**
     * Mark expr as NonNull in this context
     */
    public void addFANonNull(/*@non_null*/ Object expr) {
	    if (expr instanceof JClassFieldExpression) {
	       CFieldAccessor field = ((JClassFieldExpression) expr).getField();
	       if (field.isFinal()) {
		  if (!nonNulls.contains(field))
	    	     nonNulls.add(field);
  	    	  while(nulls.remove(field));
	       }
	    }
	    else if (expr instanceof JVariableDefinition) {
		if (!nonNulls.contains(expr))
	    	   nonNulls.add(expr);
	    	while(nulls.remove(expr));
            } else if (expr instanceof JLocalVariableExpression) {
		if (!nonNulls.contains(expr))
		   nonNulls.add(((JLocalVariableExpression)expr).variable());
		nonNulls.add(expr);
		while(nulls.remove(((JLocalVariableExpression)expr).variable()));
		while(nulls.remove(expr));
	    } else {
	    }
    }
    public void addFANull(/*@non_null*/ Object expr) {
	    if (expr instanceof JClassFieldExpression) {
	       CFieldAccessor field = ((JClassFieldExpression) expr).getField();
	       if (field.isFinal()) {
		  if (!nulls.contains(field))
	    	     nulls.add(field);
  	    	  while(nonNulls.remove(field));
	       }
	    }
	    if (expr instanceof JVariableDefinition) {
		if (!nulls.contains(expr))
	    	   nulls.add(expr);
	    	while(nonNulls.remove(expr));
            } else if (expr instanceof JLocalVariableExpression) {
		if (!nulls.contains(((JLocalVariableExpression)expr).variable()))
		   nulls.add(((JLocalVariableExpression)expr).variable());
		if (!nulls.contains(expr))
		   nulls.add(expr);
		while(nonNulls.remove(((JLocalVariableExpression)expr).variable()));
		while(nonNulls.remove(expr));
	    } else {
	    }
    }
    public void removeFANonNull(/*@non_null*/ Object expr) {
	    while(nulls.remove(expr));
	    while(nonNulls.remove(expr));
            if (expr instanceof JLocalVariableExpression) {
		while(nulls.remove(((JLocalVariableExpression)expr).variable()));
		while(nonNulls.remove(((JLocalVariableExpression)expr).variable()));
	    }
 	    else if (expr instanceof JClassFieldExpression) {
	       CFieldAccessor field = ((JClassFieldExpression) expr).getField();
  	    	  while(nulls.remove(field));
  	    	  while(nonNulls.remove(field));
	    }
    }
    public void removeAllFANullity() {
	    nulls.clear();
	    nonNulls.clear();
    }
    public void removeAllNullityExceptLocalVariables() {
	    removeAllExceptLocalVariables(nulls);
	    removeAllExceptLocalVariables(nonNulls);
    }
    private void removeAllExceptLocalVariables(List list) {
            Object[] os = list.toArray();
	    for (int i=0; i<os.length; i++) {
		if (! ((os[i] instanceof JLocalVariable) || (os[i] instanceof JLocalVariableExpression))) {
	           while(list.remove(os[i]));
		}
	    }
    }
    /**
     * adds exprs as NonNull in this context
     */
    public void addFANonNulls(/*@non_null*/ Object[] exprs) {
	    for (int i=0; i<exprs.length; i++) {
	    	addFANonNull(exprs[i]);
	    }
    }
    public void addFANulls(/*@non_null*/ Object[] exprs) {
	    for (int i=0; i<exprs.length; i++) {
	    	addFANull(exprs[i]);
	    }
    }
    /**
     * Merge the list of nulls in this context with that of the given context.
     * This is done by taking a union of the two sets. 
     */
    public void mergeNullityInfo(/*@non_null*/ CContextType other) {
	    this.nonNulls = intersectList(this.nonNulls, other.getFANonNulls());
	    this.nulls    = intersectList(this.nulls,    other.getFANulls());
    }
    public void adoptNullityInfo(/*@non_null*/ CContextType other) {
	    this.nonNulls.clear();
	    addFANonNulls(other.getFANonNulls());
	    this.nulls.clear();
	    addFANulls(other.getFANulls());
    }
    public static /*@non_null*/ List intersectList(/*@non_null*/ List a, /*@non_null*/ Object[] b) {
	    List retVal=new ArrayList();
	    for (int i=0; i<b.length; i++) {
		if (a.contains(b[i]) && !retVal.contains(b[i])) {
		   retVal.add(b[i]);
		}
	    }
	    return retVal;
    }
    public /*@pure*/ /*@non_null*/ Object[] getFANonNulls() {
	    return (Object[])nonNulls.toArray(new Object[0]);
    }
    public /*@pure*/ /*@non_null*/ Object[] getFANulls() {
	    return (Object[])nulls.toArray(new Object[0]);
    }

     /**
      * Prints the msg and then the non-nulls
      */
    public void dumpNonNulls(/*@non_null*/ String msg) {
	    System.out.println(msg+"'"+this.toString()+"'"); 
	    for (int i=0; i<nonNulls.size(); i++) {
		    System.out.println("\tcontext.nonNull["+i+"]=="+nonNulls.get(i));
	    }
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private /*@non_null*/ List nonNulls = new ArrayList();
    private /*@non_null*/ List nulls    = new ArrayList();
}
