/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CDispatcherInitializer.java,v 1.14 2004/02/23 07:09:57 tongjie Exp $
 */

package org.multijava.mjc;

import java.util.ArrayList;

import org.multijava.util.classfile.NoArgInstruction;

/**
 * This is used as the initializer of a dispatcher class, in RMJ.  It
 * generates code to add the dispatcher class to the chain of
 * responsibility for the appropriate generic function anchor class.
 * (Regular MJ uses CAnchorInitializer.java instead.)
 */

public class CDispatcherInitializer extends CSourceMethod {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------

    /**
     * Creates a new <code>CDispatcherInitializer</code> instance.
     *
     * @param dispClass		the class for which this is a static 
     *				initialization method, i.e., the dispatcher
     *                          class
     */
    public CDispatcherInitializer( CSourceDispatcherClass dispClass ) {
	super( new MemberAccess(dispClass, dispClass, ACC_PUBLIC | ACC_STATIC),
	       "init",
	       CStdType.Void, new CSpecializedType[0], CClassType.EMPTY, 
	       CTypeVariable.EMPTY,
	       false, null, null, null );
	this.dispClass = dispClass;
    }
    
    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // DERIVED ATTRIBUTES
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // INTERFACE CHECKING
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // TYPECHECKING
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // CODE GENERATION
    //---------------------------------------------------------------------

    public ArrayList genMethodInfo() {
	ArrayList result = new ArrayList( 1 );
	result.add(nonEmptyMethodInfo( ident() ));
	return result;
    }

    /**
     * Plants the bytecode for the body of this method, which
     * initializes the chain-of-responsibility for the dispatcher
     * class.  */
    protected void plantBodyBytecode( CodeSequence code ) {
	dispClass.plantCORInitialization( code );
	code.plantInstruction(new NoArgInstruction(opc_return));
    }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    private CSourceDispatcherClass dispClass;

}// CDispatcherInitializer
