/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CErasedClassType.java,v 1.1 2004/03/07 17:45:44 
 */
package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.UnpositionedError;
import org.multijava.util.compiler.FastStringBuffer;

import org.multijava.util.classfile.*;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import org.multijava.util.compiler.CompilationAbortedError;


/**
 * This class represents class type in the type structure
 */
public class CErasedClassType extends CClassType {
    
    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------
    
    /**
     * Construct a class type
     * @param	clazz		the class that will represent this type
     */
    public CErasedClassType(CClass clazz) {
	super(clazz);
    }
    
    protected boolean isChecked() {
	return true;
    }
    
    public CClass getCClass() {
	return super.getCClass();
    }

    public boolean isClassType() {
	return true;
    }
    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------
    
    /**
     * equals
     */
    public boolean equals(/*@ nullable @*/ Object other) {
	if (!(other instanceof CClassType)) {
	    return false;
	}
	CClassType ot = (CClassType)other;
	if (ot.isArrayType()) {
	    return false;
	} else {
	    return ((CClassType)other).getCClass().equals(getCClass());
	}

	//return (!other.isClassType() || other.isArrayType()) ?
	//    false :
	//    ((CClassType)other).getCClass() == (getCClass();
    }
    
    public boolean equals(/*@ nullable @*/ CType other,
			  /*@ non_null @*/ CClassType[] substitution) {
	if (other.isTypeVariable()) {
	    other = substitution[((CTypeVariable)other).getIndex()];
	}
	if (other.isArrayType()) {
	    return false;
	} else {
	    if (other.isClassType()) {
		return ((CClassType)other).getCClass().equals(getCClass());
	    } else {
		return false;
	    }
	}
    }

    // ----------------------------------------------------------------------
    // INTERFACE CHECKING
    // ----------------------------------------------------------------------
    
    /**
     * check that type is valid
     * necessary to resolve String into java/lang/String
     * @param	context		the context (may be be null)
     * @exception UnpositionedError	this error will be positioned soon
     */
    public CType checkType(CContextType context) throws UnpositionedError {
	return this;
    }

    // ----------------------------------------------------------------------
    // BODY CHECKING
    // ----------------------------------------------------------------------
    
    /**
     * Can this type be converted to the specified type by assignment conversion (JLS 5.2) ?
     * @param	dest		the destination type
     * @return	true iff the conversion is valid
     */
    public boolean isAlwaysAssignableTo(CType dest ) {
	if (!(dest.isClassType() && !dest.isArrayType())) {
	    return false;
	} else {
	    return getCClass().descendsFrom(dest.getCClass());
	}
	//  return true;
    }
    
    public boolean isAlwaysAssignableTo(CType dest, boolean inst) {
	return isAlwaysAssignableTo(dest);
    }

    public boolean isAlwaysAssignableTo(CType dest, CClassType[] substitution) {
	return isAlwaysAssignableTo(dest);
    }
    
    public boolean isAssignableTo(CType dest) {
	return isAlwaysAssignableTo(dest);
    }
}
