/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CExpressionContextType.java,v 1.10 2006/08/08 17:18:43 perryjames Exp $
 */

package org.multijava.mjc;

/**
 * This class represents the context for an expression during checking
 * passes (checkInterface, checkInitializers, typecheck).
 * @see CContextType 
 */
public interface CExpressionContextType extends CContextType {

    void setArithmeticMode(byte arithmeticMode);

    // ----------------------------------------------------------------------
    // FACTORY METHODS
    // ----------------------------------------------------------------------
 
	CExpressionContextType createExpressionContext_();

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Set this context as an increment or decrement expression, so
     * we can generate appropriate getters and setters if necessary
     * (and also generate proper code in the future).
     * @param	incDec
     */
    void setIncDec(boolean incDec);

    /**
     * @return	true if this context is part of an increment or decrement
     * 		expression
     */
    boolean isIncDec();

        /**
     * Set this context as a left side in an assignment, so access to vars
     * may be uninitialized
     * @param	leftSide		set the side of current code flow
     */
    void setLeftSide(boolean leftSide);

    /**
     * @return	true if this context is on a left side of an assignement
     */
    boolean isLeftSide();

    /**
     * set this context as a discarded value so result is ignored
     * @param	discardValue	is the value of this expression in 
     *				this context discarded?
     */
    void setDiscardValue(boolean discardValue);

    /**
     * @return	true if the result value of this expression is discarded
     */
    boolean discardValue();
}
