/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CFieldAccessor.java,v 1.11 2008/03/17 16:59:46 wdietl Exp $
 */

package org.multijava.mjc;

/**
 * This interface represents a field or proxy for a field (a field
 * getter or setter method) 
 */
public interface CFieldAccessor {
    
      /**
       * Returns the owner of this member.
       *
       * @return the class that owns this member */
      public CClass owner();

//      /**
//       * Returns the host of this member's declaration, which is the
//       * same value returned by <code>owner()</code> unless this is a
//       * top-level declaration.
//       *
//       * @return the host of this member's declaration */
//      public CMemberHost host();
    
    /**
     * Return the type of this field.
     * The method is not pure, as the CField subclass needs
     * to implement some caching.
     *
     * @return the type of this field (or the type of the associated field of
     * a getter/setter)
     */
    public CType getType();

    public void setType(CType type);
    /**
     * @return the value of this field (or the value of the associated field of
     * a getter/setter)
     */
    public /*@ pure @*/ JExpression getValue();


    public /*@ pure @*/ String ident();
    
    /**
     * @return	true if this member is static
     */
    public /*@ pure @*/ boolean isStatic();

    /**
     * @return	true if this member's associated field is static
     */
    public /*@ pure @*/ boolean isFieldStatic();

    /**
     * @return	true if this member is final
     */
    public /*@ pure @*/ boolean isFinal();

    /**
     * Returns the target of this accessor method.
     */
    public /*@ non_null pure @*/ CField getField();
    
    /*
     * Indicates whether this is accessible from the given host.
     *
     * @param	from	the host that wants access
     * @return	true iff the given host is allowed access to this member
     */
//    public boolean isAccessibleFrom( CMemberHost from );
    
    // ----------------------------------------------------------------------
    // GENERATE CLASSFILE INFO
    // ----------------------------------------------------------------------

    /**
     * Generates a sequence of bytecodes to load
     * @param	code		the code list
     */
    public void genLoad(CodeSequence code);
    

    /**
     * Generates a sequence of bytecodes to load
     * @param	code		the code list
     */
    public void genStore(CodeSequence code);

}
