/*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.multijava.mjc;


/**
 * This class is the abstract base class for the field getter and setter
 * methods.
 */
 
public abstract class CFieldAccessorMethod extends CSourceMethod
implements CFieldAccessor {
    
    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------

    /**
     * Constructs a getter method for the given method.
     * @param	target	the field
     * @param	owner	the receiver of the method
     * @param	retType	the return type of the method
     * @param   parms 	the parameter types of the method
     */
    public CFieldAccessorMethod( CField target, CClass owner, CType retType,
				 CSpecializedType[] parms) {
	super( new MemberAccess(owner, owner, ACC_STATIC), 
	       CClass.getNextAccessName(), retType, 
	       parms, new CClassType[0], CTypeVariable.EMPTY,false,
		   // fake a body so that code will be generated
		   // for this method
	       JBlock.DUM_BLOCK,
	       null, null );
	this.target = target;
	this.topConcreteMethod = this;
    }


    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

	// check whether this accessor method can be re-used for the
	// given field
    public boolean isApplicableTo(CField f) {
	return target.equals(f);
    }

    public CType getType() {
	return target.getType();
    }
    
    public void setType(CType type){
	target.setType(type);
    }

    public JExpression getValue() {
	return target.getValue();
    }

    public boolean isFieldStatic() {
	return target.isStatic();
    }

    
    /**
     * Returns the target of this accessor method.
     */
    public /*@ non_null @*/ CField getField() {
        return target;
    }
    
    // ----------------------------------------------------------------------
    // GENERATE CLASSFILE INFO
    // ----------------------------------------------------------------------

	/* Generate a load of the field. */
    public abstract void genLoad(CodeSequence code);
	

    	/* Generate a store of the field. */
    public abstract void genStore(CodeSequence code);
	
 
	
    //---------------------------------------------------------------------
    // PRIVATE FIELDS
    //---------------------------------------------------------------------

    /**
     * The target field of this wrapper. */
    protected CField target;

}
