/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CFinallyContext.java,v 1.2 2004/09/14 19:01:37 cclifton Exp $
 */
package org.multijava.mjc;

import org.multijava.util.compiler.TokenReference;

/**
 * This class implements the special definite assignment and reachability
 * calculations for combining try and finally blocks in a try-finally statement.
 */
public class CFinallyContext extends CFlowControlContext
		implements CFlowControlContextType {
	public CFinallyContext(CFlowControlContextType beforeTryContext,
						   CFlowControlContextType afterTryContext,
						   TokenReference where) {
		super(beforeTryContext, where);
		this.afterTryContext = afterTryContext;
		this.merge(afterTryContext);
		this.setReachable(true);
	}

	public void checkingComplete() {
		tryFinallyMerge();
		super.checkingComplete();
	}

	private void tryFinallyMerge() {
		isReachable &= afterTryContext.isReachable();

		int varCount = numberOfLocalVars();
		variableInfo.tryFinallyMergeUpTo(varCount, afterTryContext.variableInfo());

		fieldInfo.tryFinallyMergeUpTo(fieldCount(), afterTryContext.fieldInfo());
	}

	private final CFlowControlContextType afterTryContext;
}



