/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CFunctionAccessMethod.java,v 1.2 2005/04/26 02:39:31 kui_dai Exp $
 */

package org.multijava.mjc;

import java.util.ArrayList;
import org.multijava.util.classfile.FieldRefInstruction;
import org.multijava.util.classfile.NoArgInstruction;
import org.multijava.util.classfile.PushLiteralInstruction;

/**
 * This generates code to thread-safely return the function field of
 * an external generic function.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.2 $ */

public class CFunctionAccessMethod extends CSourceMethod {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------

    /**
     * Creates a new <code>CFunctionAccessMethod</code> instance.
     *
     * <pre><jml>
     *   requires owner != null && type != null;
     * </jml></pre>
     *
     * @param owner             the class of the generic function anchor
     * @param modifiers         the modifiers of the function field, also used
     *                          for the generated method
     * @param index             the index of the function field
     * @param type              the signature type of the function field
     */
    public CFunctionAccessMethod(CClass owner, long modifiers, int index, 
                                 CType type) {
        super( new MemberAccess(owner, 
                                modifiers | ACC_STATIC | ACC_SYNCHRONIZED),
               "getFunction" + index, type, // !FIXME!function
               new CSpecializedType[0], CClassType.EMPTY, 
               CTypeVariable.EMPTY, false, null, null, null );
        this.index = index;
    }
    
    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // DERIVED ATTRIBUTES
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // INTERFACE CHECKING
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // TYPECHECKING
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // CODE GENERATION
    //---------------------------------------------------------------------

    public ArrayList genMethodInfo() {
        ArrayList result = new ArrayList( 1 );
        result.add(nonEmptyMethodInfo( ident() ));
        return result;
    }

    /**
     * Plants the bytecode for the body of this method.  */
    protected void plantBodyBytecode( CodeSequence code ) {
        code.plantInstruction( 
            new FieldRefInstruction(opc_getstatic, owner().qualifiedName(),
                                    "function" + index, // !FIXME!function 
                                    returnType().getSignature() ));
        code.plantInstruction(new NoArgInstruction(opc_areturn));
    }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    private int index;
}
