/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CInitializerContext.java,v 1.4 2004/01/16 23:21:09 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.UnpositionedError;

/**
 * This class represents the context for a static initializer during checking
 * passes (checkInterface, checkInitializers, typecheck).
 * @see CContextType 
 */
public class CInitializerContext extends CMethodContext 
    implements CInitializerContextType {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Instantiates a context for checking initializer declarations.
     * Clients should not call this but should instead call
     * <code>CClassContextType.createInitializerContext</code>.
     *
     * <pre><jml>
     * requires (* \caller instanceof CContextType *);
     * </jml></pre>
     *
     * @param	parent		the parent context
     * @param	self		the corresponding method interface
     *
     * @see CClassContextType#createInitializerContext(CMethod)
     */
    public CInitializerContext(CContextType parent, CMethod self) {
	super(parent, self);
    }

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    /**
     * Indicates whether this context is enclosed in an instance or
     * static initializer.  */
    public boolean isInInitializer() {
	return true;
    }
}
