/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CLabeledContext.java,v 1.6 2004/01/16 23:21:09 cclifton Exp $
 */

package org.multijava.mjc;

/**
 * This class represents a labeled context during checking passes
 * (checkInterface, checkInitializers, typecheck).  It simply wraps
 * another context and stores a label for the wrapped context.
 * @see CContextType 
 */
public class CLabeledContext extends CFlowControlContext {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a block context, it supports local variable allocation
     * throw statement and return statement
     * @param	parent		the parent context, it must be different
     *				than null except if called by the top level
     */
    CLabeledContext(CFlowControlContextType parent, JLabeledStatement self) {
	super( parent, self.getTokenReference() );

	this.self = self;
    }

    /**
     * Used by cloneContext.  Should not be called by clients.  */
    protected CLabeledContext( CFlowControlContextType cachedParent,
			       CLabeledContext clone ) {
	super( cachedParent, clone );

	this.self = clone.self;
    }

    // ----------------------------------------------------------------------
    // ACCESSORS (NEW CONTEXT)
    // ----------------------------------------------------------------------

    /**
     * Create a clone of this context to handle divergent paths in the
     * control flow.  */
    public CFlowControlContextType cloneContext() {
	assertTrue( cachedParent != null );
	return new CLabeledContext( cachedParent, this );
    }

    // ----------------------------------------------------------------------
    // ACCESSORS (SWITCHES)
    // ----------------------------------------------------------------------

    /**
     * Returns the statement with the specified label.
     */
    public JLabeledStatement getLabeledStatement(String label) {
	if (label.equals(self.getLabel())) {
	    return self;
	} else {
	    return cachedParent.getLabeledStatement(label);
	}
    }


    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private final JLabeledStatement	self;
}
