/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CMemberHost.java,v 1.8 2004/01/22 20:04:39 leavens Exp $
 */

package org.multijava.mjc;

/**
 * This interface is implementation by abstractions that can host
 * members.  It provides operations for testing the permissibility of
 * access to the hosted members. */
public interface CMemberHost {

    /**
     * Indicates whether this is accessible from the given host.
     *
     * @param	from	the host that wants access
     * @return	true iff the given host is allowed access to this host's 
     *		members
     */
    /*@ pure @*/ boolean isAccessibleFrom( CMemberHost from );

    /**
     * Indicates whether this is defined inside the given host.
     *
     * @param	maybeOutside	the host that this may be inside
     * @return	true if the given host is a (possibly transitive) host of this
     */
    /*@ pure @*/ boolean isDefinedInside( CMemberHost maybeOutside );

    /**
     * Indicates whether this host is a subclass of the given class,
     * where "subclass" is the reflexive, transitive closure of the
     * extends relation.
     *
     * @param	maybeSuper	the potential superclass
     */
    /*@ pure @*/ boolean descendsFrom( CClass maybeSuper );

    /**
     * Returns true if this host is a subclass of the given class or
     * if some host of this host is a subclass of the given class.
     *
     * @param maybeOuterSuper the potential superclass */
    /*@ pure @*/ boolean nestedDescendsFrom( CClass maybeOuterSuper );

    /**
     * Indicates whether this host is a class.
     *
     * @return	true iff this host represents a class
     */
    /*@ pure @*/ boolean isClass();

    /**
     * Indicates whether this host is a nested type, i.e., a member
     * type, a type declared within another type.  */
    /*@ pure @*/ boolean isNestedType();

    /**
     * Indicates whether this host is an inner class, i.e., a
     * non-static member class.  */
    /*@ pure @*/ boolean isInnerClass();

    /**
     * Indicates whether this host is a compilation unit
     *
     * @return	true iff this host represents a compilation unit
     */
    /*@ pure @*/ boolean isCompilationUnit();

    /**
     * Gives the identifier for this host, or null for a compilation unit.
     */
    /*@ pure @*/ String ident();

    /**
     * Gives the package name for this host.
     */
    /*@ pure @*/ String packageName();

    /**
     * Returns the host of this host, that is, the context in which
     * this host is declared.  This value is null except for inner
     * classes.
     *
     * @return the host of this host */
    /*@ pure @*/ CMemberHost host();
}

