/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CMethodContextType.java,v 1.12 2004/06/29 20:02:47 leavens Exp $
 */

package org.multijava.mjc;

import java.util.Set;
import org.multijava.util.compiler.TokenReference;
import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.UnpositionedError;

/**
 * This class represents the context for a method during checking
 * passes (checkInterface, checkInitializers, typecheck).
 * @see CContextType 
 */
public interface CMethodContextType extends CContextType {

    /**
     * Verifies that all checked exceptions are defined in the throw
     * list.
     *
     * @exception PositionedError if checks fail */
    void verifyExceptions(TokenReference ref) throws PositionedError;

    // ----------------------------------------------------------------------
    // ACCESSORS (NEW CONTEXT)
    // ----------------------------------------------------------------------

    CFlowControlContextType createFlowControlContext(int localVars,
						     TokenReference where);

    CFlowControlContextType createFlowControlContext(int localVars,
						     boolean isInExternalGF,
						     TokenReference where);

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Returns the signature of the method declaration in which this
     * context is enclosed, or null if this context is not enclosed in
     * a method declaration.  */
    //@ pure
    CMethod getCMethod();

    /**
     * Returns the nearest surrounding context of type CMethodContextType.  */
    CMethodContextType getMethodContext();

    /**
     * Indicates whether this context is enclosed in an instance or
     * static initializer.  */
    boolean isInInitializer();

    /**
     * Indicates whether this context is enclosed in a constructor.  */
    boolean isInConstructor();

    /**
     * Indicates whether this context is "static".
     *
     * @return true iff this context is enclosed in a context for a
     * static initializer or static method.  */
    boolean isStatic();

    /**
     * Indicates whether this context is "pure".
     *
     * @return true iff this context is enclosed in a pure method.
     */
    //@ also
    //@ implies_that
    //@   assignable objectState;
    boolean isPure();


    // ----------------------------------------------------------------------
    // THROWABLES
    // ----------------------------------------------------------------------

    /**
     * Registers that the given throwable can be thrown within this
     * context.  The implementation passes the throwable up through
     * the nested contexts until a surrounding context is found that
     * can handle throwables.  For example, a CMethodContextType
     * instance collects the throwables that are uncaught within a
     * method body, while a CTryContext collects the throwables that are
     * thrown within a try-block and are therefore subject to handling
     * by associated catch-blocks.
     * @param throwable the type of the new throwable */
    void addThrowable(CThrowableInfo throwable);

    /**
     * @return the set of exception that may be thrown
     */
    Set throwables();
}
