/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CMethodInfo.java,v 1.7 2004/04/12 21:00:42 tongjie Exp $
 */

package org.multijava.mjc;

import java.io.*;
import org.multijava.util.classfile.MethodInfo;
import org.multijava.util.classfile.ConstantPool;
import org.multijava.util.classfile.ClassFileFormatException;
import org.multijava.util.classfile.CodeInfo;

/**
 * VMS 4.6: Methods
 *
 * Each method, and each (class or instance) initialization method is
 * described by this structure.
 */
public class CMethodInfo extends MethodInfo {

    // --------------------------------------------------------------------
    // CONSTRUCTORS
    // --------------------------------------------------------------------

    /**
     * Constructs a method entry. NOTE: VMS 4.7.5: There must be
     * exactly one Exceptions attribute in each method_info structure.
     *
     * @param	modifiers	access permission to and properties of the method
     * @param	name		the simple name of the method (or <init> or <clinit>)
     * @param	type		the method signature
     * @param	exceptions	the checked exceptions the method may throw
     * @param	method		the method that this provides information on
     * @param	deprecated	is this method deprecated ?
     * @param	synthetic	is this method synthesized by the compiler ?
     *
     */
    public CMethodInfo(short modifiers,
		       String name,
		       String type,
		       String genericsignature,
		       String[] exceptions,
		       CSourceMethod method,
		       boolean deprecated,
		       boolean synthetic) {
	super(modifiers, name, type, genericsignature, exceptions, 
	      CodeInfo.DUM_INFO, 
	      deprecated, synthetic);
	this.method = method;
    }

    /**
     * Constructs a new instance. This constructor is for subclasses.
     */
    protected CMethodInfo(short modifiers, String name, String type,
			  String genericsignature,
			  String[] exceptions, CodeInfo code, 
                          boolean deprecated, boolean synthetic) {
        super(modifiers, name, type, genericsignature, 
	      exceptions, code, deprecated, synthetic);
    }

    /**
     * Constructs an instance by reading bytecode from the given
     * stream <code>in</code> and referring to the constant pool
     * <code>cp</code>. This constructor is for subclasses.
     */
    protected CMethodInfo(/*@ non_null @*/ DataInput in, 
                            /*@ non_null @*/ ConstantPool cp,
                            boolean interfaceOnly)
        throws IOException, ClassFileFormatException {
        super(in, cp, interfaceOnly);
    }

    // --------------------------------------------------------------------
    // WRITE
    // --------------------------------------------------------------------

    /**
     * Insert or check location of constant value on constant pool
     *
     * @param	cp		the constant pool for this class
     */
    public void resolveConstants(ConstantPool cp) 
	throws ClassFileFormatException 
    {
	CodeInfo code = method.genCode();
	setCodeInfo(code);
	super.resolveConstants(cp);
    }

    // --------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // --------------------------------------------------------------------

    private CSourceMethod		method;
}
