/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CMethodNotFoundError.java,v 1.5 2004/06/28 13:52:04 wdietl Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * This error display all parameters of method call
 */
public class CMethodNotFoundError extends PositionedError {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * @param	where		the reference to token where error happen
     * @param	name		the method name
     * @param	types		the parameter types
     * @param	isConstr	true if the error is for a constructor not
     *				found
     */
    public CMethodNotFoundError( TokenReference where, String name, 
				 CType[] types, boolean isConstr ) {
	super( where, 
	       isConstr ? MjcMessages.CONSTRUCTOR_NOT_FOUND : 
	       MjcMessages.METHOD_NOT_FOUND, 
	       buildSignature(name, types));
    }

    private static String buildSignature(String name, CType[] types) {
	StringBuffer	buffer = new StringBuffer();

	buffer.append(name);
	buffer.append("(");
	if (types != null) {
	    for (int i = 0; i < types.length; i++) {
		if (i != 0) {
		    buffer.append(", ");
		}
		buffer.append(types[i].toVerboseString()); // WMD
	    }
	}
	buffer.append(")");

	return buffer.toString();
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

}
