/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CNullType.java,v 1.10 2008/03/16 15:52:43 wdietl Exp $
 */

package org.multijava.mjc;

/**
 * This class represents null class type in the type structure
 * @see	CType
 */
public class CNullType extends CClassType {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a class type
     */
    public CNullType() {
	super(new CBadClass("<NULL TYPE>"));
    }

    // ----------------------------------------------------------------------
    // INTERFACE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Is this type assignable to the given type by assignment type
     * conversion [JLS2 5.2] [JLS2 5.1.4].
     * @param	dest		the destination type
     * @return true iff assignment is valid */
    public boolean isAlwaysAssignableTo(CType dest) {
	return dest.isReference();
    }

    public boolean isAlwaysAssignableTo(CType dest, boolean inst) {
	return isAlwaysAssignableTo(dest);
    }
    
     
    public boolean isAlwaysAssignableTo(CType dest, CClassType[] subst) {
	return isAlwaysAssignableTo(dest);
    }
    
    public boolean isAlwaysAssignableToIgnoreUniverses(CType dest, 
		    CClassType[] substitution) {
    	return isAlwaysAssignableTo(dest);
    }
    
    public boolean isAlwaysAssignableToIgnoreUniverses(CType dest) {
    	return isAlwaysAssignableTo(dest);
    }
    
    /**
     * isCastableTo
     * @param dest  the destination type
     * @return  cast is valid ?
     */
    public boolean isCastableTo(CType dest) {
	return dest.isReference();
    }

    /**
     * implicitlyCastTo
     * @param	dest	the expression type to cast to
     * @return	cast is valid ?
     */
    public boolean implicitlyCastTo(CType dest) {
	return dest.isReference();
    }

    public boolean needsUncheckedConversion(CType dest) {
        return false;
    }

    /**
     * get the object, if this type was never checked (reads from class files)
     * check it!
     */
    protected CClass getObject() {
	return CStdType.Object.getCClass();
    }

    /**
     * Returns the class object associated with this type.  In this
     * case it is always the java.lang.Object CClass singleton.
     *
     * @return the class object associated with this type */
    public CClass getCClass() {
	return CStdType.Object.getCClass();
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Transforms this type to a string
     */
    public String toString() {
	return "null-literal";
    }

    /**
     * For the null literal the universe annotation is not meaningful.
     * By WMD.
     */
    public String toVerboseString() {
        return toString();
    }

    /**
     * Returns the size used in stack by value of this type
     */
    public int getSize() {
	return 1;
    }
}
