/*
 * Copyright (C) 2000-2001 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.UnpositionedError;

import org.multijava.util.classfile.*;


/**
 * This class represents a value specializer for floats and doubles.
 *
 * @see	CNumericType
 */
public class CRealValueType extends CValueType {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Constructor
     * @param	staticType	the associated static type of the formal
     * @param 	dispatchExpr	the AST of the specializer expression
     */
    public CRealValueType(CType staticType, JExpression dispatchExpr) {
	super(staticType, dispatchExpr);
	int typeID = staticType.getTypeID();
	assertTrue(typeID == TID_DOUBLE ||
		   typeID == TID_FLOAT);
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------


	/**
	 * Is the value of this specializer equal to the value of the
	 * given value specializer?
	 *
	 * @param	t	the type to compare against
	 * @return	true iff the values are equal
	 */
    protected boolean valueEquals(CValueType t) {
	if (!(t instanceof CRealValueType))
	    return false;
	CRealValueType ot = (CRealValueType) t;
	return dispatchValue.getRealLiteral().numberValue().doubleValue()
	    == ot.dispatchValue.getRealLiteral().numberValue().doubleValue();
    }

	/**
	 * Returns an integer representation of the specializer's value,
	 * for use in computing its hashCode.
	 */
    protected int intValue() {
	return dispatchValue.getRealLiteral().numberValue().intValue();
    }
    
    	/*
	 * Is this value specializer of the appropriate type?
	 * requires dispatchValue != null	
	 */
    protected boolean isValid() {
	return dispatchValue.isRealLiteral();
    }	

    // ----------------------------------------------------------------------
    // CODE GENERATION
    // ----------------------------------------------------------------------

	/* Performs the instanceof / equality test for specializers of this
	 * type.  We assume that the actual argument has already been loaded.
	 *
	 * @param	code		the bytecode sequence to append to
	 * @param	nextInstruction	the instruction to branch to if the
	 *				dispatch test fails
	 */
    public void plantDispatchTest( CodeSequence code,
				   CodeLabel nextInstruction ) {
	    // load the specializer's value
	dispatchValue.getRealLiteral().genCode(code);
	
	switch(type) {
	  case TID_FLOAT:
		// do the equality test	
	    code.plantInstruction( new NoArgInstruction(opc_fcmpl) );
	    break;
	  case TID_DOUBLE:
		// do the equality test	
	    code.plantInstruction( new NoArgInstruction(opc_dcmpl) );
	    break;
	}
	    // go to the next alternative if the test fails		
	code.plantLabelRef( opc_ifne, nextInstruction );
    }   
}
