/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CSourceDispatcherSignature.java,v 1.16 2004/02/23 07:09:57 tongjie Exp $
 */

package org.multijava.mjc;
import org.multijava.util.compiler.TokenReference;
import java.util.Hashtable;

/**
 * This class represents the type (i.e., $signature) for inner dispatcher
 * classes of an external generic function.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.16 $
 */
public class CSourceDispatcherSignature extends CSourceClass 
    implements CDispatcherSignature
{

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public CSourceDispatcherSignature( Main compiler, CClass outer,
                                       TokenReference where, 
                                       CSourceDispatcherMethod dispMethod,
				       int number ) {
        super( compiler, new MemberAccess(outer, outer, 
					  dispMethod.access().modifiers()
					  | ACC_INTERFACE | ACC_ABSTRACT ),
	       where, 
               "ident() overridden", "qualifiedName() overridden", CTypeVariable.EMPTY,false, 
	       true, false );
	this.number = number;
        CTopLevel.addClass( this );
	compiler.classToGenerate( this );
        CClassType[] emptyCTArray = new CClassType[0];
	setSuperClass(CStdType.Object);
	setCheckedInterfaces( emptyCTArray );
	setCheckedMembers( new Hashtable(), new CMethod[0] );
        
	CMethod applyMethod = 
	    new CSourceMethod( new MemberAccess(this, this, 
				    ( access().modifiers() & ~ACC_PRIVATE
				     & ~ACC_PROTECTED ) | ACC_PUBLIC ),
			            // in interface, meth must be public
			       "apply", dispMethod.returnType(), 
                               dispMethod.parameters(),
			       dispMethod.throwables(), 
			       CTypeVariable.EMPTY, false, null, 
			       null, null ) {
		// instance initializer
		{ 
		    this.topConcreteMethod = this;
		}
		
	    };
	close( new CMethod[] { applyMethod }, 
               emptyCTArray );
    }
    
    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------

    public String ident() {
	return owner().ident() + "$signature" + number;
    }

    public String qualifiedName() {
	String result = host().packageName();
	return result.length() > 1 ? result + ident() : ident();
    }

    //---------------------------------------------------------------------
    // DERIVED ATTRIBUTES
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // INTERFACE CHECKING
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // TYPECHECKING
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // CODE GENERATION
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    /**
     * The unique number assigned to this signature by the anchor class.
     */
    private int number;

}
