/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CSourceField.java,v 1.8 2004/01/16 23:21:09 cclifton Exp $
 */

package org.multijava.mjc;

/**
 * This class represents an exported member of a class (fields)
 */
public class CSourceField extends CField implements VariableDescriptor {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Constructs a field export
     * @param	access		contains the info used to determine accessibility of this field
     * @param	ident		the name of this field
     * @param	type		the type of this field
     * @param	deprecated	is this field deprecated
     */
    public CSourceField(MemberAccess access, String ident, 
			CType type, boolean deprecated) {
	super(access, ident, type, deprecated);
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    public boolean isUsed() {
	return used || !isPrivate() || ident().indexOf("$") >= 0; // $$$
    }

    public void setUsed() {
	used = true;
    }

    /**
     * Indicates that this field is initialized (unless it is declared
     * a final instance field, in which case it needs to be assigned
     * to). */
    public void initializedUnlessFinalInstance() {
	this.initialized = !isFinal() || isStatic() ;
    }

    /**
     *
     */
    public boolean isDefinitelyAssigned() {
	return initialized;
    }

    public int index() {
	return index;
    }

    public void setIndex(int index) {
	this.index = index;
    }

    public String toString() {
	return "field " + ident() + " at position " + index +
	    " in host " + host();
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private int			index;
    private boolean		used;

    /**
     * Indicates that at the current status of the compilation the
     * compiler knows that this field has been properly
     * initialized. */
    protected boolean		initialized;
}
