/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CTryContext.java,v 1.6 2004/01/16 23:21:10 cclifton Exp $
 */

package org.multijava.mjc;

import java.util.HashSet;
import java.util.Iterator;
import org.multijava.util.compiler.TokenReference;
import java.util.Set;
import java.util.Collections;

/**
 * This class represents the context for a try statement during
 * checking passes (checkInterface, checkInitializers, typecheck).  It
 * tracks all the possible exceptions within the try block so that the
 * corresponnding catch blocks can be verified to catch all and only
 * the appropriate exceptions.
 * @see CContextType 
 */
public class CTryContext extends CFlowControlContext {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a block context, it supports local variable allocation
     * throw statement and return statement
     * @param	parent		the parent context, it must be different
     *				than null except if called by the top level
     */
    CTryContext(CFlowControlContextType parent, TokenReference where) {
	super(parent, where);
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------
    // ACCESSORS (THROWABLES)
    // ----------------------------------------------------------------------

    /**
     * Registers that the given throwable can be thrown within this
     * context.  The implementation passes the throwable up through
     * the nested contexts until a surrounding context is found that
     * can handle throwables.  For example, a CMethodContextType
     * instance collects the throwables that are uncaught within a
     * method body, while a CTryContext collects the throwables that are
     * thrown within a try-block and are therefore subject to handling
     * by associated catch-blocks.
     * @param throwable the type of the new throwable */
    public void addThrowable(CThrowableInfo throwable) {
	throwables.add(throwable);
    }

    /**
     * Returns an immutable set representing the exceptions throwable
     * within this context.  */
    public Set throwables() {
	return Collections.unmodifiableSet(throwables);
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    /**
     * Tracks the exceptions that may be thrown inside this try-block.  */
    private HashSet throwables = new HashSet();
}
