/*
 * Copyright (C) 2003-2004 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CUniverse.java,v 1.7 2008/03/17 16:59:46 wdietl Exp $
 */

package org.multijava.mjc;

/**
 * This is the abstract base class that represents Universes.
 *
 * @author WMD
 */
public abstract class CUniverse {
	
    protected CUniverse() {
    }

    /**
     * Check whether two universe objects are the same.
     * For now we simply look whether it's a universe of the same class.
     */
    public boolean equals(/*@ nullable @*/  Object other) {
    	if (other instanceof CUniverse) {
    		CUniverse ot = (CUniverse) other;
    		return getClass() == ot.getClass();
    	}

    	return false;
    }
    
    /**
     * Hash code only depends on the class itself.
	 * Added because of equals method
	 * (see Effective Java by Joshua Bloch - Item 8)
     * 
     * {inheritDoc}
     */
    public int hashCode() {
    	return getClass().hashCode();
    }

    /**
     * Tests whether the current universe can be assigned to
     * the given universe.
     *
     * @param other The destination universe type.
     */
    public abstract boolean isAlwaysAssignableTo(CUniverse other);

    /**
     * Output the name of the universe.
     */
    public abstract String toString();
   
    /**
     * Output the name of the universe as it should be used for MJ code.
     */
    public String toMJString() {
    	return toString();
    }


    /**
     * Output the name of the universe as it should be used for JML code.
     */
    public String toJMLString() {
    	return toString();
    }
    
    /**
     * Returns the bytecode constant representing this particular universe.
     * By AHS.
     */
    public abstract byte getByteConstant();

}
