/*
 * Copyright (C) 2003-2004 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CUniversePeer.java,v 1.6 2008/03/17 16:59:46 wdietl Exp $
 */

package org.multijava.mjc;

/**
 * This class implements a peer/ground/default universe.
 * This is the standard universe that is used if no special annotation
 * is given.
 *
 * @author WMD
 */
public class CUniversePeer extends CUniverse {

    protected CUniversePeer() {
    	super();
    }

    /**
     * Any peer reference equals other peer or implicit-peer references.
     */
    public boolean equals(/*@ nullable @*/  Object other ) {
    	return other instanceof CUniversePeer;
    }
    
    /**
     * Same hash code for peer and implicit-peer references.
     * 
     * {inheritDoc}
     */
    public int hashCode() {
    	return CUniversePeer.class.hashCode();
    }

    /**
     * A peer is only assignable to another peer or to a readonly reference.
     */
    public boolean isAlwaysAssignableTo( CUniverse other ) {
    	return other instanceof CUniversePeer
// WMD: RZU had this???
//    			|| other instanceof CUniverseThis
    			|| other instanceof CUniverseReadonly;
    }

    /**
     * Output "peer".
     */
    public String toString() {
    	return "peer";
    }
    
    /**
     * Returns the bytecode constant representing this particular universe.
     */
    public byte getByteConstant() {
        return CUniversePeer.byteConstant;
    }

    /**
     * Factory method to return the single universe instance.
     */
    public static CUniversePeer getUniverse() {
    	return instance;
    }

    /** 
     * The singleton reference.
     */
    private static final CUniversePeer instance = new CUniversePeer();
    
    /**
     * The constant used in the bytecode to represent this universe.
     */
    public static final byte byteConstant = 0x1;
}
