/*
 * Copyright (C) 2003-2004 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CUniverseReadonly.java,v 1.6 2008/03/17 16:59:46 wdietl Exp $
 */

package org.multijava.mjc;

/**
 * Implements the readonly universe.
 *
 * @author WMD
 */
public class CUniverseReadonly extends CUniverse {

    protected CUniverseReadonly() {
    	super( );
    }

    /**
     * Any readonly reference equals other readonly or
     * implicit-readonly references.
     */
    public boolean equals(/*@ nullable @*/  Object other) {
    	return other instanceof CUniverseReadonly;
    }

    /**
     * Same hash code for peer and implicit-peer references.
     * 
     * {inheritDoc}
     */
    public int hashCode() {
    	return CUniverseReadonly.class.hashCode();
    }
    
    /**
     * A readonly reference is only assignable to another readonly
     * reference.
     */
    public boolean isAlwaysAssignableTo(CUniverse other) {
    	return other instanceof CUniverseReadonly;
    }

    /**
     * Get an instance of the readonly universe.
     */
    public static CUniverseReadonly getUniverse() {
    	return instance;
    }

    /**
     * Return "readonly".
     */
    public String toString() {
    	return "readonly";
    }

    /**
     * Returns the bytecode constant representing this particular universe.
     */
    public byte getByteConstant() {
        return CUniverseReadonly.byteConstant;
    }

    /**
     * The singleton readonly instance.
     */
    private static final CUniverseReadonly instance = new CUniverseReadonly();
    
    /** The constant used in the bytecode to represent this universe.
     */
    public static final byte byteConstant = 0x3;
}
