/*
 * Copyright (C) 2003-2004 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CUniverseRep.java,v 1.4 2008/03/16 15:52:50 wdietl Exp $
 */

package org.multijava.mjc;

/**
 * Implements the rep universe that specifies the representation of
 * the current object.
 *
 * @author WMD
 */
public class CUniverseRep extends CUniverse {

    protected CUniverseRep() {
    	super();
    }

    /**
     * A rep universe is assignable to another rep universe or 
     * to a readonly reference.
     */
    public boolean isAlwaysAssignableTo(CUniverse other) {
    	return other instanceof CUniverseRep ||
	    		other instanceof CUniverseReadonly;
    }

    /**
     * Get an instance of the rep universe.
     * Right now we simply only use the attributes from the first
     * invocation and return one instance.
     */
    public static CUniverseRep getUniverse() {
    	return instance;
    }

    /**
     * Return "rep".
     */
    public String toString() {
    	return "rep";
    }

    /**
     * Returns the bytecode constant representing this particular universe.
     */
    public byte getByteConstant() {
        return CUniverseRep.byteConstant;
    }

    /**
     * The singleton rep universe instance.
     */
    private static final CUniverseRep instance = new CUniverseRep();

    /**
     * The constant used in the bytecode to represent this universe.
     */
    public static final byte byteConstant = 0x2;
}
