/*
 * Copyright (C) 2003-2007 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.multijava.mjc;

import org.multijava.util.compiler.CompilationAbortedError;

/**
 * Implements a universe modifier used solely as the main modifier for
 * <code>this</code>.
 * 
 * It must not occur in any source and byte code but is only used for the
 * internal representation.
 * 
 * @author RZU
 */
public class CUniverseThis extends CUniverse {

	protected CUniverseThis() {
		super();
	}

	/**
	 * {inheritDoc}
	 * 
	 * So far, we write it as peer since we haven't specified the new
	 * bytecode format for Generic Universe Types yet.
	 */
	public byte getByteConstant() {
		return CUniversePeer.byteConstant;
	}

	/**
	 * {inheritDoc}
	 * 
	 * Can be assigned to peer or readonly.
	 */
	public boolean isAlwaysAssignableTo(CUniverse other) {
		return other instanceof CUniverseThis 
				|| other instanceof CUniversePeer
				|| other instanceof CUniverseReadonly;
	}

	public String toString() {
		return "[this]";
	}
	
    /**
     * Output the name of the universe as it should be used for MJ code.
     */
    public String toMJString() {
    	org.multijava.util.Utils.fail("Outputting the \"this\" Universe should never happen!");
    	return "";
    }

    /**
     * Output the name of the universe as it should be used for JML code.
     */
    public String toJMLString() {
    	org.multijava.util.Utils.fail("Outputting the \"this\" Universe should never happen!");
    	return "";
    }
    
	/**
	 * {inheritDoc}
	 */
	public static CUniverseThis getUniverse() {
		return instance;
	}
	
	/**
	 * Singleton instance
	 */
	private static final CUniverseThis instance = new CUniverseThis();

}
