/*
 * Copyright (C) 2004-2005 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package org.multijava.mjc;

import org.multijava.util.classfile.UniverseByteConstants;

/**
 * This class represents a universe annotation for a type.
 *
 * @author AHS
 */
public class CUniverseTypeAnnotation {
    
    /**
     * Creates a new universe type annotation from a given byte.
     *
     * @param byteConstant a byte constant representing a universe annotation
     */
    public CUniverseTypeAnnotation(byte byteConstant) {
        this.byteConstant = byteConstant;
    }
    
    /**
     * Return whether this is an array annotation.
     */
    public boolean isArrayType() {
        return UniverseByteConstants.isArray(byteConstant);
    }
    
    /**
     * Return the universe represented by this annotation.
     */
    public CUniverse getUniverse() {
        return UniverseByteConstants.toUniverse(byteConstant);
    }
    
    /**
     * Return the universe of the base type (if this is an array).
     */
    public CUniverse getBaseUniverse() {
        return isArrayType() ? UniverseByteConstants.toBaseUniverse(byteConstant)
                             : CUniverseImplicitPeer.getUniverse(); 
    }
    
    /**
     * Return a string representation of the annotation.
     */
    public String toString() {
        if (isArrayType()) {
            return getUniverse().toString() + " " + getBaseUniverse().toString();
        } else {
            return getUniverse().toString();
        }
    }

    /** The byte representation of the Universe. */
    private byte byteConstant;
}
