/*
 * Copyright (C) 2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: ClassCreator.java,v 1.3 2005/04/26 02:39:32 kui_dai Exp $
 */

package org.multijava.mjc;

import java.io.*;
import org.multijava.util.classfile.*;

/**
 * A factory class to create classes.
 */
public class ClassCreator {

    // --------------------------------------------------------------------
    // CONSTRUCTOR
    // --------------------------------------------------------------------

    /** 
     * Creates a new instance. This constructor is private to implement
     * the Singleton pattern.
     */
    protected ClassCreator() {
    }

    // --------------------------------------------------------------------
    // CREATOR METHODS
    // --------------------------------------------------------------------
 
    /**
     * Creates a binary field object. 
     */
    public /*@ non_null @*/ CBinaryField createBinaryField(
        /*@ non_null @*/ CClass owner, /*@ non_null @*/ FieldInfo fieldInfo) {
        return new CBinaryField(owner, fieldInfo);
    }

    /**
     * Creates a binary method object. 
     */
    public /*@ non_null @*/ CBinaryMethod createBinaryMethod(
        CClass owner, MethodInfo methodInfo, CClassContextType declCtx) {
	return new CBinaryMethod(owner, methodInfo, declCtx);
    }

    /**
     * Creates a member access object.
     */
    public /*@ non_null @*/ MemberAccess createMemberAccess(
        CClass owner, CMemberHost host, ClassInfo classInfo) {
        return new MemberAccess(owner, host, (long)classInfo.getModifiers()); 
    }

    // --------------------------------------------------------------------
    // SINGLETON METHOD
    // --------------------------------------------------------------------

    /** Returns the unique intance of this class. */
    public static /*@ non_null @*/ ClassCreator getInstance() {
        return theInstance;
    }

    // --------------------------------------------------------------------
    // DATA MEMBERS
    // --------------------------------------------------------------------

    /** The unique instance of this class. */
    private static final /*@ non_null @*/ ClassCreator theInstance 
        = new ClassCreator();
}
