/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CompilerPassEnterable.java,v 1.13 2006/12/10 22:51:21 chalin Exp $
 */

package org.multijava.mjc;
import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * This is the type of AST nodes that provide methods for handling
 * individual passes of the compilation process.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.13 $ */

public interface CompilerPassEnterable extends Comparable {

    //@ public instance model boolean arePassParametersCached;

    /**
     * Invokes the preprocessDependencies method of the implementing class
     * using previously cached arguments.
     *
     * <pre><jml>
     * requires arePassParametersCached;
     * </jml></pre> */
    void preprocessDependencies() throws PositionedError;

    /**
     * Invokes the checkInterface method of the implementing class
     * using previously cached arguments.
     *
     * <pre><jml>
     * requires arePassParametersCached;
     * </jml></pre> */
    void checkInterface() throws PositionedError;

    /**
     * Invokes the checkInitializers method of the implementing class
     * using previously cached arguments.
     *
     * <pre><jml>
     * requires arePassParametersCached;
     * </jml></pre> */
    void checkInitializers() throws PositionedError;

    /**
     * Resolves value specializer expressions to the compile-time
     * constants they represent.  Must come after the
     * checkInitializers phase.
     */
    void resolveSpecializers() throws PositionedError;
    
    /**
     * Finds the top method of every declared method.  Called after
     * the resolveSpecializers pass.  This cannot be done before then
     * because the external generic function mappings are not complete
     * until the end of the checkInterface pass and the constant value
     * specializers are not known until after the resolveSpecializers
     * pass.  This must be done before the typecheck pass so that all
     * specialized argument positions for generic functions are known
     * for ambiguity checking. */
    void resolveTopMethods() throws PositionedError;

   /**
     * Invokes the typecheck method of the implementing class
     * using previously cached arguments.
     *
     * <pre><jml>
     * requires arePassParametersCached;
     * </jml></pre> */
    void typecheck() throws PositionedError;

    /**
     * Invokes the translateMJ method of the implementing class
     * using previously cached arguments.
     *
     * <pre><jml>
     * requires arePassParametersCached;
     * </jml></pre> */
    void translateMJ();

    TokenReference getTokenReference();

    abstract public void accept(/*@ non_null @*/ MjcVisitor p);
}
