/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Constants.java,v 1.17 2006/02/11 03:05:57 chalin Exp $
 */

package org.multijava.mjc;

import java.util.ArrayList;

/**
 * Defines all constants shared by compiler
 */
public interface Constants extends org.multijava.util.classfile.Constants {

    // ----------------------------------------------------------------------
    // TYPE IDENTIFIER
    // ----------------------------------------------------------------------
    /*@ public invariant // used in, e.g. CNumericType.isAlwaysAssignableTo().
	TID_BYTE  < TID_SHORT &&
	TID_BYTE  < TID_CHAR &&
	TID_SHORT < TID_INT &&
	TID_CHAR  < TID_INT &&
	TID_INT   < TID_LONG &&
	TID_LONG  < TID_FLOAT &&
	TID_FLOAT < TID_DOUBLE;
     @*/
    int TID_VOID	= 1;
    int TID_BYTE	= 2;
    int TID_SHORT	= 3;
    int TID_CHAR	= 4;
    int TID_INT		= 5;

    int TID_LONG	= 6;
    int TID_FLOAT	= 7;
    int TID_DOUBLE	= 8;
    int TID_CLASS	= 9;
    int TID_ARRAY	= 10;
    int TID_BOOLEAN	= 11;

    int TID_MAX		= 99; // above TID_MAX reserved for subinterfaces

    // ----------------------------------------------------------------------
    // ARITHMETHIC MODE IDENTIFIERS
    // ----------------------------------------------------------------------
    
    byte AMID_JAVA_MATH = 0;
    byte AMID_SAFE_MATH = 1;
    byte AMID_MAX		= 16;	// above AMID_MAX reserved for subinterfaces

    // ----------------------------------------------------------------------
    // ACCESS FLAGS
    // ----------------------------------------------------------------------
    // An access flag has 64 bits.
    // - 16 low-order bits (L) are reserved for VM flags.
    // - middle 32 bits (m) for (language level) access/modifier flags.
    // - 16 high-order bits (H) reserved for "internal" flags.
    //
    //   -----------------------	  0xHHHHmmmmmmmmLLLL
    long ACC_MODIFIER_FLAGS_MASK	= 0x0000FFFFFFFFFFFFL;
    long IMPLICITLY_NON_NULL		= 0x0001000000000000L;

    // WMD moved here from org.jmlspecs.checker.Constants
    long ACC_PURE			= 0x00000002L << 16;
    long ACC_NON_NULL_BY_DEFAULT	= 0x00010000L << 16;
    long ACC_NULLABLE_BY_DEFAULT	= 0x00020000L << 16;
    long ACC_NON_NULL			= 0x00040000L << 16;
    long ACC_NULLABLE			= 0x00080000L << 16;

    long NULLITY_MODS			= ACC_NON_NULL | ACC_NULLABLE;

    /**
     * These arrays are used to map flags to names for pretty printing
     * and error messages and to issue style warnings for modifiers
     * out of order. 
     *
     * @see #ACCESS_FLAG_ARRAY
     * @see #ACCESS_FLAG_NAMES
     */
    // WMD taken and extended from util.classfile.Constants
    long[] ACCESS_FLAG_ARRAY = 
	new long[] {
	    ACC_PUBLIC,
	    ACC_PRIVATE,
	    ACC_PROTECTED,
	    ACC_ABSTRACT,
	    ACC_STATIC,
	    ACC_FINAL,
	    ACC_SYNCHRONIZED,
	    ACC_TRANSIENT,
	    ACC_VOLATILE,
	    ACC_NATIVE,
	    ACC_STRICT,
	    ACC_INTERFACE,
	    ACC_PURE,
	    ACC_NON_NULL,
	    ACC_NULLABLE,
 	    ACC_NULLABLE_BY_DEFAULT,
 	    ACC_NON_NULL_BY_DEFAULT
	};

    /**
     * These arrays are used to map flags to names for pretty printing
     * and error messages and to issue style warnings for modifiers
     * out of order. 
     *
     * @see #ACCESS_FLAG_ARRAY
     * @see #ACCESS_FLAG_NAMES
     */
    // WMD taken and extended from util.classfile.Constants
    String[] ACCESS_FLAG_NAMES =
	new String[] {
	    "public",
	    "private",
	    "protected",
	    "abstract",
	    "static",
	    "final",
	    "synchronized",
	    "transient",
	    "volatile",
	    "native",
	    "strictfp",
	    "", // interface flag is suppressed for pretty printing
	    "pure",
	    "non_null",
	    "nullable",
 	    "nullable_by_default",
 	    "non_null_by_default"
	};

    // ----------------------------------------------------------------------
    // COMPILER FLAGS
    // ----------------------------------------------------------------------

    int CMP_VERSION		= 0xC0DE01;

    // ----------------------------------------------------------------------
    // JAVA CONSTANTS
    // ----------------------------------------------------------------------

    String JAV_ASSERTION_ERROR  = "java/lang/AssertionError";
    String JAV_CLASS		= "java/lang/Class";
    String JAV_CLONEABLE	= "java/lang/Cloneable";
    String JAV_ERROR		= "java/lang/Error";
    String JAV_EXCEPTION	= "java/lang/Exception";
    String JAV_OBJECT		= "java/lang/Object";
    String JAV_RUNTIME_EXCEPTION= "java/lang/RuntimeException";
    String JAV_RMJ_RUNTIME_EXCEPTION=
	"org/multijava/relaxed/runtime/RMJRuntimeException";
    String JAV_STRING		= "java/lang/String";
    String JAV_STRINGBUFFER	= "java/lang/StringBuffer";
    String JAV_THROWABLE	= "java/lang/Throwable";
    String JAV_CLASSLOADER	= "java/lang/ClassLoader";
    String JAV_SERIALIZABLE	= "java/io/Serializable";
    String JAV_CLASSNOTFOUND_EXCEPTION = "java/lang/ClassNotFoundException";
    String JAV_NOCLASSDEFFOUND_ERROR = "java/lang/NoClassDefFoundError";

    String JAV_CONSTRUCTOR	= "<init>";
    String JAV_INIT		= "Block$";
    String JAV_STATIC_INIT	= "<clinit>";

    String JAV_THIS		= "this";
    String JAV_OUTER_THIS	= "this$0";
    String JAV_SUPER		= "super";

    String JAV_NAME_SEPARATOR	= "/";
    String JAV_RUNTIME		= "java/lang";
    String JAV_CLONE		= "clone";
    String JAV_LENGTH		= "length";

    // ----------------------------------------------------------------------
    // MULTIJAVA CONSTANTS
    // ----------------------------------------------------------------------

    String MJ_ANCHOR		= "$anchor";

    // ----------------------------------------------------------------------
    // UNIVERSE CONSTANTS by scdaniel
    // ----------------------------------------------------------------------

    String UNIV_TMP		= "$tmp";
    String UNIV_ARRAY_TMP	= "$atmp";

    // ----------------------------------------------------------------------
    // BINARY OPERATORS
    // ----------------------------------------------------------------------

    int OPE_SIMPLE		= 0;
    int OPE_PLUS		= 1;
    int OPE_MINUS		= 2;
    int OPE_STAR		= 3;
    int OPE_SLASH		= 4;
    int OPE_PERCENT		= 5;
    int OPE_SR			= 6;
    int OPE_BSR			= 7;
    int OPE_SL			= 8;
    int OPE_BAND		= 9;
    int OPE_BXOR		= 10;
    int OPE_BOR			= 11;
    int OPE_BNOT		= 12;
    int OPE_LNOT		= 13;
    int OPE_LT			= 14;
    int OPE_LE			= 15;
    int OPE_GT			= 16;
    int OPE_GE			= 17;
    int OPE_EQ			= 18;
    int OPE_NE			= 19;
    int OPE_LAND		= 20;
    int OPE_LOR			= 21;

    // ----------------------------------------------------------------------
    // UNARY OPERATORS
    // ----------------------------------------------------------------------

    int OPE_PREINC		= 22;
    int OPE_PREDEC		= 23;
    int OPE_POSTINC		= 24;
    int OPE_POSTDEC		= 25;
    
}
