/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JBinaryArithmeticExpression.java,v 1.6 2005/05/08 16:56:04 f_rioux Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * This class is an abstract root class for binary expressions.
 */
public abstract class JBinaryArithmeticExpression extends JBinaryExpression {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Constructs a node in the parsing tree.
     * This method is directly called by the parser.
     * @param	where		the line of this node in the source code
     * @param	left		left operand
     * @param	right		right operand
     */
    public JBinaryArithmeticExpression( TokenReference where,
					/*@non_null@*/ JExpression left,
					/*@non_null@*/ JExpression right ) {
	super( where, left, right );
    }

    // ----------------------------------------------------------------------
    // CODE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Typechecks the expression and mutates the context to record
     * information gathered during typechecking.
     *
     * @param context the context in which this expression appears
     * @return  a desugared Java expression (see <code>JExpression.typecheck()</code>)
     * @exception PositionedError if the check fails */
    public /*@non_null@*/ JExpression typecheck(/*@non_null@*/ CExpressionContextType context) 
	throws PositionedError 
    {
	left = left.typecheck( context );
	right = right.typecheck( context );

	JExpression expr = typecheckBinaryArith( context );
	if( expr == this ) {
	    left = left.convertType( type, context );
	    right = right.convertType( type, context );
	}
	return expr;
    }

    /**
     * Typechecks a binary arithmetic expression and handles simple
     * optimizations like constant folding.
     *
     * @param context the actual context of analysis
     * @return  a desugared Java expression (see <code>JExpression.typecheck()</code>)
     * @exception PositionedError if checks fail */
    public abstract /*@non_null@*/ JExpression typecheckBinaryArith(/*@non_null@*/ CExpressionContextType 
						      context ) 
	throws PositionedError;
}
