/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JClassBlock.java,v 1.5 2005/05/09 16:25:36 f_rioux Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.TokenReference;

import org.multijava.javadoc.JavadocComment;

/**
 * This class represents an initializer block in a type declaration.
 * A block is a sequence of statements and local variable declaration
 * statements within braces.  JLS 14.2: Block
 * */
public class JClassBlock extends JBlock 
    implements org.multijava.javadoc.Annotatable
{
    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a node in the parsing tree
     * @param	where		the line of this node in the source code
     * @param	isStatic	is this block a static initializer ?
     * @param	body		a list of statement
     * @param	javadoc		the javadoc comments associated with this, 
     *				or null if none
     */
    public JClassBlock( TokenReference where, 
			boolean isStatic, 
			JStatement[] body,
			JavadocComment javadoc ) {
	super( where, body, null );

	this.isStatic = isStatic;
	this.javadoc = javadoc;
    }

    /**
     * Construct a block with no javadocs annotations.
     * @param	where		the line of this node in the source code
     * @param	isStatic	is this block a static initializer ?
     * @param	body		a list of statement
     */
    public JClassBlock( TokenReference where, 
			boolean isStatic, 
			JStatement[] body ) {
	this( where, isStatic, body, null );
    }


    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * @return	is this block a static initializer
     */
    public boolean isStaticInitializer() {
	return isStatic;
    }


    public void accept( /*@non_null@*/ MjcVisitor p ) {
	p.visitClassBlock( this );
    }

    public JavadocComment javadocComment() {
	return javadoc;
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private final boolean isStatic;

    private final JavadocComment javadoc;
}
