/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JClassDeclarationType.java,v 1.10 2008/08/18 19:20:29 smshaner Exp $
 */

package org.multijava.mjc;

import java.util.Iterator;
import org.multijava.javadoc.JavadocComment;
import org.multijava.util.compiler.CWarning;
import org.multijava.util.compiler.JavaStyleComment;
import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;
import org.multijava.util.compiler.UnpositionedError;

/**
 * This type represents a java class declaration in the syntax tree.
 */
public interface JClassDeclarationType extends JTypeDeclarationType {

    // ----------------------------------------------------------------------
    // INTERFACE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Sets the super class of this class.
     */
    void setSuperClass(CClassType superType);

    /**
     * Sets the list of interfaces that this class implements.
     */
    void setInterfaces(CClassType[] interfaces);

    /**
     * Returns true if this class declaration contains an 
     * explicit constructor declaration. 
     * This method is used by <code>checkInterface</code> 
     * to determine whether or not to create a default 
     * constructor for this type.
     */
    boolean hasConstructor();

    /**
     * Creates a class context for this class declaration.
     * @param	parent		the parent context or null
     * @return	returns a CClassContextType that represents this context
     */
    CClassContextType createContext(CContextType parent);

    String superName();
}
