/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JClassFieldDeclarator.java,v 1.4 2004/01/16 23:21:10 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * JLS 14.5: Field Statement.
 *
 * This class is used as a wrapper around a field declaration when
 * generating initializers.  */
public class JClassFieldDeclarator extends JStatement {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a node in the parsing tree.
     * @param	where		the line of this node in the source code
     * @param	decl		the declarator of this field
     */
    public JClassFieldDeclarator( TokenReference where, 
				  JFieldDeclarationType decl ) {
	super( where, null );
	this.decl = decl;
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Returns this declarator's declaratee.
     */
    public JFieldDeclarationType decl() {
	return decl;
    }
    
    // ----------------------------------------------------------------------
    // CODE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Typechecks this statement in the context in which it
     * appears.  Mutates the context to record the information learned
     * during checking.
     *
     * @param context	the context in which this appears
     * @exception PositionedError if any checks fail */
    public void typecheck( CFlowControlContextType context ) throws PositionedError {
	decl.typecheck( context );

	if( decl.hasInitializer() ) {
	    ((CSourceField) decl.getField())
		.setValue( decl.variable().getValue() );
	}
    }

    /**
     * Accepts the specified visitor
     * @param	p		the visitor
     */
    public void accept(MjcVisitor p) {
	// utility class for classfile
    }

    /**
     * Generates a sequence of bytecodes
     * @param	code		the code list
     */
    public void genCode(CodeSequence code) {
	JVariableDefinition variable = decl.variable();
	boolean varIsNotConstant = 
	    variable.getValue() != null && !variable.getValue().isConstant();
	if( !decl.getField().isFinal() 
	    || varIsNotConstant 
	    || decl.getType().isReference() ) {
	    if( variable.getValue() != null 
		&& !(variable.getValue().isConstant() 
		     && variable.getValue().getLiteral().isDefault()) ) {
		decl.genCode( code );
	    }
	}
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private	JFieldDeclarationType	decl;
}
