/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JClassOrGFImportType.java,v 1.11 2006/12/11 05:55:57 chalin Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.CWarning;
import org.multijava.util.compiler.TokenReference;
import org.multijava.util.compiler.JavaStyleComment;
import org.multijava.util.compiler.PositionedError;

/**
 * This type represents (in the AST) import statements for single
 * classes or generic functions, e.g., <code>import
 * java.util.ArrayList;</code> or <code>import
 * org.multijava.samples.typecheck</code>. */
public interface JClassOrGFImportType {

    // ----------------------------------------------------------------------
    // ACCESSORS & MUTATORS
    // ----------------------------------------------------------------------

    /**
     * @return the class name defined by this statement
     */
    /*@non_null@*/ String getName();

    /**
     * @return the class name defined by this statement
     */
    /*@non_null@*/ String ident();

    /**
     * States that specified class is used.
     */
    void setUsed();

    /**
     * Indicates whether this simple import statement imports a type.
     *
     * @return	true iff this imports a type
     */
    boolean isClassImport();

    /**
     * Indicates whether this simple import statement imports an external
     * generic function.
     *
     * @return	true iff this imports an external generic function
     */
    boolean isGFImport();

    /**
     * Registers whether this imports a type or an external generic function.
     */
    void setImportKind(/*@non_null@*/ FileFinder finder) throws PositionedError;

    // ----------------------------------------------------------------------
    // CODE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Checks whether this class import statement names a class (or
     * generic function) that is used in the source code.  Reports a
     * warning if not.  */
    void typecheck(/*@non_null@*/ Main compiler);

    // ----------------------------------------------------------------------
    // CODE GENERATION
    // ----------------------------------------------------------------------

    /**
     * Accepts the specified visitor
     * @param	p		the visitor
     */
    void accept(/*@non_null@*/ MjcVisitor p);
}
