/*
 * Copyright (C) 2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JExpressionFactory.java,v 1.4 2006/12/11 05:55:57 chalin Exp $
 */

package org.multijava.mjc;

import antlr.*;
import org.multijava.util.compiler.*;

/**
 * Expression AST node factory class.  Factory methods are provided for
 * expressions whose semantics are different in sub-grammars.
 */
public class JExpressionFactory implements MjcTokenTypes {

    public /*@non_null@*/ JUnaryExpression createUnaryExpression(/*@non_null@*/ Token op, 
								 /*@non_null@*/ TokenReference where, 
								 int oper, 
								 /*@non_null@*/ JExpression expr){
    	return new JUnaryExpression(where, oper, expr);
    }

    public /*@non_null@*/ JExpression createAdditiveExpr(/*@non_null@*/ Token op,
							 /*@non_null@*/ TokenReference where,
							 /*@non_null@*/ JExpression left,
							 /*@non_null@*/ JExpression right){
        return op.getType() == PLUS
	    ? (JExpression) createAddExpression(op, where, left, right)
	    : (JExpression) createMinusExpression(op, where, left, right);
    }
    
    public /*@non_null@*/ JAddExpression createAddExpression(/*@non_null@*/ Token tok,
							     /*@non_null@*/ TokenReference where,
							     /*@non_null@*/ JExpression left,
							     /*@non_null@*/ JExpression right){
    	return new JAddExpression(where, left, right);
    }
    
    public /*@non_null@*/ JMinusExpression createMinusExpression(/*@non_null@*/ Token tok,
								 /*@non_null@*/ TokenReference where,
								 /*@non_null@*/ JExpression left,
								 /*@non_null@*/ JExpression right){
    	return new JMinusExpression(where, left, right);
    }
    
    public /*@non_null@*/ JExpression createMultiplicativeExpr(/*@non_null@*/ Token op,
							       /*@non_null@*/ TokenReference where,
							       /*@non_null@*/ JExpression left,
							       /*@non_null@*/ JExpression right){
        return op.getType() == STAR
	    ? (JExpression) createMultExpression(op, where, left, right)
	    : op.getType() == SLASH
	    ? (JExpression) createDivideExpression(op, where, left, right)
	    : (JExpression) createModuloExpression(op, where, left, right);
    }

    public /*@non_null@*/ JMultExpression createMultExpression(/*@non_null@*/ Token op,
							       /*@non_null@*/ TokenReference where,
							       /*@non_null@*/ JExpression left,
							       /*@non_null@*/ JExpression right){
    	return new JMultExpression(where, left, right);
    }
    
    public/*@non_null@*/  JDivideExpression createDivideExpression(/*@non_null@*/ Token op,
								   /*@non_null@*/ TokenReference where,
								   /*@non_null@*/ JExpression left,
								   /*@non_null@*/ JExpression right){
    	return new JDivideExpression(where, left, right);
    }
    
    public /*@non_null@*/ JModuloExpression createModuloExpression(/*@non_null@*/ Token op,
								   /*@non_null@*/ TokenReference where,
								   /*@non_null@*/ JExpression left,
								   /*@non_null@*/ JExpression right){
    	return new JModuloExpression(where, left, right);
    }
    
    public /*@non_null@*/ JShiftExpression createShiftExpression(/*@non_null@*/ TokenReference where,
								 int oper,
								 /*@non_null@*/ JExpression left,
								 /*@non_null@*/ JExpression right){
    	return new JShiftExpression(where, oper, left, right);
    }
    
    public /*@non_null@*/ JBitwiseExpression createBitwiseExpression(/*@non_null@*/ TokenReference where,
								     int oper,
								     /*@non_null@*/ JExpression left,
								     /*@non_null@*/ JExpression right){
    	return new JBitwiseExpression(where, oper, left, right);
    }

}
