/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JFieldDeclarationType.java,v 1.7 2005/05/11 16:47:15 f_rioux Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.CWarning;
import org.multijava.util.compiler.JavaStyleComment;
import org.multijava.javadoc.JavadocComment;
import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * This type represents a field declaration in the syntax tree.
 */
public interface JFieldDeclarationType extends JMemberDeclarationType {

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Returns true if this field declarator has an initializer (should
     * be initialized) */
    boolean hasInitializer();

    /**
     * Returns the type of this field
     */
    CType getType();

    /**
     * Returns true if this field need to be initialized WARNING: this
     * method returns true when initial value corresponds to a default
     * value ====> a second check should be made after typecheck
     * to ensure that an initialization is really needed */
    boolean needInitialization();

    JVariableDefinition variable();

    /** Returns the modifiers of this field declaration */
    long modifiers();

    /** Returns the identifier of this field declaration */
    String ident();

    // ----------------------------------------------------------------------
    // INTERFACE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Checks the basic interfaces to make sure things
     * generally look OK.  This pass gathers information about
     * the type signatures of everything (imported class
     * files, classes being compiled, methods, fields, etc...)
     * needed for the later passes.  This information is stored 
     * in a context hierarchy that is bound to the AST.
     *
     * @exception	PositionedError	an error with reference to the source file
     */
    /*@non_null@*/ CSourceField checkInterface(/*@non_null@*/ CClassContextType context) 
	throws PositionedError;

    // ----------------------------------------------------------------------
    // CODE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Checks whether this field declaration includes a field
     * initializer and mutates the context to store this information
     * about the field.  Records the value of a compile-time constant
     * initializer.
     *
     * @param	context	the context in which this field is declared
     * @exception PositionedError if any checks fail */
    void typecheck( /*@non_null@*/ CFlowControlContextType context ) throws PositionedError;

    // ----------------------------------------------------------------------
    // CODE GENERATION
    // ----------------------------------------------------------------------

    /**
     * Generates a sequence of bytecodes
     * @param	code		the code list
     */
    void genCode(CodeSequence code);

}
