/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JMemberDeclaration.java,v 1.11 2006/12/20 06:15:52 perryjames Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.TokenReference;
import org.multijava.javadoc.JavadocComment;
import org.multijava.util.compiler.JavaStyleComment;


/**
 * This class represents a java declaration in the syntax tree
 */
public abstract class JMemberDeclaration extends JPhylum    
    implements JMemberDeclarationType, org.multijava.javadoc.Annotatable 
{

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a node in the parsing tree
     * This method is directly called by the parser
     * @param	where		the line of this node in the source code
     * @param	javadoc		javadoc comments, including deprecated flag
     */
    public JMemberDeclaration(/*@ non_null */ TokenReference where,
    		/*@ nullable */ JavadocComment javadoc,
			      /*@ nullable */ JavaStyleComment[] comments) {
	super(where);
	this.comments = comments;
	this.javadoc = javadoc;
    }

    // ----------------------------------------------------------------------
    // ACCESSORS (INTERFACE)
    // ----------------------------------------------------------------------

    /**
     * Returns true if this member is deprecated
     */
    public /*@ pure @*/ boolean isDeprecated() {
	return javadoc != null && javadoc.isDeprecated();
    }

    /**
     * @return	the interface
     */
    public /*@ pure @*/ CField getField() {
	return export.getField();
    }

    /**
     * @return	the interface
     */
    public /*@ pure @*/ /*@nullable */ CMethod getMethod() {
	return export == null ? null : export.getMethod();
    }

    /**
     * @return	the interface
     */
    public /*@ pure @*/ CClass getCClass() {
	return export.getCClass();
    }

    /**
     * Accepts the specified visitor
     * @param	p		the visitor
     */
    public abstract void accept(/*@non_null@*/ MjcVisitor p);

    /**
     * Returns an AST of the javadoc documentation comments associated
     * with this member declaration.  The tree may simply be a single
     * leaf if javadoc parsing was not enabled. */
    public /*@ pure @*/ /*@nullable@*/ JavadocComment javadocComment() {
	return javadoc;
    }

    /**
     * Generate the code in pure java form
     * It is useful to debug and tune compilation process
     * @param	p		the printwriter into the code is generated
     */
    public void genComments(/*@non_null@*/ MjcVisitor p) {
//  !FIXME!comments 	if (comments != null) {
//  	    p.visitComments(comments);
//  	}
//  	if (javadoc != null) {
//  	    p.visitJavadoc(javadoc);
//  	}
    }

    // ----------------------------------------------------------------------
    // PROTECTED ACCESSORS
    // ----------------------------------------------------------------------

    protected void setInterface(/*@ non_null @*/ CMember export) {
	this.export = export;
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private /*@ nullable @*/ CMember	export;

    /**
     * An AST of the javadoc documentation comments associated with
     * this member declaration.  The tree may simply be a single leaf
     * if javadoc parsing was not enabled. */
    private	/*@ nullable */ JavadocComment			javadoc;
    private	/*@ nullable */ JavaStyleComment[]		comments;
}
