/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JMemberDeclarationType.java,v 1.9 2006/12/20 14:35:21 perryjames Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.JavaStyleComment;
import org.multijava.util.compiler.PhylumType;
import org.multijava.util.compiler.TokenReference;
import org.multijava.javadoc.JavadocComment;


/**
 * This type represents a java declaration in the syntax tree.
 */
public interface JMemberDeclarationType 
    extends PhylumType, org.multijava.javadoc.Annotatable 
{

    // ----------------------------------------------------------------------
    // ACCESSORS (INTERFACE)
    // ----------------------------------------------------------------------

    /**
     * Returns true if this member is deprecated
     */
    /*@ pure @*/ boolean isDeprecated();

    /**
     * @return	the interface
     */
    /*@ pure non_null @*/ CField getField();

    /**
     * @return	the interface
     */
    /*@ pure @*/ /*@ nullable */ CMethod getMethod();

    /**
     * @return	the interface
     */
    /*@ pure non_null @*/ CClass getCClass();

    /** @return the modifiers of this declaration
     */
    /*@ pure @*/ long modifiers(); 

    /**
     * @return the identifier of the object being declared
     */
    /*@ pure non_null @*/ String ident();

    /**
     * Accepts the specified visitor
     * @param	p		the visitor
     */
    void accept( /*@non_null@*/ MjcVisitor p);

    /**
     * Generate the code in pure java form
     * It is useful to debug and tune compilation process
     * @param	p		the printwriter into the code is generated
     */
    void genComments(/*@non_null@*/ MjcVisitor p);
}
