/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JOuterLocalVariableExpression.java,v 1.4 2004/01/16 23:21:11 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;
import org.multijava.util.compiler.TokenReference;

/**
 * This class is an AST node and represents a reference to a variable
 * of a surrounding lexical context from within an inner class.  */
public class JOuterLocalVariableExpression extends JLocalVariableExpression {

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * Construct a node in the parsing tree.
     * @param where the line of this node in the source code
     * @param var the variable referenced
     * @param outer the outer class that hosts the variable
     */
    public JOuterLocalVariableExpression( TokenReference where,
					  JLocalVariable var,
					  CClass outer ) {
	super(where, var);

	this.outer = outer;
    }

    // ----------------------------------------------------------------------
    // CODE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Typechecks the expression and mutates the context to record
     * information gathered during typechecking.
     *
     * @param context the context in which this expression appears
     * @return  a desugared Java expression (see <code>JExpression.typecheck()</code>)
     * @exception PositionedError if the check fails */
    public JExpression typecheck( CExpressionContextType context ) 
	throws PositionedError 
    {
	// First we have to find the right context
	CContextType body = context.getFlowControlContext();
	while( body.getClassContext().getHostClass() != outer ) {
	    body = body.getClassContext().getParentContext();
	}

	CContextType parent = body.getClassContext().getParentContext();
	CExpressionContextType  ctxt = parent instanceof CExpressionContextType 
	    ? (CExpressionContextType)parent 
	    : ((CFlowControlContextType)parent).createExpressionContext();
	JExpression expr = super.typecheck( ctxt );

	if (! (expr instanceof JLiteral)) {
	    check( context,
		   expr == this && variable().isFinal(),
		   MjcMessages.BAD_LOCAL_NOT_FINAL,
		   variable().ident() );

	    expr = ((CSourceClass)outer)
		.getOuterLocalAccess( 
		   getTokenReference(), 
		   variable(),
		   context.isInConstructor() ?
		   context.getMethodContext().getCMethod() : null)
		.typecheck( context );
	}

	return expr;
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    public CClass		outer;
}
