/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JPackageImportType.java,v 1.7 2004/01/16 23:21:11 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.CWarning;
import org.multijava.util.compiler.TokenReference;
import org.multijava.util.compiler.JavaStyleComment;

/**
 * This type represents (in the AST) full-package import statements,
 * with an asterisk.  For example, <code>import java.util.*;</code>.
 */
public interface JPackageImportType {

    // ----------------------------------------------------------------------
    // ACCESSORS & MUTATORS
    // ----------------------------------------------------------------------

    /**
     * Returns the package name defined by this declaration.
     *
     * @return	the package name defined by this declaration
     */
    String getName();

    /**
     * States that specified class in imported package is used.
     * @param	clazz		the class that is used.
     */
    void setClassUsed(String clazz);

    // ----------------------------------------------------------------------
    // CODE CHECKING
    // ----------------------------------------------------------------------

    /**
     * Checks the use of package import statements.  Issues warnings
     * for things like unused packages being imported or a package
     * import being used when only a small number of classes from the
     * package are actually referenced.  In general these are
     * stylistic issues.  
     * @param	compiler	the compiler calling this method
     * @param	pack		the name of the package being imported */
    void typecheck( Main compiler, JPackageName pack );

    // ----------------------------------------------------------------------
    // CODE GENERATION
    // ----------------------------------------------------------------------

    /**
     * Accepts the specified visitor
     * @param	p		the visitor
     */
    void accept(MjcVisitor p);
}
