header { 
/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1998, 2004 Iowa State University as part of the JML project.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Javadoc.g,v 1.8 2008/03/16 15:52:55 wdietl Exp $
 */

/*
 * These grammars handle the parsing and lexing of javadoc
 * documentation comments.  */

	// common header for lexer and parser
	
	package org.multijava.mjc; 

}

{
	// class preamble for the parser
	// import antlr.ParserSharedInputState;

	import org.multijava.javadoc.JavadocComment;
}

class JavadocParser extends Parser;
options {
	k = 2;
	importVocab = MjcID;
	exportVocab = Javadoc;
	codeGenMakeSwitchThreshold = 2;  // Some optimizations
	codeGenBitsetTestThreshold = 3;
	defaultErrorHandler = false;     // Don't generate parser error handlers
}

// custom parser constructors and members
{
	private static String eol = System.getProperty("line.separator");
}

docComment []
returns [JavadocComment self = null]
{
	String desc = null;
	StringBuffer fullDescription = null;
}
	:
		// get the HTML-formatted description from the comment
		(
			( 
				desc = description[] 
				{
					if( fullDescription == null ) {
						fullDescription = new StringBuffer( desc );
					} else {
						fullDescription.append( desc );
					}
				}
			)*
			JAVADOC_CLOSE
			{
				if (fullDescription != null) {
				    int j = fullDescription.length()-1;
				    while (j>=0 && fullDescription.charAt(j)=='*') --j;
				    fullDescription.setLength(j+1);
				    char c = fullDescription.charAt(j);
				    if (c != '\n' && c != '\r')
					fullDescription.append(eol);
				}

				JavadocComment next = new JavadocComment( 
						fullDescription == null ? 
						"" : fullDescription.toString() );
				self = self == null ? next : self.add( next );
				fullDescription = null;
			}
		)*
		EOF
	;

description[]
returns [String self = null]
	:
		d:REST_OF_LINE
		{ self = d.getText(); }
	;


{
	// class preamble for the lexer
	// import antlr.LexerSharedInputState;
}

class JavadocLexer extends Lexer;

options {
    k = 3;
    charVocabulary = '\3'..'\377';
    importVocab=Javadoc;
    testLiterals=false;     // don't automatically test for literals
}

{
	private static String eol = System.getProperty("line.separator");

	public JavadocLexer( ParsingController parsingController ) {
		this( parsingController.sharedInputState() );
		this.parsingController = parsingController;
	}

	/**
	 * Called by the scanner when a tab is detected in the input
	 * stream.  Advances the column number according to tabSize.
	 *
	 * @see	#setTabSize(int)
	 * @see #tabSize() */
	public void tab() {
	    setColumn( getColumn() + tabSize );
	}
	
	/**
	 * Sets the amount by which the column number is incremented when
	 * a tab character is detected in the input stream. 
	 *
	 * @see #tabSize() */
	public void setTabSize( int ts ) { tabSize = ts; }

	/**
	 * Indicates the amount by which the column number is incremented
	 * when a tab character is detected in the input stream.
	 *
	 * @see #setTabSize(int) */
	public int tabSize() { return tabSize; }

	/**
	 * The amount by which the column number is incremented when a tab
	 * character is detected in the input stream.  */
	private int tabSize = 8;

	/**
	 * Used to switch lexers.  Aliases should be established in all
	 * lexers that share the same input stream. */
	private ParsingController parsingController;

	/**
	 * Used to convert token text from String to char[]. */
	private char[] charCache = null;
}


DOC_NL_WS
    : 
		NEWLINE
		(NON_NL_WS)*
		( 
			{LA(2) != '/'}? '*'
		)*
		{$setType(Token.SKIP);}
    ;

// will be ambiguous if called like ( NEWLINE | ... ) * because '\r'
// '\n' can be matched in one alternative or by matching '\r' in one
// iteration and '\n' in another.
protected
NEWLINE
	:
		(
			options {
				generateAmbigWarnings=false;
			}
		:
			'\r' '\n'
		|	'\r'
		|	'\n'
		)
		{newline();}
	;

protected
NON_NL_WS
	:
		( ' ' | '\t' | '\f' )
	;


REST_OF_LINE
	:
        ~('*'|'\r'|'\n')
        (
		{LA(2) != '/'}? '*'
	|
		~('*'|'\r'|'\n') 
	)*	
        { if (LA(1)=='\n' || LA(1) == '\r') $append(eol); } 
	;

/* Implementation note:  In Javadoc the symbol equivalent to
  JAVADOC_CLOSE below can have one or more * characters before the
  final /.  If we implement the lexer that way we get ambiguity
  because of the unbounded lookahead.  It is simpler to simply remove
  those extra *s at the end of the full javadoc comment after the
  commment has been parsed.  However we also need to keep *s that
  occur at the end of a line, while getting rid of those that are
  at the end of the full comment.  To do that we move the appending of the
  newline into the REST_OF_LINE token rather than adding it in the
  description nonterminal.  This does have the effect that the very final
  line may not have an ending newline (if the javadoc comment does not).
*/

// This rule is here to ignore *s at the very beginning of a Javadoc comment
// (after the initial /**).
STAR:
	"*"
	{$setType(Token.SKIP);}
	;

JAVADOC_CLOSE
	:	
		"*/" 
		{ 
			parsingController.pop(); 
		}
	;

