/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MJTopLevelDeclaration.java,v 1.4 2004/01/16 16:26:52 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.PositionedError;

/**
 * This interface is implemented by all classes that represent top-level
 * declarations in MultiJava, i.e., all open-class declarations.
 */

public interface MJTopLevelDeclaration {
    /**
     * Accepts the specified visitor
     * @param	p		the visitor
     */
    void accept(MjcVisitor p);
    

    /**
     * Returns the type that this open member extends
     * @return	a class representing the type extended by this open member
     */
    CType getOpenClassType();

    /**
     * Indicates whether this member is external.
     * @return	a flag indicating whether this member is external
     */
    boolean isExternal();

    /**
     * Checks the basic interfaces to make sure things
     * generally look OK.  This pass gathers information about
     * the type signatures of everything (imported class
     * files, classes being compiled, methods, fields, etc...)
     * needed for the later passes.  This information is stored 
     * in a context hierarchy that is bound to the AST.
     *
     * @return	a representation of the member iff no errors are detected
     * @exception	PositionedError		if any checks fail
     */
    CMember checkInterface(CContextType context) throws PositionedError;


}

