header { 
/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Mjc.g,v 1.68 2008/05/14 18:16:32 wdietl Exp $
 */

/*
 * Grammar based on:
 * Java 1.2 Recognizer
 *
 * Contributing authors:
 *        John Mitchell        johnm@non.net
 *        Terence Parr        parrt@magelang.com
 *        John Lilley        jlilley@empathy.com
 *        Scott Stanchfield    thetick@magelang.com
 *        Markus Mohnen           mohnen@informatik.rwth-aachen.de
 *        Peter Williams        pwilliams@netdynamics.com
 *
 * Version 1.00 December 9, 1997 -- initial release
 * Version 1.01 December 10, 1997
 *        fixed bug in octal def (0..7 not 0..8)
 * Version 1.10 August 1998 (parrt)
 *        added tree construction
 *        fixed definition of WS,comments for mac,pc,unix newlines
 *        added unary plus
 * Version 1.11 (Nov 20, 1998)
 *        Added "shutup" option to turn off last ambig warning.
 *        Fixed inner class def to allow named class defs as statements
 *        synchronized requires compound not simple statement
 *        add [] after builtInType DOT class in primaryExpression
 *        "const" is reserved but not valid..removed from modifiers
 * Version 1.12 (Feb 2, 1999)
 *        Changed LITERAL_xxx to xxx in tree grammar.
 *        Updated java.g to use tokens {...} now for 2.6.0 (new feature).
 *
 * Version 1.13 (Apr 23, 1999)
 *        Didn't have (stat)? for else clause in tree parser.
 *        Didn't gen ASTs for interface extends.  Updated tree parser too.
 *        Updated to 2.6.0.
 * Version 1.14 (Jun 20, 1999)
 *        Allowed final/abstract on local classes.
 *        Removed local interfaces from methods
 *        Put instanceof precedence where it belongs...in relationalExpr
 *            It also had expr not type as arg; fixed it.
 *        Missing ! on SEMI in classBlock
 *        fixed: (expr) + "string" was parsed incorrectly (+ as unary plus).
 *        fixed: didn't like Object[].class in parser or tree parser
 * Version 1.15 (Jun 26, 1999)
 *        Screwed up rule with instanceof in it. :(  Fixed.
 *        Tree parser didn't like (expr).something; fixed.
 *        Allowed multiple inheritance in tree grammar. oops.
 * Version 1.16 (August 22, 1999)
 *        Extending an interface built a wacky tree: had extra EXTENDS.
 *        Tree grammar didn't allow multiple superinterfaces.
 *        Tree grammar didn't allow empty var initializer: {}
 *
 * The original grammar was in the PUBLIC DOMAIN
 *
 * BUGS
 *
 */

    // common parser/lexer header
    package org.multijava.mjc; 

    import org.multijava.util.compiler.*;
    // import java.io.Reader;
}

{
    // Parser class preamble
    import java.util.ArrayList;
    
    import org.multijava.javadoc.JavadocComment;
    import org.multijava.util.Utils;
    

    import antlr.CharScanner;
}

// ----------------------------------------------------------------------
// THE PARSER STARTS HERE
// ----------------------------------------------------------------------

class MjcParser extends Parser;
options {
    k = 2;                           // two token lookahead
    importVocab = MjcID;
    exportVocab = Mjc;
    codeGenMakeSwitchThreshold = 2;  // Some optimizations
    codeGenBitsetTestThreshold = 3;
    defaultErrorHandler = false;     // Don't generate parser error handlers
}

{
    public MjcParser( Main compiler, TokenStream lexer,
        ParsingController parsingController,
        boolean allowGeneric,
        boolean allowMultiJava, boolean allowRelaxedMultiJava, 
        boolean allowUniverseKeywords, // WMD
        boolean parseJavadocs ) {
        this( lexer ); 
        this.utility = 
        new ParserUtility( compiler, parsingController, allowGeneric,
            allowMultiJava, allowRelaxedMultiJava, allowUniverseKeywords, parseJavadocs );
    }

    /**
     * Returns an exception from our exception hierarchy by wrapping
     * the ANTLR excepton.  The exception is really positioned, by the
     * positioning information is embedded in the message. */
    public PositionedError beautifyParserError( ANTLRException e ) {
        return utility.beautifyParserError( e );
    }

    /**
     * Generates the appropriate data structure for buffering class
     * members during parsing.  Implemented as a method for
     * sub-grammars can use a different data structure (of the same
     * name).  This is a hack that takes advantage of ANTLR's textual
     * grammar inheritance.  */
    private CParseClassContext getParseClassContext() {
        return CParseClassContext.getInstance();
    }



    
    // --------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // --------------------------------------------------------------------

    /**
     * Instances of MjcParser delegate to utility.  */
    private ParserUtility utility;

    private JExpressionFactory exprFactory = new JExpressionFactory();

	/**
	 * Counts the number of LT seen in the typeArguments production.
	 * It is used in semantic predicates to ensure we have seen
	 * enough closing '>' characters. 
	 */
	private int unmatchedTypeLT = 0;
}

// Compilation Unit: In Java, this is a single file.  This is the start
//   rule for this parser
jCompilationUnit []
returns [JCompilationUnitType self = null]
{
    JPackageName pack = JPackageName.UNNAMED;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    CParseCompilationUnitContext context =
        CParseCompilationUnitContext.getInstance();
}
    :
        // A compilation unit starts with an optional package definition
        ( pack = jPackageDefinition[] )?

        { context.setPackage(pack); }
        
        // Next we have a series of zero or more import statements
        ( jImportDefinition[context] )*
        

        // Wrapping things up with any number of top-level
        // definitions (Java classes or interfaces or MultiJava top-level
        // declarations)
        ( mjTopLevelDefinition[context] )*
        
        EOF
        {
            self = new JCompilationUnit(
                sourceRef,
                pack,
                context.compilationUnitExport(),
                context.getPackageImports(),
                context.getSingleImports(),
                context.getTypeDeclarations(), 
                context.getMJTopLevelMethodDeclarations()
            );
            context.release();
        }
    ;

// Package statement: "package" followed by an identifier.
jPackageDefinition []
returns [JPackageName self = null]
{
    String    name = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        "package" name = jIdentifier[] e:SEMI
        {
            self = new JPackageName(sourceRef, 
                name, utility.getStatementComment()); 
            utility.flushJavadocTokensWithWarning( e );
        }
    ;

// Import statement: import followed by a package, class, or generic 
// function name
jImportDefinition [CParseCompilationUnitContext cunit]
{
    StringBuffer    buffer = null;
    boolean    star = false;
    String    name = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        "import" i:IDENT  
        (
            // CONFLICT: ANTLR thinks this is ambiguous, apparently because
            //           it doesn't recognize that IDENT and STAR are 
            //           disjoint? !FIXME! if possible.
            options { warnWhenFollowAmbig = false; } :
            DOT j:IDENT
            { (buffer == 
                    null ? (buffer = new StringBuffer(i.getText())) : buffer)
                .append('/').append(j.getText()); 
            }
        )*
        { name = buffer == null ? i.getText() : buffer.toString(); }
        ( DOT STAR { star = true; } )?
        e1:SEMI
        {
            if (star) {
                cunit.addPackageImport(
                    new JPackageImport(sourceRef, 
                        name, utility.getStatementComment()));
            } else {
                JClassOrGFImportType importee = 
                new JClassOrGFImport(sourceRef, 
                    name, utility.getStatementComment());
                cunit.addSingleImport( importee );
            }
            utility.flushJavadocTokensWithWarning( e1 );
        }
    ;

// top-level definitions (Java classes or interfaces or MultiJava
// top-level declarations)
mjTopLevelDefinition [CParseCompilationUnitContext context]
{
    long mods = 0;                // a bit mask indicating the modifiers  
    Token startToken = LT(1);
}
    :
        mods = jModifiers[]
        ( 
            jTypeDefinition[context, mods, startToken]  
        | 
            {utility.allowMultiJava}?
            mjTopLevelDeclaration[context, mods, startToken]  
        )
    ;

// A type definition in a file is either a class or interface definition.
jTypeDefinition [CParseCompilationUnitContext context,
    long mods, Token startToken] 
{
    JTypeDeclarationType    decl = null;
    TokenReference sourceRef = utility.buildTokenReference( startToken );
}
    :
        (
            decl = jClassDefinition[mods,startToken]
        |
            decl = jInterfaceDefinition[mods,startToken]
        )
        { context.addTypeDeclaration( utility.getCompiler(), decl ); }
    |
        s:SEMI
        { 
            utility.reportTrouble(
                new CWarning( sourceRef,
                    MjcMessages.STRAY_SEMICOLON, null )); 
            utility.flushJavadocTokensWithWarning( s );
        }
    ;

// A MultiJava top-level declaration in a file is a top-level method
// declaration
mjTopLevelDeclaration [CParseCompilationUnitContext context, 
    long mods, Token startToken] 
{
    MJTopLevelMethodDeclaration    mDecl = null;
}
    :
        mDecl = mjTopLevelMethodDeclaration[mods, startToken]
        { context.addMJTopLevelMethodDeclaration(utility.getCompiler(), mDecl); }
    ;

/** A declaration is the creation of a reference or primitive-type variable
 *  Create a separate Type/Var tree for each var in the var list.
 */
jDeclaration []
returns [JVariableDeclarationStatement self = null]
{
    long         mods; 
    CType        type;
    JVariableDefinition[] decl;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        mods = jModifiers[] 
        type = jTypeSpec[] 
        decl = jVariableDefinitions[mods, type] 
        { 
            if ( Utils.hasOtherFlags(mods, Constants.ACC_FINAL) ) {
		        utility.reportTrouble(
			        new PositionedError(sourceRef,
			            MjcMessages.INVALID_LOCAL_MODIFIER, 
                        utility.getModifierNames(mods))); 
            }
            self = new JVariableDeclarationStatement(
                sourceRef, decl, utility.getStatementComment() );
        }
    ;

// A list of zero or more modifiers.  We could have used (modifier)* in
//   place of a call to modifiers, but I thought it was a good idea to keep
//   this rule separate so they can easily be collected in a ArrayList if
//   someone so desires
jModifiers []
returns [long self = 0] 
{
    long        mod; 
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        ( 
            // subgrammar includes rules that can call jModifiers
            // twice in succession, we just greedily match using the
            // first jModifiers call
            options { greedy = true; } :
            mod = jModifier[]
            {
                if ((mod & self) != 0) {
                    utility.reportTrouble(
                        new PositionedError(sourceRef,
                            MjcMessages.DUPLICATE_MODIFIER,
                            utility.getModifierName(mod))); 
                }

                if (!utility.modifiersInPreferredOrder(self, mod)) { 
                    utility.reportTrouble(
                        new CWarning(sourceRef,
                            MjcMessages.MODIFIER_ORDER,
                            utility.getModifierName(mod))); 
                }
                self |= mod;
            }
        )*
    ;


// A type specification is a type name with possible brackets afterwards
//   (which would make it an array type).
jTypeSpec []
returns [CType self = null]
    :
        self = mjTypeSpec[]
    ;

// WMD factored out to allow inheritance in JML
mjTypeSpec []
returns [CType self = null]
{
    CUniverse array_univ = null;
    CUniverse elem_univ = null;  // WMD
}
    :
        self = jClassTypeSpec[null, null]
    |
        self = jBuiltInTypeSpec[null,null]
    |
        // WMD
        elem_univ = jUniverseSpec[] self = jClassTypeSpec[elem_univ, null]
    |
        // WMD
        array_univ = jUniverseSpec[] self = jBuiltInTypeSpec[null, array_univ]
    |
        // WMD
        array_univ = jUniverseSpec[] elem_univ = jUniverseSpec[]
        self = mjTypeSpecArray[elem_univ, array_univ]
    ;

// WMD added this construct, so that with a lookahead of 2 we can
// still parse arrays with two modifiers correctly.
mjTypeSpecArray [CUniverse elem_univ, CUniverse array_univ]
returns [CType self = null]
{
    int bounds = 0 ;
}
    :
        self = jClassTypeSpec[elem_univ, array_univ]
    |
        self = jBuiltInTypeSpec[elem_univ, array_univ]
    ;



// A class type specification is a class type with possible brackets afterwards
//   (which would make it an array type).
// jClassTypeSpec []
// returns [CType self = null]
// {
//     String        name = null;
//     int            bounds = 0;
// }
//     :
//         name = jIdentifier[]
//         { self = CTopLevel.getTypeRep(name,false); }
//         ( 
//             LBRACK RBRACK { bounds += 1; } 
//         )*
//         {
//             if (bounds > 0) {
//                 self = new CArrayType(self, bounds);
//             }
//         }
//     ;

jWildCard []
returns [CClassType self = null]
{
    CClassType bound = null;
}
    :
        QUESTION
        {
            self = CWildcardType.createWildcard();
        }
    |
        QUESTION "extends" bound = jWildcardBound[]
        {
    	    self = CWildcardType.createUpperBoundedWildcard(bound);
        }
    |
        QUESTION "super" bound = jWildcardBound[]
        {
    	    self = CWildcardType.createLowerBoundedWildcard(bound);
        }
    ;

jWildcardBound []
returns [CClassType self = null]
{
    int bounds = 0;
    CType type;
    CUniverse elem_univ = null;  // WMD
}
    :
    	self = jClassTypeSpec[null, null]
    |
        // WMD
        elem_univ = jUniverseSpec[] self = jClassTypeSpec[elem_univ, null]
    	// WMD TODO do we need array Universe modifier below?
    |
        type = jBuiltInType[] ( LBRACK RBRACK { bounds += 1; } )+
        {
            self = new CArrayType(type, bounds, null);
        }
    ;

jClassTypeSpec [CUniverse elem_univ, CUniverse array_univ]
returns [CClassType self = null]
{
    int bounds = 0 ;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        self = jTypeName[elem_univ]
        ( 
            LBRACK RBRACK { bounds += 1; } 
        )*
        {
            // WMD: added the additional checks for the universe modifiers
            if (bounds > 0) {
                if( elem_univ != null && array_univ == null ) {
                    array_univ = CUniverseImplicitPeer.getUniverse();

		    // removed the warning here. defaulting/propagation
		    // will find a better value later.
                    // utility.reportTrouble(new CWarning(sourceRef, 
                    //    CUniverseMessages.ARRAY_WITH_NO_UNIVERSE,
                    //    null));
                }

                self = new CArrayType(self, bounds, array_univ);
            } else {
                if( elem_univ != null && array_univ != null &&
                    // ignore if the array universe was implicitly added
                    // this makes the cast expression easier
                    !(array_univ instanceof CUniverseImplicitPeer) ) {
                    utility.reportTrouble(
                         new PositionedError(
                         sourceRef, 
                         CUniverseMessages.NONARRAY_WITH_TWO_UNIVERSES_FORBIDDEN, null));

                }
            }
        }
    ;

jTypeName [CUniverse elem_univ]
returns [CClassType self = null]
{
    String        name = null;
    CClassType[]  typeParameters = null;
    CClassType[][] allTypeParameters ;
    ArrayList vect = new ArrayList();
    StringBuffer buffer = null;
}
    :
        //name = jIdentifier[]
        i : IDENT
        (typeParameters = jParameterizedClassTypeList[])?
        {
            vect.add(typeParameters);
            typeParameters = null;
        }
        (
            DOT j:IDENT ( typeParameters = jParameterizedClassTypeList[] )?
            {
                vect.add(typeParameters);
                typeParameters = null;
                (buffer == null 
                    ? (buffer = new StringBuffer(i.getText())) : buffer)
                .append('/').append(j.getText()); 
            }   
            
        )*
        { //need to be change later
            name = buffer == null ? i.getText() : buffer.toString();
            allTypeParameters = (CClassType[][])vect.toArray(new CClassType[vect.size()][]);
            self = CTopLevel.getTypeRep(name,elem_univ,allTypeParameters,false);
        }
       
    ;

jTypeParameter[]
returns [CClassType self = null]
{
	CUniverse main_univ = null;
	CUniverse elem_univ = null;
    int bounds = 0;
    CType type;
}
    :
    		self = jClassTypeSpec[null, null]
    	|
	        elem_univ = jUniverseSpec[] self = jClassTypeSpec[elem_univ, null]
	    |
	        main_univ = jUniverseSpec[] elem_univ = jUniverseSpec[]
	        self = jClassTypeSpec[elem_univ, main_univ]
		|
    		(main_univ = jUniverseSpec[])?
    		type = jBuiltInType[] ( LBRACK RBRACK { bounds += 1; } )+
    		{
        		self = new CArrayType(type, bounds, main_univ);
    		}
		|
        	self = jWildCard[]
    ;

jParameterizedClassTypeList []
returns [CClassType[] self = null]
{
    CType typeParameter =null;
    ArrayList vect = new ArrayList();
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
:
        // WMD: do we want to allow universes for parameterized types???

        LT {utility.setUnmatchedTypeLT(unmatchedTypeLT++);}
        typeParameter = jTypeParameter[] {vect.add(typeParameter);}
        (
            COMMA typeParameter = jTypeParameter[] {vect.add(typeParameter);}
        )*
        { 
            self =(CClassType[])vect.toArray(new CClassType[vect.size()]);
        }
        GT {utility.setUnmatchedTypeLT(unmatchedTypeLT--);}
        {
            if (!utility.allowGeneric) {
                utility.reportTrouble(new PositionedError(sourceRef,MjcMessages.UNSUPPORTED_GENERIC_TYPE,null));
            }        
        }
    ;
   
jTypeVariableDeclarationList []
returns [CTypeVariable[] self= CTypeVariable.EMPTY]
{
    CTypeVariable tv = null;
    ArrayList  vect = new ArrayList();
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}   
:
        LT {utility.setUnmatchedTypeLT(unmatchedTypeLT++);}
        tv = jTypeVariableDeclaration[] { vect.add(tv); }
        (
            COMMA
            tv = jTypeVariableDeclaration[]  { vect.add(tv); }
        )*
        {
            self = (CTypeVariable[])vect.toArray(new CTypeVariable[vect.size()]);
            for (int i =0; i< self.length; i++) {
                self[i].setIndex(i);
            }
        }
        GT {utility.setUnmatchedTypeLT(unmatchedTypeLT--);}
        {
            if (!utility.allowGeneric) {
                utility.reportTrouble(new PositionedError(sourceRef, MjcMessages.UNSUPPORTED_GENERIC_TYPE, null));
            }
        }
    ;

jTypeVariableDeclaration []
  returns [CTypeVariable self = null]
{
    String name = null;
    // RZU - Readonly is the default upper bound for type variables
    CUniverse main_univ = CUniverseImplicitReadonly.getUniverse();
    CClassType     bound = null;
    ArrayList vect = new ArrayList();
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
:
        ident : IDENT
        (
            "extends"
            (main_univ = jUniverseSpec[])?
            bound = jTypeName[main_univ]
            {
            	vect.add(bound);
            	main_univ = CUniverseImplicitReadonly.getUniverse();
        	}
            (BAND) => (
            	BAND
            	(main_univ = jUniverseSpec[])?
            	bound = jTypeName[main_univ]
            	{
            		vect.add(bound);
            		main_univ = CUniverseImplicitReadonly.getUniverse();
        		}
            )*
        )?
        {
        	// RZU - If there is no upper bound, we default to any Object
        	//       If there are multiple upper bounds, all have to be
        	//       assignable to the first and we change them to be the
        	//       same as the first
        	if (vect.size() == 0) {
        		vect.add(CTopLevel.getTypeRep(Constants.JAV_OBJECT,
        				CUniverseImplicitReadonly.getUniverse(),
        				CClassType.EMPTY_ARG,
        				true));
        	} else if (vect.size() > 1) {
        		CUniverse firstModifier = ((CClassType) vect.get(0)).getCUniverse();
        		for (int i = 1; i < vect.size(); i++) {
        			CClassType curBound = (CClassType) vect.get(i);
        			if (!firstModifier.isAlwaysAssignableTo(curBound.getCUniverse())) {
        				utility.reportTrouble(new PositionedError(
        					sourceRef,
							CUniverseMessages.MULTIPLE_BOUNDS_INCOMPATIBLE,
							ident.getText()));        				
        			} else {
        				curBound.setCUniverse(firstModifier);
        			}
        		}
        	}
        	
            self = new CTypeVariable(ident.getText(), 
                (CClassType[])vect.toArray(new CClassType[vect.size()]));
        }
    ;

// An external class type specification is a class type used to
// indicate the receiver class for an external method declaration.
// This needs to be a separate production because we need to look-ahead
// to avoid including the method identifier in the class type.
mjExternalClassTypeSpec []
returns [CType self = null]
{
    String        name = null;
    StringBuffer    nameBuf = null;
    CClassType[] typeParameters = null;
    CClassType[][] allTypeParameters ;
    ArrayList vect = new ArrayList();
}
    :
        i:IDENT  
        (typeParameters = jParameterizedClassTypeList[])?
        {
            vect.add(typeParameters);
            typeParameters = null;
        }
        (
            (DOT IDENT LPAREN) => DOT // don't read the last identifier 
                                // before the paren
        |
            DOT j:IDENT (typeParameters = jParameterizedClassTypeList[])?
            { 
                vect.add(typeParameters);
                typeParameters = null;
                (nameBuf == null 
                    ? (nameBuf = new StringBuffer(i.getText())) : nameBuf)
                .append('/').append(j.getText()); }
        )*
        { 
            name = nameBuf == null ? i.getText() : nameBuf.toString();
            allTypeParameters = (CClassType[][])vect.toArray(new CClassType[vect.size()][]);
            self = CTopLevel.getTypeRep(name,allTypeParameters,false);
        }
    ;

// A builtin type specification is a builtin type with possible brackets
// afterwards (which would make it an array type).
jBuiltInTypeSpec [CUniverse elem_univ, CUniverse array_univ]
returns [CType self = null]
{
    int            bounds = 0;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        self = jBuiltInType[]
        ( 
            LBRACK RBRACK { bounds += 1; } 
        )*
        {
            if (bounds > 0) {
                if ( bounds==1 ) {
                    if (elem_univ!=null ) {
                      // one-dim arrays of primitive types
                      // only take one ownership modifier
                      utility.reportTrouble(
                         new PositionedError(
                         sourceRef, 
                         CUniverseMessages.ONEDIM_BUILTIN_ARRAY_WITH_TWO_UNIVERSES_FORBIDDEN, null));
                    }
                    self = new CArrayType(self, bounds, array_univ);
                } else {
                    if (array_univ!=null && elem_univ==null) {
                        // switch modifiers: if there is only one, it is parsed
                        // as array modifier, but we want it to be the element modifier
                        self = new CArrayType(self, bounds, null, array_univ);
                    } else {
                        self = new CArrayType(self, bounds, array_univ, elem_univ);
                    }
                }
            } else {
                // WMD: no universe spec allowed if its just a basic type
                // this gives an error, if the array brackets are after
                // the identifier, e.g. rep int len[] = ... gives an error
                // but this shouldn't restrict usage...
                if( array_univ != null ||
                    elem_univ != null ) {
                    utility.reportTrouble(
                         new PositionedError(
                         sourceRef, 
                         CUniverseMessages.BUILTIN_WITH_UNIVERSE_FORBIDDEN, null));
                }
            }
        }
    ;


// WMD
// this is only used for the new statement I think, and then
// we already get the universe for the element type.
// I tried parsing it like for the jTypeSpec, but that created
// ambiguities...
jType [CUniverse elem_univ]
returns [CType type = null]
{
    String    name;
}
    :
        type = jBuiltInType[]
    |
      //  name = jIdentifier[]
      //  { type = CTopLevel.getTypeRep(name,false); }
        type = jTypeName[elem_univ]
    ;



// WMD
jUniverseSpec []
returns [CUniverse universe = null]
    :
        universe = jUniversePeerSpec[]
    |
        universe = jUniverseRepSpec[]
    |
        universe = jUniverseReadonlySpec[]
    ;


// WMD
// The jUniverseType is used in "new ..." expressions.
// Only Peer and Rep are allowed there.
jUniverseType []
returns [CUniverse universe = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        universe = jUniversePeerSpec[]
    |
        universe = jUniverseRepSpec[]
    |
        universe = jUniverseReadonlySpec[]
        {
            utility.reportTrouble(
                new PositionedError(
                    sourceRef, 
                    CUniverseMessages.READONLY_FORBIDDEN_HERE, null));
        }
    ;

// WMD
jUniverseExceptionSpec []
returns [CUniverse universe = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        universe = mjUniverseImplicitReadonlySpec[]
    |
        universe = jUniverseReadonlySpec[]
    ;


// WMD
jUniversePeerSpec []
returns [CUniverse universe = null]
    :
        {utility.allowUniverseKeywords}?
        universe = mjUniversePeerSpec[]
    ;

// WMD
mjUniversePeerSpec []
returns [CUniverse universe = null]
    :
        "peer"
        {
            // explicit peer reference
            universe = CUniversePeer.getUniverse(); 
        }
    ;

// WMD
mjUniverseImplicitPeerSpec []
returns [CUniverse universe = null]
    :
        {
            // implicit peer reference
            universe = CUniverseImplicitPeer.getUniverse(); 
        }
    ;


// WMD
jUniverseRepSpec []
returns [CUniverse universe = null]
    :
        {utility.allowUniverseKeywords}?
        universe = mjUniverseRepSpec[]
    ;

// WMD
mjUniverseRepSpec []
returns [CUniverse universe = null]
    :
        "rep"
        {
            // simple rep reference
            universe = CUniverseRep.getUniverse();
        }
    ;


// WMD
jUniverseReadonlySpec []
returns [CUniverse universe = null]
    :
        universe = mjUniverseReadonlySpec[]
    ;

// WMD
mjUniverseReadonlySpec []
returns [CUniverse universe = null]
    :
    	// WMD tried combining these two and using ("readonly" | "any")
    	// below, but for invalid input the error message was bad.
    	// Also, moving the keyword check to earlier does not work.
	{utility.allowUniverseKeywords}?
        "readonly"
        {
            universe = CUniverseReadonly.getUniverse();
        }
    |
    	{utility.allowUniverseKeywords}?
        "any"
        {
            universe = CUniverseReadonly.getUniverse();
        }
    ;

// WMD
mjUniverseImplicitReadonlySpec []
returns [CUniverse universe = null]
    :
        {
            universe = CUniverseImplicitReadonly.getUniverse();
        }
    ;



// all primitive types, broken out as a separate rule so
// subgrammars can override this but inherit mjBuiltInType
jBuiltInType []
returns [CType self = null]
    :
        self = mjBuiltInType[]
    ;

// !!!JLS The primitive types.
mjBuiltInType []
returns [CType self = null]
    :
        "boolean"    { self = CStdType.Boolean; }
    |
        "byte"        { self = CStdType.Byte; }
    |
        "char"         { self = CStdType.Char; }
    |
        "double"     { self = CStdType.Double; }
    |
        "float"     { self = CStdType.Float; }
    |
        "int"         { self = CStdType.Integer; }
    |
        "long"         { self = CStdType.Long; }
    |
        "short"     { self = CStdType.Short; }
    |
        "void"         { self = CStdType.Void; }
    ;

// A (possibly-qualified) java identifier.  We start with the first IDENT
//   and expand its name by adding dots and following IDENTS
jIdentifier []
returns [String self = null]
{
    StringBuffer buffer = null;
}
    :
        i:IDENT  
        (
            DOT j:IDENT
            { 
                (buffer == null 
                    ? (buffer = new StringBuffer(i.getText())) : buffer)
                .append('/').append(j.getText()); }
        )*
        { 
            self = buffer == null ? i.getText() : buffer.toString(); 
        }
    ;

// all modifiers, broken out as a separate rule so subgrammars can
// override this but inherit mjModifier
jModifier []
returns [long self = 0]
    :
        self = mjModifier[]
    ;

// modifiers for Java classes, interfaces, class/instance vars and methods
mjModifier []
returns [long self = 0]
    :
        "public" 
        { self = org.multijava.mjc.Constants.ACC_PUBLIC; }
    |
        "protected" 
        { self = org.multijava.mjc.Constants.ACC_PROTECTED; }
    |
        "private" 
        { self = org.multijava.mjc.Constants.ACC_PRIVATE; }
    |
        "static" 
        { self = org.multijava.mjc.Constants.ACC_STATIC; }
    |
        "abstract" 
        { self = org.multijava.mjc.Constants.ACC_ABSTRACT; }
    |
        "final" 
        { self = org.multijava.mjc.Constants.ACC_FINAL; }
    |
        "native" 
        { self = org.multijava.mjc.Constants.ACC_NATIVE; }
    |
        "synchronized" 
        { self = org.multijava.mjc.Constants.ACC_SYNCHRONIZED; }
    |
        "transient" 
        { self = org.multijava.mjc.Constants.ACC_TRANSIENT; }
    |
        "volatile" 
        { self = org.multijava.mjc.Constants.ACC_VOLATILE; }
    |
        "strictfp" 
        { self = org.multijava.mjc.Constants.ACC_STRICT; }
    |
        // WMD moved here from Jml.g
        "pure"
        { self = org.multijava.mjc.Constants.ACC_PURE; }
    ;


// Definition of a Java class
jClassDefinition [long mods, Token startToken] 
returns [JClassDeclarationType self = null]
{
    CTypeVariable[] typevariables = CTypeVariable.EMPTY;
    CClassType        superClass = null;
    CClassType[]        interfaces = CClassType.EMPTY;
    CParseClassContext    context = getParseClassContext();
    JavadocComment javadoc = null;
    JavaStyleComment[]    comments = utility.getStatementComment();
    TokenReference sourceRef = utility.buildTokenReference( startToken );
}
    :
        t:"class" ident:IDENT
        (typevariables = jTypeVariableDeclarationList[] )?
        {
            javadoc = utility.getJavadocComment( startToken );
        }
        // it _might_ have a superclass...
        superClass = jSuperClassClause[]
        // it might implement some interfaces...
        interfaces = jImplementsClause[]
        // now parse the body of the class
        jClassBlock[context]
        {
            self = new JClassDeclaration(
                sourceRef, mods, 
                ident.getText(), typevariables,superClass, interfaces,
                context.getMethods(), context.getInnerClasses(),
                context.getFieldsAndInits(), javadoc, comments );
            context.release();
        }
    ;

jSuperClassClause []
returns [CClassType self = null]
    :
        ( "extends" self = jTypeName[null] )?
    ;

// Definition of a Java Interface
jInterfaceDefinition [long mods, Token startToken] 
returns [JInterfaceDeclarationType self = null]
{
    CTypeVariable[]    typevariables = CTypeVariable.EMPTY;
    CClassType[]        interfaces =  CClassType.EMPTY;
    CParseClassContext    context = getParseClassContext();
    JavadocComment javadoc = null;
    JavaStyleComment[]    comments = utility.getStatementComment();
    TokenReference sourceRef = utility.buildTokenReference( startToken );
}
    :
        t:"interface" ident:IDENT
        (typevariables = jTypeVariableDeclarationList[] )?
        {            
            javadoc = utility.getJavadocComment( startToken );
        }
        // it might extend some other interfaces
        interfaces = jInterfaceExtends[]
        // now parse the body of the interface (looks like a class...)
        jClassBlock[context]
        {
            self = new JInterfaceDeclaration(
                sourceRef, mods, ident.getText(),typevariables, interfaces,
                context.getMethods(), context.getInnerClasses(),
                context.getFieldsAndInits(), javadoc, comments );
            context.release();
        }
    ;

// Declaration of a MultiJava External Method
mjTopLevelMethodDeclaration [long mods, Token startToken] 
returns [MJTopLevelMethodDeclaration self = null]
{
    CTypeVariable[] typevariables = CTypeVariable.EMPTY;
    CType type;                    // the method return type
    CType openClass;            // the class that this method extends
    JFormalParameter[]    params;    // an array of the formals
    CClassType[] throwsList = CClassType.EMPTY;    // array of poss. exceptions
    JStatement[] body = null;    // array of the statements in the body
    int bounds = 0;                // does the method return an array
    JavadocComment javadoc = null;
    JavaStyleComment[]    comments = utility.getStatementComment();
    TokenReference sourceRef = utility.buildTokenReference( startToken );
    ParsingController.TokenWrapper declEnd =
    new ParsingController.TokenWrapper();
}
    :
        (typevariables = jTypeVariableDeclarationList[])?
        // get the return type of the method
        type = jTypeSpec[]
        {
            javadoc = utility.getJavadocComment( startToken );
        }
        // get the class that this method enhances, the method identifier, 
        // and the formals
        openClass = mjExternalClassTypeSpec[]
        // mjExternalClassTypeSpec consumes the DOT preceding the identifier
        ident:IDENT
        // parse the formal parameter declarations.
        LPAREN
        params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER] 
        RPAREN
        (
            LBRACK RBRACK { bounds += 1; }
        )*
        {
            if (bounds > 0) {
                if( utility.getCompiler().universeChecks() ) {
                    utility.reportTrouble(new CWarning(sourceRef, 
                        CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                        null));
                } else {
                    utility.reportTrouble(new CWarning(sourceRef, 
                        MjcMessages.OLD_STYLE_ARRAY_BOUNDS, 
                        null));
                }
                type = new CArrayType(type, bounds, null);
            }
        }
        // get the list of exceptions that this method is declared to throw
        (throwsList = jThrowsClause[])?
        // get the method body
        ( body = jCompoundStatement[declEnd] | SEMI )
        {
            if (utility.allowRelaxedMultiJava && 
                (mods & org.multijava.mjc.Constants.ACC_ABSTRACT) != 0) {
                // For Relaxed MultiJava - if abstract, use special subclass
                self = new MJTopLevelAbstractMethodDeclaration(
                    sourceRef,
                    mods,
                    type,
                    openClass,
                    ident.getText(),
                    params,    
                    throwsList,
                    null,
                    javadoc,
                    comments );
            } else {
                self = new MJTopLevelMethodDeclaration(
                    sourceRef,
                    mods,
                    typevariables,
                    type,
                    openClass,
                    ident.getText(),
                    params,
                    throwsList,
                    body == null ? null : new JBlock(sourceRef, body, null),
                    javadoc,
                    comments );
                }
            utility.flushJavadocTokensWithWarning( declEnd );
        }
    ;

// This is the body of a class.  You can have members and extra semicolons,
jClassBlock [CParseClassContext context]
{
}
    :
        f1:LCURLY { utility.flushJavadocTokensWithWarning( f1 ); }
        
        ( 
            jMember[context] 
        | 
            t:SEMI
            {
                TokenReference pos = utility.buildTokenReference( t );
                utility.reportTrouble(
                    new CWarning( pos, MjcMessages.STRAY_SEMICOLON, null )); 
            }
        )*
        f2:RCURLY { utility.flushJavadocTokensWithWarning( f2 ); }
    ;

// An interface can extend several other interfaces...
jInterfaceExtends []
returns [CClassType[] self = CClassType.EMPTY]
    :
        ( "extends" self = jNameList[] )?
    ;

// A class can implement several interfaces...
jImplementsClause[]
returns [CClassType[] self = CClassType.EMPTY]
    :
        ( "implements" self = jNameList[] )?
    ;

// Now the various things that can be defined inside a class or interface:
// methods, constructors, or variable declaration
// Note that not all of these are really valid in an interface (constructors,
// for example), and if this grammar were used for a compiler there would
// need to be some semantic checks to make sure we're doing the right thing...
jMember [CParseClassContext context]
{
    CTypeVariable[] typevariables = CTypeVariable.EMPTY;
    JStatement[] body = null;
    JTypeDeclarationType decl;
    CType type;
    JFormalParameter[] params;
    int bounds = 0;
    CClassType[] throwsList = CClassType.EMPTY;
    JVariableDefinition[] vars;
    Token startToken = LT(1);
    TokenReference sourceRef = utility.buildTokenReference( startToken );
    ParsingController.TokenWrapper declEnd = 
    new ParsingController.TokenWrapper();
    JavadocComment javadoc = null;
    long mods;  
}
    :
        mods = jModifiers[]
        (
            // constructor
            ctor:IDENT
            {
                javadoc = utility.getJavadocComment( startToken );
            }
            // parse the formal parameter declarations.
            LPAREN 
            params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER] 
            RPAREN

            // get the list of exceptions that this method is declared to throw
            ( throwsList = jThrowsClause[] )?
            ( SEMI
                {
                    if (!Main.contextBehavior().noBodyOK(null))
                        throw new TokenStreamException( "Constructor must have a body" );
                    context.addMethodDeclaration(
                    new JConstructorDeclaration(
                        sourceRef,
                        mods, 
                        ctor.getText(),
                        params,
                        throwsList,
                        new JConstructorBlock(sourceRef, null),
                        javadoc,
                        utility.getStatementComment()
                    ));
                }
            |
            body = jCompoundStatement[declEnd]
            {
                utility.flushJavadocTokensWithWarning( declEnd );
                context.addMethodDeclaration(
                    new JConstructorDeclaration(
                        sourceRef,
                        mods, 
                        ctor.getText(),
                        params,
                        throwsList,
                        new JConstructorBlock(sourceRef, body),
                        javadoc,
                        utility.getStatementComment()
                    ));
            }
            )
        |
            decl = jClassDefinition[mods,startToken] // inner class             
            { context.addInnerDeclaration(decl); }              
        |
            decl = jInterfaceDefinition[mods,startToken] // inner interface 
            { context.addInnerDeclaration(decl); }
        |
            //Method with type variables
            typevariables = jTypeVariableDeclarationList[]
            type = jTypeSpec[]
            ident1:IDENT
            LPAREN params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER] RPAREN
            (
                LBRACK RBRACK { bounds += 1; }
            )*
            {
                if (bounds > 0) {
                    if( utility.getCompiler().universeChecks() ) {
                        utility.reportTrouble( new CWarning(sourceRef, 
                            CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                            null));
                    } else {
                        utility.reportTrouble( new CWarning(sourceRef, 
                            MjcMessages.OLD_STYLE_ARRAY_BOUNDS,
                            null));
                    }
                    type = new CArrayType(type, bounds, null);
                }
            }
            (throwsList = jThrowsClause[])?
            ( 
                body = jCompoundStatement[declEnd] 
                {
                    utility.flushJavadocTokensWithWarning( declEnd );
                }
            | 
                f3:SEMI 
                {
                    utility.flushJavadocTokensWithWarning( f3 );
                }
            )
            {
                context.addMethodDeclaration(
                    new JMethodDeclaration(sourceRef,
                        mods, 
                        typevariables,
                        type,
                        ident1.getText(),
                        params,
                        throwsList,
                        body == null ? null : new JBlock(sourceRef, body, null),
                        javadoc,
                        utility.getStatementComment()
                    ));
            }
 
        |
            // method or variable declaration(s)
            type = jTypeSpec[]
            {
                javadoc = utility.getJavadocComment( startToken );
            }
            (
                ident:IDENT
                // parse the formal parameter declarations.
                LPAREN params = jParameterDeclarationList[JLocalVariable.DES_PARAMETER] RPAREN
                (
                    LBRACK RBRACK { bounds += 1; }
                )*
                {
                    if (bounds > 0) {
                        if( utility.getCompiler().universeChecks() ) {
                            utility.reportTrouble( new CWarning(sourceRef, 
                               CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                               null));
                        } else {
                            utility.reportTrouble( new CWarning(sourceRef, 
                               MjcMessages.OLD_STYLE_ARRAY_BOUNDS, null));
                        }
                        type = new CArrayType(type, bounds, null);
                    }
                }
                (throwsList = jThrowsClause[])?
                ( 
                    body = jCompoundStatement[declEnd] 
                    {
                        utility.flushJavadocTokensWithWarning( declEnd );
                    }
                | 
                    f1:SEMI 
                    {
                        utility.flushJavadocTokensWithWarning( f1 );
                    }
                )
                {
                    context.addMethodDeclaration(
                        new JMethodDeclaration(sourceRef,
                            mods, 
                            CTypeVariable.EMPTY,
                            type,
                            ident.getText(),
                            params,
                            throwsList,
                            body == null ? null : new JBlock(sourceRef, body, null),
                            javadoc,
                            utility.getStatementComment()
                        ));
                }
            |
                vars = jVariableDefinitions[mods, type] f2:SEMI 
                {
                    // WMD: there was a pure in an unexpected
                    // location
                    // TODO where to put?
                    /*
                      if (isFunctional) {
                      utility.reportTrouble(
                      new PositionedError(
                        sourceRef, 
                        CUniverseMessages.VARIABLE_WITH_FUNCTIONAL_FORBIDDEN, null));

                        }
                    */

                    utility.flushJavadocTokensWithWarning( f2 );
                    for (int i = 0; i < vars.length; i++) {
                        context.addFieldDeclaration(
                            new JFieldDeclaration(vars[i].getTokenReference(),
                                vars[i], 
                                javadoc,
                                utility.getStatementComment())); 
                    }
                }
            )
        )
    |
        // "static { ... }" class initializer 
        "static"
        body = jCompoundStatement[declEnd]
        {
            context.addBlockInitializer(
                new JClassBlock(
                    sourceRef, true, body, 
                    utility.getJavadocComment( startToken )
                )); 
            utility.flushJavadocTokensWithWarning( declEnd );
        }
    |
        // "{ ... }" instance initializer
        body = jCompoundStatement[declEnd]
        {
            context.addBlockInitializer(
                new JClassBlock(
                    sourceRef, false, body, 
                    utility.getJavadocComment( startToken )
                )); 
            utility.flushJavadocTokensWithWarning( declEnd );
        }
    ;

jVariableDefinitions [long mods, CType type]
returns [JVariableDefinition[] self = null]
{
    ArrayList        vars = new ArrayList();
    JVariableDefinition    decl;
}
    :
        decl = jVariableDeclarator[mods, type]
        { vars.add(decl); }
        (    
            COMMA decl = jVariableDeclarator[mods, type]
            { vars.add(decl); }
        )*
        {
            self = new JVariableDefinition[vars.size()];
            self = (JVariableDefinition[]) vars.toArray(self);
                        
        }
    ;

/** Declaration of a variable.  This can be a class/instance variable,
 *   or a local variable in a method
 * It can also include possible initialization.
 */
jVariableDeclarator [long mods, CType type]
returns [JVariableDefinition self = null]
{
    int            bounds = 0;
    JExpression        expr = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :

        ident:IDENT ( LBRACK RBRACK { bounds += 1; } )*
        expr = jVarInitializer[]
        {
            if (bounds > 0) {
                if( utility.getCompiler().universeChecks() ) {
                    utility.reportTrouble(new CWarning(sourceRef, 
                        CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                        null));
                } else {
                    utility.reportTrouble(new CWarning(sourceRef, MjcMessages.OLD_STYLE_ARRAY_BOUNDS, null));
                }
                type = new CArrayType(type, bounds, null);
            }
            self = new JVariableDefinition(sourceRef, mods, type, ident.getText(), expr);
        }
    ;


jVarInitializer []
returns [JExpression self = null]
    :
        ( ASSIGN self = jInitializer[] )?
    ;

// This is an initializer used to set up an array.
jArrayInitializer[]
returns [JArrayInitializer self = null]
{
    JExpression        expr = null;
    ArrayList        vect = new ArrayList();
    TokenReference    sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:LCURLY 
        (    
            expr = jInitializer[]
            { vect.add(expr); }
            (
                // CONFLICT: does a COMMA after an initializer start a new
                //           initializer or start the option ',' at end?
                //           ANTLR generates proper code by matching
                //             the comma as soon as possible.
                options { warnWhenFollowAmbig = false; } :
                COMMA expr = jInitializer[]
                { vect.add(expr); }
            )*
        )?
        (
            COMMA
            { 
                utility.reportTrouble(
                    new CWarning(sourceRef, 
                        MjcMessages.STRAY_COMMA, null)); 
            }
        )?
        RCURLY
        {
            self = new JArrayInitializer(sourceRef, 
                (JExpression[]) vect.toArray(new JExpression[0])); 
            
        }
    ;

// The two "things" that can initialize an array element are an expression
//   and another (nested) array initializer.
jInitializer []
returns [JExpression self = null]
    :
        self = jExpression[]
    |    
        self = jArrayInitializer[]
    ;

// This is a list of exception classes that the method is declared to throw
jThrowsClause []
returns [CClassType[] self]
    :    
        // WMD TODO allow universe specifiers here
        "throws" self = jNameList[]
    ;


// A list of formal parameters
jParameterDeclarationList [int desc]
returns [JFormalParameter[] self = JFormalParameter.EMPTY]
{
    JFormalParameter        elem = null;
    ArrayList            vect = new ArrayList();
}
    :

        ( 
            elem = jParameterDeclaration[desc] 
            { vect.add(elem); }
            (
                COMMA elem = jParameterDeclaration[desc] 
                { vect.add(elem); }
            )*
            {
                self = new JFormalParameter[vect.size()];
                self = (JFormalParameter[]) vect.toArray(self);
                
            }
        )?
    ;

// A formal parameter.
jParameterDeclaration [int desc]
returns [JFormalParameter self = null]
{
    boolean    isFinal = false;
    int        bounds = 0;
    CType    type;
    CType    ddType = null;        // type of (optional) dynamic dispatch annot.
    TokenReference    sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        ( "final" { isFinal = true; } )? 
        type = jTypeSpec[] 
        // optional dynamic dispatch annotation
        ( {utility.allowMultiJava}?
            ( AT ddType = jTypeSpec[]
                // two @ signs for a value dispatch!
            | AT AT ddType = jValueSpecializer[type, sourceRef] )
        )?
        ident:IDENT
        ( LBRACK RBRACK { bounds += 1; } )*
        {
            if (bounds > 0) {
                if( utility.getCompiler().universeChecks() ) {
                    utility.reportTrouble(new CWarning(sourceRef, 
                        CUniverseMessages.OLD_STYLE_ARRAY_BOUNDS, 
                        null));
                } else {
                    utility.reportTrouble( new CWarning(sourceRef, 
                        MjcMessages.OLD_STYLE_ARRAY_BOUNDS));
                }
                type = new CArrayType(type, bounds, null);
            }
            CSpecializedType specializedType;
            if (ddType == null) {
                specializedType = new CSpecializedType( type );
            } else {
                specializedType = new CSpecializedType( type, ddType );
            }
            self = new JFormalParameter( sourceRef, desc, specializedType,
                ident.getText(), isFinal);                
        }
    ;

// Dispatch on a value
jValueSpecializer [CType type, TokenReference sourceRef]
returns [CType specType = null]
{
    JExpression expr;
}
    :
        expr = jExpression[]
        
    { if (type.isOrdinal())
            specType = new COrdinalValueType(type, expr);
      else if (type.isFloatingPoint())
            specType = new CRealValueType(type, expr);
      else if (type.isBoolean())
            specType = new CBooleanValueType(expr);
      else
            // we can't check here whether the static type is java.lang.String,
            // so we'll assume it is and check later
            specType = new CStringValueType(expr);
   }

            
    ;



// Compound statement.  This is used in many contexts:
//   Inside a class definition prefixed with "static":
//      it is a class initializer
//   Inside a class definition without "static":
//      it is an instance initializer
//   As the body of a method
//   As a completely indepdent braced block of code inside a method
//      it starts a new scope for variable definitions

jCompoundStatement [ParsingController.TokenWrapper declEnd]
returns [JStatement[] self = null]
{
    ArrayList        body = new ArrayList();
    JStatement        stmt;
}
    :
        s:LCURLY
        // include the (possibly-empty) list of statements
        (
            stmt = jStatement[]
            {
                if (stmt instanceof JEmptyStatement) {
                    utility.reportTrouble(
                        new CWarning(stmt.getTokenReference(), 
                            MjcMessages.STRAY_SEMICOLON, null));
                }
                body.add(stmt); }
        )*
        e:RCURLY
        {
            utility.wrapIfEmptyNonNullWrapper( declEnd, e );
            self = new JStatement[body.size()];
            self = (JStatement[]) body.toArray(self);
            
        }
    ;

// a Java statement, broken out as a separate rule so subgrammars can
// override this but inherit mjStatement
jStatement []
returns [JStatement self = null]
    :
        self = mjStatement[]
    ;

mjStatement []
returns [JStatement self = null]
{
    JExpression        expr;
    JStatement[]        stmts;
    JTypeDeclarationType    type;
    JavaStyleComment[]    comments = utility.getStatementComment();
    TokenReference    sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        // A list of statements in curly braces -- start a new scope!
        stmts = jCompoundStatement[null]
        { self = new JBlock(sourceRef, stmts, comments); }
    |
        // class definition
        type = jClassDefinition[0,null]
        { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
    |
        // final class definition
        "final" 
        type = jClassDefinition[org.multijava.mjc.Constants.ACC_FINAL,null]
        { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
    |
        // abstract class definition
        "abstract" 
        type = jClassDefinition[org.multijava.mjc.Constants.ACC_ABSTRACT,null]
        { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
    |
        // declarations are ambiguous with "ID DOT" relative to expression
        // statements.  Must backtrack to be sure.  Could use a semantic
        // predicate to test symbol table to see what the type was coming
        // up, but that's pretty hard without a symbol table ;)
        (jModifiers[] jTypeSpec[] IDENT) =>
        self = jDeclaration[] SEMI
    |
        // An expression statement.  This could be a method call,
        // assignment statement, or any other expression evaluated for
        // side-effects.
        expr = jExpression[] SEMI
        { self = new JExpressionStatement(sourceRef, expr, comments); }
    |
        self = jLabeledStatement[]
    |
        self = jIfStatement[]
    |
        self = jLoopStatement[]
    |
        self = jBreakStatement[]
    |
        self = jContinueStatement[]
    |
        self = jReturnStatement[]
    |
        self = jSwitchStatement[]
    |
        // exception try-catch block
        self = jTryBlock[]
    |
        self = jThrowStatement[]
    |
        self = jAssertStatement[]
    |
        self = jSynchronizedStatement[]
    |
        // empty statement
        SEMI
        { self = new JEmptyStatement(sourceRef, comments); }
    ;

jLabeledStatement []
returns [JLabeledStatement self = null]
{
    JStatement        stmt;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        // Attach a label to the front of a statement
        label:IDENT COLON stmt = jStatement[]
        {
            if (stmt instanceof JEmptyStatement) {
                utility.reportTrouble(
                    new CWarning(sourceRef, 
                        MjcMessages.STRAY_SEMICOLON, null));
            }
            self = new JLabeledStatement(sourceRef, label.getText(), stmt, utility.getStatementComment());
        }
    ;

jIfStatement []
returns [JIfStatement self = null]
{
    JExpression        cond;
    JStatement        thenClause;
    JStatement        elseClause = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"if" LPAREN cond = jExpression[] RPAREN
        thenClause = jStatement[]
//         {
//             // LOOKAHEAD OF 2 FORBIDDEN
//             if (LA(1)==LITERAL_else) {
//                 match(LITERAL_else);
//                 elseClause = jStatement();
//             }
//         }
        (
            // CONFLICT: the old "dangling-else" problem...
            // ANTLR generates proper code matching as soon as possible,
            // binding dangling else's to the innermost if statement
            // Hush warning.
            options { warnWhenFollowAmbig = false; } :
            "else" elseClause = jStatement[]
        )?
        {
            if (! (thenClause instanceof JBlock)) {
                utility.reportTrouble(new CWarning(sourceRef, MjcMessages.ENCLOSE_IF_THEN_IN_BLOCK, null));
            }
            if (elseClause != null && !(elseClause instanceof JBlock || elseClause instanceof JIfStatement)) {
                utility.reportTrouble(new CWarning(sourceRef, MjcMessages.ENCLOSE_IF_ELSE_IN_BLOCK, null));
            }
            self = new JIfStatement(sourceRef, cond, thenClause, elseClause, utility.getStatementComment());
        }
    ;

jWhileStatement []
returns [JWhileStatement self = null]
{
    JExpression        cond;
    JStatement        body;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"while" LPAREN cond = jExpression[] RPAREN body = jStatement[]
        {
            if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
                utility.reportTrouble(new CWarning(sourceRef, MjcMessages.ENCLOSE_LOOP_BODY_IN_BLOCK, null));
            }
            self = new JWhileStatement(sourceRef, cond, body, utility.getStatementComment());
        }
    ;

jDoStatement []
returns [JDoStatement self = null]
{
    JExpression        cond;
    JStatement        body;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"do" body = jStatement[] 
        "while" LPAREN cond = jExpression[] RPAREN SEMI
        {
            if (! (body instanceof JBlock)) {
                utility.reportTrouble(new CWarning(sourceRef, MjcMessages.ENCLOSE_LOOP_BODY_IN_BLOCK, null));
            }
            self = new JDoStatement(sourceRef, cond, body, utility.getStatementComment());
        }
    ;

jBreakStatement []
returns [JBreakStatement self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"break" ( label:IDENT )? SEMI
        { 
            self = new JBreakStatement(sourceRef, 
                label == null ? null : label.getText(), 
                utility.getStatementComment()); 
        }
    ;

jContinueStatement []
returns [JContinueStatement self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"continue" ( label:IDENT )? SEMI
        { 
            self = new JContinueStatement(sourceRef, 
                label == null ? null : label.getText(), 
                utility.getStatementComment()); 
        }
    ;

jReturnStatement []
returns [JReturnStatement self = null]
{
    JExpression        expr = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"return" ( expr = jExpression[] )? SEMI
        { 
            self = new JReturnStatement(sourceRef, expr, 
                utility.getStatementComment()); 
        }
    ;

jThrowStatement []
returns [JThrowStatement self = null]
{
    JExpression        expr;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"throw" expr = jExpression[] SEMI
        { 
            self = new JThrowStatement(sourceRef, expr, 
                utility.getStatementComment()); 
        }
    ;

jAssertStatement []
returns [JAssertStatement self = null]
{
    JExpression expr1;
    JExpression expr2 = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"assert" expr1 = jExpression[]
           (COLON expr2 = jExpression[])? SEMI
        { 
            self = new JAssertStatement(sourceRef, expr1, expr2, 
                utility.getStatementComment()); 
        }
    ;

jSynchronizedStatement []
returns [JSynchronizedStatement self = null]
{
    JExpression        expr;
    JStatement[]        body;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"synchronized" LPAREN expr = jExpression[] RPAREN 
        body = jCompoundStatement[null]
        { 
            self = new JSynchronizedStatement(sourceRef,
                expr, 
                new JBlock(sourceRef, body, null),
                utility.getStatementComment());
        }
    ;

jSwitchStatement []
returns [JSwitchStatement self = null]
{
    JExpression        expr = null;
    ArrayList        body = null;
    JSwitchGroup        group;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"switch" LPAREN expr = jExpression[] RPAREN LCURLY
        { 
            body = new ArrayList(); 
        }
        ( group = jCasesGroup[]  { body.add(group); } )*
        RCURLY
        { 
            self = new JSwitchStatement(sourceRef,
                expr, 
                (JSwitchGroup[]) body.toArray( new JSwitchGroup[0] ),
                utility.getStatementComment());
            
        }
    ;

jCasesGroup []
returns [JSwitchGroup self = null]
{
    ArrayList        labels = new ArrayList();
    ArrayList        stmts = new ArrayList();

    JSwitchLabel        label;
    JStatement        stmt;
    TokenReference    sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        (
            // CONFLICT: to which case group do the statements bind?
            //           ANTLR generates proper code: it groups the
            //           many "case"/"default" labels together then
            //           follows them with the statements
            options {
                warnWhenFollowAmbig = false;
            } :
            label = jACase[] { labels.add(label); }
        )+
        (
            stmt = jStatement[]
            {
                if (stmt instanceof JEmptyStatement) {
                    utility.reportTrouble(
                        new CWarning(stmt.getTokenReference(), 
                            MjcMessages.STRAY_SEMICOLON, null));
                }
                stmts.add(stmt);
            }
        )*
        {
            self = new JSwitchGroup( sourceRef,
                (JSwitchLabel[]) labels.toArray( new JSwitchLabel[0] ), 
                (JStatement[]) stmts.toArray( new JStatement[0] ) );
            
            
        }
    ;

jACase []
returns [JSwitchLabel self = null]
{
    JExpression        expr = null;
    TokenReference    sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        (
            "case" expr = jExpression[] 
            { self = new JSwitchLabel(sourceRef, expr); }
        |
            "default"
            { self = new JSwitchLabel(sourceRef, null); }
        ) COLON
    ;

jLoopStatement []
returns [JStatement self = null]
    :
        self = jForStatement[]
    |
        self = jWhileStatement[]
    |
        self = jDoStatement[]
    ;

jForStatement []
returns [JForStatement self = null]
{
    JStatement        init;
    JExpression        cond;
    JStatement        iter;        // !FIXME! change type to JExpressionListStatement and propagate
    JStatement        body;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"for" LPAREN init = jForInit[] SEMI cond = jForCond[] SEMI
        iter = jForIter[] RPAREN
        body = jStatement[]
        {
            if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
                utility.reportTrouble(
                    new CWarning( sourceRef, 
                        MjcMessages.ENCLOSE_LOOP_BODY_IN_BLOCK, null));
            }
            self = new JForStatement( sourceRef, init, cond, iter, body, 
                utility.getStatementComment());
        }
    ;

// The initializer for a for loop
jForInit []
returns [JStatement self = null]
{
    JExpression[]    list;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        (
            // if it looks like a declaration, it is
            (jDeclaration[])=> self = jDeclaration[]
        |
            // otherwise it could be an expression list...
            list = jExpressionList[]
            { self = new JExpressionListStatement( sourceRef, 
                    list, utility.getStatementComment()); }
        )?
    ;

jForCond []
returns [JExpression expr = null]
    :
        ( expr = jExpression[] )?
    ;

jForIter []
returns [JExpressionListStatement self = null]
{
    JExpression[] list;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        (
            list = jExpressionList[]
            { self = new JExpressionListStatement( sourceRef, list, null); }
        )?
    ;

// an exception handler try/catch block
jTryBlock []
returns [JStatement self = null]
{
    JBlock        tryClause = null;
    JStatement[]        compound;
    ArrayList        catchClauses = new ArrayList();
    JBlock        finallyClause = null;
    JCatchClause        catcher = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"try" compound = jCompoundStatement[null]
        { 
            tryClause = new JBlock(sourceRef, compound, null); 
        }
        (
            catcher = jHandler[]
            { catchClauses.add(catcher); }
        )*
        ( 
            "finally" compound = jCompoundStatement[null] 
            { finallyClause = new JBlock(sourceRef, compound, null); }
        )?
        {
            if (catchClauses.size() > 0) {
                self = new JTryCatchStatement(sourceRef,
                    tryClause,
                    (JCatchClause[]) catchClauses.toArray(new JCatchClause[0] ),
                    finallyClause == null ? utility.getStatementComment() : null);
                
            }
            if (finallyClause != null) {
                // If both catch and finally clauses are present,
                // the try-catch is embedded as try clause into a
                // try-finally statement.
                if (self != null) {
                    tryClause = new JBlock(sourceRef, new JStatement[] {self}, null);
                }
                self = new JTryFinallyStatement(sourceRef, tryClause, finallyClause, utility.getStatementComment());
            }
            
            if (self == null) {
                // try without catch or finally: error
                utility.reportTrouble(new PositionedError(sourceRef, MjcMessages.TRY_NOCATCH, null));
                self = tryClause;
            }
        }
    ;


// an exception handler
jHandler []
returns [JCatchClause self = null]
{
    JFormalParameter    param;
    JStatement[]        body;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        s:"catch" LPAREN 
        param = jParameterDeclaration[JLocalVariable.DES_CATCH_PARAMETER] 
        RPAREN body = jCompoundStatement[null]
        { 
            self = new JCatchClause(sourceRef, 
                param, new JBlock(sourceRef, body, null)); 
        }
    ;

// expressions
// Note that most of these expressions follow the pattern
//   thisLevelExpression :
//       nextHigherPrecedenceExpression
//           (OPERATOR nextHigherPrecedenceExpression)*
// which is a standard recursive definition for a parsing an expression.
// The operators in java have the following precedences:
//    lowest  (13)  = *= /= %= += -= <<= >>= >>>= &= ^= |=
//            (12)  ?:
//            (11)  ||
//            (10)  &&
//            ( 9)  |
//            ( 8)  ^
//            ( 7)  &
//            ( 6)  == !=
//            ( 5)  < <= > >=
//            ( 4)  << >>
//            ( 3)  +(binary) -(binary)
//            ( 2)  * / %
//            ( 1)  ++ -- +(unary) -(unary)  ~  !  (type)
//                  []   () (method call)  . (dot -- identifier qualification)
//                  new   ()  (explicit parenthesis)
//
// the last two are not usually on a precedence chart; I put them in
// to point out that new has a higher precedence than '.', so you
// can validy use
//     new Frame().show()
// 
// Note that the above precedence levels map to the rules below...
// Once you have a precedence chart, writing the appropriate rules as below
//   is usually very straightfoward



// the mother of all expressions
jExpression []
returns [JExpression self = null]
    :    self = jAssignmentExpression[]
    ;


// This is a list of expressions.
jExpressionList []
returns [JExpression[] self = null]
{
    JExpression        expr;
    ArrayList        vect = new ArrayList();
}
    :
        expr = jExpression[]
        { vect.add(expr); }
        (
            COMMA expr = jExpression[]
            { vect.add(expr); }
        )*
        {
            self = new JExpression[vect.size()];
            self = (JExpression[]) vect.toArray(self);
            
        }
    ;


// assignment expression (level 13)
jAssignmentExpression []
returns [JExpression self = null]
{
    int            oper = -1;
    JExpression        right;
}
    :
        self = jConditionalExpression[]
        (    
            (
                ASSIGN { oper = Constants.OPE_SIMPLE; }
            |   PLUS_ASSIGN { oper = Constants.OPE_PLUS; }
            |   MINUS_ASSIGN { oper = Constants.OPE_MINUS; }
            |   STAR_ASSIGN { oper = Constants.OPE_STAR; }
            |   SLASH_ASSIGN { oper = Constants.OPE_SLASH; }
            |   PERCENT_ASSIGN { oper = Constants.OPE_PERCENT; }
            |   SR_ASSIGN { oper = Constants.OPE_SR; }
            |   BSR_ASSIGN { oper = Constants.OPE_BSR; }
            |   SL_ASSIGN { oper = Constants.OPE_SL; }
            |   BAND_ASSIGN { oper = Constants.OPE_BAND; }
            |   BXOR_ASSIGN { oper = Constants.OPE_BXOR; }
            |   BOR_ASSIGN { oper = Constants.OPE_BOR; }
            )
            right = jAssignmentExpression[]
            { self = JAssignmentExpression.create(self.getTokenReference(), oper, self, right); }
        )?
    ;


// conditional test (level 12)
jConditionalExpression []
returns [JExpression self = null]
{
    JExpression        middle;
    JExpression        right;
}
    :
        self = jLogicalOrExpression[]
        ( 
            QUESTION middle = jConditionalExpression[] COLON right = jConditionalExpression[] 
            { self = new JConditionalExpression(self.getTokenReference(), self, middle, right); }
        )?
    ;


// logical or (||)  (level 11)
jLogicalOrExpression []
returns [JExpression self = null]
{
    JExpression        right;
}
    :
        self = jLogicalAndExpression[] 
        (
            LOR right = jLogicalAndExpression[]
            { self = new JConditionalOrExpression(self.getTokenReference(), self, right); }
        )*
    ;


// logical and (&&)  (level 10)
jLogicalAndExpression []
returns [JExpression self = null]
{
    JExpression        right;
}
    :
        self = jInclusiveOrExpression[] 
        (
            LAND right = jInclusiveOrExpression[]
            { self = new JConditionalAndExpression(self.getTokenReference(), self, right); }
        )*
    ;


// bitwise or non-short-circuiting or (|)  (level 9)
jInclusiveOrExpression []
returns [JExpression self = null]
{
    JExpression        right;
}
    :
        self = jExclusiveOrExpression[] 
        (
            BOR right = jExclusiveOrExpression[]
            { self = exprFactory.createBitwiseExpression(self.getTokenReference(), Constants.OPE_BOR, self, right); }
        )*
    ;


// exclusive or (^)  (level 8)
jExclusiveOrExpression []
returns [JExpression self = null]
{
    JExpression        right;
}
    :
        self = jAndExpression[] 
        (
            BXOR right = jAndExpression[]
            { self = exprFactory.createBitwiseExpression(self.getTokenReference(), Constants.OPE_BXOR, self, right); }
        )*
    ;


// bitwise or non-short-circuiting and (&)  (level 7)
jAndExpression []
returns [JExpression self = null]
{
    JExpression        right;
}
    :
        self = jEqualityExpression[] 
        (
            BAND right = jEqualityExpression[]
            { self = exprFactory.createBitwiseExpression(self.getTokenReference(), Constants.OPE_BAND, self, right); }
        )*
    ;


// equality/inequality (==/!=) (level 6)
jEqualityExpression []
returns [JExpression self = null]
{
    int            operator = -1;
    JExpression        right;
}
    :
        self = jRelationalExpression[] 
        (
            (
                NOT_EQUAL { operator = Constants.OPE_NE; }
            |
                EQUAL { operator = Constants.OPE_EQ; }
            ) 
            right = jRelationalExpression[]
            { self = new JEqualityExpression(self.getTokenReference(), operator, self, right); }
        )*
    ;


// boolean relational expressions (level 5)
jRelationalExpression []
returns [JExpression self = null]
{
    int            operator = -1;
    JExpression        right;
    CType            type;
}
    :
        self = jShiftExpression[]
        (    (    (    LT { operator = Constants.OPE_LT; }
                |    GT { operator = Constants.OPE_GT; }
                |    LE { operator = Constants.OPE_LE; }
                |    GE { operator = Constants.OPE_GE; }
                )
                right = jShiftExpression[]
                { self = new JRelationalExpression(self.getTokenReference(), operator, self, right); }
            )*
        |    "instanceof" type = jTypeSpec[]
            // WMD: TODO should we forbid annotations?
            { self = new JInstanceofExpression(self.getTokenReference(), self, type); }
        )
    ;


// bit shift expressions (level 4)
jShiftExpression []
returns [JExpression self = null]
{
    int            operator = -1;
    JExpression        right;
}
    :
        self = jAdditiveExpression[] 
        (
            (
                SL { operator = Constants.OPE_SL; } 
            | 
                SR { operator = Constants.OPE_SR; }
            | 
                BSR { operator = Constants.OPE_BSR; }
            )
            right = jAdditiveExpression[]
            { self = exprFactory.createShiftExpression(self.getTokenReference(), operator, self, right); }
        )*
    ;


// binary addition/subtraction (level 3)
jAdditiveExpression []
returns [JExpression self = null]
{
    Token        op = null;
    JExpression  right;
}
    :
        self = jMultiplicativeExpression[] 
        (
            (
                op_p:PLUS { op = op_p; }
            | 
                op_m:MINUS { op = op_m; }
            )
            right = jMultiplicativeExpression[]
            {
                self = exprFactory.createAdditiveExpr(op, self.getTokenReference(), self, right);
            }
        )*
    ;


// multiplication/division/modulo (level 2)
jMultiplicativeExpression []
returns [JExpression self = null]
{
    Token        op = null;
    JExpression  right;
}
    :
        self = jUnaryExpression[]
        (
            (
                op_m:STAR { op = op_m; }
            |    
                op_d:SLASH { op = op_d; }
            | 
                op_r:PERCENT { op = op_r; }
            ) 
            right = jUnaryExpression[]
            {
                self = exprFactory.createMultiplicativeExpr(op, self.getTokenReference(), self, right);
            }
        )*
    ;


// taken together, jUnaryExpression and jUnaryExpressionNotPlusMinus
// correspond to the Expression3 non-terminal of the Java grammar from
// section 18.1 of the Java Language Specification, 2nd ed.
jUnaryExpression []
returns [JExpression self = null]
{
    TokenReference    sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        INC self = jUnaryExpression[]
        { self = new JPrefixExpression(sourceRef, Constants.OPE_PREINC, self); }
    |
        DEC self = jUnaryExpression[]
        { self = new JPrefixExpression(sourceRef, Constants.OPE_PREDEC, self); }
    |   
        op_m:MINUS self = jUnaryExpression[]
        { self = exprFactory.createUnaryExpression(op_m, self.getTokenReference(), Constants.OPE_MINUS, self); }
    |
        op_p:PLUS self = jUnaryExpression[]
        { self = exprFactory.createUnaryExpression(op_p, self.getTokenReference(), Constants.OPE_PLUS, self); }
    |
        self = jUnaryExpressionNotPlusMinus[]
    ;

jUnaryExpressionNotPlusMinus []
returns [JExpression self = null]
    :
        self = mjOrJmlUnaryExpressionNotPlusMinus []
    ;

//==============================================================================
// WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING 
// Any changes made to mjOrJmlUnaryExpressionNotPlusMinus (below) must be copied
// textually into JML.g.
// WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING 

mjOrJmlUnaryExpressionNotPlusMinus []
returns [JExpression self = null]
{
    JExpression        expr;
    CType            dest;
    TokenReference    sourceRef = utility.buildTokenReference( LT(1) );
    CUniverse        array_univ = null;
    CUniverse        elem_univ = null;
}
    :
        op_b:BNOT self = jUnaryExpression[]
        { self = exprFactory.createUnaryExpression(op_b, self.getTokenReference(), Constants.OPE_BNOT, self); }
    |
        op_l:LNOT self = jUnaryExpression[]
        { self = exprFactory.createUnaryExpression(op_l, self.getTokenReference(), Constants.OPE_LNOT, self); }
    |
	// If typecast is built in type, must be numeric operand
        // array with one modifier
	( LPAREN (jUniverseSpec[])? jBuiltInTypeSpec[null,null] RPAREN )=>
	LPAREN (array_univ = jUniverseSpec[])? dest = jBuiltInTypeSpec[null, array_univ] RPAREN
	expr = jUnaryExpression[] 
	{ self = new JCastExpression(sourceRef, expr, dest); }
    |
	// If typecast is built in type, must be numeric operand
        // array with two modifiers
	( LPAREN (jUniverseSpec[] jUniverseSpec[])? jBuiltInTypeSpec[null,null] RPAREN )=>
	LPAREN (array_univ = jUniverseSpec[] elem_univ = jUniverseSpec[])? dest = jBuiltInTypeSpec[elem_univ, array_univ] RPAREN
	expr = jUnaryExpression[] 
	{ self = new JCastExpression(sourceRef, expr, dest); }
    |
	// Have to backtrack to see if operator follows.  If no operator
	// follows, it's a typecast.  No semantic checking needed to parse.
	// if it _looks_ like a cast, it _is_ a cast; else it's a "(expr)"

	// WMD
	// old:
	// ( LPAREN jClassTypeSpec[] RPAREN jUnaryExpressionNotPlusMinus[] )=>
	// LPAREN dest = jClassTypeSpec[] RPAREN
	// expr = jUnaryExpressionNotPlusMinus[]

	( LPAREN (jUniverseSpec[])? jClassTypeSpec[null,null] RPAREN
	  jUnaryExpressionNotPlusMinus[] )=>
	LPAREN (array_univ = mjUniverseImplicitPeerSpec[] elem_univ = jUniverseSpec[])? dest = jClassTypeSpec[elem_univ,array_univ] RPAREN
	    expr = jUnaryExpressionNotPlusMinus[]
	{ self = new JCastExpression(sourceRef, expr, dest); }
        // WMD added an implicit peer for the array universe
        // prevents a spurious warning
    |
    // allow cast of arrays with two universe specifiers
	( LPAREN (jUniverseSpec[] jUniverseSpec[])? jClassTypeSpec[null,null] RPAREN
	  jUnaryExpressionNotPlusMinus[] )=>
	LPAREN (array_univ = jUniverseSpec[] elem_univ = jUniverseSpec[])? dest = jClassTypeSpec[elem_univ,array_univ] RPAREN
	    expr = jUnaryExpressionNotPlusMinus[]
	{ self = new JCastExpression(sourceRef, expr, dest); }

    |    self = jPostfixExpression[]
    ;

// WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING 
// Any changes made to mjOrJmlUnaryExpressionNotPlusMinus (above) must be copied
// textually into JML.g.
// WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING 
//==============================================================================

// -------------------------------------------------------------
// jPostfixExpression corresponds to the third production of the
// Expression3 non-terminal from the Java grammar in Section 18.1 of
// the Java Language Specification, 2nd ed.

// qualified names, array expressions, method invocation, post inc/dec
jPostfixExpression []
returns [JExpression self = null]
{
    int        bounds = 0;
    JExpression    expr;
    JExpression[]    args = null;
}
    :
        self = jPrimaryExpression[] // start with a primary

        // This is the optional repeated Selector from JLS2, 18.1.
        (    // qualified id (id.id.id.id...) -- build the name
            DOT
            ( 
                ident:IDENT 
                { self = new JNameExpression(utility.buildTokenReference(), 
                        self, ident.getText(), null ); }
                ( l:LPAREN args = jArgList[] RPAREN
                    { self = new JMethodCallExpression(utility.buildTokenReference(), 
                            self, args); } )?
            | "this" { self = new JThisExpression(utility.buildTokenReference(), 
                        self); }
            | "super" 
                self = jSuperSuffix[new JSuperExpression(utility.buildTokenReference(), self)]
            | self = jNewExpression[self]
            | // or a MultiJava resend expression
              "resend" self = mjResendSuffix[self]
            )
        |    // an array indexing operation
            LBRACK expr = jExpression[] RBRACK
            { self = new JArrayAccessExpression(utility.buildTokenReference(), 
                    self, expr); }
        )*

        // possibly add on a post-increment or post-decrement.
        // allows INC/DEC on too much, but semantics can check
        (    INC
            { self = new JPostfixExpression(utility.buildTokenReference(), Constants.OPE_POSTINC, self); }
         |    DEC
            { self = new JPostfixExpression(utility.buildTokenReference(), Constants.OPE_POSTDEC, self); }
        )?

    ;

// The basic element of an expression, having the highest precedence.
jPrimaryExpression []
returns [JExpression self = null]
{
}
    :   self = mjPrimaryExpression[]
    ;

// factored out of jPrimaryExpression to allow extension in JML
// This corresponds to the Primary non-terminal from JLS2, 18.1.
mjPrimaryExpression []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JExpression[] args = null;
    JExpression expr = null;
    CType type;
    CUniverse univ = null;
    int bounds = 0;
}
    :   self = jParenthesizedExpression[]
    |    "this" { self = new JThisExpression(sourceRef); }
        ( LPAREN args = jArgList[] RPAREN 
            { self = new JExplicitConstructorInvocation(
                    utility.buildTokenReference(),
                    ((JThisExpression) self).prefix(), 
                    Constants.JAV_THIS, args);
            }
        )?
    |    "super" self = jSuperSuffix[ new JSuperExpression(sourceRef) ]
        // next 4 productions are various sorts of literals
    |    self = jConstant[]
    |    "true" { self = new JBooleanLiteral(sourceRef, true); }
    |    "false" { self = new JBooleanLiteral(sourceRef, false); }
    |    "null" { self = new JNullLiteral(sourceRef); }
    |    self = jNewExpression[null]
    |    (univ = jUniverseSpec[])? ident:IDENT { self = new JNameExpression(sourceRef, ident.getText(), univ); }
         // WMD: should this be UniverseSpec or UniverseType?
        ( 
            // CONFLICT: This is ambiguous between consuming here and
            //           consuming using the repeated Selector
            //           (inlined in jPostfixExpression above), greedily
            //           matching here works fine
            options { warnWhenFollowAmbig = false; } :
            DOT
	    id:IDENT 
            { self = new JNameExpression(utility.buildTokenReference(), self, 
                    id.getText(), univ ); }
            // WMD: we only initialise univ once, if at all, and then
            // pass it to all created JNameExpressions.
        )*

        // The following is an inlined version of the optional
        // IdentifierSuffix non-terminal from JLS2, 18.1.  It is
        // simplified from the JLS2 version since we have 2 tokens of
        // lookahead and can match most of the original
        // IdentifierSuffix grammar using the repeated Selector
        // (inlined in jPostfixExpression above)
        (
            // Array Type Class Expression (syntactic predicate
            // disambiguates from array access expression in Selector
            // above, since the indexing expression there requires
            // arbitrary lookahead)
            ( LBRACK RBRACK ) =>
            ( LBRACK RBRACK { bounds++; } )+ DOT "class"
            { self = new JClassExpression(utility.buildTokenReference(), 
                    self, bounds);
            }
        |    
            // Non-array, Reference Type Class Expression (syntactic
            // predicate disambiguates from DOT IDENT in Selector
            // above, since ANTLR doesn't reason using our custom
            // token for "class")
            ( DOT "class" ) =>
            DOT "class"
            { self = new JClassExpression(utility.buildTokenReference(), 
                    self, 0);
            }
        |    
            l:LPAREN args = jArgList[] RPAREN
            { self = new JMethodCallExpression(utility.buildTokenReference(), 
                    self, args); 
            }
        |
            // nothing
        )

        // look for int.class and int[].class
    |    type = jBuiltInType[]
        ( LBRACK RBRACK { bounds++; } )*
        d:DOT "class"
        {
            if (bounds > 0 && type == CStdType.Void) {
                throw new NoViableAltException(d, getFilename());
            }
            type = bounds > 0 ? new CArrayType(type, bounds, null) : type;
            self = new JClassExpression(utility.buildTokenReference(), type);
        }
    |   // or an unqualified MultiJava resend
        "resend" self = mjResendSuffix[null]
    |   self = mjWarnExpression[]
    ;

mjWarnExpression []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
    JExpression expr = null;
}
    :   op1:"_warn"      LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op1.getText(), expr, true); }
    |   op2:"_warn_op"   LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op2.getText(), expr, true, true); }
    |   op3:"_nowarn"    LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op3.getText(), expr, false); }
    |   op4:"_nowarn_op" LPAREN expr = jExpression[] RPAREN
        { self = new MJWarnExpression(sourceRef, op4.getText(), expr, false, true); }
    ;

// the parens and optional arguments for a MultiJava resend expression
mjResendSuffix [JExpression prefix] 
returns [JExpression self = null]
{
    JExpression[] args = null;
}
    :   LPAREN args = jArgList[] RPAREN
        { self = new JResendExpression( utility.buildTokenReference(), prefix,
                args ); }
    ;

jParenthesizedExpression []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        LPAREN self = jAssignmentExpression[] RPAREN
        { self = new JParenthesedExpression(sourceRef, self); }
    ;

jSuperSuffix [JSuperExpression supExpr]
returns [JExpression self = null]
{
    JExpression[] args = null;
}
    :
        LPAREN args = jArgList[] RPAREN
        { self = new JExplicitConstructorInvocation(utility.buildTokenReference(), 
                supExpr.prefix(), Constants.JAV_SUPER, args); }
    |
        DOT ident:IDENT 
        { self = new JNameExpression(utility.buildTokenReference(), supExpr, 
                ident.getText(), null); }
        ( LPAREN args = jArgList[] RPAREN 
            { self = new JMethodCallExpression(utility.buildTokenReference(), 
                    self, args); 
            }
        )?
    ;

/** object instantiation.
 *  Trees are built as illustrated by the following input/tree pairs:
 *  
 *  new T()
 *  
 *  new
 *   |
 *   T --  ELIST
 *           |
 *          arg1 -- arg2 -- .. -- argn
 *  
 *  new int[]
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *  
 *  new int[] {1,2}
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR -- ARRAY_INIT
 *                                  |
 *                                EXPR -- EXPR
 *                                  |      |
 *                                  1      2
 *  
 *  new int[3]
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *                |
 *              EXPR
 *                |
 *                3
 *  
 *  new int[1][2]
 *  
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *               |
 *         ARRAY_DECLARATOR -- EXPR
 *               |              |
 *             EXPR             1
 *               |
 *               2
 *  
 */
jNewExpression [JExpression expr]
returns [JExpression self = null]
{
    CType            type;
    JExpression[]        args;
    JArrayInitializer    init = null;
    JClassDeclarationType    decl = null;
    CParseClassContext    context = null;
    CUniverse             array_univ = null;
    CUniverse             elem_univ = null;
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        "new"

        // WMD: different cases for arrays and normal types
        ( ( jUniverseType[] jUniverseSpec[] )=>
            // for arrays it is allowed to have a readonly element type
            array_univ = jUniverseType[] elem_univ = jUniverseSpec[]
        |
            elem_univ = jUniverseSpec[]
            // we have to allow readonly here and then check
            // later whether it's an array or not!
        |
            { /* nothing */ }
            elem_univ = mjUniverseImplicitPeerSpec[]
        )

        type = jType[elem_univ]

        (
            // anonymous class
            LPAREN args = jArgList[] RPAREN 
            ( 
                { context = getParseClassContext(); }

                { // WMD: TODO anonymous classes can't have a universe right now
                    if( (array_univ != null && 
                            !(array_univ instanceof CUniverseImplicitPeer))
                        || (elem_univ != null &&
                            !(elem_univ instanceof CUniverseImplicitPeer)) ) {
                        // report error if there was a universe specified
                        utility.reportTrouble(
                           new PositionedError(
                              sourceRef, 
                              CUniverseMessages.ANON_CLASS_WITH_UNIVERSE_FORBIDDEN, null));
                    }
                }

                jClassBlock[context]
                {
                    decl = new JClassDeclaration(
                        sourceRef, 0,
                        ((CClassType)type).qualifiedName().replace('/','$'), // !!!
                        CTypeVariable.EMPTY, null, CClassType.EMPTY, context.getMethods(),
                        context.getInnerClasses(), context.getFieldsAndInits(),
                        null,    // no javadoc comments on anonymous classes
                        utility.getStatementComment() );
                    context.release();
                }
                {
                    self = new JNewAnonymousClassExpression(
                        sourceRef, (CClassType)type, expr, args, decl
                    ); 
                } // !!!
            |
                {
                    // WMD some error checking
                    if( array_univ != null && elem_univ != null ) {
                        // report error if there was an array universe specified
                        utility.reportTrouble(
                           new PositionedError(
                              sourceRef, 
                              CUniverseMessages.NONARRAY_WITH_TWO_UNIVERSES_FORBIDDEN, null));
                    }

                    if( elem_univ instanceof CUniverseReadonly ) {
                        utility.reportTrouble(
                          new PositionedError(
                             sourceRef, 
                             CUniverseMessages.READONLY_FORBIDDEN_HERE, null));
                    }

                    self = new JNewObjectExpression(
                        sourceRef, (CClassType)type, expr, args );
                } // !!!
            )
        |
            // java 1.1
            // Note: This will allow bad constructs like
            //    new int[4][][3] {exp,exp}.
            //    There needs to be a semantic check here...
            // to make sure:
            //   a) [ expr ] and [ ] are not mixed
            //   b) [ expr ] and an init are not used together
            args = jNewArrayDeclarator[] ( init = jArrayInitializer[])?
            {
                // WMD distinction between a class type and a builtin
                if( type instanceof CClassType ) {

		    // will find better default/propagation later.
                    // if( elem_univ != null &&
                    //    array_univ == null ) {
                    //        utility.reportTrouble(new CWarning(sourceRef, 
                    //           CUniverseMessages.ARRAY_WITH_NO_UNIVERSE,
                    //           null));
                    //    }

                    self = new JNewArrayExpression(
                               sourceRef, type,
                               new JArrayDimsAndInits( sourceRef,
                                                       array_univ,
                                                       args, init));
                } else {
                    // WMD: the element is a builtin type

                    if( args.length==1 ) {
                        if( array_univ != null ) {
                            // report error if there were two universes specified
                            utility.reportTrouble(
                               new PositionedError( sourceRef, 
                               CUniverseMessages.BUILTIN_WITH_UNIVERSE_FORBIDDEN,
                                               null));
                        }

                        // use the elem_univ for the whole array
                        self = new JNewArrayExpression(
                               sourceRef, type,
                               new JArrayDimsAndInits( sourceRef,
                                                       elem_univ,
                                                       args, init));
                    } else {
                        // use the elem_univ and array_univ
                        self = new JNewArrayExpression(
                               sourceRef, type,
                               new JArrayDimsAndInits( sourceRef,
                                                       array_univ, elem_univ,
                                                       args, init));
                    }
                }
            }
        )
    ;

jArgList[]
returns [JExpression[] self = JExpression.EMPTY]
    :
        ( self = jExpressionList[] )?
    ;

jNewArrayDeclarator[]
returns [JExpression[] self = null]
{
    boolean        unspec = false;
    ArrayList        vect = new ArrayList();
    JExpression        exp  = null;
}
    :
        (
            // CONFLICT:
            // newExpression is a primaryExpression which can be
            // followed by an array index reference.  This is ok,
            // as the generated code will stay in this loop as
            // long as it sees an LBRACK (proper behavior)
            options { warnWhenFollowAmbig = false; } :
            LBRACK
            (
                { !unspec }?
                exp = jExpression[] 
            | 
                { unspec = true; }
            )
            RBRACK
            { vect.add(exp); exp = null; }
        )+
        {
            self = new JExpression[vect.size()];
            self = (JExpression[]) vect.toArray(self);
                 
        }
    ;

jConstant []
returns [JExpression self = null]
{
    TokenReference sourceRef = utility.buildTokenReference( LT(1) );
}
    :
        c:CHARACTER_LITERAL
        { self = new JCharLiteral(sourceRef, c.getText()); }
    |
        s:STRING_LITERAL
        { self = new JStringLiteral(sourceRef, 
                s.getText(), true); }
    |
        i:INTEGER_LITERAL
        { self = new JOrdinalLiteral(sourceRef, i.getText()); }
    |
        r:REAL_LITERAL
        { self = new JRealLiteral(sourceRef, r.getText()); }
    ;

jNameList []
returns [CClassType[] self = null]
{
    //String    name;
    CClassType name;
    ArrayList    vect = new ArrayList();
}
    :
        name = jTypeName[null] { vect.add(name); }
        (
            COMMA 
            name = jTypeName[null] 
            { vect.add(name); } 
        )*
        {
            self = new CClassType[vect.size()];
            self = (CClassType[]) vect.toArray(self);
            
        }
    ;

//----------------------------------------------------------------------------
// The Java scanner
//----------------------------------------------------------------------------
{
    import org.multijava.util.compiler.TroubleReporter;

    // import java.io.Reader;
}
class MjcLexer extends Lexer;

options {
    importVocab=Mjc;
    testLiterals=false;    // don't automatically test for literals
    k=4;
//    charVocabulary='\u0003'..'\uFFFE'; !FIXME!this will blow-up the
//    lexer because the ~(...) expression includes so many options.  Need
//    an update to ANTLR for this to really work.
}

{
    public MjcLexer( ParsingController parsingController, 
	boolean allowJavaAssert, boolean allowResend, 
    boolean allowUniverseKeywords, // WMD
	TroubleReporter reporter ) 
    {
	this( parsingController.sharedInputState() );
	this.parsingController = parsingController;
	this.allowJavaAssert = allowJavaAssert;
	this.allowResend = allowResend;
    this.allowUniverseKeywords = allowUniverseKeywords; // WMD
        this.reporter = reporter;
    }

    /**
     * If tok represents a conditionally reserved word that is not 
     * reserved during this lexical analysis, then returns null, otherwise
     * returns tok.  For reserved words whose non-reserved use will be
     * deprecated (e.g., 'assert'), throws an . */
    private Token checkReservedTokens(Token tok) {
	if (tok == null) { return null; }
	Token result = tok;
	switch (tok.getType()) {
	    case MjcTokenTypes.LITERAL_assert:
	    if (!allowJavaAssert) {
		reporter.reportTrouble(
		    new CWarning(
			TokenReference.build( parsingController.file(),
			    getLine(), getColumn()), 
			MjcMessages.ASSERT_IS_KEYWORD));
		result = null;
	    }
	    break;
	    
	    case MjcTokenTypes.LITERAL_resend:
	    if (!allowResend) {
		result = null;
	    }
	    break;

        // WMD
        case MjcTokenTypes.LITERAL_peer:
        case MjcTokenTypes.LITERAL_readonly:
        case MjcTokenTypes.LITERAL_rep:
        case MjcTokenTypes.LITERAL_pure:
            if( !allowUniverseKeywords ) {
                result = null;
            }
            break;
	}
	return result;
    }

    /**
     * Find the keyword literal token for the given buffer.  This
     * method is modified in the lexers of grammars that inherit
     * from this to search different keyword sets.  */
    private Token keywordLookup( char[] cache, int start, int len ) {
    return MjcIDKeywords.lookup( cache, start, len );
    }

    /**
     * Called by the scanner when a tab is detected in the input
     * stream.  Advances the column number according to tabSize.
     *
     * @see    #setTabSize(int)
     * @see #tabSize() */
    public void tab() {
    setColumn( getColumn() + tabSize );
    }
    
    /**
     * Sets the amount by which the column number is incremented when
     * a tab character is detected in the input stream. 
     *
     * @see #tabSize() */
    public void setTabSize( int ts ) { tabSize = ts; }
    
    /**
     * Indicates the amount by which the column number is incremented
     * when a tab character is detected in the input stream.
     *
     * @see #setTabSize(int) */
    public int tabSize() { return tabSize; }
    
    /**
     * The amount by which the column number is incremented when a tab
     * character is detected in the input stream.  */
    private int tabSize = 8;
    
    /**
     * Used to switch lexers.  Aliases should be established in all
     * lexers that share the same input stream. */
    private ParsingController parsingController;
    
    /**
     * Used to convert token text from String to char[]. */
    private char[] charCache = null;

    /*
     * Indicate which conditionally reserved words should be
     * reserved during this analysis.
     */
    private boolean allowJavaAssert;
    private boolean allowResend;

    /* WMD: Indicate whether the conditionally reserved words used
     * for the universe type system should be reserved during this
     * analysis.
     */
    private boolean allowUniverseKeywords;

    /**
     * Used to report warnings during lexing.
     */
    private TroubleReporter reporter;
}

// OPERATORS
ASSIGN            :    '='        ;
AT                :   '@'        ;
BAND            :    '&'        ;
BAND_ASSIGN        :    "&="    ;
BNOT            :    '~'        ;
BOR                :    '|'        ;
BOR_ASSIGN        :    "|="    ;
BSR                : {parsingController.getUnmatchedTypeLT()==0}?   ">>>"    ;
BSR_ASSIGN        :  {parsingController.getUnmatchedTypeLT()==0}?  ">>>="    ;
BXOR            :    '^'        ;
BXOR_ASSIGN        :    "^="    ;
COLON            :    ':'        ;
COMMA            :    ','        ;
DEC                :    "--"    ;
//DOT                :    '.'        ; // imported
EQUAL            :    "=="    ;
GE                :    ">="    ;
GT                :    ">"        ;
INC                :    "++"    ;
LAND            :    "&&"    ;
LBRACK            :    '['        ;
LCURLY            :    '{'        ;
LE                :    "<="    ;
LNOT            :    '!'        ;
LOR                :    "||"    ;
LPAREN            :    '('        ;
LT                :    '<'        ;
MINUS            :    '-'        ;
MINUS_ASSIGN    :    "-="    ;
NOT_EQUAL        :    "!="    ;
PERCENT            :    '%'        ;
PERCENT_ASSIGN    :    "%="    ;
PLUS            :    '+'        ;
PLUS_ASSIGN        :    "+="    ;
QUESTION        :    '?'        ;
RBRACK            :    ']'        ;
RCURLY            :    '}'        ;
RPAREN            :    ')'        ;
SEMI            :    ';'        ;
SL                :    "<<"    ;
SLASH            :    '/'        ;
SLASH_ASSIGN    :    "/="    ;
SL_ASSIGN        :    "<<="    ;
SR                :   {parsingController.getUnmatchedTypeLT()==0}? ">>"    ;
SR_ASSIGN        :    {parsingController.getUnmatchedTypeLT()==0}? ">>="    ;
STAR            :    '*'        ;
STAR_ASSIGN        :    "*="    ;


// Whitespace -- ignored
WS    :    (
            NON_NL_WS
        |    
            NEWLINE
        )
        { _ttype = Token.SKIP; }
    ;

// non-newline whitespace, helpful to separate for sub-grammar purposes
protected NON_NL_WS
    :
        ( ' ' | '\t' | '\f' )
    ;


// Single-line comments.
SL_COMMENT
// if the file ends on this line, make sure not to consume the EOF character
    :    "//" (~('\n'|'\r'))* ({LA(1)!= EOF_CHAR}? NEWLINE)?
        {
            $setType(Token.SKIP);
        }
    ; 


// multiple-line comments and annotations
// also see ML_EMPTY_COMMENT below
ML_COMMENT
    :
        "/*"
        ( 
            NEWLINE 
        | 
            ~('*'|'\n'|'\r')
        )
        (    // see comments on NEWLINE
            options {
                generateAmbigWarnings=false;
            }
        :
            { LA(2)!='/' }? '*'
        |    NEWLINE
        |    ~('*'|'\n'|'\r')
        )*
        "*/" 
        {$setType(Token.SKIP);}
    ;

// a special case for an empty mulit-line comment, which is otherwise
// erroneously parsed as a javadoc comment
ML_EMPTY_COMMENT
    :
        "/**/"
        {$setType(Token.SKIP);}
    ;

// Will be ambiguous if called like ( NEWLINE | ... ) * because '\r'
// '\n' can be matched in one alternative or by matching '\r' in one
// iteration and '\n' in another.  ANTLR will greedily do the right
// thing.
protected
NEWLINE
    :
        (
            options {
                generateAmbigWarnings=false;
            }
        :
            '\r' '\n'
        |    '\r'
        |    '\n'
        )
        {newline();}
    ;

JAVADOC_OPEN
    :    { LA(4)!='/' }? "/**" {
            try {
                parsingController.push("javadoc");
            } catch( ParsingController.KeyException ke ) {
                throw 
                new TokenStreamException( "No javadoc lexer available." );
            }
            $setToken( ParsingController.IGNORE );
        }
    ;

// character literals
CHARACTER_LITERAL
    :    '\''! ( ESC | ~'\'' ) '\''!
    ;

// string literals
STRING_LITERAL
    :    '"'! (ESC|~('"'|'\\'))* '"'!
    ;


// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
// There are various ambiguities hushed in this rule.  The optional
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched.  ANTLR does the
// right thing by matching immediately; hence, it's ok to shut off
// the FOLLOW ambig warnings.
protected
ESC
    :    '\\'
        (    'n'
        |    'r'
        |    't'
        |    'b'
        |    'f'
        |    '"'
        |    '\''
        |    '\\'
        |    ('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT 
        |    ('0'..'3')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :    ('0'..'7')
                (    
                    options {
                        warnWhenFollowAmbig = false;
                    }
                :    '0'..'7'
                )?
            )?
        |    ('4'..'7')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :    ('0'..'9')
            )?
        )
    ;


// hexadecimal digit (again, note it's protected!)
protected
HEX_DIGIT
    :    ('0'..'9'|'A'..'F'|'a'..'f')
    ;


// a dummy rule to force vocabulary to be all characters (except special
//   ones that ANTLR uses internally (0 to 2)
protected
VOCAB
    :    '\3'..'\377'
    ;


// an identifier
IDENT
    :    ('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$')*
        { 
            String stext = $getText;
            int tokenLen = stext.length();
            if( charCache == null || charCache.length < tokenLen ) {
                charCache = stext.toCharArray();
            } else {
                stext.getChars( 0, tokenLen, charCache, 0 );
            }
            Token tok = keywordLookup( charCache, 0, tokenLen );
	    tok = checkReservedTokens( tok );
            if( tok == null ) {
                tok = CToken.lookupToken( MjcTokenTypes.IDENT, 
                    charCache, 0, tokenLen );
            }
            tok.setLine( getLine() );
            tok.setColumn( getColumn() );
            $setToken( tok );
        }
    ;


// a numeric literal, REAL_LITERAL is imported
INTEGER_LITERAL
    {boolean isDecimal=false;}
    :    '.' {_ttype = DOT;}
            (('0'..'9')+ (EXPONENT)? (FLOAT_SUFFIX)? { _ttype = REAL_LITERAL; })?
    |    (    '0' {isDecimal = true;} // special case for just '0'
            (    ('x'|'X')
                (                                            // hex
                    // the 'e'|'E' and float suffix stuff look
                    // like hex digits, hence the (...)+ doesn't
                    // know when to stop: ambig.  ANTLR resolves
                    // it correctly by matching immediately.  It
                    // is therefor ok to hush warning.
                    options {
                        warnWhenFollowAmbig=false;
                    }
                :    HEX_DIGIT
                )+
            |    ('0'..'7')+                                    // octal
            )?
        |    ('1'..'9') ('0'..'9')*  {isDecimal=true;}        // non-zero decimal
        )
        (    ('l'|'L')
        
        // only check to see if it's a float if looks like decimal so far
        |    {isDecimal}?
            (    '.' ('0'..'9')* (EXPONENT)? (FLOAT_SUFFIX)?
            |    EXPONENT (FLOAT_SUFFIX)?
            |    FLOAT_SUFFIX
            )
            { _ttype = REAL_LITERAL; }
        )?
    ;


// a couple protected methods to assist in matching floating point numbers
protected
EXPONENT
    :    ('e'|'E') ('+'|'-')? ('0'..'9')+
    ;


protected
FLOAT_SUFFIX
    :    'f'|'F'|'d'|'D'
    ;
