/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * @author David Cok
 * $Id: MjcFileFinder.java,v 1.9 2004/01/16 23:21:11 cclifton Exp $
 */

package org.multijava.mjc;
import org.multijava.util.classfile.ClassPath;

/** 
 * This FileFinder looks for a .class file and a .java file, returning
 * whichever one is newer.  */
public class MjcFileFinder extends AbstractFileFinder {

    public MjcFileFinder() {}

    /** 
     * This method looks for a java file for the given class name on
     * the source path, returning null if none could be found. */
    //@ also requires name != null;
    public ClassPath.ClassDescription findSourceFile(String name) {
	return ClassPath.getJavaFile(sourceFilePrefixFor(name));
    }

    /** 
     * This method finds a file for the given class name; it looks for
     * both a java file on the source path and a class file on the
     * class path, returning the newer one if both are found (and null
     * if neither is found).  */
    //@ also requires name != null; 
    public ClassPath.ClassDescription find(String name) {
	// look for both - use whichever is present; if both are
	// present, use the newer
	ClassPath.ClassDescription bytecodeD = 
	    ClassPath.getClassPathFile(name);
	String sourceName = sourceFilePrefixFor(name);
	ClassPath.ClassDescription sourceD = 
	    ClassPath.getJavaFile(sourceName);
	boolean useSource;	
	
	if (bytecodeD != null && sourceD == null) {
	    // class file only
	    //System.out.println("Class only: " + name);
	    useSource = false;
	} else if (bytecodeD == null && sourceD != null) {
	    // source file only
	    //System.out.println("Source only: " + name);
	    useSource = true;
		
	} else if (bytecodeD == null && sourceD == null) {
	    // no files
	    //System.out.println("No files: " + name);
	    return null;
		
	} else { // both files present
	    long classTime = bytecodeD.getTime();
	    long sourceTime = sourceD.getTime();
		
	    if (classTime < sourceTime) {
		// source file newer (includes case where class file
		// time is unknown: classTime == -1)
		useSource = true;
		//System.out.println("Source file newer: " + name);
		    
	    } else {
		// class file newer
		useSource = false;
		//System.out.println("Class file newer: " + name);
	    }
	}

	if (useSource) {
	    return sourceD; // will be good (not null or NO_CLASS_DESC)
	} else {
	    return bytecodeD;
	}
    }

}
