/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjcVisitor.java,v 1.10 2004/01/16 23:21:12 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.compiler.JavaStyleComment;
import org.multijava.javadoc.JavadocComment;

/**
 * Implementation of Visitor Design Pattern for KJC.
 *
 * Suggested from: Max R. Andersen(max@cs.auc.dk)
 * !CONVERT! use open classes
 */
public interface MjcVisitor {

    // ----------------------------------------------------------------------
    // COMPILATION UNIT
    // ----------------------------------------------------------------------

    /**
     * visits a compilation unit
     */
    void visitCompilationUnit(/*@non_null@*/ JCompilationUnit self );

    // ----------------------------------------------------------------------
    // TYPE DECLARATION
    // ----------------------------------------------------------------------

    /**
     * visits a class declaration
     */
    void visitClassDeclaration(/*@non_null@*/ JClassDeclaration self );

    /**
     * visits an interface declaration
     */
    void visitInterfaceDeclaration(/*@non_null@*/ JInterfaceDeclaration self );

    /**
     * visits a generic function anchor
     */
    void visitGenericFunctionDecl(/*@non_null@*/ MJGenericFunctionDecl self );

    // ----------------------------------------------------------------------
    // METHODS AND FIELDS
    // ----------------------------------------------------------------------

    /**
     * visits a field declaration
     */
    void visitFieldDeclaration(/*@non_null@*/ JFieldDeclaration self );

    /**
     * visits a method declaration
     */
    void visitMethodDeclaration(/*@non_null@*/ JMethodDeclaration self );

    /**
     * visits an initializer declaration
     */
    void visitInitializerDeclaration(/*@non_null@*/ JInitializerDeclaration self );

    /**
     * visits an external method declaration
     */
    void visitTopLevelMethodDeclaration(/*@non_null@*/ MJTopLevelMethodDeclaration self );

    /**
     * visits a constructor declaration
     */
    void visitConstructorDeclaration(/*@non_null@*/ JConstructorDeclaration self );

    // ----------------------------------------------------------------------
    // STATEMENTS
    // ----------------------------------------------------------------------

    /** Visits a while statement. */
    void visitAssertStatement(/*@non_null@*/ JAssertStatement self);

    /**
     * visits a while statement
     */
    void visitWhileStatement(/*@non_null@*/ JWhileStatement self );

    /**
     * visits a variable declaration statement
     */
    void visitVariableDeclarationStatement(/*@non_null@*/JVariableDeclarationStatement self);

    /**
     * visits a variable declaration statement
     */
    void visitVariableDefinition(/*@non_null@*/ JVariableDefinition self );

    /**
     * visits a try-catch statement
     */
    void visitTryCatchStatement(/*@non_null@*/ JTryCatchStatement self );

    /**
     * visits a try-finally statement
     */
    void visitTryFinallyStatement(/*@non_null@*/ JTryFinallyStatement self );

    /**
     * visits a throw statement
     */
    void visitThrowStatement(/*@non_null@*/ JThrowStatement self );

    /**
     * visits a synchronized statement
     */
    void visitSynchronizedStatement(/*@non_null@*/ JSynchronizedStatement self );

    /**
     * visits a switch statement
     */
    void visitSwitchStatement(/*@non_null@*/ JSwitchStatement self );

    /**
     * visits a return statement
     */
    void visitReturnStatement(/*@non_null@*/ JReturnStatement self );

    /**
     * visits a labeled statement
     */
    void visitLabeledStatement(/*@non_null@*/ JLabeledStatement self );

    /**
     * visits a if statement
     */
    void visitIfStatement(/*@non_null@*/ JIfStatement self );

    /**
     * visits a for statement
     */
    void visitForStatement(/*@non_null@*/ JForStatement self );

    /**
     * visits a compound statement
     */
    void visitCompoundStatement(/*@non_null@*/ JCompoundStatement self );

    /**
     * visits an expression statement
     */
    void visitExpressionStatement(/*@non_null@*/ JExpressionStatement self );

    /**
     * visits an expression list statement
     */
    void visitExpressionListStatement(/*@non_null@*/ JExpressionListStatement self );

    /**
     * visits a empty statement
     */
    void visitEmptyStatement(/*@non_null@*/ JEmptyStatement self );

    /**
     * visits a do statement
     */
    void visitDoStatement(/*@non_null@*/ JDoStatement self );

    /**
     * visits a continue statement
     */
    void visitContinueStatement(/*@non_null@*/ JContinueStatement self );

    /**
     * visits a break statement
     */
    void visitBreakStatement(/*@non_null@*/ JBreakStatement self );

    /**
     * visits an expression statement
     */
    void visitBlockStatement(/*@non_null@*/ JBlock self );

    /**
     * visits a constructor block
     */
    void visitConstructorBlock(/*@non_null@*/ JConstructorBlock self );
    /**
     * visits a class block (initializer)
     */
    void visitClassBlock(/*@non_null@*/ JClassBlock self );

    /**
     * visits a type declaration statement
     */
    void visitTypeDeclarationStatement(/*@non_null@*/ JTypeDeclarationStatement self );

    // ----------------------------------------------------------------------
    // EXPRESSION
    // ----------------------------------------------------------------------

    /**
     * visits an unary expression
     */
    void visitUnaryExpression(/*@non_null@*/ JUnaryExpression self );

    /**
     * visits a type name expression
     */
    void visitTypeNameExpression(/*@non_null@*/ JTypeNameExpression self );

    /**
     * visits a this expression
     */
    void visitThisExpression(/*@non_null@*/ JThisExpression self );

    /**
     * visits a super expression
     */
    void visitSuperExpression(/*@non_null@*/ JSuperExpression self );

    /**
     * visits a shift expression
     */
    void visitShiftExpression(/*@non_null@*/ JShiftExpression self );

    /**
     * visits a shift expressiona
     */
    void visitRelationalExpression(/*@non_null@*/ JRelationalExpression self );

    /**
     * visits a prefix expression
     */
    void visitPrefixExpression(/*@non_null@*/ JPrefixExpression self );

    /**
     * visits a postfix expression
     */
    void visitPostfixExpression(/*@non_null@*/ JPostfixExpression self );

    /**
     * visits a parenthesed expression
     */
    void visitParenthesedExpression(/*@non_null@*/ JParenthesedExpression self );

    /**
     * visits an object allocator expression
     */
    void visitNewObjectExpression(/*@non_null@*/ JNewObjectExpression self );

    /**
     * visits an object allocator expression for an anonymous class
     */
    void visitNewAnonymousClassExpression(/*@non_null@*/ JNewAnonymousClassExpression self );

    /**
     * visits an array allocator expression
     */
    void visitNewArrayExpression(/*@non_null@*/ JNewArrayExpression self );

    /**
     * visits a name expression
     */
    void visitNameExpression(/*@non_null@*/ JNameExpression self );

    /**
     * visits an add expression
     */
    void visitAddExpression(/*@non_null@*/ JAddExpression self );

    /**
     * visits a boolean AND expression
     */
    void visitConditionalAndExpression(/*@non_null@*/ JConditionalAndExpression self );

    /**
     * visits a boolean OR expression
     */
    void visitConditionalOrExpression(/*@non_null@*/ JConditionalOrExpression self );

    /**
     * visits a divide expression
     */
    void visitDivideExpression(/*@non_null@*/ JDivideExpression self );

    /**
     * visits a minus expression
     */
    void visitMinusExpression(/*@non_null@*/ JMinusExpression self );

    /**
     * visits a modulo division expression
     */
    void visitModuloExpression(/*@non_null@*/ JModuloExpression self );

    /**
     * visits a multiplication expression
     */
    void visitMultExpression(/*@non_null@*/ JMultExpression self );

    /**
     * visits a method call expression
     */
    void visitMethodCallExpression(/*@non_null@*/ JMethodCallExpression self );
    /**
     * visits a local variable expression
     */
    void visitLocalVariableExpression(/*@non_null@*/ JLocalVariableExpression self );

    /**
     * visits an instanceof expression
     */
    void visitInstanceofExpression(/*@non_null@*/ JInstanceofExpression self );

    /**
     * visits an equality expression
     */
    void visitEqualityExpression(/*@non_null@*/ JEqualityExpression self );

    /**
     * visits a conditional expression
     */
    void visitConditionalExpression(/*@non_null@*/ JConditionalExpression self );

    /**
     * visits a compound expression
     */
    void visitCompoundAssignmentExpression(/*@non_null@*/JCompoundAssignmentExpression self);
    /**
     * visits a field expression
     */
    void visitFieldExpression(/*@non_null@*/ JClassFieldExpression self );

    /**
     * visits a class expression
     */
    void visitClassExpression(/*@non_null@*/ JClassExpression self );

    /**
     * visits a cast expression
     */
    void visitCastExpression(/*@non_null@*/ JCastExpression self );

    /**
     * visits a cast expression
     */
    void visitUnaryPromoteExpression(/*@non_null@*/ JUnaryPromote self );

    /**
     * visits a compound assignment expression
     */
    void visitBitwiseExpression(/*@non_null@*/ JBitwiseExpression self );
    /**
     * visits an assignment expression
     */
    void visitAssignmentExpression(/*@non_null@*/ JAssignmentExpression self );

    /**
     * visits an array length expression
     */
    void visitArrayLengthExpression(/*@non_null@*/JArrayLengthExpression self );

    /**
     * visits an array access expression
     */
    void visitArrayAccessExpression(/*@non_null@*/ JArrayAccessExpression self );

    /**
     * visits a warn expression
     */
    void visitWarnExpression(/*@non_null@*/ MJWarnExpression self );
    
    /**
     * visits a math mode expression
     */
    void visitMathModeExpression(/*@non_null@*/ MJMathModeExpression self );

    // ----------------------------------------------------------------------
    // OTHERS
    // ----------------------------------------------------------------------

    /**
     * visits a switch label
     */
    void visitSwitchLabel(/*@non_null@*/ JSwitchLabel self );

    /**
     * visits a switch group
     */
    void visitSwitchGroup(/*@non_null@*/ JSwitchGroup self );

    /**
     * visits a catch clause
     */
    void visitCatchClause(/*@non_null@*/ JCatchClause self );

    /**
     * visits a boolean literal
     */
    void visitBooleanLiteral(/*@non_null@*/ JBooleanLiteral self );

    /**
     * visits a character literal
     */
    void visitCharLiteral(/*@non_null@*/ JCharLiteral self );

    /**
     * prints an ordinal literal
     */
    void visitOrdinalLiteral(/*@non_null@*/ JOrdinalLiteral self );

    /**
     * prints a real literal
     */
    void visitRealLiteral(/*@non_null@*/ JRealLiteral self );

    /**
     * visits a string literal
     */
    void visitStringLiteral(/*@non_null@*/ JStringLiteral self );

    /**
     * visits a null literal
     */
    void visitNullLiteral(/*@non_null@*/ JNullLiteral self );

    /**
     * visits a package name declaration
     */
    void visitPackageName(/*@non_null@*/ JPackageName self );

    /**
     * visits a package import declaration
     */
    void visitPackageImport(/*@non_null@*/ JPackageImport self );

    /**
     * visits a class import declaration
     */
    void visitClassOrGFImport(/*@non_null@*/ JClassOrGFImport self );

    /**
     * visits a formal parameter
     */
    void visitFormalParameters(/*@non_null@*/ JFormalParameter self );

    /**
     * visits an explicit constructor invocation
     */
    void visitExplicitConstructorInvocation(/*@non_null@*/JExplicitConstructorInvocation self);

    /**
     * visits an array initializer expression
     */
    void visitArrayInitializer(/*@non_null@*/ JArrayInitializer self );

    /**
     * visits an array dimension and initialization expression
     */
    void visitArrayDimsAndInit(/*@non_null@*/ JArrayDimsAndInits self );

}
