/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: SigSourceMember.java,v 1.1 2002/09/18 17:40:15 cclifton Exp $
 */

package org.multijava.mjc;

/**
 * This interface is implemented by signature forest classes that
 * correspond to specific member declarations in an AST.  The
 * interface allows one to find the AST node for a declaration from
 * the signature of that declaration.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $
 */

public interface SigSourceMember {

    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------
    
    /**
     * Returns the AST node for the declaration corresponding to this
     * signature.  May return null if mapping to declaration is not
     * available.  For example, an objects representing a synthetic
     * method might be a subtype of CSourceMethod, yet such an object
     * does not have a corresponding AST node.
     *
     * <pre><jml>
     * ensures \result instanceof JTypeDeclaration ==> 
     *		\result.getCClass() == this;
     * ensures \result instanceof JMethodDeclaration ==> 
     *		\result.getMethod() == this;
     * ensures \result instanceof JFieldDeclaration ==> 
     *		\result.getField() == this;
     * </jml></pre>
     *
     */
    JMemberDeclaration declarationASTNode();

}
