/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * $Id: TestCContext.java,v 1.13 2005/04/26 02:39:33 kui_dai Exp $
 */

package org.multijava.mjc;

import org.multijava.util.testing.TestCase;

import java.io.Reader;
import java.io.StringReader;

import antlr.Token;
import antlr.TokenStreamException;

/**
 * Unit tests for CContext */
public class TestCContext extends TestCase {
    //---------------------------------------------------------------------
    // CONSTUCTORS
    //---------------------------------------------------------------------

    public TestCContext( String name ) {
	super( name );
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------

    public void test_isInLoop_method1() {
	assertTrue( "compUnit", !compUnit.isInLoop() );
	assertTrue( "method", !method.isInLoop() );
	assertTrue( "methodBlock", !methodBlock.isInLoop() );
	assertTrue( "loop", loop.isInLoop() );
	assertTrue( "loopBody", loopBody.isInLoop() );
	assertTrue( "expr with flow context: " + 
		    expr.getFlowControlContext().toString(), expr.isInLoop() );
    }

    /**
     * Should be able to call getCMethod on any well-formed context,
     * retreiving the correct method signature if one exists or null
     * otherwise. */
    public void test_getCMethod_method1() {
	assertTrue( "compUnit", compUnit.getCMethod() == null );
	assertTrue( "method", method.getCMethod() == methodSig );
	assertTrue( "methodBlock", methodBlock.getCMethod() == methodSig );
	assertTrue( "loop", loop.getCMethod() == methodSig );
	assertTrue( "loopBody", loopBody.getCMethod() == methodSig );
	assertTrue( "expr", expr.getCMethod() == methodSig );
    }

    //---------------------------------------------------------------------
    // COMMON FIXTURE
    //---------------------------------------------------------------------

    public void setUp() {
	Main m = new Main();
	CTopLevel.initSession(m, TypeLoader.getSingleton());
	// build some signatures
	classSig = 
	    new CSourceClass( null, new MemberAccess(null, null, 0L), 
			      org.multijava.util.compiler.TokenReference.NO_REF, 
			      "C", "C", CTypeVariable.EMPTY,false, false, false );
	methodSig = 
	    new CSourceMethod( new MemberAccess(classSig, null, 0L), 
			       "m", CStdType.Void, 
			       new CSpecializedType[0],
			       new CClassType[0],
			       CTypeVariable.EMPTY,false, null, null, null );
	// build nested contexts
	compUnit = 
	    m.createCompilationUnitContext( null, 
					    new CCompilationUnit("unnamed") );
	clazz = compUnit.createClassContext( null );
	method = clazz.createMethodContext( methodSig );
	methodBlock = method.createFlowControlContext(1,null);
	loop = methodBlock.createLoopContext( null );
	loopBody = loop.createFlowControlContext( null );
	expr = loopBody.createExpressionContext();
    }
    
    protected CClass classSig;
    protected CMethod methodSig;

    protected CCompilationUnitContextType compUnit;
    protected CClassContextType clazz;
    protected CMethodContextType method;
    protected CFlowControlContextType methodBlock;
    protected CLoopContext loop;
    protected CFlowControlContextType loopBody;
    protected CExpressionContextType expr;

}
