/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestJTypeDeclaration.java,v 1.15 2004/12/01 03:56:30 cclifton Exp $
 */

package org.multijava.mjc;

import antlr.RecognitionException;
import antlr.TokenStreamException;

import org.multijava.util.classfile.ClassPath;
import org.multijava.util.compiler.PositionedError;
import org.multijava.util.testing.TestCase;

import java.util.ArrayList;

/**
 * Unit tests for JTypeDeclaration */
public class TestJTypeDeclaration extends TestCase {

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------
    
    public TestJTypeDeclaration( String name ) {
        super( name );
        parserHelper = new TestMjcParser( name );
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------
    // use names like test_memberName_memberKindX,
    // e.g. test_hashCode_method1

    /**
     * Test one internal method of local internal generic function using single
     * dispatch.  */
    public void test_dispatcherWrapMethods_method1() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public class Sample {" + NEWLINE +
                           "  public boolean m( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 27 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [Sample.m( java.lang.Object ), Sample.<init>(  )] }", wrappedMethods.toString(), true );
    }

    /**
     * Test one internal method of non-local internal gf using single
     * dispatch.  */
    public void test_dispatcherWrapMethods_method2() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public class Sample {" + NEWLINE +
                           "  public boolean equals( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [Sample.equals( java.lang.Object ), Sample.<init>(  )] }", wrappedMethods.toString(), true );
    }

    /**
     * Test one external method of local external generic function using single
     * dispatch.  */
    public void test_dispatcherWrapMethods_method3() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public boolean Object.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getGFsMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [m$anchor.<init>(  ), m$anchor.mjc$initializeCOR(  ), m$anchor.<clinit>(  ), m$anchor.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test one internal method of local external generic function using single
     * dispatch.  */
    public void test_dispatcherWrapMethods_method4() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "private boolean Object.m( Object o ) {" + NEWLINE +
                           "  return true;" + NEWLINE +
                           "}" + NEWLINE +
                           "public class Sample {" + NEWLINE +
                           "  private boolean m( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 8 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 33 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 28 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 10 warning: Private method \"m\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [Sample.<init>(  ), Sample.mjc$initializeCOR(  ), Sample.<clinit>(  ), Sample.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@Sample, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test one internal method of non-local internal gf using multiple
     * dispatch.  */
    public void test_dispatcherWrapMethods_method5() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public class Sample {" + NEWLINE +
                           "  public boolean equals( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [/* grouped */ Sample.equals( java.lang.Object ) [Sample.equals( java.lang.Object@Sample )], Sample.<init>(  )] }", wrappedMethods.toString(), true );
    }

    /**
     * Test one internal method of local external gf using multiple
     * dispatch.  */
    public void test_dispatcherWrapMethods_method6() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "private boolean Object.m( Object o ) {" + NEWLINE +
                           "  return true;" + NEWLINE +
                           "}" + NEWLINE +
                           "public class Sample {" + NEWLINE +
                           "  private boolean m( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 8 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 33 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 28 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 10 warning: Private method \"m\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [Sample.<init>(  ), Sample.mjc$initializeCOR(  ), Sample.<clinit>(  ), Sample.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@Sample, java.lang.Object@Sample )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test two external method of local external gf using single
     * dispatch.  */
    public void test_dispatcherWrapMethods_method7() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "public boolean Object.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" + NEWLINE + 
                           "public boolean Number.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getGFsMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [m$anchor.<init>(  ), m$anchor.mjc$initializeCOR(  ), m$anchor.<clinit>(  ), m$anchor.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@java.lang.Number, java.lang.Object ), m( java.lang.Object, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test two internal method of local internal gf using multiple
     * dispatch.  */
    public void test_dispatcherWrapMethods_method8() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public class Sample {" + NEWLINE +
                           "  public boolean m( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  public boolean m( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 27 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 27 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [/* grouped */ Sample.m( java.lang.Object ) [Sample.m( java.lang.Object@Sample ), Sample.m( java.lang.Object )], Sample.<init>(  )] }", wrappedMethods.toString(), true );
    }

    /**
     * Test two internal method of non-local internal gf using multiple
     * dispatch.  */
    public void test_dispatcherWrapMethods_method9() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public class Sample {" + NEWLINE +
                           "  public boolean equals( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  public boolean equals( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [/* grouped */ Sample.equals( java.lang.Object ) [Sample.equals( java.lang.Object@Sample ), Sample.equals( java.lang.Object )], Sample.<init>(  )] }", wrappedMethods.toString(), true );
    }

    /**
     * Test two external methods of local external generic function
     * using multiple dispatch.  */
    public void test_dispatcherWrapMethods_method10() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "public boolean Object.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" + NEWLINE +
                           "public boolean Number.m( Object@Number n ) {" + 
                           NEWLINE +
                           "  return true;" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getGFsMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 32 warning: Parameter \"n\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [m$anchor.<init>(  ), m$anchor.mjc$initializeCOR(  ), m$anchor.<clinit>(  ), m$anchor.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@java.lang.Number, java.lang.Object@java.lang.Number ), m( java.lang.Object, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test two internal methods of local external generic function
     * using multiple dispatch.  */
    public void test_dispatcherWrapMethods_method11() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "private boolean Object.m( Object o ) {" + NEWLINE +
                           "  return true;" + NEWLINE +
                           "}" + NEWLINE +
                           "public class Sample {" + NEWLINE +
                           "  private boolean m( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  private boolean m( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 8 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 33 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 28 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 9, character 28 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 10 warning: Private method \"m\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 9, character 10 warning: Private method \"m\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [Sample.<init>(  ), Sample.mjc$initializeCOR(  ), Sample.<clinit>(  ), Sample.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@Sample, java.lang.Object@Sample ), m( java.lang.Object@Sample, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test three external method of local external generic function
     * using single dispatch.  */
    public void test_dispatcherWrapMethods_method12() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "public boolean Object.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" + NEWLINE + 
                           "public boolean String.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" + NEWLINE + 
                           "public boolean Number.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getGFsMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 8, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 8, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [m$anchor.<init>(  ), m$anchor.mjc$initializeCOR(  ), m$anchor.<clinit>(  ), m$anchor.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@java.lang.Number, java.lang.Object ), m( java.lang.Object@java.lang.String, java.lang.Object ), m( java.lang.Object, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test three internal method of local internal generic function
     * using multiple dispatch.  */
    public void test_dispatcherWrapMethods_method13() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public class Sample {" + NEWLINE +
                           "  public boolean m( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  public boolean m( Object@Number o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  public boolean m( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 27 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 27 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 8, character 27 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [/* grouped */ Sample.m( java.lang.Object ) [Sample.m( java.lang.Object@Sample ), Sample.m( java.lang.Object@java.lang.Number ), Sample.m( java.lang.Object )], Sample.<init>(  )] }", wrappedMethods.toString(), true );
    }

    /**
     * Test three internal method of non-local internal generic function
     * using multiple dispatch.  */
    public void test_dispatcherWrapMethods_method14() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "public class Sample {" + NEWLINE +
                           "  public boolean equals( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  public boolean equals( Object@Number o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  public boolean equals( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 1, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 8, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [/* grouped */ Sample.equals( java.lang.Object ) [Sample.equals( java.lang.Object@Sample ), Sample.equals( java.lang.Object@java.lang.Number ), Sample.equals( java.lang.Object )], Sample.<init>(  )] }", wrappedMethods.toString(), true );
    }

    /**
     * Test three external methods of local external generic function
     * using multiple dispatch.  */
    public void test_dispatcherWrapMethods_method15() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "public boolean Object.m( Object o ) {" + NEWLINE +
                           "  return false;" + NEWLINE +
                           "}" + NEWLINE +
                           "public boolean Number.m( Object@Number n ) {" + 
                           NEWLINE +
                           "  return true;" + NEWLINE +
                           "}" + NEWLINE +
                           "public boolean String.m( Object@String n ) {" + 
                           "  return true;" + NEWLINE +
                           NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getGFsMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 8, character 15 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 32 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 32 warning: Parameter \"n\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 8, character 32 warning: Parameter \"n\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [m$anchor.<init>(  ), m$anchor.mjc$initializeCOR(  ), m$anchor.<clinit>(  ), m$anchor.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@java.lang.String, java.lang.Object@java.lang.String ), m( java.lang.Object@java.lang.Number, java.lang.Object@java.lang.Number ), m( java.lang.Object, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test three internal method of local external generic function
     * using multiple dispatch.  */
    public void test_dispatcherWrapMethods_method16() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "private boolean Object.m( Object o ) {" + NEWLINE +
                           "  return true;" + NEWLINE +
                           "}" + NEWLINE +
                           "public class Sample {" + NEWLINE +
                           "  private boolean m( Object o ) {" + NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  private boolean m( Object@Number o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "  private boolean m( Object@Sample o ) {" + 
                           NEWLINE +
                           "    return false;" + NEWLINE +
                           "  }" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 13 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 8 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 33 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 28 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 9, character 28 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 12, character 28 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 10 warning: Private method \"m\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 9, character 10 warning: Private method \"m\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 12, character 10 warning: Private method \"m\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [Sample.<init>(  ), Sample.mjc$initializeCOR(  ), Sample.<clinit>(  ), Sample.Block$(  ), /* grouped */ m( java.lang.Object, java.lang.Object ) [m( java.lang.Object@Sample, java.lang.Object@Sample ), m( java.lang.Object@Sample, java.lang.Object@java.lang.Number ), m( java.lang.Object@Sample, java.lang.Object )]] }", wrappedMethods.toString(), true );
    }

    /**
     * Test pleomorphic method of internal and external generic
     * function.  */
    public void test_dispatcherWrapMethods_method17() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "public void Number.m() { }" + NEWLINE +
                           "public abstract class Sample extends Number" + 
                           NEWLINE +
                           "    implements Inter {" + NEWLINE +
                           "  public void m() { }" + NEWLINE +
                           "}" + NEWLINE +
                           "interface Inter {" + NEWLINE +
                           "  void m();" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 12 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 3, character 22 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [/* redirected */ Sample.m(  ), Sample.<init>(  ), Sample.mjc$initializeCOR(  ), Sample.<clinit>(  ), Sample.Block$(  ), /* grouped */ m( java.lang.Number ) [m( java.lang.Number@Sample )]] }", wrappedMethods.toString(), true );
    }


    /**
     * Test multiple pleomorphic method of internal and external generic
     * function.  */
    public void test_dispatcherWrapMethods_method18() 
        throws RecognitionException, TokenStreamException
    {
        JCompilationUnitType unit = 
            getCheckedAST( "" + NEWLINE +
                           "public void Number.m( Number o ) { }" + NEWLINE +
                           "public abstract class Sample extends Number" + 
                           NEWLINE +
                           "    implements Inter {" + NEWLINE +
                           "  public void m( Number o ) { }" + NEWLINE +
                           "  public void m( Number@Integer o ) { }" + 
                           NEWLINE +
                           "}" + NEWLINE +
                           "interface Inter {" + NEWLINE +
                           "  void m( Number o );" + NEWLINE +
                           "}" );
        CMethodSet wrappedMethods = getTypesMethods( unit );
        assertEquals( "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 12 warning: Method \"m\" is public and should be defined in a file with same name and not \"unknown_source_file\" [CLCM 00, 2.2.3]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 3, character 22 warning: Class \"Sample\" is public and should be defined in a file with same name and not \"unknown_source_file\" [JLS 7.6]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 7 notice: There is no package declaration for this compilation unit [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 5, character 24 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 6, character 24 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE + "org.multijava.util.compiler.CWarning: File \"unknown_source_file\", line 2, character 29 warning: Parameter \"o\" is not used [KOPI]" + NEWLINE, warnings(), true );
        assertEquals( "CMethodSet{ [/* redirected */ Sample.m( java.lang.Number ), Sample.<init>(  ), Sample.mjc$initializeCOR(  ), Sample.<clinit>(  ), Sample.Block$(  ), /* grouped */ m( java.lang.Number, java.lang.Number ) [m( java.lang.Number@Sample, java.lang.Number@java.lang.Integer ), m( java.lang.Number@Sample, java.lang.Number )]] }", wrappedMethods.toString(), true );
    }


    //---------------------------------------------------------------------
    // PRIVATE HELPERS
    //---------------------------------------------------------------------

    /**
     * Returns a typechecked AST for the given source code string.  */
    private JCompilationUnitType getCheckedAST( String sourceCode ) {
        try {
            JCompilationUnitType unit = parserHelper.getAST( sourceCode, false );
            unit.preprocessDependencies( parserHelper.compiler );
            unit.checkInterface( parserHelper.compiler );
            unit.checkInitializers( parserHelper.compiler );
            unit.typecheck( parserHelper.compiler );
            unit.translateMJ( parserHelper.compiler );
            return unit;
        } catch( RecognitionException e ) {
            throw new RuntimeException( e.getMessage() );
        } catch( TokenStreamException e ) {
            throw new RuntimeException( e.getMessage() );
        } catch( PositionedError e ) {
            throw new RuntimeException( e.getMessage() );
        }
    }

    /**
     * Returns a set of the methods that will be used for generating
     * the first type declaration in the given compilation unit.
     *
     * @param   unit    an typechecked and refactored AST for the 
     * compilation unit */
    private CMethodSet getTypesMethods( JCompilationUnitType unit ) {
        CSourceClass clazz = 
            ((JTypeDeclaration) unit.typeDeclarations()[0]).sourceClass;
        return methodsFromSourceClass( clazz );
    }

    /**
     * Returns a set of the methods that will be used for generating
     * the first external generic function in the given compilation
     * unit.
     *
     * @param   unit    an typechecked and refactored AST for the 
     * compilation unit */
    private CMethodSet getGFsMethods( JCompilationUnitType unit ) {
        CSourceClass clazz = unit.gfDeclarations()[0].sourceClass;
        return methodsFromSourceClass( clazz );
    }

    private CMethodSet methodsFromSourceClass( CSourceClass clazz ) {
        CMethodSet result = clazz.methodsForCodeGen();
        CClassType[] inners = clazz.getInnerClasses();
        for (int i = 0; i < inners.length; i++) {
            CClass c = inners[i].getCClass();
            if (c instanceof CSourceDispatcherClass) {
                result.addMethod( ((CSourceDispatcherClass) c).
                                  dispatcherMethod );
            } // end of if 
        } // end of for

        return result;
    }


    /**
     * Returns the warnings generated by the must recent compilation
     * attempt.  */
    private String warnings() {
        return parserHelper.errorOut.toString();
    }

    //---------------------------------------------------------------------
    // COMMON FIXTURE
    //---------------------------------------------------------------------

    public void setUp() {     
        ClassPath.init( System.getProperty( "java.class.path" ) , null);
        parserHelper.compiler.initSession();
    }

    public void tearDown() { 
        parserHelper.compiler.initSession();
        classes.clear();
    }

    private ArrayList classes = new ArrayList();

    private TestMjcParser parserHelper;
}
