/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestMain_CodeGen.java,v 1.11 2005/07/21 01:37:33 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.testing.TestCase;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;

import java.util.StringTokenizer;

public class TestMain_CodeGen extends FunctionalTestSuite.TestCase {

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------
    
    public TestMain_CodeGen( String name ) {
	super( name );
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // Some simple self-tests
    //---------------------------------------------------------------------

    /**
     * Should successfully compile and match the (very simple) source code. */
    public void test_simple_class_1() {
	String className = "SimpleClass1";
	String filename = className + ".java";
	try {
	    CompilationResults compRes = 
		compile( saveAs( filename,
				 "public class SimpleClass1 {" + NEWLINE +
				 "  public static void main( String[] args ) {"
				 + NEWLINE +
				 "    System.out.print( \"Hello, world!\" );" 
				 + NEWLINE +
				 "  }" + NEWLINE +
				 "}" 
				 ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes = disasm( className, true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut, true );
	    assertEquals("@source \"SimpleClass1.java\"" + NEWLINE +
			 "public class SimpleClass1 extends java.lang.Object {" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * main" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  2" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public static void main(java.lang.String[]) {" + NEWLINE +
			 "    @line  3" + NEWLINE +
			 "      @getstatic  java.io.PrintStream java.lang.System.out" + NEWLINE +
			 "      @const    \"Hello, world!\"" + NEWLINE +
			 "      @invokevirtual  void java.io.PrintStream.print(java.lang.String)" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * <init>" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  1" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public void <init>() {" + NEWLINE +
			 "    @line  1" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( filename );
	}
    }


    // -------------------------------------------------------------
    // string literals
    // -------------------------------------------------------------

    public void test_stringLiterals_1() {
	try {
	    CompilationResults compRes = 
		compile( saveAs( "StringLit1.java",
				 "public class StringLit1 {" + NEWLINE +
				 "  void m() {" + NEWLINE +
				 "    System.out.println( \"x\\ny\" );" + NEWLINE +
				 "  }" + NEWLINE +
				 "}" ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes;

	    // check anchor
	    cgRes = disasm( "StringLit1", true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertEquals(
			 "@source \"StringLit1.java\"" + NEWLINE +
			 "public class StringLit1 extends java.lang.Object {" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * m" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  2" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  void m() {" + NEWLINE +
			 "    @line  3" + NEWLINE +
			 "      @getstatic  java.io.PrintStream java.lang.System.out" + NEWLINE +
			 "      @const    \"x\\ny\"" + NEWLINE +
			 "      @invokevirtual  void java.io.PrintStream.println(java.lang.String)" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * <init>" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  1" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public void <init>() {" + NEWLINE +
			 "    @line  1" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( "StringLit1.java" );
	}
    }

    public void test_stringLiterals_2() {
	try {
	    CompilationResults compRes = 
		compile( saveAs( "StringLit2.java",
				 "public class StringLit2 {" + NEWLINE +
				 "  void m() {" + NEWLINE +
				 "    System.out.println( \"x\\\"\" );" + NEWLINE +
				 "  }" + NEWLINE +
				 "}" ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes;

	    // check anchor
	    cgRes = disasm( "StringLit2", true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertEquals(
			 "@source \"StringLit2.java\"" + NEWLINE +
			 "public class StringLit2 extends java.lang.Object {" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * m" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  2" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  void m() {" + NEWLINE +
			 "    @line  3" + NEWLINE +
			 "      @getstatic  java.io.PrintStream java.lang.System.out" + NEWLINE +
			 "      @const    \"x\\\"\"" + NEWLINE +
			 "      @invokevirtual  void java.io.PrintStream.println(java.lang.String)" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * <init>" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  1" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public void <init>() {" + NEWLINE +
			 "    @line  1" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( "StringLit2.java" );
	}
    }

    // -------------------------------------------------------------
    // constructors
    // -------------------------------------------------------------

    /**
     * Must generate default constructors.  */
    public void test_constructor_1() {
	try {
	    CompilationResults compRes = 
		compile( saveAs("Constructor1.java",
				 "public class Constructor1 {" + NEWLINE +
				 "}" ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes = disasm( "Constructor1", true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertDiff(
"@source \"Constructor1.java\"" + NEWLINE +
"public class Constructor1 extends java.lang.Object {" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * <init>" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  1" + NEWLINE +
"   * @locals  1" + NEWLINE +
"   */" + NEWLINE +
"  public void <init>() {" + NEWLINE +
"    @line  1" + NEWLINE +
"      @aload    0" + NEWLINE +
"      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
"      @return    " + NEWLINE +
"  }" + NEWLINE +
"}" + NEWLINE, cgRes.stdOut );
	} finally {
	    remove("Constructor1.java" );
	}
    }


    //---------------------------------------------------------------------
    // Invocations
    //---------------------------------------------------------------------

    /**
     * Should successfully compile an invocation of an internal
     * generic function. */
    public void test_invocation_1() {
		String className = "Invocation1";
	String filename = className + ".java";
	try {
	    CompilationResults compRes = 
		compile( saveAs( filename,
				 "public class Invocation1 {" + NEWLINE +
				 "  public Invocation1() {" + NEWLINE +
				 "    m( 1 );" + NEWLINE +
				 "  }" + NEWLINE +
				 "  public void m( int i ) { }" + NEWLINE +
				 "}" ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes = disasm( className, true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertEquals(
			 "@source \"Invocation1.java\"" + NEWLINE +
			 "public class Invocation1 extends java.lang.Object {" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * <init>" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  2" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public void <init>() {" + NEWLINE +
			 "    @line  2" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
			 "    @line  3" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @const    1" + NEWLINE +
			 "      @invokevirtual  void Invocation1.m(int)" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * m" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  0" + NEWLINE +
			 "   * @locals  2" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public void m(int) {" + NEWLINE +
			 "    @line  5" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( filename );
	}
    }

    /**
     * Should successfully compile an invocation of an external
     * generic function. */
    public void test_invocation_2() {
	String className = "Invocation2";
	String filename = className + ".java";
	try {
	    CompilationResults compRes = 
		compile( saveAs( filename,
				 "" + NEWLINE +
				 "public class Invocation2 {" + NEWLINE +
				 "  public Invocation2() {" + NEWLINE +
				 "    invocation2Func( 1 );" + NEWLINE +
				 "  }" + NEWLINE +
				 "}" + NEWLINE +
				 "void Object.invocation2Func( int i ) { }" ) );
	    expectSuccess( compRes );
	    assertEquals( "File \"Invocation2.java\", line 4, character 25 caution: This self call appears in a constructor or initializer and targets an external generic function \"invocation2Func\".  If a subclass extends the generic function, then the dispatch code may not be fully initialized when this call is executed. [MultiJava]" + NEWLINE, compRes.errorOut );
	    CodeGenResults cgRes = disasm( className, true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertEquals( 
			 "@source \"Invocation2.java\"" + NEWLINE +
			 "public class Invocation2 extends java.lang.Object {" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * <init>" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  3" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public void <init>() {" + NEWLINE +
			 "    @line  3" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
			 "    @line  4" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokestatic  invocation2Func$anchor$signature0 invocation2Func$anchor.getFunction0()" + NEWLINE +
			 "      @swap    " + NEWLINE +
			 "      @const    1" + NEWLINE +
			 "      @invokeinterface  void invocation2Func$anchor$signature0.apply(java.lang.Object, int) 3" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( filename );
	}
    }

    /**
     * Should invoke the correct method when overriding method changes
     * privileged access level. */
    public void test_invocation_3() {
	try {
	    CompilationResults compRes = 
		compile( saveAs( "Invocation3.java",
				 "import java.util.HashMap;" + NEWLINE +
				 "public class Invocation3 {" + NEWLINE +
				 "  void m() {" + NEWLINE +
				 "    map = (HashMap) map.clone();" + NEWLINE +
				 "  }" + NEWLINE +
				 "  HashMap map = new HashMap();" + NEWLINE +
				 "}" ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes;

	    // check anchor
	    cgRes = disasm( "Invocation3", true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertEquals(
			 "@source \"Invocation3.java\"" + NEWLINE +
			 "public class Invocation3 extends java.lang.Object {" + NEWLINE +
			 "  java.util.HashMap map;" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * m" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  2" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  void m() {" + NEWLINE +
			 "    @line  4" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @getfield  java.util.HashMap Invocation3.map" + NEWLINE +
			 "      @invokevirtual  java.lang.Object java.util.HashMap.clone()" + NEWLINE +
			 "      @checkcast  java.util.HashMap" + NEWLINE +
			 "      @putfield  java.util.HashMap Invocation3.map" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * <init>" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  1" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  public void <init>() {" + NEWLINE +
			 "    @line  2" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @invokespecial  void Invocation3.Block$()" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "" + NEWLINE +
			 "  /**" + NEWLINE +
			 "   * Block$" + NEWLINE +
			 "   *" + NEWLINE +
			 "   * @stack  3" + NEWLINE +
			 "   * @locals  1" + NEWLINE +
			 "   */" + NEWLINE +
			 "  private void Block$() {" + NEWLINE +
			 "    @line  6" + NEWLINE +
			 "      @aload    0" + NEWLINE +
			 "      @new    java.util.HashMap" + NEWLINE +
			 "      @dup    " + NEWLINE +
			 "      @invokespecial  void java.util.HashMap.<init>()" + NEWLINE +
			 "      @putfield  java.util.HashMap Invocation3.map" + NEWLINE +
			 "      @return    " + NEWLINE +
			 "  }" + NEWLINE +
			 "}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( "Invocation3.java" );
	}
    }

    //---------------------------------------------------------------------
    // External generic function anchors and signatures
    //---------------------------------------------------------------------

    // see testcase/codegen

    //---------------------------------------------------------------------
    // Dispatchers
    //---------------------------------------------------------------------

    /**
     * Should successfully compile an internal dispatcher for the top of a
     * generic function lattice. */
    public void test_dispatcher_1() {
	try {	   
	    CompilationResults compRes = 
		compile( saveAs( "Dispatcher1.java",
				 "public class Dispatcher1 {" + NEWLINE + 
				 "  boolean m( Object o ) {" + NEWLINE + 
				 "    return false;" + NEWLINE + 
				 "  }" + NEWLINE + 
				 "  boolean m( Object@Dispatcher1 d ) {" + 
				 NEWLINE + 
				 "    return true;" + NEWLINE + 
				 "  }" + NEWLINE + 
				 "}" ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes = disasm( "Dispatcher1", true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertEquals( 
"@source \"Dispatcher1.java\"" + NEWLINE +
"public class Dispatcher1 extends java.lang.Object {" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * m" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  2" + NEWLINE +
"   * @locals  2" + NEWLINE +
"   */" + NEWLINE +
"  boolean m(java.lang.Object) {" + NEWLINE +
"    @line  2" + NEWLINE +
"      @aload    1" + NEWLINE +
"      @instanceof  Dispatcher1" + NEWLINE +
"      @ifeq    _L8" + NEWLINE +
"      @aload    0" + NEWLINE +
"      @aload    1" + NEWLINE +
"      @checkcast  Dispatcher1" + NEWLINE +
"      @invokespecial  boolean Dispatcher1.m$body2$0(Dispatcher1)" + NEWLINE +
"      @ireturn  " + NEWLINE +
"    _L8:  @aload    0" + NEWLINE +
"      @aload    1" + NEWLINE +
"      @invokespecial  boolean Dispatcher1.m$body2$1(java.lang.Object)" + NEWLINE +
"      @ireturn  " + NEWLINE +
"  }" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * m$body2$0" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  1" + NEWLINE +
"   * @locals  2" + NEWLINE +
"   */" + NEWLINE +
"  private boolean m$body2$0(Dispatcher1) {" + NEWLINE +
"    @line  6" + NEWLINE +
"      @const    1" + NEWLINE +
"      @ireturn  " + NEWLINE +
"  }" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * m$body2$1" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  1" + NEWLINE +
"   * @locals  2" + NEWLINE +
"   */" + NEWLINE +
"  private boolean m$body2$1(java.lang.Object) {" + NEWLINE +
"    @line  3" + NEWLINE +
"      @const    0" + NEWLINE +
"      @ireturn  " + NEWLINE +
"  }" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * <init>" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  1" + NEWLINE +
"   * @locals  1" + NEWLINE +
"   */" + NEWLINE +
"  public void <init>() {" + NEWLINE +
"    @line  1" + NEWLINE +
"      @aload    0" + NEWLINE +
"      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
"      @return    " + NEWLINE +
"  }" + NEWLINE +
"}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( "Dispatcher1.java" );
	}
    }

    /**
     * Should successfully compile an internal dispatcher for a
     * non-local generic function. */
    public void test_dispatcher_2() {
	try {	   
	    CompilationResults compRes = 
		compile( saveAs( "Dispatcher2.java",
				 "public class Dispatcher2 {" + NEWLINE + 
				 "  public int val;" + NEWLINE +
				 "  public boolean equals( Object@Dispatcher2 d ) {" 
				 + NEWLINE + 
				 "    return val == d.val;" + NEWLINE + 
				 "  }" + NEWLINE + 
				 "}" ) );
	    expectSuccess( compRes );
	    assertEquals( "", compRes.errorOut );
	    CodeGenResults cgRes = disasm( "Dispatcher2", true );
	    expectSuccess( cgRes );
	    assertEquals( "", cgRes.errorOut );
	    assertEquals(
"@source \"Dispatcher2.java\"" + NEWLINE +
"public class Dispatcher2 extends java.lang.Object {" + NEWLINE +
"  public int val;" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * equals" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  2" + NEWLINE +
"   * @locals  2" + NEWLINE +
"   */" + NEWLINE +
"  public boolean equals(java.lang.Object) {" + NEWLINE +
"    @line  3" + NEWLINE +
"      @aload    1" + NEWLINE +
"      @instanceof  Dispatcher2" + NEWLINE +
"      @ifeq    _L8" + NEWLINE +
"      @aload    0" + NEWLINE +
"      @aload    1" + NEWLINE +
"      @checkcast  Dispatcher2" + NEWLINE +
"      @invokespecial  boolean Dispatcher2.equals$body1$0(Dispatcher2)" + NEWLINE +
"      @ireturn  " + NEWLINE +
"    _L8:  @aload    0" + NEWLINE +
"      @aload    1" + NEWLINE +
"      @invokespecial  boolean java.lang.Object.equals(java.lang.Object)" + NEWLINE +
"      @ireturn  " + NEWLINE +
"  }" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * equals$body1$0" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  2" + NEWLINE +
"   * @locals  2" + NEWLINE +
"   */" + NEWLINE +
"  private boolean equals$body1$0(Dispatcher2) {" + NEWLINE +
"    @line  4" + NEWLINE +
"      @aload    0" + NEWLINE +
"      @getfield  int Dispatcher2.val" + NEWLINE +
"      @aload    1" + NEWLINE +
"      @getfield  int Dispatcher2.val" + NEWLINE +
"      @if_icmpeq  _L7" + NEWLINE +
"      @const    0" + NEWLINE +
"      @ireturn  " + NEWLINE +
"    _L7:  @const    1" + NEWLINE +
"      @ireturn  " + NEWLINE +
"  }" + NEWLINE +
"" + NEWLINE +
"  /**" + NEWLINE +
"   * <init>" + NEWLINE +
"   *" + NEWLINE +
"   * @stack  1" + NEWLINE +
"   * @locals  1" + NEWLINE +
"   */" + NEWLINE +
"  public void <init>() {" + NEWLINE +
"    @line  1" + NEWLINE +
"      @aload    0" + NEWLINE +
"      @invokespecial  void java.lang.Object.<init>()" + NEWLINE +
"      @return    " + NEWLINE +
"  }" + NEWLINE +
"}" + NEWLINE, cgRes.stdOut, true );
	} finally {
	    remove( "Dispatcher2.java" );
	}
    }

    //---------------------------------------------------------------------
    // PRIVATE HELPERS
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // COMMON FIXTURE
    //---------------------------------------------------------------------

    // public void setUp() { }

    // public void tearDown() { }

    private static final String PACKAGE = "runtime";
    private static final String PACKAGE_DECL = 
	"package " + PACKAGE + ";" + NEWLINE;
}
