/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestMain_TransCompile.java,v 1.5 2004/01/16 23:21:12 cclifton Exp $
 */

package org.multijava.mjc;

import org.multijava.util.classfile.ClassPath;

import org.multijava.util.testing.TestCase;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;

import java.util.StringTokenizer;


/**
 * This class includes test cases for the transitive compilation of
 * referenced files.  */
public class TestMain_TransCompile extends FunctionalTestSuite.TestCase {

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------
    
    public TestMain_TransCompile( String name ) {
	super( name );
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------

    // -------------------------------------------------------------
    // unnamed package
    // -------------------------------------------------------------

    /**
     * Test simple reference. */
    public void test_unnamedPackage_1() {
	try {
	    remove( "UnnamedPackage1A.class UnnamedPackage1B.class" );
	    saveAs( "UnnamedPackage1A.java",
		    "public class UnnamedPackage1A {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"got it\");" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    saveAs( "UnnamedPackage1B.java",
		    "public class UnnamedPackage1B {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage1A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    CompilationResults compRes = 
		compile( "UnnamedPackage1B.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage1B" );
	    expectSuccess( runRes );
	    assertEquals( "got it", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage1A.java UnnamedPackage1B.java" );
	}
    }


    /**
     * Test mutually recursive reference.
     */
    public void test_unnamedPackage_2() {
	try {
	    remove( "UnnamedPackage2A.class UnnamedPackage2B.class" );
	    saveAs( "UnnamedPackage2A.java",
		    "public class UnnamedPackage2A {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage2B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    saveAs( "UnnamedPackage2B.java",
		    "public class UnnamedPackage2B {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage2A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"got it again\");" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    CompilationResults compRes = 
		compile( "UnnamedPackage2B.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage2B" );
	    expectSuccess( runRes );
	    assertEquals( "got it again", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage2A.java UnnamedPackage2B.java" );
	}
    }

    /**
     * Test mutually recursive reference starting compilation with the
     * other member of the pair.  */
    public void test_unnamedPackage_3() {
	try {
	    remove( "UnnamedPackage3A.class UnnamedPackage3B.class" );
	    saveAs( "UnnamedPackage3A.java",
		    "public class UnnamedPackage3A {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage3B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    saveAs( "UnnamedPackage3B.java",
		    "public class UnnamedPackage3B {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage3A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"and again\");" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    CompilationResults compRes = 
		compile( "UnnamedPackage3A.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage3B" );
	    expectSuccess( runRes );
	    assertEquals( "and again", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage3A.java UnnamedPackage3B.java" );
	}
    }

    /**
     * Test transitive references.  */
    public void test_unnamedPackage_4() {
	try {
	    remove( "UnnamedPackage4A.class UnnamedPackage4B.class " +
		    "UnnamedPackage4C.class " );

	    saveAs( "UnnamedPackage4A.java",
		    "public class UnnamedPackage4A {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage4B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage4B.java",
		    "public class UnnamedPackage4B {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage4C.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage4C.java",
		    "public class UnnamedPackage4C {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"transitive works\");" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    CompilationResults compRes = 
		compile( "UnnamedPackage4A.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage4A" );
	    expectSuccess( runRes );
	    assertEquals( "transitive works", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage4A.java UnnamedPackage4B.java " +
		    "UnnamedPackage4C.java " );
	}
    }

    /**
     * Test transitive references with mutual recursion. */
    public void test_unnamedPackage_5() {
	try {
	    remove( "UnnamedPackage5A.class UnnamedPackage5B.class " +
		    "UnnamedPackage5C.class " );

	    saveAs( "UnnamedPackage5A.java",
		    "public class UnnamedPackage5A {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage5B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"transitive recursion works\");" + 
		    NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage5B.java",
		    "public class UnnamedPackage5B {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage5C.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage5C.java",
		    "public class UnnamedPackage5C {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage5A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    CompilationResults compRes = 
		compile( "UnnamedPackage5A.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage5A" );
	    expectSuccess( runRes );
	    assertEquals( "transitive recursion works", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage5A.java UnnamedPackage5B.java " +
		    "UnnamedPackage5C.java " );
	}
    }

    /**
     * Test transitive references with mutual recursion using a
     * different starting class for the compilation session. */
    public void test_unnamedPackage_6() {
	try {
	    remove( "UnnamedPackage6A.class UnnamedPackage6B.class " +
		    "UnnamedPackage6C.class " );

	    saveAs( "UnnamedPackage6A.java",
		    "public class UnnamedPackage6A {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage6B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"transitive recursion works\");" + 
		    NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage6B.java",
		    "public class UnnamedPackage6B {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage6C.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage6C.java",
		    "public class UnnamedPackage6C {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage6A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    CompilationResults compRes = 
		compile( "UnnamedPackage6B.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage6A" );
	    expectSuccess( runRes );
	    assertEquals( "transitive recursion works", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage6A.java UnnamedPackage6B.java " +
		    "UnnamedPackage6C.java " );
	}
    }

    /**
     * Test transitive references with mutual recursion using yet
     * another starting class for the compilation session. */
    public void test_unnamedPackage_7() {
	try {
	    remove( "UnnamedPackage7A.class UnnamedPackage7B.class " +
		    "UnnamedPackage7C.class " );

	    saveAs( "UnnamedPackage7A.java",
		    "public class UnnamedPackage7A {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage7B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"transitive recursion works\");" + 
		    NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage7B.java",
		    "public class UnnamedPackage7B {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage7C.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage7C.java",
		    "public class UnnamedPackage7C {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage7A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    CompilationResults compRes = 
		compile( "UnnamedPackage7C.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage7A" );
	    expectSuccess( runRes );
	    assertEquals( "transitive recursion works", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage7A.java UnnamedPackage7B.java " +
		    "UnnamedPackage7C.java " );
	}
    }

    /**
     * Test transitive references with mutual recursion using two
     * starting classes for the compilation session. */
    public void test_unnamedPackage_8() {
	try {
	    remove( "UnnamedPackage8A.class UnnamedPackage8B.class " +
		    "UnnamedPackage8C.class " );

	    saveAs( "UnnamedPackage8A.java",
		    "public class UnnamedPackage8A {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage8B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"transitive recursion works\");" + 
		    NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage8B.java",
		    "public class UnnamedPackage8B {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage8C.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage8C.java",
		    "public class UnnamedPackage8C {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage8A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    CompilationResults compRes = 
		compile( "UnnamedPackage8A.java UnnamedPackage8B.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage8A" );
	    expectSuccess( runRes );
	    assertEquals( "transitive recursion works", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage8A.java UnnamedPackage8B.java " +
		    "UnnamedPackage8C.java " );
	}
    }

    /**
     * Test transitive references with mutual recursion using two
     * different starting classes for the compilation session. */
    public void test_unnamedPackage_9() {
	try {
	    remove( "UnnamedPackage9A.class UnnamedPackage9B.class " +
		    "UnnamedPackage9C.class " );

	    saveAs( "UnnamedPackage9A.java",
		    "public class UnnamedPackage9A {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage9B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"transitive recursion works\");" + 
		    NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage9B.java",
		    "public class UnnamedPackage9B {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage9C.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage9C.java",
		    "public class UnnamedPackage9C {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage9A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    CompilationResults compRes = 
		compile( "UnnamedPackage9A.java UnnamedPackage9C.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage9A" );
	    expectSuccess( runRes );
	    assertEquals( "transitive recursion works", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage9A.java UnnamedPackage9B.java " +
		    "UnnamedPackage9C.java " );
	}
    }

    /**
     * Test transitive references with mutual recursion using two
     * different starting classes for the compilation session. */
    public void test_unnamedPackage_10() {
	try {
	    remove( "UnnamedPackage10A.class UnnamedPackage10B.class " +
		    "UnnamedPackage10C.class " );

	    saveAs( "UnnamedPackage10A.java",
		    "public class UnnamedPackage10A {" + NEWLINE +
		    "  public static void main( String[] args ) {" + NEWLINE +
		    "    UnnamedPackage10B.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    System.out.print(\"transitive recursion works\");" + 
		    NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage10B.java",
		    "public class UnnamedPackage10B {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage10C.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );

	    saveAs( "UnnamedPackage10C.java",
		    "public class UnnamedPackage10C {" + NEWLINE +
		    "  public static void m() {" + NEWLINE +
		    "    UnnamedPackage10A.m();" + NEWLINE +
		    "  }" + NEWLINE +
		    "}" + NEWLINE );
	    CompilationResults compRes = 
		compile( "UnnamedPackage10B.java UnnamedPackage10C.java" );
	    expectSuccess( compRes );
	    RuntimeResults runRes = run( "UnnamedPackage10A" );
	    expectSuccess( runRes );
	    assertEquals( "transitive recursion works", runRes.stdOut, true );
	} finally {
	    remove( "UnnamedPackage10A.java UnnamedPackage10B.java " +
		    "UnnamedPackage10C.java " );
	}
    }

    // -------------------------------------------------------------
    // Named package, implicit package references
    // -------------------------------------------------------------

    //---------------------------------------------------------------------
    // PRIVATE HELPERS
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // COMMON FIXTURE
    //---------------------------------------------------------------------

//      public void setUp() { }

//      public void tearDown() { }

}
