package org.multijava.mjc.testcase.java5.generics;

import java.math.BigInteger;

/*
 * Make sure that capture conversion is applied whenever needed, i.e. whenever
 * a type is evaluated. There are 7 types of expressions (eventually with
 * different flavors such as with the name expression) to which capture conversion
 * must be applied:
 * - name expression (local variable, method parameter, this field)
 * - field access expression
 * - array access expression
 * - method call expression
 * - type cast expression
 * - assignment expression
 * - conditional expression
 *
 * Note that this is basically identical to the WildcardCapture6OK unit test
 * with the only difference that we try to assign the wildcard capture to an
 * Integer instead of a BigInteger which is not correct.
 */
public class WildcardCapture6Fail<T extends BigInteger> {

    T t;

    WildcardCapture6Fail<?> field;

    public WildcardCapture6Fail<?> foo(WildcardCapture6Fail<?> param) {
        Integer integer;

        // check name expression ==> local variable
        WildcardCapture6Fail<?> local = null;
        integer = local.t;

        // check name expression ==> method parameter
        integer = param.t;

        // check name expression ==> this.field
        integer = field.t;

        // check field access
        WildcardCapture6Fail<?> wc = null;
        integer = wc.field.t;

        // check array access
        WildcardCapture6Fail<?>[] wcArray = null;
        integer = wcArray[0].t;

        // check method call
        integer = foo(null).t;

        // check cast expression
        WildcardCapture6Fail<BigInteger> tmp = null;
        integer = ((WildcardCapture6Fail<?>) tmp).t;

        // check assignment expression
        integer = (local = null).t;

        // check conditional expression
        boolean bool = false;
        integer = (bool ? local : field).t;

        return null;
    }
}
