package org.multijava.mjc.testcase.java5.generics;

import java.math.BigInteger;

/*
 * Make sure that capture conversion is applied whenever needed, i.e. whenever
 * a type is evaluated. There are 7 types of expressions (eventually with
 * different flavors such as with the name expression) to which capture conversion
 * must be applied:
 * - name expression (local variable, method parameter, this field)
 * - field access expression
 * - array access expression
 * - method call expression
 * - type cast expression
 * - assignment expression
 * - conditional expression
 */
public class WildcardCapture6OK<T extends BigInteger> {

    T t;

    WildcardCapture6OK<?> field;

    public WildcardCapture6OK<?> foo(WildcardCapture6OK<?> param) {
        BigInteger bigInteger;

        // check name expression ==> local variable
        WildcardCapture6OK<?> local = null;
        bigInteger = local.t;

        // check name expression ==> method parameter
        bigInteger = param.t;

        // check name expression ==> this.field
        bigInteger = field.t;

        // check field access
        WildcardCapture6OK<?> wc = null;
        bigInteger = wc.field.t;

        // check array access
        WildcardCapture6OK<?>[] wcArray = null;
        bigInteger = wcArray[0].t;

        // check method call
        bigInteger = foo(null).t;

        // check cast expression
        WildcardCapture6OK<BigInteger> tmp = null;
        bigInteger = ((WildcardCapture6OK<?>) tmp).t;

        // check assignment expression
        bigInteger = (local = null).t;

        // check conditional expression
        boolean bool = false;
        bigInteger = (bool ? local : field).t;

        return null;
    }
}
