/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjConstructorDoc.java,v 1.1 2004/01/14 11:05:44 davidcok Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_141;
import com.sun.javadoc.*;
import org.multijava.mjc.CMethod;
import org.multijava.mjc.JMethodDeclarationType;

/** This class represents a constructor of a class; it is the bridge between the MJC syntax tree
    object and the ConstructorDoc interface of javadoc doclets.  It wraps a 
    JMethodDeclaration object for the constructor if source code is available; otherwise it
    uses the corresponding CMethod object.
    @author David R. Cok
 */
public class MjConstructorDoc extends MjExecutableMemberDoc implements ConstructorDoc {

    /** Constructs a wrapper object for a CMethod; use when source code (i.e. a JMethodDeclarationType
	object) is not available.
	@param cm The CMethod object to be wrappped
	@param containiner The class that contains this constructor
    */
    public MjConstructorDoc(CMethod cm, MjClassDoc container) {
	super(cm,container);
    }

    /** Constructs a wrapper object for a method; use when source code is available;  
	using this constructor makes the javadoc information available.
	@param cm The CMethod object to be wrappped
	@param containiner The class that contains this constructor
    */
    public MjConstructorDoc(JMethodDeclarationType jm, MjClassDoc container) {
	super(jm,container);
    }
    
    /** Returns true since this object represents a constructor. */
    //@ pure
    public boolean isConstructor() { return true; }
    
    /** Returns the name of the class (i.e. the constructor identifier) along with any
	enclosing class names, but without the package name. // !FIXME! DOes not agree with Method or Executabale method
     */
    //@ pure
    public String name() { 
	return containingClass().name(); 
    }
    
    /** Returns the name of the class (i.e. the constructor identifier) along with any
	enclosing class names, but without the package name. 
     */
    public String toString() { 
	return containingClass().name(); 
    }

    /** Returns a String representation, used for ordering. */
    protected String compareString() {
	return containingClass().name() + signature();
    }
    

    /** Return the source position of the documented entity, or null
     *  if no position is available.
     *  Added for JDK 1.4 compatability.
     */
//      public SourcePosition position() {
//          return null; /* !FIXME!JDK 1.4 compatability */
//      }
}
