/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocClassWriter.java,v 1.1 2004/01/14 11:05:44 davidcok Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_141;

import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/** A derivative of a class in the javadoc doclet API, so as to be able to
    instantiate a MjdocClassWriter instead of a ClassWriter and a 
    MjdocMethodSubWriter instead of a MethodSubWriter.
    @author David R. Cok
*/
public class MjdocClassWriter extends ClassWriter {

    /** This generates an html page describing a class. */
    public static void generate(ConfigurationStandard configuration,
			    ClassDoc classdoc, ClassDoc prev,
                             ClassDoc next, ClassTree classtree,
                             boolean nopackage) {
            ClassWriter clsgen;
            String pkgpath =
               DirectoryManager.getDirectoryPath(classdoc.containingPackage());
            String filename = classdoc.name() + ".html";
            try {
                clsgen = new MjdocClassWriter(configuration,
					pkgpath, filename, classdoc,
                                         prev, next, classtree, nopackage);
                clsgen.generateClassFile();
		
		// FIXME - what does this do ????
		PackageDoc cp = classdoc.containingPackage();
		if((configuration.packages == null ||
			Arrays.binarySearch(configuration.packages, cp) < 0) &&
		    ! containingPackagesSeen.contains(cp.name())){
		    
		    //Only copy doc files dir if the containing package is not documented
		    //AND if we have not documented a class from the same package already.
		    //Otherwise, we are making duplicate copies.
		    clsgen.copyDocFiles(configuration, getSourcePath(configuration, classdoc.containingPackage()),
				pkgpath + fileseparator + DOC_FILES_DIR_NAME, true);
		    containingPackagesSeen.add(cp.name());
		}

                clsgen.close();
            } catch (IOException exc) {
                configuration.standardmessage.
                    error("doclet.exception_encountered",
                           exc.toString(), filename);
                throw new DocletAbortException();
            }
    }
    
    public MjdocClassWriter(ConfigurationStandard configuration, 
		String path, String filename, ClassDoc classdoc,
                ClassDoc prev, ClassDoc next, ClassTree classtree,
                boolean nopackage) throws IOException {
	super(configuration,path,filename,classdoc,prev,next,classtree,nopackage);
	methodSubWriter = new MjdocMethodSubWriter(this,classdoc,configuration);
	constrSubWriter = new MjdocConstructorSubWriter(this,classdoc);
    }

    /**
     * Print the html file header. Also print Html page title and stylesheet
     * default properties.
     * This is overridden just to alter the application name.
     *
     * @param title String title for the generated html file.
     * @param metakeywords String keywords for META tag.
     */
    public void printHeader(String title, String metakeywords) {
        println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 " +
                    "Transitional//EN\"" +
                    "\"http://www.w3.org/TR/REC-html40/loose.dtd\">");
        println("<!--NewPage-->");
        html();
        head();
        print("<!-- Generated by " + MjdocWrapper.appName() + " on ");
        print(today());
        println(" -->");
        if (configuration.charset.length() > 0) {
            println("<META http-equiv=\"Content-Type\" content=\"text/html; "
                        + "charset=" + configuration.charset + "\">");
        }
        if ( configuration.windowtitle.length() > 0 ) {
            title += " (" + configuration.windowtitle  + ")";
        }
        title(title);
        println(title);
        titleEnd();
        if ( metakeywords.length() > 0 ) {
            println("<META NAME=\"keywords\" "
                        + "CONTENT=\"" + metakeywords + "\">");
        } 
        printStyleSheetProperties();
        headEnd();
        body("white");
    }

	// FIXME - document
    public void printSummaryLinkComment(AbstractSubWriter mw,
                                        ProgramElementDoc member) {
        codeEnd();
        println(); br();
	//System.out.println("printSummaryLinkComment " + member.name());
	if (member instanceof MjMethodDoc && ((MjMethodDoc)member).isExternal()) {
	    print("<B>External</B>"); br();
	}
        printCommentDef(member);
        summaryRowEnd();
        trEnd();   
    }   

    /** This is used to generate a link to the file that contains the description
	of a external method unit.  It is used by the PREF and NEXT buttons in the
	header of a generic function page.
    */ // !FIXME! - cleanup
    public String getEMLink(MjMethodDoc cd, String where,
                               String label, boolean bold, String color) {
        boolean nameUnspecified = label.length() == 0;
        if (nameUnspecified) {
            label = cd.name();
        }       
        displayLength += label.length();
        //if (cd.isIncluded()) {
            //if (isGeneratedDoc(cd)) {
                String filename = cd.externalUnit().name() + ".html";
		filename = pathString(cd.cuPackage(), filename);
                return getHyperLink(filename, where, label, bold, color);
            //}
        //} else {
        //    String crosslink = getCrossClassLink(cd);
        //    if (crosslink != null) {
        //        return getHyperLink(crosslink, where, label, bold, color);
        //    }
       // }
        //if (nameUnspecified) {
        //    displayLength -= label.length();
        //    label = cd.qualifiedName();
        //    displayLength += label.length();
        //}            
        //return label;
    }
    
    public String getGFLink(MjMethodDoc cd, String where,
                               String label, boolean bold, String color) {
        boolean nameUnspecified = label.length() == 0;
        if (nameUnspecified) {
            label = cd.name();
        }       
        displayLength += label.length();
        if (cd.isIncluded()) {
            if (isGeneratedDoc(cd)) {
                String filename = cd.gfFileName();
		filename = pathString(cd.cuPackage(), filename);
                return getHyperLink(filename, where, label, bold, color);
            }
        } else {
        //    String crosslink = getCrossClassLink(cd);
        //    if (crosslink != null) {
        //        return getHyperLink(crosslink, where, label, bold, color);
        //    }
        }
        //if (nameUnspecified) {
        //    displayLength -= label.length();
        //    label = cd.qualifiedName();
        //    displayLength += label.length();
        //}            
        return label;
    }

}








