/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocMethodSubWriter.java,v 1.2 2004/07/01 22:16:47 leavens Exp $
 * Author: David R. Cok
 */
// FIXME - needs documentation in this file. 

package org.multijava.mjdoc.mjdoc_141;
import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.util.*;
import java.lang.reflect.Modifier;
import com.sun.tools.doclets.standard.tags.*;

/** This is an extension of the doclet api in order to provide
    functionality for writing external methods.
    @author David R. Cok
*/
public class MjdocMethodSubWriter extends MethodSubWriter {

    protected boolean inMethodHTMLPage = false; // false for a class page, true for comp unit or generic function page
    
    public boolean inGFHTMLPage = false; // false for a class or comp unit page, true for a gf page

    protected ConfigurationStandard configuration;
	
    public MjdocMethodSubWriter(SubWriterHolderWriter writer, ClassDoc classdoc, ConfigurationStandard config) {
        super(writer, classdoc, config);
	this.configuration = config;
    }

    public MjdocMethodSubWriter(SubWriterHolderWriter writer, ConfigurationStandard config) {
        super(writer,config);
	this.configuration = config;
    }
    
    public void printMembersSummary() { 
        List members = new ArrayList(members(classdoc));
	boolean something = members.size() > 0 ; // FIXME || (classdoc instanceof MjClassDoc && ((MjClassDoc)classdoc).tlmethods().size() > 0);

	if (something) printSummaryHeader(classdoc);
	
        if (members.size() > 0) {
            Collections.sort(members);
            for (int i = 0; i < members.size(); ++i) {
                MethodDoc member = (MethodDoc)members.get(i);
                boolean commentChanged = false;
                String prevRawComment = "";
                Tag[] tags = member.inlineTags(); 
                if (tags.length == 0) {
                    prevRawComment = member.getRawCommentText();
                    MethodDoc meth =
                         new CommentedMethodFinder().search(classdoc, member);
                    if (meth != null && (!(meth instanceof MjMethodDoc) || 
			    ! meth.containingClass().isClass() ||
			    ((MjMethodDoc)member).topConcreteMethod() == ((MjMethodDoc)meth).topConcreteMethod()  )) { 
				    //set raw comment text for now.
                        ((MjMethodDoc)member).setRawCommentText(meth.commentText(),((MjMethodDoc)meth).jmethod().getTokenReference());// FIXME- Test Position
                        commentChanged = true;
                    }
		}
                printSummaryMember(classdoc, member);
                if (commentChanged) {  // reset it to prevRawComment.
                  ((MjMethodDoc)member).setRawCommentText(prevRawComment,((MjMethodDoc)member).jmethod().getTokenReference()); // FIXME- Test Position
                }
            }
        }
	if (something) printSummaryFooter(classdoc);
    }

// FIXME - not inherited; what does it do
    public void printMethodsSummary(ArrayList methods) {
	inMethodHTMLPage = true;
	boolean something = methods.size() > 0;

	if (something) printSummaryHeader(classdoc); 
	
        if (methods.size() > 0) {
            //Collections.sort(members); // !FIXME!
            for (int i = 0; i < methods.size(); ++i) {
                MethodDoc method = (MethodDoc)methods.get(i);
                boolean commentChanged = false;
                String prevRawComment = "";
                Tag[] tags = method.inlineTags(); 
/*              if (tags.length == 0) {
                    prevRawComment = method.getRawCommentText();
                    MethodDoc meth =
                         new CommentedMethodFinder().search(classdoc, method);
                    if (meth != null) { //set raw comment text for now.
                        method.setRawCommentText(meth.commentText());
                        commentChanged = true;
                    }
                } */ // FIXME - insert comment
                printSummaryMember(classdoc, method);
                if (commentChanged) {  // reset it to prevRawComment.
                  ((MjMethodDoc)method).setRawCommentText(prevRawComment,method.position());// FIXME- Test Position
                }
            }
        }
	if (something) printSummaryFooter(classdoc); 
	inMethodHTMLPage = false;
    }

    public void printMembers() {
        List members = members(classdoc);
	boolean something = members.size() > 0 ;// FIXME  || (classdoc instanceof MjClassDoc && ((MjClassDoc)classdoc).tlmethods().size() > 0);
	if (something) printHeader(classdoc);
	boolean needsMemberHeader = false;
	
        if (members.size() > 0) {
            for (int i = 0; i < members.size(); ++i) {
                if (needsMemberHeader) { writer.printMemberHeader(); } else { needsMemberHeader = true; }
                writer.println("");
                printMember((ProgramElementDoc)(members.get(i)));
                writer.printMemberFooter();
            }
	}
        if (something) printBodyHtmlEnd(classdoc);
    }   
    
    public void printMethods(ArrayList methods) {
	inMethodHTMLPage = true;
	boolean something = methods.size() > 0;
	if (something) printHeader(classdoc);
	boolean needsMemberHeader = false;
	
        if (methods.size() > 0) {
            for (int i = 0; i < methods.size(); ++i) {
                if (needsMemberHeader) { writer.printMemberHeader(); } else { needsMemberHeader = true; }
                writer.println("");
                printMember((ProgramElementDoc)(methods.get(i)));
                writer.printMemberFooter();
            }
	}
        if (something) printBodyHtmlEnd(classdoc);
	inMethodHTMLPage = false;
    }   



    protected void printTypedName(Type type, String name) {
	if (type instanceof MjType && type != null) {
	    MjType mj = (MjType) type;
	    if (! mj.staticType().equals(mj.dynamicType())) {
		printTypeLink(new MjType(mj.staticType()));
		writer.print(mj.dynamicType().specializerSymbol());
		printTypeLink(new MjType(mj.dynamicType()));
		if (name.length() > 0) {
		    writer.space();
		    writer.print(name);
		}
		return;
	    }
	}
	if (type != null) {
	    printTypeLink(type);
	}
	if (name.length() > 0) {
	    writer.space();
	    writer.print(name);
	}
    }


    protected void printSummaryLink(ClassDoc cd, ProgramElementDoc member) {
        ExecutableMemberDoc emd = (ExecutableMemberDoc)member;
        String name = inMethodHTMLPage ? emd.qualifiedName() : emd.name();
	writer.bold();
	writer.printHyperLink("",name + emd.signature(),name,true);
	writer.boldEnd();
        writer.displayLength = name.length();
	printParameters(emd);
    }
    
    
    /** This prints the detail entry for a constructor or method in either a class
	page or a generic function page.
    */
    protected void printMember(ProgramElementDoc member) {
        ExecutableMemberDoc emd = (ExecutableMemberDoc)member;
        String name = inMethodHTMLPage ? emd.qualifiedName() : emd.name();
        writer.anchor(name + emd.signature());
        // The following is printHead(emd); 
	// but with a fully qualified name for gf pages	
	    writer.h3();
	    writer.print(name);
	    if (!inMethodHTMLPage && emd instanceof MjMethodDoc && ((MjMethodDoc)emd).isExternal()) {
		MjMethodDoc m = (MjMethodDoc)emd;
		print("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");

		String link;
		if (MjdocWrapper.docupages) link = ((MjdocClassWriter)writer).getEMLink(m,"",m.externalUnit().qualifiedName(),true,"");
		else                        link = m.externalUnit().qualifiedName();
		print("<FONT SIZE=-1>[external method declared in " + link + "]</FONT>");
	    }
	    if (inGFHTMLPage && emd instanceof MjMethodDoc && !((MjMethodDoc)emd).isExternal()) {
		MjMethodDoc m = (MjMethodDoc)emd;
		print("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");

		String link = m.containingClass().qualifiedName();
		print("<FONT SIZE=-1>[internal method declared in " + link + "]</FONT>");
	    }
	    writer.h3End();
        printSignature(emd);
        printFullComment(emd);
	
	if (inMethodHTMLPage) {
	    // Puts link to the class page at the bottom of the member detail entry on the gen. fcn. page
	    writer.printClassLink(member.containingClass(),emd.name()+emd.signature(),
			    "<B><FONT SIZE=\"-2\">CLASS VIEW</FONT></B>",true);
	    if (inGFHTMLPage && emd instanceof MjMethodDoc) {
		// FIXME - this will not work across packages
		if (MjdocWrapper.docupages && ((MjMethodDoc)emd).isExternal()) 
			writer.printHyperLink(emd.name() + ".html",emd.name()+emd.signature(),
			    " <B><FONT SIZE=\"-2\">EXTMETHOD</FONT></B>",true);
	    }
	} 
	if (!inGFHTMLPage && member instanceof MjMethodDoc) {
	    // Puts link to the generic function page at the bottom of the member detail entry
	    // on the class page

	    //!FIXME! Not sure this works across packages; only write the link if there are multiple members in the family
	    
	    // This makes a link to the top of the generic function page
	    MjMethodDoc top = ((MjMethodDoc)member).topConcreteMethod();
	    if (MjdocWrapper.doallfcns || (!MjdocWrapper.donofcns && top.isExternal())) {
		String link;
		if (writer instanceof MjdocClassWriter)
		    link = ((MjdocClassWriter)writer).getGFLink(top,""," <B><FONT SIZE=\"-2\">GENERIC FUNCTION VIEW</FONT></B>",true,"");
		else if (writer instanceof MjdocMethodWriter)
		    link = ((MjdocMethodWriter)writer).getGFLink(top,""," <B><FONT SIZE=\"-2\">GENERIC FUNCTION VIEW</FONT></B>",true,"");
		else {
		    System.out.println("UNKNOWNS WRITER " + writer.getClass());
		    link = "GENERIC FUNCTION VIEW";
		}
		writer.print(link);
	    }
	    
	    // This goes to the detail member in the generic function page
	    //writer.printHyperLink(MjdocGFWriter.methodFilename(top,true),
	    //	emd.qualifiedName() + emd.signature(),"<B><FONT SIZE=\"-2\">FCN</FONT></B>",true);
	}
    }
    
    // FIXME - still have a problem with @ type inheritance - there will be multiple overrides
    // This at least uses the signature from the overriden class so the link works
    protected void printOverridden(ClassDoc overridden, MethodDoc method) {
        if(configuration.nocomment){
            return;
        }
        String label = "doclet.Overrides";
        
        if (overridden != null) {
            if(overridden.isAbstract()){
                MethodDoc found = Util.findMethod(overridden, method);
                if(found != null && found.isAbstract()){
                    //Abstract method is implemented from abstract class,
                    //not overridden
                    label = "doclet.Specified_By";
                }
            }
            String overriddenclasslink = writer.codeText(
                writer.getClassLink(overridden));
            String methlink = "";
            String name = method.name();
            writer.dt();
            writer.boldText(label);
            writer.dd();
	    MjClassDoc mc = ((MjClassDoc)overridden);
	    MjMethodDoc mjm = (MjMethodDoc)method;
	    MjMethodDoc mm = mc.findMethod(mjm);
	    //if (mm.topConcreteMethod() != mjm.topConcreteMethod()) return; // FIXME - should we fix the determination of overridden???
            methlink = writer.codeText(writer.getClassLink(overridden,
                                       name + mm.signature(),
                                       name, false));
            writer.printText("doclet.in_class", methlink, overriddenclasslink);
        }
    }

// FIXME - do we need this for external functions ???
/*
// Original:
    protected void printComment(ProgramElementDoc member) {
        if(configuration.nocomment){
            return;
        }
        if (member.inlineTags().length > 0) {
            writer.dd();
            writer.printInlineComment(member);
        } else {
            MethodDoc method = new CommentedMethodFinder().
                search(member.containingClass(),
                           (MethodDoc)member);
            printCommentFromCommentedMethod(method);
        }
    }

MY JAVA 1.3 VERSION:
*//*    protected void printComment(ProgramElementDoc member) {
        if(configuration.nocomment){
            return;
        }
        if (member.inlineTags().length > 0) {
            writer.dd();
            writer.printInlineComment(member);
        } else {
            MethodDoc method = new CommentedMethodFinder().
                                            search(member.containingClass(),
                                                   (MethodDoc)member);
	    if (method == null || !(member instanceof MjMethodDoc) || 
		!method.containingClass().isClass() ||
		((MjMethodDoc)member).topConcreteMethod() == ((MjMethodDoc)method).topConcreteMethod())  {
		    printCommentFromCommentedMethod(method);  
	    }
        }
    }
*/

    /** Prints the html for the tag information.  This method is overridden from
	MethodSubWriter to fix this bug: when there are not throws tags but
	there are exceptions declared as thrown, the header for the throws tags is
	not placed in the html.  It is also overridden to clean up some formatting
	(blank lines) in the output.
    */
    protected void printTags(ProgramElementDoc member) {
        if(configuration.nocomment){
            return;
        }
        MethodDoc method = (MethodDoc)member;
        ClassDoc[] intfacs = member.containingClass().interfaces();
        ClassDoc overridden = method.overriddenClass();
	// The test on implementedMethod below is added to avoid adding spurious extra lines
	// in the output
        if ((intfacs.length > 0 && implementedMethod(method) != null) || overridden != null) {
            printTagsInfoHeader(); 
            printImplementsInfo(method);
            printOverridden(overridden, method);
            printTagsInfoFooter();
        }
        configuration.tagletManager.checkTags(member, member.tags(), false);
        configuration.tagletManager.checkTags(member, member.inlineTags(), true);
        Taglet[] definedTags = configuration.tagletManager.getMethodCustomTags();
        Tag[][] tagsInDoc = new Tag[definedTags.length][];
        for (int i = 0; i < definedTags.length; i++) {
            tagsInDoc[i] = method.tags(definedTags[i].getName());
        }
        String output;
	StringBuffer alloutput = new StringBuffer();
	boolean earlyHeader = anyParamTags(method);
	if (earlyHeader) printTagsInfoHeader();
        for (int i = 0; i < definedTags.length; i++) {
            if (definedTags[i] instanceof SimpleTaglet) {
                output = ((SimpleTaglet) definedTags[i]).toString(tagsInDoc[i], writer);
            } else if (definedTags[i] instanceof ThrowsTaglet && tagsInDoc[i].length == 0) {
		// This case is here solely to fix a bug in Apple/Sun's library.  The
		// header for the section of throws information is not otherwise printed
		// if there are declared exceptions but no throws tags.
                output = ((AbstractExecutableMemberTaglet) definedTags[i]).toString(method, writer);
		// The following string has to match the header used in ThrowsTaglet.java
		// The need is to add a header when it is incorrectly omitted.
                // !FIXME! should that \n be below?
		String header = "\n<DT>" + "<B>" + writer.getText("doclet.Throws") + "</B>";
		if (output != null && !output.startsWith(header))
		    output = header + output;
            } else if (definedTags[i] instanceof AbstractExecutableMemberTaglet) {
                output = ((AbstractExecutableMemberTaglet) definedTags[i]).toString(method, writer);
            } else {
                output = definedTags[i].toString(tagsInDoc[i]);
            }
            if (output != null && output.length() > 0) {
                configuration.tagletManager.seenCustomTag(definedTags[i].getName());
                alloutput.append(output); //print(output);
            }
        }
	output = alloutput.toString();
	// The writing to the output is combined here so as to avoid adding a header and
	// footer when there is no output.
	if (output.length() > 0) {
	    if (!earlyHeader) printTagsInfoHeader();
	    print(output);
	    printTagsInfoFooter();
	} else if (earlyHeader) {
	    printTagsInfoFooter();
	}
    }
    

    // Overridden in order to put in modifiers.
    protected void printParam(Parameter param) {
	String s = ((MjParameter)param).modString();
	if (s.length() != 0) writer.bold(s);
        printTypedName(param.type(), param.name());
    }

    // There is a design flaw in the printing of the param tags in that
    // the header is printed directly to the writer, but the rest of the
    // info is stored in the output String.  Thus above we need this 
    // helper function in order to determine if any output is going to be
    // written, so that we know whether to generate the printTagsInfoHeader
    protected boolean anyParamTags(ExecutableMemberDoc execHolder) {
        ParamTag[] tags = execHolder.paramTags();
        String result = "";
	if (execHolder.paramTags().length > 0) return true;
	return false;
    }

    // This is overridden simply to fix a bug in which the Method Summary header is
    // not written if there are no methods but only inherited methods.
    public void printInheritedMembersSummary() {
        for (int i = 0; i < visibleClasses.size(); i++) {
            ClassDoc inhclass = (ClassDoc)(visibleClasses.get(i));
            if (inhclass == classdoc) {
                continue;
            }
            List inhmembers = new ArrayList(members(inhclass));
            if (inhmembers.size() > 0) {
                if(! (printedSummaryHeader )){
                    printSummaryHeader(classdoc);
                    printSummaryFooter(classdoc);
                    printedSummaryHeader = true;
                }
                Collections.sort(inhmembers);
                printInheritedSummaryHeader(inhclass);
                boolean printedPrev = false;
                ProgramElementDoc ped = (ProgramElementDoc)(inhmembers.get(0));
                if(isInherited(ped)){
                    printInheritedSummaryMember(inhclass, ped);
                    printedPrev = true;
                }
                
                for (int j = 1; j < inhmembers.size(); ++j) {
                   ped = (ProgramElementDoc)(inhmembers.get(j));
                   if(isInherited(ped)){
                         if(printedPrev){
                             print(", ");
                         }
                         printInheritedSummaryMember(inhclass, ped);
                         printedPrev = true;
                   }
                }
                printInheritedSummaryFooter(inhclass);
            }
        }
    }

}
