/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocMethodWriter.java,v 1.2 2004/06/14 05:09:50 leavens Exp $
 * Author: David R. Cok
 */
 
package org.multijava.mjdoc.mjdoc_141;

import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// !FIXME!  Needs more internal documentation
/**
 * This is just used as a common super class for 
 * MjdocEFWriter and MjdocGFWriter, both of which
 * produce html pages containing a list of methods,
 * but not as part of a class.  

 * @see com.sun.javadoc.ClassDoc 
 * @see java.util.Collection
 * @see java.util.List
 * @see java.util.ArrayList
 * @see java.util.HashMap
 *
 * @author David Cok
 */

public class MjdocMethodWriter extends SubWriterHolderWriter {

//!FIXME! Delete stuff not used or for which the super class version is ok.

    protected ClassDoc classdoc;

    protected MjMethodDoc prev;
    
    protected MjMethodDoc next;
    
    //protected ClassTree classtree;

    protected boolean nopackage;

    protected MjdocMethodSubWriter methodSubWriter;

    //protected ConstructorSubWriter constrSubWriter;

   // protected FieldSubWriter fieldSubWriter;

    //protected ClassSubWriter nestedSubWriter;

    public MjdocMethodWriter(ConfigurationStandard config,
		String path, String filename, MjDoc methoddoc,
                MjMethodDoc prev, MjMethodDoc next, ArrayList methods,
                boolean nopackage) throws IOException {

        super(config, path, filename, 
         DirectoryManager.getRelativePath(((MjMethodDoc)methoddoc).cuPackage().name()));

        // !FIXME!JDK 1.4 compatability, no such field, currentcd
        //HtmlStandardWriter.currentcd = classdoc;
        //this.classtree = classtree;
        this.prev = prev;
        this.next = next;
        this.nopackage = nopackage;
        // !FIXME! classdoc is always null here?
        if (classdoc == null)  methodSubWriter = new MjdocMethodSubWriter(this, config);
	else                   methodSubWriter = new MjdocMethodSubWriter(this, classdoc, config);
        //constrSubWriter = new ConstructorSubWriter(this, classdoc);
        //fieldSubWriter = new FieldSubWriter(this, classdoc);
        //nestedSubWriter = new ClassSubWriter(this, classdoc);
    }



    /**
     * Print this package link
     */
    protected void navLinkPackage() {
        navCellStart();
        printHyperLink("package-summary.html", "", getText("doclet.Package"),
                       true, "NavBarFont1");
        navCellEnd();
    }
                       
    /**
     * Print class page indicator
     */
    protected void navLinkClass() {
        navCellStart();
        fontStyle("NavBarFont1");
        boldText("doclet.Class");
        fontEnd();
        navCellEnd();
    }

    /**
     * Print class page indicator
     */
    protected void navLinkGF() {
        navCellRevStart();
        fontStyle("NavBarFont1Rev");
        bold("FCNS");
        fontEnd();
        navCellEnd();
    }

    /**
     * Print previous package link
     */
    protected void navLinkPrevious() {
        if (prev == null) {
            print("PREV FUNCTION"); 
        } else {
            print(getGFLink(prev, "", "PREV FUNCTION", true, ""));
        }
    }
                                
    /**
     * Print next package link
     */
    protected void navLinkNext() {
        if (next == null) {
            print("NEXT FUNCTION"); 
        } else {
            print(getGFLink(next, "", "NEXT FUNCTION", true, ""));
        }
    }                               

    /**
     * Generate the class file contents.
     */
    public void generateGenericFunctionFile(MjMethodDoc top, ArrayList methods) {
        PackageDoc pkg = classdoc.containingPackage();
        String pkgname = (pkg != null)? pkg.name(): "";
        String label = "generic function " + top.name();
        String label2 = "<FONT SIZE=\"-1\">generic function</FONT> " + top.name();

        printHeader(getText("doclet.Window_ClassFile_label", 
                            configuration.windowtitle, label));
        navLinks(true);
        hr();
        println("<!-- ======== START OF GENERIC FUNCTION DATA ======== -->");
        h2();
        if (pkgname.length() > 0) {
            font("-1"); print(pkgname); fontEnd(); br();
        }
        print(label2);
        h2End();

        hr();

//        printDeprecated();
/*
        printClassDescription();
        p();
        // generate documentation for the class.
        if (classdoc.inlineTags().length > 0) {
            printInlineComment(classdoc);
            p();
        }
        // Print Information about all the tags here
        generateTagInfo(classdoc);
        hr();
        p();
*/
        println("<!-- ========== METHOD SUMMARY =========== -->"); println();
	methodSubWriter.printMethodsSummary(methods);

        p();

        println("<!-- ============ METHOD DETAIL ========== -->"); println();
	methodSubWriter.printMethods(methods);

        println("<!-- ========= END OF GENERIC FUNCTION DATA ========= -->");
        hr();
        navLinks(false);
        printBottom();
        printBodyHtmlEnd();
    }

  
    /**
     * Mark the class as deprecated if it is.
     */ //!FIXME! Do we need this for deprecated gfs
    protected void printDeprecated() {
        Tag[] deprs = classdoc.tags("deprecated");
        if (deprs.length > 0) {
            Tag[] commentTags = deprs[0].inlineTags();
            if (commentTags.length > 0) {
                boldText("doclet.Deprecated");
                space();
                // !FIXME!JDK 1.4 compatability, no such method printInline...
                // FIXME -- printInlineDeprecatedComment(deprs[0]);
            }
            p();
        }
    }

    //!FIXME! Do we need this?
    protected void printSummaryDetailLinks() {
        tr();
        tdVAlignClass("top", "NavBarCell3");
        font("-2");
        print("  ");
        navSummaryLinks();
        fontEnd();
        tdEnd();
        
        tdVAlignClass("top", "NavBarCell3");
        font("-2");
        navDetailLinks();
        fontEnd();
        tdEnd();
        trEnd();
    }   

    protected void navSummaryLinks() {
        printText("doclet.Summary");
        print("&nbsp;");
        //nestedSubWriter.navSummaryLink();
        //navGap();
        //fieldSubWriter.navSummaryLink();
        //navGap();
        //constrSubWriter.navSummaryLink();
        //navGap();
        //methodSubWriter.navSummaryLink();
    }

    protected void navDetailLinks() {
        printText("doclet.Detail");
        print("&nbsp;");
        //fieldSubWriter.navDetailLink();
        //navGap();
        //constrSubWriter.navDetailLink();
        //navGap();
        //methodSubWriter.navDetailLink();
    }


    /** This is used to generate a link to the file that contains the description
	of a generic function.  It is used by the PREF and NEXT buttons in the
	header of a generic function page.
    */ // !FIXME! - cleanup
    public String getGFLink(MjMethodDoc cd, String where,
                               String label, boolean bold, String color) {
        boolean nameUnspecified = label.length() == 0;
        if (nameUnspecified) {
            label = cd.name();
        }       
        displayLength += label.length();
        if (cd.isIncluded()) {
            if (isGeneratedDoc(cd)) {
                String filename = methodFilename(cd);
		filename = pathString(cd.cuPackage(), filename);
                return getHyperLink(filename, where, label, bold, color);
            }
        } else {
        //    String crosslink = getCrossClassLink(cd);
        //    if (crosslink != null) {
        //        return getHyperLink(crosslink, where, label, bold, color);
        //    }
        }
        //if (nameUnspecified) {
        //    displayLength -= label.length();
        //    label = cd.qualifiedName();
        //    displayLength += label.length();
        //}            
        return label;
    }

    /** This generates the filename (not the directory part) for the html
	file that holds the decription of the given top method.
    */
    static String methodFilename(MjMethodDoc m) {
	return m.gfFileName();
    }
    
    public void printSummaryLinkComment(AbstractSubWriter mw,
                                        ProgramElementDoc member) {
        codeEnd();
        println(); br();
	//System.out.println("printSummaryLinkComment " + member.name());
	if (member instanceof MjMethodDoc && !((MjMethodDoc)member).isExternal()) {
	    print("<B>Internal</B>"); br();
	}
        printCommentDef(member);
        summaryRowEnd();
        trEnd();   
    }   

    /**
     * Print the navigation bar for the Html page at the top and and the bottom.
     *
     * @param header If true print navigation bar at the top of the page else
     * print the nevigation bar at the bottom.
     */
    protected void navLinks(boolean header) {
        println("");
        println("<!-- ========== START OF NAVBAR ========== -->");
        if (!Standard.configuration().nonavbar) {
            if (header) {
                anchor("navbar_top");
            } else {
                anchor("navbar_bottom");
            }
            table(0, "100%", 1, 0);
            tr();
            tdColspanBgcolorStyle(2, "#EEEEFF", "NavBarCell1");
            println("");
            if (header) {
                anchor("navbar_top_firstrow");
            } else {
                anchor("navbar_bottom_firstrow");
            }
            table(0, 0, 3);
            print("  ");
            trAlignVAlign("center", "top");

	    if (configuration.createoverview) {
                navLinkContents();
            }

            if (configuration.packages.length > 0) {
                navLinkPackage();
            }

            navLinkClass();
	    
	    navLinkGF();

            if(configuration.classuse) {
                navLinkClassUse();
            }
            if(configuration.createtree) {
                navLinkTree();
            }
            if(!(configuration.nodeprecated || 
                    configuration.nodeprecatedlist)) {
                navLinkDeprecated();
            }
            if(configuration.createindex) {
                navLinkIndex();
            }
            if (!configuration.nohelp) {
                navLinkHelp();
            }
            print("  ");
            trEnd();
            tableEnd();
            tdEnd();

            tdAlignVAlignRowspan("right", "top", 3);

            printUserHeaderFooter(header);
            tdEnd();
            trEnd();
            println("");

            tr();
            tdBgcolorStyle("white", "NavBarCell2");
            font("-2");
            space();
            navLinkPrevious();
            space();
            println("");
            space();
            navLinkNext();
            fontEnd();
            tdEnd();

            tdBgcolorStyle("white", "NavBarCell2");
            font("-2");
            print("  ");
            navShowLists();
            print("  ");
            space();
            println("");
            space();
            navHideLists();
            fontEnd();
            tdEnd();

            trEnd();
            
            printSummaryDetailLinks();

            tableEnd();
            println("<!-- =========== END OF NAVBAR =========== -->");
            println("");
        }
    }  
}


