/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocPackageWriter.java,v 1.1 2004/01/14 11:05:44 davidcok Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_141;
import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;

/** A derivative of a class in the javadoc doclet API, so as to be able to
    instantiate a MjdocClassWriter instead of a ClassWriter and a 
    MjdocMethodSubWriter instead of a MethodSubWriter.
    @author David R. Cok
*/
public class MjdocPackageWriter extends PackageWriter {

    protected MjPackageDoc packagedoc; 
    /**
     * Generate a package summary page for the right-hand frame. Construct
     * the PackageFrameWriter object and then uses it generate the file.
     *
     * @param pkg The package for which "pacakge-summary.html" is to be
     * generated.
     * @param prev Previous package in the sorted array.
     * @param next Next package in the sorted array.
     */
    public static void generate(ConfigurationStandard configuration, PackageDoc pkg, PackageDoc prev,
                                PackageDoc next) throws DocletAbortException {
        MjdocPackageWriter packgen;
        String path = DirectoryManager.getDirectoryPath(pkg);
        String filename = "package-summary.html";
        try {
            packgen = new MjdocPackageWriter(configuration, path, filename, pkg, prev, next);
            packgen.generatePackageFile();
            packgen.close();
            packgen.copyDocFiles(configuration, getSourcePath(configuration, pkg), path + fileseparator
                + DOC_FILES_DIR_NAME, true);
        } catch (IOException exc) {
            configuration.standardmessage.
                error("doclet.exception_encountered", exc.toString(), filename);
            throw new DocletAbortException();
        }
    }

    
    public MjdocPackageWriter(ConfigurationStandard config, String path, String filename, PackageDoc packagedoc,
                PackageDoc prev, PackageDoc next) throws IOException, DocletAbortException {
	super(config,path,filename,packagedoc,prev,next);
	this.packagedoc = (MjPackageDoc)packagedoc;
	//methodSubWriter = new MjdocMethodSubWriter(this);
    }

    protected void generateClassListing() {
	super.generateClassListing();
	
	// Generate links to any documented compilation unit
	if (MjdocWrapper.docupages) {
	// FIXME - Should we sort the compilation units???
	    java.util.Iterator i = ((MjPackageDoc)packagedoc).extMethodsUnits().iterator();
	    if (i.hasNext()) {
		tableIndexSummary();
		printFirstRow("External Method Compilation Units");
		while (i.hasNext()) {
		    MjExtMethodsDoc m = (MjExtMethodsDoc)(i.next());
		    trBgcolorStyle("white", "TableRowColor");
		    summaryRow(15);
		    bold();
		    printHyperLink(m.name()+".html","",m.name());
		    boldEnd();
		    summaryRowEnd();
		    summaryRow(0);
		    printSummaryComment(m);
		    summaryRowEnd();
		    trEnd();
		}
		tableEnd();
		println("&nbsp;");
		p();
	    }
	}

	// Generate links to any documented generic functions
	if (!MjdocWrapper.donofcns) {
	    java.util.Iterator i = 
		MjdocWrapper.root().topConcreteMethods(packagedoc).iterator();
	    if (i.hasNext()) {
		tableIndexSummary();
		printFirstRow("External Generic Functions");
		while (i.hasNext()) {
		    MjMethodDoc m = (MjMethodDoc)(i.next());
		    trBgcolorStyle("white", "TableRowColor");
		    summaryRow(15);
		    bold();
		    //System.out.println("GF " + packagedoc + " " + m.qualifiedName()+m.signature());
		    print(getGFLink(m,"",m.name()+m.signature(),true,""));
		    boldEnd();
		    summaryRowEnd();
		    summaryRow(0);
		    printSummaryComment(m);
		    summaryRowEnd();
		    trEnd();
		}
		tableEnd();
		println("&nbsp;");
		p();
	    }
	}

    }  

    /**
     * Print the html file header. Also print Html page title and stylesheet
     * default properties.
     * 
     * @param title String title for the generated html file.
     */
    public void printHeader(String title) {
        println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Frameset//EN\"" +
                "\"http://www.w3.org/TR/REC-html40/frameset.dtd\">");
        println("<!--NewPage-->");
        html();
        head();
	print("<!-- Generated by " + MjdocWrapper.appName() + " on ");
	print(today());              
	println(" -->");
        if (Standard.configuration().charset.length() > 0) {
            println("<META http-equiv=\"Content-Type\" content=\"text/html; "
                    + "charset=" + Standard.configuration().charset + "\">");
        }
        title();
        println(title);
        titleEnd(); 
        printStyleSheetProperties();
        headEnd();
        body("white");
    }  
/*
    public void copyDocFiles(ConfigurationStandard configuration,
                                    String srcDirName, String path, boolean overwrite) throws DocletAbortException {
	super.copyDocFiles(configuration, srcDirName, path, overwrite); // true for overwrite ??? FIXME
    }
*/
    public String getGFLink(MjMethodDoc cd, String where,
                               String label, boolean bold, String color) {
        boolean nameUnspecified = label.length() == 0;
        if (nameUnspecified) {
            label = cd.name();
        }       
        displayLength += label.length();
        if (cd.isIncluded()) {
            if (isGeneratedDoc(cd)) {
                String filename = cd.gfFileName();
		filename = pathString(cd.cuPackage(), filename);
                return getHyperLink(filename, where, label, bold, color);
            }
        } else {
        //    String crosslink = getCrossClassLink(cd);
        //    if (crosslink != null) {
        //        return getHyperLink(crosslink, where, label, bold, color);
        //    }
        }
        //if (nameUnspecified) {
        //    displayLength -= label.length();
        //    label = cd.qualifiedName();
        //    displayLength += label.length();
        //}            
        return label;
    }

}








