/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocWrapper.java,v 1.9 2005/05/03 04:24:55 kui_dai Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_141;

import com.sun.javadoc.Doclet;
import com.sun.javadoc.MethodDoc;
import com.sun.tools.doclets.standard.Extern;
import com.sun.tools.doclets.standard.Standard;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Iterator;
import org.multijava.mjdoc.JavadocOptions;
import org.multijava.mjdoc.JavadocOptionsInterface;
import org.multijava.mjc.*;
import org.multijava.util.compiler.Compiler;
import org.multijava.util.compiler.TokenReference;

//FIXME - for reentrancy and multithreadedness, this class should not have everything static

/** This class is a bridge between the Mjc parsing mechanisms in Main and the 
    html generation mechanisms in MjRootDoc and related classes.
    @author David R. Cok
 */
public class MjdocWrapper {

    public MjdocWrapper(Compiler c) { 
        compiler = c;
    }
    
    protected Compiler compiler;
    
    /** Holds the RootDoc value that is the root of the tree representing all the classes
        being documented.
    */
    //@ spec_public
    static private MjRootDoc root = null; //!FIXME! - need to ensure that setOptions is called - should be private?.

    static public MjRootDoc root() { return root; }

    
    // NOTE:  As implemented, we are going to generate a page for each generic function that has an external
    // top method, but not for the external methods by compilation unit.

    /** Allows subclasses to change the wrapper behavior. */
    static public MjdocWrapper wrapper() { return wrapper; }
    static public void setWrapper(MjdocWrapper w) { wrapper = w; }
    
    static protected MjdocWrapper wrapper = null;
    
    /** If true, then create generic function html pages for internal as well as external functions.
        If doallfcns = false and donofcns = false, then only external function pages are generated.
    */
    static boolean doallfcns = false;
    
    /** If true, then generate no html pages for any generic functions. */
    static boolean donofcns = false;

    /** If true, then generate pages for the external method part of each compilation unit. */
    static boolean docupages = false;
    

    /** The name of a doclet, as specified in the options. */
    static private String docletname = null;
    
    /** The path in which to search for the  doclet, if specified in the options. */
    static private String docletpath = null;

    /** The classpath specified by the -classpath option */
    static private String classpath = null;
        
    /** The sourcepath specified by the -sourcepath option */
    static private String sourcepath = null;
        
    /** The destination specified by the -d option. */
    static private File destination = new File(".");
    
    /** The privacy level controlling the items listed in the html: 
        0 - public only, 1 - public and protected,
        2 - all but private, 3 - everything.
    */
    static private long privacy = 1;

    static public long privacy() { return privacy; }
    
    static public PrivacyChecker privacyChecker = new MjPrivacyChecker(1);

    static public interface PrivacyChecker {
        public boolean isVisible(long mods);
    }

    static public class MjPrivacyChecker implements PrivacyChecker {
        protected long privacy = 1;
        public MjPrivacyChecker(long privacy) {
                this.privacy = privacy;
        }

        public boolean isVisible(long mods) {
            if (privacy == 3) return true;
            if ((mods & Constants.ACC_PUBLIC) != 0) return true;
            if ((mods & Constants.ACC_PRIVATE) != 0) return false;
            if (privacy == 2) return true;
            if (privacy == 0) return false;
            if ((mods & Constants.ACC_PROTECTED) != 0) return true;
            return false;
        }
    }

    /** Returns true if the given modifiers combined with the privacy level set in the options are such
        as to have the program element listed in the html.
    */
    static public boolean checkPrivacy(long mods) {
        return privacyChecker.isVisible(mods);
    }
    
    /** Sets the doclet to run */
    static public void setDoclet(String n) { docletname = n; }

    /** Turns off all informational messages. */
    static public void quiet(boolean b) { 
        if (root!=null) root.quiet(b); 
    }
    
    static private String appName = null;

    static public String appName() { return appName; }

    static public void setAppName(String n) { appName = n; }

    static public class CClassMap {
        public MjClassDoc lookup(CClass cclass) {
            if (cclass == null) return null;
            MjClassDoc c = (MjClassDoc)(map.get(cclass));
            if (c == null) {
                c = newClassDoc(cclass);
                MjdocWrapper.map.put(cclass,c);
                root.add(c);
                //System.out.println("NEW MJDOC " + c + " " + c.jtype());
            }
            return c;
        }
        
        public MjClassDoc newClassDoc(CClass cclass) {
            return new MjClassDoc(cclass);
        }
        
    }

        /** Map of CClass to MjClassDoc for all classes encountered in parsing. 
            Note: from a JTypeDeclarationType, one can get a CClass using getCClass().
        */
        // FIXME - this map should be reset when initSession is called,
        // but only after the caching of ASTs between compilation
        // sessions is implemented (Bug#670384)
    static protected java.util.Map map = new java.util.HashMap(); 
    
    static protected CClassMap mapper = new CClassMap();
    
    /** Returns the MjClassDoc object generated from the given CClass; if the CClass object
        was not previously registered, an MjClassDoc object is created. 
     */
    //@ ensures cclass == null <==> \result == null;
    static public MjClassDoc lookup(CClass cclass) {
        return mapper.lookup(cclass);
    }
    
    /** This function creates and specifies the list of packages in the given ArrayList;
        the ArrayList consists of String objects, each of which is a '/'-separated and
        '/'-terminated package name.
    */
    static public void selectPackages(ArrayList v) {
        java.util.Iterator i = v.iterator();
        while (i.hasNext()) {
            String n = (String)(i.next());
            root.specify(MjPackageDoc.createPackage(n));
        }
    }
    
    /** Loads all the classes and interfaces contained in 'trees' as command-line specified classes, for which
        javadoc documents will be generated. */
    public void selected(CompilerPassEnterable tree) throws Exception {
        if (!(tree instanceof JCompilationUnitType)) { //!FIXME! Use a better Exception
            if (tree instanceof JTypeDeclarationType) {
                //selected((JTypeDeclarationType)tree);
                return;

                // FIXME - one gets here when a calling mjdoc/jmldoc on a
                // class with an annonymous class within it, e.g. 
                // JML2/org/jmlspecs/models/resolve/TestSuite.java
            }
            throw new Exception("Internal error: Method MjdocWrapper.selected was called with "
                    + "an argument that is a " + tree.getClass().toString()
                    + " rather than an instance of JCompilationUnitType.");
        }
        JCompilationUnitType unit = (JCompilationUnitType)tree;
        selectTypeDeclarations(unit);

        if (unit.tlMethods().size() != 0) {
            MjPackageDoc pkg = MjPackageDoc.createPackage(unit.packageNameAsString()); 
            if (!root.isSpecified(pkg)) root.specify(pkg.lookup(unit));
        }
    }
    
    public void selectTypeDeclarations(JCompilationUnitType unit) {
        selected(unit.typeDeclarations(),unit);
    }
    
    public void selected(JTypeDeclarationType[] jtypes, JCompilationUnitType unit) {
        for (int j=0; j<jtypes.length; ++j) {
            // Catch exceptions so that we can go on with other classes in the
            // file. // !FIXME! Should we really do this ??? - set some error found value?
            try {
                selected(jtypes[j]);
            } catch (Exception e) {
                System.err.println(unit.file() + "::" + jtypes[j].ident() + " - " + e);
                e.printStackTrace(System.err);
            } catch (Error e) {
                System.err.println(unit.file() + "::" + jtypes[j].ident() + " - " + e);
                e.printStackTrace(System.err);
            }
        }
    }
    
    /** Loads all the given class as command-line specified classes, for which
        documents will be generated. */
    public void selected(JTypeDeclarationType jtype) throws Exception {
        if (!checkPrivacy(jtype.modifiers())) return;
        select(jtype);
        // Be sure to select all nested classes (recursively) as well.
        Iterator i = jtype.inners().iterator(); // FIXME -- inners or nested? are they really only inners
        while (i.hasNext()) {
            JTypeDeclarationType jt = (JTypeDeclarationType)(i.next());
            if (checkPrivacy(jt.modifiers())) selected(jt);
        }
    }
    
    public void select(JTypeDeclarationType jtype) {
        MjClassDoc mjd = lookup(jtype.getCClass());
        if (!root.isSpecified(mjd.containingPackage())) root.specify(mjd);
    }
    
    /** Loads all the classes and interfaces contained in 'tree' into the RootDoc. */
    public void register(CompilerPassEnterable tree) throws Exception {
        if (tree instanceof JCompilationUnitType) {
            register((JCompilationUnitType)tree);
        } else if (tree instanceof JTypeDeclarationType) {
            register((JTypeDeclarationType)tree);
        } else {
            throw new Exception("Internal error: Method MjdocWrapper.register was called with "
                    + "an argument that contained a " + tree.getClass().toString()
                    + " rather than an instance of JCompilationUnitType or JTypeDeclarationType.");
        }
    }
    
    /** Loads all the classes and interfaces contained in 'tree' into the RootDoc. */
    public void register(JCompilationUnitType unit) throws Exception {
            // Note: packageNameAsString() returns a name with / as the separator
        MjPackageDoc pkg = MjPackageDoc.createPackage(unit.packageNameAsString()); 
        root.add(pkg);

        registerTypeDeclarations(unit);

        // Multi-java may have external methods; do those as well

        registerGFs(unit.tlMethods(),pkg.lookup(unit));
    }
    
    public void registerTypeDeclarations(JCompilationUnitType unit) {
        registerTypeDeclarations(unit.typeDeclarations(),unit);
    }
    
    public void registerTypeDeclarations(JTypeDeclarationType[] jtypes, JCompilationUnitType unit) {
        // Now loop over all the classes and interfaces declared in
        // the compilation unit.  Note that there can
        // be only one public class (named in a way corresponding
        // to the name of the file), but there can be
        // other non-public classes.
        
        for (int j=0; j<jtypes.length; ++j) {
            // Catch exceptions so that we can go on with other classes in the
            // file. !FIXME! Should we ??? Or at least remeber that we have an error
            try {
                if (checkPrivacy(jtypes[j].modifiers())) register(jtypes[j]);
            } catch (Throwable e) {
                System.err.println(unit.file() + "::" + jtypes[j].ident() + " - " + e);
                e.printStackTrace(System.err);
            }
        }
    }

    public void registerGFs(ArrayList tlmethods, MjExtMethodsDoc mje) {
        if (tlmethods.size() != 0) {
            java.util.Iterator i = tlmethods.iterator();
            while (i.hasNext()) {
                MJTopLevelMethodDeclaration jm = (MJTopLevelMethodDeclaration)(i.next());
                CType ct = jm.getOpenClassType();
                MjClassDoc cl = lookup(ct.getCClass());
                if (checkPrivacy(jm.modifiers())) {
                    MjMethodDoc mjm = cl.addTLMethod(jm);
                    mje.add(mjm);
                    mjm.setExternalUnit(mje);
                    root.add(mjm);
                }
            }
            root.add(mje);
        }
    }

    /** Registers a specific class or interface (and any classes or interfaces nested 
        or referenced within it) into the RootDoc.
        @param jtype A specific class or interface to register as referenced
    */
    //@ requires jtype != null;
    void register(JTypeDeclarationType jtype) throws Exception {

//System.out.println("REGA" + jtype.getTokenReference());
        MjClassDoc c = lookup(jtype.getCClass());
        c.setJtype(jtype);
//System.out.println("REG " + c.cclass() + " " + c.jtype().getTokenReference() + " " + jtype.getTokenReference());
        
        // Register any referenced classes
        // Types of fields
        
        JFieldDeclarationType[] jf = jtype.fields();
        for (int i=0; i<jf.length; ++i) {
            register(jf[i].getType());
        }
        
        // Argument, return, exception types of methods and constructors

        Iterator itr = jtype.methods().iterator();
        while (itr.hasNext()) {
            JMethodDeclarationType mem = (JMethodDeclarationType)(itr.next());
            if (!(mem instanceof JConstructorDeclarationType)) register(mem.returnType());
            
            CClassType[] cc = mem.getExceptions();
            for (int j=0; j<cc.length; ++j) {
                register(cc[j]);
            }
            
            JFormalParameter[] pp = mem.parameters();
            for (int j=0; j<pp.length; ++j) {
                register(pp[j].staticType());
                register(pp[j].specializedType().dynamicType());
            }
        }
        
        MethodDoc[] mm = c.methods();
        for (int k=0; k<mm.length; ++k) {
            root.add((MjMethodDoc)(mm[k]));
        }
        
        c.superclass(); // registers the super class as a side effect
                // !FIXME! Do we need to do this recursively, for everything in the super class?
                
        c.interfaces(); // registers the implemented interfaces as a side effect
                // !FIXME! Do we need to do this recursively, for everything in the super class?
                
        // Now recursively call this for any nested classes
        
        itr = jtype.inners().iterator(); // FIXME -- inners or nested
        while (itr.hasNext()) {
            JTypeDeclarationType intype = (JTypeDeclarationType)(itr.next());
            // Catch the exception so that we can go on to try the other
            // nested classes //!FIXME! Should we?  How to remember the error?
            try {
                if (checkPrivacy(intype.modifiers())) register(intype);
            } catch (Throwable e) {
                System.err.println( e);
                e.printStackTrace(System.err);
            }
        }
    }

    /** Makes sure that the given type is registered as a referenced class, if appropriate. */
    static public void register(CType ct) {
        if (ct.isArrayType()) ct = ((CArrayType)ct).getBaseType();
        if (ct.isClassType()) lookup(ct.getCClass());
    }
    
    /** This initiates the generation of html, given the
        tree of information loaded into the javadoc classes.
        @return true if the html generation was successful
    */
    static public boolean complete() {
        try {
/*
            if (!destination.isDirectory()) {
                if (destination.exists()) {
                    root.printError("Specified destination exists but is not a directory: " + destination);
                    throw new org.multijava.util.compiler.CompilationAbortedError();
                }
                if (!destination.mkdirs()) {
                    root.printError("Failed to create destination directory: " + destination);
                    throw new org.multijava.util.compiler.CompilationAbortedError();
                }
                root.printNotice("Creating destination directory " + destination);
            } else if (!destination.canWrite()) {
                    root.printError("Specified destination directory exists but is not writable: " + destination);
                    throw new org.multijava.util.compiler.CompilationAbortedError();
            }
*/

            boolean b;
            if (docletname == null) {
                b = MjdocStandard.start(root);
            } else {
                //!FIXME! -docletpath is not implemented (it does not work to reset java.class.path here)
                try {
                    Class c = Class.forName(docletname);
                    java.lang.reflect.Method m = c.getMethod("start",new Class[]{ Class.forName("com.sun.javadoc.RootDoc") } );
                    Object o = m.invoke(null,new Object[] {root});
                    b = ((Boolean)o).booleanValue();
                } catch (java.lang.reflect.InvocationTargetException e) {
                    java.lang.Throwable ee = e.getTargetException();
                    System.err.println(ee.toString());
                    ee.printStackTrace(System.err);
                    b = false;
                } catch (Throwable e) {
                    System.err.println("Failed to run or execute the doclet " + docletname + ": " + e);
                    b = false;
                }
            }
            return b;
        } catch (org.multijava.util.compiler.CompilationAbortedError e) {
            throw e;
        } catch (Exception e) {         
            System.out.println(e);
            e.printStackTrace(System.out);
            // false is returned to signify that an error has happened
        }
        return false;
    }
        
    //--------------------------------------------------------------------------------------------
 
    /** This is a helper method that creates a String[] of length n containing 
        the elements args[i] .. args[i+n-1].
    */
    //@ requires 0<=i && i< args.length;
    //@ requires 0<=n && i+n <= args.length;
    //@ ensures \result != null;
    //@ ensures \result.length == n;
    //@ ensures (\forall int j; 0<=j && j<\result.length; \result[j] == args[(int)(i+j)]);
    static private String[] formArray(String[] args, int i, int n) {
        String[] sp = new String[n];
        for (int k=0; k<n; k++) sp[k] = args[i+k];
        return sp;
    }

    /** This function returns the number of arguments that follow a given option (+1 for the option itself)
        in the command-line.
    */
    //@ requires s != null;
    //@ ensures \result > 0;
    static private int optionLength(String s) {
        int nn = Standard.optionLength(s); 
        // length includes the option itself, so is at least 1
        // if nn is 0 then s is not a recognized option (by the Standard doclet)

        return nn;
    }

    /** This routine parses the command-line options provided in 'args', recognizing any javadoc-related options
        and their arguments.  A new String array is returned that contains only the arguments in 'args' that were
        not recognized.  The recognized options are loaded into the RootDoc to be used in the html generation.
    */
    //@ requires args !=null;
    //@ ensures \result != null;
    //@ ensures root != null;
    public String[] setOptions(String[] args, MjcCommonOptions options) {

        root = new MjRootDoc(compiler,new String[][]{}); // Just so error messages will print
        
        ArrayList opt = new ArrayList(); // ArrayList of String[]
        ArrayList remainingArgs = new ArrayList(); // ArrayList of String
        
        // Take the javadoc arguments from the command line and put them in
        // the Options AST, leaving all other arguments in remainingArgs
        parseArguments(args, options, remainingArgs);
        
        // Put all of the javadoc options and put them in the opt array so
        // they can be passed to javadoc
        parseOptions(options, opt);
        
        privacyChecker = new MjPrivacyChecker(privacy);
        String[][] opts = (String[][])opt.toArray(new String[0][]);

        // Even though the destination directory may well be created later, we have to do
        // it here, because in javadoc 1.3, validOptions, called below will terminate with
        // an error if the destination directory does not exist.
        // We save any real errors to be flagged later.
        if (!destination.isDirectory()) {
            if (!destination.mkdirs()) {
                root.printError("Failed to create destination directory: " + destination);
                throw new org.multijava.util.compiler.CompilationAbortedError();
            }
            //root.printNotice("Creating destination directory " + destination);
        }

        // We have to call this, both to catch any errors in the options, but also because the 
        // analysis and saving of information about groups (as in -group option) is performed
        // in validOptions (even though you might think it should be in the routines that
        // process the options...).
        try {
            if (!MjdocStandard.validOptions(opts,root)) { 
                throw new org.multijava.util.compiler.CompilationAbortedError();
            }
        } catch (java.io.IOException e) {
                throw new org.multijava.util.compiler.CompilationAbortedError();
        }
        root = new MjRootDoc(compiler,opts);
        
        //FIXME Need to know about -overview
        setDocComment(root,options,sourcepath);
        return (String[])remainingArgs.toArray(new String[0]);
    }
        
    /**
     *  Parse the Javadoc options into the options structure
     */
    private void parseArguments(String[] args, MjcCommonOptions options, 
                                ArrayList remainingArgs) {
        if (args == null) return;

        // MjdocStandard.reset(root);
        
        int len = args.length;

        for (int i=0; i<len; ++i) {
        
            // Some options can be specified either in mj style (--) or in javadoc style (-).
            // Translate so users can specify either one.
            boolean haveQuiet = false; // FIXME - this is for jmldoc
            
            if (args[i].equals("--destination")) args[i] = "-d";
            else if (args[i].equals("--classpath")||args[i].equals("-C")) args[i] = "-classpath";
            else if (args[i].equals("--sourcepath")||args[i].equals("-S")) args[i] = "-sourcepath";
                // FIXME - don't need this for mjdoc
            else if (args[i].equals("--Quiet")||args[i].equals("-Q")
                     ||args[i].equals("-Quiet")) {
                args[i] = "-quiet";
                haveQuiet = true;
            }
                        
            // Handle help away from the main loop since optionLength(s)
            // (which calls Standard.optionLength(s) in Javadoc) prints the
            // Standard Doclet help options automatically while returning the
            // length of the option.  Since we are handling help through the
            // Options AST instead, optionLength(s) needs to be skipped.
            if (args[i].equals("--help") || args[i].equals("-help")) {
                remainingArgs.add("--help");
                break;
            }

            // FIXME - also need to handle the case of -C and -S and -d with concatenated argment.
            // Now handle all javadoc style options, creating the opt array and removing from args
            // any options+arguments that are known only to the javadoc part of mjdoc.
            // Note: optionLength only recognizes the 'standard doclet' options, not those that are
            // part of the core javadoc; apparently some options (the core javadoc ones) are 
            // interpreted before the opt array is formed in the standard javadoc.
            
            String sOriginalCase = args[i];
            String s = sOriginalCase.toLowerCase();
            int nn;
                
            if ((nn=optionLength(s)) != 0) {
                if (nn < 0) nn = 1;
                // Handle the 'standard doclet' options
                
                if (sOriginalCase.equals("-D")) {
                    root.printError("Option -D is ambiguous - use either -d or --deprecation");
                    throw new org.multijava.util.compiler.CompilationAbortedError("");
                }
                if (i+nn > len) { 
                    root.printError("Incorrect number of command-line arguments: " + s + " requires " + (nn-1) + " arguments"); 
                    throw new org.multijava.util.compiler.CompilationAbortedError("");
                }
                // Some options need to be retained for mjc as well
                // or for other processing:
                if (s.equals("-d")) {
                    String dir = args[i+1];
                    destination = new File(dir);
                    options.setOption("destination", dir);
                } else if (s.equals("-quiet") && haveQuiet) {
                    handleQuiet(options);
                } else if (s.equals("-use") || s.equals("-author") 
                           || s.equals("-linksource")
                           || s.equals("-splitindex") 
                           || s.equals("-nodeprecated") 
                           || s.equals("-nodeprecatedlist")
                           || s.equals("-nosince") || s.equals("-notree") 
                           || s.equals("-noindex")
                           || s.equals("-nohelp") || s.equals("-nonavbar") 
                           || s.equals("-serialwarn")
                           || s.equals("-docfilessubdirs") 
                           || s.equals("-nocomment")
                           || s.equals("-x") || s.equals("-xnodate")) {
                    options.setOption(sOriginalCase.substring(1), Boolean.valueOf(true));
                } else if (s.equals("-windowtitle") || s.equals("-doctitle") 
                           || s.equals("-header") || s.equals("-footer") 
                           || s.equals("-bottom") || s.equals("-helpfile") 
                           || s.equals("-stylesheetfile") 
                           || s.equals("-charset") || s.equals("-docencoding") 
                           || s.equals("-taglet") || s.equals("-tagletpath") 
                           || s.equals("-subpackages") || s.equals("-exclude") 
                           || s.equals("-excludedocfilessubdir")
                           || s.equals("-noqualifier")) {
                    options.setOption(sOriginalCase.substring(1), args[i+1]);
                } else if (s.equals("-group")) {
                    String[][] group 
                        = ((JavadocOptionsInterface)options).group();
                    int arrayLen = ((group == null) ? 0 : group.length);
                    String[][] newGroup = new String[arrayLen + 1][2];
                    if (group != null) {
                        System.arraycopy(group, 0, 
                                         newGroup, 0, arrayLen);
                    }
                    newGroup[newGroup.length - 1][0] = args[i+1];
                    newGroup[newGroup.length - 1][1] = args[i+2];
                    options.setOption("group", newGroup);
                } else if (s.equals("-link")) {
// FIXME - does the doclete api now handle these options itself????
                    //  // !FIXME!JDK 1.4 compatability need another argument to Extern.url
                    //  Extern.url(args[i+1],args[i+1],root,false);
                    String[][] link 
                        = ((JavadocOptionsInterface)options).link();
                    int arrayLen = ((link == null) ? 0 : link.length);
                    String[][] newLink = new String[arrayLen + 1][1];
                    if (link != null) {
                        System.arraycopy(link, 0, newLink, 0, arrayLen);
                    }
                    newLink[newLink.length - 1][0] = args[i+1];
                    options.setOption("link", newLink);
                } else if (s.equals("-linkoffline")) {
                    //  // !FIXME!JDK 1.4 compatability need another argument to Extern.url
                    //  Extern.url(args[i+1],args[i+2],root);
                    String[][] linkoffline 
                        = ((JavadocOptionsInterface)options).linkoffline();
                    int arrayLen 
                        = ((linkoffline == null) ? 0 : linkoffline.length);
                    String[][] newLinkoffline = new String[arrayLen + 1][2];
                    if (linkoffline != null) {
                        System.arraycopy(linkoffline, 0, 
                                         newLinkoffline, 0, arrayLen);
                    }
                    newLinkoffline[newLinkoffline.length - 1][0] = args[i+1];
                    newLinkoffline[newLinkoffline.length - 1][1] = args[i+2];
                    options.setOption("linkoffline", newLinkoffline);
                } else if (s.equals("-tag")) {
                    String[][] tag = ((JavadocOptionsInterface)options).tag();
                    int arrayLen = ((tag == null) ? 0 : tag.length);
                    String[][] newTag = new String[arrayLen + 1][1];
                    if (tag != null) {
                        System.arraycopy(tag, 0, newTag, 0, arrayLen);
                    }
                    newTag[newTag.length - 1][0] = args[i+1];
                    options.setOption("tag", newTag);
                } else if (s.equals("-version")) {
                    options.setOption("Version", Boolean.valueOf(true));
                }
                i += nn-1; // -1 because the loop conditions also increment i
            } else { // not a standard doclet option - check for core javadoc options
                if (s.equals("-classpath")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    classpath = args[++i];
                    options.setOption("classpath", classpath);
                } else if (s.equals("-sourcepath")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    sourcepath = args[++i];
                    options.setOption("sourcepath", sourcepath);
                } else if (s.equals("-quiet")) {
                    options.setOption("quiet", Boolean.valueOf(!options.quiet()));
                } else if (s.equals("-overview") || s.equals("-encoding")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    options.setOption(sOriginalCase.substring(1), args[++i]);
                } else if (s.equals("-privacy")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    int arg = Integer.parseInt(args[++i]);
                    if (arg != 0 && arg != 1 && arg != 2 && arg != 3) {
                        root.printError("Incorrect usage of privacy: requires an integer of 0, 1, 2, or 3" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    privacy = arg;
                    options.setOption("privacy", args[++i]);
                } else if (s.equals("-public")) {
                    privacy = 0;
                    options.setOption("privacy", "0");
                } else if (s.equals("-protected")) {
                    privacy = 1;
                    options.setOption("privacy", "1");
                } else if (s.equals("-package")) {
                    privacy = 2;
                    options.setOption("privacy", "2");
                } else if (s.equals("-private")) {
                    privacy = 3;
                    options.setOption("privacy", "3");
                } else if (s.startsWith("-j")) {
                    options.setOption("J", (args[i]).substring(1));
                } else if (s.equals("-doclet")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    docletname = args[++i];
                } else if (s.equals("-docletpath")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    docletpath = args[++i];
                    root.printWarning("mjdoc - Warning: docletpath is not implemented"); // FIXME -docletpath not implemented
                } else if (s.equals("-1.1")) {
                    root.printWarning("Ignoring unsupported command-line option: " + s);
                } else if (s.equals("-extdirs")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                    ++i;
                    root.printWarning("Ignoring unsupported command-line option: " + s); // FIXME -extdirs not implemented
                } else if (s.equals("-locale")) {
                    ++i;
                    root.printWarning("Ignoring unsupported command-line option: " + s); // FIXME -locale not implemented
                } else if (s.equals("-verbose")) {
                    root.printWarning("Ignoring inapplicable command-line option: " + s); 
                } else if (s.equals("-breakiterator")) {
                    // Silently ignored.  mjdoc always behaves like javadoc with -breakiterator
                } else if (s.equals("-bootclasspath")) {
                    if (i+2 > len) {
                        root.printError("Incorrect number of command-line arguments: " + s + " requires 1" );
                        throw new org.multijava.util.compiler.CompilationAbortedError();
                    }
                        options.setOption("bootclasspath", args[++i]);
                    org.multijava.util.classfile.ClassPath.setBootClassPath(args[i]);
                } else {
                    // Not matched as a javadoc option
                        remainingArgs.add(sOriginalCase);
                }
            }
        }
        }
        
    /**
     *  Loads the javadoc options from the Options AST into an array so 
     *  they can be passed to javadoc
     */
// FIXME - Need to uncomment uninmplemented options when they are implemented
    private void parseOptions(MjcCommonOptions options, ArrayList opt) {
        JavadocOptionsInterface opts = (JavadocOptionsInterface)options;    
        // Locale must be first in the options array
        //if (options.locale() != null) 
            //opt.add(new String[] {"-locale", options.locale()});
        
        // Standard Doclet Options
        if (options.destination() != null) 
            opt.add(new String[] {"-d", options.destination()});
        if (opts.use()) opt.add(new String[] {"-use"});
        if (opts.Version()) opt.add(new String[] {"-version"});
        if (opts.author()) opt.add(new String[] {"-author"});
        if (opts.splitindex()) opt.add(new String[] {"-splitindex"});
        if (opts.windowtitle() != null) 
            opt.add(new String[] {"-windowtitle", opts.windowtitle()});
        if (opts.doctitle() != null) 
            opt.add(new String[] {"-doctitle", opts.doctitle()});
        if (opts.header() != null) 
            opt.add(new String[] {"-header", opts.header()});
        if (opts.footer() != null) 
            opt.add(new String[] {"-footer", opts.footer()});
        if (opts.bottom() != null) 
            opt.add(new String[] {"-bottom", opts.bottom()});
        if (opts.link() != null) {
            for (int i = 0; i < (opts.link()).length; i++) {
                String firstArg = (opts.link())[i][0];
                if ((firstArg != null) && (!firstArg.equals(""))) {
                    opt.add(new String [] {"-link", firstArg});
                }
            }
        }
        if (opts.linkoffline() != null) {
            for (int i = 0; i < (opts.linkoffline()).length; i++) {
                String firstArg = (opts.linkoffline())[i][0];
                String secondArg = (opts.linkoffline())[i][1];
                if (((firstArg != null) && (!firstArg.equals("")))
                    && ((secondArg != null) && (!secondArg.equals("")))) {
                    opt.add(new String [] {"-linkoffline", firstArg, 
                                           secondArg});
                }
            }
        }
        if (opts.linksource()) opt.add(new String[] {"-linksource"});
        if (opts.group() != null) {
            for (int i = 0; i < (opts.group()).length; i++) {
                String firstArg = (opts.group())[i][0];
                String secondArg = (opts.group())[i][1];
                if (((firstArg != null) && (!firstArg.equals("")))
                    && ((secondArg != null) && (!secondArg.equals("")))) {
                    opt.add(new String [] {"-group", firstArg, secondArg});
                }
            }
        }
        if (opts.nodeprecated()) opt.add(new String[] {"-nodeprecated"});
        if (opts.nodeprecatedlist()) 
            opt.add(new String[] {"-nodeprecatedlist"});
        if (opts.nosince()) opt.add(new String[] {"-nosince"});
        if (opts.notree()) opt.add(new String[] {"-notree"});
        if (opts.noindex()) opt.add(new String[] {"-noindex"});
        if (opts.nohelp()) opt.add(new String[] {"-nohelp"});
        if (opts.nonavbar()) opt.add(new String[] {"-nonavbar"});
        if (opts.helpfile() != null) 
            opt.add(new String[] {"-helpfile", opts.helpfile()});
        if (opts.stylesheetfile() != null) 
            opt.add(new String[] {"-stylesheetfile", opts.stylesheetfile()});
        //if (opts.serialwarn()) opt.add(new String[] {"-serialwarn"});
        if (opts.charset() != null) 
            opt.add(new String[] {"-charset", opts.charset()});
        if (opts.docencoding() != null) 
            opt.add(new String[] {"-docencoding", opts.docencoding()});
        if (opts.tag() != null) {
            for (int i = 0; i < (opts.tag()).length; i++) {
                String firstArg = (opts.tag())[i][0];
                if ((firstArg != null) && (!firstArg.equals(""))) {
                    opt.add(new String [] {"-tag", firstArg});
                }
            }
        }
        if (opts.taglet() != null) 
            opt.add(new String[] {"-taglet", opts.taglet()});
        if (opts.tagletpath() != null) 
            opt.add(new String[] {"-tagletpath", opts.tagletpath()});
        if (opts.subpackages() != null) 
            opt.add(new String[] {"-subpackages", opts.subpackages()});
        if (opts.exclude() != null) 
            opt.add(new String[] {"-exclude", opts.exclude()});
        if (opts.docfilessubdirs()) opt.add(new String[] {"-docfilessubdirs"});
        if (opts.excludedocfilessubdir() != null) 
            opt.add(new String[] {"-excludedocfilessubdir", 
                                  opts.excludedocfilessubdir()});
        if (opts.noqualifier() != null) 
            opt.add(new String[] {"-noqualifier", opts.noqualifier()});
        if (opts.nocomment()) opt.add(new String[] {"-nocomment"});
        if (opts.x()) opt.add(new String[] {"-x"});
        if (opts.xnodate()) opt.add(new String[] {"-xnodate"});
        
        // Core Doclet Options
        if (opts.overview() != null) 
            opt.add(new String[] {"-overview", opts.overview()});
        if (opts.privacy() == 0) {
            opt.add(new String[] {"-public"});
        } else if (opts.privacy() == 1) {
            opt.add(new String[] {"-protected"});
        } else if (opts.privacy() == 2) {
            opt.add(new String[] {"-package"});
        } else {
            opt.add(new String[] {"-private"});
        }
        if (opts.doclet() != null) 
            opt.add(new String[] {"-doclet", opts.doclet()});
        //if (opts.docletpath() != null) 
            //opt.add(new String[] {"-docletpath", opts.docletpath()});
        if (options.source() != null) 
            opt.add(new String[] {"-source", options.source()});
        if (options.sourcepath() != null) 
            opt.add(new String[] {"-sourcepath", options.sourcepath()});
        if (options.classpath() != null) 
            opt.add(new String[] {"-classpath", options.classpath()});
        if (opts.bootclasspath() != null) 
            opt.add(new String[] {"-bootclasspath", opts.bootclasspath()});
        //if (opts.extdirs() != null) 
            //opt.add(new String[] {"-extdirs", extdirs()});
        if (options.quiet()) opt.add(new String[] {"-quiet"});
        if (opts.encoding() != null) 
            opt.add(new String[] {"-encoding", opts.encoding()});
        if (opts.J() != null) opt.add(new String[] {"-J-" + opts.J()});
    }
        
    /**
     *  Handle Jmldoc's Quiet option
     */
    protected void handleQuiet(MjcCommonOptions options) {
        // Do nothing, this is for Jmldoc
    }
        
    /** Reads a text file into a String. */
    static public String readFile(File f) throws Exception {
        StringBuffer fileBuffer = new StringBuffer();
        FileReader ff = null;
        try {
            ff = new FileReader(f);
            char[] cbuf = new char[10000];
            int n;
            while ((n=ff.read(cbuf,0,cbuf.length)) != -1) {
                    fileBuffer.append(cbuf,0,n);
            }
        } catch (Exception e) {
            //System.out.println(e);
            throw e;
        } finally {
            if (ff != null) ff.close();
        }
        return fileBuffer.toString();
    }

    /** Sets the overview doc comment using the overview option in 'opts' . */
    // NOTE: mjdoc looks for the overview file in the sourcepath as stated in the javadoc
    // documentation.  But javadoc 1.3 appears to look only in the current directory.  FIXME.
    static void setDocComment(MjDoc doc, MjcCommonOptions opts, String pathlist) {
        
        String overviewFile = null;
        if (((JavadocOptionsInterface)opts).overview() != null) {
                overviewFile = ((JavadocOptionsInterface)opts).overview();
                File f = findFile(pathlist,overviewFile); 
                if (f != null) setDocComment(doc,f);
                else {
                        root.printError("Could not find file " + overviewFile + " in path " + pathlist);
                }
        }
    }

    /** Searches for the given file in some component of the given pathlist, returning a 
        corresponding File object if found or null if not found.
    */ // !FIXME! - isn't there a system function to do this?
    static private File findFile(String dirlist, String filename) {
        File f = new File(filename);
        if (dirlist == null || f.isAbsolute()) {
            if (f.exists()) return f; else return null;
        }

        int i=0;
        if (dirlist != null) while (i < dirlist.length()) {
            int j = dirlist.indexOf(File.pathSeparatorChar,i);
            if (j == -1) j = dirlist.length();
            f = new File(dirlist.substring(i,j), filename);
            if (f.exists()) return f;
            i = j+1;
        }
        return null;
    }
        

    /** Sets the javadoc comment in the given Doc using the content
        (between <body> and </body>) of the given file.
    */
    static void setDocComment(MjDoc doc, File f) {
        try {
        if (!f.canRead()) {
            root.printError("Failed to read overview comment file " + f.getAbsolutePath());
            return;
        }
        String content = readFile(f);
        int n = content.indexOf("<BODY");
        if (n == -1) n = content.indexOf("<body");
        if (n == -1) {
                root.printError("No <body> tag found in the comment file " + f);
                return;
        }
        int nn = content.lastIndexOf("</BODY>");
        if (nn == -1) nn = content.lastIndexOf("</body>");
        if (nn == -1) {
            root.printError("No </body> tag found in the comment file " + f);
            return;
        }
        n += ("<body").length();
        while (content.charAt(n) != '>') ++n;
        ++n;
        content = content.substring(n,nn);
        doc.setRawCommentText(content,TokenReference.build(f,0,0));
        
        } catch (Exception e) {
            root.printError("Failed to read comment from file " + f.getAbsolutePath());
        }
        return;
    }
}
