/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjParamTag.java,v 1.1 2004/01/14 11:05:44 davidcok Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_142;
import com.sun.javadoc.*;

/** This class represents a javadoc comment @param tag; it is special
    in that it has a parameter name as well as a decription.
    @author David R. Cok
*/
public class MjParamTag extends MjTag implements ParamTag {

    /** A constant defining the 'kind' of this tag. */
    final public static String KIND = "@param";

    /** A holder for the name of the parameter begin documented. */ 
    //@ spec_public
    private String parameterName;
    
    /** The description associated with the parameter (but you normally
	should use the sequence of tags provided by inlineTags() ). */
    //@ spec_public
    private String comment;

    /** The constructor for an instance of this class; the argument 
	is the text that follows the '@param' in a javadoc comment; it
	will be parsed into the parameter name and the sequence of text
	and inline tags that make up the description.
	@param t unparsed text (parameter name and description)
    */ //!FIXME! verify behavior when t is empty or just has a parameter name
    //@ requires t != null;
    //@ ensures parameterName != null;
    //@ ensures comment != null;
    //@ ensures tags != null;
    public MjParamTag(Doc owner, String t, SourcePosition pos) {
	super(owner,KIND,t,pos);
	int i = 0;
	int len = t.length();
	while (i<len && Character.isWhitespace(t.charAt(i))) ++i;
	int j = i;
	while (i<len && !Character.isWhitespace(t.charAt(i))) ++i;	    
	parameterName = t.substring(j,i);
	fstags = i==j ? new Tag[0] : new Tag[]{new MjTextTag(owner,t.substring(j,len),pos)};
	while (i<len && Character.isWhitespace(t.charAt(i))) ++i;
	comment = t.substring(i,len);
	tags = new Tag[0];
	if (i == len) return;
	tags = inlineTags(comment,pos);
    }

    /** Accessor for the unprocessed comment (normally you would use inlineTags() ). */
    //@ also ensures \result != null;
    //@ pure
    public String parameterComment() { return comment; }
    
    /** Accessor for the name of the paramater being documented. */
    //@ also ensures \result != null;
    //@ pure
    public String parameterName() { return parameterName; }

    /** Returns the 'kind' of the parameter as required by the Tag interface. */
    //@ also ensures \result != null;
    //@ pure
    public String kind() { return KIND; }
    
    /** Return the source position of the documented entity, or null
     *  if no position is available.
     *  Added for JDK 1.4 compatability.
     */
//      public SourcePosition position() {
//          return null; /* !FIXME!JDK 1.4 compatability */
//      }
}
