/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjParameter.java,v 1.1 2004/01/14 11:05:44 davidcok Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_142;
import com.sun.javadoc.*; 
import org.multijava.mjc.CSpecializedType;
import org.multijava.mjc.CType;
import org.multijava.mjc.JFormalParameter;

/** The class implements the javadoc Parameter interface, and wraps the name and type of a MJC
    formal parameter to supply the information about the parameter to javadoc.
    @author David R. Cok
 */
public class MjParameter implements Parameter {

    /** The type of the formal parameter. */
    private MjType type; 
    
    /** The formal variable identifier. */
    private String name; 
    private String modString;

    /** Constructs a javadoc Parameter object from a MJC CSpecializedType object (from a CMethod object)
	and a formal parameter identifier.
     */
    public MjParameter(CSpecializedType cst, String name) { 
	if (cst.isSpecialized()) type = new MjType(cst);
	else                     type = new MjType(cst.staticType()); 
	this.name = name; 
        modString = "";
    }

    /** Constructs a javadoc Parameter object from a MJC CType object and formal parameter identifier. */
    public MjParameter(JFormalParameter jfp) { 
	if (jfp.isSpecialized()) type = new MjType(jfp.specializedType()); 
	else                     type = new MjType(jfp.staticType());
	this.name = jfp.ident(); 
	modString = jfp.modifiersAsString();
    }

    /** Returns the identifier of the formal parameter. */
    public String name() { return name; }

    /** Returns the fully-qualified type name, with dimensions, followed by a space and the formal identifier. */
    public String toString() { return typeName() + " " + name(); } 
    
    /** Returns any special modifiers of the parameter as a String. */
    public String modString() { return modString; }

    /** Returns the type of the formal parameter (as a MjType object). */ 
    public Type type() { return type; }

    /** Returns the fully-qualified type name, with dimension information. */
    public String typeName() { return type().toString(); } 

}
