/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocAllClassesFrameWriter.java,v 1.1 2004/01/14 11:05:44 davidcok Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_142;

import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/**
 * Generate the file with list of all the classes in this run. This page will be
 * used in the left-hand bottom frame, when "All Classes" link is clicked in 
 * the left-hand top frame. The name of the generated file is 
 * "allclasses-frame.html".
 *
 * @author Atul M Dambalkar
 * @author Doug Kramer
 */
public class MjdocAllClassesFrameWriter extends AllClassesFrameWriter {

    /**
     * Construct AllClassesFrameWriter object. Also initilises the indexbuilder
     * variable in this class.
     */
    public MjdocAllClassesFrameWriter(ConfigurationStandard configuration, 
		    String filename, IndexBuilder indexbuilder)
                              throws IOException {
        super(configuration, filename,indexbuilder);
    }

    /**
     * Create AllClassesFrameWriter object. Then use it to generate the 
     * "allclasses-frame.html" file. Generate the file in the current or the
     * destination directory.
     *
     * @param indexbuilder IndexBuilder object for all classes index.
     */ 
    public static void generate(ConfigurationStandard configuration,
				IndexBuilder indexbuilder) {
        MjdocAllClassesFrameWriter allclassgen;
        String filename = OUTPUT_FILE_NAME_FRAMES;
        try {
            allclassgen = new MjdocAllClassesFrameWriter(configuration,
                                                    filename, indexbuilder);
            allclassgen.generateAllClassesFile(true);
            allclassgen.close();
            filename = OUTPUT_FILE_NAME_NOFRAMES;
            allclassgen = new MjdocAllClassesFrameWriter(configuration,
                                                    filename, indexbuilder);
            allclassgen.generateAllClassesFile(false);
            allclassgen.close();
        } catch (IOException exc) {
            configuration.standardmessage.
                     error("doclet.exception_encountered",
                           exc.toString(), filename);
            throw new DocletAbortException();
        }
    }

    /**
     * Print all the classes in table format in the file.
     * Overridden in order to write a table of all the external functions.
     */
    protected void generateAllClassesFile(boolean wantFrames) throws IOException {
        String label = getText("doclet.All_Classes");
	
        printHtmlHeader(label); // CHANGED FOR 1.4.1_03

        printAllClassesTableHeader();
        printAllClasses(wantFrames);
        printAllClassesTableFooter();

	// Add in the external functions
	java.util.Iterator i = 
	    MjdocWrapper.root().topConcreteMethods(null).iterator();
	if (i.hasNext()) {
	    printAllGFTableHeader();
	    printAllGF(i);
	    printAllClassesTableFooter();
	}

        printBodyHtmlEnd();
    }

    /**
     * Print links for all the GFs.
     */
    protected void printAllGF(java.util.Iterator i) {
	while (i.hasNext()) {
	    MjMethodDoc m = (MjMethodDoc)(i.next());
            printTargetHyperLink(pathString(m.cuPackage(), m.gfFileName()),
				"classFrame", 
				((MjPackageDoc)(m.cuPackage())).prefix()+m.name()+m.signature());
            br();
	}
    }

    /**
     * Print the heading "All Classes" and also print Html table tag.
     */
    protected void printAllGFTableHeader() {
        fontSizeStyle("+1", "FrameHeadingFont");
        bold("All Generic Functions"); 
        fontEnd();
        br();
        table();
        tr();
        tdNowrap();
        fontStyle("FrameItemFont");
    }


    /**
     * Print the html file header. Also print Html page title and stylesheet
     * default properties.  Only overridden to change 'javadoc' to 'mjdoc'.
     * 
     * @param title String title for the generated html file.
     */
    public void printHeader(String title, String metakeywords) {
        println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Frameset//EN\"" +
                "\"http://www.w3.org/TR/REC-html40/frameset.dtd\">");    
        println("<!--NewPage-->");
        html();
        head();
        print("<!-- Generated by " + MjdocWrapper.appName() + " on ");
        print(today());
        println(" -->");
        if (configuration.charset.length() > 0) {
            println("<META http-equiv=\"Content-Type\" content=\"text/html; " 
                    + "charset=" + configuration.charset + "\">");
        }
        if ( configuration.windowtitle.length() > 0 ) {
            title += " (" + configuration.windowtitle  + ")";
        }
        title();
        println(title);
        titleEnd();
        if ( metakeywords.length() > 0 ) {
            println("<META NAME=\"keywords\" "
                        + "CONTENT=\"" + metakeywords + "\">");
        } 
        printStyleSheetProperties();
        headEnd();
        body("white",true); // CHANGED FOR 1.4.1_03
    }

}



