/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using
 * the doclet API, or you can start with the standard doclet and modify it to suit
your needs."
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocConstructorSubWriter.java,v 1.2 2004/07/01 22:16:47 leavens Exp $
 * Author: David R. Cok
 */

// FIXME -- needs javadoc documentation

package org.multijava.mjdoc.mjdoc_142;

import com.sun.tools.doclets.*;
import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.standard.tags.*;
import com.sun.javadoc.*;
import java.lang.reflect.Modifier;
import java.util.*;

/** This is an extension of the doclet api in order to provide
    for adjusting the spacing in the javadoc and specs.
    @author David R. Cok 
*/ 

public class MjdocConstructorSubWriter extends ConstructorSubWriter {

    public MjdocConstructorSubWriter(SubWriterHolderWriter writer, ClassDoc classdoc) {
        super(writer, classdoc);
    }

    public MjdocConstructorSubWriter(SubWriterHolderWriter writer) {
        super(writer);
    }

    // Overridden in order to insert modifiers
    protected void printParam(Parameter param) { 
        String s = ((MjParameter)param).modString();
        if (s.length() != 0) writer.bold(s);
        super.printParam(param);
    }       

    /** Prints the html for the tag information.  This method is overridden from
	MethodSubWriter to fix this bug: when there are not throws tags but
	there are exceptions declared as thrown, the header for the throws tags is
	not placed in the html.  It is also overridden to clean up some formatting
	(blank lines) in the output.
    */
    protected void printTags(ProgramElementDoc member) {
	ConfigurationStandard configuration = configuration();
        if(configuration.nocomment){
            return;
        }
        configuration.tagletManager.checkTags(member, member.tags(), false);
        configuration.tagletManager.checkTags(member, member.inlineTags(), true);
        ConstructorDoc constructor = (ConstructorDoc) member;
        Taglet[] definedTags = configuration.tagletManager.getConstructorCustomTags();
        Tag[][] tagsInDoc = new Tag[definedTags.length][];
        for (int i = 0; i < definedTags.length; i++) {
            tagsInDoc[i] = constructor.tags(definedTags[i].getName());
        }
        String output;
	StringBuffer alloutput = new StringBuffer();
	boolean earlyHeader = anyParamTags(constructor);
	if (earlyHeader) printTagsInfoHeader();
        for (int i = 0; i < definedTags.length; i++) {
            if (tagsInDoc[i].length == 0 && !(definedTags[i] instanceof ThrowsTaglet)) {
                output = null;
            } else {
                if (definedTags[i] instanceof SimpleTaglet) {
                    output = ((SimpleTaglet) definedTags[i]).toString(tagsInDoc[i], writer);
		} else if (definedTags[i] instanceof ThrowsTaglet && tagsInDoc[i].length == 0) {
		    // This case is here solely to fix a bug in Apple/Sun's library.  The
		    // header for the section of throws information is not otherwise printed
		    // if there are declared exceptions but no throws tags.
		    output = ((AbstractExecutableMemberTaglet) definedTags[i]).toString(constructor, writer);
		    // The following string has to match the header used in ThrowsTaglet.java
		    // The need is to add a header when it is incorrectly omitted.
                    // !FIXME! should that \n be below?
		    String header = "\n<DT>" + "<B>" + writer.getText("doclet.Throws") + "</B>";
		    if (output != null && !output.startsWith(header))
			output = header + output;
		} else if (definedTags[i] instanceof AbstractExecutableMemberTaglet) {
		    output = ((AbstractExecutableMemberTaglet) definedTags[i]).toString(constructor, writer);
		} else {
		    output = definedTags[i].toString(tagsInDoc[i]);
		}
            }
            if (output != null) {
                configuration.tagletManager.seenCustomTag(definedTags[i].getName());
                alloutput.append(output); //print(output);
            }
        }
	output = alloutput.toString();
	// The writing to the output is combined here so as to avoid adding a header and
	// footer when there is no output.
	if (output.length() > 0) {
	    if (!earlyHeader) printTagsInfoHeader();
	    print(output);
	    printTagsInfoFooter();
	} else if (earlyHeader) {
	    printTagsInfoFooter();
	}
    }

    // There is a design flaw in the printing of the param tags in that
    // the header is printed directly to the writer, but the rest of the
    // info is stored in the output String.  Thus above we need this 
    // helper function in order to determine if any output is going to be
    // written, so that we know whether to generate the printTagsInfoHeader
    protected boolean anyParamTags(ExecutableMemberDoc execHolder) {
        ParamTag[] tags = execHolder.paramTags();
        String result = "";
        MethodDoc inheritedMethod;
	if (execHolder.paramTags().length > 0) return true;
	if (execHolder instanceof MethodDoc &&
                (inheritedMethod = getInheritedMethodDoc((MethodDoc) execHolder)) != null) {
		if (inheritedMethod.paramTags().length > 0) return true;
	}
	return false;
    }

    // Copied from AbstractExecutableMemberTaglet just to provide 
    // functionality needed in anyParamTags
    protected MethodDoc getInheritedMethodDoc(MethodDoc method){
        //Find method to inherit tags from.
        TaggedMethodFinder finder = new TaggedMethodFinder();
        MethodDoc inheritedMethod = finder.search(method.containingClass(), method);
        if (inheritedMethod == null) {
            //check interfaces since method is not in superclass.
            ClassDoc[] interfaces = method.containingClass().interfaces();
            for (int i = 0; i < interfaces.length; i++) {
                inheritedMethod = finder.search(interfaces[i], method);
                if (inheritedMethod != null) {
                    return inheritedMethod;
                }
            }
        }
        return inheritedMethod;
    }

    protected void printTagsInfoHeader() {
        writer.dd();
        writer.dl();
    }
    
    protected void printTagsInfoFooter() {
        writer.dlEnd();
        writer.ddEnd();
    }
    
 
}





