/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocEMWriter.java,v 1.3 2004/04/17 20:39:29 davidcok Exp $
 * Author: David R. Cok
 */
 
package org.multijava.mjdoc.mjdoc_142;

import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// !FIXME!  Needs more internal documentation
/**
 * This class takes care of writing an html page for the
 * contents of a compilation unit that contains 
 * external methods rather than classes.
 
 * It was written by analogy with ClassWriter, which is
 * proprietary information of Sun Microsystems, Inc.  

 * @see com.sun.javadoc.ClassDoc 
 * @see java.util.Collection
 * @see java.util.List
 * @see java.util.ArrayList
 * @see java.util.HashMap
 *
 * @author David Cok
 */
public class MjdocEMWriter extends MjdocMethodWriter {

    MjExtMethodsDoc prev;
    MjExtMethodsDoc next;
    
    public MjdocEMWriter(ConfigurationStandard configuration,
		String path, String filename, MjExtMethodsDoc methoddoc,
                MjExtMethodsDoc prev, MjExtMethodsDoc next, ArrayList methods,
                boolean nopackage) throws IOException {
        super(configuration, path, filename, methoddoc, null, null, methods, nopackage);
	this.prev = prev;
	this.next = next;
        this.classdoc = null;
    }

    /**
     * Generate a external method compilation unit page.
     *
     * @param prev the previous class to generated, or null if no previous.
     * @param methoddoc the class to generate.
     * @param next the next class to be generated, or null if no next.
     */
    public static void generate(ConfigurationStandard configuration,
			     MjExtMethodsDoc methoddoc, MjExtMethodsDoc prev, 
                             MjExtMethodsDoc next, ArrayList methods, 
                             boolean nopackage) {
            MjdocEMWriter clsgen;
            String path = 
               DirectoryManager.getDirectoryPath(methoddoc.containingPackage());
            String filename = methodFilename(methoddoc,true);
            try {
                clsgen = new MjdocEMWriter(configuration,
					path, filename, methoddoc, 
					prev, next, methods, nopackage);
                clsgen.generateExtMethodsFile(methoddoc,methods);
                clsgen.close();
            } catch (IOException exc) {
                configuration.standardmessage.
                    error("doclet.exception_encountered",
                           exc.toString(), filename);
                throw new DocletAbortException();
            }
    }

    /**
     * Print class page indicator
     */
    protected void navLinkGF() {
        navCellRevStart();
        fontStyle("NavBarFont1Rev");
        bold("FCNS");
        fontEnd();
        navCellEnd();
    }

    /**
     * Print previous package link
     */
    protected void navLinkPrevious() {
        if (prev == null) {
            print("PREV EXT METHOD UNIT"); 
        } else {
            print(getEMLink(prev, "", "PREV EXT METHOD UNIT", true, ""));
        }
    }
                                
    /**
     * Print next package link
     */
    protected void navLinkNext() {
        if (next == null) {
            print("NEXT EXT METHOD UNIT"); 
        } else {
            print(getEMLink(next, "", "NEXT EXT METHOD UNIT", true, ""));
        }
    }                               

    /**
     * Generate the file contents.
     */
    public void generateExtMethodsFile(MjExtMethodsDoc top, ArrayList methods) {
        PackageDoc pkg = top.containingPackage();
        String pkgname = (pkg != null)? pkg.name(): "";
        String label = "external method unit " + top.name();
        String label2 = "<FONT SIZE=\"-1\">external method unit</FONT> " + top.name();

        printHtmlHeader(configuration.windowtitle + " - " + label);
        navLinks(true);
        hr();
        println("<!-- ======== START OF GENERIC FUNCTION DATA ======== -->");
        h2();
        if (pkgname.length() > 0) {
            font("-1"); print(pkgname); fontEnd(); br();
        }
        print(label2);
        h2End();

        hr();

        println("<!-- ========== METHOD SUMMARY =========== -->"); println();
	methodSubWriter.printMethodsSummary(methods);

        p();

        println("<!-- ============ METHOD DETAIL ========== -->"); println();
	methodSubWriter.printMethods(methods);

        println("<!-- ========= END OF GENERIC FUNCTION DATA ========= -->");
        hr();
        navLinks(false);
        printBottom();
        printBodyHtmlEnd();
    }

  
    /** This is used to generate a link to the file that contains the description
	of a external method unit.  It is used by the PREF and NEXT buttons in the
	header of a generic function page.
    */ // !FIXME! - cleanup
    public String getEMLink(MjExtMethodsDoc cd, String where,
                               String label, boolean bold, String color) {

        //if (cd.isIncluded()) {
            //if (isGeneratedDoc(cd)) {
                String filename = cd.name() + ".html";
		filename = pathString(cd.containingPackage(), filename);
                return getHyperLink(filename, where, label, bold, color);
            //}
        //} else {
        //    String crosslink = getCrossClassLink(cd);
        //    if (crosslink != null) {
        //        return getHyperLink(crosslink, where, label, bold, color);
        //    }
       // }
        //if (nameUnspecified) {
        //    displayLength -= label.length();
        //    label = cd.qualifiedName();
        //    displayLength += label.length();
        //}            
        //return label;
    }

    /** This generates the filename (not the directory part) for the html
	file that holds the decription of the given top method.
    */
    static String methodFilename(MjExtMethodsDoc m, boolean nameOnly) {
	String f;
	f = m.name() + ".html";
	return f;
    }
    
/*    public void printSummaryLinkComment(AbstractSubWriter mw,
                                        ProgramElementDoc member) {
        codeEnd();
        println(); br();
	//System.out.println("printSummaryLinkComment " + member.name());
	if (member instanceof MjMethodDoc && ((MjMethodDoc)member).isExternal()) {
	    print("<B>External</B>"); br();
	}
        printCommentDef(member);
        summaryRowEnd();
        trEnd();   
    }   
*/
    /**
     * Print the navigation bar for the Html page at the top and and the bottom.
     *
     * @param header If true print navigation bar at the top of the page else
     * print the nevigation bar at the bottom.
     */
    protected void navLinks(boolean header) {
        println("");
        println("<!-- ========== START OF NAVBAR ========== -->");
        if (!configuration.nonavbar) {
            if (header) {
                anchor("navbar_top");
            } else {
                anchor("navbar_bottom");
            }
            table(0, "100%", 1, 0);
            tr();
            tdColspanBgcolorStyle(2, "#EEEEFF", "NavBarCell1");
            println("");
            if (header) {
                anchor("navbar_top_firstrow");
            } else {
                anchor("navbar_bottom_firstrow");
            }
            table(0, 0, 3);
            print("  ");
            trAlignVAlign("center", "top");

	    if (configuration.createoverview) {
                navLinkContents();
            }

            if (configuration.packages.length > 0) {
                navLinkPackage();
            }

            navLinkClass();
	    
	    navLinkGF();

            if(configuration.classuse) {
                navLinkClassUse();
            }
            if(configuration.createtree) {
                navLinkTree();
            }
            if(!(configuration.nodeprecated || 
                    configuration.nodeprecatedlist)) {
                navLinkDeprecated();
            }
            if(configuration.createindex) {
                navLinkIndex();
            }
            if (!configuration.nohelp) {
                navLinkHelp();
            }
            print("  ");
            trEnd();
            tableEnd();
            tdEnd();

            tdAlignVAlignRowspan("right", "top", 3);

            printUserHeaderFooter(header);
            tdEnd();
            trEnd();
            println("");

            tr();
            tdBgcolorStyle("white", "NavBarCell2");
            font("-2");
            space();
            navLinkPrevious();
            space();
            println("");
            space();
            navLinkNext();
            fontEnd();
            tdEnd();

            tdBgcolorStyle("white", "NavBarCell2");
            font("-2");
            print("  ");
            navShowLists();
            print("  ");
            space();
            println("");
            space();
            navHideLists();
            fontEnd();
            tdEnd();

            trEnd();
            
            printSummaryDetailLinks();

            tableEnd();
            println("<!-- =========== END OF NAVBAR =========== -->");
            println("");
        }
    }  
}


