/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocGFWriter.java,v 1.3 2004/04/17 20:39:29 davidcok Exp $
 * Author: David R. Cok
 */
 
package org.multijava.mjdoc.mjdoc_142;

import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// !FIXME!  Needs more internal documentation
/**
 * This class takes care of writing html pages that document individual
 * generic functions (a family of related external and internal methods).
 * It was written by analogy with ClassWriter, which is
 * proprietary information of Sun Microsystems, Inc.  

 * @see com.sun.javadoc.ClassDoc 
 * @see java.util.Collection
 * @see java.util.List
 * @see java.util.ArrayList
 * @see java.util.HashMap
 *
 * @author David Cok
 */
public class MjdocGFWriter extends MjdocMethodWriter {


    public MjdocGFWriter(ConfigurationStandard configuration,
		String path, String filename, MjMethodDoc methoddoc,
                MjMethodDoc prev, MjMethodDoc next, ArrayList methods,
                boolean nopackage) throws IOException {
        super(configuration, path, filename, methoddoc, prev, next, methods, nopackage);
        this.classdoc = methoddoc.containingClass();
    }

    /**
     * Generate a method page.
     *
     * @param prev the previous method to generated, or null if no previous.
     * @param methoddoc the method to generate.
     * @param next the next method to be generated, or null if no next.
     */
    public static void generate(ConfigurationStandard configuration,
			    MjMethodDoc methoddoc, MjMethodDoc prev, 
			    MjMethodDoc next, ArrayList methods, 
			    boolean nopackage) {
            MjdocGFWriter clsgen;
            String path = 
               DirectoryManager.getDirectoryPath(methoddoc.cuPackage());
            String filename = methodFilename(methoddoc);
            try {
                clsgen = new MjdocGFWriter(configuration, path, filename, methoddoc, 
                                         prev, next, methods, nopackage);
                clsgen.generateGenericFunctionFile(methoddoc,methods);
                clsgen.close();
            } catch (IOException exc) {
                configuration.standardmessage.
                    error("doclet.exception_encountered",
                           exc.toString(), filename);
                throw new DocletAbortException();
            }
    }

    /**
     * Generate the file contents.
     */
    public void generateGenericFunctionFile(MjMethodDoc top, ArrayList methods) {
	methodSubWriter.inGFHTMLPage = true;
        PackageDoc pkg = top.cuPackage();
        String pkgname = (pkg != null)? pkg.name(): "";
        String label = "generic function " + top.name();
        String label2 = "<FONT SIZE=\"-1\">generic function</FONT> " + top.name()+top.flatSignature();

		// Line below CHANGED FOR 1.4.1_03
        printHtmlHeader(configuration.windowtitle + " - " + label);
        navLinks(true);
        hr();
        println("<!-- ======== START OF GENERIC FUNCTION DATA ======== -->");
        h2();
        if (pkgname.length() > 0) {
            font("-1"); print(pkgname); fontEnd(); br();
        }
        print(label2);
        h2End();

        hr();

        println("<!-- ========== METHOD SUMMARY =========== -->"); println();
	methodSubWriter.printMethodsSummary(methods);

        p();

        println("<!-- ============ METHOD DETAIL ========== -->"); println();
	methodSubWriter.printMethods(methods);

        println("<!-- ========= END OF GENERIC FUNCTION DATA ========= -->");
        hr();
        navLinks(false);
        printBottom();
        printBodyHtmlEnd();
    }


    /** This is used to generate a link to the file that contains the description
	of a generic function.  It is used by the PREF and NEXT buttons in the
	header of a generic function page.
    */ // !FIXME! - cleanup
    public String getGFLink(MjMethodDoc cd, String where,
                               String label, boolean bold, String color) {
        boolean nameUnspecified = label.length() == 0;
        if (nameUnspecified) {
            label = cd.name();
        }       
        displayLength += label.length();
        //if (cd.isIncluded()) {
            //if (isGeneratedDoc(cd)) {
                String filename = methodFilename(cd);
		filename = pathString(cd.cuPackage(), filename);
                return getHyperLink(filename, where, label, bold, color);
            //}
        //} else {
        //    String crosslink = getCrossClassLink(cd);
        //    if (crosslink != null) {
        //        return getHyperLink(crosslink, where, label, bold, color);
        //    }
       // }
        //if (nameUnspecified) {
        //    displayLength -= label.length();
        //    label = cd.qualifiedName();
        //    displayLength += label.length();
        //}            
        //return label;
    }

    /** This generates the filename (not the directory part) for the html
	file that holds the decription of the given top method.
    */
    static String methodFilename(MjMethodDoc m) {
	return m.gfFileName();
    }

}


