/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * also based in part on material from the javadoc Doclet API,
 * that is Copyright by Sun Microsystems, Inc.
 * about which Sun writes: "You can write the doclets from scratch using 
 * the doclet API, or you can start with the standard doclet and modify it to suit your needs."
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MjdocStandard.java,v 1.5 2004/07/01 22:16:47 leavens Exp $
 * Author: David R. Cok
 */

package org.multijava.mjdoc.mjdoc_142;

import com.sun.tools.doclets.standard.*;
import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.util.*;
import java.io.*;

/**
 * This class is an extension and modification of the Standard doclet, that
 * is the usual javadoc implementation.  In the mjdoc tool we provide our own
 * parser and parse tree for (multi-)java, but use the doclet library (with some
 * extensions) in order to render the html.  This class provides the connection
 * between the two worlds.
 * @author David R. Cok
 */
public class MjdocStandard extends com.sun.tools.doclets.standard.Standard {

    /**
     * The "start" method as required by Javadoc.
     *
     * @param root
     * @see com.sun.javadoc.RootDoc
     * @return boolean true indicates a successful generation of html
     */
    public static boolean start(RootDoc root) throws IOException {
	MjdocStandard s;
	try {
	    (s = new MjdocStandard()).startGeneration(root);
	} catch (DocletAbortException exc) {
	    return false;
	}
        s.configuration.tagletManager.printReport();
	return true;
    }

    public String name() { return "Mjdoc"; }
    
    // All of the following comes from or by analogy to the parent Standard class.  Most functionality
    // is simply used by inheritance.  A few things are (or may be overridden).

    /** This is the superclass version of startGeneration, which we could use directly except for 
	having to replace some classes by the Mjdoc version.
    */
    protected void startGenerationSuperMJ(RootDoc root) throws DocletAbortException {
        configuration.setOptions(root);
        if(!configuration.quiet) {
            // !FIXME! should that \n be below?
            root.printNotice(name() + " " + BUILD_DATE + "\n");
            configuration.printedVersion = true;
        }
        if (configuration.genSrc) {
            if (configuration.destdirname.length() > 0) {
                SourceToHTMLConverter.convertRoot(configuration,
                                                  root, configuration.destdirname + File.separator
                                                      + configuration.SOURCE_OUTPUT_DIR_NAME);
            } else {
                SourceToHTMLConverter.convertRoot(configuration,
                                                  root, configuration.SOURCE_OUTPUT_DIR_NAME);
            }
        }
        ConstantsSummaryWriter.generate(configuration);
        if (root.classes().length == 0) {
            configuration.standardmessage.
                error("doclet.No_Public_Classes_To_Document");
            return;
        }

        if (configuration.topFile.length() == 0) {
            configuration.standardmessage.
                        error("doclet.No_Non_Deprecated_Classes_To_Document");
            return;
        }            

        boolean nodeprecated = configuration.nodeprecated;

        String configdestdir = configuration.destdirname;
        String confighelpfile = configuration.helpfile;
        String configstylefile = configuration.stylesheetfile;

        performCopy(configdestdir, confighelpfile);
        performCopy(configdestdir, configstylefile);

        ClassTree classtree = new ClassTree(configuration, nodeprecated);

        // do early to reduce memory footprint
        if (configuration.classuse) {
            ClassUseMapper.generate(configuration, classtree);
        }

        IndexBuilder indexbuilder = new IndexBuilder(configuration, nodeprecated);
        PackageDoc[] inclPackages = configuration.packages;

        if (configuration.createtree) {
            TreeWriter.generate(configuration,classtree);
        }

        if (configuration.createindex) {
            if (configuration.splitindex) {
                SplitIndexWriter.generate(configuration,indexbuilder);
            } else {
                SingleIndexWriter.generate(configuration,indexbuilder);
            }
        }

        if (!(configuration.nodeprecatedlist || nodeprecated)) { 
            DeprecatedListWriter.generate(configuration);
        }

        MjdocAllClassesFrameWriter.generate(configuration,
			new IndexBuilder(configuration, nodeprecated, 
                                                        true));
		    // true above for 'classesOnly'

        FrameOutputWriter.generate(configuration);

        PackagesFileWriter.generate(configuration);
            
        if (configuration.createoverview) {
            PackageIndexWriter.generate(configuration);
        }
        
        if (inclPackages.length > 1) {
            PackageIndexFrameWriter.generate(configuration);
        }

        if (! (inclPackages.length == 1 && inclPackages[0].name().equals(""))) {
            PackageDoc prev = null, next;
	    for(int i = 0; i < inclPackages.length; i++) {
                PackageFrameWriter.generate(configuration, inclPackages[i]);
                //Don't generate package page for unnamed package
                if (inclPackages[i].name().length() == 0) {
                    continue;
                }
                next = (i + 1 < inclPackages.length && inclPackages[i+1].name().length() > 0) ?
                    inclPackages[i+1]:null;
                //If the next package is unnamed package, skip 2 ahead if possible
                next = (i + 2 < inclPackages.length && next == null) ?
                    inclPackages[i+2]: next;
		MjdocPackageWriter.generate(configuration, inclPackages[i], prev, next);
		if (configuration.createtree) {
		    PackageTreeWriter.generate(configuration, inclPackages[i], prev, next, 
					    nodeprecated);
		}
		// FIXME ??? MjdocPackageFrameWriter.generate(packages[i]);
                prev = inclPackages[i];
	    }
	}

        generateClassFiles(root, classtree);

        if (configuration.sourcepath != null &&
            configuration.sourcepath.length() > 0) {
            StringTokenizer pathTokens =
                new StringTokenizer(configuration.sourcepath, 
					File.pathSeparator);
            boolean first = true;
            while(pathTokens.hasMoreTokens()){
                MjdocStandard.copyDocFiles(configuration, 
			pathTokens.nextToken() + File.separator, 
			HtmlStandardWriter.DOC_FILES_DIR_NAME, first);
                first = false;
            }
        }
        if (generateSerializedFormPage(root)) {
            SerializedFormWriter.generate(configuration);
        }

        PackageListWriter.generate(configuration);

        if (configuration.helpfile.length() == 0 &&
               !configuration.nohelp) {
            HelpWriter.generate(configuration);
        }
        if (configuration.stylesheetfile.length() == 0) {
            StylesheetWriter.generate(configuration);
        }
	
    }
    
    /**
     * Start the generation of files. Call generate methods in the individual
     * writers, which will in turn genrate the documentation files. Call the
     * TreeWriter generation first to ensure the Class Hierarchy is built
     * first and then can be used in the later generation.
     *
     * For new format.
     *
     * @see com.sun.javadoc.RootDoc
     */
    protected void startGeneration(RootDoc root) throws DocletAbortException {

	startGenerationSuperMJ(root);
	
	// Now do any compilation units that hold external methods

	if (MjdocWrapper.docupages) generateEMCUFiles(root);	
	
	// The remainder writes out generic function files.
	
	if (MjdocWrapper.donofcns) return;	
	
	java.util.Map map = new java.util.TreeMap(new MjExecutableMemberDoc.Comp());

	// First collect and group all the methods by their top method
	java.util.Iterator i = ((MjRootDoc)root).allMethods().iterator();
	while (i.hasNext()) {
	    MjMethodDoc mem = (MjMethodDoc)(i.next());
	    MjMethodDoc top = mem.topConcreteMethod();
	    if (!MjdocWrapper.doallfcns && !top.isExternal()) continue;
	    if (map.containsKey(top)) {
		Object o = map.get(top);
		java.util.ArrayList v = (java.util.ArrayList)o;
		v.add(mem);
	    } else {
		java.util.ArrayList v = new java.util.ArrayList();
		v.add(mem);
		map.put(top,v);
	    }
	}
	
	// !FIXME! Should any sorting be done?
	
	// Then iterate over the top methods, printing out a page
	// for each one, with all of its family of methods.
	// The odd looping is because we need a previous and
	// next function available when we write the page
	// of the current function.
	i = map.entrySet().iterator();
	MjMethodDoc prev = null;
	MjMethodDoc top = null;
	Map.Entry next = null;
	java.util.ArrayList v;
	if (i.hasNext()) {
	    next = (Map.Entry)(i.next());
	    prev = top;
	    top = (MjMethodDoc)(next.getKey()); 
	}
	while (i.hasNext()) {
	    v = (java.util.ArrayList)(next.getValue());
	    next = (Map.Entry)(i.next());
	    writeGenericFunction(top,prev,(MjMethodDoc)(next.getKey()),v);
	    prev = top; 
	    top = (MjMethodDoc)(next.getKey()); 
	}
	if (top != null) {
	    v = (java.util.ArrayList)(next.getValue());
	    writeGenericFunction(top,prev,null,v);
	}
    }



    protected void generateEMCUFiles(RootDoc root) {
	// Do any that are specified individually
	generateEMCUCycle(((MjRootDoc)root).specifiedExtMethods().iterator());
	// Do any in packages
        PackageDoc[] packages = configuration.packages;
        for (int k = 0; k < packages.length; k++) {
 	    generateEMCUCycle(((MjPackageDoc)(packages[k])).extMethodsUnits().iterator());
	}
    }
    
    public void generateEMCUCycle(Iterator i) {
	    MjExtMethodsDoc prev = null;
	    MjExtMethodsDoc next = null;
	    MjExtMethodsDoc mje = null;

	    if (i.hasNext()) {
		mje = (MjExtMethodsDoc)(i.next());
	    }
	    while (i.hasNext()) {
		next = (MjExtMethodsDoc)(i.next());
		MjdocEMWriter.generate(configuration,mje,prev,next,mje.methods(),false);
		prev = mje;
		mje = next;
	    }
	    if (mje != null) {
		next = null;
		MjdocEMWriter.generate(configuration,mje,prev,next,mje.methods(),false);
	    }    
    
    }

    /** Writes out a specific generic function page. */
    public void writeGenericFunction(MjMethodDoc top, MjMethodDoc prev, MjMethodDoc next, java.util.ArrayList v) throws DocletAbortException {

	PackageDoc pkg = top.containingPackage();
	
	MjdocGFWriter.generate(configuration,top,prev,next,v,false); 
    
    }
    
    /**
     * Instantiate ClassWriter for each Class within the ClassDoc[]
     * passed to it and generate Documentation for that.  This is overridden
     * simply to be able to use a MjdocClassWriter.
     */
    protected void generateClassCycle(ClassDoc[] arr, ClassTree classtree) {
	generateClassCycle(arr,classtree,false);
    }
    protected void generateClassCycle(ClassDoc[] arr, ClassTree classtree,
						boolean nopackage) {
        Arrays.sort(arr);
        for(int i = 0; i < arr.length; i++) {
            if (configuration.nodeprecated && 
                     arr[i].tags("deprecated").length > 0) {
                continue;
            }
            ClassDoc prev = (i == 0)? 
                            null:
                            arr[i-1];
            ClassDoc curr = arr[i];
            ClassDoc next = (i+1 == arr.length)? 
                            null:
                            arr[i+1];

            generateClass(configuration,
		    curr, prev, next, classtree, nopackage);
        }
    }
    
    protected void generateClass(ConfigurationStandard configuration,
			    ClassDoc curr, ClassDoc prev,
                             ClassDoc next, ClassTree classtree,
                             boolean nopackage) {
	MjdocClassWriter.generate(configuration,
		    curr, prev, next, classtree, nopackage);
    } 
    // This copies everything in directory arg#2 + File.separator + arg#3
    // into directory destination + File.separator + arg#3
    // With the above patch, it should not be called if arg#2 is
    // the same as the destination directory
    // This method exists just to patch a bug.  The HtmlStandardWriter
    // method uses : explicitly as a path separator, so it fails on
    // Windows.
    static public void copyDocFiles(ConfigurationStandard configuration,
		    String srcDirName, String path, boolean overwrite) 
			throws DocletAbortException {
        File srcdir = new File(srcDirName,path);
        if (! srcdir.exists()) {
            return;
        }
	String destname = configuration.destdirname;
        if (destname.length() > 0 && !destname.endsWith(File.separator)) {
            destname += File.separator;
        }
        String dest = destname + path;
        try {
            File destdir = new File(dest);
	    if (!srcdir.getCanonicalPath().equals(destdir.getCanonicalPath()))
		HtmlStandardWriter.copyDocFiles(configuration, srcDirName, path, overwrite);        

            DirectoryManager.createDirectory(configuration, dest);
            String[] files = srcdir.list();
            for (int i = 0; i < files.length; i++) {
                File srcfile = new File(srcdir, files[i]);
                File destfile = new File(destdir, files[i]);
                if (srcfile.isFile()) {
                    if(destfile.exists() && ! overwrite) {
                        configuration.standardmessage.warning((SourcePosition) null,
                                                              "doclet.Copy_Overwrite_warning",
                                                              srcfile.toString(), destdir.toString());
                    } else {
                        configuration.standardmessage.notice("doclet.Copying_File_0_To_Dir_1",
                                                             srcfile.toString(), destdir.toString());
                        Util.copyFile(destfile, srcfile);
                    }
                } else if(srcfile.isDirectory()) {
                    if(configuration.copydocfilesubdirs
                       && ! configuration.shouldExcludeDocFileDir(srcfile.getName())){
                        copyDocFiles(configuration, srcDirName,
                                     path + File.separator + srcfile.getName(), overwrite);
                    }
                }
            }
        } catch (SecurityException exc) {
            throw new DocletAbortException();
        } catch (IOException exc) {
            throw new DocletAbortException();
        }

    }

}        

