// An object of this class represents a particular argument
// combination in a gf that's an ambiguous message error.

package org.multijava.relaxed.runtime;

import java.util.*;


public class RMJErrorSignature
{
    public RMJErrorSignature(RMJSignature s, Object[] specs)
    {
	sig = s;
	specializers = specs;
    }

    public String ambigErrorClassName()
    {
	return sig.ambigErrorClassName();
    }

    public Class[] errorSpecializers()
    {
	// we only want to return the specializers that are actually
	// specializable by some method in an ambiguous way, i.e.,
	// that have class types as static argument types.

	List errorSpecializers = new LinkedList();

	Class[] staticTypes = sig.staticTypes();
	// staticTypes and specializers should have the same length
	
	for (int i = 0; i < staticTypes.length; i++) {
	    Class staticType = staticTypes[i];
	    if (staticType == null) {
		// not specializable; skip this
	    } else {
		Class specializer = (Class) specializers[i];
		if (staticType.equals(specializer)) {
		    // this position is unspecialized
		    specializer = null;
		}
		errorSpecializers.add(specializer);
	    }
	}

	// convert to an array of Classes
	int numErrorSpecs = errorSpecializers.size();
	Class[] errorSpecs = new Class[numErrorSpecs];
	Iterator iter = errorSpecializers.iterator();
	for (int i = 0; i < numErrorSpecs; i++) {
	    errorSpecs[i] = (Class) iter.next();
	}

	return errorSpecs;
    }

    public boolean isInternalGF()
    {
	return sig.isInternalGF();
    }

    public String toString()
    {
	return "RMJErrorSignature[" + bodyString() + "]";
    }

    public String bodyString()
    {
	String s = 
	    sig.operation().bodyString() + "{" + ambigErrorClassName() + "} (";
	Class[] errorSpecs = errorSpecializers();
	for (int i = 0; i < errorSpecs.length; i++) {
	    Class errorSpec = errorSpecs[i];
	    if (i > 0) s += ",";
	    s += errorSpec;
	}
	s += ")";
	return s;
    }

    private RMJSignature sig;
    private Object[] specializers;
}
