package org.multijava.relaxed.runtime;

import org.multijava.relaxed.util.*;
import org.multijava.relaxed.util.classfile.*;

import java.io.*;
import java.util.*;
import java.net.URL;
import java.lang.reflect.*;

public class RMJPreloader extends RMJClassLoader
{
    static public void main(String args[])
	throws ClassNotFoundException, NoSuchMethodException, 
	       IllegalAccessException, InvocationTargetException
    {
	// Create the special RMJ-aware class pre-loader
	RMJClassLoader loader = new RMJPreloader();
	
	// run its main function
	loader.loaderMain(args);
    }

    protected String usage() 
    {
	return "usage: " +
	    "java org.multijava.relaxed.runtime.RMJPreloader class";
    }

    protected void runProgram(Class mainCls, String[] args) 
	throws ClassNotFoundException
    {
	// we "run" the program by recursively traversing all classes
	// that have been loaded, loading any classes that they
	// reference
	while (! classesToTraverse.isEmpty()) {
	    Class cls = (Class) classesToTraverse.removeFirst();
	    traverseClass(cls);
	}
    }


    ///////////////////////////////////////////////////////////////////////

    public RMJPreloader()
    {
    }
    
    public RMJPreloader(ClassLoader parent)
    {
	super(parent);
    }


    ///////////////////////////////////////////////////////////////////////

    // after we load a class, we remember that we need to traverse it
    public Class loadClass(String name)
	throws ClassNotFoundException
    {
	Class cls = super.loadClass(name);
	classesToTraverse.addLast(cls);
	return cls;
    }

    // load every class mentioned by this class, recursively
    protected void traverseClass(Class cls) 
	throws ClassNotFoundException
    {
	if (RMJDebug.verbose) {
	    RMJDebug.out("Traversing " + cls.getName());
	}

	byte[] bytes = readClassByteCode(cls.getName());
	
	if (RMJDebug.classdump) {
	    Readdump.dump(bytes);
	}

	PreloaderClassInfo info = new PreloaderClassInfo();
	info.process(bytes);

	Iterator iter = info.classNames().iterator();
	while (iter.hasNext()) {
	    String className = (String) iter.next();
	    if (! isLoaded(className)) {
		Class cls2 = loadClass(className);
	    }
	}

	if (RMJDebug.verbose) {
	    RMJDebug.out("Done traversing " + cls.getName());
	}
    }


    ///////////////////////////////////////////////////////////////////////

    // A list of classes (Class) that have been loaded but that
    // haven't yet been traversed
    protected LinkedList classesToTraverse = new LinkedList();

}
