// An object of this class represents an unloaded signature, e.g. a
// glue method or a signature for an internal method of an external
// operation that hasn't been loaded yet.  The main thing this object
// tracks is the signature annotation plus information about how many
// unloaded classes this annotation is waiting on.

package org.multijava.relaxed.runtime;

import org.multijava.relaxed.util.*;

import java.util.*;

public class RMJUnreachableSignature
{
    public RMJUnreachableSignature(RMJAnnotation.Method anno, 
				   Set unloadedClasses) {
	_anno = anno;
	_unloadedCount = unloadedClasses.size();
    }


    // OPERATIONS

    // register the fact that a class that this signature was waiting
    // on has now been loaded
    public void registerLoadedClass(String className) {
	_unloadedCount --;
    }

    // return whether this signature is now reachable
    public boolean isReachable() {
	return _unloadedCount == 0;
    }


    public String toString() {
	return "RMJUnreachableSignature[" + bodyString() + "]";
    }

    public String bodyString() {
	return _anno + "#" + _unloadedCount;
    }


    public boolean equals(/*@ nullable @*/ Object other) {
	return other instanceof RMJUnreachableSignature &&
	    _anno.toString().equals(
	        ((RMJUnreachableSignature)other)._anno.toString());
    }

    public int hashCode() {
	return _anno.toString().hashCode();
    }


    // ACCESSORS

    // return the annotation
    public RMJAnnotation.Method annotation() {
	return _anno;
    }


    // INSTANCE VARIABLES

    // the annotation for this signature
    private RMJAnnotation.Method _anno;

    // the number of unloaded classes & interfaces that must be loaded
    // before this signature becomes reachable
    private int _unloadedCount;
}
