package org.multijava.relaxed.util.classfile;

import java.io.*;
import java.util.*;

import org.multijava.util.classfile.Constants;

/**
 * Parses a Java .class file
 */
public class ClassInfo implements Constants {

    protected String[] constantUTFStrings;

    protected List /* of String */ rmjAttributes = new ArrayList();

    public void process(byte[] bytes)
    {
	try {
	    DataInput in = 
		new DataInputStream(new ByteArrayInputStream(bytes));

	    in.skipBytes(8);  // magic, minorVersion, majorVersion

	    readConstantPool(in);

	    in.skipBytes(6);  // accessFlags, thisClass, superClass

	    int numInterfaces = in.readUnsignedShort();
	    in.skipBytes(numInterfaces * 2);  // the interfaces

	    int numFields = in.readUnsignedShort();
	    for (int i = 0; i < numFields; i += 1) {
		readFieldInfo(in);
	    }

	    int numMethods = in.readUnsignedShort();
	    for (int i = 0; i < numMethods; i += 1) {
		readMethodInfo(in);
	    }

	    readAttributes(in);
	} catch (IOException ex) {
	    // throw new RuntimeException("not expecting an IO exception");
	    RuntimeException rte = new RuntimeException(ex);
	    rte.setStackTrace(ex.getStackTrace());
	    throw rte;
	}
    }

    public List rmjAttributes() { return rmjAttributes; }


    protected void readConstantPool(DataInput in)
	throws IOException 
    {
	int size = in.readUnsignedShort();

	constantUTFStrings = new String[size];

	for (int i = 1; i < size; i++) {
	    byte tag = (byte)in.readUnsignedByte();

	    switch (tag) {
	    case CST_UTF8:
		constantUTFStrings[i] = in.readUTF();
		break;

	    case CST_CLASS:
		processClassReference(in);
		break;

	    case CST_STRING:
		in.skipBytes(2);
		break;

	    case CST_INTEGER:
	    case CST_FLOAT:
	    case CST_FIELD:
	    case CST_METHOD:
	    case CST_INTERFACEMETHOD:
	    case CST_NAMEANDTYPE:
		in.skipBytes(4);
		break;

	    case CST_LONG:
	    case CST_DOUBLE:
		in.skipBytes(8);
		i += 1;		// uses 2 slots
		break;

	    default:
		throw new RuntimeException("Bad constant tag: " + tag);
	    }
	}
    }

    // by default, skip over class references
    protected void processClassReference(DataInput in)
	throws IOException 
    {
	in.skipBytes(2);
    }

    protected void readFieldInfo(DataInput in)
	throws IOException
    {
	in.skipBytes(6); // accessFlags, name, descriptor
	readAttributes(in);
    }

    protected void readMethodInfo(DataInput in)
	throws IOException 
    {
	in.skipBytes(6); // accessFlags, name, descriptor
	readAttributes(in);
    }

    protected void readAttributes(DataInput in)
	throws IOException
    {
	int count = in.readUnsignedShort();
	for (int i = 0; i < count; i += 1) {
	    readAttribute(in);
	}
    }

    protected void readAttribute(DataInput in)
	throws IOException
    {
	int attributeNameIndex = in.readUnsignedShort();
	String attributeName = lookupStringConstant(attributeNameIndex);
	int count = in.readInt();
	if (attributeName.equals("org.multijava.relaxed")) {
	    // we want this one
	    int attributeIndex = in.readUnsignedShort();
	    String attribute = lookupStringConstant(attributeIndex);
	    rmjAttributes.add(attribute);
	} else {
	    in.skipBytes(count);
	}
    }

    protected String lookupStringConstant(int index) {
	String s = constantUTFStrings[index];
	if (s == null) {
	    throw new RuntimeException("expecting the constant at index " + 
				       index + " to be a UTF string");
	}
	return s;
    }

}
