package org.multijava.relaxed.util.classfile;

import java.io.*;
import java.util.*;

/**
 * Parses a Java .class file
 */
public class PreloaderClassInfo extends ClassInfo {

    // in the first pass (process), only remember the class name
    // indices, to account for forward references.  then, when asking
    // for the classNames list for the first time, convert the indices
    // to the resulting class names
    protected List /* of Integer */ classNameIndices = new ArrayList();
    protected List /* of String */ classNames = null;

    protected void processClassReference(DataInput in)
	throws IOException 
    {
	int encodedClassNameIndex = in.readUnsignedShort();
	classNameIndices.add(new Integer(encodedClassNameIndex));
    }

    public List classNames()
    {
	if (classNames != null) {
	    // already computed
	    return classNames;
	}
	// compute it
	classNames = new ArrayList();
	Iterator iter = classNameIndices.iterator();
	while (iter.hasNext()) {
	    Integer index = (Integer) iter.next();
	    String encodedClassName = lookupStringConstant(index.intValue());
	    processEncodedClassName(encodedClassName);
	}
	return classNames;
    }

    protected void processEncodedClassName(String encodedClassName)
    {
	if (encodedClassName.charAt(0) == '[') {
	    // an array; process the element type by skipping over the
	    // first character
	    processEncodedArrayElemType(encodedClassName, 1);
	} else {
	    // should be a plain class name
	    String className = encodedClassName.replace('/', '.');
	    classNames.add(className);
	}
    }

    protected void processEncodedArrayElemType(String encodedElemType, 
					       int start)
    {
	if (encodedElemType.charAt(start) == 'L') {
	    // an encoded class type; pull out the class name
	    // (excluding the leading L and the trailing ;) and
	    // process it
	    int end = encodedElemType.length()-1;
	    if (encodedElemType.charAt(end) != ';') {
		throw new RuntimeException("unexpected array elem type: " + 
					   encodedElemType.substring(start));
	    }
	    String className = encodedElemType.substring(start+1, end);
	    processEncodedClassName(className);
	} else if (encodedElemType.charAt(start) == '[') {
	    // yet another array; visit the element type by skipping
	    // over the first character
	    processEncodedArrayElemType(encodedElemType, start+1);
	} else {
	    // some other array elem type, e.g. an int; ignore it.
	}
    }
}
