package org.multijava.samples.basic;

/**
 * This class uses value dispatching to implement a simple state machine.
 */
public class EndsWithA {
    public static void main (String[] args) {
	if (args.length != 1) {
	    System.err.println(USE);
	} else {
	    new EndsWithA(args[0]).run();
	}
    }
    

    /**
     * Instantiates a state machine for processing the given input.
     */
    EndsWithA(String input) {
	this.input = input;
	this.inputChars = input.toCharArray();
    }

    /**
     * Runs this state machine on the input and prints the result to
     * System.out.
     */
    final void run() {
	int state = startState();
	for (int i = 0; i < inputChars.length; i++) {
	    state = transition(state, inputChars[i]);
	}
	if (accepts(state)) {
	    System.out.println("accepts '" + input + "'");
	} else {
	    System.out.println("rejects '" + input + "'");
	}
    }


    /**
     * Returns the start state of this state machine.  May be
     * overridden in subclass to change start state.
     */
    int startState() {
	return 0;
    }

    /**
     * Indicates whether the given state is an accepting state.
     */
    boolean accepts(int finalState) {
	return false;
    }

    boolean accepts(int@@1 finalState) {
	return true;
    }


    /**
     * Transitions the state machine from the given state
     * and character.  Returns the next state of the state machine.
     */
    int transition(int st, char ch) {
	throw new IllegalArgumentException("Unexpected state, " + st + 
					   ", or character, " + ch + ".");
    }

    int transition(int@@0 st, char@@'a' ch) { return 1; }
    int transition(int@@0 st, char@@'b' ch) { return 0; }
    int transition(int@@1 st, char@@'a' ch) { return 1; }
    int transition(int@@1 st, char@@'b' ch) { return 0; }

    static final String USE = "Expected a single string argument";
    private final String input;
    private final char[] inputChars;
}
