/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

import cobalt.gen.sre.*;
import java.io.PrintWriter;


public class Association extends FieldDecl {

    // -------------------------------------------------------------
    // CONSTRUCTORS AND FACTORIES
    // -------------------------------------------------------------
    
    public Association( String preText, String midText, String postText, 
			StereotypeAnnotation stereotype, Modifiers modifiers, 
			String type, String name, 
			SREAnnotation[] annotations ) 
    {
	super( preText, midText, postText, stereotype, modifiers, type, name );
	this.annotations = 
	    annotations == null ? new SREAnnotation[0] : annotations;
    }

    /**
     * Instantiates a new association VST node for the given UML
     * association.  */
    static Association create( CAssociationAccessor assoc ) {
        // Cobalt convention: start -----<> end
	String preText = "\n";
	Modifiers mods = new Modifiers();
        mods.setPublicMod(true);
	String type = assoc.start().qualifiedName();
        // !FIXME! generate a unique field name
	String name = "lnk" + assoc.start().name(); 
	String midText = INDENT + mods + " " + type + " " + name;
	String postText = ";\n";
        // !FIXME! generate SRE tags for association
	return new Association( preText, midText, postText, null, 
                                mods, type, name, null );
    }
    
    // -------------------------------------------------------------
    // PRETTY-PRINTING
    // -------------------------------------------------------------

    /**
     * Appends to <code>wrtr</code> the source code represented by the
     * annotation child nodes of this, excluding the common stereotype
     * annotation.  The default implementation here does nothing, some
     * subclasses will override this implementation. */
    void prettyPrintAnnotations(PrintWriter wrtr) {
	for (int i = 0; i < annotations.length; i++) {
	    annotations[i].prettyPrint(wrtr);
	}
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Appends to <code>accum</code> an abstract representation of the 
     * annotation child nodes of this, excluding the common stereotype
     * annotation.  The default implementation here does nothing, some
     * subclasses will override this implementation. */
    protected void buildAnnotationReps(StringBuffer accum) {
	for (int i = 0; i < annotations.length; i++) {
	    annotations[i].abstractRepBuilder(accum);
	}
    }

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Asc"; }

    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The annotations associated with the association.
     *
     * <pre><jml>
     * invariant annotations != null;
     * </jml></pre>
     *
     */
    private SREAnnotation[] annotations;
}
