/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

import java.util.List;

/**
 * This class represents a class declaration in a VST.  */
public class ClassDecl extends TypeDecl {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Generate a new class declaration node for a VST.
     * @param preText source code text before any SREAnnotations
     * @param midText source code text between any SREAnnotations and the
     *			first curly brace of the body
     * @param postText source code text following the last member
     * @param stereotype a <code>StereotypeAnnotation</code> value, or null
     * @param modifiers the modifiers of this
     * @param name the short name of this class
     * @param invariants the invariants of this class
     * @param members the members of this class
     * @param interfaces the interfaces implemented by this class
     * @param superType the superclass of this class, or null
     */
    public ClassDecl( String preText, String midText, String postText, 
		      StereotypeAnnotation stereotype, Modifiers modifiers, 
		      String name, TextValuedAnnotation[] invariants, 
		      List members, String[] interfaces, 
		      String superType ) {
	super( preText, midText, postText, stereotype, modifiers, name,
	       invariants, members, interfaces );
	if (superType == null) {
	    this.superType = DEFAULT_SUPER;
	} else {
	    this.superType = superType;
	}
	this.kind = CLASS_DECL;
    }

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Appends to <code>accum</code> an abstract representation of the 
     * attributes of this. */
    protected void buildAttributeReps(StringBuffer accum) {
	// this comparison intentionally uses '!=' instead of
	// !...equals(...) since if "extends java.lang.Object" was
	// present in source we want it in the abstract representation
	// as well
	if (superType != DEFAULT_SUPER) {
	    accum.append("<:"+superType);
	}
	super.buildAttributeReps(accum);
    }

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Cls"; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The type of the supertype of this.
     */
    private String superType;

    private static final String DEFAULT_SUPER = "java.lang.Object";
}
