/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

/**
 * This class represents a constructor in a VST.
 *
 * @author Curtis Clifton
 * @version $Id: ConstructorDecl.java,v 1.2 2002/06/25 17:08:29 cclifton Exp $
 */
public class ConstructorDecl extends MethodDecl {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Construct a VST node representing a constructor declaration.
     *
     * @param preText text preceding any SRE-specific javadoc annotations
     * @param midText text between any SRE-specific and the first left paren
     * @param postText text from the end of the last parameter name to the 
     *			end of the line following the closing right brace
     * @param stereotype a stereotype for this, or null
     * @param modifiers the constructor's modifiers
     * @param name 	the constructor's identifier
     * @param inputAnnotation an input annotation for this, or null
     * @param outputAnnotation an output annotation for this, or null
     * @param params an array of the parameters of this
     * @param thrown an array of the names of the exceptions declared to be 
     *			thrown by this
     */
    public ConstructorDecl( String preText, String midText, String postText, 
			    StereotypeAnnotation stereotype, 
			    Modifiers modifiers, String name, 
			    TextValuedAnnotation inputAnnotation,
			    TextValuedAnnotation outputAnnotation, 
			    Parameter[] params, String[] thrown ) 
    {
	super( preText, midText, postText, stereotype, modifiers, name,
	       inputAnnotation, outputAnnotation, params, null, thrown );
    }
    //  !FIXME! need to override methods that rely on return type

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    public boolean equals(Object other) {
	return other instanceof ConstructorDecl &&
	    super.equals(other);
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Cst"; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

}
