/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

import java.io.PrintWriter;

public class FieldDecl extends Member {

    // -------------------------------------------------------------
    // CONSTRUCTORS AND FACTORIES
    // -------------------------------------------------------------
    
    /**
     * Creates a new field declaration node for a VST.
     *
     * @param preText source code text before any SREAnnotations
     * @param midText source code text between any SREAnnotations and the
     *			first curly brace of the body
     * @param postText source code text following the last member
     * @param stereotype a <code>StereotypeAnnotation</code> value, or null
     * @param modifiers the modifiers of this
     * @param type a String giving the type of this field
     * @param name the short name of this class
     */
    public FieldDecl( String preText, String midText, String postText, 
		      StereotypeAnnotation stereotype, Modifiers modifiers, 
		      String type, String name ) 
    {
	super( preText, midText, postText, stereotype, modifiers, name );
	this.type = type;
	this.otherVars = new FieldDecl[0];
	this.kind = FIELD_DECL;
    }


    /**
     * Sets the otherVars field of this.  Only called by the parser.
     */
    void setOtherVars( FieldDecl[] otherVars ) {
	this.otherVars = otherVars == null ? new FieldDecl[0]: otherVars;
    }

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------
    
    /**
     * Returns the type of this member.  */
    public String type() {
	return type;
    }

    public boolean equals(Object other) {
	return other instanceof FieldDecl &&
	    name().equals(((FieldDecl)other).name());
    }

    public int hashCode() {
	return name().hashCode();
    }

    // -------------------------------------------------------------
    // PRETTY-PRINTING
    // -------------------------------------------------------------

    /**
     * Appends to <code>wrtr</code> the source code represented by the
     * non-annotation child nodes of this. */
    void prettyPrintChildren(PrintWriter wrtr) {
	for (int i = 0; i < otherVars.length; i++) {
	    otherVars[i].prettyPrint(wrtr);
	}
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Appends to <code>accum</code> an abstract representation of the
     * non-annotation child nodes of this. */
    protected void buildChildrenReps(StringBuffer accum) {
	for (int i = 0; i < otherVars.length; i++) {
	    otherVars[i].abstractRepBuilder(accum);
	}
    }

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Fld"; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The type of this field.
     */
    private String type;


    /**
     * Declarations of other fields in the same variable declarator as
     * this.  That is, in a variable declarator like <code>int x, y,
     * z;</code>, <code>name.equals("x") &&
     * otherVars[0].name.equals("y") && otherVars[1].name.equals("z")
     * */
    private FieldDecl[] otherVars;
}
