/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;


/**
 * This class represents an import clause in a VST.
 *
 * @author Curtis Clifton
 * @version $Id: ImportClause.java,v 1.2 2002/06/25 17:08:29 cclifton Exp $
 */
public class ImportClause extends SimpleNode {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Creates a new import clause VST node.
     *
     * @param text the verbatim text of this import clause, beginning with
     * 			the start of the line following the preceding 
     *			package or import clause and continuing through the
     *			newline following the semi-colon at the end of this
     *			import clause. 
     * @param name the qualified package name in the import clause
     */
    public ImportClause( String text, String name ) {
	super(text);
	this.name = name;
	isOnDemand = name.endsWith("*");
    }

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    public boolean equals(Object other) {
	return other instanceof ImportClause &&
	    name.equals( ((ImportClause)other).name );
    }

    public int hashCode() {
	return name.hashCode();
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Imp:" + name; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The identifier imported by this.
     */
    private String name;

    /**
     * Indicates whether this is an import-on-demand (i.e. a .*)
     * clause.  */
    private boolean isOnDemand;
}
