/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

import java.io.PrintWriter;

/**
 * This class represents an initializer block in a VST.
 * @author Curtis Clifton
 * @version $Id: InitializerDecl.java,v 1.2 2002/06/25 17:08:29 cclifton Exp $
 */
public class InitializerDecl extends Member {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Construct a VST node representing an initializer.
     * @param preText source code text before any SREAnnotations
     * @param midText source code text between any SREAnnotations and the
     *			newline following the last curly brace of the body
     * @param stereotype a <code>StereotypeAnnotation</code> value, or null
     * @param modifiers the modifiers of this
     */
    public InitializerDecl( String preText, String midText,
			    StereotypeAnnotation stereotype, 
			    Modifiers modifiers ) 
    {
	// postText is always empty for initializers
	super(preText, midText, "", stereotype, modifiers, NAME);
    }

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------


    /**
     * Two initializer declarations only compare equal if they are the
     * same object.
     * @see Node#equals
     */
    public boolean equals(Object other) {
	return this == other;
    }

    public int hashCode() {
	return System.identityHashCode(this);
    }

    // -------------------------------------------------------------
    // PRETTY-PRINTING
    // -------------------------------------------------------------

    /**
     * Appends to <code>wrtr</code> the source code represented by the
     * non-annotation child nodes of this. */
    void prettyPrintChildren(PrintWriter wrtr) { 
	// nothing to do
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Appends to <code>accum</code> an abstract representation of the
     * non-annotation child nodes of this. */
    protected void buildChildrenReps(StringBuffer accum) {
	// nothing to do
    }

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Izr"; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // PRIVILEGED DATA
    // -------------------------------------------------------------

    public static final String NAME = "$Init";

}
