/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

import java.util.List;

/**
 * This class represents an interface declaration in a VST.  */
public class InterfaceDecl extends TypeDecl {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Generate a new interface declaration node for a VST.
     * @param preText source code text before any SREAnnotations
     * @param midText source code text between any SREAnnotations and the
     *			first curly brace of the body
     * @param postText source code text following the last member
     * @param stereotype a <code>StereotypeAnnotation</code> value, or null
     * @param modifiers the modifiers of this
     * @param name the short name of this class
     * @param invariants the invariants of this class
     * @param members the members of this class
     * @param interfaces the interfaces implemented by this class
     */
    public InterfaceDecl( String preText, String midText, String postText, 
			  StereotypeAnnotation stereotype, 
			  Modifiers modifiers, 
			  String name, TextValuedAnnotation[] invariants, 
			  List members, String[] interfaces ) 
    {
	super( preText, midText, postText, stereotype, modifiers, name,
	       invariants, members, interfaces );
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Itr"; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

}
