header { 
/*
 * Copyright (C) 2002 Iowa State University
 *
 * $Id: Javadoc.g,v 1.1 2002/03/26 00:31:33 cclifton Exp $
 */

/*
 * These grammars handle the parsing and lexing of javadoc
 * documentation comments.  */

	// common parser/lexer header
	package org.multijava.samples.cmb.parser; 
	
}

{
	// Parser class preamble
	import java.util.ArrayList;
}

class JavadocParser extends Parser;
options {
	k = 2;
	importVocab = Shared;
	codeGenMakeSwitchThreshold = 2;  // Some optimizations
	codeGenBitsetTestThreshold = 3;
	defaultErrorHandler = false;     // Don't generate parser error handlers
	buildAST = false;
}

{
	// custom code for Parser class
	public JavadocParser( TokenStream tokStream, TokenManifold tokManifold ) {
		this(tokStream);
		this.tokManifold = tokManifold;
	}

	/**
	 * The token manifold feeding this parser.
	 */
	private TokenManifold tokManifold;

}

// accumulate the text in the body of the comment up to the first
// SRE-specific annotation, then the SRE-specific annotations, then
// the text up to the end of the comment
docComment []
returns [TextAndAnnotations self = new TextAndAnnotations()]
	:
		// First capture descriptive paragraph
		( 
			options {greedy = true;} // if no notes then put all in preText
		:
			r:REST_OF_LINE 
			{ self.addVerbatimPreText(tokManifold.textBeforeWithNL(r)); } 
		)*

		// Next capture all the annotations
		annotations[self]

//  		( 
//  			r2:REST_OF_LINE 
//  			{ midText.append(tokManifold.textBeforeWithNL(r2)); } 
//  		)*

		// a false EOF is added to the end of the stream by TokenManifold
		EOF		 
//  		{
//  			self = new TextAndAnnotations( preText.toString(), notes,
//  				midText.toString() );
//  		}
	;


annotations [TextAndAnnotations tAndA]
		//  !FIXME!2/26 need to handle tags other than our own
{
	Token endToken = null;
}
	:
		( id:IDENT { endToken=id; tAndA.beginTaggedRegion(id.getText());}
			(
				r:REST_OF_LINE 
				{ endToken=r; tAndA.addTaggedText(r.getText()); } 
			)* 
			{ tAndA.endTaggedRegion(tokManifold.textBeforeWithNL(endToken)); }
		)*
	;

{
	// class preamble for the lexer
	import antlr.LexerSharedInputState;
}

class JavadocLexer extends Lexer;

options {
    k = 3;
	importVocab = JavadocParser;
	exportVocab = Javadoc;
    testLiterals = false;     // don't automatically test for literals
	charVocabulary = '\u0003'..'\uFFFF';
	// without inlining some bitset tests, couldn't do unicode;
	codeGenBitsetTestThreshold = 20;
}

{
	/**
	 * This lexer assumes the shared input stream has been advanced
	 * past the initial <code>SLASH STAR STAR (STAR)*</code> before
	 * the token manifold calls its nextToken method. */
	public JavadocLexer( TokenManifold tokManifold ) {
		this( tokManifold.sharedInputState() );
		this.tokManifold = tokManifold;
	}

	/**
	 * Used to switch lexers.  Aliases should be established in all
	 * lexers that share the same input stream. */
	private TokenManifold tokManifold;

	/**
	 * A key for this lexer used to identify an instance of it in the
	 * token manifold.  */
	public static final String KEY = "javadocLex";
}

START_LINE
    : 
		NEWLINE
		// consuming all leading spaces, and if no '*' on line consume
		// up to first non-NEWLINE whitespace
		( options { greedy=true; } : NON_NL_WS )*
		( 
			{LA(2) != '/'}? '*'
		)*
		(NON_NL_WS)*
		{$setType(NON_SEMANTIC);}
    ;

// will be ambiguous if called like ( NEWLINE | ... ) * because '\r'
// '\n' can be matched in one alternative or by matching '\r' in one
// iteration and '\n' in another.
protected
NEWLINE
	:
		(
			options {
				generateAmbigWarnings=false;
			}
		:
			'\r' '\n'
		|	'\r'
		|	'\n'
		)
		{ 
			newline();
		}
	;

protected
NON_NL_WS
	:
		( ' ' | '\t' | '\f' )
	;

// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
IDENT
	options {testLiterals=true;}
	:	'@' ('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$')*
	;

REST_OF_LINE
	:
        ~('*'|'\r'|'\n'|'@')
        (
			{LA(2) != '/'}?
			'*'
		|
            ~('*'|'\r'|'\n') 
		)*
	;

/** This rule simply prevents JAVADOC_CLOSE from being
 *  called for every '*' in a comment.  Calling JAVADOC_CLOSE
 *  will fail for simple '*' and cause an exception, which
 *  is slow.  In other words, the grammar will work without
 *  this rule, but is slower.
 */
//  STAR
//  	:	'*' {$setType(NON_SEMANTIC);}
//    	;

JAVADOC_CLOSE
	:	
		"*/" 
		{ 
			tokManifold.pop(); 
			$setType(NON_SEMANTIC);
		}
	;

