/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

/**
 * This class represents a package clause in the VST.
 *
 * @author Curtis Clifton
 * @version $Id: PackageClause.java,v 1.2 2002/06/25 17:08:29 cclifton Exp $
 */
public class PackageClause extends SimpleNode {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Construct a VST node representing a package clause.
     * @param text the verbatim text of this package clause, beginning with
     * 			the start of the file and continuing through the
     *			 newline following the semi-colon at the end of the
     *			 package clause. 
     * @param name the qualified package name in the package clause
     */
    public PackageClause( String text, String name ) {
	super(text);
	this.name = name;
	this.kind = PACKAGE_CLAUSE;
    }

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    /**
     * Returns the name of this package clause as a series of one or
     * more Java identifiers separated by "dots".  */
    public String name() {
	return name;
    }

    /**
     * Two package clauses always compare as equal since only one
     * package clause may be given in the same VST.  
     * @see Node#equals
     */
    public boolean equals(Object other) {
	return other instanceof PackageClause;
    }

    public int hashCode() {
	return 2;		// any constant will do, see equals()
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Returns an identifier for this node type.  */
    protected String codon() { return "Pac"; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The identifier of the package named in this clause.  */
    private String name;
}
