/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;


/**
 * This class represents a parameter declaration in a VST.
 *
 * @author Curtis Clifton
 * @version $Id: Parameter.java,v 1.2 2002/06/25 17:08:29 cclifton Exp $
 */
public class Parameter extends SimpleNode {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Constructs a VST node representing a parameter as parsed from
     * source code. 
     *
     * @param text	the verbatim text of this parameter, starting with, 
     *			but not including, the preceding left paren or 
     *			previous parameter's identifier and ending with 
     *			this parameter's identifier
     * @param name	the identifier of the parameter 
     * @param type	the type of this parameter
     */
    public Parameter( String text, String name, String type ) 
    {
	super(text);
	this.name = name;
	this.type = type;
    }

    // -------------------------------------------------------------
    // FACTORY METHODS
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    public boolean equals(Object other) {
	return other instanceof Parameter &&
	    type.equals( ((Parameter)other).type );
    }

    public int hashCode() {
	return type.hashCode();
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Returns an identifier for this node type. */
    protected String codon() { return "Par"; }

    // -------------------------------------------------------------
    // REVERSE ENGINEERING
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The kind of this parameter (input, output, or in/out).
     */
    private int kind;		//  !FIXME! should this be OO?

    /**
     * The identifier of this parameter.
     */
    private String name;

    /**
     * The type of this parameter.
     */
    private String type;

    /**
     * The default value of this parameter in the model.
     */
    private String defaultValue;
}
