/*
 * Copyright (c) 2002, Iowa State University
 *
 * $Id: SimpleNode.java,v 1.1 2002/03/26 00:31:33 cclifton Exp $
 */

package org.multijava.samples.cmb.parser;

import java.io.PrintWriter;


/**
 * This class is the abstract superclass of all VST leaf nodes.
 *
 * @author Curtis Clifton
 * @version $Id: SimpleNode.java,v 1.1 2002/03/26 00:31:33 cclifton Exp $
 */
public abstract class SimpleNode extends Node {

    // -------------------------------------------------------------
    // CONSTRUCTORS
    // -------------------------------------------------------------

    /**
     * Construct a leaf node in the VST with the given verbatim text
     * representation.  */
    public SimpleNode( String text ) {
	this.text = text;
    }

    // -------------------------------------------------------------
    // PRETTY-PRINTING
    // -------------------------------------------------------------

    /**
     * Appends to <code>wrtr</code> the source code represented by the
     * VST anchored at this.  */
    public void prettyPrint(PrintWriter wrtr) {
	wrtr.print(text);
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    /**
     * Appends an abstract representation of this node onto the given
     * accumulator buffer.
     *
     * @see #abstractRep()
     */
    protected void abstractRepBuilder(StringBuffer accum) {
	accum.append('[');
	accum.append(codon());
	accum.append(']');
    }
  
    /**
     * Returns an identifier for this node type.  */
    protected abstract String codon();
    
    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The actual source code text up to and including this node.  */
    private String text;
}
