/*
 * Copyright (c) 2002, Iowa State University
 */

package org.multijava.samples.cmb.parser;

public class StereotypeAnnotation extends SREAnnotation {

    /**
     * Generates a <code>@stereotype</code> javadoc annotation node for a VST.
     * @param text the verbatim text of this annotation starting with the 
     *			beginning of the line containing the annotation and 
     *			continuing to the trailing newline
     * @param stereotype the text following the <code>@stereotype</code> 
     *			keyword up to but not including the trailing newline
     */
    public StereotypeAnnotation( String text, String stereotype ) {
	super( text, STEREOTYPE );
	this.stereotype = stereotype;
    }

    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------

    /**
     * Returns true if other is a StereotypeAnnotation, since a single
     * declaration can only have one stereotype.
     * @see Node#equals
     */
    public boolean equals(Object other) {
	return other instanceof StereotypeAnnotation;
    }

    public int hashCode() {
	return stereotype.hashCode();
    }

    // -------------------------------------------------------------
    // ABSTRACT REPRESENTATION OUTPUT
    // -------------------------------------------------------------

    // -------------------------------------------------------------
    // PRIVILEDGED DATA MEMBERS
    // -------------------------------------------------------------

    /**
     * The text following the <code>@stereotype</code> keyword up to
     * but not including the trailing newline.  */
    private String stereotype;
}
