header {
/**
 * Copyright (C) 2002, Iowa State University
 * 
 * Based loosely on: 
 * 
 * Java 1.3 Recognizer
 *
 * Run 'java Main [-showtree] directory-full-of-java-files'
 *
 * [The -showtree option pops up a Swing frame that shows
 *  the AST constructed from the parser.]
 * 
 * Run 'java Main <directory full of java files>'
 *
 * Contributing authors:
 *		John Mitchell		johnm@non.net
 *		Terence Parr		parrt@magelang.com
 *		John Lilley			jlilley@empathy.com
 *		Scott Stanchfield	thetick@magelang.com
 *		Markus Mohnen       mohnen@informatik.rwth-aachen.de
 *      Peter Williams      pete.williams@sun.com
 *      Allan Jacobs        Allan.Jacobs@eng.sun.com
 *      Steve Messick       messick@redhills.com
 *
 * Version 1.00 December 9, 1997 -- initial release
 * Version 1.01 December 10, 1997
 *		fixed bug in octal def (0..7 not 0..8)
 * Version 1.10 August 1998 (parrt)
 *		added tree construction
 *		fixed definition of WS,comments for mac,pc,unix newlines
 *		added unary plus
 * Version 1.11 (Nov 20, 1998)
 *		Added "shutup" option to turn off last ambig warning.
 *		Fixed inner class def to allow named class defs as statements
 *		synchronized requires compound not simple statement
 *		add [] after builtInType DOT class in primaryExpression
 *		"const" is reserved but not valid..removed from modifiers
 * Version 1.12 (Feb 2, 1999)
 *		Changed LITERAL_xxx to xxx in tree grammar.
 *		Updated java.g to use tokens {...} now for 2.6.0 (new feature).
 *
 * Version 1.13 (Apr 23, 1999)
 *		Didn't have (stat)? for else clause in tree parser.
 *		Didn't gen ASTs for interface extends.  Updated tree parser too.
 *		Updated to 2.6.0.
 * Version 1.14 (Jun 20, 1999)
 *		Allowed final/abstract on local classes.
 *		Removed local interfaces from methods
 *		Put instanceof precedence where it belongs...in relationalExpr
 *			It also had expr not type as arg; fixed it.
 *		Missing ! on SEMI in classBlock
 *		fixed: (expr) + "string" was parsed incorrectly (+ as unary plus).
 *		fixed: didn't like Object[].class in parser or tree parser
 * Version 1.15 (Jun 26, 1999)
 *		Screwed up rule with instanceof in it. :(  Fixed.
 *		Tree parser didn't like (expr).something; fixed.
 *		Allowed multiple inheritance in tree grammar. oops.
 * Version 1.16 (August 22, 1999)
 *		Extending an interface built a wacky tree: had extra EXTENDS.
 *		Tree grammar didn't allow multiple superinterfaces.
 *		Tree grammar didn't allow empty var initializer: {}
 * Version 1.17 (October 12, 1999)
 *		ESC lexer rule allowed 399 max not 377 max.
 *		java.tree.g didn't handle the expression of synchronized
 *		statements.
 * Version 1.18 (August 12, 2001)
 *      Terence updated to Java 2 Version 1.3 by observing/combining work of
 *      Allan Jacobs and Steve Messick.  Handles 1.3 src.
 *		Summary:
 *		o  primary didn't include boolean.class kind of thing
 *      o  constructor calls parsed explicitly now:
 * 		   see explicitConstructorInvocation
 *		o  add strictfp modifier
 *      o  missing objBlock after new expression in tree grammar
 *		o  merged local class definition alternatives, moved after declaration
 *		o  fixed problem with ClassName.super.field
 *      o  reordered some alternatives to make things more efficient
 *		o  long and double constants were not differentiated from int/float
 *		o  whitespace rule was inefficient: matched only one char
 *		o  add an examples directory with some nasty 1.3 cases
 *		o  made Main.java use buffered IO and a Reader for Unicode support
 *		o  supports UNICODE?
 *		   Using Unicode charVocabulay makes code file big, but only
 *		   in the bitsets at the end. I need to make ANTLR generate
 *		   unicode bitsets more efficiently.
 *
 * class Test {
 *   public static void main( String args[] ) {
 *     if (boolean.class.equals(boolean.class)) {
 *       System.out.println("works");
 *     }
 *   }
 * }
 *
 * This grammar (was) in the PUBLIC DOMAIN.  This version is not in
 * the public domain.
 */

	// common parser/lexer header
	package org.multijava.samples.cmb.parser;

}

{
	// Parser class preamble
	import java.io.*;
	import java.util.ArrayList;
	import java.util.Date;
}

class VSTBuilder extends Parser;
options {
	k = 2;                           // two token lookahead
	importVocab = Shared;
	codeGenMakeSwitchThreshold = 2;  // Some optimizations
	codeGenBitsetTestThreshold = 3;
	defaultErrorHandler = false;     // Don't generate parser error handlers
	buildAST = false;
}

tokens {
	BLOCK; MODIFIERS; OBJBLOCK; SLIST; CTOR_DEF; METHOD_DEF; VARIABLE_DEF; 
	INSTANCE_INIT; STATIC_INIT; TYPE; CLASS_DEF; INTERFACE_DEF; 
	PACKAGE_DEF; ARRAY_DECLARATOR; EXTENDS_CLAUSE; IMPLEMENTS_CLAUSE;
	PARAMETERS; PARAMETER_DEF; LABELED_STAT; TYPECAST; INDEX_OP; 
	POST_INC; POST_DEC; METHOD_CALL; EXPR; ARRAY_INIT; 
	IMPORT; UNARY_MINUS; UNARY_PLUS; CASE_GROUP; ELIST; FOR_INIT; 
	FOR_CONDITION; FOR_ITERATOR; EMPTY_STAT; FINAL="final"; 
	ABSTRACT="abstract"; STRICTFP="strictfp"; SUPER_CTOR_CALL; CTOR_CALL;
}
	
{
	// custom code for Parser class
	public static CompilationUnit parse(File file) 
	throws FileNotFoundException, SyntaxErrorException
	{
		BufferedReader reader = new BufferedReader(new FileReader(file));
		CompilationUnit result = parse(file.toString(), reader);
		result.setFile(file);
		result.setTimestamp(file.lastModified());
		return result;
	}

	public static CompilationUnit parse(Reader reader) 
	throws SyntaxErrorException
	{
		return parse("unknown file", reader);
	}

	public static CompilationUnit parse(String filename, Reader reader) 
	throws SyntaxErrorException
	{
		if (reader == null) {
			throw new NullPointerException();
		}
		VSTBuilder parser = null;
		try {
			TokenManifold tokManifold = new TokenManifold(reader,filename);
			JavaLexer jLex = new JavaLexer(tokManifold);
			tokManifold.addInputStream(jLex, JavaLexer.KEY);
			tokManifold.selectInitial(JavaLexer.KEY);
			JavadocLexer javadocLex = new JavadocLexer(tokManifold);
			tokManifold.addInputStream(javadocLex, JavadocLexer.KEY);
			parser = new VSTBuilder(
				tokManifold.initialOutputStream(), tokManifold );
			
			return parser.compilationUnit();
		} catch(RecognitionException re) {
			throw parser.buildSyntaxErrorException(re);
		} catch(TokenStreamException tse) {
			throw parser.buildSyntaxErrorException(tse);
		} catch(TokenManifold.KeyException ke) { 
			//  !FIXME! better msg
			throw new RuntimeException( "there is a bug in this method" );
		} catch(TokenManifold.ConfigurationException ce) { 
			//  !FIXME! better msg
			throw new RuntimeException( "there is a bug in this method" );
		}
	}

	public VSTBuilder( TokenStream tokStream, TokenManifold tokManifold ) {
		this(tokStream);
		this.tokManifold = tokManifold;
	}

	private SyntaxErrorException buildSyntaxErrorException( Exception e ) {
		String msg = "Error near " + 
		tokManifold.filename() + ":" + tokManifold.line() + ":" +
		tokManifold.column() + ":" + e.getMessage();
		return new SyntaxErrorException(msg);
	}

	/**
	 * A helper method for discarding javadoc tokens when syntactic
	 * analysis determines that they aren't needed.  */
	private void discardJavadocBefore( Token tok ) 
	throws TokenStreamException {
		try {
			tokManifold.discardTokensBefore( JavadocLexer.KEY, tok );
		} catch( TokenManifold.KeyException e ) {
			throw new TokenStreamException(e.getMessage());
		}
	}

	/**
	 * A helper method for parsing javadoc tokens when syntactic
	 * analysis determines that they are needed.  
	 *
	 * <pre><jml>
	 * requires tokWrap != null && tokWrap.tok != null;
	 * </jml></pre>
	 */
	private TextAndAnnotations parseJavadocsBefore( TokenWrapper tokWrap ) 
	throws TokenStreamException {
		return parseJavadocsBefore( tokWrap.tok );
	}

	/**
	 * A helper method for parsing javadoc tokens when syntactic
	 * analysis determines that they are needed.  
	 *
	 * <pre><jml>
	 * requires tok != null;
	 * </jml></pre>
	 */
	private TextAndAnnotations parseJavadocsBefore( Token tok ) 
	throws TokenStreamException {
		try {
			TokenStream strm = tokManifold.streamForBefore( JavadocLexer.KEY, 
				tok );
			JavadocParser prsr = new JavadocParser( strm, tokManifold );
			return prsr.docComment();
		} catch( TokenManifold.KeyException e ) {
			throw new TokenStreamException(e.getMessage());
		} catch( antlr.RecognitionException e ) {
			throw new TokenStreamException(e.getMessage());
		}
	}

	/**
	 * A simple nested class used to track the first token processed
	 * in a declaration that begins with optional tokens.  */
	static class TokenWrapper {
		TokenWrapper() {
			this.tok = null;
		}

		TokenWrapper( Token tok ) {
			this.tok = tok;
		}

		void wrapFirst( Token tok ) {
			if (this.tok == null) {
				this.tok = tok;
			}
		}

		Token tok;
	}

	/**
	 * A simple nested class used to return a triple of results from
	 * the classBlock production.  */
	static class ClassBlock {
		ClassBlock( String midText, String postText, ArrayList members ) {
			this.midText = midText;
			this.postText = postText;
			this.members = members;
		}
		String midText;
		String postText;
		ArrayList members;
	}

	/**
	 * A simple nested class used to return results from the ctorHead
	 * production.  */
	static class CtorHead {
		CtorHead( String name, String midText,
			TextAndAnnotations tAndA, Parameter[] params, String[] thrown ) {
			this.name = name;
			this.midText = midText;
			this.tAndA = tAndA;
			this.params = params;
			this.thrown = thrown;
		}
		String name;
		String midText;
		TextAndAnnotations tAndA;
		Parameter[] params;
		String[] thrown;
	}

	static public class SyntaxErrorException extends Exception {
		public SyntaxErrorException( String msg ) {
			super(msg);
		}
	}

	/**
	 * The token manifold feeding this parser.
	 */
	private TokenManifold tokManifold;
}

// Compilation Unit: In Java, this is a single file.  This is the start
//   rule for this parser
compilationUnit []
returns [CompilationUnit self = null]
{
	PackageClause packClause = null;
	ImportClause[] imps = null;
	TypeDecl typ = null;
	ArrayList typeAccum = new ArrayList();
	String post;
}
	:	// A compilation unit starts with an optional package definition
		(	packClause = packageDefinition[]
		|	/* nothing */
		)

		// Next we have a series of zero or more import statements
		imps = importDefinitions[]

		// Wrapping things up with any number of class or interface
		//    definitions (also matches stray semi-colons, returning null
		( typ = typeDefinition[] { if (typ!=null) typeAccum.add(typ); } )*

		e:EOF
		{
			post = tokManifold.textBefore(e);
			TypeDecl[] types = new TypeDecl[ typeAccum.size() ];
			typeAccum.toArray(types);
			self = new CompilationUnit( packClause, imps, types, post);
		}
	;


// Package statement: "package" followed by an identifier.
packageDefinition []
returns [PackageClause self = null]
options {defaultErrorHandler = true;} // let ANTLR handle errors
{
	String name;
}
	:	"package" name = identifier[] s:SEMI
		{
			discardJavadocBefore(s);
			self = new PackageClause( tokManifold.textBeforeWithNL(s), name );
		}
	;


// Import statement: import followed by a package or class name
importDefinitions []
returns [ImportClause[] self = null]
{
	ArrayList impList = new ArrayList();
	ImportClause ic = null;
}
	:
		( ic = importDefinition[] { impList.add(ic); } )*
		{
			self = new ImportClause[impList.size()];
			impList.toArray(self);
		}
	;

importDefinition []
returns [ImportClause self = null]
options {defaultErrorHandler = true;}
{
	String name;
}
	:	"import" name = identifierStar[] s:SEMI
		{
			discardJavadocBefore(s);
			self = new ImportClause( tokManifold.textBeforeWithNL(s), name );
		}
	;

// A type definition in a file is either a class or interface definition.
typeDefinition []
returns [TypeDecl self = null]
	options {defaultErrorHandler = true;}
{
	Modifiers mods;
	Token startToken = LT(1);
}
	:	
		mods = modifiers[]
		( self = classDefinition[true,mods,startToken]
		| self = interfaceDefinition[true,mods,startToken]
		)
	|	
		// we leave stray semi-colons in the text buffer and return null
		s:SEMI
		{ discardJavadocBefore(s); }
	;

/** A declaration is the creation of a reference or primitive-type variable
 *  Create a separate Type/Var tree for each var in the var list.
 */
declaration
{ 
	Modifiers mods; 
	String type;
	FieldDecl discard;
}
	:	mods=modifiers[] type=typeSpec[] 
		discard=variableDefinitions[false,mods,type,null]
	;

// A type specification is a type name with possible brackets
// afterwards (which would make it an array type).
typeSpec []
returns [String self = null]
	: self = classTypeSpec[]
	| self = builtInTypeSpec[]
	;

// A class type specification is a class type with possible brackets afterwards
//   (which would make it an array type).
classTypeSpec []
returns [String self = null]
{
	String id;
	StringBuffer sb = new StringBuffer();
}
	:	id=identifier[] { sb.append(id); } 
		(LBRACK RBRACK { sb.append("[]"); } )*
		{ self = sb.toString(); }
	;

// A builtin type specification is a builtin type with possible brackets
// afterwards (which would make it an array type).
builtInTypeSpec[]
returns [String self = null]
{
	String id;
	StringBuffer sb = new StringBuffer();
}
	:	id=builtInType[] { sb.append(id); } 
		(LBRACK RBRACK { sb.append("[]"); } )*
			{ self = sb.toString(); }
	;
			
// A type name. which is either a (possibly qualified) class name or
//   a primitive (builtin) type
type
{
	Object discard;
}
	:	discard=identifier[]
	|	discard=builtInType[]
	;

// The primitive types.
builtInType []
returns [String self=null]
	:	t0:"void"		{ self = t0.getText(); }
	|	t1:"boolean"	{ self = t1.getText(); }
	|	t2:"byte"		{ self = t2.getText(); }
	|	t3:"char"		{ self = t3.getText(); }
	|	t4:"short"		{ self = t4.getText(); }
	|	t5:"int"		{ self = t5.getText(); }
	|	t6:"float"		{ self = t6.getText(); }
	|	t7:"long"		{ self = t7.getText(); }
	|	t8:"double"		{ self = t8.getText(); }
	;

// A (possibly-qualified) java identifier.  We start with the first IDENT
// and expand its name by adding dots and following IDENTS
identifier []
returns [String self=null]
{
	StringBuffer name= new StringBuffer();
}
	:	
		i:IDENT  { name.append(i.getText()); }
		( 
			DOT j:IDENT 
			{ name.append( "." ); name.append( j.getText() ); }
		)*
		{ self = name.toString(); } 
	;

// A (possibly-qualified) java identifier with optional trailing star
// for import clauses
identifierStar []
returns [String self=null]
{
	StringBuffer name = new StringBuffer();
}
	:	
		i:IDENT  { name.append(i.getText()); }
		( 
			DOT j:IDENT 
			{ name.append( "." ); name.append( j.getText() ); }
		)*
		( DOT STAR { name.append( ".*" ); } )?
		{ self = name.toString(); } 
	;

// A list of zero or more modifiers.  We could have used (modifier)* in
//   place of a call to modifiers, but I thought it was a good idea to keep
//   this rule separate so they can easily be collected in an ArrayList if
//   someone so desires
modifiers []
returns [Modifiers self = new Modifiers()]
	:	( modifier[self]  )*
	;

// modifiers for Java classes, interfaces, class/instance vars and methods
modifier [Modifiers accum]
	: "private"		{ accum.setPrivateMod(true); }
	| "public"		{ accum.setPublicMod(true); }
	| "protected"	{ accum.setProtectedMod(true); }
	| "static"		{ accum.setStaticMod(true); }
	| "transient"	{ accum.setTransientMod(true); }
	| "final"		{ accum.setFinalMod(true); }
	| "abstract"	{ accum.setAbstractMod(true); }
	| "native"		{ accum.setNativeMod(true); }
	| "threadsafe"  { accum.setThreadsafeMod(true); }
	| "synchronized"{ accum.setSynchronizedMod(true);}
//	| "const"		//	reserved word, but not valid
	| "volatile"	{ accum.setVolatileMod(true); }
	| "strictfp"	{ accum.setStrictfpMod(true); }
	;

// Definition of a Java class
classDefinition [boolean accumText, Modifiers mods, Token startToken]
returns [ClassDecl self = null]
{
	TextAndAnnotations tAndA = null;
	String midText;
	String sup;
	String[] impl;
	ClassBlock cb;
}
	:	
		s:"class" 
		{ 
			if (accumText) { tAndA = parseJavadocsBefore(startToken); }
		}
		i:IDENT

		// it _might_ have a superclass...
  		sup = superClassClause[]
		
		// it might implement some interfaces...
  		impl = implementsClause[]

		// now parse the body of the class
		cb = classBlock[accumText,accumText ? tAndA.postText() : ""]

		{ 
			if (accumText) {
				self = new ClassDecl( tAndA.preText(), cb.midText, 
					cb.postText, tAndA.stereotype(), mods, i.getText(), 
					tAndA.invariants(), cb.members, impl, sup); 
			}
		}
	;

// returns string naming superclass, or null if extends clause is omitted
superClassClause []
returns [String self = null]
	:	( "extends" self = identifier[] )?
		
	;

// Definition of a Java Interface
interfaceDefinition [boolean accumText, Modifiers mods, Token startToken]
returns [InterfaceDecl self=null]
{
	TextAndAnnotations tAndA = null;
	String midText;
	String[] ext;
	ClassBlock cb;
}
	:	
		s:"interface" 
		{ 
			if (accumText) { tAndA = parseJavadocsBefore(startToken); }
		}
		i:IDENT

		// it might extend some other interfaces
		ext = interfaceExtends[]

		// now parse the body of the interface (looks like a class...)
  		cb = classBlock[accumText,accumText ? tAndA.postText() : ""]

		{ 
			if (accumText) {
				self = new InterfaceDecl( tAndA.preText(), cb.midText, 
					cb.postText, tAndA.stereotype(), mods, i.getText(), 
					tAndA.invariants(), cb.members, ext ); 
			}
		}
	;


// This is the body of a class.  You can have fields and extra semicolons,
// That's about it (until you see what a field is...)
classBlock [boolean accumText, String jdMidText]	// midText from javadoc
returns [ClassBlock self=null]
{
	String midText = jdMidText;
	ArrayList mbrList = new ArrayList();
	Member mbr = null;
}
	:	
		mid:LCURLY 
		{ 
			if (accumText) {
				midText += tokManifold.textBeforeWithNL(mid);
				discardJavadocBefore(mid); 
			}
		}

		( 
			mbr = field[accumText] { if (accumText) { mbrList.add(mbr); } } 
		| 
			SEMI 
		)*

		end:RCURLY
		{
			if (accumText) {
				discardJavadocBefore(end); 
				self = new ClassBlock( midText, 
					tokManifold.textBeforeWithNL(end), mbrList);
			}
		}
  	;

// An interface can extend several other interfaces...
interfaceExtends []
returns [String[] self=null]
{
	ArrayList intList = new ArrayList();
	String name = null;
}
	:	(
			"extends" name = identifier[] { intList.add(name); }
			( COMMA name = identifier[] { intList.add(name); } )*
			{
				self = new String[intList.size()];
				intList.toArray(self);
			}
		)?
	;

// A class can implement several interfaces...
implementsClause []
returns [String[] self=null]
{
	ArrayList intList = new ArrayList();
	String name = null;
}
	:	(
			"implements" name = identifier[] { intList.add(name); }
			( COMMA name = identifier[] { intList.add(name); } )*
			{
				self = new String[intList.size()];
				intList.toArray(self);
			}
		)?
	;

// Now the various things that can be defined inside a class or
// interface...  Note that not all of these are really valid in an
// interface (constructors, for example), and if this grammar were
// used for a compiler there would need to be some semantic checks to
// make sure we're doing the right thing...  The accumText parameter
// to this production is set to false when parsining local type
// declarations or anonymous class declarations inside method,
// constructor, or initializer bodies
field [boolean accumText]
returns [Member self = null]
{
	// common vars
	Modifiers mods = new Modifiers();
	Token startToken = LT(1);
	Token endToken = null;

	// vars for constructor decls
	CtorHead head;

	// vars for method and field decls
	String typ;
	TextAndAnnotations tAndA=null;
	String midText = null;
	Parameter[] params;
	String[] th = null;
}
	:	// method, constructor, or variable declaration
		mods=modifiers[]
		(	
			// constructor
			head=ctorHead[accumText,startToken] 
			self=constructorBody[accumText,mods,head]

		|	self=classDefinition[accumText,mods,startToken]	// inner class...
		|	self=interfaceDefinition[accumText,mods,startToken]	// or interface

		|	typ=typeSpec[] // method or variable declaration(s)
			{ if (accumText) { tAndA = parseJavadocsBefore(startToken); } }
			(	// method
				n:IDENT  

				// parse the formal parameter declarations.
				m:LPAREN 
				{ 
					if (accumText) {
						midText = tAndA.postText() + tokManifold.textBefore(m);
					}
				} 
				params = parameterDeclarationList[accumText] 
				RPAREN

				declaratorBrackets

				// get the list of exceptions that this method is
				// declared to throw
				( th=throwsClause[] )?

				( endToken=compoundStatement | mSemi:SEMI { endToken=mSemi; } )
				{
					if (accumText) {
						discardJavadocBefore(endToken);
						String postText = 
						tokManifold.textBeforeWithNL(endToken);
						self = new MethodDecl(tAndA.preText(), midText, 
							postText, tAndA.stereotype(), mods, n.getText(), 
							tAndA.input(), tAndA.output(), params, typ, th);
					}
				}
  			|	self=variableDefinitions[accumText,mods,typ,tAndA] vEnd:SEMI
				{ 
					if (accumText) {
						self.setPostText( tokManifold.textBeforeWithNL(vEnd)); 
					}
				}
			)
		)

    // "static { ... }" class initializer
	|	"static" { if (accumText) { tAndA = parseJavadocsBefore(startToken);} }
		endToken=compoundStatement
		{ 
			if (accumText) {
				mods = new Modifiers();
				mods.setStaticMod(true);
				self = new InitializerDecl( tAndA.preText(), 
					tAndA.postText() + tokManifold.textBeforeWithNL(endToken),
					tAndA.stereotype(), mods );
			}
		}
    // "{ ... }" instance initializer
	|	
		{ if (accumText) { tAndA = parseJavadocsBefore(startToken); } }
		endToken=compoundStatement
		{ 
			if (accumText) {
				mods = new Modifiers();
				self = new InitializerDecl( tAndA.preText(), 
					tAndA.postText() + tokManifold.textBeforeWithNL(endToken),
					tAndA.stereotype(), mods );
			}
		}
	;

// This is the header of a constructor.  It includes the name and
// parameters for the constructor.  This also watches for a list of
// exception classes in a "throws" clause.
ctorHead [boolean accumText,Token startToken]
returns [CtorHead self = null]
{
	TextAndAnnotations tAndA=null;
	String midText="";
	Parameter[] params;
	String[] th = null;
}
	:	n:IDENT { if (accumText) { tAndA = parseJavadocsBefore(startToken); } }

		// parse the formal parameter declarations.
		m:LPAREN 
		{ 
			if (accumText) {
				midText = tAndA.postText() + tokManifold.textBefore(m);
			}
		} 
		params = parameterDeclarationList[accumText] 
		RPAREN

		// get the list of exceptions that this method is declared to throw
		( th=throwsClause[] )?
		{ 
			if (accumText) {
				self = new CtorHead( n.getText(), midText, tAndA, params, th); 
			}
		}
	;

// This is a list of exception classes that the method is declared to throw
throwsClause []
returns [String[] self = null]
{
	ArrayList l = new ArrayList();
	String id;
}
	:	"throws" id=identifier[] { l.add(id); }
		( COMMA id=identifier[] { l.add(id); } )*
		{
			self = new String[l.size()];
			l.toArray(self);
		}
	;


// A list of formal parameters
parameterDeclarationList [boolean accumText]
returns [Parameter[] self = null]
{
	ArrayList pList = new ArrayList();
	Parameter p = null;
}
	:	
		( 
			p=parameterDeclaration[accumText] { pList.add(p); }
			( COMMA p=parameterDeclaration[accumText] { pList.add(p); } )* 
		)?
		{
			self = new Parameter[pList.size()];
			pList.toArray(self);
		}
	;

// A formal parameter.
parameterDeclaration [boolean accumText]
returns [Parameter self = null]
{
	String type;
}
	:	parameterModifier[] type=typeSpec[] id:IDENT
		declaratorBrackets
		{
			if (accumText) {
				String text = tokManifold.textBefore(LT(0));
				self = new Parameter(text, id.getText(), type );
			}
		}
	;

parameterModifier []
	:	(f:"final")?
	;

constructorBody [boolean accumText, Modifiers mods, CtorHead head]
returns [ConstructorDecl self = null]
{
}
    :   LCURLY 
		// Predicate might be slow but only checked once per constructor def
		// not for general methods.
		(	(explicitConstructorInvocation) => explicitConstructorInvocation
		|
		)
		(statement)*

        end:RCURLY
		{
			if (accumText) {
				discardJavadocBefore(end);
				// Yes, that is head.midText in the following not
				// head.tAndA.postText().  The ctorHead production
				// calculates midText to include text between end of
				// javadocs and left paren before parameters.
				self = new ConstructorDecl(head.tAndA.preText(), head.midText, 
					tokManifold.textBeforeWithNL(end), 
					head.tAndA.stereotype(), mods, head.name, 
					head.tAndA.input(), head.tAndA.output(), 
					head.params, head.thrown);
			}
		}
    ;

explicitConstructorInvocation
    :   (	options {
				// this/super can begin a primaryExpression too; with finite
				// lookahead ANTLR will think the 3rd alternative conflicts
				// with 1, 2.  I am shutting off warning since ANTLR resolves
				// the nondeterminism by correctly matching alts 1 or 2 when
				// it sees this( or super(
				generateAmbigWarnings=false;
			}
		:	"this" LPAREN argList RPAREN SEMI
	    |   "super" LPAREN argList RPAREN SEMI
		|
			// (new Outer()).super()  (create enclosing instance)
			primaryExpression DOT "super" LPAREN argList RPAREN SEMI
		)
    ;

variableDefinitions [boolean accumText, Modifiers mods, String typ, 
	TextAndAnnotations tAndA]
returns [FieldDecl self=null]
{
	ArrayList fdList = new ArrayList();
	FieldDecl fd;
}
	:	self=variableDeclarator[accumText,mods,typ,tAndA] 
		(	COMMA
			fd=variableDeclarator[accumText,mods,typ,null] 
			{ if (accumText) { fdList.add(fd); } }
		)*
		{ 
			if (accumText) {
				FieldDecl[] ov = new FieldDecl[fdList.size()];
				fdList.toArray(ov);
				self.setOtherVars(ov);
			}
		} 
	;

/** Declaration of a variable.  This can be a class/instance variable,
 *   or a local variable in a method
 * It can also include possible initialization.
 */
variableDeclarator [boolean accumText, Modifiers mods, String typ, 
	TextAndAnnotations tAndA]
returns [FieldDecl self=null]
	:	id:IDENT declaratorBrackets varInitializer
		{ 
			if (accumText) {
				String preText = "";
				String midText = "";
				String postText = "";
				StereotypeAnnotation seo = null;
				if (tAndA != null) {
					preText = tAndA.preText();
					midText = tAndA.postText();
					postText = null; // leave null to be completed later
					seo = tAndA.stereotype();
				}
				midText += tokManifold.textBefore(LT(0)); 
				self = new FieldDecl( preText, midText, postText, seo, mods, 
					typ, id.getText() );
			}
		}
	;

declaratorBrackets
	:
		(LBRACK RBRACK )*
	;

varInitializer
	:	( ASSIGN initializer )?
	;

// This is an initializer used to set up an array.
arrayInitializer
	:	LCURLY
		(	initializer
			(
				// CONFLICT: does a COMMA after an initializer start a new
				//           initializer or start the option ',' at end?
				//           ANTLR generates proper code by matching
				//			 the comma as soon as possible.
				options {
					warnWhenFollowAmbig = false;
				}
			:
				COMMA initializer
			)*
			(COMMA)?
		)?
		RCURLY
	;


// The two "things" that can initialize an array element are an expression
//   and another (nested) array initializer.
initializer
	:	expression
	|	arrayInitializer
	;

// Compound statement.  This is used in many contexts:
//   Inside a class definition prefixed with "static":
//      it is a class initializer
//   Inside a class definition without "static":
//      it is an instance initializer
//   As the body of a method
//   As a completely indepdent braced block of code inside a method
//      it starts a new scope for variable definitions
compoundStatement
returns [Token endToken=null]
	:	LCURLY
		// include the (possibly-empty) list of statements
		(statement)*
		end:RCURLY { endToken = end; }
	;

statement
	// A list of statements in curly braces -- start a new scope!
{ 
	Object discard;
	Modifiers mods;
}
	:	discard=compoundStatement

	// declarations are ambiguous with "ID DOT" relative to expression
	// statements.  Must backtrack to be sure.  Could use a semantic
	// predicate to test symbol table to see what the type was coming
	// up, but that's pretty hard without a symbol table ;)
	|	(declaration)=> declaration SEMI

	// An expression statement.  This could be a method call,
	// assignment statement, or any other expression evaluated for
	// side-effects.
	|	expression SEMI

	// class definition
	|	mods=modifiers[] discard=classDefinition[false,mods,LT(0)]

	// Attach a label to the front of a statement
	|	IDENT COLON statement

	// If-else statement
	|	"if" LPAREN expression RPAREN statement
		(
			// CONFLICT: the old "dangling-else" problem...
			//           ANTLR generates proper code matching
			//			 as soon as possible.  Hush warning.
			options {
				warnWhenFollowAmbig = false;
			}
		:
			"else" statement
		)?

	// For statement
	|	"for"
			LPAREN
				forInit SEMI   // initializer
				forCond	SEMI   // condition test
				forIter        // updater
			RPAREN
			statement                     // statement to loop over

	// While statement
	|	"while" LPAREN expression RPAREN statement

	// do-while statement
	|	"do" statement "while" LPAREN expression RPAREN SEMI

	// get out of a loop (or switch)
	|	"break" (IDENT)? SEMI

	// do next iteration of a loop
	|	"continue" (IDENT)? SEMI

	// Return an expression
	|	"return" (expression)? SEMI

	// switch/case statement
	|	"switch" LPAREN expression RPAREN LCURLY
			( casesGroup )*
		RCURLY

	// exception try-catch block
	|	tryBlock


	// throw an exception
	|	"throw" expression SEMI

	// synchronize a statement
	|	"synchronized" LPAREN expression RPAREN discard=compoundStatement

	// empty statement
	|	SEMI
  	;


casesGroup
	:	(	// CONFLICT: to which case group do the statements bind?
			//           ANTLR generates proper code: it groups the
			//           many "case"/"default" labels together then
			//           follows them with the statements
			options {
				warnWhenFollowAmbig = false;
			}
			:
			aCase
		)+
		caseSList
	;

aCase
	:	("case" expression | "default") COLON
	;

caseSList
	:	(statement)*
	;

// The initializer for a for loop
forInit
		// if it looks like a declaration, it is
	:	(	(declaration)=> declaration
		// otherwise it could be an expression list...
		|	expressionList
		)?
	;

forCond
	:	(expression)?
	;

forIter
	:	(expressionList)?
	;

// an exception handler try/catch block
tryBlock
{
	Object discard;
}
	:	"try" discard=compoundStatement
		(handler)*
		( "finally" discard=compoundStatement )?
	;


// an exception handler
handler
{
	Object discard;
}
	:	"catch" LPAREN discard=parameterDeclaration[false] RPAREN 
		discard=compoundStatement
	;


// expressions
// Note that most of these expressions follow the pattern
//   thisLevelExpression :
//       nextHigherPrecedenceExpression
//           (OPERATOR nextHigherPrecedenceExpression)*
// which is a standard recursive definition for a parsing an expression.
// The operators in java have the following precedences:
//    lowest  (13)  = *= /= %= += -= <<= >>= >>>= &= ^= |=
//            (12)  ?:
//            (11)  ||
//            (10)  &&
//            ( 9)  |
//            ( 8)  ^
//            ( 7)  &
//            ( 6)  == !=
//            ( 5)  < <= > >=
//            ( 4)  << >>
//            ( 3)  +(binary) -(binary)
//            ( 2)  * / %
//            ( 1)  ++ -- +(unary) -(unary)  ~  !  (type)
//                  []   () (method call)  . (dot -- identifier qualification)
//                  new   ()  (explicit parenthesis)
//
// the last two are not usually on a precedence chart; I put them in
// to point out that new has a higher precedence than '.', so you
// can validly use
//     new Frame().show()
// 
// Note that the above precedence levels map to the rules below...
// Once you have a precedence chart, writing the appropriate rules as below
//   is usually very straightfoward

// the mother of all expressions
expression
	:	assignmentExpression
	;


// This is a list of expressions.
expressionList
	:	expression (COMMA expression)*
	;


// assignment expression (level 13)
assignmentExpression
	:	conditionalExpression
		(	(	ASSIGN
            |   PLUS_ASSIGN
            |   MINUS_ASSIGN
            |   STAR_ASSIGN
            |   DIV_ASSIGN
            |   MOD_ASSIGN
            |   SR_ASSIGN
            |   BSR_ASSIGN
            |   SL_ASSIGN
            |   BAND_ASSIGN
            |   BXOR_ASSIGN
            |   BOR_ASSIGN
            )
			assignmentExpression
		)?
	;


// conditional test (level 12)
conditionalExpression
	:	logicalOrExpression
		( QUESTION assignmentExpression COLON conditionalExpression )?
	;


// logical or (||)  (level 11)
logicalOrExpression
	:	logicalAndExpression (LOR logicalAndExpression)*
	;


// logical and (&&)  (level 10)
logicalAndExpression
	:	inclusiveOrExpression (LAND inclusiveOrExpression)*
	;


// bitwise or non-short-circuiting or (|)  (level 9)
inclusiveOrExpression
	:	exclusiveOrExpression (BOR exclusiveOrExpression)*
	;


// exclusive or (^)  (level 8)
exclusiveOrExpression
	:	andExpression (BXOR andExpression)*
	;


// bitwise or non-short-circuiting and (&)  (level 7)
andExpression
	:	equalityExpression (BAND equalityExpression)*
	;


// equality/inequality (==/!=) (level 6)
equalityExpression
	:	relationalExpression ((NOT_EQUAL | EQUAL) relationalExpression)*
	;


// boolean relational expressions (level 5)
relationalExpression
{
	Object discard;
}
	:	shiftExpression
		(	(	(	LT
				|	GT
				|	LE
				|	GE
				)
				shiftExpression
			)*
		|	"instanceof" discard=typeSpec[]
		)
	;


// bit shift expressions (level 4)
shiftExpression
	:	additiveExpression ((SL | SR | BSR) additiveExpression)*
	;


// binary addition/subtraction (level 3)
additiveExpression
	:	multiplicativeExpression ((PLUS | MINUS) multiplicativeExpression)*
	;


// multiplication/division/modulo (level 2)
multiplicativeExpression
	:	unaryExpression ((STAR | DIV | MOD ) unaryExpression)*
	;

unaryExpression
	:	INC unaryExpression
	|	DEC unaryExpression
	|	MINUS unaryExpression
	|	PLUS unaryExpression
	|	unaryExpressionNotPlusMinus
	;

unaryExpressionNotPlusMinus
{
	Object discard;
}
	:	BNOT unaryExpression
	|	LNOT unaryExpression

	|	(	// subrule allows option to shut off warnings
			options {
				// "(int" ambig with postfixExpr due to lack of sequence
				// info in linear approximate LL(k).  It's ok.  Shut up.
				generateAmbigWarnings=false;
			}
		:	// If typecast is built in type, must be numeric operand
			// Also, no reason to backtrack if type keyword like int, float...
			LPAREN discard=builtInTypeSpec[] RPAREN
			unaryExpression

			// Have to backtrack to see if operator follows.  If no operator
			// follows, it's a typecast.  No semantic checking needed to parse.
			// if it _looks_ like a cast, it _is_ a cast; else it's a "(expr)"
		|	(LPAREN discard=classTypeSpec[] RPAREN 
				unaryExpressionNotPlusMinus)=>
			LPAREN discard=classTypeSpec[] RPAREN
			unaryExpressionNotPlusMinus

		|	postfixExpression
		)
	;

// qualified names, array expressions, method invocation, post inc/dec
postfixExpression
	:	primaryExpression // start with a primary

		(	// qualified id (id.id.id.id...) -- build the name
			DOT ( IDENT
				| "this"
				| "class"
				| newExpression
				| "super" // ClassName.super.field
				)
			// the above line needs a semantic check to make sure "class"
			// is the _last_ qualifier.

			// allow ClassName[].class
		|	( LBRACK RBRACK )+
			DOT^ "class"

			// an array indexing operation
		|	LBRACK expression RBRACK

			// method invocation
			// The next line is not strictly proper; it allows x(3)(4) or
			//  x[2](4) which are not valid in Java.  If this grammar were used
			//  to validate a Java program a semantic check would be needed, or
			//   this rule would get really ugly...
			// It also allows ctor invocation like super(3) which is now
			// handled by the explicit constructor rule, but it would
			// be hard to syntactically prevent ctor calls here
		|	LPAREN argList RPAREN
		)*

		// possibly add on a post-increment or post-decrement.
		// allows INC/DEC on too much, but semantics can check
		(	INC
	 	|	DEC
		|	// nothing
		)
	;

// the basic element of an expression
primaryExpression
{
	Object discard;
}
	:	IDENT
	|	constant
	|	"true"
	|	"false"
	|	"this"
	|	"null"
	|	newExpression
	|	LPAREN! assignmentExpression RPAREN!
	|	"super"
		// look for int.class and int[].class
	|	discard=builtInType[] ( LBRACK RBRACK )* DOT "class"
	;

/** object instantiation.
 *  Trees are built as illustrated by the following input/tree pairs:
 *  
 *  new T()
 *  
 *  new
 *   |
 *   T --  ELIST
 *           |
 *          arg1 -- arg2 -- .. -- argn
 *  
 *  new int[]
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *  
 *  new int[] {1,2}
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR -- ARRAY_INIT
 *                                  |
 *                                EXPR -- EXPR
 *                                  |      |
 *                                  1      2
 *  
 *  new int[3]
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *                |
 *              EXPR
 *                |
 *                3
 *  
 *  new int[1][2]
 *  
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *               |
 *         ARRAY_DECLARATOR -- EXPR
 *               |              |
 *             EXPR             1
 *               |
 *               2
 *  
 */
newExpression
{
	Object discard;
}
	:	"new" type
		(	LPAREN argList RPAREN (discard=classBlock[false,""])?

			//java 1.1
			// Note: This will allow bad constructs like
			//    new int[4][][3] {exp,exp}.
			//    There needs to be a semantic check here...
			// to make sure:
			//   a) [ expr ] and [ ] are not mixed
			//   b) [ expr ] and an init are not used together

		|	newArrayDeclarator (arrayInitializer)?
		)
	;

argList
	:	(	expressionList
		|	/*nothing*/
		)
	;

newArrayDeclarator
	:	(
			// CONFLICT:
			// newExpression is a primaryExpression which can be
			// followed by an array index reference.  This is ok,
			// as the generated code will stay in this loop as
			// long as it sees an LBRACK (proper behavior)
			options {
				warnWhenFollowAmbig = false;
			}
		:
			LBRACK (expression)? RBRACK
		)+
	;

constant
	:	NUM_INT
	|	CHAR_LITERAL
	|	STRING_LITERAL
	|	NUM_FLOAT
	|	NUM_LONG
	|	NUM_DOUBLE
	;


//----------------------------------------------------------------------------
// The Java scanner
//----------------------------------------------------------------------------
class JavaLexer extends Lexer;

options {
	importVocab = VSTBuilder;
	exportVocab = Java;      // call the vocabulary "Java"
	testLiterals=false;    // don't automatically test for literals
	k=4;                   // four characters of lookahead
	charVocabulary='\u0003'..'\uFFFF';
	// without inlining some bitset tests, couldn't do unicode;
	codeGenBitsetTestThreshold=20;
}

{
	public JavaLexer( TokenManifold tokManifold ) {
		this( tokManifold.sharedInputState() );
		this.tokManifold = tokManifold;
	}

	/**
	 * Used to switch lexers.  Aliases should be established in all
	 * lexers that share the same input stream. */
	private TokenManifold tokManifold;

	/**
	 * A key for this lexer used to identify an instance of it in the
	 * token manifold.  */
	public static final String KEY = "jLex";
}

// OPERATORS
QUESTION		:	'?'		;
LPAREN			:	'('		;
RPAREN			:	')'		;
LBRACK			:	'['		;
RBRACK			:	']'		;
LCURLY			:	'{'		;
RCURLY			:	'}'		;
COLON			:	':'		;
COMMA			:	','		;
//DOT			:	'.'		;
ASSIGN			:	'='		;
EQUAL			:	"=="	;
LNOT			:	'!'		;
BNOT			:	'~'		;
NOT_EQUAL		:	"!="	;
DIV				:	'/'		;
DIV_ASSIGN		:	"/="	;
PLUS			:	'+'		;
PLUS_ASSIGN		:	"+="	;
INC				:	"++"	;
MINUS			:	'-'		;
MINUS_ASSIGN	:	"-="	;
DEC				:	"--"	;
STAR			:	'*'		;
STAR_ASSIGN		:	"*="	;
MOD				:	'%'		;
MOD_ASSIGN		:	"%="	;
SR				:	">>"	;
SR_ASSIGN		:	">>="	;
BSR				:	">>>"	;
BSR_ASSIGN		:	">>>="	;
GE				:	">="	;
GT				:	">"		;
SL				:	"<<"	;
SL_ASSIGN		:	"<<="	;
LE				:	"<="	;
LT				:	'<'		;
BXOR			:	'^'		;
BXOR_ASSIGN		:	"^="	;
BOR				:	'|'		;
BOR_ASSIGN		:	"|="	;
LOR				:	"||"	;
BAND			:	'&'		;
BAND_ASSIGN		:	"&="	;
LAND			:	"&&"	;
SEMI			:	';'		;

// Non-newline whitespace
protected
NON_NL_WS
	:	' '
	|	'\t'
	|	'\f'
	;

// Whitespace
WS	:	(	
			NON_NL_WS
		|	
			NEWLINE
		)+
		{ $setType(NON_SEMANTIC); }
	;

// Single-line comments
SL_COMMENT
	:	"//"
		(~('\n'|'\r'))* NEWLINE
		{ $setType(NON_SEMANTIC); }
	;

// Multiple-line comments, with special case for javadoc comment
// style.  These are kept in one rule because otherwise the giant
// if-then-else generated within nextToken() must reference a huge
// bitset to differentiate between ML_COMMENT and JAVADOC_OPEN.  This
// bitset is final and when it is inlined the nextToken() method
// becomes too big for the verifier.
ML_COMMENT
	:
		"/*"
		(
			// javadoc start
			('*')+ ( NON_NL_WS )*
			{
				try {
					tokManifold.push(JavadocLexer.KEY);
				} catch( TokenManifold.KeyException ke ) {
					throw 
					new TokenStreamException( "No javadoc lexer available." );
				}
				$setType(NON_SEMANTIC);
			}
		|
			NON_STAR
			// regular multiple-line comment
			(	
				{ LA(2)!='/' }? '*'
			|	
				NON_STAR
			)*
			"*/"
			{$setType(NON_SEMANTIC);}
		)
	;

// this is a separate rule because otherwise ML_COMMENT includes two
// huge inlined tokenSets, which results in a verifier error because
// the generated method is too big
protected
NON_STAR
	:
		~('*')
		{ 
			// Now we need a hack to handle newlines.  If the non-star
			// character matched was a Mac or Unix newline then we
			// advance the line field by calling newline().  If the
			// non-star character was the first character of a Windows
			// newline then we do not advance the line field because
			// the next invocation do the advancing when it matches
			// what looks like a Unix newline.
			char matchedChar = $getText.charAt(0);
			if (matchedChar=='\n' || (matchedChar=='\r' && LA(1)!='\n')) {
				newline();
			}
		}
	;


// Will be ambiguous if called like ( NEWLINE | ... ) * because '\r'
// '\n' can be matched in one alternative or by matching '\r' in one
// iteration and '\n' in another.  ANTLR will greedily do the right
// thing.
protected
NEWLINE
	:
		(
			options {
				generateAmbigWarnings=false;
			}
		:
			'\r' '\n'
		|	'\r'
		|	'\n'
		)
		{newline();}
	;

// character literals
CHAR_LITERAL
	:	'\'' ( ESC | ~'\'' ) '\''
	;

// string literals
STRING_LITERAL
	:	'"' (ESC|~('"'|'\\'))* '"'
	;

// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
// There are various ambiguities hushed in this rule.  The optional
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched.  ANTLR does the
// right thing by matching immediately; hence, it's ok to shut off
// the FOLLOW ambig warnings.
protected
ESC
	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT 
		|	('0'..'3')
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	('0'..'7')
				(	
					options {
						warnWhenFollowAmbig = false;
					}
				:	'0'..'7'
				)?
			)?
		|	('4'..'7')
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	('0'..'9')
			)?
		)
	;


// hexadecimal digit (again, note it's protected!)
protected
HEX_DIGIT
	:	('0'..'9'|'A'..'F'|'a'..'f')
	;


// a dummy rule to force vocabulary to be all characters (except special
//   ones that ANTLR uses internally (0 to 2)
protected
VOCAB
	:	'\3'..'\377'
	;


// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
IDENT
	options {testLiterals=true;}
	:	('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$')*
	;


// a numeric literal
NUM_INT
	{boolean isDecimal=false; Token t=null;}
    :   '.' {_ttype = DOT;}
            (	('0'..'9')+ (EXPONENT)? (f1:FLOAT_SUFFIX {t=f1;})?
                {
				if (t != null && t.getText().toUpperCase().indexOf('D')>=0) {
                	_ttype = NUM_DOUBLE;
				}
				else {
                	_ttype = NUM_FLOAT;
				}
				}
            )?

	|	(	'0' {isDecimal = true;} // special case for just '0'
			(	('x'|'X')
				(											// hex
					// the 'e'|'E' and float suffix stuff look
					// like hex digits, hence the (...)+ doesn't
					// know when to stop: ambig.  ANTLR resolves
					// it correctly by matching immediately.  It
					// is therefor ok to hush warning.
					options {
						warnWhenFollowAmbig=false;
					}
				:	HEX_DIGIT
				)+
			|	('0'..'7')+									// octal
			)?
		|	('1'..'9') ('0'..'9')*  {isDecimal=true;}		// non-zero decimal
		)
		(	('l'|'L') { _ttype = NUM_LONG; }
		
		// only check to see if it's a float if looks like decimal so far
		|	{isDecimal}?
            (   '.' ('0'..'9')* (EXPONENT)? (f2:FLOAT_SUFFIX {t=f2;})?
            |   EXPONENT (f3:FLOAT_SUFFIX {t=f3;})?
            |   f4:FLOAT_SUFFIX {t=f4;}
            )
            {
			if (t != null && t.getText().toUpperCase() .indexOf('D') >= 0) {
                _ttype = NUM_DOUBLE;
			}
            else {
                _ttype = NUM_FLOAT;
			}
			}
        )?
	;


// a couple protected methods to assist in matching floating point numbers
protected
EXPONENT
	:	('e'|'E') ('+'|'-')? ('0'..'9')+
	;


protected
FLOAT_SUFFIX
	:	'f'|'F'|'d'|'D'
	;

