/*
 * Copyright (c) 2002, Iowa State University
 *
 * $Id: BuildVST.java,v 1.1 2002/03/26 00:31:34 cclifton Exp $
 */

package org.multijava.samples.cmb.parser.testcases;

import java.io.File;
import java.util.Date;
import org.multijava.samples.cmb.parser.*;
import java.io.PrintWriter;
import java.io.StringWriter;

/**
 * This class builds a simple VST for testing and experimentation.  */
public class BuildVST {
    public static Node build() {
	File f = new File("undefined");
	long ts = new Date().getTime();
	String preText = "/** comment that looks like javadoc but isn't **/\n\npackage foo;\n";
	PackageClause pc = new PackageClause(preText,"foo");
	ImportClause[] imps = new ImportClause[0];
	TypeDecl[] types = new TypeDecl[] {
	    buildClassDecl()
	    };
	String postText = "\n// trailing unit comment\n";
	return new CompilationUnit( f, ts, pc, imps, types, postText );
    }

    private static ClassDecl buildClassDecl() {
	String preText = 
	    "\n// comment\n\n/**\n * A real javadoc comment.\n * ";
	String midText = "\n */\npublic /*@ pure @*/ class Class1 {\n";
	String postText = "\n} // trailing class comment\n";
	StereotypeAnnotation sT = 
	    new StereotypeAnnotation("@stereotype utility", "utility");
	return new ClassDecl( preText, midText, postText, sT, new Modifiers(), 
			      "Class1", null, null, null, null );
    }
    
    public static void main (String[] args) {
	Node vst = build();

	// Print using pretty printer
	StringWriter out = new StringWriter();
	PrintWriter wrtr = new PrintWriter(out);
	vst.prettyPrint(wrtr);
	wrtr.close();
	System.out.println(out.toString());

	// Print again just using toString
	System.out.println(vst.toString());

	// Print the abstract structure
	System.out.println(vst.abstractRep());
    }
    
}
