header {
/**
 * Copyright (C) 2002, Iowa State University
 * 
 * Based loosely on: 
 * 
 * Java 1.3 Recognizer
 *
 * Run 'java Main [-showtree] directory-full-of-java-files'
 *
 * [The -showtree option pops up a Swing frame that shows
 *  the AST constructed from the parser.]
 * 
 * Run 'java Main <directory full of java files>'
 *
 * Contributing authors:
 *		John Mitchell		johnm@non.net
 *		Terence Parr		parrt@magelang.com
 *		John Lilley			jlilley@empathy.com
 *		Scott Stanchfield	thetick@magelang.com
 *		Markus Mohnen       mohnen@informatik.rwth-aachen.de
 *      Peter Williams      pete.williams@sun.com
 *      Allan Jacobs        Allan.Jacobs@eng.sun.com
 *      Steve Messick       messick@redhills.com
 *
 * Version 1.00 December 9, 1997 -- initial release
 * Version 1.01 December 10, 1997
 *		fixed bug in octal def (0..7 not 0..8)
 * Version 1.10 August 1998 (parrt)
 *		added tree construction
 *		fixed definition of WS,comments for mac,pc,unix newlines
 *		added unary plus
 * Version 1.11 (Nov 20, 1998)
 *		Added "shutup" option to turn off last ambig warning.
 *		Fixed inner class def to allow named class defs as statements
 *		synchronized requires compound not simple statement
 *		add [] after builtInType DOT class in primaryExpression
 *		"const" is reserved but not valid..removed from modifiers
 * Version 1.12 (Feb 2, 1999)
 *		Changed LITERAL_xxx to xxx in tree grammar.
 *		Updated java.g to use tokens {...} now for 2.6.0 (new feature).
 *
 * Version 1.13 (Apr 23, 1999)
 *		Didn't have (stat)? for else clause in tree parser.
 *		Didn't gen ASTs for interface extends.  Updated tree parser too.
 *		Updated to 2.6.0.
 * Version 1.14 (Jun 20, 1999)
 *		Allowed final/abstract on local classes.
 *		Removed local interfaces from methods
 *		Put instanceof precedence where it belongs...in relationalExpr
 *			It also had expr not type as arg; fixed it.
 *		Missing ! on SEMI in classBlock
 *		fixed: (expr) + "string" was parsed incorrectly (+ as unary plus).
 *		fixed: didn't like Object[].class in parser or tree parser
 * Version 1.15 (Jun 26, 1999)
 *		Screwed up rule with instanceof in it. :(  Fixed.
 *		Tree parser didn't like (expr).something; fixed.
 *		Allowed multiple inheritance in tree grammar. oops.
 * Version 1.16 (August 22, 1999)
 *		Extending an interface built a wacky tree: had extra EXTENDS.
 *		Tree grammar didn't allow multiple superinterfaces.
 *		Tree grammar didn't allow empty var initializer: {}
 * Version 1.17 (October 12, 1999)
 *		ESC lexer rule allowed 399 max not 377 max.
 *		java.tree.g didn't handle the expression of synchronized
 *		statements.
 * Version 1.18 (August 12, 2001)
 *      Terence updated to Java 2 Version 1.3 by observing/combining work of
 *      Allan Jacobs and Steve Messick.  Handles 1.3 src.
 *		Summary:
 *		o  primary didn't include boolean.class kind of thing
 *      o  constructor calls parsed explicitly now:
 * 		   see explicitConstructorInvocation
 *		o  add strictfp modifier
 *      o  missing objBlock after new expression in tree grammar
 *		o  merged local class definition alternatives, moved after declaration
 *		o  fixed problem with ClassName.super.field
 *      o  reordered some alternatives to make things more efficient
 *		o  long and double constants were not differentiated from int/float
 *		o  whitespace rule was inefficient: matched only one char
 *		o  add an examples directory with some nasty 1.3 cases
 *		o  made Main.java use buffered IO and a Reader for Unicode support
 *		o  supports UNICODE?
 *		   Using Unicode charVocabulay makes code file big, but only
 *		   in the bitsets at the end. I need to make ANTLR generate
 *		   unicode bitsets more efficiently.
 *
 * class Test {
 *   public static void main( String args[] ) {
 *     if (boolean.class.equals(boolean.class)) {
 *       System.out.println("works");
 *     }
 *   }
 * }
 *
 * This grammar (was) in the PUBLIC DOMAIN.  This version is not in
 * the public domain.
 */

	// common parser/lexer header
	package org.multijava.samples.cmb.query;

}

//----------------------------------------------------------------------------
// The Java scanner
//----------------------------------------------------------------------------
class CommentFinder extends Lexer;

options {
	testLiterals=false;    // don't automatically test for literals
	k=4;                   // four characters of lookahead
	charVocabulary='\u0003'..'\uFFFF';
	// without inlining some bitset tests, couldn't do unicode;
	codeGenBitsetTestThreshold=20;
}

{
	// custom member code
}

// OPERATORS
ASSIGN			:	'='		{ $setType(Token.SKIP); };
AT				:   '@'		{ $setType(Token.SKIP); };
BAND			:	'&'		{ $setType(Token.SKIP); };
BAND_ASSIGN		:	"&="	{ $setType(Token.SKIP); };
BNOT			:	'~'		{ $setType(Token.SKIP); };
BOR				:	'|'		{ $setType(Token.SKIP); };
BOR_ASSIGN		:	"|="	{ $setType(Token.SKIP); };
BSR				:	">>>"	{ $setType(Token.SKIP); };
BSR_ASSIGN		:	">>>="	{ $setType(Token.SKIP); };
BXOR			:	'^'		{ $setType(Token.SKIP); };
BXOR_ASSIGN		:	"^="	{ $setType(Token.SKIP); };
COLON			:	':'		{ $setType(Token.SKIP); };
COMMA			:	','		{ $setType(Token.SKIP); };
DEC				:	"--"	{ $setType(Token.SKIP); };
EQUAL			:	"=="	{ $setType(Token.SKIP); };
GE				:	">="	{ $setType(Token.SKIP); };
GT				:	">"		{ $setType(Token.SKIP); };
INC				:	"++"	{ $setType(Token.SKIP); };
LAND			:	"&&"	{ $setType(Token.SKIP); };
LBRACK			:	'['		{ $setType(Token.SKIP); };
LCURLY			:	'{'		{ $setType(Token.SKIP); };
LE				:	"<="	{ $setType(Token.SKIP); };
LNOT			:	'!'		{ $setType(Token.SKIP); };
LOR				:	"||"	{ $setType(Token.SKIP); };
LPAREN			:	'('		{ $setType(Token.SKIP); };
LT				:	'<'		{ $setType(Token.SKIP); };
MINUS			:	'-'		{ $setType(Token.SKIP); };
MINUS_ASSIGN	:	"-="	{ $setType(Token.SKIP); };
NOT_EQUAL		:	"!="	{ $setType(Token.SKIP); };
PERCENT			:	'%'		{ $setType(Token.SKIP); };
PERCENT_ASSIGN	:	"%="	{ $setType(Token.SKIP); };
PLUS			:	'+'		{ $setType(Token.SKIP); };
PLUS_ASSIGN		:	"+="	{ $setType(Token.SKIP); };
QUESTION		:	'?'		{ $setType(Token.SKIP); };
RBRACK			:	']'		{ $setType(Token.SKIP); };
RCURLY			:	'}'		{ $setType(Token.SKIP); };
RPAREN			:	')'		{ $setType(Token.SKIP); };
SEMI			:	';'		{ $setType(Token.SKIP); };
SL				:	"<<"	{ $setType(Token.SKIP); };
SLASH			:	'/'		{ $setType(Token.SKIP); };
SLASH_ASSIGN	:	"/="	{ $setType(Token.SKIP); };
SL_ASSIGN		:	"<<="	{ $setType(Token.SKIP); };
SR				:	">>"	{ $setType(Token.SKIP); };
SR_ASSIGN		:	">>="	{ $setType(Token.SKIP); };
STAR			:	'*'		{ $setType(Token.SKIP); };
STAR_ASSIGN		:	"*="	{ $setType(Token.SKIP); };

// Non-newline whitespace
protected
NON_NL_WS
	:	' '
	|	'\t'
	|	'\f'
	;

// Whitespace
WS	:	(	
			NON_NL_WS
		|	
			NEWLINE
		)+
		{ $setType(Token.SKIP); }
	;

// Single-line comments
SL_COMMENT
	:	"//"! (~('\n'|'\r'))* NEWLINE!
	;

// Multiple-line comments
ML_COMMENT
	:
		"/*"!
		(	
			{ LA(2)!='/' }? '*'
		|	
			~'*'
		)*
		"*/"!
	;

// this is a separate rule because otherwise ML_COMMENT includes two
// huge inlined tokenSets, which results in a verifier error because
// the generated method is too big
//  protected
//  NON_STAR
//  	:
//  		~('*')
//  		{ 
//  			// Now we need a hack to handle newlines.  If the non-star
//  			// character matched was a Mac or Unix newline then we
//  			// advance the line field by calling newline().  If the
//  			// non-star character was the first character of a Windows
//  			// newline then we do not advance the line field because
//  			// the next invocation do the advancing when it matches
//  			// what looks like a Unix newline.
//  			char matchedChar = $getText.charAt(0);
//  			if (matchedChar=='\n' || (matchedChar=='\r' && LA(1)!='\n')) {
//  				newline();
//  			}
//  		}
//  	;


// Will be ambiguous if called like ( NEWLINE | ... ) * because '\r'
// '\n' can be matched in one alternative or by matching '\r' in one
// iteration and '\n' in another.  ANTLR will greedily do the right
// thing.
protected
NEWLINE
	:
		(
			options {
				generateAmbigWarnings=false;
			}
		:
			'\r' '\n'
		|	'\r'
		|	'\n'
		)
		{newline();}
	;

// character literals
CHAR_LITERAL
	:	'\'' ( ESC | ~'\'' ) '\''
		{ $setType(Token.SKIP); }
	;

// string literals
STRING_LITERAL
	:	'"' (ESC|~('"'|'\\'))* '"'
		{ $setType(Token.SKIP); }
	;

// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
// There are various ambiguities hushed in this rule.  The optional
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched.  ANTLR does the
// right thing by matching immediately; hence, it's ok to shut off
// the FOLLOW ambig warnings.
protected
ESC
	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT 
		|	('0'..'3')
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	('0'..'7')
				(	
					options {
						warnWhenFollowAmbig = false;
					}
				:	'0'..'7'
				)?
			)?
		|	('4'..'7')
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	('0'..'9')
			)?
		)
	;


// hexadecimal digit (again, note it's protected!)
protected
HEX_DIGIT
	:	('0'..'9'|'A'..'F'|'a'..'f')
	;


// a dummy rule to force vocabulary to be all characters (except special
//   ones that ANTLR uses internally (0 to 2)
protected
VOCAB
	:	'\3'..'\377'
	;


// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
IDENT
	options {testLiterals=true;}
	:	('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$')*
		{ $setType(Token.SKIP); }
	;


// a numeric literal
NUM_INT
	{boolean isDecimal=false; Token t=null;}
    :   '.' 
		(	('0'..'9')+ (EXPONENT)? (f1:FLOAT_SUFFIX {t=f1;})?   )?

	|	(	'0' {isDecimal = true;} // special case for just '0'
			(	('x'|'X')
				(											// hex
					// the 'e'|'E' and float suffix stuff look
					// like hex digits, hence the (...)+ doesn't
					// know when to stop: ambig.  ANTLR resolves
					// it correctly by matching immediately.  It
					// is therefor ok to hush warning.
					options {
						warnWhenFollowAmbig=false;
					}
				:	HEX_DIGIT
				)+
			|	('0'..'7')+									// octal
			)?
		|	('1'..'9') ('0'..'9')*  {isDecimal=true;}		// non-zero decimal
		)
		(	('l'|'L')
		
		// only check to see if it's a float if looks like decimal so far
		|	{isDecimal}?
            (   '.' ('0'..'9')* (EXPONENT)? (f2:FLOAT_SUFFIX {t=f2;})?
            |   EXPONENT (f3:FLOAT_SUFFIX {t=f3;})?
            |   f4:FLOAT_SUFFIX {t=f4;}
            )
            
        )?
		{ $setType(Token.SKIP); }
	;


// a couple protected methods to assist in matching floating point numbers
protected
EXPONENT
	:	('e'|'E') ('+'|'-')? ('0'..'9')+
	;


protected
FLOAT_SUFFIX
	:	'f'|'F'|'d'|'D'
	;

