/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestCommentTokenizer.java,v 1.4 2002/04/30 14:00:11 cclifton Exp $
 */

package org.multijava.samples.cmb.query;

import org.multijava.util.testing.TestCase;
import org.multijava.samples.cmb.parser.*;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * This class provides unit testing for <CODE>CommentTokenizer</CODE>.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.4 $
 */

public class TestCommentTokenizer extends TestCase implements TestConstants {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public TestCommentTokenizer(String name) {
	super(name);
    }
    
    //---------------------------------------------------------------------
    // TESTS
    //---------------------------------------------------------------------

    public void test_stripPunctuation() {
	String expected = "Cubs";
	String result;

	result = CommentTokenizer.stripPunctuation(expected);
	assertEquals(expected,result);
	
 	result = CommentTokenizer.stripPunctuation(expected + ",");
	assertEquals(expected,result);
	
  	result = CommentTokenizer.stripPunctuation("--" + expected);
	assertEquals(expected,result);
	
  	result = CommentTokenizer.stripPunctuation("--" + expected + ".");
	assertEquals(expected,result);
	
  	result = CommentTokenizer.stripPunctuation(expected + "1");
	assertEquals(expected + "1",result);
	
  	result = CommentTokenizer.stripPunctuation("1" + expected);
	assertEquals("1" + expected,result);

	result = CommentTokenizer.stripPunctuation("@-%^#%");
	assertEquals("",result);
    }

    public void test_lexSingleLine() {
	ArrayList accum;

	accum = new ArrayList();
	CommentTokenizer.lexSingleLine("This is a comment",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexSingleLine("This is a comment.",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexSingleLine("This is   a     comment     .",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexSingleLine("This <is>a<comment>",accum);
	assertEquals(expected,accum);
    }

     public void test_lexMultiLine() {
	ArrayList accum;

	accum = new ArrayList();
	CommentTokenizer.lexMultiLine("This is a comment",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexMultiLine("This\nis\na\ncomment.",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexMultiLine("This\ris\n\ra\r\ncomment",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexMultiLine("\n\t* This is \n\t* a comment",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexMultiLine("\r\t* This is \r\t* a comment",accum);
	assertEquals(expected,accum);

	accum = new ArrayList();
	CommentTokenizer.lexMultiLine("\r\n\t* This is \r\n\t* a comment",
				      accum);
	assertEquals(expected,accum);
    }
    
    public void test_keywords() {
	String[] words = new String[] {
	    "zero", "one", "two", "three", "four", "five", "six", "seven"
	};
	
	String code = 
	    "// " + words[0] + " " + words[1] + "\n" +
	    "\n" +
	    "package foo;\n" +
	    "\n" +
	    "/**\n" +
	    " * " + words[2] + " " + words[3] + "\n" +
	    " */\n" +
	    "public /*" + words[4] + "*/ class Bar { // " + words[5] + "\n" +
	    "}\t// " + words[6] + " " + words[7] + "\n";
	    
	
	CommentTokenizer toker = new CommentTokenizer(code);
	assertTrue(Arrays.equals(words,toker.keywords()));
    }

    // -------------------------------------------------------------
    // TEST FIXTURE SETUP AND TEARDOWN
    // -------------------------------------------------------------

//      public void setUp() {
//      }

//      public void tearDown() {
//      }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    final ArrayList expected = new ArrayList();
    {
	expected.add("This");
	expected.add("is");
	expected.add("a");
	expected.add("comment");
    };
    

}
