/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestMemberWrapper.java,v 1.1 2002/03/26 00:31:34 cclifton Exp $
 */

package org.multijava.samples.cmb.query;

import org.multijava.util.testing.TestCase;
import org.multijava.samples.cmb.parser.*;
import java.util.TreeSet;
import java.util.SortedSet;

/**
 * This class provides unit testing for <CODE>MemberWrapper</CODE>.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $
 */

public class TestMemberWrapper extends TestCase implements TestConstants {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public TestMemberWrapper(String name) {
	super(name);
    }
    
    //---------------------------------------------------------------------
    // TESTS
    //---------------------------------------------------------------------
    
    public void test_equals() {
	assertTrue( mW1a.equals(mW1a) );
	assertTrue( mW1a.equals(mW1b) );
	assertTrue( mW1b.equals(mW1a) );
	assertTrue( mW1b.equals(mW1b) );
	assertTrue( mW2a.equals(mW2a) );
	assertTrue( mW2a.equals(mW2b) );
	assertTrue( mW2b.equals(mW2a) );
	assertTrue( mW2b.equals(mW2b) );
	assertFalse( mW1a.equals(mW2a) );
	assertFalse( mW1a.equals(mW2b) );
 	assertFalse( mW1b.equals(mW2a) );
 	assertFalse( mW1b.equals(mW2b) );
	assertFalse( mW2a.equals(mW1a) );
	assertFalse( mW2a.equals(mW1b) );
 	assertFalse( mW2b.equals(mW1a) );
 	assertFalse( mW2b.equals(mW1b) );
    }
    
    public void test_compareTo() {
	assertEquals( 0, mW1a.compareTo(mW1a) );
	assertEquals( 0, mW1a.compareTo(mW1b) );
	assertEquals( 0, mW1b.compareTo(mW1a) );
	assertEquals( 0, mW1b.compareTo(mW1b) );
	assertEquals( 0, mW2a.compareTo(mW2a) );
	assertEquals( 0, mW2a.compareTo(mW2b) );
	assertEquals( 0, mW2b.compareTo(mW2a) );
	assertEquals( 0, mW2b.compareTo(mW2b) );
	// We don't know the hashCodes from session to session, so we
	// first find the difference and then make sure it is used
	// consistently.

	int diff = mW1a.compareTo(mW2a);
	assertFalse( diff == 0 );

	assertEquals(  diff, mW1a.compareTo(mW2a) );
	assertEquals(  diff, mW1a.compareTo(mW2b) );
 	assertEquals(  diff, mW1b.compareTo(mW2a) );
 	assertEquals(  diff, mW1b.compareTo(mW2b) );

	assertEquals( -diff, mW2a.compareTo(mW1a) );
	assertEquals( -diff, mW2a.compareTo(mW1b) );
 	assertEquals( -diff, mW2b.compareTo(mW1a) );
 	assertEquals( -diff, mW2b.compareTo(mW1b) );
    }

    public void test_successor1() {
	Comparable successor;

	successor = mW1a.successor();
	assertTrue( mW1a.compareTo(successor) < 0 );
	assertTrue( successor.compareTo(mW1a) > 0 );

	successor = mW1b.successor();
	assertTrue( mW1b.compareTo(successor) < 0 );
	assertTrue( successor.compareTo(mW1b) > 0 );

	successor = mW2a.successor();
	assertTrue( mW2a.compareTo(successor) < 0 );
	assertTrue( successor.compareTo(mW2a) > 0 );

	successor = mW2b.successor();
	assertTrue( mW2b.compareTo(successor) < 0 );
	assertTrue( successor.compareTo(mW2b) > 0 );
    }

    public void test_successor2() {
	// build a set
	TreeSet set = new TreeSet();
	set.add(mW1a);
	set.add(mW1b);
	set.add(mW2a);
	set.add(mW2b);

	// check the set
	assertEquals( 2, set.size() );
	assertTrue( set.contains(mW1a) );
	assertTrue( set.contains(mW1b) );
	assertTrue( set.contains(mW2a) );
	assertTrue( set.contains(mW2b) );

	// find a tail set
	SortedSet tail = 
	    set.tailSet(((MemberWrapper) set.first()).successor());

	// check the tail set
	assertEquals( 1, tail.size() );
	if (set.first().equals(mW1a)) {
	    assertFalse( tail.contains(mW1a) );
	    assertFalse( tail.contains(mW1b) );
	    assertTrue ( tail.contains(mW2a) );
	    assertTrue ( tail.contains(mW2b) );
	} else {
	    assertTrue ( tail.contains(mW1a) );
	    assertTrue ( tail.contains(mW1b) );
	    assertFalse( tail.contains(mW2a) );
	    assertFalse( tail.contains(mW2b) );
	}
    }

    // -------------------------------------------------------------
    // TEST FIXTURE SETUP AND TEARDOWN
    // -------------------------------------------------------------

//      public void setUp() {
//      }

//      public void tearDown() {
//      }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------
}
