/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestOccuranceList.java,v 1.1 2002/03/26 00:31:34 cclifton Exp $
 */

package org.multijava.samples.cmb.query;

import org.multijava.util.testing.TestCase;
import org.multijava.samples.cmb.parser.*;

/**
 * This class provides unit testing for <CODE>OccuranceList</CODE>.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $
 */

public class TestOccuranceList extends TestCase implements TestConstants {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public TestOccuranceList(String name) {
	super(name);
    }
    
    //---------------------------------------------------------------------
    // TESTS
    //---------------------------------------------------------------------

    public void test_filter() {
	Member[] results;

	results = l12.filter(new NameMatcher("i1")).members();
	assertEquals(1,results.length);
	assertEquals(m1,results[0]);
	
	results = l12.filter(new NameMatcher("i2")).members();
	assertEquals(1,results.length);
	assertEquals(m2,results[0]);
	
	results = l12.filter(new NameMatcher("i3")).members();
	assertEquals(0,results.length);
	
	results = l23.filter(new NameMatcher("i1")).members();
	assertEquals(0,results.length);
	
	results = l23.filter(new NameMatcher("i2")).members();
	assertEquals(1,results.length);
	assertEquals(m2,results[0]);
	
	results = l23.filter(new NameMatcher("i3")).members();
	assertEquals(1,results.length);
	assertEquals(m3,results[0]);

	// make sure l12 and l23 are unchanged
	assertEquals(2,l12.members().length);
	assertEquals(2,l23.members().length);
    }

    public void test_intersect() {
	Member[] results;

	// intersection of non-empty sets
	results = l12.intersect(l12).members();
	assertEquals(2,results.length);

	results = l12.intersect(l23).members();
	assertEquals(1,results.length);
	assertEquals(m2,results[0]);

	results = l23.intersect(l23).members();
	assertEquals(2,results.length);

	results = l23.intersect(l12).members();
	assertEquals(1,results.length);
	assertEquals(m2,results[0]);

	// intersection with empty sets
	OccuranceList empty = OccuranceList.EMPTY;
	results = l12.intersect(empty).members();
	assertEquals(0,results.length);

	results = empty.intersect(empty).members();
	assertEquals(0,results.length);

	results = empty.intersect(l12).members();
	assertEquals(0,results.length);

	// make sure l12 and l23 are unchanged
	assertEquals(2,l12.members().length);
	assertEquals(2,l23.members().length);
    }
    
    public void test_union() {
	Member[] results;

	// union of non-empty sets
	results = l12.union(l12).members();
	assertEquals(2,results.length);

	results = l12.union(l23).members();
	assertEquals(3,results.length);

	results = l23.union(l23).members();
	assertEquals(2,results.length);

	results = l23.union(l12).members();
	assertEquals(3,results.length);

	// union with empty sets
	OccuranceList empty = OccuranceList.EMPTY;
	results = l12.union(empty).members();
	assertEquals(2,results.length);

	results = empty.union(empty).members();
	assertEquals(0,results.length);

	results = empty.union(l12).members();
	assertEquals(2,results.length);

	// make sure l12 and l23 are unchanged
	assertEquals(2,l12.members().length);
	assertEquals(2,l23.members().length);
    }
    
    public void test_minus() {
	Member[] results;

	// difference of non-empty sets
	results = l12.minus(l12).members();
	assertEquals(0,results.length);

	results = l12.minus(l23).members();
	assertEquals(1,results.length);
	assertEquals(m1,results[0]);

	results = l23.minus(l23).members();
	assertEquals(0,results.length);

	results = l23.minus(l12).members();
	assertEquals(1,results.length);
	assertEquals(m3,results[0]);

	// difference with empty sets
	OccuranceList empty = OccuranceList.EMPTY;
	results = l12.minus(empty).members();
	assertEquals(2,results.length);

	results = empty.minus(empty).members();
	assertEquals(0,results.length);

	results = empty.minus(l12).members();
	assertEquals(0,results.length);

	// make sure l12 and l23 are unchanged
	assertEquals(2,l12.members().length);
	assertEquals(2,l23.members().length);
    }
    
    // -------------------------------------------------------------
    // TEST FIXTURE SETUP AND TEARDOWN
    // -------------------------------------------------------------

//    public void setUp() {
//    }

//      public void tearDown() {
//      }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------
}
