/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestQueryParser.java,v 1.2 2002/06/25 17:08:29 cclifton Exp $
 */

package org.multijava.samples.cmb.query;

import org.multijava.util.testing.TestCase;
import antlr.Token;
import java.util.ArrayList;

/**
 * This class provides unit testing for the QueryParser class.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.2 $
 */
public class TestQueryParser extends TestCase implements QueryTokenTypes {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public TestQueryParser(String name) {
	super(name);
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------
    
    // -------------------------------------------------------------
    public void test_term_1() throws QueryProcessingException {
	result = QueryParser.parse("cat");
	assertEquals("cat",result.abstractRep());
    }

    public void test_term_2() throws QueryProcessingException {
	result = QueryParser.parse("(cat)");
	assertEquals("cat",result.abstractRep());
    }

    public void test_term_3() throws QueryProcessingException {
	result = QueryParser.parse(" ( cat ) ");
	assertEquals("cat",result.abstractRep());
    }

    // -------------------------------------------------------------
    public void test_not_1() throws QueryProcessingException {
	result = QueryParser.parse("dog NOT cat");
	assertEquals("[dog/\\!cat]",result.abstractRep());
    }

    public void test_not_2() throws QueryProcessingException {
	result = QueryParser.parse("(dog NOT cat)");
	assertEquals("[dog/\\!cat]",result.abstractRep());
    }

    public void test_not_3() throws QueryProcessingException {
	result = QueryParser.parse(" ( dog NOT cat ) ");
	assertEquals("[dog/\\!cat]",result.abstractRep());
    }

    public void test_not_4() throws QueryProcessingException {
	result = QueryParser.parse(" ( dog ) NOT ( cat ) ");
	assertEquals("[dog/\\!cat]",result.abstractRep());
    }

    // -------------------------------------------------------------
    public void test_and_1() throws QueryProcessingException {
	result = QueryParser.parse("dog AND cat");
	assertEquals("[dog/\\cat]",result.abstractRep());
    }

    // -------------------------------------------------------------
    public void test_or_1() throws QueryProcessingException {
	result = QueryParser.parse("dog OR cat");
	assertEquals("[dog\\/cat]",result.abstractRep());
    }

    // -------------------------------------------------------------
    public void test_precedence_1() throws QueryProcessingException {
	result = QueryParser.parse("dog OR cat AND horse");
	assertEquals("[dog\\/[cat/\\horse]]",result.abstractRep());
    }

    public void test_precedence_2() throws QueryProcessingException {
	result = QueryParser.parse("(dog OR cat) AND horse");
	assertEquals("[[dog\\/cat]/\\horse]",result.abstractRep());
    }

    // -------------------------------------------------------------
    public void test_badQuery_1() throws QueryProcessingException {
	try {
	    result = QueryParser.parse("?");
	    fail("should have thrown an exception");
	} catch (QueryProcessingException e) {
	    assertEquals(0,e.position());
	    assertEquals("Expected letter or digit but found '?'",
			 e.getMessage());
	}
    }

    public void test_badQuery_2() throws QueryProcessingException {
	try {
	    result = QueryParser.parse("cat OR ?");
	    fail("should have thrown an exception");
	} catch (QueryProcessingException e) {
	    assertEquals(7,e.position());
	    assertEquals("Expected letter or digit but found '?'",
			 e.getMessage());
	}
    }

    public void test_badQuery_3() throws QueryProcessingException {
	try {
	    result = QueryParser.parse("dog (NOT cat)");
	    fail("should have thrown an exception");
	} catch (QueryProcessingException e) {
	    assertEquals(4,e.position());
	    assertEquals("expecting EOF, found '('",
			 e.getMessage());
	}
    }

    // -------------------------------------------------------------
    // HELPERS
    // -------------------------------------------------------------


    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    private ParseNode result;
}
