/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: addToIndices.java,v 1.3 2002/06/25 17:08:29 cclifton Exp $
 */

package org.multijava.samples.cmb.query; //  !FIXME! should be in parser package, but we have to support finding of generic functions not specified on command line first

import org.multijava.samples.cmb.parser.*;
import java.util.Iterator;
import java.io.File;

public void CompilationUnit.addToIndices(Index idIndex, Index kwIndex) {
    Member[] mbrs = types();
    for (int i = 0; i < mbrs.length; i++) {
	mbrs[i].addToIndices( file(), idIndex, kwIndex );
    }
}


/**
 * Adds the information for a <CODE>Member</CODE> to the indices.
 * This default implementation assumes that the <CODE>Member</CODE>
 * does not contain any child nodes that are also
 * <CODE>Member</CODE>s.  */
void Member.addToIndices( File f, Index idIndex, Index kwIndex) {
    MemberWrapper mW = new MemberWrapper( this, f, 0 ); //  !FIXME! position is hardcoded

    idIndex.insert( name(), mW );

    CommentTokenizer cT = new CommentTokenizer(toString());
    cT.addMappingsForTo( mW, kwIndex );
}

/**
 * Adds the information for a <CODE>TypeDecl</CODE> to the indices.
 * This implementation overrides the one for <CODE>Member</CODE> since
 * <CODE>TypeDecl</CODE>s can have child nodes that are also
 * <CODE>Member</CODE>s.  */
void TypeDecl.addToIndices( File f, Index idIndex, Index kwIndex) {
    MemberWrapper mW = new MemberWrapper( this, f, 0 ); //  !FIXME! position is hardcoded

    idIndex.insert( name(), mW );

    CommentTokenizer cT = new CommentTokenizer( preText() + midText() );
    cT.addMappingsForTo( mW, kwIndex );

    cT = new CommentTokenizer( postText() );
    cT.addMappingsForTo( mW, kwIndex );

    Iterator iter = members().iterator();
    while (iter.hasNext()) {
	Member m = (Member) iter.next();
	m.addToIndices( f, idIndex, kwIndex );
    }
}
