/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: createParentLinks.java,v 1.1 2002/04/30 14:00:11 cclifton Exp $
 */

package org.multijava.samples.cmb.query;

import org.multijava.samples.cmb.parser.*;
import java.util.List;

/**
 * This generic function initializes a synthetic parentOf field.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $ 
 * @return parent of this in the VST, or null if this is parentless 
 * @see parentOf()
 */
void CompilationUnit.createParentLinks() {
    cplHelper(null);
}

/**
 * This helper generic function builds the parent links map below the
 * compilation unit level.  */
private void Node.cplHelper(Node parent) {
    Index.parentLinksMap.put(this,parent);
    Member[] mbrs = memberChildren();
    for (int i = 0; i < mbrs.length; i++) {
	mbrs[i].cplHelper(this);
    }
}

/**
 * This helper generic function provides a standard access mechanism
 * for getting at the children of a node.  */
private Member[] Node.memberChildren() {
    // only CUs and TypeDecls have children
    return new Member[0];
}

private Member[] CompilationUnit.memberChildren() {
    return types();
}

private Member[] TypeDecl.memberChildren() {
    List mbrs = members();
    Member[] result = new Member[mbrs.size()];
    mbrs.toArray(result);
    return result;
}
