/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: AboutDialog.java,v 1.1 2002/04/04 15:37:23 cclifton Exp $
 */

package org.multijava.samples.cmb.ui;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This dialog displays the about information for the program.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $
 */
public class AboutDialog extends JDialog {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public AboutDialog( MainFrame owner ) {
	super(owner, "Comment Mining Browser");
	setResizable(false);

	// content
	tPane = new JTabbedPane();
	getContentPane().add(tPane);
	addTab("About",ABOUT);
	addTab("License",LICENSE);
	addTab("Warranty",WARRANTY);
	
	pack();
	show();
    }
    
    private void addTab( String title, String message ) {
	JPanel tab = new JPanel();
	tab.setLayout(new BorderLayout(10,10));
	JPanel textPanel = new JPanel();
	JTextArea textArea = new JTextArea(message);
	textArea.setEditable(false);
	textArea.setWrapStyleWord(true);
	textArea.setBackground(tab.getBackground());
	textPanel.add(textArea);
	JPanel buttonPanel = new JPanel();
	JButton closeButton = new JButton("Close");
	closeButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    hide();
		}
	    });
	buttonPanel.add(closeButton);
	tab.add(textPanel,BorderLayout.CENTER);
	tab.add(buttonPanel,BorderLayout.SOUTH);
	tPane.addTab(title,tab);
    }

    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------

    void selectAndShow() {
	tPane.setSelectedIndex(0);
	setLocationRelativeTo(this.getOwner());

	show();
	toFront();
    }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    private final JTabbedPane tPane;

    private static final String ABOUT =
	"Comment Mining Browser\n" +
	"by Curtis Clifton\n" +
	"\n" +
	"This program is part of the MultiJava project.\n" +
	"http://www.multijava.org";

    private static final String GPL =
	"\n" +
	"You should have received a copy of the GNU \n" +
	"General Public License along with this \n" +
	"program; if not, write to the Free Software\n" +
	"Foundation, Inc., 59 Temple Place, \n" +
	"Suite 330, Boston, MA 02111-1307 USA";

    private static final String LICENSE =
	"This program is free software; you can \n" +
	"redistribute it and/or modify it under\n" +
	"the terms of the GNU General Public \n" +
	"License as published by the Free Software\n" +
	"Foundation; either version 2 of the License, \n" +
	"or (at your option) any later version.\n" +
	GPL;

    private static final String WARRANTY =
	"This program is distributed in the hope \n" +
	"that it will be useful, but WITHOUT ANY \n" +
	"WARRANTY; without even the implied warranty \n" +
	"of MERCHANTABILITY or FITNESS FOR A \n" +
	"PARTICULAR PURPOSE. See the GNU General \n" +
	"Public License for more details.\n" +
	GPL;
}
