/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MainFrame.java,v 1.7 2003/11/01 00:33:25 leavens Exp $
 */

package org.multijava.samples.cmb.ui;

import javax.swing.JFrame;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.BorderFactory;
import java.io.File;
import org.multijava.samples.cmb.query.Index;

/**
 * This class represents the main user-interface frame for the
 * application.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.7 $ */
public class MainFrame extends JFrame {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public MainFrame() {
	// set up display
	setTitle("Comment Mining Browser");
	Toolkit tk = Toolkit.getDefaultToolkit();
	setIconImage(tk.createImage("icon.gif"));

	//  !FIXME! recover position from properties file
	setLocation(120,40);
	fileTree = new FileTreeModel(); //  !FIXME! get from properties

	// Handle closing
	setDefaultCloseOperation(EXIT_ON_CLOSE);//  !FIXME! write properties
	
	// Add menu bar
	JMenuBar mBar = new JMenuBar();
	setJMenuBar(mBar);

	JMenu fileMenu = new JMenu("File");
	fileMenu.setMnemonic(KeyEvent.VK_F);
	JMenuItem openItem = new JMenuItem("Open program...");
	openItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    openProgram();
		}
	    });
	openItem.setMnemonic(KeyEvent.VK_O);
	openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
						       Event.CTRL_MASK));
	fileMenu.add(openItem);
	JMenuItem exitItem = new JMenuItem("Close");
	exitItem.setMnemonic(KeyEvent.VK_C);
	exitItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    System.exit(0); //  !FIXME! write properties
		}
	    });
	fileMenu.add(exitItem);
	mBar.add(fileMenu);

	JMenu helpMenu = new JMenu("Help");
	helpMenu.setMnemonic(KeyEvent.VK_H);
	JMenuItem aboutItem = new JMenuItem("About...");
	aboutItem.setMnemonic(KeyEvent.VK_A);
	aboutItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    about();
		}
	    });
	helpMenu.add(aboutItem);
	mBar.add(helpMenu);

	JMenu debugMenu = new JMenu("Debug");
	debugMenu.setMnemonic(KeyEvent.VK_D);
	JMenuItem indicesItem = new JMenuItem("Indices...");
	indicesItem.setMnemonic(KeyEvent.VK_I);
	indicesItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    System.out.println("Keyword Index:");
		    System.out.println(kwIndex);
		    System.out.println("Identifier Index:");
		    System.out.println(idIndex);
		}
	    });
	debugMenu.add(indicesItem);
	JMenuItem reportItem = new JMenuItem("Parsing report...");
	reportItem.setMnemonic(KeyEvent.VK_P);
	reportItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    System.out.println("Parsing report:");
		    System.out.println(Index.messages());
		}
	    });
	debugMenu.add(reportItem);
	mBar.add(debugMenu);

	// Add content
	JPanel contents = new JPanel(new BorderLayout());
	setContentPane(contents);
	contents.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	rPanel = new ResultsPanel();
	rPanel.setTreeModel(fileTree);
	qPanel = new QueryPanel(this,rPanel);
	contents.add(qPanel,BorderLayout.NORTH);
	contents.add(rPanel,BorderLayout.CENTER);
	
	// Show ourself
	pack();			//  !FIXME! get size from properties instead
	show();
    }

    public static void main (String[] args) {
	JFrame self = new MainFrame();
    }
    
    //---------------------------------------------------------------------
    // PROCESS MENU ITEMS
    //---------------------------------------------------------------------
    

    /**
     * Handles a user action that requests the opening of a new
     * program.  Dialog will call-back to this when user is done with
     * it. */
    void openProgram() {
	FileTreeModel updated = 
	    OpenDialog.show(/* !FIXME! use properties file to get last dir used */
			    new File(System.getProperty("user.dir")),
			    this,fileTree);
	if (updated != null) {
	    fileTree = updated;
	    rPanel.setTreeModel(fileTree);
	    parsingInProgress = true;
	    final File[] files = fileTree.regularFiles();
	    final ProgressMonitor progMon = 
		new ProgressMonitor(this,"Parsing program","",0,files.length);
	    progMon.setProgress(0);
	    progMon.setMillisToDecideToPopup(500);
	    //  spawn a new thread to build the indices.
	    Thread thrd = new Thread( new Runnable() {
		    public void run() {
			// clear the old indices so garbage collection
			// can happen during the build
			idIndex = null;
			kwIndex = null;
			Index[] indices = Index.buildIndexFor(files,progMon);
			idIndex = indices[0];
			kwIndex = indices[1];
			parsingInProgress = false;
		    }
		});
	    thrd.start();
	}
	
    }

    /**
     * Displays the "about" dialog box for the application.  */
    void about() {
	if (about == null) {
	    about = new AboutDialog(this);
	}
	about.selectAndShow();
    }
    
    // -------------------------------------------------------------
    // ACCESSORS
    // -------------------------------------------------------------
    
    Index idIndex() { return idIndex; }

    Index kwIndex() { return kwIndex; }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    private AboutDialog about;
    private QueryPanel qPanel;
    private ResultsPanel rPanel;
    private FileTreeModel fileTree;
    private boolean parsingInProgress = false;
    private Index idIndex;
    private Index kwIndex;
}
