/*
 * Copyright (C) 2002 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: ResultsPanel.java,v 1.5 2002/06/25 17:08:30 cclifton Exp $
 */

package org.multijava.samples.cmb.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.Border;
import org.multijava.samples.cmb.query.OccuranceList;
import org.multijava.samples.cmb.parser.*;
import java.io.File;

import org.multijava.samples.cmb.query.parentOf;

/**
 * This class represents the results panel for the Comment Mining Browser. 
 *
 * @author Curtis Clifton
 * @version $Revision: 1.5 $
 */
public class ResultsPanel extends JPanel {

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public ResultsPanel() {
	super();
	setLayout(new BorderLayout());
	// Next the result statistics
	{
	    JPanel statPanel = new JPanel();
	    statPanel.setLayout(new BorderLayout()); //  !FIXME!use Y_AXIS box?
	    Border b = 
		BorderFactory.
		createCompoundBorder(BorderFactory.
				     createTitledBorder("Query statistics"),
				     BorderFactory.
				     createEmptyBorder(0,5,0,5));
	    statPanel.setBorder(b);
	    statistics = new JTextField("");
	    statistics.setEditable(false);
	    statistics.setBackground(statPanel.getBackground());
	    statistics.setBorder(BorderFactory.createEmptyBorder());
	    statPanel.add(statistics,BorderLayout.NORTH);
	    add(statPanel,BorderLayout.NORTH);
	}

	// create a panel to hold result tree and text
	JPanel resultSubPanel = new JPanel();
	resultSubPanel.setLayout(new BoxLayout(resultSubPanel,
					       BoxLayout.X_AXIS));
	Border b = 
	    BorderFactory.
	    createCompoundBorder(BorderFactory.createTitledBorder("Results"),
				 BorderFactory.createEmptyBorder(5,5,5,5));
	resultSubPanel.setBorder(b);
	add(resultSubPanel,BorderLayout.CENTER);

	// Create a result tree area
	tree = new JTree();
	tree.setEditable(false);
	tree.setRootVisible(false);
	tree.setShowsRootHandles(true);
	JScrollPane treeScroll = new JScrollPane(tree);

	// Create a result text area
	text = new JTextArea("");
	text.setEditable(false);
	text.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	text.setFont(new Font("Monospaced",Font.PLAIN,14));
	JScrollPane textScroll = new JScrollPane(text);

	// Create a split pane for the results
	JSplitPane splitP = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					   treeScroll,textScroll);
	splitP.setDividerLocation(0.25);
	splitP.setOneTouchExpandable(true);
	resultSubPanel.add(splitP);

    }
    
    //---------------------------------------------------------------------
    // ACCESSORS
    //---------------------------------------------------------------------

    public void display(OccuranceList results) {
	Member[] mbrs = results.members();
	final int l = mbrs.length;

	// set text
	StringBuffer disp = new StringBuffer();

	for (int i = 0; i < l; i++) {
	    Member m = mbrs[i];
	    File f = file(m);
	    disp.append("In " + f.getName());

	    Member container = container(m);
	    if (container != null) {
		disp.append(" in type " + container.name());
	    }

	    disp.append('\n')
		.append(m.toString())
		.append("\n------------------------------------------------\n");
	}
	
	text.setText(disp.toString());

	// set statistics
	String stats;
	switch (l) {
	case 0:
	    stats = "No declarations matched";
	    break;

	case 1:
	    stats = "1 declaration matched";
	    break;

	default:
	    stats = l + " declarations matched";
	    break;
	}
	statistics.setText(stats);
    }

    public void setTreeModel(FileTreeModel treeModel) {
	tree.setModel(treeModel);
    }

    // -------------------------------------------------------------
    // HELPER METHODS
    // -------------------------------------------------------------

    private File file(Node m) {
	throw new NullPointerException();
    }

    private File file(Node@CompilationUnit cu) {
	return cu.file();
    }

    private File file(Node@Member m) {
	return file(m.parentOf());
    }

    private Member container(Member n) {
	Node p = n.parentOf();
	if (p instanceof CompilationUnit) {
	    return null;
	} else {
	    return (Member) p;
	}
	
    }

    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    private JTextField statistics;
    private JTextArea text;
    private JTree tree;
}
