/*
 * Copyright (C) 2003 Iowa State University
 *
 * This file is part of the MultiJava project.  More information is 
 * available from www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: ClassPathContains.java,v 1.4 2004/01/16 16:26:56 cclifton Exp $
 */

package org.multijava.util;
import java.io.File;
/**
 * This class is a command-line utility to test for the presence of a
 * given class on the CLASSPATH.  It is invoked like:
 *
 * <CODE>java org.multijava.util.Classpath {pack.subpack.Class}+</CODE>
 *
 * It returns an error code of 0 if all the listed classes exist on
 * the CLASSPATH and 1 otherwise.
 *
 * Alternatively, with an option of -d as the first argument, it interprets
 * argument 2 as a file or directory name, and tests which of arguments 3
 * and following contain argument 2, printing the first such argument that
 * does contain it.  It exits with 0 if one is found, and 1 if none is found.
 *		- DRCok 11/9/2003
 *
 * @author Curtis Clifton
 * @version $Revision: 1.4 $
 */
public class ClassPathContains {

    public static void main (String[] args) {
	if (args.length > 0 && args[0].equals("-d")) {
	    find(args);
	}

	for (int i = 0; i < args.length; i++) {
	    if (!classExists(args[i])) {
		System.out.println("no");
		System.exit(1);
	    }
	}
	System.out.println("yes");
	System.exit(0);
    }
 
    private static boolean classExists(String fullyQualifiedClassName) {
	try {
	    Class test = Class.forName(fullyQualifiedClassName);
	} catch (ClassNotFoundException e) {
	    return false;
	}
	return true;
    }

    private static void find(String[] args) {
	if (args.length <3) { System.exit(1); }
	String f = args[1];
	for (int i=2; i<args.length; ++i) {
	    String d = args[i];
	    File file = new File(d);
	    if ( (new File(file,f)).exists()) {
		System.out.println(d);
		System.exit(0);
	    }
        }
	System.exit(1);
    }

}
