/*
 * Copyright (C) 2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, www.multijava.org.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestArrayListCache.java,v 1.1 2003/01/08 00:32:14 cclifton Exp $
 */

package org.multijava.util;

import org.multijava.util.testing.TestCase;

import java.util.ArrayList;

/**
 * TestArrayListCache.java
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $
 */

public class TestArrayListCache extends TestCase {

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------
    
    public TestArrayListCache( String name ) {
	super( name );
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------
    // use names like test_memberName_memberKindX,
    // e.g. test_hashCode_method1

    // Request and immediately release a bunch of ArrayLists.
    public void test_catchAndRelease() {
	for (int i = 0; i < num; i++) {
	    ArrayList l = ArrayListCache.request(10);
	    assertTrue(l != null);
	    ArrayListCache.release(l);
	}
    }

    // Request a bunch of ArrayLists and then release them in same order.
    public void test_catchHoldAndRelease() {
	initLists();
	for (int i = 0; i < num; i++) {
	    lists[i] = ArrayListCache.request(10);
	    assertTrue(lists[i] != null);
	}
	for (int i = 0; i < num; i++) {
	    ArrayListCache.release(lists[i]);
	    lists[i] = null;
	}
    }

    // Request a bunch of ArrayLists and then release them in reverse order.
    public void test_catchHoldAndReverseRelease() {
	initLists();
	for (int i = 0; i < num; i++) {
	    lists[i] = ArrayListCache.request(10);
	    assertTrue(lists[i] != null);
	}
	for (int i = num-1; i >= 0; i--) {
	    ArrayListCache.release(lists[i]);
	    lists[i] = null;
	}
    }

    // Request, mutate, and release a bunch of ArrayLists.  Request a
    // bunch again and make sure they are empty.
    public void test_checkClearing() {
	initLists();
	for (int i = 0; i < num; i++) {
	    lists[i] = ArrayListCache.request(10);
	    assertTrue(lists[i] != null);
	    lists[i].add(this);
	}
	for (int i = 0; i < num; i++) {
	    ArrayListCache.release(lists[i]);
	    lists[i] = null;
	}
	for (int i = 0; i < num; i++) {
	    lists[i] = ArrayListCache.request(10);
	    assertTrue(lists[i].isEmpty());
	}
	for (int i = 0; i < num; i++) {
	    ArrayListCache.release(lists[i]);
	    lists[i] = null;
	}
    }

    //---------------------------------------------------------------------
    // PRIVATE HELPERS
    //---------------------------------------------------------------------
    
    private void initLists() {
	lists = new ArrayList[num];
    }
    
    //---------------------------------------------------------------------
    // COMMON FIXTURE
    //---------------------------------------------------------------------

    ArrayList[] lists;
    int num = ArrayListCache.MIN_CACHE_SIZE * 5;

    // public void setUp() {}

    // public void tearDown() { }
    
    
}
