/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestDirectedAcyclicGraph.java,v 1.1 2002/10/22 19:58:10 cclifton Exp $
 */

package org.multijava.util;

import org.multijava.util.testing.TestCase;
import java.util.*;

/**
 * JUnit tests for DirectedAcyclicGraph.
 *
 * @author Curtis Clifton
 * @version $Revision: 1.1 $
 */
public class TestDirectedAcyclicGraph extends TestCase {

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------
    
    public TestDirectedAcyclicGraph( String name ) {
	super( name );
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------
    // use names like test_memberName_memberKindX,
    // e.g. test_hashCode_method1

    public void test_inDFSOrder_method1() {
	DirectedAcyclicGraph dag = 
	    new DirectedAcyclicGraph(powerSet, EC);
	check(dag.inDFSOrder());
    }

    public void test_inDFSOrder_method2() {
	Set[] reversed = new Set[powerSet.length];
	for (int i = 0, j=powerSet.length - 1; i < powerSet.length; i++, j--) {
	    reversed[j] = powerSet[i];
	}
	
	DirectedAcyclicGraph dag = 
	    new DirectedAcyclicGraph(reversed, EC);
	check(dag.inDFSOrder());
    }

    public void test_inDFSOrder_method3() {
	Collections.shuffle(Arrays.asList(powerSet));
	DirectedAcyclicGraph dag = 
	    new DirectedAcyclicGraph(powerSet, EC);
	check(dag.inDFSOrder());
    }

    //---------------------------------------------------------------------
    // COMMON FIXTURE
    //---------------------------------------------------------------------

    private void check(Object[] powerSet) {
	for (int i = 0; i < powerSet.length; i++) {
	    Set earlier = (Set) powerSet[i];
	    for (int j = i+1; j < powerSet.length; j++) {
		Set later = (Set) powerSet[j];
		if (earlier.containsAll(later)) {
		    fail("Out of order: " + earlier + " before " + later);
		}
	    }
	}
    }

    private Set[] powerSet(String[] elts) {
	int count = 1 << elts.length;
	Set[] powerSet = new Set[count];
	for (int i = 0; i < count; i++ ) {
	    int sel = i;
	    Set s = new HashSet();
	    int pos = 0;
	    while (sel != 0 && pos < elts.length) {
		if ((1 & sel) != 0) {
		    s.add(elts[pos]);
		}
		sel >>= 1;
		pos++;
	    }
	    powerSet[i] = s;
	}
	return powerSet;
    }

    private Set[] powerSet;

    private static final DirectedAcyclicGraph.EdgeCalculator EC =
	new DirectedAcyclicGraph.EdgeCalculator() {
	    public boolean edgeExists(Object o1, Object o2) {
		Set s1 = (Set) o1;
		Set s2 = (Set) o2;
		return s2.containsAll(s1) && !s1.containsAll(s2);
	    }
	};

    public void setUp() { 
	String[] elts = { "w", "x", "y", "z" };
	powerSet = powerSet(elts);
    }
    
    // public void tearDown() { }
    
    
}
