/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestIntStack.java,v 1.4 2004/01/16 16:26:56 cclifton Exp $
 */

package org.multijava.util;
import org.multijava.util.testing.TestCase;
import java.util.EmptyStackException;

/**
 * TestIntStack.java
 *
 * @author Curtis Clifton
 * @version $Revision: 1.4 $
 */

public class TestIntStack extends TestCase {

    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------
    
    public TestIntStack( String name ) {
	super( name );
    }

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------
    // use names like test_memberName_memberKindX,
    // e.g. test_hashCode_method1

    public void test_isEmpty_method1() {
	assertTrue( stack.isEmpty() );
	stack.push(1);
	assertTrue( !stack.isEmpty() );
	stack.peek();
	assertTrue( !stack.isEmpty() );
	stack.pop();
	assertTrue( stack.isEmpty() );
    }

    public void test_size_method1() {
	assertEquals( 0, stack.size() );
	stack.push(1);
	assertEquals( 1, stack.size() );
	stack.peek();
	assertEquals( 1, stack.size() );
	stack.pop();
	assertEquals( 0, stack.size() );
    }

    public void test_peek_method1() {
	stack.push(1);
	assertEquals( 1, stack.peek() );
	stack.push(2);
	assertEquals( 2, stack.peek() );
	stack.push(3);	
	assertEquals( 3, stack.peek() );
    }

    public void test_peek_method2() {
	try {
	    stack.peek();
	    fail( "expected EmptyStackException" );
	} catch (EmptyStackException e) {
	    assertTrue( stack.isEmpty() );
	} // end of try-catch
    }

    public void test_pop_method1() {
	stack.push(1);
	assertEquals( 1, stack.pop() );
	assertTrue( stack.isEmpty() );
	stack.push(2);
	assertEquals( 2, stack.pop() );
	assertTrue( stack.isEmpty() );
	stack.push(3);	
	assertEquals( 3, stack.pop() );
	assertTrue( stack.isEmpty() );
    }
    
    public void test_capacity_method1() {
	final int ITERS = 1000;
	for (int i = 0; i < ITERS; i++) {
	    stack.push(i);
	    assertTrue( stack.capacity() > i );
	} // end of for

	int oldCapacity = stack.capacity();
	for (int i = ITERS-1; i >= 0; i--) {
	    assertEquals(i, stack.pop());
	    assertTrue( stack.capacity() <= oldCapacity );
	    oldCapacity = stack.capacity();
	} // end of for
	assertTrue( stack.capacity() == IntStack.DEFAULT_CAPACITY );
    }

    //---------------------------------------------------------------------
    // PRIVATE HELPERS
    //---------------------------------------------------------------------

    //---------------------------------------------------------------------
    // COMMON FIXTURE
    //---------------------------------------------------------------------

    private IntStack stack;

    public void setUp() { 
	stack = new IntStack();
    }

    // public void tearDown() { }
    
    
}
