/*
 * Copyright (C) 2005 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: AnnotDoubleElementValue.java,v 1.1 2005/06/24 12:37:58 wdietl Exp $
 */

package org.multijava.util.classfile;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * A class representing a double value in an element-value pair of an
 * annotation structure.
 * 
 * @see Annotation.ElementValuePair
 * @author AHS
 */
public class AnnotDoubleElementValue extends AnnotElementValue {
    
    // --------------------------------------------------------------------
    // CONSTRUCTORS
    // --------------------------------------------------------------------
    
    /**
     * Construct a new element-value with the given value.
     */
    public AnnotDoubleElementValue(double value) {
        this.value = new DoubleConstant(value);
    }

    /**
     * Construct a new element-value from a classfile stream.
     * 
     * @param cp	the constant pool
     * @param in	the input stream to read from
     * @throws IOException	an IO exception has occurred
     */
    public AnnotDoubleElementValue(ConstantPool cp, DataInput in)
    		throws IOException {
        value = (DoubleConstant) cp.getEntryAt(in.readUnsignedShort());
    }

        
    // --------------------------------------------------------------------
    // WRITE
    // --------------------------------------------------------------------    

    /**
     * Insert or check location of constant value on constant pool.
     *
     * @param cp	the constant pool for this class
     */
    /* package */ void resolveConstants(ConstantPool cp) {
        cp.addItem(value);
    }

    /**
     * Write this value to a classfile stream.
     * 
     * @param cp	the constant pool
     * @param out	the output stream
     * @throws IOException	an IO error has occurred
     */
    /* package */ void write(ConstantPool cp, DataOutput out) throws IOException {
        out.writeByte('D');
        out.writeShort(value.getIndex());
    }

        
    // --------------------------------------------------------------------
    // ACCESSORS
    // --------------------------------------------------------------------

    /**
     * Return the size in bytes that this value takes up in the class file
     * (not including the tag).
     */
    /* package */ int getSize() {
        return 2;
    }
    
    /**
     * Return a textual representation of the value.
     */
    public String toString() {
        return Double.toString(value());
    }
    
    /**
     * Return the encapsulated value.
     */
    public double value() {
        return ((Double) value.getLiteral()).doubleValue();
    }

        
    // --------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // --------------------------------------------------------------------

    /** the constant pool entry for this value */
    private DoubleConstant value;
}
