/*
 * Copyright (C) 2005 Swiss Federal Institute of Technology Zurich
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: AnnotElementValue.java,v 1.1 2005/06/24 12:37:58 wdietl Exp $
 */

package org.multijava.util.classfile;

import java.io.DataOutput;
import java.io.IOException;

/**
 * A class representing a value in an element-value pair of an annotation
 * structure. This serves as the abstract super class of all value classes.
 * 
 * @see Annotation
 * @see Annotation.ElementValuePair
 * 
 * @author AHS
 */
public abstract class AnnotElementValue {
        
    /**
     * Insert or check location of constant value on constant pool.
     *
     * @param cp	the constant pool for this class
     */
    /* package */ abstract void resolveConstants(ConstantPool cp);
    
    /**
     * Write this value to a classfile stream.
     * 
     * @param cp	the constant pool
     * @param out	the output stream
     * @throws IOException	an IO error has occurred
     */
    /* package */ abstract void write(ConstantPool cp, DataOutput out) throws IOException;
    
    /**
     * Return the size in bytes that this value takes up in the class file
     * (not including the tag).
     */
    /* package */ abstract int getSize();
    
    /**
     * Return a textual representation of the value.
     */
    public abstract String toString();
}
