/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Attribute.java,v 1.8 2004/01/16 16:26:56 cclifton Exp $
 */

package org.multijava.util.classfile;

import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;

/**
 * This is the root class of every attribute
 */
public abstract class Attribute {

    // --------------------------------------------------------------------
    // CONSTRUCTORS
    // --------------------------------------------------------------------

    // --------------------------------------------------------------------
    // ACCESSORS
    // --------------------------------------------------------------------

    /**
     * Returns the attribute's tag
     */
    protected abstract int getTag();

    /**
     * Returns the space in bytes used by this attribute in the classfile
     */
    protected abstract int getSize();

    // --------------------------------------------------------------------
    // WRITE
    // --------------------------------------------------------------------

    /**
     * Insert or check location of constant value on constant pool
     *
     * @param	cp		the constant pool for this class
     */
    protected abstract void resolveConstants(ConstantPool cp) throws ClassFileFormatException;

    /**
     * Write this class into the the file (out) getting data position from
     * the constant pool
     *
     * @param	cp		the constant pool that contain all data
     * @param	out		the file where to write this object info
     *
     * @exception	java.io.IOException	an io problem has occured
     * @exception	ClassFileFormatException	attempt to
     *					write a bad classfile info
     */
    protected abstract void write(ConstantPool cp, DataOutput out)
	throws IOException, ClassFileFormatException;

    // ----------------------------------------------------------------------
    // DATA MEMBERS
    // ----------------------------------------------------------------------

}

