/*
 * Copyright (C) 2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: ClassInfoCreator.java,v 1.1 2003/11/14 21:46:57 cheon Exp $
 */

package org.multijava.util.classfile;

import java.io.*;

/**
 * A factory class to create class info objects.
 */
public class ClassInfoCreator {

    // --------------------------------------------------------------------
    // CONSTRUCTOR
    // --------------------------------------------------------------------

    /** 
     * Creates a new instance. This constructor is private to implement
     * the Singleton pattern.
     */
    protected ClassInfoCreator() {
    }

    // --------------------------------------------------------------------
    // CREATOR METHODS
    // --------------------------------------------------------------------
 
    /**
     * Creates a class info object by reading bytecode from the data
     * input stream <code>data</code>.
     */
    public /*@ non_null @*/ ClassInfo createClassInfo(
        /*@ non_null @*/ DataInput data, boolean interfaceOnly)
        throws IOException, ClassFileFormatException {

        return new ClassInfo(data, interfaceOnly);
    }

    /**
     * Creates a field info object by reading bytecode from the data
     * input stream <code>data</code>.
     */
    public /*@ non_null @*/ FieldInfo createFieldInfo(
        /*@ non_null @*/ DataInput data, /*@ non_null @*/ ConstantPool cp)
        throws IOException, ClassFileFormatException {

        return new FieldInfo(data, cp);
    }

    /**
     * Creates a method info object by reading bytecode from the data
     * input stream <code>data</code>.
     */
    public /*@ non_null @*/ MethodInfo createMethodInfo(
        /*@ non_null @*/ DataInput data, /*@ non_null @*/ ConstantPool cp,
        boolean interfaceOnly)
        throws IOException, ClassFileFormatException {

        return new MethodInfo(data, cp, interfaceOnly);
    }

    // --------------------------------------------------------------------
    // SINGLETON METHOD
    // --------------------------------------------------------------------

    /** Returns the unique intance of this class. */
    public static /*@ non_null @*/ ClassInfoCreator getInstance() {
        return theInstance;
    }

    // --------------------------------------------------------------------
    // DATA MEMBERS
    // --------------------------------------------------------------------

    /** The unique instance of this class. */
    private static final /*@ non_null @*/ ClassInfoCreator theInstance 
        = new ClassInfoCreator();
}
