/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MultimethodInfo.java,v 1.7 2004/04/12 21:00:43 tongjie Exp $
 */

package org.multijava.util.classfile;

import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;

import org.multijava.util.InternalError;

/**
 * An instance of this data structure describes each multimethod in a
 * class. */
public class MultimethodInfo extends MethodInfo {

    // --------------------------------------------------------------------
    // CONSTRUCTORS
    // --------------------------------------------------------------------

    /**
     * Constructs a multimethod info data structure from source.
     *
     * @param	modifiers	access permission to and properties of the method
     * @param	name		the simple name of the method
     * @param	type		the multimethod signature
     * @param number the generic function number, not part of the
     *				bytecode representation of this (it is
     *				stored in the parent gf_info), but
     *				convenient to have here
     * @param topConcMethColl if this is an internal method of an external
     *generic function, then <CODE>topConcMethColl</CODE> is the fully
     *qualified name of the external generic function to which this
     *belongs, otherwise <CODE>topConcMethColl</CODE> is null.  This is
     *not part of the bytecode representation of this (it is stored in
     *the parent gf_info), but convenient to have here.
     * @param	exceptions	the checked exceptions the method may throw
     * @param	deprecated	is this method deprecated ?
     *
     * NOTE: There must be exactly one Exceptions attribute in each
     * method_info structure.  */
    public MultimethodInfo(short modifiers, String name, String type,
			   String genericsignature,
			   short number, String topConcMethColl,
			   String[] exceptions, boolean deprecated) 
    {
	super(modifiers,name,type,genericsignature,exceptions,null,deprecated,false);
	this.number = number;
	this.topConcMethColl = topConcMethColl;
    }

    /**
     * Constructs a multimethod info data structure from a class file stream
     *
     * @param	in		the stream to read from
     * @param	cp		the constant pool
     * @param number the generic function number, not part of the
     *				bytecode representation of this (it is
     *				stored in the parent gf_info), but
     *				convenient to have here
     * @param topConcMethColl if this is an internal method of an external
     *generic function, then <CODE>topConcMethColl</CODE> is the fully
     *qualified name of the external generic function to which this
     *belongs, otherwise <CODE>topConcMethColl</CODE> is null.  This is
     *not part of the bytecode representation of this (it is stored in
     *the parent gf_info), but convenient to have here.
     *
     * @exception	java.io.IOException	an io problem has occured
     * @exception	ClassFileFormatException	attempt to
     *					write a bad classfile info
     */
    public MultimethodInfo(DataInput in, ConstantPool cp, 
			   short number, String topConcMethColl)
	throws IOException, ClassFileFormatException 
    {
	super(in, cp, true);
	this.number = number;
	this.topConcMethColl = topConcMethColl;
    }

    // --------------------------------------------------------------------
    // ACCESSORS
    // --------------------------------------------------------------------

    protected int getSize() {
	return 2 + 2 + 2 + 2 + attributes.getSize();
    }

    public int functionNumber() {
	return number;
    }

    public String topConcreteMethodCollection() {
	return topConcMethColl;
    }

    /**
     * Returns true if the method represented by this has a synthetic
     * receiver parameter (in the first parameter position).
     */
    public boolean hasSyntheticReceiver() {
	return topConcMethColl != null;
    }

    // --------------------------------------------------------------------
    // WRITE
    // --------------------------------------------------------------------

    // --------------------------------------------------------------------
    // PRIVATE METHODS
    // --------------------------------------------------------------------

    // --------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // --------------------------------------------------------------------

    private final short number;

    private final String topConcMethColl;
}
