/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: RMJAttribute.java,v 1.3 2004/01/16 16:26:56 cclifton Exp $
 */

package org.multijava.util.classfile;

import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;
import java.util.ArrayList;

/**
 * This attribute corresponds to a sequence of RMJ annotations.  */
public class RMJAttribute extends Attribute {

    // --------------------------------------------------------------------
    // CONSTRUCTORS
    // --------------------------------------------------------------------

    /**
     * Create a dispatcher attribute from source.
     */
    public RMJAttribute(String rmjAnnote) {
	val = new AsciiConstant(rmjAnnote);
    }

    // --------------------------------------------------------------------
    // ACCESSORS
    // --------------------------------------------------------------------

    /**
     * Returns the attribute's tag
     */
    protected int getTag() {
	return Constants.ATT_RMJ_SIGNATURE;
    }

    /**
     * Returns the space in bytes used by this attribute in the classfile
     */
    protected int getSize() {
	return 2 + 4 + 2;
    }

    // --------------------------------------------------------------------
    // WRITE
    // --------------------------------------------------------------------

    /**
     * Insert or check location of constant value on constant pool
     *
     * @param	cp		the constant pool for this class
     */
    protected void resolveConstants(ConstantPool cp) 
	throws ClassFileFormatException
    {
	cp.addItem(attr);
	cp.addItem(val);
    }

    /**
     * Write this class into the the file (out) getting data position from
     * the constant pool
     *
     * @param	cp		the constant pool that contain all data
     * @param	out		the file where to write this object info
     *
     * @exception	java.io.IOException	an io problem has occured
     */
    protected void write(ConstantPool cp, DataOutput out) 
	throws IOException, ClassFileFormatException
    {
	out.writeShort(attr.getIndex());
	out.writeInt(getSize() - 6);
	out.writeShort(val.getIndex());
    }

    // --------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // --------------------------------------------------------------------

    static final String tagString = "org.multijava.relaxed";
    private static AsciiConstant attr = new AsciiConstant(tagString);

    // the value associated with this tag
    private AsciiConstant val;
}
