/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Phylum.java,v 1.7 2007/12/22 14:57:06 chalin Exp $
 */

package org.multijava.util.compiler;

import org.multijava.util.Utils;

/**
 * This class represents the root class for all elements of the parsing tree
 */
public abstract class Phylum extends Utils 
    implements PhylumType, Cloneable 
{

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------

    /**
     * construct an element of the parsing tree
     * @param where the token reference of this node
     */
    public Phylum(/*@ non_null @*/ TokenReference where) {
	if(where == null) throw new NullPointerException(); // Chalin: SNO.
	this.where = where;
    }

    // ----------------------------------------------------------------------
    // ACCESSORS
    // ----------------------------------------------------------------------

    /**
     * Returns the token reference of this node in the source text.
     * @return the entire token reference
     */
    public /*@ pure non_null @*/ TokenReference getTokenReference() {
	return where;
    }

    /**
     * Change the token reference of this node
     * @param where the new token reference
     */
    public void setTokenReference(/*@ non_null @*/ TokenReference where) {
	this.where = where;
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private /*@ non_null @*/ TokenReference where;	// token reference in the source text
}
