/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 */


/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: UnpositionedError.java,v 1.5 2008/05/14 18:17:10 wdietl Exp $
 */

package org.multijava.util.compiler;

import org.multijava.util.Message;
import org.multijava.util.MessageDescription;

/**
 * This class is the root class for all compiler errors without a reference
 * to the source text.
 */

public class UnpositionedError extends Throwable {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * An exception with a formatted message as argument
   * @param	message		the formatted message
   */
  public UnpositionedError(/*@\readonly@*/ Message message) {
    super(message.getDescription().getFormat());

    this.message = message;
  }

  /**
   * An exception with an arbitrary number of parameters
   * @param	description	the message description
   * @param	parameters	the array of parameters
   */
  public UnpositionedError(/*@\readonly@*/ MessageDescription description, /*@\readonly \readonly@*/ Object[] parameters) {
    this(new Message(description, parameters));
  }

  /**
   * An exception with two parameters
   * @param	description	the message description
   * @param	parameter1	the first parameter
   * @param	parameter2	the second parameter
   */
  public UnpositionedError(/*@\readonly@*/ MessageDescription description, /*@\readonly@*/ Object parameter1, /*@\readonly@*/ Object parameter2) {
    this(description, new /*@\peer \readonly@*/ Object[] { parameter1, parameter2 });
  }

  /**
   * An exception with one parameter
   * @param	description	the message description
   * @param	parameter	the parameter
   */
  public UnpositionedError(/*@\readonly@*/ MessageDescription description, /*@\readonly@*/ Object parameter) {
    this(description, new /*@\peer \readonly@*/ Object[] { parameter });
  }

  /**
   * An exception without parameters
   * @param	description	the message description
   */
  public UnpositionedError(/*@\readonly@*/ MessageDescription description) {
    this(description, null);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns a string explaining the exception.
   */
  public /*@ pure @*/ String getMessage() {
    return message.getMessage();
  }

  /**
   * Returns the formatted message.
   */
  public /*@ pure @*/ /*@\readonly@*/ Message getFormattedMessage() {
    return message;
  }

  /**
   * Returns the string explaining the exception.
   */
  public /*@ pure @*/ String getErrorMessage() {
    return getMessage();
  }

  /**
   * Returns true iff the error has specified description.
   */
  public /*@ pure @*/ boolean hasDescription(MessageDescription description) {
    return message.getDescription() == description;
  }

  /**
   * Returns an error with a reference to the source file.
   * @param	where		the position in the source file responsible for the error
   */
  public PositionedError addPosition(TokenReference where) {
    return new PositionedError(where, message);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private /* javac bug final */ /*@\readonly@*/  Message		message;
}
