/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
package org.multijava.util.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JList;

/**
 *  An abstract class that defines the general behavior of an Open dialog
 *  window
 *
 *@author     Kristina Boysen
 *@version    $Revision: 1.2 $
 */

public abstract class AbstractOpenHandler implements ActionListener {
    
    // ----------------------------------------------------------------------
    // CONSTRUCTOR
    // ----------------------------------------------------------------------
        
    /**
     *  Provides a way for the user to select files
     */
    protected AbstractOpenHandler() {
    }
        
    // ----------------------------------------------------------------------
    // OPEN HANDLER WINDOW ACTIONS
    // ----------------------------------------------------------------------

    /**
     *  Compiles the selected files into a list
     */
    public void actionPerformed(ActionEvent event) {
        JFileChooser fileChooser = createFileChooser();
        JList fileList = getFileList();
        DefaultListModel fileModel = (DefaultListModel)(fileList.getModel());
        int returnVal = fileChooser.showOpenDialog(null);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File[] file = fileChooser.getSelectedFiles();
            for (int i = 0; i < file.length; i++) {
                if (!GUIUtil.filenameAlreadyAdded(
                    fileList, file[i].toString())) {
                    fileModel.addElement(file[i].toString());
                }
            }
            fileList.setSelectedIndex(fileModel.getSize() - 1);
            setDirectory(file[file.length - 1].getParent());
        }
    }
        
    /**
     *  Creates a new file chooser
     */
    protected JFileChooser createFileChooser() {
        JFileChooser fileChooser = new JFileChooser(getCurrentDirectory());
        fileChooser.setDialogTitle("Select Files to Add");
        fileChooser.setApproveButtonText("Add");
        fileChooser.setMultiSelectionEnabled(true);
        fileChooser.setFileSelectionMode(
            JFileChooser.FILES_AND_DIRECTORIES);
        fileChooser.setAccessory(new SelectAllPanel(fileChooser));
        
        return fileChooser;
    }

    // ----------------------------------------------------------------------
    // ABSTRACT METHODS
    // ----------------------------------------------------------------------

    /**
     *  The current list of files to process
     */
    protected abstract JList getFileList();
        
    /**
     *  The directory that the user prefers to start in for this file
     *  chooser
     */
    protected abstract String getCurrentDirectory();
        
    /**
     *  Sets the directory last accessed by the OpenHandler as the current
     *  preferred directory (if preferences are being used).
     */
    protected abstract void setDirectory(String file);
        
    // ----------------------------------------------------------------------
    // PRIVATE CLASSES
    // ----------------------------------------------------------------------

    /**
     *  Adds a "Select All" button to the side of the Open dialog box
     */
    private static class SelectAllPanel extends JComponent {
    
        /**
         *  Implements the "Select All" button panel
         */
        private SelectAllPanel(JFileChooser fc) {
            final JFileChooser fileChooser = fc;
            setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
            JButton selectAll = new JButton("Select All");
            selectAll.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        fileChooser.setSelectedFiles(
                        fileChooser.getCurrentDirectory().listFiles());
                    }
                });

            add(selectAll);
        }
    }
}
