/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.multijava.util.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;

import java.io.File;

import java.util.StringTokenizer;

import javax.swing.DefaultListModel;
import javax.swing.JList;

/**
 * Utilities for the Mjc's graphical user interface.
 * @author  David Behroozi
 * @author Gary T. Leavens
 * @author Kristina Boysen
 * @version $Revision 1.0$
 */

public class GUIUtil {
        
    // ----------------------------------------------------------------------
    // CENTER WINDOW
    // ----------------------------------------------------------------------

    /**
     * Center the given window within the screen.
     * @param window The window to center.
     * 
     * <pre><jml>
     *    requires window != null;
     * </jml></pre>
     */
    public static void centerWindow(Component window) {
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        int x = (int) ((screen.getWidth() - window.getWidth()) / 2);
        int y = (int) ((screen.getHeight() - window.getHeight()) / 2);
        window.setLocation(x, y);
    }
        
    // ----------------------------------------------------------------------
    // LOAD ICON FROM JAR
    // ----------------------------------------------------------------------
        
    /**
     * Load an icon from the given jar file.
     * @param filename The name of the file from which to load the icon.
     * @return an image containing the icon.
     * 
     * <pre><jml>
     *    requires filename != null;
     * </jml></pre>
     */
    public static Image loadIconFromJar(String filename) {
        return Toolkit.getDefaultToolkit().createImage(
            ClassLoader.getSystemClassLoader().getResource(
            "org/multijava/util/icons/" + filename));
    }
        
    // ----------------------------------------------------------------------
    // DIR IN PATH
    // ----------------------------------------------------------------------
                
    /**
     *  Determines whether a directory is in the classpath
     *  @param classpath The classpath to be checked
     *  @param dir the directory to check for
     *  @return true if the directory is found in the classpath
     */
    public static boolean dirInPath(String classpath, String dir) {
        if (classpath != null && dir != null) {
            StringTokenizer tokenizer 
                = new StringTokenizer(classpath, File.pathSeparator);
            while (tokenizer.hasMoreTokens()) {
                File file = new File(tokenizer.nextToken());
                if (file.isDirectory()
                    && ((file.getAbsolutePath()).indexOf(dir)) != -1) {
                    return true;
                }
            }
        }       
        return false;
    }
        
    // ----------------------------------------------------------------------
    // FILENAME ALREADY ADDED
    // ----------------------------------------------------------------------
        
    /**
     *  Tests to see whether the file is already in the list of files
     *  @param list the list to check
     *  @param file the file to check for in the list
     *  @return true if the file is in the list
     */
    public static boolean filenameAlreadyAdded(JList list, String file) {
        DefaultListModel fileModel = (DefaultListModel)list.getModel();
        for (int i = 0; i < fileModel.size(); i++) {
            // Convert to files for stricter checking
            File existingFile = new File(
                (fileModel.get(i)).toString());
            File newFile = new File(file);
            if (existingFile.equals(newFile)) {
                return true;
            }
        }
        return false;
    }
}
