/*
 * Copyright (C) 2000-2004 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Guigen.g,v 1.8 2004/07/08 01:07:10 netgecko Exp $
 */

// Import the necessary classes
header { package org.multijava.util.guigen; }

{
	import java.util.ArrayList;
}


//-----------------------------------------------------------------------------
// THE PARSER STARTS HERE
//-----------------------------------------------------------------------------

class GuigenParser extends Parser;

options {
	k = 2;				// two token lookahead
	exportVocab=Guigen;			// call the vocabulary
	codeGenMakeSwitchThreshold = 2;	// Some optimizations
	codeGenBitsetTestThreshold = 3;
	defaultErrorHandler = false;		// Don't generate parser error handlers
}

aCompilationUnit [String sourceFile]
returns [DefinitionFile self]
{
	String		    prefix = null;
	String		    packageName = null;
	ArrayList	    imports = null;
	String          readableFiles = null;
	String			webpageName = null;
	String			webpageLocation = null;
	ArrayList       acceptedSuffixes = null;

}
	:

		// First a prefix to for the generated file, i.e. <prefix>GUIBuilder.java
		"prefix" prefix = aIdentifier[]
		
		// The package to which the generated class should belong
		( "package" packageName = aName[] ) ?

		// any custom imports ?
		( "imports" imports = aNameList[] ) ?

		// The string that describes the files that can be read by the tool
		"readableFiles"	readableFiles = aString[]
		
		// The string that describes the files that can be read by the tool
		"webpageName"	webpageName = aString[]
		
		// The string that describes the files that can be read by the tool
		"webpageLocation"	webpageLocation = aString[]
		
		// A list of values that are possible for this option
		( "acceptedSuffixes" acceptedSuffixes = aStringList[] )?
		
		{
			self = new DefinitionFile(sourceFile, packageName, imports, prefix, 
				readableFiles, webpageName, webpageLocation, acceptedSuffixes); 
		}
	;

aIdentifier []
returns [String self = null]
	:
		t : IDENT
		{ self = t.getText(); }
	;

aName []
returns [String self]
{
	String	ident;
}
	:
		ident = aIdentifier[]
		{ self = ident; }
		(
			DOT ident = aIdentifier[]
			{ self += "." + ident; }
		)*
	;


aNameList[]
returns [ArrayList self]
{
	String val = "";
	String val2 = "";
}
	:
		val = aName[]
		{ 
		      self = new ArrayList(); 
		      self.add(val);
		}
		(
			COMMA val2 = aName[]
			{ self.add(val2); }
		)*
	;

aString []
returns [String self]
	:
		t : STRING
		{ self = t.getText(); }
	;


aStringList[]
returns [ArrayList self]
{
	String val = "";
	String val2 = "";
}
	:
		val = aString[]
		{ 
		      self = new ArrayList(); 
		      self.add(val);
		}
		(
			COMMA val2 = aString[]
			{ self.add(val2); }
		)*
	;

//-----------------------------------------------------------------------------
// THE SCANNER STARTS HERE
//-----------------------------------------------------------------------------

class GuigenLexer extends Lexer;

options {
	importVocab=Guigen;    // call the vocabulary
	testLiterals=false;    // don't automatically test for literals
	k=2;                   // 2 characters of lookahead
}

WS :
		(
			' '
		| '\t'
		| '\f'
		| '\r' ('\n')?	{ newline(); }
		| '\n'		{ newline(); }
		)
		{ _ttype = Token.SKIP; }
	;

DOT		: '.';
SHARP		: '#';
COMMA		: ',';

// Single-line comments
SL_COMMENT :
		"//"
		(~('\n'|'\r'))* ('\n'|'\r'('\n')?)
		{ _ttype = Token.SKIP; newline(); }
	;

// multiple-line comments
ML_COMMENT :
		"/*"
		(
			'\n' { newline(); }
		| '*' ~'/'
		| ~'*'
		)*
		"*/"
		{ _ttype = Token.SKIP; }
	;

STRING :
		'"' (ESC|~('"'|'\\'))* '"'
	;
	
INT:
		('0' .. '9')+
	;

protected
ESC :
		'\\'
		(
			'n'
		| 'r'
		| 't'
		| 'b'
		| 'f'
		| '"'
		| '\''
		| '\\'
		| ('0'..'3') OCT_DIGIT OCT_DIGIT
		| 'u' HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
		)
	;

protected
OCT_DIGIT :
		'0'..'7'
	;

protected
HEX_DIGIT :
		'0'..'9'
	| 'A'..'F'
	| 'a'..'f'
	;

// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
IDENT
options { testLiterals=true; }
	:
		('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$')*
	;

// an identifier preceded by a '@' Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
DUMMY
options { testLiterals=true; }
	:
		'@' IDENT
	;

// a dummy rule to force vocabulary to be all characters except special
// ones that ANTLR uses internally (0 to 2)
protected
VOCAB :
		'\3'..'\377'
	;
