/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Lexgen.g,v 1.4 2003/09/02 19:13:43 cclifton Exp $
 */

// Import the necessary classes
header { package org.multijava.util.lexgen; }

{
    import java.util.ArrayList;
}


//-----------------------------------------------------------------------------
// THE PARSER STARTS HERE
//-----------------------------------------------------------------------------

class LexgenParser extends Parser;

options {
    k = 2;				// two token lookahead
    exportVocab=Lexgen;			// call the vocabulary
    codeGenMakeSwitchThreshold = 2;	// Some optimizations
    codeGenBitsetTestThreshold = 3;
    defaultErrorHandler = false;		// Don't generate parser error handlers
}

aCompilationUnit [String sourceFile]
returns [DefinitionFile self]
{
    String		prefix = null;
    String		packageName = null;
    String		vocabulary = null;
    String flag = null;
    ArrayList flags = new ArrayList();
    ArrayList		definitions = new ArrayList();
    TokenDefinition	def;
}
    :
        ( "@prefix" prefix = anIdentifier[] )?
        ( "@package" packageName = aName[] )?
        "@vocabulary" vocabulary = anIdentifier[]

        // optional flags that can be specified for literals and keywords
        ( "@flags" 
            (flag = anIdentifier[] { flags.add(flag); } )+ 
        )?

        (
            def = aTokenDefinition[]
            { definitions.add(def); }
        )+
        { 
            self = new DefinitionFile(sourceFile, packageName, 
                vocabulary, prefix, flags, definitions); 
        }
    ;

anIdentifier []
returns [String self = null]
    :
        t : IDENT
        { self = t.getText(); }
    ;

aName []
returns [String self]
{
    String	ident;
}
    :
        ident = anIdentifier[]
        { self = ident; }
        (
            DOT ident = anIdentifier[]
            { self += "." + ident; }
        )*
    ;

aString []
returns [String self]
    :
        t : STRING
        { self = t.getText(); }
    ;

aTokenDefinition []
returns [TokenDefinition self]
{
    int		tokenType = TokenDefinition.OTHER;
    String	ident;
    String	value = null;
    String flag = null;
    ArrayList flags = new ArrayList();
}
    :
        (
            "@literal"
            { tokenType = TokenDefinition.LITERAL; }
        |
            "@keyword"
            { tokenType = TokenDefinition.KEYWORD; }
        |
            "@token"
            { tokenType = TokenDefinition.OTHER; }
        )
        ident = anIdentifier[]
        ( value = aString[] ) ?

        // optional flags that are true for this literal or keyword
        ( "@flags" ( flag = anIdentifier[] { flags.add(flag); } )+ )?

        { self = new TokenDefinition( tokenType, ident, value, flags ); }
    ;

//-----------------------------------------------------------------------------
// THE SCANNER STARTS HERE
//-----------------------------------------------------------------------------

class LexgenLexer extends Lexer;

options {
    importVocab=Lexgen;       // call the vocabulary
    testLiterals=false;    // don't automatically test for literals
    k=2;                   // 2 characters of lookahead
}

WS :
        (
            ' '
        | '\t'
        | '\f'
        | '\r' ('\n')?	{ newline(); }
        | '\n'		{ newline(); }
        )
        { _ttype = Token.SKIP; }
    ;

DOT		: '.';
SHARP		: '#';

// Single-line comments
SL_COMMENT :
        "//"
        (~('\n'|'\r'))* ('\n'|'\r'('\n')?)
        { _ttype = Token.SKIP; newline(); }
	;

// multiple-line comments
ML_COMMENT :
        "/*"
        (
            '\n' { newline(); }
        | '*' ~'/'
        | ~'*'
        )*
        "*/"
        { _ttype = Token.SKIP; }
    ;

// returns the actual ASCII value represented by the String literal, e.g.,
// "\\hello\"\\world" in the *.t file will be returned from this production as
// \hello"\world
STRING :
        '"'! (ESC|~'"')* '"'!
    ;

// returns the actual ASCII value represented by the escape sequence
protected
ESC 
{
	int base = -1;
}
	:
		'\\'!
		( 'n' { $setText( '\n' ); }
		| 'r' { $setText( '\r' ); }
		| 't' { $setText( '\t' ); }
		| 'b' { $setText( '\b' ); }
		| 'f' { $setText( '\f' ); }
		| '"'					// ignoring the leading \ is sufficient
		| '\''					// ignoring the leading \ is sufficient
		| '\\'					// ignoring the leading \ is sufficient
		| ('0'..'3') OCT_DIGIT OCT_DIGIT { base = 8; }
		| 'u'! HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT { base = 16; }
		)
		{
			if (base > 0) {
				String t = new Character(
					(char) Integer.parseInt( $getText, base )
				).toString();
				$setText( t );
			}
		}
	;

protected
OCT_DIGIT :
		'0'..'7'
	;

protected
HEX_DIGIT :
        '0'..'9'
    | 'A'..'F'
    | 'a'..'f'
    ;

// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
IDENT
options { testLiterals=true; }
    :
        ('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$')*
    ;

// an identifier preceded by a '@' Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
DUMMY
options { testLiterals=true; }
    :
        '@' IDENT
    ;

// a dummy rule to force vocabulary to be all characters except special
// ones that ANTLR uses internally (0 to 2)
protected
VOCAB :
        '\3'..'\377'
    ;
