/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler, and the JML Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestLexgenLexer.java,v 1.4 2003/09/08 15:31:10 cclifton Exp $
 */

package org.multijava.util.lexgen;
import java.io.Reader;
import java.io.StringReader;
import org.multijava.util.testing.TestCase;
import antlr.TokenStreamException;
import antlr.Token;
import java.util.ArrayList;


/**
 * TestLexgenLexer.java
 *
 * @author Curtis Clifton
 * @version $Revision: 1.4 $
 */

public class TestLexgenLexer extends TestCase implements LexgenLexerTokenTypes 
{

    //---------------------------------------------------------------------
    // CONSTRUCTORS 
    //---------------------------------------------------------------------
    public TestLexgenLexer( String name ) {
	super( name );
    }
    
    //---------------------------------------------------------------------
    // lexing strings
    //---------------------------------------------------------------------

    private void test_string_helper( String source, String[] expected ) 
	throws TokenStreamException
    {
	String[] result = lexThis( source );
	for (int e=0, r=0; e<expected.length && r<result.length; e++,r++) {
	    assertEquals( expected[e], result[r] );
	} // end of for
	assertEquals( "lengths should match but don't", 
		      expected.length, result.length );
    }

    public void test_string_1() throws TokenStreamException {
	test_string_helper( "\"plain vanilla\"",
			    new String[] { "plain vanilla" } );
    }

    public void test_string_2() throws TokenStreamException {
	test_string_helper( "plain vanilla",
			    new String[] { "plain", "vanilla" } );
    }

    public void test_string_3() throws TokenStreamException {
	test_string_helper( "\"hello\\nworld\"",
			    new String[] { "hello\nworld" } );
    }

    public void test_string_4() throws TokenStreamException {
	test_string_helper( "\"\\\"hello world\\'s fair\\\"\"",
			    new String[] { "\"hello world's fair\"" } );
    }

    public void test_string_5() throws TokenStreamException {
	test_string_helper( "\"\\\\slash\" \"\\rreturn\" \"\\ttab\"",
			    new String[] { "\\slash", "\rreturn", "\ttab" } );
    }

    public void test_string_6() throws TokenStreamException {
	test_string_helper( "\"\\bbackspace\" \"\\fformfeed\"",
			    new String[] { "\bbackspace", "\fformfeed" } );
    }

    public void test_string_7() throws TokenStreamException {
	test_string_helper( "\"\\060\\101\\172\"",
			    new String[] { "0Az" } );
    }

    public void test_string_8() throws TokenStreamException {
	test_string_helper( "\"\\u0030\\u0041\\u007a\"",
			    new String[] { "0Az" } );
    }


    //---------------------------------------------------------------------
    // PRIVILEGED DATA
    //---------------------------------------------------------------------

    private String[] lexThis( String source ) 
	throws TokenStreamException
    {
	Reader r = new StringReader( source );
	LexgenLexer lex = new LexgenLexer( r );
	Token t;
	ArrayList resultV = new ArrayList();
	while ((t = lex.nextToken()).getType() != EOF) {
	    resultV.add( t.getText() );
	} // end of while 
	String[] resultA = new String[ resultV.size() ];
	resultV.toArray( resultA );
	
	return resultA;
    }


}// TestLexgenLexer
