/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * based in part on work:
 *
 */


/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Main.java,v 1.3 2004/01/16 16:26:57 cclifton Exp $
 */

package org.multijava.util.msggen;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Hashtable;

import gnu.getopt.Getopt;
import gnu.getopt.LongOpt;

/**
 * This class is the entry point for the Message generator.
 */
public class Main {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * Only main can construct Main
   */
  private Main(String[] args) {
    parseArgs(args);

    parseSourceFiles();
    checkIdentifiers();
    buildInterfaceFile();

    System.exit(0);
  }

  /**
   *
   */
  private void parseSourceFiles() {
    boolean		errorsFound = false;

    try {
      definition = DefinitionFile.read(sourceFiles[0]);
    } catch (MsggenError e) {
      System.err.println(e.getMessage());
      errorsFound = true;
    }

    if (errorsFound) {
      System.exit(1);
    }
  }


  /**
   *
   */
  private void checkIdentifiers() {
    Hashtable		identifiers = new Hashtable();
    boolean		errorsFound = false;

    try {
      definition.checkIdentifiers(identifiers);
    } catch (MsggenError e) {
      System.err.println(e.getMessage());
      errorsFound = true;
    }

    if (errorsFound) {
      System.exit(1);
    }
  }

  /**
   *
   */
  private void buildInterfaceFile() {
    String		prefix = definition.getPrefix();
    File		outputFile = new File(prefix + "Messages.java");

    try {
      PrintWriter	out = new PrintWriter(new BufferedWriter(new FileWriter(outputFile)));

      definition.printFile(out);

      out.flush();
      out.close();
    } catch (java.io.IOException e) {
      System.err.println("I/O Exception on " + outputFile.getPath() + ": " + e.getMessage());
      System.exit(1);
    }
  }

  // --------------------------------------------------------------------
  // ENTRY POINT
  // --------------------------------------------------------------------

  /**
   * Entry point to the assembler
   */
  public static void main(String[] args) {
    new Main(args);
  }

  // --------------------------------------------------------------------
  // ACCESSORS
  // --------------------------------------------------------------------

  /*
   * Parse command line arguments.
   */
  private void parseArgs(String[] args) {
    Getopt	g = new Getopt("msggen", args, shortopts, longopts, true);

  _loop_:
    for (;;) {
      switch (g.getopt()) {
      case 'h':
	printUsage();
	System.exit(0);

      case -1:
	break _loop_;
      }
    }

    this.sourceFiles = new String[args.length - g.getOptind()];
    System.arraycopy(args, g.getOptind(), this.sourceFiles, 0, this.sourceFiles.length);
  }

  private static void printUsage() {
    System.err.println("org.multijava.util.msggen.Main [-d <directory>] [-V] <file>+");
    System.err.println();
    System.err.println("THIS PROGRAM IS A PART OF THE KOPI SUITE, A SET OF GNU PACKAGE FOR");
    System.err.println("HANDLING JAVA COMPILATION.   (optimize, dis, kjc, xkjc, ksm, ikjc)");
    System.err.println("MORE INFO:                             see: http://www.dms.at/kopi");
    System.err.println("THE GRAMMAR WAS DEVELOPED USING ANTLR.   see: http://www.antlr.org");
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private static final String	shortopts = "h";
  private static final
    LongOpt[]			longopts = {
      new LongOpt("help", LongOpt.NO_ARGUMENT, null, 'h')
    };

  private String[]		sourceFiles;
  private DefinitionFile	definition;
}
