/*
 * Copyright (C) 2000-2003 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestFilesCompileOnlyTestSuite.java,v 1.4 2003/02/19 02:25:24 davidcok Exp $
 */

package org.multijava.util.testing;

import org.multijava.util.testing.TestCase;
import org.multijava.util.testing.*;
import junit.framework.*;
import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.lang.reflect.Method;

/** This is a JUnit TestSuite that is created from a number of tests
 *  as follows.  Each TestCase is an instance of the inner class
 *  Helper, instantiated with a name of a file.  The file names are
 *  read from the file named by the parameter
 *  <code>fileOfTestFilenames</code>.  The argument to the constructor
 *  named <code>args</code> provides a set of command-line arguments; the
 *  filename for the TestCase is added on to the end of the list of
 *  command-line arguments.  Then the static compile method of the
 *  given class is called on those command-line arguments.
 *
 *  <P> The standard output and error output is captured from the
 *  execution of the compile method. The TestCase succeeds if If the
 *  output is empty; otherwise, it fails.
 *
 *  <P> The test must be run from the directory in which it resides -
 *  because it runs external processes that are expected to be in the
 *  current directory and creates and opens files in the current
 *  directory.
 *
 *  @author David R. Cok
 *  @author Yoonsik Cheon
 */
public class TestFilesCompileOnlyTestSuite 
    extends TestFilesTestSuite {

    /** Constructs a new test suite.
     *
     * @param testName The name of the test suite
     * @param fileOfTestFilenames The file to be read for filenames of
     *        tests
     * @param args The command-line arguments that the static compile
     *        method will be applied to, with the filename added on
     * @param cls The class in which to find the static compile method
     */
    public TestFilesCompileOnlyTestSuite(String testName, 
                                         String fileOfTestFilenames,
                                         String[] args, 
                                         Class cls) {
	super(testName,fileOfTestFilenames,args,cls);
    }

    protected TestFilesTestSuite.Helper makeHelper(String testname, String[] args) {
	return new Helper(testname,args);
    }

    // -------------------------------------------------------------
    // INNER CLASSES
    // -------------------------------------------------------------

    /** This is a helper class that is actually a TestCase; it is run
     * repeatedly with different constructor arguments.
     */
    public class Helper extends TestFilesTestSuite.Helper {

	public Helper(String filename, String[] args) {
	    super(filename,args);
	}
	
	protected void dotest(ByteArrayOutputStream ba) 
			    throws IllegalAccessException, IllegalArgumentException, 
						java.io.IOException,
						java.lang.reflect.InvocationTargetException {
						
                // compile the test file and check for any error
		boolean b = ((Boolean)(method.invoke(null, new Object[]{args})))
					.booleanValue();
                String s = ba.toString();
                assertTrue(s, "".equals(s));

		expectedStatusReport(b,s);
	}

     }

}
