/*
 * Copyright (C) 2000-2002 Iowa State University
 *
 * This file is part of mjc, the MultiJava Compiler.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: TestMain.java,v 1.3 2004/01/16 16:26:58 cclifton Exp $
 */

package org.multijava.util.testing;

import java.io.PrintStream;
import java.io.ByteArrayOutputStream;

/**
 * Unit tests for Main */
public class TestMain extends TestCase {
    
    //---------------------------------------------------------------------
    // CONSTRUCTORS
    //---------------------------------------------------------------------

    public TestMain( String name ) {
	super( name );
    }

    //---------------------------------------------------------------------
    // FIXTURE
    //---------------------------------------------------------------------

    protected void setUp() {
	this.main = new Main();
    }

    protected void tearDown() {
	this.main = null;
    }

    private Main main;

    //---------------------------------------------------------------------
    // TEST CASES
    //---------------------------------------------------------------------

    public void testDefault() {
	PrintStream oldOut = System.out;
	ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
	PrintStream newOut = new PrintStream( byteOut );
	System.setOut( newOut );
	boolean result = main.run( new String[0] );
	System.setOut( oldOut );
	assertTrue( result );
    }
    
    public void testArgs1() {
	String pack = "org/multijava";
	String result = main.fixPackage( pack );
	assertEquals( "org.multijava", result );
    }

    public void testArgs2() {
	String tc = "TestSuite";
	assertTrue( main.isSuite( tc ) );
    }

    public void testArgs3() {
	String tc = "TestSweet";
	assertTrue( !main.isSuite( tc ) );
    }

    public void testArgs4() {
	String tc = "foo.TestSuite";
	assertTrue( main.isSuite( tc ) );
    }

    public void testArgs5() {
	String tc = "foo.TestSweet";
	assertTrue( !main.isSuite( tc ) );
    }

}
